<?php

DEFINE('__MSF', true);
DEFINE("CNF_ABS_PATH", str_replace('\\', '/', dirname(__FILE__)));
$support_locale = "ru:'ru'";
DEFINE('SUPPORT_LOCALE',$support_locale);
DEFINE('CNF_LANGUAGE',"ru");
set_include_path(get_include_path() . PATH_SEPARATOR . CNF_ABS_PATH);
include_once 'cs/library/CSLauncher.php';
include_once 'system/utils/MSF_Logger.php';

class CSApplication extends CSLauncher {

    public static function main() {
        try {
            self::initIncludes();
            self::initErrorHandler();
            self::initSettings();
            self::initConstants();
            $sTimeStampFilePath = CNF_ABS_PATH . "/var/.lock";
            if (!empty($_GET) && isset($_GET["debug"])) {
                echo "<pre>";
                self::update();
                self::run();
            } else if (!file_exists($sTimeStampFilePath)) {
                echo '[SERVER RESTARTED AT: ' . date("Y.m.d H:i:s") . ' ]' . "\n";
                MSF_Logger::logMessage('[SERVER RESTARTED]', "cron");
                self::write($sTimeStampFilePath, time());
                self::resetAtoms();
                self::hibernate($sTimeStampFilePath);
            }
            // Cron is launched every 60 secs
            // If CURRENT_TIME - LOCK_TIME > 60 then we know that server was restarted
            else if ((time() - (int) file_get_contents($sTimeStampFilePath)) > 60) {
                echo '[SERVER STARTED AT: ' . date("Y.m.d H:i:s") . ' ]' . "\n";
                MSF_Logger::logMessage('[SERVER STARTED]', "cron");
                self::resetAtoms();
                self::hibernate($sTimeStampFilePath);
            } else {
                self::update();
                self::run();
            }
        } catch (Exception $E) {
            MSF_Logger::logTrace($E, "cron");
            exitApp($this->getTrace($E));
        }
    }

    private function getTrace($e) {
        $aTrace = $e->getTrace();
        $sMsg = "Exception: " . "\n" . $e->getMessage() . "\n\n\n";
        foreach ($aTrace AS $k => $v) {
            $sMsg .=get_class($e) . ": #" . $k . "\n"
                    . "Location: " . ((array_key_exists("class", $v)) ? $v["class"] : "?") . ((array_key_exists("type", $v)) ? $v["type"] : "?") . ((array_key_exists("function", $v)) ? $v["function"] : "?") . "\n"
                    . "File: " . ((array_key_exists("file", $v)) ? $v["file"] : "?") . "\n"
                    . "Line: " . ((array_key_exists("line", $v)) ? $v["line"] : "?") . "\n"
                    . "Args: " . "\n" . "[ " . "\n";
            if (array_key_exists("args", $v) && is_array($v["args"]) && (count($v["args"]) > 0)) {
                foreach ($v["args"] AS $k => $v) {
                    $type = gettype($v);
                    switch (true) {
                        case is_object($v):
                            $v = get_class($v);
                            break;
                        case is_array($v):
                            try {
                                $v = serialize($v);
                            } catch (Exception $e) {
                                $v = "not defined";
                            }

                        case is_string($v):
                            $v = $this->trn2space($v);
                            break;
                        case is_null($v):
                            $v = "null";
                            break;
                    }

                    $sMsg .= "    arg: #" . $k . " : (" . $type . ") " . $v . "\n";
                }
            } else {
                $sMsg .="    no args" . "\n";
            }

            $sMsg .= "]\n\n\n";
        }

        $sMsg .="Address: " . (isset($_SERVER["REMOTE_ADDR"]) ? $_SERVER["REMOTE_ADDR"] : "A.A.A.A") . "\n"
                . "Date: " . MSF_DATE . "\n";

        return $sMsg;
    }

}

CSApplication::main();
?>