<?php
//DEFINE('__MSF', true);
//DEFINE('CNF_ABS_PATH', 'D:/projects/msf.lcl/wwwroot');
set_include_path(get_include_path() . PATH_SEPARATOR . CNF_ABS_PATH);

include_once "cs/library/AtomicOperation.php";
include_once "system/MSF_ThrowableException.php";
include_once "system/MSF_Locale.php";
include_once "modules/references/library/Reference/REF_Value.php";
include_once "system/utils/MSF_Date.php";
include_once "system/utils/MSF_DB.php";
include_once "system/MSF_Registry.php";
include_once "modules/workflow/library/common/WF_Exception.php";
include_once "system/utils/MSF_XmlWrapper.php";
include_once "system/utils/MSF_SL.php";
include_once "library/MSF_SearchUtil/MSF_CsArchiveDB.php";
include_once "modules/archive/library/ARCH_DBRecord.php";
include_once "modules/workflow/library/actions/WF_DocFacade.php";
include_once "modules/main/library/orgStructure/MSF_OrgStructureFacade.php";
include_once "library/MSF_UniqueField/MSF_UniqueField.php";
include_once "system/MSF_IOException.php";
include_once "system/utils/MSF_Dir.php";
include_once "msfx/userbox/library/MSF_UserBox.php";
include_once "system/MSF_Template.php";
include_once "modules/archive/library/ARCH_Media.php";

class ArchivingAtomicOperation extends AtomicOperation {
	
	const CNF_ARCHIVE_LIMIT = 100;
	private $ArchDb;
	private $XmlWrapper;
	private $DocFacade;
	private $StructFacade;
	private $ArchMedia;	
	private $DB;
	private $Constants;	
	
	public function execute()
	{
		try {
			echo "Path to settings file: " . CNF_ABS_PATH . "/settings.xml\n";
			MSF_Settings::clearInstance();			
			$sContent  	 = file_get_contents(CNF_ABS_PATH . '/settings.xml');
			$oSettings 	 = MSF_Settings::getInstance($sContent);		
			MSF_Registry::set("MSF_Settings", $oSettings);
			MSF_Registry::set("MSF_DB", $this->getDb());
			MSF_Registry::set("MSF_Locale", new MSF_Locale('cs/languages/russian.ini'));
			MSF_Registry::set("MSF_UserBox", MSF_UserBox::getInstance());
			MSF_Registry::set("MSF_Template", new MSF_Template());
			$this->ArchDb		= new ARCH_DBRecord();
			$this->XmlWrapper 	= new MSF_XmlWrapper;
			$this->DocFacade 	= new WF_DocFacade();
			$this->StructFacade = MSF_OrgStructureFacade::getInstance();	
			$this->DB		  	= $this->getDb();
			$this->ArchMedia	= new ARCH_Media();
			$this->Constants	= $oSettings->getConstants();
			$aSettings  = MSF_Registry::get("MSF_Settings")->getModule("archive");			 
			$this->ArchMedia->updateUploadDirectory(CNF_ABS_PATH . "/" . $aSettings["settings"]["arch_files_catalog"]);
			$sDownloadDirectory = CNM_MEDIA_UPLOAD_DIRECTORY;
			if($sDownloadDirectory[0] == '/') {
				$sDownloadDirectory = substr($sDownloadDirectory, 1);
			}
			
			$this->ArchMedia->updateDownloadDirectory(CNF_ABS_PATH . "/" . $sDownloadDirectory);
			define("CNF_DATE_FORMAT", $this->Constants["CNF_DATE_FORMAT"]);
			define("CNF_DATE_TIME_FORMAT", $this->Constants["CNF_DATE_TIME_FORMAT"]);		
			$this->archive();				
		}
		catch(Exception $e) {
			print_r($e);
		}
		
	}
	
	public function archive()
	{	
		if(!$this->hasParam("lastid")) {
			$iLastId = $this->getParam("defaultLastid");										
		}
		else {
			$iLastId = $this->getParam("lastid");
		}
		
		echo "Last ID = " . $iLastId . "\n";		
		$DBRecord	  	  = new MSF_CsArchiveDB('wf_documents', 0);		
		
		$bLowParty		  = false;
		if($iLastId == -1) {
			$sWhere = "true";
		}
		else {
			$sWhere = "document_id < " . $iLastId;			
		}		
		
		$aDocumentIds 	  = $DBRecord->find("document_id, doctype_id, document_version_id, doctype_version_id",
		$sWhere . " ORDER BY document_id desc LIMIT " . self::CNF_ARCHIVE_LIMIT, array());
		if(count($aDocumentIds) < self::CNF_ARCHIVE_LIMIT) {
			$bLowParty = true;	
		}
		
		for($i = 0; $i < count($aDocumentIds); $i++)
		{	
			$oArchivedDoc = $this->needArchive($aDocumentIds[$i]["document_id"]);					
			if($oArchivedDoc != null) {												
				$iDocumentId	   = $aDocumentIds[$i]["document_id"]; 
				$iCurrentVersionId = $aDocumentIds[$i]["document_version_id"];
				$iDoctypeVersionId = $aDocumentIds[$i]["doctype_version_id"];
				$iDoctypeId		   = $aDocumentIds[$i]["doctype_id"];
				$oDocVersion	   = $oArchivedDoc->getVersion($iCurrentVersionId);
				$iMediaId		   = $oArchivedDoc->getMediaId();
				
				try {					
                                        $aDeleteVersions = array();
					$UF	= new MSF_UniqueField($this->DB);
					$UF->freeFields($oDocVersion->getFieldsAsList(), MSF_UniqueField::TYPE_DOCTYPE, $iDoctypeId);
			
					foreach ($oArchivedDoc->getDocumentVersionIds() AS $iVersionId) {
						if($iVersionId != $iCurrentVersionId) {
//							$oArchivedDoc->getVersion($iVersionId)->remove();
                                                        $aDeleteVersions[] = $iVersionId;
						}
						else {							
							$oDoctype 	   =	$oArchivedDoc->getDoctype();
							$aDocFields	   = $oDocVersion->getRefFields()->getFieldsFull();														
							$iStructureId  = $oArchivedDoc->getStructureId();
							$aTmp 		   = explode("/^/col/^/", $oDocVersion->getSystemField("author_name"));
							if(isset($aTmp[0])) {
								$sAuthor   = $aTmp[0];
							}
							else {
								$sAuthor   = "";							
							}
							$aTmp 		   = $this->StructFacade->getAccountByStructureId($iStructureId);
							if(isset($aTmp["structure"]["subdivision"])) {
								$sSubdivision = $aTmp["structure"]["subdivision"];
							}
							else {
								$sSubdivision = "";
							}
							
							$oAb		   = WF_ActionBox::newInstance();
							$oAb->loadAction("view", 0, $iDocumentId, null, true);
							$oAction	   = $oAb->getAction();
							$oVars		   = new WF_Vars($oAction);
							$aSysDocFields = $oVars->getList();
							$this->XmlWrapper->clear();							
							foreach($aDocFields AS $sKey => $aValue) {
								$sTitle = $aValue["title"];
								unset($aValue["title"]);								
								$this->XmlWrapper->set($sKey, $aValue, $sTitle);
								if($aValue["type"] == "file" || $aValue["type"] == "picture") {
									try {
										$this->ArchMedia->upload($aValue["value"]);
									}
									catch(Exception $e) {
										print_r($e);
									}
																		
								}
								else if($aValue["type"] == "text") {
									preg_match_all('!"/media/(image|download)/(\d{1,})"!', $aValue["value"], $aSearch);
									$sFiles  = implode(",", $aSearch[2]);
									try {									
										$this->ArchMedia->upload($sFiles);
									}
									catch(Exception $e) {
										print_r($e);
									}
									
								}
								
							}
							
							$sFields = $this->XmlWrapper->getXml();
							$this->XmlWrapper->clear();							
							foreach($aSysDocFields AS $aValue) {								
								$this->XmlWrapper->set($aValue["name"], $aValue["html"]);
							}
							
							$oDate 							  = new MSF_Date();
							$sSysFields 					  = $this->XmlWrapper->getXml();
							$aArchData						  = array();
							$aArchData["structure_id"] 		  = $iStructureId;
							$aArchData["author_name"] 		  = $sAuthor;
							$aArchData["created_at"]   		  = $oDocVersion->getSystemField("created_at");
							$aArchData["modified_at"]  		  = $oDocVersion->getSystemField("modified_at");
							$aArchData["archived_at"] 		  = $oDate->formatTimestamp();
							$aArchData["doctype_id"]   		  = $iDoctypeId;
							$aArchData["document_id"]  		  = $iDocumentId;
							$aArchData["registration_number"] = $oDocVersion->getSystemField("registration_number");
							$aArchData["document_title"]	  = addslashes($oArchivedDoc->getTitle());
							$aArchData["fields_xml"]		  = addslashes($sFields);
							$aArchData["sysfields_xml"]		  = addslashes($sSysFields);
							$aArchData["template_xml"]		  = $oDoctype->getVersion($iDoctypeVersionId)->getTemplateXml();
							$aArchData["doctype_title"]		  = $oDoctype->getTitle();
							$aArchData["subdivision_name"]	  = $sSubdivision;
							$aArchData["media_id"]			  =	$iMediaId;

							$this->ArchDb->addItem($aArchData);
							//$oArchivedDoc->getVersion($iVersionId)->remove();												
                                                        $aDeleteVersions[] = $iVersionId;
						}
						
					}
                                        if(count($aDeleteVersions) > 0) {
                                            $iDeleteVersionId = 0;
                                            foreach ($aDeleteVersions AS $iDeleteVersionId) {
                                                if($iDeleteVersionId > 0) {
                                                    print_r(" id = " . $iDeleteVersionId . "; ");
                                                    $oArchivedDoc->getVersion($iDeleteVersionId)->remove();
                                                }    
                                            }
                                        }
					$oDb = $oArchivedDoc->getDB();
					if (($oDb->delete($iDocumentId) <> 1)) {
						throw new WF_DocumentNotRemovedException($iDocumentId);
					}
				}
				catch(Exception $e) {
					print_r($e);
				}
				
			}
			
		}
		
		// Zapis' v parametry poslednego id documenta
		if($bLowParty) {
			$this->setParam("lastid", -1);
		}
		else {
			$this->setParam("lastid", $aDocumentIds[count($aDocumentIds) - 1]["document_id"] + 1);
		}
		
	}
	
	public function needArchive($iDocumentId)
	{
		$oDocument = null;
		try {			
			$oPreDocument 	= $this->DocFacade->getDocumentById($iDocumentId);			
			if($oPreDocument == NULL) {
				return null;			
			}
			
			$DoctypeVersion = $oPreDocument->getDoctypeVersion();
			$sConditions	= $DoctypeVersion->getArchiveXml();
			if($sConditions != NULL && $sConditions != "") {								
				$aConditions 	   = $this->XmlWrapper->XmlToArray($sConditions);
				if(!empty($aConditions)) {
					foreach($aConditions AS $sKind => $aOptions) {
						$conditionKind = $sKind;
						$aParams	   = $aOptions["value"];
						break;
					}
					 
					if($conditionKind == "days_run_out_field") {
						$iCurrentDate = time();
						$aFields	  = $oPreDocument->getCurrentVersion()->getFields();
						if(isset($aFields[$aParams["field"]])) {
							$iFieldDate	  = strtotime($aFields[$aParams["field"]]);						
							if((int)(($iCurrentDate - $iFieldDate) / 86400) >= $aParams["days"]) {						
								$oDocument = $oPreDocument;	
							}
							
						}					
						
					}
					elseif($conditionKind == "days_run_out_action") {
						$iCurrentDate 	   = time();
						$oLog			   = $oPreDocument->getLog();
						$aExecutingActions = $oLog->getDisplay();
						if(!empty($aExecutingActions)) {
							$iLastActionTime = $aExecutingActions[count($aExecutingActions) - 1]["timestamp"];
						}
						else {
							$iLastActionTime = strtotime($oPreDocument->getCurrentVersion()->getSystemField("created_at"));						
						}
						
						if((int)(($iCurrentDate - $iLastActionTime) / 86400) >= $aParams["days"]) {
							$oDocument = $oPreDocument;
						}
						
					}				
									
				}							
				
			}
			
			return $oDocument;						
		}
		catch(Exception $e) {
			print_r($e);
			return null;
		}
		
	}
	
	public function moveFiles()
	{
		
	}
	
	public function dbg($Data)
	{
		echo "<pre>";
		print_r($Data);
		echo "</pre>";
	}
	
	public static function test()
	{
		$oDb 		= new MSF_DB();
		$oDb->connect('server', 'manuscript', 'detrix', '1', 'manuscript');
		MSF_Registry::set("MSF_DB", $oDb);		
		$oTestAO 	= AtomicOperation::create(MSF_Registry::get("MSF_DB"), 'ArchivingAtomicOperation.php',
											   'ArchivingAtomicOperation', array('limit' => 50));
		
		$oTestAO->execute();
	}
	
}
