<?php
/**
 * 
 */
abstract class AtomicOperation {

	/**
	 * @var string
	 */
	private $sError		= '';

	/**
	 * @var array
	 */
	private $aParams	= array();
	
	/**
	 * @var MSF_DB
	 */
	private $oDb		= null;
	
	protected function __construct(MSF_DB $oDb) {
		$this->oDb = $oDb;
	}
	
	protected function __clone() {}

	final public function setError($sError = '') {
		$this->sError = $sError;
	}

	final public function getError() {
		return $this->sError;
	}

	final public function hasError() {
		return ($this->sError !== '');
	}
	
	/**
	 * @return MSF_DB
	 */
	final public function getDb() {
		return $this->oDb;
	}
	
	/**
	 * @return AtomicOperation
	 */
	final public static function create(MSF_DB $oDb, $sFilepath, $sClassname, array $aParams) {
		include_once $sFilepath;
		$oAtomOperation = new $sClassname($oDb);
		foreach ($aParams AS $sKey => $mValue) {
			$oAtomOperation->setParam($sKey, $mValue);
		}
		
		return $oAtomOperation;
	}

	/**
	 * @return void
	 */
	final public function setParam($sKey, $mValue) {
		$this->aParams[$sKey] = $mValue;
	}
	
	/**
	 * @return mixed
	 */
	final public function getParam($sKey) {
		if ($this->hasParam($sKey)) {
			return $this->aParams[$sKey];
		}
		else {
			return null;
		}
		
	}
	
	/**
	 * @return boolean
	 */
	final public function hasParam($sKey) {
		return isset($this->aParams[$sKey]);
	}

	/**
	 * @return void
	 */
	abstract public function execute();

	public static function test() {
		$oTestAO = AtomicOperation::create('TestAtomicOperation.php', 'TestAtomicOperation', array(1 => 1));
		$oTestAO->execute();
		echo 'AtomicOperation::test()...';
		if ($oTestAO->hasError()) {
			if (($oTestAO->getError() == 'error') && ($oTestAO->getParam(1) == 1)) {
				echo 'passed';
			}
			else {
				echo 'failed';
			}
			
		}
		else {
			echo 'failed';
		}
		
	}
	
	final public function getParams()
	{
		return $this->aParams;
	}
	
	final public function debug($sMessage)
	{
		if (CNF_IS_DEBUG) {
			echo $this->sSessionId . ": " . $sMessage . "\n";
		}
		
	}
	
	private $sSessionId;
	
	final public function setSessionId($sSessionId)
	{
		$this->sSessionId = $sSessionId;
	}

}

// AtomicOperation::test();
?>