<?php
include_once 'AtomicOperation.php';
include_once 'AtomUtil.php';
include_once 'system/utils/MSF_Logger.php';

class CronoSphere {

	const DEEP				= 100;
	
	const WAITING 			= 0;
		
	const PROCESSING 		= 1;
	
	const PASSING 			= 2;
	
	const FAILING			= 3;
	
	private $iDeep			= 0;
	
	private $iCountOfParts 	= 0;
	
	private $aAtoms 		= array();
	
	private $sAbsPath		= '';
	
	private $oUtil			= null;
	
	private $oDb			= null;
	
	private $aTypes			= array();
	
	private $oMc;
	
	
	public function __construct($iCountOfParts) {
		$this->iCountOfParts 	= $iCountOfParts;
	}
	
	public function setDb(MSF_DB $oDb) {
		$this->oDb 		= $oDb;
		$this->oUtil 	= new AtomUtil($this->oDb);
	}
	
	public function resetAtomsByExpiredTTL() {
		foreach ($this->aAtoms AS $sType => $aParams) {
			if (isset($aParams["params"]["ttl"])) {
				$this->oUtil->resetAtomsByExpiredTTL($sType, (int)$aParams["params"]["ttl"]);
			}
			
		}

	}

	public function addAtom($sType, $sFilepath, $sClassname, array $aParams = array()) {
		if (!in_array($sType, $this->aTypes)) {
			$this->aAtoms[$sType] = array(
					'filepath' 	=> $sFilepath,
					'classname' => $sClassname,
					'params' 	=> $aParams,
					);
			$this->aTypes[] = $sType;
					
		}

	}
	
	public function execute()
	{
		if ($this->iCountOfParts > $this->oUtil->getCountOfAtomsInStatus(self::PROCESSING)) {
			$aAtom = $this->oUtil->getNextAtom();
			if (!empty($aAtom)) {
				if (isset($this->aAtoms[$aAtom['atom_type']])) {
					$oAtom 		= $this->createAtom($aAtom['atom_type'], $aAtom['subject_params']);
					$sSessionId = strtoupper(substr(md5($aAtom['atom_id'] . $aAtom['atom_type'] . getmypid()), 0, 4));
					$this->updateStartAtAndStatus($aAtom, time(), self::PROCESSING, $sSessionId, $oAtom->getParams());
					echo "[S-" . $sSessionId . " "
                                            . "id:" . $aAtom['atom_id'] . " "
                                            . "type:" . $aAtom['atom_type'] . " "
                                            . "pid:" . getmypid() . " "
                                            . "datetime:" . date("Y.m.d H:i:s") . " "
                                            . "status: " . $this->oUtil->getCountOfAtomsInStatus(self::PROCESSING) . "/" . $this->iCountOfParts
                                            . "]\n";
                                            MSF_Logger::logMessage("[S-" . $sSessionId . " "
                                                    . "id:" . $aAtom['atom_id'] . " "
                                                    . "type:" . $aAtom['atom_type'] . " "
                                                    . "pid:" . getmypid() . " "
                                                    . "datetime:" . date("Y.m.d H:i:s") . " "
                                                    . "status: " . $this->oUtil->getCountOfAtomsInStatus(self::PROCESSING) . "/" . $this->iCountOfParts
                                                    . "]","cron");
					try {
						$oAtom->execute();
					}
					catch (Exception $oE) {
						echo "[$sSessionId]";
						echo "Error:";
						echo $oE->getMessage();
						echo "Backtrace:";
						echo $oE->getTraceAsString();
						echo "[/$sSessionId]";
                                                
                                                MSF_Logger::logMessage("[$sSessionId]","cron");
                                                MSF_Logger::logMessage("Error:","cron");
                                                MSF_Logger::logMessage($oE->getMessage(),"cron");
                                                MSF_Logger::logMessage("Backtrace:","cron");
                                                MSF_Logger::logMessage($oE->getTraceAsString(),"cron");
                                                MSF_Logger::logMessage("[/$sSessionId]","cron");
					}
					
					echo  "[E-" . $sSessionId . " "
						. "id:".$aAtom['atom_id'] . " "
						. "type:".$aAtom['atom_type'] . " "
						. "pid:" . getmypid() . " "
						. "datetime:" . date("Y.m.d H:i:s"). " " 
						. "timeout: " . $oAtom->getParam("timeout") . " "
						. "status:" . $this->oUtil->getCountOfAtomsInStatus(self::PROCESSING) . "/" . $this->iCountOfParts
						. "]\n";
                                                MSF_Logger::logMessage("[E-" . $sSessionId . " "
						. "id:".$aAtom['atom_id'] . " "
						. "type:".$aAtom['atom_type'] . " "
						. "pid:" . getmypid() . " "
						. "datetime:" . date("Y.m.d H:i:s"). " " 
						. "timeout: " . $oAtom->getParam("timeout") . " "
						. "status:" . $this->oUtil->getCountOfAtomsInStatus(self::PROCESSING) . "/" . $this->iCountOfParts
						. "]","cron");
						
					if ($oAtom->hasError()) {
						echo $oAtom->getError();
					}

					$iNewStartAt = time() + (int) $oAtom->getParam("timeout");
					$this->updateStartAtAndStatus($aAtom, $iNewStartAt, $aAtom['status'], $sSessionId, $oAtom->getParams());
					if (++$this->iDeep < self::DEEP) {
						$this->execute();
					}
					
				}
				else {
					echo "[404 datetime:" . date("Y.m.d H:i:s"). " type:" . $aAtom['atom_type'] . " current:".implode(',', $this->aTypes)."/]\n";
                                        MSF_Logger::logMessage("[404 datetime:" . date("Y.m.d H:i:s"). " type:" . $aAtom['atom_type'] . " current:".implode(',', $this->aTypes)."/]","cron");
				}

				
			}
			
		}
		else {
			echo "[FULL atoms:".implode(',', $this->aTypes)." progress:" . ($this->oUtil->getCountOfAtomsInStatus(self::PROCESSING) . "of" . $this->iCountOfParts) ."/]\n";
                        MSF_Logger::logMessage("[FULL atoms:".implode(',', $this->aTypes)." progress:" . ($this->oUtil->getCountOfAtomsInStatus(self::PROCESSING) . "of" . $this->iCountOfParts) ."/]\n","cron");
		}
		
	}
	
	private function updateStartAtAndStatus($aAtom, $iStartAt, $iStatus, $sSessionId, $aParams) {
		$this->oUtil->save(
				$aAtom['atom_type'],
				$aAtom['subject_id'],
				$aParams,
				(int)$aParams['weight'],
				$iStartAt,
				$iStatus,
				$sSessionId,
				$aAtom['atom_id']
				);
	}
	
	/**
	 * @return AtomicOperation
	 */
	private function createAtom($sAtomType, $aParams) {
		return AtomicOperation::create(
					$this->oDb,
					$this->aAtoms[$sAtomType]['filepath'],
					$this->aAtoms[$sAtomType]['classname'],
					array_merge($aParams, $this->aAtoms[$sAtomType]['params'])
					);
	}
	
}

?>