<?php
//pear
include_once 'Archive/Tar.php';
class Updater 
{
	private $sSystemPath		= null;
	
	private $sExtractionPath 	= null;
	
	private $sExecPath			= null;
	
	private $sPatchPath			= null;
	
	private $sZipPath 			= null;
	
	private $sSummPath 			= null;
	
	private $sLogPath			= null;
	
	private $sLockPath			= null;
	
	/**
	 * @var MSF_DB
     */
	private $oDb				= null;
	
	public function __construct($sSystemPath, MSF_DB $oDb)
	{
		$this->sSystemPath		= $sSystemPath;
		$this->sExtractionPath 	= $sSystemPath . "/temp/update";
		$this->sExecPath		= $sSystemPath . "/temp/update/exec/index.php";
		$this->sPatchPath		= $sSystemPath . "/temp/update/patch";
		$this->sZipPath			= $sSystemPath . "/temp/update.tar.gz";
		$this->sSummPath		= $sSystemPath . "/temp/update.md5";
		$this->sLockPath		= $sSystemPath . "/temp/.update";
		$this->sLogPath			= $sSystemPath . "/var/logs/update.log";
		$this->oDb				= $oDb;
		
	}
	
	public function isProcessing() 
	{
		return MSF_File::exists($this->sLockPath);
	}
	
	public function update() 
	{
		if (MSF_File::exists($this->sZipPath) && MSF_File::exists($this->sSummPath)) {
			if ($this->verify()) {
				try {
					MSF_File::write($this->sLockPath, "");
					$this->clear();
					if ($this->unzip()) {
						$this->log("Patching...");
						$this->copy();
						$this->exec();
						
						MSF_File::remove($this->sZipPath);
						MSF_File::remove($this->sSummPath);
						
					}
					
					$this->clear();	
					MSF_File::remove($this->sLockPath);
				}
				catch (Exception $oException) {
					$this->log($oException->getMessage());
				}
				
			}
			
		}
	
	}
	
	
	private function verify()
	{
		$sValid = $this->hash(MSF_File::read($this->sZipPath));
		$sGiven = MSF_File::read($this->sSummPath);
		if ($sValid == $sGiven) {
			$this->log("Verification passed");
			return true;
		}
		else {
			$this->log("Verification failed");
			$this->log("Valid hash: " . $sValid);
			$this->log("Given hash: " . $sGiven);
			return false;
		}
		
	}
	
	
	private function hash($sString)
	{
		return /*md5(*/md5($sString);// . "nit");
	}
	
	private function clear()
	{
		if (MSF_Dir::exists($this->sExtractionPath)) {
			MSF_Dir::remove($this->sExtractionPath);
		}

		$this->log("Extraction path " . $this->sExtractionPath . " cleared");
	}
	
	private function log ($sMsg)
	{
		$oDate = new MSF_Date();
		MSF_File::write($this->sLogPath, $oDate->format("Ymd H:i") . "\t" . $sMsg . "\n", "a+");
	}
	
	private function unzip() 
	{
		
		if (MSF_File::exists($this->sZipPath)) 
		{
	  		if (!MSF_Dir::exists($this->sExtractionPath)) 
	  		{
	    		MSF_Dir::create($this->sExtractionPath);
	    		$this->log("Opening..." . $this->sZipPath);
				$oTar = new Archive_Tar($this->sZipPath, true);
				if ($oTar->extract($this->sExtractionPath)) {
					$this->log("Extracted to... " . $this->sExtractionPath);
					return true;
				}
				else {
					$this->log("Not extracted.");
					return false;
				}
	          	
	      	}
	      	else {
	      		$this->log("Not unziped. Directory is already created");
	      		return false;
	      	}
	      	
	   	}
	   	else {
		   	$this->log("Not unziped. Zip file not exist");
		   	return false;
	   	}
	
	   	
	} 
	
	private function exec()
	{
		$this->log("Exec...");
		try {
			include_once $this->sExecPath;
		}
		catch (Exception $oE) {
			$this->log("Exec failed");
			$this->log($oE->getMessage() . "\n");
			$this->log($oE->getTraceAsString() . "\n");
		}
		
	}
	
	private function copy($sPath = "/")
	{
		foreach (MSF_Dir::read($this->sPatchPath . $sPath, "file") AS $sName)
		{
			$sNewRealFile = $this->sSystemPath . $sPath . "/" . $sName;
			$sOldRealFile = $this->sPatchPath . $sPath . "/" . $sName;
			if (MSF_File::exists($sNewRealFile)) 
			{
				MSF_File::remove($sNewRealFile);
			}
			
			MSF_File::copy($sOldRealFile, $sNewRealFile);
			chmod($sNewRealFile, 0755);
		}
		
		foreach (MSF_Dir::read($this->sPatchPath . $sPath, "dir") AS $sName)
		{
			$sVirtDir = $sPath . "/". $sName;
			$sRealDir = $this->sSystemPath . $sVirtDir;
			if (!MSF_Dir::exists($sRealDir)) {
				MSF_Dir::create($sRealDir);
				chmod($sRealDir, 0755);
			}

			$this->copy($sVirtDir);
		}

	}
	
	public static function test()
	{
		define("CNF_ABS_PATH", "D:/projects/msf.lcl/wwwroot");
		$oU = new Updater(CNF_ABS_PATH . "/temp");
		if (!$oU->isProcessing()) {
			$oU->update();
		}

	}

}

