<?php
set_include_path(get_include_path() . PATH_SEPARATOR . CNF_ABS_PATH);
include_once CNF_ABS_PATH . "/system/utils/MSF_SL.php";
include_once CNF_ABS_PATH . "/system/mvc/MSF_ModelDBSupport.php";
include_once CNF_ABS_PATH . "/modules/references/library/Reference/UserReferenceDB.php";
include_once CNF_ABS_PATH . "/modules/references/library/Reference/UserReference.php";
include_once CNF_ABS_PATH . "/system/utils/MSF_String.php";

class ReferenceImporter extends AtomicOperation
{
	const S_WEIGHT 	= 1;
	
	const R_WEIGHT 	= 2;
	
	const S_TIMEOUT = 90;
	
	const R_TIMEOUT = 90;
	
	public function execute()
	{
		try {
			$this->begin();
		}
		catch (Exception $oE) {
			echo $oE->getMessage() . "\n";
		}
	}
	
	private function begin()
	{
		$Ref = new UserReference(new UserReferenceDB($this->getDb()));
		$Utl = new AtomUtil($this->getDb());
		$Ref->load(1);
		$aRef = $Ref->getAsArray();
		$aRec = $Utl->getDb()->select("*", "atom_type = 'recievemail' OR atom_type = 'sendmail'");
		$aIds = array();
		
		foreach ($aRef as $aItem) {
			$bExSend 	= false;
			$bExRecieve = false;
			$aIds[] 	= $aItem["id"];
			$aSend 		= array();
			$aRecieve 	= array();
			
			if (!empty($aItem["email"]) && !empty($aItem["email_smtp_address"])) {

				$aSend = array(
					"id" 	=> $aItem["id"],
					"host" 	=> $aItem["email_smtp_address"],
					"login" => $aItem["email_smtp_login"],
					"email" => $aItem["email"],
					"pass" 	=> $aItem["email_smtp_password"]
				); 
				if (!empty($aItem["email_pop3_address"]) &&  !empty($aItem["email_pop3_login"]) && !empty($aItem["email_pop3_password"])) {
					$aRecieve 	= array(
						"id" 	=> $aItem["id"],
						"host" 	=> $aItem["email_pop3_address"],
						"login" => $aItem["email_pop3_login"],
						"email" => $aItem["email"],
						"pass" 	=> $aItem["email_pop3_password"],
						"delete_emails" => ($aItem["email_remove_on_recieve"] == "|^|row|^|1") ? 1 : 0
					);
				}
			}
			else {
				$sContent  = MSF_File::read(CNF_ABS_PATH . '/settings.xml');
				$oSettings = MSF_Settings::getInstance($sContent);
				$aSettings 		= $oSettings->getModule("lms");	
				$sSysEmail		= $aSettings["settings"]["system_email"];
				$sSysSmtpServer = $aSettings["settings"]["system_email_smtp_server"];
				$sSysEmailLogin	= $aSettings["settings"]["system_email_login"];
				$sSysEmailPwd	= $aSettings["settings"]["system_email_password"];
				if (!empty($sSysEmail) && !empty($sSysSmtpServer) && !empty($sSysEmailLogin) && !empty($sSysEmailPwd)) {
					$aSend = array(
						"id" 	=> $aItem["id"],
						"host" 	=> $sSysSmtpServer,
						"login" => $sSysEmailLogin,
						"email" => $sSysEmail,
						"pass" 	=> MSF_String::encrypt($sSysEmailPwd)
					);
				} 
			}
			
			foreach ($aRec as $aElement) {
				if ($aItem["id"] == $aElement["subject_id"]) {
					switch ($aElement["atom_type"]) {
						case "sendmail":
							if ($aElement["subject_params"] != $aSend || (int) $aElement["weight"] != self::S_WEIGHT) {
								if (empty($aSend)) {
									$Utl->getDb()->remove($aElement["atom_id"]);
								}
								else if (($aElement["status"] != 1)) {
									$Utl->save(
										$aElement["atom_type"],
										$aElement["subject_id"],
										$aSend,
										self::S_WEIGHT,
										$aElement["start_at"],
										$aElement["status"],
										$aElement["session_id"],
										$aElement["atom_id"]
									);
								}
							}
							$bExSend = true;
							break;
						case "recievemail":
							if ($aElement["subject_params"] != $aRecieve || (int) $aElement["weight"] != self::R_WEIGHT) {
								if (empty($aRecieve)) {
									$Utl->getDb()->remove($aElement["atom_id"]);
								}
								else if (($aElement["status"] != 1)) {
									$Utl->save(
										$aElement["atom_type"],
										$aElement["subject_id"],
										$aRecieve,
										self::R_WEIGHT,
										$aElement["start_at"],
										$aElement["status"],
										$aElement["session_id"],
										$aElement["atom_id"]
									);
								}
							}
							$bExRecieve = true;
							break;
					}
				}
			}
			
			if (!$bExSend && !empty($aSend)) {
				$Utl->save(
					"sendmail",
					$aItem["id"],
					$aSend,
					self::S_WEIGHT, 0,
					0,
					"---"
				);
			}
			if (!$bExRecieve && !empty($aRecieve)) {
				$Utl->save(
					"recievemail",
					$aItem["id"],
					$aRecieve,
					self::R_WEIGHT, 0,
					0,
					"---"
				);
			}
		}
		
		foreach ($aRec as $aItem) {
			if (!in_array($aItem["subject_id"], $aIds)) {
				$Utl->getDb()->remove($aItem["atom_id"]);
			}
		}
	}
}
?>