<?php

defined("__MSF") or die("Direct access is not allowed!");

include_once "Mail.php";
include_once "Mail/smtp.php";
include_once "Mail/mime.php";

include_once "system/mvc/MSF_ModelDBSupport.php";
include_once "system/mvc/MSF_ViewSupportXml.php";
include_once 'system/MSF_Registry.php';
include_once "modules/workflow/library/common/WF_Exception.php";
include_once 'modules/lms/library/LMS_Mail.php';
include_once 'modules/lms/library/LMS_MailDB.php';
include_once 'system/MSF_MsfxSupport.php';
include_once 'library/MSF_DBTree/MSF_DBTree.php';
include_once 'system/utils/MSF_XmlWrapper.php';
include_once 'system/MSF_Locale.php';
include_once 'msfx/userbox/MSF_UserBoxExtension.php';
include_once 'system/MSF_Template.php';
include_once 'system/utils/MSF_String.php';
include_once 'system/utils/MSF_SL.php';
include_once 'system/MSF_Settings.php';
include_once 'modules/main/library/orgStructure/MSF_OrgStructureFacade.php';

include_once "modules/lms/library/LMS_MailDecode.php";
include_once "modules/media/library/MD_MediaDB.php";
include_once "modules/media/library/MD_Media.php";

class LMS_Smtp
{
	private $Mail 			= null;
	
	private $DB 			= null;
	
	private $rError 		= null;
	
	private $iUserId 		= 0;
	
	private $sEmail 		= "";
	
	private $bIsConn 		= false;
	
	private $aImageTypes 	= array();
	
	private $DbConnection	= array();
	
	private $aParams		= array();
	
	public function __construct(LMS_SmtpDB $DB, MSF_DB $DbConnection)
	{
		MSF_Registry::set("MSF_DB", $DbConnection);
		MSF_Registry::set("MSF_Locale", new MSF_Locale('cs/languages/russian.ini'));
		MSF_Registry::set("MSF_UserBox", MSF_UserBox::getInstance());
		MSF_Registry::set("MSF_Template", new MSF_Template());
		MSF_Registry::set("MSF_Settings", MSF_Settings::getInstance('<document></document>'));
		$this->oLc				= new MSF_Locale('modules/lms/languages/russian.ini');
		
		$this->DB 			= $DB;
		$this->DbConnection	= $DbConnection;
		$this->imageTypes();
	}
	
	public function connect($iUserId, $sHost, $sLogin, $sEmail, $sPassword, $iTimeout = 30)
	{
		if ($this->DB->exists($iUserId)) {
			$aHost 					= parse_url($sHost);
			$aParams 				= array();
			$aParams["auth"] 		= true;
			$aParams["username"] 	= $sLogin;
			$this->iUserId 			= $iUserId;
			$this->sEmail 			= $sEmail;
			$aParams["password"] 	= MSF_String::decrypt($sPassword);
			if (strlen($aParams["password"]) == 0) {
				$aParams["auth"] 	= false;
			}
			$aParams["timeout"] 	= $iTimeout;
			
			$aParams["persist"] 	= true;
			
			if (isset($aHost["path"])) {
				$aParams["host"] = $aHost["path"];
			}
			else {
				$aParams["host"] = $aHost["host"];
			}
			
			if (isset($aHost["port"])) {
				$aParams["port"] = $aHost["port"];
			}
			
			if (isset($aHost["scheme"]) && ($aHost["scheme"] == "tls" || $aHost["scheme"] == "ssl")) {
				$aParams["host"] = $aHost["scheme"] . "://" . $aParams["host"];
			}
			
			$this->aParams	= $aParams;
			$this->Mail 	= Mail::factory("smtp", $aParams);

			$this->bIsConn 	= true;
		}

	}
	
	public function sendMessages()
	{
		$aSentFiles 	= array();
		$iSentMessages  = 0;
		if ($this->bIsConn) {
			$aMessages = $this->DB->open($this->iUserId);
			$LmsMail 		  = new LMS_Mail(new LMS_MailDB($this->DbConnection));
			$oStructureFacade = MSF_OrgStructureFacade::getInstance();
			$oUserBox		  = MSF_UserBox::getInstance();
			$oUserBox->load($this->iUserId);
			for ($i = 0; $i < sizeof($aMessages); $i++) {				
				$Mime 		= new Mail_mime("\r\n");
				$aHeaders 	= $this->parseHeaders(base64_decode($aMessages[$i]["headers"]));
				$aFileIds 	= array();
				$sBody 		= stripslashes($aMessages[$i]["body"]) . "<p style=\"color: #FFF; display: none\">" . $aMessages[$i]["identifier"] . "</p>";
				$Mime->setFrom($aHeaders["from"]);
				$Mime->setSubject($aHeaders["subject"]);
				if (isset($aHeaders["cc"])) {
					$Mime->addCc($this->prepareTo($aHeaders["cc"]));
				}
				
				if (isset($aHeaders["bcc"])) {
					$Mime->addBcc($this->prepareTo($aHeaders["bcc"]));
				}
				
				$Mime->setHTMLBody($sBody);
				$Mime->setTXTBody(strip_tags(preg_replace('/^<br(.*)>$/', "\r\n", $sBody)));
				if ($aMessages[$i]["attachments"] == "Y") {
					$Media 	= new MD_Media(new MD_MediaDB($this->DbConnection));
					$aFiles = $Media->getFiles("lms_1", $aMessages[$i]["email_id"]);
					for ($j = 0; $j < count($aFiles); $j++) {
						$aFileIds[] = $aFiles[$j]["file_id"];
						$sFilePath 	= CNF_ABS_PATH . "/" . $aFiles[$j]["directory"] . "/" . $aFiles[$j]["file_id"];
						$sFileName 	= "=?UTF-8?B?" . base64_encode($aFiles[$j]["name"]) . "?=";
						if (MSF_File::exists($sFilePath)) {
							if (in_array($aFiles[$j]["type"], $this->aImageTypes) && $aMessages[$i]["type"] == "text/html") {
								$Mime->addHTMLImage(file_get_contents($sFilePath), $aFiles[$j]["type"], $sFileName, false);
							}
							else {
								$Mime->addAttachment(file_get_contents($sFilePath), $aFiles[$j]["type"], $sFileName, false);
							}
							
						}
						
					}
					
				}
				
				$aExtra 				= array();
				$aExtra["To"] 			= $this->prepareTo($aHeaders["to"]);				
				$aMail = LMS_MailDecode::getSendArray(
					$Mime->getMessage(
						"\r\n\r\n",
						array(
							"head_encoding" => "base64",
							//"html_encoding" => "base64",
							"html_encoding" => "8bit",
							"html_charset" 	=> "UTF-8",
							"text_charset" 	=> "UTF-8",
							"head_charset" 	=> "UTF-8"
						),
						$Mime->headers($aExtra)
					)
				);
				
				list($r, $h, $b) 	= $aMail;
				try {
					$mail = $this->Mail->send($r, $h, $b);
					if($mail !== true) {
						throw new Exception($mail->getMessage(), $mail->getCode());
					}
					
					$this->DB->sent($aMessages[$i]);
					$aSentFiles[] = $aFileIds;
					$iSentMessages++;
				}
				catch(Exception $e) {					
					print_r($e);
					$sText = '<p><b>Ошибка:</b>' . htmlspecialchars($e->getMessage()) . '</p>';
					$sText .= '<p><b>Backtrace:</b>' . htmlspecialchars($e->getTraceAsString()) . '</p>';	
					$sText .= '<p>Отправитель: ' . htmlspecialchars($aMessages[$i]["from_name"] . ' <' . $aMessages[$i]["from_email"] . '>') . '</p>';
					$sText .= '<p>Получатель(и): ' . htmlspecialchars($aMessages[$i]["to_email"]) . '</p>';
					$sText .= '<p>Сообщение:</p>';
					$sText .= $aMessages[$i]["body"];
					$sSubject = 'Ошибка отправки письма - ' . $aMessages[$i]["subject"];
					$this->DB->saveError($sText, $sSubject, strlen($sText) + strlen($sSubject));
					$this->DB->sent($aMessages[$i]);
				}
				
			}
			
			try {
				$this->Mail->disconnect();
			}
			catch (Exception $e) {
				print_r($e);
			}
			
		}
		
		$Media = new MD_Media(new MD_MediaDB($this->DbConnection));
		foreach ($aSentFiles as $aFileIds) {
			$Media->saveModuleName($aFileIds, "lms_2");
		}
	}
	
	private function prepareTo($sTo)
	{
		$sRes = "";
		$aTmp = explode(",", $sTo);
		
		for ($i = 0; $i < count($aTmp); $i++) {
			if (substr_count($aTmp[$i], "@") != false) {
				$sRes .= trim($aTmp[$i]) . ",";
			}
		}
		
		return substr($sRes, 0, strlen($sRes) - 1);
	}
	
	private function parseHeaders($sHeaders)
	{
		$aHeaders 	= array();
		$aTmp 		= split("\r\n", $sHeaders);
		
		for ($i = 0; $i < sizeof($aTmp); $i++) {
			$iPos = strpos($aTmp[$i], ":");
			$k = strtolower(substr($aTmp[$i], 0, $iPos));
			$v = substr($aTmp[$i], ($iPos + 1));
			
			/*
			if ($k == "to" || $k == "cc" || $k == "bcc") {
				$aEmails = explode(",", $v);
				$aResult = array();
				
				for ($j = 0; $j < count($aEmails); $j++) {
					if (strstr($aEmails[$j], "@") != false) {
						$aResult[] = trim($aEmails[$j]);
					}
				}
				
				$v = implode(", ", $aResult);
			}
			*/
			
			if (!empty($v)) {
				$aHeaders[$k] = stripslashes($v);
			}
		}
		
		return $aHeaders;
	}
	
	private function imageTypes()
	{
		$this->aImageTypes[] = "image/jpeg";
		$this->aImageTypes[] = "image/pjpeg";
		$this->aImageTypes[] = "image/jpg";
		$this->aImageTypes[] = "image/gif";
		$this->aImageTypes[] = "image/png";
		$this->aImageTypes[] = "image/x-png";
		//$this->aImageTypes[] = "image/bmp";
		//$this->aImageTypes[] = "image/x-windows-bmp";
		//$this->aImageTypes[] = "image/tiff";
		//$this->aImageTypes[] = "image/x-tiff";
	}
}

?>