MSFW.DataSource.AjaxTable	= Class.create(MSFW.DataSource, {
	
	EVENT_TYPES			: ["loadstart", "loadend", "refreshposition"],
	
	ASC_ORDER_IMAGE		: "/templates/default/images/arrow_small_down.png",
	
	DESC_ORDER_IMAGE	: "/templates/default/images/arrow_small_up.png",
	
	NONE_ORDER_IMAGE	: "/templates/default/images/arrow_small.png",
	
	TABLE_CLASS			: "RSMiniRightPart",
	
	PAGINATION_CLASS	: "",
	
	CONTAINER_CLASS		: "RSMiniContentScroll",
	
	initialize : function(options)
	{		
		MSFW.DataSource.prototype.initialize.apply(this, arguments);
		
		this.Options		= Object.extend({			
			"url"			: "",
			"params"		: "",
			"multiselect"	: false,
			"idColumn"		: "id"
		}, this.Options || {});
						
		this.Table				= new Element("table", {"cellpadding":"5", "cellspacing":"0"});
		this.Table.addClassName(this.TABLE_CLASS);
		this.TableContainer		= new Element("div");
		this.TableContainer.addClassName(this.CONTAINER_CLASS);
		this.Pagination			= new Element("div");
		this.Pagination	.addClassName(this.PAGINATION_CLASS);
		this.inLoad				= false;			
		this.Data				= null;
		this.LastRow			= null;
		
		this.allRows			= $A();
		
		
		this.FiltersParams		= {};
		this.GroupsParams		= {"get":{}, "post":{}};
		this.OtherParams		= {};
		
		this.TableContainer.insert(this.Table);
		this.Container.insert(this.TableContainer).insert(this.Pagination);								
	},
	
	getData : function()
	{
		return this.Data;
	},
	
	getTitle : function()
	{
		return this.title;
	},
	
	setFilters : function(value)
	{			
		this.FiltersParams	= value;		
	},
	
	setGroups : function(method, value)
	{
		this.GroupsParams[method]	= value;
	},
					
	reload : function()
	{		
		this.load(this.OtherParams);
	},
	
	load : function(getParams)
	{		
		this._fireEvent("loadstart");
		
		var me			= this;
		var postParams	= this.Options.params;
		var url			= this.Options.url;		

		if((!getParams || !getParams.rs_orderBy) && this.Options.defaultOrder) {
			if(!getParams) {
				getParams	= {};
			}
			
			getParams.rs_orderBy		= this.Options.defaultOrder;
			getParams.rs_orderDirection	= "asc";
		}
		
		getParams	= Object.extend({}, getParams || {});
		url		+= "&" + Object.toQueryString(getParams);
		url		+= "&" + Object.toQueryString(this.FiltersParams);
		url		+= "&" +  Object.toQueryString(this.GroupsParams.get);
		
		postParams	+= "&" +  Object.toQueryString(this.GroupsParams.post);
		
		this.OtherParams	= getParams;
		new Ajax.Request(
			url,
			{
				method 		: "POST",
				parameters 	: postParams,
			  	onSuccess 	: function(resp) {
					try	{
						me.title 	 = resp.responseText.evalJSON().title;
						me.Data 	 = resp.responseText.evalJSON().data;
						me.error	 = resp.responseText.evalJSON().error;
						me.reference = resp.responseText.evalJSON().reference;
					}
					catch(err){
						document.write(resp.responseText);
					}
					
					if(me.Data && !me.Data.denied) {
						me.allRows.clear();
						me._clearTable();
						me._buildTable();
						me._buildPagination();
						me._fireEvent("refreshposition");
					}	

					me._fireEvent("loadend");
				}
			
			}
			
		);
		
	},
	
	_draw : function() {
		if(this.Data) {
			this.allRows.clear();
			this._clearTable();
		//	this._fireEvent("loadend");
			this._buildTable();
			this._buildPagination();
		}

	},
	
	_clearTable : function()
	{	
		var me	= this;
		
		for(var i=($A(this.Table.childNodes).size()-1); i>=0; i--) {		
			$(this.Table.childNodes[i]).remove();
		}

		
	},
	
	_buildTable : function()
	{ 	
		var me				= this;
		var rowCount		= 1;
		var cellCount		= 0;				
		
			
		var tHead		= new Element("thead");
		var TitlesRow 	= new Element("tr");
		this.Table.insert(tHead.insert(TitlesRow));
		
		if(this.Options.multiselect) {			
			var CheckAll	= new Element("input", {"type":"checkbox"});
			var oThis = this;
			CheckAll.onclick = function () {
				oThis._selectAll(oThis, CheckAll);
			}
		}
		else {
			var CheckAll	= "&#160;";
		}
		
		
		var Cell		= new Element("th").insert(CheckAll);
		
		TitlesRow.insert(Cell);		
		
		$A(this.Data.titles).each(function(e) {										
			if(!e.name.include(me.Options.idColumn) && typeof(e.hidden) == 'undefined'){			
				var Cell		= new Element("th");
				TitlesRow.insert(Cell);
								
				var A		= new Element("a", {"class":"title", "onclick":"return false;"});
				var Nobr	= new Element("nobr");
				var Img		= new Element("img");
				var order	= "asc";
								
				switch(e.orderBy) {					
					case "asc"	:
						Img.src	= me.ASC_ORDER_IMAGE;
						order	= "desc";
						
						break;
					case "desc"	:
						Img.src	= me.DESC_ORDER_IMAGE;
						order	= "asc";					
					
						break;
					default		:
						Img.src	= me.NONE_ORDER_IMAGE;
						order	= "asc";
						break;
				}
		
				Nobr.insert(e.title).insert("&#160;").insert(Img);
				A.insert(Nobr);
								
				var params	= me._getLastParams();
				params["rs_orderBy"]		= e.name;
				params["rs_orderDirection"]	= order;
				params["rs_limitFrom"]		= 0;
				
				Cell.observe("click", me.load.bind(me, params));
				
				Cell.insert(A);
				cellCount++;
			}
						
		});
		
		var tBody = new Element("tbody");
		this.Table.insert(tBody);	
		$A(this.Data.records).each(function(e){												
			var Row		= new Element("tr");			
			tBody.insert(Row);
			var id		= e.columns[0].value;						
			cellCount	= 0;
			rowCount++;
			
			var Checkbox	= new Element("input", {"type":"checkbox"});
			var Cell		= new Element("td").insert(Checkbox);
			Row.insert(Cell);
			
			$A(e.columns).each(function(k){								
				if(!k.name.include(me.Options.idColumn) && typeof(k.hidden) == 'undefined') {					
					var Cell		= new Element("td");
					Row.insert(Cell);
					if (k.value.length > 0) {
						Cell.insert(k.value);
					}
					else {
						Cell.insert("&#160;");	
					}				
					
				}
				
			});			
									
			Row.observe("mouseover", me._markRow.bind(me, Row, true));
			Row.observe("click", me._select.bind(me, Row, e.columns, Checkbox, true));
			/*
			Checkbox.observe("click", function(event){
				//me._select.bind(me, Row, e.columns, Checkbox, false)
				console.log(event);
			}.bindAsEventListener(me));*/
			
			
			me.allRows.push({"row":Row, "data":e.columns, "checkbox":Checkbox});
			var bCheck	= true;
			if(!me.Options.multiselect && me.LastRow && id != me.LastRow.id) {
				bCheck	= false;	
			}
			
			if(me.Values[id]) {
				if(bCheck) {								
					delete me.Values[id];
					me._select(Row, e.columns, Checkbox, true);
				}
				else {
					delete me.Values[id];
				}
								
			}		
			
		});
		
		this.iMaxWidth	= 420; 
		this.iMaxHeight	= 370;
		    
		if (this.TableContainer.getWidth() > this.iMaxWidth) {
			this.TableContainer.style.width 	= this.iMaxWidth + "px";
			this.TableContainer.style.maxWidth 	= this.iMaxWidth + "px";
		}

		if (this.TableContainer.getHeight() > this.iMaxHeight) {
			this.TableContainer.style.maxHeight = this.iMaxHeight + "px";
		}
		
	},
	
	_buildPagination : function()
	{
		this.Pagination.update("");
		
		var Table	= new Element("table");
		this.Pagination.insert(Table);
		Table.style.width = "100%";
		Table.cellpadding = "0";
		Table.cellspacing = "0";
		Table.addClassName("RSMiniPagesPanel");
		
		var Row		= $(Table.insertRow(0));
		
		var me		= this;
		var Total	= $(Row.insertCell(0)).insert(aGlobalMsfWords["RS_TOTAL_PAGES"] + " " + this.Data.pagination.totalPages);
		var Pages	= $(Row.insertCell(1)).addClassName("RSMiniPages");
				
		var firstPageParams		= me._getLastParams();
		var previusPageParams	= me._getLastParams();
		var lastPageParams		= me._getLastParams();
		var nextPageParams		= me._getLastParams();
		
		firstPageParams["rs_limitFrom"]		= 0;
		previusPageParams["rs_limitFrom"]	= this.Data.pagination.previusPage;
		lastPageParams["rs_limitFrom"]		= this.Data.pagination.lastPage;
		nextPageParams["rs_limitFrom"]		= this.Data.pagination.nextPage;
		

		if(this.Data.pagination.previusPage != -1) {			
			var First	= new Element("img", {"src":"/templates/default/images/win-rs/rs-pages-first.png", "align":"absmiddle"});	
			First.observe("click", me.load.bind(me, firstPageParams));
			Pages.insert(new Element("span", {style:"background:#F5F5F5;"}).insert(First));
			
			var Previus	= new Element("img", {"src":"/templates/default/images/win-rs/rs-pages-previous.png", "align":"absmiddle"});	
			Previus.observe("click", me.load.bind(me, previusPageParams));
			Pages.insert(new Element("span", {style:"background:#F5F5F5;"}).insert(Previus));
		}
				
		$A(this.Data.pagination.pages).each(function(e){
			if(e.currentPage == 1) {
				var Page	= new Element("span");
			}
			else {
				var Page	= new Element("a", {"style":"cursor:pointer"});
				
				var params	= me._getLastParams();
				params["rs_limitFrom"]	= e.limitFrom;
								
				Page.observe("click", me.load.bind(me, params));				
			}
			
			Page.insert(e.number);
			Pages.insert(Page);						
		});
		
		if(this.Data.pagination.nextPage != -1) {			
			var Next	= new Element("img", {"src":"/templates/default/images/win-rs/rs-pages-next.png", "align":"absmiddle"});	
			Next.observe("click", me.load.bind(me, nextPageParams));
			Pages.insert(new Element("span", {style:"background:#F5F5F5;"}).insert(Next));
			
			var Last	= new Element("img", {"src":"/templates/default/images/win-rs/rs-pages-last.png", "align":"absmiddle"});	
			Last.observe("click", me.load.bind(me, lastPageParams));
			Pages.insert(new Element("span", {style:"background:#F5F5F5;"}).insert(Last));
		}		
	
	},
	
	_getLastParams : function()
	{			
		var ret	= {};
		$H(this.OtherParams).each(function(e){
			ret[e.key]	= e.value;
		});
		
		return ret;
	},
	
	_markRow : function(Row, flag)
	{
		var me	= this;
		this._changeRowClassName(Row, "RSMiniRowOver", flag);
		if(flag) {			
			this._UnMarkFunction	= function(){me._markRow(Row, false)}.bindAsEventListener(this);
			Row.observe("mouseout", this._UnMarkFunction);
		}
		else {			
			Row.stopObserving("mouseout", this._UnMarkFunction);
			this._UnMarkFunction	= null;
		}
		
	},
	
	_selectAll : function(oThis, CheckAll)
	{
		var me		= this;
		oThis.allRows.each(function(e){
			if (CheckAll.checked) {
				e.checkbox.checked = true;
				var id	= e.data[0].value;
				me.Values[id]	= e.data;
				me._changeRowClassName(e.row, "RSMiniRowSelected", true);
			}
			else {
				delete me.Values[e.data[0].value];
				e.checkbox.checked = false;
				me._changeRowClassName(e.row, "RSMiniRowSelected", false);
			}
			
		});
		
	},
	
	_select : function(Row, data, Checkbox, flag)
	{	
		var id	= data[0].value;		
		if(this.Values[id]) {
			delete this.Values[id];
			if(flag) {
				Checkbox.checked	= false;
			}
			
			this._changeRowClassName(Row, "RSMiniRowSelected", false);
		}
		else {			
			if(!this.Options.multiselect && this.LastRow) {
				delete this.Values[this.LastRow.id];
				this._changeRowClassName(this.LastRow.row, "RSMiniRowSelected", false);
				this.LastRow.checkbox.checked	= false;
			}
			
			if(flag) {
				Checkbox.checked	= true;
			}
			
			this.Values[id]	= data;
			this._changeRowClassName(Row, "RSMiniRowSelected", true);
			this.LastRow	= {"row":Row, "id":id, "checkbox":Checkbox};
		}
				
	},
	
	_changeRowClassName : function(Row, className, flag)
	{
		if(flag) {
			$A(Row.cells).each(function(e){
				$(e).addClassName(className);
			});	
			
		}
		else {
			$A(Row.cells).each(function(e){
				if (e) {
					$(e).removeClassName(className);
				}
				
			});
			
		}
		
	}
	
});