MSFW.DataSource.AjaxTree	= Class.create(MSFW.DataSource, {
	
	EVENT_TYPES					: ["loadstart", "loadend", "groupclick"],
		
	CONTAINER_CLASS				: "RSMiniContentScroll MSFW_DataSource_AjaxTree",
	
	TREE_CONTAINER_CLASS		: "RSMiniContentScroll",
	
	CLOSED_GROUP_IMAGE			: "/templates/default/images/folder_closed.png",
	
	OPENED_GROUP_IMAGE			: "/templates/default/images/folder_open.png",
	
	PAGINATION_CLASS			: "",
	
	initialize : function(options)
	{
		MSFW.DataSource.prototype.initialize.apply(this, arguments);		
		this.clear();
		this.Container.addClassName(this.CONTAINER_CLASS);
	},
	
	clear : function()
	{
		this.Groups				= {};
		this.Data				= {};
		this.levels				= 0;
		this.AllRecords			= {};
		this.LastGroup			= null;		
		this.treeLoaded			= false;
		this.LastOptions		= {"get":{}, "post":{}};
		this.FiltersParams		= {};					
		this.Container.update();
	},
	
	setParam : function(method, name, value)
	{
		this.LastOptions[method][name]	= value;
	},
	
	setFilters : function(value)
	{		
		this.FiltersParams	= value;		
	},
		
	loaded : function()
	{
		return this.treeLoaded;
	},	
	
	getData : function()
	{
		return this.Data;
	},
	
	getLastGroupValue : function()
	{		
		if(this.LastGroup) {
			return  this.LastGroup.value;
		}
		else {
			return {};
		}
					
	},
	
	reload : function()
	{
		this.load(this.Groups);
	},
	
	load : function(values)
	{
		var me			= this;
		this.Groups		= values;
		this.levels		= 0;
		this.treeLoaded	= true;		
		this.Container.update();
		$H(this.Groups).each(function(e){
			if(e.value == 1) {
				me.levels++;
			}
			
		});

		this._load(this.Container, {});
	},
	
	_load : function(container, groupValues, limit)
	{
		this._fireEvent("loadstart");
		var me		= this;
		var params	= this.Options.params + "&" + Object.toQueryString(groupValues);
		var url		= this.Options.url;
		this.Container.scrollTop = 0;		
		url			+= Object.toQueryString(this.Groups);	
		url			+= "&" + Object.toQueryString(this.FiltersParams);
		if(limit) {
			url		+= "&" + limit;
		}

		new Ajax.Request(
			url,
			{
				method 		: "post",
				parameters 	: params,
			  	onSuccess 	: function(resp){						
					try	{
						me.Data = resp.responseText.evalJSON().data;
					}
					catch(err){
						window.location.href	= window.location.href;
					}
					
					me._onDataLoad(container, groupValues);					
					me._fireEvent("loadend");
				}
			
			}
			
		);
		
	},
	
	_onDataLoad : function(container, groupValues)
	{
		container.update();		
		this._draw(container, groupValues);		
		this._buildPagination(container, groupValues);
	},
	
	_draw : function(container, groupValues)
	{
		var me			= this;
		var Ul			= new Element("ul");
		var lastGroup	= false;
		container.insert(Ul);
		if(Object.isArray(this.Data.groupValues)) {
			if((this.levels - 1) == 0) {
				lastGroup	= true;
			}
			else {
				lastGroup	= false;
			}
			
		}
		else {
			((this.levels - 1) == $H(this.Data.groupValues).size()) ? lastGroup = true : lastGroup = false;
		}

		$A(this.Data.records).each(function(e){
			var Li	= new Element("li");
			Ul.insert(Li);
			var Nobr	= new Element("nobr");
			Li.insert(Nobr);
			var Div	= new Element("div");			
			Nobr.insert(Div);
			var Img	= new Element("img", {"src":me.CLOSED_GROUP_IMAGE});
			Div.insert(Img);
			var colVal = e.columns[0].value.replace(/&quot;/g, '"');
			if(colVal) {
				var val	= colVal.truncate(23);
			}
			else {
				var val	= "&#160";
			}

			var Content	= new Element("span", {"class":"content", "title":"test"});
			var Count	= new Element("span").insert(e.total_in_group);			
			Content.insert(val).insert(Count).setAttribute("title", colVal);
			Count.addClassName("count");
			Div.insert(Content);
			Div.style.cursor = "pointer";
			var tmp	= {};		
			tmp["rs_groupValues[" + e.columns[0].name + "]"]	= e.columns[0].original.replace(/\+/g, "&#43;");
			var newValues	= Object.extend(tmp, groupValues);
			var NextGroupContainer	= new Element("span");
			Li.insert(NextGroupContainer);			
			Div.observe("mouseover", function(){me._mark(Div, true)});
			if(lastGroup) {
				Div.observe("click", function(){
					if(me.LastGroup) {
						me.LastGroup.img.src	= me.CLOSED_GROUP_IMAGE;
						me.LastGroup.div.removeClassName("selected");
					}
					
					me.LastGroup	= {
						"img"	: Img,
						"div"	: Div,
						"value"	: newValues
					};
					
					Img.src			= me.OPENED_GROUP_IMAGE;
					Div.addClassName("selected");
					me._fireEvent("groupclick");
				});
				
			}
			else {
				Div.observe("click", function(){
					me._toggleGroup(Li, Img, NextGroupContainer, newValues);					
				});
				
			}
		
		});
		
		this.iMaxWidth 		= (window.innerWidth) ? window.innerWidth - 328 : (document.body.clientWidth - 328);
		if (window.innerHeight)  
		    this.iMaxHeight = window.innerHeight - 328;  
		else if (document.documentElement && document.documentElement.clientHeight)  
		    this.iMaxHeight = document.documentElement.clientHeight - 328;  
		else if (document.body)  
		    this.iMaxHeight = document.body.clientHeight - 328;

		if (this.Container.getWidth() > this.iMaxWidth) {
			this.Container.style.width 	= this.iMaxWidth + "px";
			this.Container.style.maxWidth 	= this.iMaxWidth + "px";
		}
	/*	
		if (this.Container.getHeight() > this.iMaxHeight) {
		//	this.TableContainer.style.height	= this.iMaxHeight + "px";
			this.Container.style.maxHeight = this.iMaxHeight + "px";
		}	
		*/
	},
	
	_buildPagination : function(Container, groupValues)
	{	
		if(this.Data.pagination.totalPages <= 1) {
			this.Options.page.update();	
			return;
		}
		
		var Pagination	= new Element("div");
		this.Options.page.update(Pagination);
		Pagination.addClassName(this.PAGINATION_CLASS);
		var Table	= new Element("table");
		var Pages	= new Element("div");
		Table.insert(new Element("tr").insert(new Element("td").insert(Pages)));
		Pagination.insert(Table);
		Pages.addClassName("RSMiniPages");
		Pages.style.paddingTop		= "5pt";
		Table.style.width			= "100%";
		Table.style.height			= "100%";
		Table.style.padding 		= "5px";
		Table.style.height 			= "35px";
		Table.style.backgroundColor = "#F5F5F5";
		Table.style.border 			= "1px solid #CCCCCC";
		Table.style.whiteSpace 		= "nowrap";


		var Row		= $(Table.insertRow(0));

		
		var me		= this;
		//var Total	= $(Row.insertCell(0)).insert(aGlobalMsfWords["RS_TOTAL_PAGES"] + " " + this.Data.pagination.totalPages);
		

		
		
		if(this.Data.pagination.previusPage != -1) {			
			var First	= new Element("img", {"src":"/templates/default/images/win-rs/rs-pages-first.png", "align":"absmiddle"});	
			First.observe("click", me._load.bind(me, Container, groupValues, "rs_limitFrom=0"));
			Pages.insert(new Element("span", {style:"background:#F5F5F5;"}).insert(First));
			
			var Previus	= new Element("img", {"src":"/templates/default/images/win-rs/rs-pages-previous.png", "align":"absmiddle"});	
			Previus.observe("click", me._load.bind(me, Container, groupValues, "rs_limitFrom=" + this.Data.pagination.previusPage));
			Pages.insert(new Element("span", {style:"background:#F5F5F5;"}).insert(Previus));
		}
		
		$A(this.Data.pagination.pages).each(function(e){			
			if(e.currentPage == 1) {
				var Page	= new Element("span");
			}
			else {
				var Page	= new Element("a", {"style":"cursor:pointer"});				
								
				Page.observe("click", me._load.bind(me, Container, groupValues, "rs_limitFrom=" + e.limitFrom));				
			}
			
			Page.insert(e.number);
			Pages.insert(Page);						
		});
		
		if(this.Data.pagination.nextPage != -1) {					
			var Next	= new Element("img", {"src":"/templates/default/images/win-rs/rs-pages-next.png", "align":"absmiddle"});	
			Next.observe("click", me._load.bind(me, Container, groupValues, "rs_limitFrom=" + this.Data.pagination.nextPage));
			Pages.insert(new Element("span", {style:"background:#F5F5F5;"}).insert(Next));
			var Last	= new Element("img", {"src":"/templates/default/images/win-rs/rs-pages-last.png", "align":"absmiddle"});	
			Last.observe("click", me._load.bind(me, Container, groupValues, "rs_limitFrom=" + this.Data.pagination.lastPage));
			Pages.insert(new Element("span", {style:"background:#F5F5F5;"}).insert(Last));
		}	
				
	},
	
	_mark : function(Div, flag)
	{
		var me	= this;
		if(flag) {
			Div.addClassName("marked");
		}
		else {
			Div.removeClassName("marked");
		}

		if(flag) {			
			this._UnMarkFunction	= function(){me._mark(Div, false)}.bindAsEventListener(this);
			Div.observe("mouseout", this._UnMarkFunction);
		}
		else {			
			Div.stopObserving("mouseout", this._UnMarkFunction);
			this._UnMarkFunction	= null;
		}
		
	},
	
	_toggleGroup : function(Li, Img, container, groupValues)
	{
		var id	= Li.identify();
		if(this.AllRecords[id]) {
			if(this.AllRecords[id].groupContainer.visible()) {
				this.AllRecords[id].groupContainer.hide();
				this.AllRecords[id].img.src	= this.CLOSED_GROUP_IMAGE;
			}
			else {
				this.AllRecords[id].groupContainer.show();
				this.AllRecords[id].img.src	= this.OPENED_GROUP_IMAGE;
			}
			
		}
		else {
			this.AllRecords[id]	= {
				"groupContainer"	: container,
				"img"				: Img
			};
			
			this.AllRecords[id].img.src	= this.OPENED_GROUP_IMAGE;
			this._load(container, groupValues);
		}
		
	}
	
});