var MSF_DocumentPlaceholder = Class.create();
MSF_DocumentPlaceholder.prototype =
{
	initialize: function(sPlaceId, sInputName, sDefault, sClassName)
	{
		this.Place 		= $(sPlaceId);
		this.sInputname = sInputName;
		this.Window 	= null;
		this.sDefault 	= (sDefault) ? sDefault : "";
		this.sClassName = sClassName;
		
		this.sRowDelim 	= '|^|row|^|';
		this.sColDelim 	= '/^/col/^/';

		this.iDocId 	= 0;
		this.iFoldId 	= 0;
		
		this._buildArea();
	},
	
	_buildArea: function()
	{
		var me 		= this;
		//this.Input 	= new Element("textarea", {"readOnly": true});
		this.Input 	= new Element("input", {"readOnly": true});
		this.Hidden = new Element("input", {"type": "hidden", "name": this.sInputname, "value": this.sDefault});
		this.Button = new Element("img", {src: "/modules/workflow/view/templates/default/img/documents.png"});
		this.Input.value = this._parseValue(this.sDefault);
		
		if (this.sClassName) {
			this.Input.addClassName(this.sClassName);
		}
		else {
			this.Input.addClassName("doc-placeholder-input");
		}
		
		this.Button.style.cursor = "pointer";
		this.Button.onclick = function()
		{
			if (!me.Window) {
				me.Window = new MSF_DocumentWindow(
					me.Button,
					me.handler,
					me
				);
			}
			else {
				me.Window.show();
			}
		}
		
		this.Place.insert(this.Hidden);
		this.Place.insert(this.Input);
		this.Place.insert(this.Button);
		
		this._loadWindow(this.Hidden.value);
	},
	
	_loadWindow: function(sDocument)
	{
		if (sDocument.strip() != "") {
			var aTmp = sDocument.split(this.sColDelim);
			
			this.iDocId 	= aTmp[1];
			this.iFoldId 	= aTmp[2];
			this.Window 	= new MSF_DocumentWindow(
				this.Button,
				this.handler,
				this
			);
		}
	},
	
	_parseValue: function(sDefault)
	{
		var iLength 	= 0;
		var aTmp 		= [];
		var sResult 	= "";
		var aDocuments 	= sDefault.split(this.sRowDelim);
		
		for (var i = 0; i < aDocuments.length; i++) {
			aTmp 	= aDocuments[i].split(this.sColDelim);
			iLength = aTmp.length;
			sResult += aTmp[0] + ", ";
		}
		
		return sResult.substr(0, sResult.length - 2);
	},
	
	handler: function(self, aDoc)
	{
		self.Hidden.value 	= "";
		self.Input.value 	= "";
		
		if (aDoc) {
			self.Hidden.value 	= aDoc.title + self.sColDelim + aDoc.id + self.sColDelim + aDoc.fid;
			self.Input.value 	= aDoc.title;
		}
	}
}

var MSF_DocumentWindow = Class.create();
MSF_DocumentWindow.prototype =
{
	initialize: function(mPlace, handler, Opener)
	{
		this.Place 		= $(mPlace);
		this.Window 	= null;
		this.Json 		= null;
		this.aOpened 	= new Array();
		this.handler 	= handler;
		this.Opener 	= Opener;
		this.aFolders 	= new Array();
		
		this.iFolderId 	= 0;
		
		this._buildWindow();
	},
	
	setTitle: function(sTitle)
	{
		this.Window.setTitle(sTitle);
	},
	
	_buildWindow: function()
	{
		this._buildContainer();
		this._loadFolders();
		if (!this.Window) {
			var aPos = this.Place.cumulativeOffset();
			this.Window = new MSF_Window();
			
			if (this.Opener.iFoldId > 0) {
				this.Window.hide();
			}
			
			this.Window.setTitle(aGlobalMsfWords["ROOT_DOCUMENT_WINDOW_TITLE"]);
			this.Window.moveTo(aPos[0], aPos[1]);
			this.Window.appendToBody(this.Cont);
		}
		else {
			this.show();
		}
	},
	
	_loadFolders: function()
	{
		this.Loader.show();
		var me = this;
		new Ajax.Request(
			"/workflow/ajaxfolders",
			{
				method 		: "post",
				onSuccess 	: function(resp) {
					me.aFolders = resp.responseText.evalJSON().folders;
					me._buildFoldersSelect();
				},
				onComplete 	: function() { me.Loader.hide(); }
			}
		);
	},
	
	_buildContainer: function()
	{
		var Table, Td, Tr;
		
		Table 		= new Element("table", {"class": "doc-placeholder-folders"});
		this.Cont 	= new Element("div");
		this.Submit = new Element("input", {"type": "button", "value": "OK"});
		this.Loader = new Element("img", {"src": "/templates/default/images/loader.gif"});
		this.Action = new Element("div", {"style": "padding: 5px"});
		this.Table 	= new Element("div", {"class": "doc-placeholder"});
		this.Select = new Element("select");
		
		this.Loader.hide();
		Tr = $(Table.insertRow(0));
		Td = $(Tr.insertCell(0));
		Td.width = "20%";
		Td.insert(aGlobalMsfWords["ROOT_DOCUMENT_WINDOW_FOLDER"]);
		Td = $(Tr.insertCell(1));
		Td.width = "80%";
		Td.insert(this.Select);
		Td.insert(this.Loader);
		this.Cont.insert(Table);
		
		Table = new Element("table", {"class": "doc-placeholder-docs"});
		Tr = $(Table.insertRow(0));
		Td = $(Tr.insertCell(0));
		Td.insert(this.Action);
		Td.insert(this.Table);
		
		Tr = $(Table.insertRow(1));
		Td = $(Tr.insertCell(0));
		Td.insert(this.Submit);
		this.Cont.insert(Table);
	},
	
	_buildFoldersSelect: function()
	{
		var Select 	= this.Select;
		var iCount 	= 0;
		var me 		= this;
		
		Select.options[iCount++] = new Option("------------", "");
		$H(this.aFolders).each(function(e) {
			if (e.value) {
				if (e.value.name) {
					Select.options[iCount++] = new Option(e.value.name, "");
					if (e.value.folders && e.value.folders.size() > 0) {
						$A(e.value.folders).each(function (folder) {
							Select.options[iCount] = new Option("-----" + folder.name, folder.folder_id);
							if (me.Opener.iFoldId == folder.folder_id) {
								Select.options[iCount].setAttribute("selected", true);
								me._buildFolderTable(folder.folder_id);
							}
							iCount++;
						});
					}
				}
			}
		});
		
		Select.observe("change", function() {
			if (this.value != "") {
				me._buildFolderTable(this.value);
			}
		});
		
		return Select;
	},
	
	_buildFolderTable: function(iFolderId)
	{
		var bExists = false;
		
		for (var i = 0; i < this.aOpened.length; i++) {
			if (this.aOpened[i].id == iFolderId) {
				bExists 		= true;
				this.Json 		= this.aOpened[i].json;
				this.iFolderId 	= this.aOpened[i].id;
				break;
			}
		}
		
		if (!bExists) {
			this.Loader.show();
			var me = this;
			var callback = function(resp)
			{
				var Obj 		= new Object();
				Obj.id 			= iFolderId;
				Obj.json 		= resp.responseText.evalJSON();
				me.Json 		= Obj.json;
				me.iFolderId 	= iFolderId;
				me.aOpened.push(Obj);
				me._drawActionPanel();
				me._drawTable(iFolderId);
			}
			this._makeRequest(iFolderId, callback);
		}
		else {
			this._drawActionPanel();
			this._drawTable(iFolderId);
		}
	},
	
	_drawActionPanel: function()
	{
		for (var i = this.Action.childNodes.length - 1; i >= 0; i--) {
			this.Action.removeChild(this.Action.childNodes[i]);
		}
		
		var Table = new Element("table");
		this.Action.insert(Table);
		this.buildActionPanel(Table);
	},
	
	_drawTable: function(iFolderId)
	{
		for (var i = this.Table.childNodes.length - 1; i >= 0; i--) {
			this.Table.removeChild(this.Table.childNodes[i]);
		}
		
		var conf, Table, aResult;
		aResult = this.Json;
		
		conf 				= new Object();
		conf.Data 			= aResult.documents;
		conf.aColumns 		= aResult.columns;
		conf.bMultiselect 	= false;
		conf.group 			= this.getSelectGroups();
		conf.filters 		= this.FilterPanel.getFilters();
		
		if (Table) {
			conf.selected = Table.getSelected();
		}
		
		if(conf.group.size() > 0) {			
			Table = new MSF_GroupTable(conf);
		}
		else {
			Table = new MSF_Table(conf);
		}
		
		this.Table.insert(Table.getDOM());
		this._onSubmit(Table);
		
		if (this.Opener.iFoldId == iFolderId && this.Opener.iDocId > 0) {
			Table.setValueById([this.Opener.iDocId]);
		}
	},
	
	_onSubmit: function(Table)
	{
		var me = this;
		this.Submit.onclick = function() {
			var aDocs 	= Table.getSelected();
			var bCheck 	= false;
			var aDoc 	= new Object();
			for (var i = 0; i < me.Json.titles.length; i++) {
				for (var j = 0; j < aDocs.length; j++) {
					if (aDocs[j].id == me.Json.titles[i].id) {
						aDoc 		= aDocs[j];
						aDoc.fid 	= me.iFolderId;
						aDoc.title 	= me.Json.titles[i].title;
						bCheck 		= true;
						break;
					}
				}
				if (bCheck) {
					break;
				}
			}

			me.Window.hide();
			
			if (aDoc.title) {
				me._changeTitle(aDoc);
			}
			else {
				me.handler(me.Opener, null);
			}
		}
	},
	
	_changeTitle: function(aDoc)
	{
		if (!this.PromptWindow) {
			this.PromptWindow = new MSF_Window();
			this.PromptWindow.setTitle(aGlobalMsfWords["ROOT_DOCUMENT_WINDOW_CHANGE_TITLE"]);
			
			this.PromptDiv 		= new Element("div", {"style": "padding: 10px;"});
			this.PromptInput 	= new Element("textarea", {"class": "doc-placeholder-title"});
			this.PromptButton 	= new Element("input", {"type": "button", "value" : "OK"});
			
			this.PromptDiv.insert(this.PromptInput);
			this.PromptDiv.insert(new Element("br"));
			this.PromptDiv.insert(this.PromptButton);
			this.PromptWindow.appendToBody(this.PromptDiv);
		}
		else {
			this.PromptWindow.show();
		}
		
		this.PromptWindow.moveTo(
			document.body.offsetWidth/2 - this.PromptDiv.getWidth()/2,
			document.body.offsetHeight/2 - this.PromptDiv.getHeight()/2
		);
		
		var me = this;
		this.PromptInput.value = aDoc.title;
		this.PromptButton.onclick = function() {
			aDoc.title = me.PromptInput.value; 
			me.handler(me.Opener, aDoc);
			me.PromptWindow.hide();
		}
	},
	
	buildActionPanel : function(Table)
	{
		//var Table 	= new Element("table");
		var Row		= Table.insertRow(0);
		
		var FilterContainer	= new Element("table", {"class" : "rs-action"});
		var FilterRow		= FilterContainer.insertRow(0);
		$(FilterRow.insertCell(0)).addClassName("rs-action-cell-title").insert(aGlobalMsfWords["ROOT_RS_FILTER_TITLE"]);		
		$(Row.insertCell(0)).insert(FilterContainer);
		this.buildFilterPanel($(FilterRow.insertCell(1)).addClassName("rs-action-cell-button").insert("&#160;"));
		this.bBlockFilters ? FilterContainer.hide() : new RS_ActionPanel(FilterContainer);
		
		var GroupContainer	= new Element("table", {"class" : "rs-action"});
		var GroupRow		= GroupContainer.insertRow(0);
		$(GroupRow.insertCell(0)).addClassName("rs-action-cell-title").insert(aGlobalMsfWords["ROOT_RS_GROUP_TITLE"]);		
		$(Row.insertCell(1)).insert(GroupContainer);
		this.buildGroupPanel($(GroupRow.insertCell(1)).addClassName("rs-action-cell-button").insert("&#160;"));
		this.bBlockGroups ? GroupContainer.hide() : new RS_ActionPanel(GroupContainer);
		
		//return Table;
	},
	
	buildGroupPanel : function(MainContainer)
	{
		var me			= this;	
		var Table	= new Element("table");
		
		var Div		= new Element("div", {"class" : "rs-action-panel"}).insert(Table);
		MainContainer.insert(Div);		
		this.GroupPanel	= new Element("form");		
		$(Table.insertRow(0).insertCell(0)).insert(this.GroupPanel);
		
		var Ul = new Element("ul", {"class" : "RD_GM_groupPanel"});
		Ul.identify();
		this.GroupPanel.insert(Ul);
		
		$H(this.Json.columns).each(function(e) {
			if(e.value.bGroup) {
				var Checkbox = new Element("input", {
					name	: "group_" + e.key,
					type	: "checkbox",
					value	: e.key
				});
				
				var Label 	= new Element("label").insert(Checkbox).insert(me.Json.documents.fields[e.key]);
				var Li 		= new Element("li");
				
				Li.identify();
				Li.insert(new Element("span", {"class" : "dragHandler"}).insert("&#160;")).insert(Label);				
				Ul.insert(Li);
			}	
		});
		Sortable.create(Ul);
		
		var Button = new Element("input", { "type": "button", "value": aGlobalMsfWords["ROOT_RS_GROUP_BTN"] });
		
		$(Table.insertRow(1).insertCell(0)).insert(Button);
		Button.observe("click", function(e){			
			e.stop();
			Div.hide();
			me._drawTable();		
		});
		
	},
	
	buildFilterPanel : function(MainContainer)
	{
		var me				= this;
		var Table			= new Element("table");		
		this.FilterPanel	= new MSF_FiltersPanel(this.Json.columns, this.Json.documents.fields, {});		
				
		
		var Div		= new Element("div", {"class" : "rs-action-panel"}).insert(Table);
		MainContainer.insert(Div);		
		$(Table.insertRow(0).insertCell(0)).insert(this.FilterPanel.getDom());
		
		var Button	= new Element("input", {"type": "button", "value": aGlobalMsfWords["ROOT_RS_FILTER_BTN"]})
			.observe("click", function(e){
				e.stop();
				Div.hide();
				me._drawTable(); 
			});
			
		$(Table.insertRow(1).insertCell(0)).insert(Button);
	},
	
	getSelectGroups: function()
	{
		var aChecks 	= Form.getElements(this.GroupPanel);
		var aChecked 	= [];
		aChecks.each(function(e) {
			if(e.checked) {
				aChecked.push(e.value);
			}
		});
		
		return aChecked;
	},
	
	_makeRequest: function(iFolderId, callback)
	{
		var me = this;
		
		new Ajax.Request(
			"/workflow/ajaxdocuments/" + iFolderId,
			{
				method 		: "post",
				onSuccess 	: callback,
				onComplete 	: function() { me.Loader.hide(); }
			}
		);
	},
	
	show: function()
	{
		if (this.PromptWindow) {
			this.PromptWindow.hide();
		}
		this.Window.show();
	}
}