<?php

/**
 *
 * Description
 *
 * File: MSF_DBTree.php
 * Created: 30.07.2007
 * Author: T. Anre
 */
require_once "library/MSF_DBTreeDIL.php";
require_once "library/MSF_DBTreeException.php";

/**
 * @uncompleted
 */
class MSF_DBTree {

    private $DIL = null;
    private $sFieldId = "";
    private $sFieldParentId = "";
    private $aList = array();
    private $bConditions = false;

    public function __construct($DB, $sTableName, $sFieldId, $sFieldParentId, $sFieldPath) {
        $this->sFieldId = $sFieldId;
        $this->sFieldParentId = $sFieldParentId;
        $this->DIL = new MSF_DBTreeDIL($DB, $sTableName, $sFieldId, $sFieldParentId, $sFieldPath);
    }

    /**
     * return array
     * @uncompleted
     */
    public function select($iId, $aOtherFields, $aConditions, $aOrders, $bGetTree = true) {
        $bConditions = false;

        if (!empty($aConditions)) {
            $this->bConditions = true;
            $bConditions = true;
        }

        $aResult = array();
        if ($this->exists($iId)) {
            if (empty($this->aList)) {
                $this->aList = $this->DIL->selectByPathCount($iId, $this->sFieldParentId, $aOtherFields, $aConditions, $aOrders);
            } elseif (!empty($this->aList) && $this->bConditions == $bConditions) {
                $this->aList = $this->DIL->selectByPathCount($iId, $this->sFieldParentId, $aOtherFields, $aConditions, $aOrders);
            }

            if (!empty($this->aList)) {
                if ($bGetTree) {
                    $aResult = $this->makeTree($this->aList, $this->aList[0][$this->sFieldId]);
                } else {
                    $aResult = $this->aList;
                }
            }
            if ($bGetTree) {
                $aResult["nodes"] = $this->sortByPosition($aResult["nodes"]);
            }
        } else {
            throw new MSF_ThrowableException("Node does not exists");
        }

        return $aResult;
    }

    public function sortByPosition($aList) {
        if (array_key_exists('position', $aList[0])) {
            foreach ($aList as $k => $key) {
                if (array_key_exists('nodes', $key)) {
                    $subs = $this->sortByPosition($key['nodes']);
                    $key['nodes'] = $subs;
                    $aList[$k] = $key;
                }
                $position[] = ((int) $key['position']);
            }
            array_multisort($position, SORT_ASC, SORT_NUMERIC, $aList);
        }
        return $aList;
    }

    public function searchByValues($aHash) {
        if (!is_array($aHash)) {
            throw new MSF_DBTreeException("Entered parameter is not an array");
        }

        $aResult = array();

        foreach ($aHash as $k => $v) {
            for ($i = 0; $i < sizeof($this->aList); $i++) {
                $aArray = array();

                if (!isset($this->aList[$i][$k])) {
                    throw new MSF_DBTreeException("Column \"" . $k . "\" not exists");
                }

                if (is_array($v)) {
                    for ($j = 0; $j < sizeof($v); $j++) {
                        if ($v[$j] == $this->aList[$i][$k]) {
                            $aArray = $this->aList[$i];
                        }
                    }
                    if (!empty($aArray)) {
                        $aResult[] = $aArray;
                    }
                } else {
                    if ($this->aList[$i][$k] == $v) {
                        $aResult[] = $this->aList[$i];
                    }
                }
            }
        }

        return $aResult;
    }

    private function makeTree($aDBTree, $iNodeId) {
        $aResult = array();
        foreach ($aDBTree AS $aNode) {
            if ($aNode[$this->sFieldId] == $iNodeId) {
                $aResult = $aNode;
            }
            if ($aNode[$this->sFieldParentId] == $iNodeId) {
                $aResult["nodes"][] = $this->makeTree($aDBTree, $aNode[$this->sFieldId]);
            }
        }

        return $aResult;
    }

    public function exists($iParentId) {
        return $this->DIL->exists($iParentId);
    }

    /**
     * return integer
     * @uncompleted
     */
    public function insert($iParentId, $aOtherFields) {
        if ($this->exists($iParentId)) {
            return $this->DIL->insert($iParentId, $aOtherFields);
        } else {
            throw new MSF_ThrowableException('Node does not exists');
        }
    }

    /**
     * return boolean
     * @uncompleted
     */
    public function update($iNodeId, $aOtherFields) {
        return $this->DIL->update($iNodeId, $aOtherFields);
    }

    /**
     * @uncompleted
     */
    public function move($iNodeId1, $iNodeId2) {
        $aSel1 = $this->DIL->selectAll($iNodeId1);
        $aSel2 = $this->DIL->selectAll($iNodeId2);

        unset($aSel1[$this->sFieldId]);
        unset($aSel2[$this->sFieldId]);

        $this->DIL->update($iNodeId1, $aSel2);
        $this->DIL->update($iNodeId2, $aSel1);
    }

    public function replace($iNode, $iElem) {
        $aNode = $this->DIL->selectAll($iNode);
        $aElem = $this->DIL->selectAll($iElem);

        $aNodePaths = explode(".", $aNode["path"]);
        if (array_search($iElem, $aNodePaths)) {
            $aNode["parent_id"] = $aElem["parent_id"];
            unset($aNodePaths[array_search($iElem, $aNodePaths)]);
            $aNode["path"] = implode(".", $aNodePaths);
        }
        $this->DIL->update($iNode, $aNode);

        $aElem["parent_id"] = $aNode["node_id"];
        $aElem["path"] = $aNode["path"] . "." . $aElem["node_id"];
        $this->DIL->update($iElem, $aElem);

        $aPosts = $this->DIL->selectPosts($iElem);
        foreach ($aPosts as $aPost) {
            $aPost["path"] = $aElem["path"] . "." . $aPost["node_id"];
            $this->DIL->update($aPost["node_id"], $aPost);
        }
    }

    // TODO snesti
    public function replacePath($iNodeId1, $iNodeId2) {

        $sPath1 = $this->DIL->getPos($iNodeId1);
        $sPath2 = $this->DIL->getPos($iNodeId2);
        $bResult1 = $this->DIL->updatePos($iNodeId1, $sPath2);
        $bResult2 = $this->DIL->updatePos($iNodeId2, $sPath1);
        return ($bResult1 && $bResult2);
    }

    public function moveEnd($iThisId, $iLastId) {
        $iThisPos = $this->DIL->getPos($iThisId);
        $iLastPos = $this->DIL->getPos($iLastId);
        $this->DIL->moveEnd($iThisPos, $iLastPos);
        $this->DIL->updatePos($iThisId, $iLastPos);
    }

    public function moveBegin($iThisId, $iFirstId) {
        $iThisPos = $this->DIL->getPos($iThisId);
        $iFirstPos = $this->DIL->getPos($iFirstId);
        $this->DIL->moveBegin($iThisPos, $iFirstPos);
        $this->DIL->updatePos($iThisId, $iFirstPos);
    }

    /**
     * return boolean
     * @uncompleted
     */
    public function remove($iNodeId, $bRemoveChildren = true) {
        // [remove node with children]
        if ($bRemoveChildren) {
            return $this->DIL->removeBranch($this->DIL->getPath($iNodeId), $this->DIL->getPos($iNodeId));
        }
        // [remove only this node]
        else {
            throw new MSF_ThrowableException('public function remove($iNodeId, $bRemoveChildren = false) is not realised');
        }
    }

    public function updatePositionQuery($iNodeId, $iPosition) {
        $this->DIL->updatePositionQuery($iNodeId, $iPosition);
    }

}

//$DBTree = new MSF_DBTree(MSF_Dispatcher::getObject("MSF_DB"), "#__exp_tree", "id", "parent_id", "path");
/*
  $p1 = $DBTree->insert(0, array("title" => "Node 1"));
  $DBTree->insert($p1, array("title" => "Sub Node 1"));
  $p2 = $DBTree->insert(0, array("title" => "Node 2"));
  $DBTree->insert($p2, array("title" => "Sub Node 2"));
  $DBTree->insert($p2, array("title" => "Sub Node 3"));
 */
//debug($DBTree->select(1, array("title"), array()));
?>