<?php
if(!defined("__MSF")) {
	die("Direct access not allowed");
}

class MSF_ExportDoctype {

	private $iDoctypeId 	= 0;

	private $xmlFileName	= "";
	
	public function __construct($iDoctypeId)
	{
		$this->iDoctypeId = $iDoctypeId;
	}
	
	public function download() {

		$this->setXMLFile("doctype_" . $this->iDoctypeId . ".xml");
		$this->saveXML();

		$sFile = $this->getFileName();
		if (file_exists($sFile)) {
		    header('Content-Description: File Transfer');
		    header('Content-Type: application/octet-stream');
		    header('Content-Disposition: attachment; filename=' . basename($sFile));
		    header('Content-Transfer-Encoding: binary');
		    header('Expires: 0');
		    header('Cache-Control: must-revalidate, post-check=0, pre-check=0');
		    header('Pragma: public');
		    header('Content-Length: ' . filesize($sFile));
		    readfile(str_replace(CNF_ABS_PATH . "/", "", $sFile));
		    exit;
		}		
	}
	
	private function createXML()
	{
		$Doctype =	new WF_Doctype($this->iDoctypeId);
		$sOutput = "<?xml version=\"1.0\"?>";

		$aDoctypeProps	= array("Title", "Description", "FullDescription", "FieldsXml", "ComparedXml", "RouteXml");
		$sOutput .=	"<document>" .
					"<element type=\"array\" name=\"doctype\">";
		foreach ($aDoctypeProps as $sName) {
			 $propName	= "get" . $sName;
			 $sContent	= $Doctype->$propName();
			 if ($sName == "FieldsXml") $sContent = $this->removeDefaultValues($sContent);
			 $sItem		= strtolower($sName);
			 $sOutput	.= "<item name=\"$sItem\" value=\"" . htmlspecialchars($sContent) . "\" />";  
		}
		$sOutput .= "</element>";

		$Version = new WF_DoctypeVersion($Doctype, $Doctype->getDoctypeVersionId());
		$aVersionProps	= array("RouteXml", "TemplateXml", "FieldsXml", "SpaXml", "ArchiveXml" , "CreatedAt");
		$sOutput .=	"<element type=\"array\" name=\"version\">";
		foreach ($aVersionProps as $sName) {
			 $propName	= "get" . $sName;
			 $sContent	= $Version->$propName();
			 if ($sName == "FieldsXml") $sContent = $this->removeDefaultValues($sContent);
			 $sItem		= strtolower($sName);
			 $sOutput	.= "<item name=\"$sItem\" value=\"" . htmlspecialchars($sContent) . "\" />";  
		}
		$sOutput .= "</element>";		
		$sOutput .= "</document>";

		return $sOutput;
	}

	private function removeDefaultValues($sValue)
	{
		if ($sValue) {
			$Fields	= new MSF_XmlWrapper($sValue);
			$aData	= $Fields->getAsArray();
			$sValue = "<document>";
			foreach ($aData as $sKey=>$sVal) {
				switch ($aData[$sKey]["value"]["type"]) {
					case "structure":
					case "reference":
						$aData[$sKey]["value"]["defaultValue"] = "";
						break;
				}			
				$sValue .= "<element type=\"array\" name=\"$sKey\" title=\"".$aData[$sKey]["title"]."\" >";
				$sValue .= $this->buildXml($aData[$sKey]["value"]);
				$sValue .= "</element>";
			}
			$sValue .= "</document>";
		}
		return $sValue;
	}

	private function buildXml(array $aData)
	{
		$sResult = '';
		foreach ($aData AS $sKey => $mValue) {
			$sType = $this->getType($mValue);
			switch ($sType) {
				case "array":
					$sResult .= '<item type="' . $sType 
								. '" name="' . $sKey 
								. '">';
					$sResult .= $this->buildXml($mValue);
					$sResult .= '</item>';
					break;
				default:
					$sResult .= '<item name="' . $sKey 
							. '" value="' . $this->esaceTagSigns($mValue) 
							. '"/>';
					break;
			}
			
		}
		
		return $sResult;
	}
	private function esaceTagSigns($sMessage)
	{
		$sMessage = str_replace('&', '&#38;', $sMessage);
		$sMessage = str_replace('<', '&#60;', $sMessage);
		$sMessage = str_replace('>', '&#62;', $sMessage);
		$sMessage = str_replace('"', '&#34;', $sMessage);
		$sMessage = str_replace('*', '&#42;', $sMessage);
		return $sMessage;
	}	
	private function getType($mValue)
	{
		$sType = gettype($mValue);
		switch ($sType) {
			case "integer":
				$sType = "int";
				break;
			case "string":
				$sType = "text";
				break;
		}

		return $sType;
	}
	private function saveXML()
	{
		$sFile = $this->getFileName();
		$sContents = $this->createXML();
		MSF_File::write($sFile, $this->createXML(), "w", 0777);
	}

	private function setXMLFile($sName)
	{
		$aSettings = MSF_Registry::get("MSF_Settings")->getModule("media");
		$sPath = CNF_ABS_PATH . $aSettings["settings"]["export_directory"] . "/";
		$this->removeOldFiles($sPath, "/^doctype.[0-9]+.xml/i");
		$sFile = $sPath . $sName;
		$this->xmlFileName = $sFile;
	}
	
	private function getFileName()
	{
		return $this->xmlFileName; 
	}
	
	private function removeOldFiles($sPath='.', $sMask='*')
	{
		static $aDir = array();
		if (!isset($aDir[$sPath])) {
			$aDir[$sPath] = scandir($sPath);
		}
		foreach ($aDir[$sPath] as $i=>$sEntry) {
			if ($sEntry!='.' && $sEntry!='..' && preg_match($sMask, $sEntry)) {
				$iCreated = filemtime($sPath . $sEntry);
				$iAge = time() - $iCreated;
				$iExpire = 1*60*60; 
				if ($iAge > $iExpire) unlink($sPath . $sEntry);
		 	}
		}
		unset($aDir);
	}
	
}
?>