<?php
if ( !function_exists('sys_get_temp_dir') )
{   
   function sys_get_temp_dir()
   {
       // Try to get from environment variable
       if ( !empty($_ENV['TMP']) )
       {
           return realpath( $_ENV['TMP'] );
       }
       else if ( !empty($_ENV['TMPDIR']) )
       {
           return realpath( $_ENV['TMPDIR'] );
       }
       else if ( !empty($_ENV['TEMP']) )
       {
           return realpath( $_ENV['TEMP'] );
       }
       // Detect by creating a temporary file
       else
       {
           // Try to use system's temporary directory
           // as random name shouldn't exist
           $temp_file = tempnam( md5(uniqid(rand(), TRUE)), '' );
           if ( $temp_file )
           {
               $temp_dir = realpath( dirname($temp_file) );
               unlink( $temp_file );
               return $temp_dir;
           }
           else
           {
               return FALSE;
           }
       }
   }
}

include_once "library/PhpExcel/Classes/PHPExcel.php";
include_once "library/PhpExcel/Classes/PHPExcel/Writer/Excel5.php";

class MSF_ExportToExcel {
	
	private $RecordsCount = 0;
	private $phpExcel	  = null;
	
	public function __construct()
	{
		$this->phpExcel = new PHPExcel();
		$this->phpExcel->setActiveSheetIndex(0);										
	}

	public function exportTitles($aRecordset)
	{	
		for($i = 0; $i < count($aRecordset["titles"]); $i++) {
			$this->phpExcel->getActiveSheet()->setCellValueByColumnAndRow($i, $this->RecordsCount + 1, $this->replaceEntities($aRecordset["titles"][$i]["title"]));
			$oStyle	= $this->phpExcel->getActiveSheet()->getStyleByColumnAndRow($i, $this->RecordsCount + 1);
			$oStyle->getFont()->setBold(true);
			$oStyle->getBorders()->getAllBorders()->setBorderStyle(PHPExcel_Style_Border::BORDER_MEDIUM);
			$this->phpExcel->getActiveSheet()->getColumnDimensionByColumn($i)->setWidth($this->getSizeByType($aRecordset["titles"][$i]["type"]));			
		}
		
	}
	
	public function exportBody($aRecordset)
	{		
		for($i = 0; $i < count($aRecordset["records"]); $i++) {
			for($j = 0; $j < count($aRecordset["records"][$i]["columns"]); $j++) {
				$sValue = strip_tags($aRecordset["records"][$i]["columns"][$j]["value"]);
				$this->phpExcel->getActiveSheet()->setCellValueByColumnAndRow($j, $this->RecordsCount + $i + 2, $this->replaceEntities($sValue));	
				$oStyle	= $this->phpExcel->getActiveSheet()->getStyleByColumnAndRow($j, $this->RecordsCount + $i + 2);
				$oStyle->getBorders()->getAllBorders()->setBorderStyle(PHPExcel_Style_Border::BORDER_MEDIUM);
				$oStyle->getAlignment()->setWrapText(true);
			}
						
		}
		
		$this->RecordsCount += count($aRecordset["records"]);					
	}
	
	public function getSizeByType($sType)
	{
		$iResult = 10;
		switch($sType) {
			case "int" :
				$iResult = 10;
				break;
			case "string" :
				$iResult = 50;
				break;
			case "float" :
				$iResult = 15;
				break;
			case "text" :
				$iResult = 60;
				break;
			case "reference" :
				$iResult = 30;
				break;
			case "date" :								
			case "timestamp" :
				$iResult = 15;
				break;
			case "bool" :
				$iResult = 5;
				break;
			case "structure" :
				$iResult = 20;
				break;
			case "enum" :
				$iResult = 20;
				break;
			case "picture" :
			case "file" :
				$iResult = 10;
				break;
			case "document" :
				$iResult = 20;
				break;
			case "password" :
				$iResult = 10;
				break;
			case "table" :	
				$iResult = 20;
				break;				
		}
		
		return $iResult;
	}
	
	public function replaceEntities($sHtml)
	{		
		$aEntities	= array("&#160;", "-", ".");
		$aReplace	= array(" ", "- ", ". ");
		$sHtml		= htmlspecialchars_decode($sHtml);
		return mb_str_replace($aEntities, $aReplace, $sHtml);		
	}
	
	public function saveSpreadsheet()
	{
		$oUserBox  = MSF_Registry::get("MSF_UserBox");
		$aSettings = MSF_Registry::get("MSF_Settings")->getModule("media");		
		$sRandom   = MSF_String::random(10);
		$sFileName = $aSettings["settings"]["export_directory"] . "/" . $sRandom;
		$sFullName = CNF_ABS_PATH . "/" . $sFileName;
		$this->phpExcel->getActiveSheet()->setTitle('Simple');
		$oWriter = new PHPExcel_Writer_Excel5($this->phpExcel);
		$oWriter->save(str_replace(".php", ".xls", $sFullName . ".php"));
		return $sRandom;	
	}
	
}

?>