<?php
if(!defined("__MSF")) {
	die("Direct access not allowed");
}

class MSF_ImportDoctype {

	private $iImportedDoctypeId = 0;

	private $bImportSuccess = false;
	
	private $aDoctype		= array();
	
	private $aVersion		= array();

	public function __construct($aFile) {
		$this->bImportSuccess = $this->setImportSuccess($aFile);
	}
	
	private function setImportSuccess($aFile)
	{
		$aCheck = array("name", "type", "tmp_name", "error", "size");
		$bCheck = !empty($aFile);

		if ($bCheck) {
			$i=0;
			while ($i<sizeof($aCheck) && $bCheck) {
				$bCheck = isset($aFile[$aCheck[$i]]);
				if ($bCheck) {
					switch ($aCheck[$i]) {
						case "type":
							$bCheck = ($aFile[$aCheck[$i]] == "text/xml");
							break;
						case "error":
							$bCheck = ($aFile[$aCheck[$i]] == 0);
							break;
						case "size":
							$bCheck = ($aFile[$aCheck[$i]] > 0);
							break;
						case "tmp_name":
							$bCheck = $this->checkXML($aFile[$aCheck[$i]]);
							break;
					}
				}
				$i++;
			}
		}
		
		return $bCheck;
	}
	
	private function checkXML($filepath) {

		$sData = MSF_File::read($filepath);
		$aData	= MSF_XmlWrapper::xmlToArray($sData);
		$bCheck = !empty($aData);
		
		$aDoctypeProps	= array("Title", "Description", "FullDescription", "FieldsXml", "ComparedXml", "RouteXml");
		$bCheck = ($bCheck) ? $this->setXMLData("doctype", "aDoctype", $aData, $aDoctypeProps) : false;

		$aVersionProps	= array("RouteXml", "TemplateXml", "FieldsXml", "SpaXml", "ArchiveXml" , "CreatedAt");
		$bCheck = ($bCheck) ? $this->setXMLData("version", "aVersion", $aData, $aVersionProps) : false;

		return $bCheck;
	}
	
	private function setXMLData ($type, $sName, $aData, $aProps) {

		$bCheck = isset($aData[$type]);
		$bCheck = ($bCheck) ? isset($aData[$type]["value"]) : false;

		if ($bCheck) {
			$aData = $aData[$type]["value"];
			for ($i=0; $i<sizeof($aProps); $i++) {
				$aCheck[$i] = strtolower($aProps[$i]);
			}
			$i = 0;
			while (($i<sizeof($aCheck)) && $bCheck) {
				$bCheck = isset($aData[$aCheck[$i]]);
				if ($bCheck) {
					$aData[$aCheck[$i]] = htmlspecialchars_decode($aData[$aCheck[$i]]);
				}
				$i++;
			}
		}
		
		if ($bCheck) $this->$sName = $aData;
		
		return $bCheck; 			 
	}
	
	public function importSuccess() {
		return $this->bImportSuccess;	
	}
	
	private function getDoctypeData() {
		return $this->aDoctype;
	}

	private function getVersionData() {
		return $this->aVersion;
	}

	public function complete() {
		try {
			$aDoctypeProps		= array("getDoctypeId", "Title", "Description", "FullDescription", "FieldsXml", "ComparedXml", "RouteXml", "UserId");
			$aDoctype			= $this->getDoctypeData();
			$aDoctype["userid"]	= MSF_Registry::get("MSF_UserBox")->getUserAttribute("id");
			$Doctype			= new WF_Doctype();
			$iDoctypeId 		= $this->setProps($Doctype, $aDoctypeProps, $aDoctype);
			if ($iDoctypeId) {
				$this->setImportedDoctypeId($iDoctypeId);
				$aVersionProps		= array("getDoctypeVersionId", "RouteXml", "TemplateXml", "FieldsXml", "SpaXml", "ArchiveXml" , "CreatedAt", "UserId");
				$aVersion			= $this->getVersionData();
				$aVersion["userid"] = $aDoctype["userid"];
				$Version			= new WF_DoctypeVersion($Doctype);
				$Version->setDoctypeId($iDoctypeId);
				$iVersionId			= $this->setProps($Version, $aVersionProps, $aVersion);
				if ($iVersionId) {
					$Doctype->setDoctypeVersionId($iVersionId);
					return true;
				}
			}
			
		}
		catch(Exception $oException) {

		}
		return false;
	}
	
	private function setProps($oItem, $aProps, $aValue) {
		for ($i=1; $i<sizeof($aProps); $i++) {
			$sProp = "set" . $aProps[$i];
			$sIndex = strtolower($aProps[$i]);
			$oItem->$sProp($aValue[$sIndex]);
		}
		$oItem->save();
		$sProp = $aProps[0];
		return $oItem->$sProp();
	}
	
	private function setImportedDoctypeId($iDoctypeId) {
		$this->iImportedDoctypeId = $iDoctypeId;
	}
	
	public function getImportedDoctypeId() {
		return $this->iImportedDoctypeId;
	}

}
?>