MSFW.DataRefinement.Filters	= Class.create(MSFW.DataRefinement, {
	
	LABEL_TEXT	: aGlobalMsfWords["REF_PLACEHOLDER_FILTERS"],
	
	BUTTON_TEXT	: aGlobalMsfWords["MSFW_SEARCH"],
	
	TABLE_CLASS	: "MSF_DataRefinement_Filter",
	
	visibleElements	: $A(),
	
	initialize : function(options)
	{
		MSFW.DataRefinement.prototype.initialize.apply(this, arguments);
		
		this.lastOn	= 0;
		this.Form	= new Element("form");
		this.Table	= new Element("table", {"class":this.TABLE_CLASS});
		this.Panel	= new Element("div", {"class":"panel"});
		
		this.Values	= {};
		
		this.Container.insert(this.Table);
		this.Panel.insert(this.Form);
		
		this._buildTable();
		this._buildPanel();
		this._buildButton();
	},
	
	setDefaultValues : function(values)
	{			
		var inputs		= this.Form.getInputs();
		var lastCheck	= null;
		inputs.each(function(e) {			
			if(e.name != "") {
				$H(values).each(function(k){
					if(e.name.include(k.key)) {
						e.value				= k.value;						
						lastCheck.checked	= true;
						e.enable();
						return;
					}
					
				})
				
			}
			else {
				lastCheck	= e;
			}
			
		});
		
	},
	
	_setValues : function()
	{		
		var inputs	= this.Form.getInputs();
		var values	= {};
		inputs.each(function(e) {			
			if(e.name != "" && !e.disabled) {
				values[e.name]	= e.value; 
			}
			
		});
		
		
		this.Values	= values;
	},
	
	_buildButton : function()
	{
		var me		= this;
		var Button	= new Element("input", {"type":"button", "value":this.BUTTON_TEXT});
		
		Button.observe("click", function(event){
			event.stop();
			
			
			me._setValues();
			me._fireEvent("buttonclick");
			me._clearElements();			
		});
		
		this.Panel.insert(Button);
	},
	
	_buildPanel : function()
	{
		var me		= this;
		
		var Table	= new Element("table", {"cellpadding":"5","cellspacing":"5"});
		var i		= 0;
		
		this.Form.insert(Table);		
		
		$H(this.Options.columns).each(function(e){
			if(e.value.filter) {
				var Row	= Table.insertRow(i);
				i++;
				
				var Cell1	= Row.insertCell(0);
				var Cell2	= Row.insertCell(1);
				var Cell3	= Row.insertCell(2);
				
				var Check	= new Element("input", {"type":"checkbox"});
				Cell1.insert(Check);
				
				Cell2.insert(e.value.title);
				
				var Input	= new Element("input", {"disabled":true, "type":"text", "name":"rs_searchParam_" + e.key});
				Cell3.insert(Input);
				
				Check.observe("click", function(){(Check.checked) ? Input.enable() : Input.disable()});
			}
			
		});
		
		this.Panel.hide();
	},
	
	_buildTable : function()
	{		
		var me		= this;
		var Row		= this.Table.insertRow(0);
		var Cell1	= $(Row.insertCell(0));
		var Cell2	= $(Row.insertCell(1));
		
		Cell1.addClassName("title").insert(this.LABEL_TEXT);
		Cell2.addClassName("button").insert(this.Panel);
	
		this.Table.observe("mouseover", function(){
			Cell2.addClassName("button-active");
			Cell1.addClassName("title-active");			
		});
		
		this.Table.observe("mouseout", function(){
			Cell1.removeClassName("title-active");
			Cell2.removeClassName("button-active");
		});
		
		Cell2.observe("click", function(){			
			if(me.Panel.visible()) {
				return;
			}
			
			me._clearElements();			
			var tmp	= {
				Panel	: me.Panel,
				onClick	: me._stop.bindAsEventListener(me, me.Panel, me)
			};
			
			me.visibleElements.push(tmp);
			var d		= new Date();				
			me.lastOn	= d.getTime();	
			
			var Html	= $$("html")[0];
			Html.observe("click", tmp.onClick);
			me.Panel.show();			
		});
		
	},
	
	_stop : function(e, Panel, me)
	{		
		var Html	= $$("html")[0];
		var d		= new Date();
		
		if(!e || (Panel.visible() && (d.getTime() - me.lastOn) > 100 && !me._inPanel(e.target))) {			
			this._clearElements();
		}
	},
	
	_clearElements : function()
	{
		var me		= this;
		var Html	= $$("html")[0];		
		
		this.visibleElements.each(function(e)
		{			
			e.Panel.hide();						
			Html.stopObserving("click", e.onClick);			
		});
		
		this.visibleElements.clear();		
	},
	
	_inPanel : function(el)
	{
		var ret	= false;
		
		while(el) {
			if(el == this.Panel) {
				ret	= true;
				break;
			}
			
			el	= el.parentNode;
		}		
			
		return ret;
	}
});