MSFW.DataSource.AjaxTable	= Class.create(MSFW.DataSource, {
	
	EVENT_TYPES			: ["loadstart", "loadend"],
	
	ASC_ORDER_IMAGE		: "/templates/default/images/arrow_small_down.png",
	
	DESC_ORDER_IMAGE	: "/templates/default/images/arrow_small_up.png",
	
	NONE_ORDER_IMAGE	: "/templates/default/images/arrow_small.png",
	
	TABLE_CLASS			: "MSFW_Table",
	
	PAGINATION_CLASS	: "MSFW_Pagination",
	
	initialize : function(options)
	{		
		MSFW.DataSource.prototype.initialize.apply(this, arguments);
		
		this.Options		= Object.extend({			
			"url"			: "",
			"params"		: "",
			"multiselect"	: false,
			"idColumn"		: "id"
		}, this.Options || {});
						
		this.Table				= new Element("table", {"class":this.TABLE_CLASS});
		this.Pagination			= new Element("div", {"class":this.PAGINATION_CLASS});
		this.inLoad				= false;			
		this.Data				= null;
		this.LastRow			= null;
		
		
		this.FiltersParams		= {};
		this.GroupsParams		= {"get":{}, "post":{}};
		this.OtherParams		= {};
		
		this.Container.insert(this.Table);
		this.Container.insert(this.Pagination);				
	},
	
	getData : function()
	{
		return this.Data;
	},
	
	setFilters : function(value)
	{			
		this.FiltersParams	= value;		
	},
	
	setGroups : function(method, value)
	{
		this.GroupsParams[method]	= value;
	},
					
	reload : function()
	{		
		this.load(this.OtherParams);
	},
	
	load : function(getParams)
	{		
		this._fireEvent("loadstart");
		
		var me			= this;
		var postParams	= this.Options.params;
		var url			= this.Options.url;		
		
		
		getParams	= Object.extend({}, getParams || {});
		url		+= "&" + Object.toQueryString(getParams);
		url		+= "&" + Object.toQueryString(this.FiltersParams);
		url		+= "&" +  Object.toQueryString(this.GroupsParams.get);
		
		postParams	+= "&" +  Object.toQueryString(this.GroupsParams.post);
		
		this.OtherParams	= getParams;
						
		new Ajax.Request(
			url,
			{
				method 		: "post",
				parameters 	: postParams,
			  	onSuccess 	: function(resp){					
					me.Data = resp.responseText.evalJSON().data;					
					me._draw();
					me._fireEvent("loadend");
				}
			
			}
			
		);
		
	},
	
	_draw : function(resp)
	{
		this._clearTable();
		this._buildTable();
		this._buildPagination();
	},
	
	_clearTable : function()
	{		
		var me	= this;
		for(var i=($A(this.Table.rows).size()-1); i>=0; i--) {
			me.Table.deleteRow(i);
		}
	},
	
	_buildTable : function()
	{ 
		var me				= this;
		var TitlesRow		= $(this.Table.insertRow(0));
		var rowCount		= 1;
		var cellCount		= 0;				
		
		$A(this.Data.titles).each(function(e) {			
			if(!e.name.include(me.Options.idColumn)){			
				var Cell		= $(TitlesRow.insertCell(cellCount));
				Cell.className	= "headers";		
				
				var A		= new Element("a", {"class":"title", "href":"#", "onclick":"return false;"});
				var Nobr	= new Element("nobr");
				var Img		= new Element("img");
				var order	= "asc";
								
				switch(e.orderBy) {					
					case "asc"	:
						Img.src	= me.ASC_ORDER_IMAGE;
						order	= "desc";
						
						break;
					case "desc"	:
						Img.src	= me.DESC_ORDER_IMAGE;
						order	= "asc";					
					
						break;
					default		:
						Img.src	= me.NONE_ORDER_IMAGE;
						order	= "asc";
						break;
				}
								
				Nobr.insert(e.title).insert("&#160;").insert(Img);
				A.insert(Nobr);
								
				var params	= me._getLastParams();
				params["rs_orderBy"]		= e.name;
				params["rs_orderDirection"]	= order;
				params["rs_limitFrom"]		= 0;
				
				A.observe("click", me.load.bind(me, params));
				
				Cell.insert(A);
				cellCount++;
			}
						
		});
		
				
		$A(this.Data.records).each(function(e){			
			var Row		= $(me.Table.insertRow(rowCount));
			var id		= e.columns[0].value;
			cellCount	= 0;
			rowCount++;
									
			Row.observe("mouseover", me._markRow.bind(me, Row, true));
			Row.observe("click", me._select.bind(me, Row, e.columns));	
			
			$A(e.columns).each(function(k){				
				if(!k.name.include(me.Options.idColumn)) {
					var Cell		= $(Row.insertCell(cellCount));					
					k.value != "" ? Cell.insert(k.value) : Cell.insert("&#160;")					
					cellCount++;
				}
				
			});
			
			if(me.Values[id]) {				
				me._changeRowClassName(Row, "selected", true);
			}		
			
		});		
		
	},
	
	_buildPagination : function()
	{
		this.Pagination.update("");
		
		var me		= this;
		var Total	= new Element("div", {"class":"total"}).insert(aGlobalMsfWords["RS_TOTAL_PAGES"] + " " + this.Data.pagination.totalPages);
		var Pages	= new Element("div", {"class":"pages"});
		
		var firstPageParams		= me._getLastParams();
		var previusPageParams	= me._getLastParams();
		var lastPageParams		= me._getLastParams();
		var nextPageParams		= me._getLastParams();
		
		firstPageParams["rs_limitFrom"]		= 0;
		previusPageParams["rs_limitFrom"]	= this.Data.pagination.previusPage;
		lastPageParams["rs_limitFrom"]		= this.Data.pagination.lastPage;
		nextPageParams["rs_limitFrom"]		= this.Data.pagination.nextPage;
		
		this.Pagination.insert(Total).insert(Pages);
		
		if(this.Data.pagination.previusPage != -1) {			
			var First	= new Element("a", {"href":"#", "onclick":"return false;"});	
			First.insert(aGlobalMsfWords["MSFW_DOCUMENT_PAGINATION_FIRST"]).observe("click", me.load.bind(me, firstPageParams));
			Pages.insert(First);
			
			var Previus	= new Element("a", {"href":"#", "onclick":"return false;"});	
			Previus.insert(aGlobalMsfWords["MSFW_DOCUMENT_PAGINATION_PREVIUS"]).observe("click", me.load.bind(me, previusPageParams));
			Pages.insert(Previus);
		}
				
		$A(this.Data.pagination.pages).each(function(e){			
			if(e.currentPage) {
				var Page	= new Element("span");
			}
			else {
				var Page	= new Element("a", {"href":"#", "onclick":"return false;"});
				
				var params	= me._getLastParams();
				params["rs_limitFrom"]	= e.limitFrom;
								
				Page.observe("click", me.load.bind(me, params));				
			}
			
			Page.insert(e.number);
			Pages.insert(Page);						
		});
		
		if(this.Data.pagination.nextPage != -1) {			
			var Next	= new Element("a", {"href":"#", "onclick":"return false;"});	
			Next.insert(aGlobalMsfWords["MSFW_DOCUMENT_PAGINATION_NEXT"]).observe("click", me.load.bind(me, nextPageParams));
			Pages.insert(Next);
						
			var Last	= new Element("a", {"href":"#", "onclick":"return false;"});	
			Last.insert(aGlobalMsfWords["MSFW_DOCUMENT_PAGINATION_LAST"]).observe("click", me.load.bind(me, lastPageParams));
			Pages.insert(Last);
		}		
	
	},
	
	_getLastParams : function()
	{			
		var ret	= {};
		$H(this.OtherParams).each(function(e){
			ret[e.key]	= e.value;
		});
		
		return ret;
	},
	
	_markRow : function(Row, flag)
	{
		var me	= this;
		this._changeRowClassName(Row, "marked", flag);
		if(flag) {			
			this._UnMarkFunction	= function(){me._markRow(Row, false)}.bindAsEventListener(this);
			Row.observe("mouseout", this._UnMarkFunction);
		}
		else {			
			Row.stopObserving("mouseout", this._UnMarkFunction);
			this._UnMarkFunction	= null;
		}
		
	},
	
	_select : function(Row, data)
	{	
		var id	= data[0].value;		
		if(this.Values[id]) {
			delete this.Values[id];
			this._changeRowClassName(Row, "selected", false);
		}
		else {			
			if(!this.Options.multiselect && this.LastRow) {
				delete this.Values[this.LastRow.id];
				this._changeRowClassName(this.LastRow.row, "selected", false);
			}
			
			this.Values[id]	= data;
			this._changeRowClassName(Row, "selected", true);
			
			this.LastRow	= {"row":Row, "id": id};
		}
				
	},
	
	_changeRowClassName : function(Row, className, flag)
	{
		if(flag) {
			$A(Row.cells).each(function(e){
				$(e).addClassName(className);
			});	
			
		}
		else {
			$A(Row.cells).each(function(e){
				$(e).removeClassName(className);
			});
			
		}
		
	}
	
});