MSFW.DataSource.AjaxTree	= Class.create(MSFW.DataSource, {
	
	EVENT_TYPES				: ["loadstart", "loadend", "groupclick"],
	
	CONTAINER_CLASS			: "MSFW_DataSource_AjaxTree",
	
	CLOSED_GROUP_IMAGE		: "/templates/default/images/folder_closed.png",
	
	OPENED_GROUP_IMAGE		: "/templates/default/images/folder_open.png",
	
	PAGINATION_CLASS	: "MSFW_Pagination",
	
	initialize : function(options)
	{
		MSFW.DataSource.prototype.initialize.apply(this, arguments);		
		this.clear();
		
		this.Container.addClassName(this.CONTAINER_CLASS);		
	},
	
	clear : function()
	{
		this.Groups				= {};
		this.Data				= {};
		this.levels				= 0;
		this.AllRecords			= {};
		this.LastGroup			= null;		
		this.treeLoaded			= false;
		
		this.LastOptions		= {"get":{}, "post":{}};
		
		this.FiltersParams		= {};
					
		this.Container.update();
	},
	
	setParam : function(method, name, value)
	{
		this.LastOptions[method][name]	= value;
	},
	
	setFilters : function(value)
	{		
		this.FiltersParams	= value;		
	},
		
	loaded : function()
	{
		return this.treeLoaded;
	},	
	
	getData : function()
	{
		return this.Data;
	},
	
	getLastGroupValue : function()
	{		
		if(this.LastGroup) {
			return  this.LastGroup.value;
		}
		else {
			return {};
		}
					
	},
	
	reload : function()
	{
		this.load(this.Groups);
	},
	
	load : function(values)
	{
		var me			= this;
		this.Groups		= values;
		this.levels		= 0;
		this.treeLoaded	= true;
		this.Container.update();
		
		
		$H(this.Groups).each(function(e){
			if(e.value == 1) {
				me.levels++;
			}
		});
		
		
		this._load(this.Container, {});
	},
	
	_load : function(container, groupValues, limit)
	{
		this._fireEvent("loadstart");
		
		var me		= this;
		var params	= this.Options.params + "&" + Object.toQueryString(groupValues);
		var url		= this.Options.url;
		
				
		url	+= Object.toQueryString(this.Groups);	
		url	+= "&" + Object.toQueryString(this.FiltersParams);
		
		if(limit) {
			url	+= limit;
		}
		
		new Ajax.Request(
			url,
			{
				method 		: "post",
				parameters 	: params,
			  	onSuccess 	: function(resp){					
					me.Data = resp.responseText.evalJSON().data;
					me._onDataLoad(container, groupValues);					
					me._fireEvent("loadend");
				}
			
			}
			
		);
		
	},
	
	_onDataLoad : function(container, groupValues)
	{
		container.update();
		this._draw(container, groupValues);
		this._buildPagination(container, groupValues);
	},
	
	_draw : function(container, groupValues)
	{
		var me			= this;
		var Ul			= new Element("ul");
		var lastGroup	= false;
		
		container.insert(Ul);
		
		((this.levels - 1) == $A(this.Data.groupValues).size()) ? lastGroup = true : lastGroup = false;
		
		
		
		$A(this.Data.records).each(function(e){
			var Li	= new Element("li");
			Ul.insert(Li);
			
			var Nobr	= new Element("nobr");
			Li.insert(Nobr);
			
			var Div	= new Element("div");			
			Nobr.insert(Div);
			
			var Img	= new Element("img", {"src":me.CLOSED_GROUP_IMAGE});
			Div.insert(Img);
			
			var Content	= new Element("span", {"class":"content"});
			Content.insert(e.columns[0].value);
			Div.insert(Content);
			/*
			var Count	= new Element("span", {"class":"count"});
			Count.insert(e.total_in_group);
			Div.insert(Count);
			*/
	
			var tmp	= {};			
			tmp["rs_groupValues[" + e.columns[0].name + "]"]	= e.columns[0].original;
			var newValues	= Object.extend(tmp, groupValues);
			
			
			var NextGroupContainer	= new Element("span");
			Li.insert(NextGroupContainer);			
			
			Div.observe("mouseover", function(){me._mark(Div, true)});
			
			if(lastGroup) {
				Div.observe("click", function(){
					if(me.LastGroup) {
						me.LastGroup.img.src	= me.CLOSED_GROUP_IMAGE;
						me.LastGroup.div.removeClassName("selected");
					}
					
					me.LastGroup	= {
						"img"	: Img,
						"div"	: Div,
						"value"	: newValues
					};
					Img.src			= me.OPENED_GROUP_IMAGE;
					Div.addClassName("selected");
					me._fireEvent("groupclick");
				});
			}
			else {
				Div.observe("click", function(){
					me._toggleGroup(Li, Img, NextGroupContainer, newValues);					
				});
			}
						
		});
		
	},
	
	_buildPagination : function(Container, groupValues)
	{	
		if(this.Data.pagination.totalPages <= 1) {
			return;
		}
		
		var Pagination	= new Element("div", {"class":this.PAGINATION_CLASS});
		Container.insert(Pagination);
				
		var me		= this;
		var Total	= new Element("div", {"class":"total"}).insert(aGlobalMsfWords["RS_TOTAL_PAGES"] + " " + this.Data.pagination.totalPages);
		var Pages	= new Element("div", {"class":"pages"});
		
		Pagination.insert(Total).insert(Pages);		
		
		$A(this.Data.pagination.pages).each(function(e){			
			if(e.currentPage) {
				var Page	= new Element("span");
			}
			else {
				var Page	= new Element("a", {"href":"#", "onclick":"return false;"});				
								
				Page.observe("click", me._load.bind(me, Container, groupValues, "rs_limitFrom=" + e.limitFrom));				
			}
			
			Page.insert(e.number);
			Pages.insert(Page);						
		});
		
	},
	
	_mark : function(Div, flag)
	{
		var me	= this;
		
		if(flag) {
			Div.addClassName("marked");
		}
		else {
			Div.removeClassName("marked");
		}
				
		if(flag) {			
			this._UnMarkFunction	= function(){me._mark(Div, false)}.bindAsEventListener(this);
			Div.observe("mouseout", this._UnMarkFunction);
		}
		else {			
			Div.stopObserving("mouseout", this._UnMarkFunction);
			this._UnMarkFunction	= null;
		}
		
	},
	
	_toggleGroup : function(Li, Img, container, groupValues)
	{
		var id	= Li.identify();
		if(this.AllRecords[id]) {
			if(this.AllRecords[id].groupContainer.visible()) {
				this.AllRecords[id].groupContainer.hide();
				this.AllRecords[id].img.src	= this.CLOSED_GROUP_IMAGE;
			}
			else {
				this.AllRecords[id].groupContainer.show();
				this.AllRecords[id].img.src	= this.OPENED_GROUP_IMAGE;
			}
			
		}
		else {
			this.AllRecords[id]	= {
				"groupContainer"	: container,
				"img"				: Img
			};
			
			this.AllRecords[id].img.src	= this.OPENED_GROUP_IMAGE;
			this._load(container, groupValues);
		}
		
	}
	
});