MSFW.DataSource.Calendar	= Class.create(MSFW.DataSource, {
	initialize : function(options)
	{
		MSFW.DataSource.prototype.initialize.apply(this, arguments);
		
		this.Container.addClassName("calendar_date_select");		
		this._buildCalendar();
	},
	
	_buildCalendar : function()
	{		
		this._parseDate();
		this._buildHeaderDiv();
		this._buildCalendarGrid();
		this._buildButtonsDiv();
		
		this._refresh();		
	},
	
	_parseDate: function()
	{				
		if(this.Options.default) {
			this.Date	= Date.parseFormattedString(this.Options.default);
			if(!this.Date) {
				this.Date	= new Date();
			}
		}
		else {
			this.Date			= new Date();
		}		
		
		this.SelectedDate	= new Date(this.Date);		
		this.Date.setDate(1);
	},
	
	_refresh: function ()
	{
		this._refreshMonthYear();
		this._refreshCalendarGrid();
		this._refreshTime();
		
		this._updateSelectedDate();
		this._setSelectedClass();		
	},
	
	_refreshMonthYear: function()
	{		
	    var m = this.Date.getMonth();
	    var y = this.Date.getFullYear();
	    
	    this.MonthSelect.setValue(m, false);
	    this.YearSelect.setValue(y);  
	},
	
	_refreshTime : function()
	{
		if(this.Options.time) {
			this.HourSelect.setValue(this.SelectedDate.getHours());
		    this.MinuteSelect.setValue(this.SelectedDate.getMinutes())
		}
		
	},
	
	_refreshCalendarGrid: function ()
	{
		this.BeginingDate	= new Date(this.Date).stripTime();
		this.BeginingDate.setDate(1);
		this.BeginingDate.setHours(12); // Prevent daylight savings time boundaries from showing a duplicate day
		var preDays = this.BeginingDate.getDay() // draw some days before the fact
		
		if (preDays < 3) {
			preDays += 7;
		}
		
	    this.BeginingDate.setDate(1 - preDays + Date.first_day_of_week);
	    
	    var iterator	= new Date(this.BeginingDate);	    
	    var today		= new Date().stripTime();
	    var thisMonth	= this.Date.getMonth();
	    
	       
	    for(var cellIndex = 0; cellIndex<42; cellIndex++)
	    {
	    	var day		= iterator.getDate();
	    	var month	= iterator.getMonth();
	    	var cell	= this.calendarDayGrid[cellIndex];
	    	var div		= $(cell.childNodes[0]);
	      
	    	div.update(day);
	      
	    	if (month!=thisMonth) {
	    		div.className = "other";
	    	}
	    	
	    	cell.day	= day;
	    	cell.month	= month;
	    	cell.year	= iterator.getFullYear();
	    	
	    	iterator.setDate(day + 1);	      
	    }
	    
	    if(this.todayCell) {
	    	this.todayCell.removeClassName("today");
	    }
	    
	    if ($R(0, 41).include(daysUntil = this.BeginingDate.stripTime().daysDistance(today)) ) {
	    	this.todayCell = this.calendarDayGrid[daysUntil];
	    	this.todayCell.addClassName("today");
	    }
		
	},
	
	_buildHeaderDiv: function()
	{
		this.HeaderDiv	= new Element("div", {"class":"cds_header", "clear":"left"});
		this.Container.insert(this.HeaderDiv);
		
		this.NextMonthButton	= new Element("a", {"href":"#", "class":"next"}).insert("&gt;");
		this.NextMonthButton.observe("click",  this._navMonth.bind(this, this.Date.getMonth() + 1));		
		this.HeaderDiv.insert(this.NextMonthButton);
					
		this.PrevMonthButton	= new Element("a", {"href":"#", "class":"prev"}).insert("&lt;");
		this.PrevMonthButton.observe("click",  this._navMonth.bind(this, this.Date.getMonth() - 1));
		this.HeaderDiv.insert(this.PrevMonthButton);
		
		this.MonthSelect	= new MSFW.DataSource.Calendar.SelectBox(this.HeaderDiv, $R(0,11).map(function(m){return [Date.months[m], m]}), {className: "month", onchange: function () { this._navMonth(this.MonthSelect.getValue()) }.bindAsEventListener(this)});;
		this.YearSelect		= new MSFW.DataSource.Calendar.MaskInput(this.HeaderDiv, [], {size: 4, className: "year"}, null, {format:"    ", regex:/^\d{1,4}$/}, function () { this._navYear(this.YearSelect.getValue()) }.bindAsEventListener(this));
	},
	
	_buildCalendarGrid: function() 
	{
		this.BodyDiv			= new Element("div", {"class":"cds_body", "clear":"left"});
		this.Container.insert(this.BodyDiv);
		this.calendarDayGrid	= [];
		var daysTable			= new Element("table", {"cellpadding":"0px", "cellspacing":"0px", "width":"100%"});
		
		this.BodyDiv.insert(daysTable);
		var tHead		= new Element("thead");
		var weekdaysRow = new Element("tr");
		daysTable.insert(tHead.insert(weekdaysRow));
		Date.weekdays.each( function(weekday) { 			
			var Th	= new Element("th").insert(weekday);
			weekdaysRow.insert(Th);
	    });
		
		var daysTBody = new Element("tbody");
		daysTable.insert(daysTBody);
		
		var rowNumber	= 0, weekday;
		var daysRow		= null;
	    for(var cellIndex = 0; cellIndex < 42; cellIndex++)
	    {
	    	weekday	= (cellIndex + Date.first_day_of_week ) % 7;
	    	if(cellIndex % 7 == 0) {
	    		daysRow	= new Element("tr", {"class": "row_" + rowNumber++});
	    		daysTBody.insert(daysRow);	    		
	    	}
	    	
	    	this.calendarDayGrid[cellIndex]	= new Element("td",{
	    		"class"	: (weekday==0) || (weekday==6) ? " weekend" : "",
	    		"style"	: "cursor:pointer"
	    	});
	    	
	    	this.calendarDayGrid[cellIndex].insert(new Element("div"));
	    	
	    	daysRow.insert(this.calendarDayGrid[cellIndex]);
	    	
	    	this.calendarDayGrid[cellIndex].observe("click", this._updateSelectedDate.bind(this, this.calendarDayGrid[cellIndex], true));
	    }		
		
	},
	
	_buildButtonsDiv: function()
	{		
		
		if(this.Options.time)
	    {
			this.ButtonsDiv	= new Element("div", {"class":"cds_buttons", "clear":"left"});
			this.Container.insert(this.ButtonsDiv);
			
			this.HourSelect	= new MSFW.DataSource.Calendar.MaskInput(
				this.ButtonsDiv, 
			    [], 
			    {
					size: 2,
			      	calendar_date_select: this,
			      	className: "hour"
			    }, 
			    null, 
			    {format:"  ", regex:/([1-6]{1})?[0-9]{1}/, check:function(val){if(val>=0 && val<=23){return true;}else{return false}}},
			    function () { this._updateSelectedDate({"hour":this.HourSelect.getValue()}) }.bindAsEventListener(this)
			);
			
			this.ButtonsDiv.insert((new Element("span",{"class":"separator"})).insert(":"));
			
			this.MinuteSelect	= new MSFW.DataSource.Calendar.MaskInput(
				this.ButtonsDiv, 
				[], 
		      	{
		      		size: 2,
		      		calendar_date_select: this,
		      		className: "minute"
		      	}, 
		      	null, 
		      	{format:"  ", regex:/([0-6]{1})?[0-9]{1}/, check:function(val){if(val>=0 && val<=59){return true;}else{return false}}},
		      	function () { this._updateSelectedDate({"minute":this.MinuteSelect.getValue()}) }.bindAsEventListener(this)
			);
			
	    }
		
	},
	
	_navMonth: function(month)
	{
		(targetDate = new Date(this.Date)).setMonth(month);
		return (this._navTo(targetDate));
	},
	
	_navYear: function(year)
	{
		(targetDate = new Date(this.Date)).setYear(year);
		return (this._navTo(targetDate));
	},
	
	_navTo: function(date)
	{
	    this.Date = date;
	    this.Date.setDate(1);
	    
	    this._refresh();
	   
	    return true;
	},
	
	_updateSelectedDate : function(partsOrElement)
	{		
	    var parts = $H(partsOrElement);
	    	    
	    if (parts.get("day")) {	    	
	    	this.SelectedDate.setDate(parts.get("day"));
	    	this.SelectedDate.setYear(parts.get("year"));
	    	this.SelectedDate.setMonth(parts.get("month"));	    	
	    }
	    
	    
	    if (!isNaN(parts.get("hour"))) {
	    	this.SelectedDate.setHours(parts.get("hour"));
	    }
	    	 
	    if (!isNaN(parts.get("minute"))) {
	    	this.SelectedDate.setMinutes(parts.get("minute"));
	    }
	    
	    
	    this.Values[0]	= [{"name":"date", "value":this.SelectedDate.toFormattedString(this.Options.time)}]
	    this._setSelectedClass();	  
	},
	
	_clearSelectedClass: function()
	{
		if(this.selectedCell){
			this.selectedCell.removeClassName("selected");
		}
		
	},
	
	_setSelectedClass: function()
	{
		this._clearSelectedClass()
		if($R(0,42).include(daysUntil = this.BeginingDate.stripTime().daysDistance(this.SelectedDate.stripTime()))) {
			this.selectedCell = this.calendarDayGrid[daysUntil];
			this.selectedCell.addClassName("selected");
		}
		
	},
	
	clear : function()
	{
		this.Values = {};		
	}
	
});