<?php
if(!defined("__MSF")) {
	die("Direct access is not allowed");
}

require_once "library/MSF_EnumValue/MSF_EnumValue.php";
class MSF_RS_Params
{

	/**
	 * @var array
	 */
	private $aParams			= array();

	private $aColumns			= array();

	private $aColumnsTypes		= array();
	
	private $sDefaultOrder		= "";
	
	private $sDefaultDirection	= "";

	private $aDefaultFilters	= array();

	private $aDefaultGroups		= array();

	public function __construct()
	{
		$this->aParams["sQuickFilter"]	= MSF_String::unescape(MSF_SL::getParam($_REQUEST, "rs_quickFilter", ""));
	}

	public function grabParams()
	{
		if (isset($_REQUEST["rs_search"])) {
			$this->aParams["rs_search"] = $_REQUEST["rs_search"];
			$_REQUEST["noStateRecover"] = true;
		}
		
		$this->prepareTypes();

		$this->grabFilters();
		$this->grabGroups();
		$this->grabMain();
	}

	public function get($name, $sDefault = "")
	{
		if(isset($this->aParams[$name])) {
			return $this->aParams[$name];
		}
		else {
			return $sDefault;
		}
	}
	
	public function getTypes()
	{
		return $this->aColumnsTypes;
	}
	
	public function set($sName, $Value) {
		$this->aParams [$sName] = $Value;
	}
	
	public function setColumns($aInput)
	{
		$this->aColumns	= $aInput;
	}
	
	public function setDefaultOrder($sColumn)
	{
		$this->sDefaultOrder = $sColumn;
	}
	
	public function setDefaultDirection($sDirection)
	{
		$this->sDefaultDirection = $sDirection;
	}
	
	public function setDefaultFilters($aInput)
	{
		$this->aDefaultFilters	= $aInput;
	}
	
	public function setDefaultGroups($aInput)
	{
		$this->aDefaultGroups	= $aInput;
	}
	
	public function setParams($aInput)
	{
		$this->aParams	= $aInput;
	}
	
	public function getParams()
	{
		return $this->aParams;
	}
	
	private function grabMain()
	{
		if(!isset($this->aParams["limitFrom"])) {
			$this->aParams["limitFrom"]	= (int)MSF_SL::getParam($_GET, "rs_limitFrom", 0);
		}
		
		$this->aParams["groupMode"]	= "none";
				
		$aValidOrder	= array();
		foreach($this->aColumns as $aColumn) {
			array_push($aValidOrder, $aColumn["name"]);
		}
		
		if(empty($this->sDefaultOrder)) {
			isset($aValidOrder[1]) ? $sDefaultOrder	= $aValidOrder[1] : $sDefaultOrder	= $aValidOrder[0];
		}
		else {
			$sDefaultOrder = $this->sDefaultOrder;
		}
					
		if($this->get("isGroup")) {
			$aGroupValues	= $this->get("aGroupValues");
			
			if(!empty($aGroupValues)  && $this->get("nextGroup") != "") {
				$this->aParams["orderBy"]	= $this->get("nextGroup");
			}
			else {
				if(isset($_GET["rs_orderBy"]) && !empty($aValidOrder)) {					
					$this->aParams["orderBy"]	= $this->grabEnumFromGet("rs_orderBy", $aValidOrder, $sDefaultOrder);
				}
				else {
					$this->aParams["orderBy"]	= $sDefaultOrder;
				}
							
			}

			if(!empty($aGroupValues)) {
				if($this->get("nextGroup") != "") {
					$this->aParams["groupMode"]	= "group";
				}
				else {
					$this->aParams["groupMode"]	= "table";
				}
				
			}
			
		}
		else {			
			if(!empty($aValidOrder)) {		
				$this->aParams["orderBy"]	= $this->grabEnumFromGet("rs_orderBy", $aValidOrder, $sDefaultOrder);				
			}
			else {
				$this->aParams["orderBy"] 	= $this->aColumns[1]["name"];
			}
		}
		
		!empty($this->sDefaultDirection) ? $sDefaultDirection = $this->sDefaultDirection : $sDefaultDirection = "asc"; 
		$this->aParams["orderDirection"]	= $this->grabEnumFromGet("rs_orderDirection", array("asc", "desc"), $sDefaultDirection);
	}
	
	public static function toInput($sStr, $sType)
	{
		if(in_array($sType, array("date", "time", "timestamp"))) {
			if(trim($sStr) == "") {
				$sStr	= null;
			}
			else {
				$Date	= new MSF_Date();
				$Date->parse($sStr);
				if($sType == "date") {
					$sStr	= $Date->format("Y-m-d");
				}
				elseif($sType == "timestamp") {
					$sStr	= $Date->format("Y-m-d H:i:s");
				}
				else {
					$sStr	= $Date->format("H:i:s");
				}
				
			}
			
		}
		elseif($sType == "int") {
			if($sStr == "" || !is_numeric($sStr)) {
				$sStr	= null;
			}		
		}
		elseif($sType == "float") {
			if($sStr == "" || !is_numeric($sStr)) {
				$sStr	= null;
			}			
		}

		return trim($sStr);
	}
	
	private static function stripTags($sText)
	{
		$sText = preg_replace('@<script[^>]*?>.*?</script>@si',' ',$sText);
		$sText = preg_replace('@<style[^>]*?>.*?</style>@si',' ',$sText);
		$sText = preg_replace('@<[\/\!]*?[^<>]*?>@si',' ',$sText);
		
		return strip_tags($sText);
	}
	
	public static function toOutput($sStr, $sType, $aParam = array())
	{
		$sReturn = "";
		if ($sType == "text") {
			$sStr = self::stripTags($sStr);
		}
		elseif ($sType == "reference" || $sType == "table") {
			
		}
		elseif ($sType == "document") {
			$sStr	= htmlspecialchars($sStr);
		}
		elseif ($sType == "string") {
			if(!isset($aParam["notChange"]) || $aParam["notChange"] == false) {
				$sStr	= htmlspecialchars($sStr);
			}
		}
		else {
			$sStr = stripslashes($sStr);
		}
		if(in_array($sType, array("date", "timestamp", "time"))) {
			
			if($sStr == "") {
				return "";
			}
			else {
				$sReturn = MSF_DATE::toViewFormat($sStr);
			}

		}
		elseif($sType == "reference" || $sType == "structure" || $sType == "document") {
			$Value		= new REF_Value();
			$sReturn	= "";
			$Value->parse($sStr);
			for($i=0; $i<$Value->getSize(); $i++) {
				$El			= $Value->get($i);
				if (isset($El["value"])) {
					$sReturn 	.= trim($El["value"]);
	
					if($i<($Value->getSize() - 1)) {
						$sReturn .= "<br/>";
					}
				}
				
			}
		}
		elseif($sType == "password") {
			$sReturn = str_repeat("*", strlen($sStr));
		}
		elseif($sType == "bool") {
			$aValue		= MSF_EnumValue::parse($sStr);
			$sReturn 	= $sStr;	
			
			if ($aValue == null) {
				// do nothing
			}
			else if(empty($aParam) && !$aValue) {
				// do nothing
			}
			elseif(empty($aParam) && $aValue) {
				$sReturn = $aValue["value"];
			}
			else {
				if($aValue["key"] == 1) {
					$sReturn = $aParam["labelYes"];
				}
				else {
					$sReturn = $aParam["labelNo"];
				}
				
			}
			$sReturn = htmlspecialchars($sReturn);
		}
		elseif($sType == "enum") {
			if (!empty($sStr)) {
				$aValues = MSF_EnumValue::parse($sStr);

                //2.01.12 ubral proverku "empty"
                /*
				if(!empty($aParam) && isset($aValues["value"])) {
					$sReturn = $aValues["value"];
				}
				else {
					if (!empty($aParam) && !empty($aValues)) {
						for ($i=0; $i<count($aValues); $i++) {
							$aTmp[] = $aValues[$i]["value"]; 
						}
						if (count($aTmp)) {
							$sReturn = implode(", ", $aTmp);
						}
					}
				}*/
                if(isset($aValues["value"])) {
                    $sReturn = $aValues["value"];
                }
                else {
                    if(!empty($aParam) && !empty($aValues)) {
                        for ($i=0; $i<count($aValues); $i++) {
                            $aTmp[] = $aValues[$i]["value"];
                        }
                        if (count($aTmp)) {
                            $sReturn = implode(", ", $aTmp);
                        }
                    }
                }
			}
			
		}
		elseif($sType == "picture") {
			if((int)$sStr === 0) {
			}
			else {
				$aReturn	= array();
				$aIds		= explode(",", $sStr);
				
				foreach($aIds as $iId) {
					$sSrc 			= "/media/image/" . (int)$iId;
					$sResizedSrc	= $sSrc . "/" . CNF_RECORDSET_IMAGE_SIZE . "/" . CNF_RECORDSET_IMAGE_SIZE;
					array_push($aReturn, "<img src=\"" . $sResizedSrc . "\"/>");
				}
				
				$sReturn = implode("<br/><br/>", $aReturn);
			}
			
		}
		elseif($sType == "file") {
			if((int)$sStr === 0) {
			}
			else {
				$aReturn	= array();
				$aIds		= explode(",", $sStr);
				
				foreach($aIds as $iId) {
					$sTmp	=  "<a href=\"/media/download/" . $iId . "\">" . MSF_Registry::get("MSF_Locale")->getWord("ROOT_RS_DOWNLOAD") . "</a>";
					array_push($aReturn, $sTmp);
				}
				
				$sReturn = implode(", ", $aReturn);
			}
		}
		else if ($sType == "table") {
			$sReturn = self::toOutputTable($sStr);
		}
		else {
			$sReturn = $sStr;
		}
		return $sReturn;
	}
	
	private static function toOutputTable($sStr) {
		$sResult	= "";
		if ($sStr) {
			$aVar 	= explode("^R^", urldecode(str_replace(array("&#160;", "^S^"), " ", $sStr)));
			$aVar0	= explode("^C^", $aVar[0]);
			$sField = $aVar0[0];
			if (isset($aVar0[1])) { 
				$sType = $aVar0[1];   
				$aVar1	= explode("^C^", $aVar[1]);
				for ($i=0; $i<sizeof($aVar1); $i++) {
					if ($sField == $aVar1[$i]) $iField = $i;
				}
				if (isset($iField)) {
					for ($i=2; $i<sizeof($aVar); $i++) {
						$aVar2	= explode("^C^", $aVar[$i]);
						switch ($sType) {
							case "string":
							case "int":
							case "float":
							case "date":
							case "image":
							case "timestamp":
							case "text":
								$aTmp[] = htmlspecialchars($aVar2[$iField]);
								break;
							case "bool":
							case "reference":
							case "document":
							case "structure":
								$aRows 	= explode("|^|row|^|", $aVar2[$iField]);
								foreach ($aRows AS $aRow) {
									$aCells = explode("/^/col/^/", $aRow);
									$aTmp[] = $aCells[0];
								}						
								break;
							case "enum":
								$aCols 	= explode("/^/col/^/", $aVar2[$iField]);
								foreach ($aCols AS $aCol) {
									$aCells = explode("|^|row|^|", $aCol);
									$aTmp[] = $aCells[0];
								}						
								break;
							case "file":
								$aRows 	= explode(":.:", $aVar2[$iField]);
								if (isset($aRows[1])) {
									$aTmp[] = "<a href=\"/media/download/" . $aRows[1] . "\" id=\"file\">" . $aRows[0] . "</a>"; 
								}
								break;
						}
					}
					if (isset($aTmp)) {
						$sResult = implode(",", $aTmp);
					}
				}
			}
		}
		return $sResult;

	}

	
	private function grabFilters()
	{
		$aFilters	= array();
		
		foreach($this->aColumns as $aColumn) {
			$sGetColumn		= str_replace(".", "_", $aColumn["name"]);
			$sRealColumn	= $aColumn["name"];
			$sMultiIndex	= "rs_searchParam_isMulti_" . $sGetColumn;
			$sName			= $aColumn["name"];
			//echo $sName . "\n";
						
			$sType		= $this->aColumnsTypes[$sName];
			$iMulti 	= (int)$this->grabEnumFromGet($sMultiIndex, array(1, 0), 0);
			
			
			$aToParam				= array();
			$aToParam["isMulti"]	= $iMulti;
			$aToParam["sType"]		= $sType;
			$aToParam["sCompare"]	= $aColumn["compare"];
			
			if($iMulti) {
				$sFrom	= "rs_searchParam_" . $sGetColumn . "_from";
				$sTo	= "rs_searchParam_" . $sGetColumn . "_to";
				
				if(isset($_GET[$sFrom]) && isset($_GET[$sTo])) {
					$aToParam["mFrom"]			= self::toInput(MSF_SL::getParam($_GET, $sFrom, ""), $sType);
					$aToParam["mTo"]			= self::toInput(MSF_SL::getParam($_GET, $sTo, ""), $sType);
					$aToParam["mFrom_original"]	= self::toOutput($aToParam["mFrom"], $sType, array("notChange" => true));
					$aToParam["mTo_original"]	= self::toOutput($aToParam["mTo"], $sType, array("notChange" => true));
					
					if($aToParam["mFrom"] !== "" || $aToParam["mTo"] !== "") {
						if($sType == "timestamp") {
							$Date	= new MSF_Date();
							$Date->parse($aToParam["mTo"]);
							
							$sStr	= $Date->getYear() . "-" . $Date->getMonth() . "-" . $Date->getDay();
							if(strtotime($Date->formatTimestamp()) == strtotime($sStr . " 00:00:00")) {
								$aToParam["mTo"]			= self::toInput($sStr . " 23:59:59", $sType);
								$aToParam["mTo_original"]	= self::toOutput($aToParam["mTo"], $sType, array("notChange" => true));
							}
						}
						
						$aFilters[$sRealColumn]	= $aToParam;
					}
											
				}
				
			}
			else {
				$sParam	= "rs_searchParam_" . str_replace(".", "_", $aColumn["name"]);
							
				if(isset($_GET[$sParam])) {
					$aToParam["mValue"]				= self::toInput(MSF_SL::getParam($_GET, $sParam, ""), $sType);
					if($aToParam["mValue"] !== "") {
						$aToParam["mValue_original"]	= self::toOutput($aToParam["mValue"], $sType, array("notChange" => true));
						$aToParam["sCondition"]	= $this->grabEnumFromGet("rs_searchParam_" . $sGetColumn  . "_condition", array("=", "<", ">", "!="), "=");		
						$aFilters[$sRealColumn]	= $aToParam;
					}
										
				}
				
			}
			
		}
		
		if(empty($aFilters)) {
			foreach($this->aColumns as $aColumn) {
				if(isset($this->aDefaultFilters[$aColumn["name"]])) {
					$aTmp				= $this->aDefaultFilters[$aColumn["name"]];
					$aTmp["sType"]		= $this->aColumnsTypes[$aColumn["name"]];
					$aTmp["sCompare"]	= $aColumn["compare"];
					
					$aFilters[$aColumn["name"]]	= $aTmp;
				}
				
			}
			
		}
		
		$this->aParams["aFilters"]		= $aFilters;
		//$this->aParams["sQuickFilter"]	= MSF_String::unescape(MSF_SL::getParam($_GET, "rs_quickFilter", ""));
	}
	
	private function grabGroups()
	{
		$aGroups		= array();
		$bIsGroupFlag	= false;
		$aValidGroup	= array();
		
		foreach($this->aColumns as $aColumn) {
			if($aColumn["group"]) {
				array_push($aValidGroup, $aColumn["name"]);
			}

		}
		
		if(isset($_GET["rs_groups"]))
		{
			$aTmpGroups		= $_GET["rs_groups"];
			foreach($aTmpGroups as $sKey=>$iValue) {
				if(in_array($sKey, $aValidGroup)) {
					$aTmp["name"]	= $sKey;
					$aTmp["active"]	= (int) $iValue;
					
					if($aTmp["active"] == 1) {
						$bIsGroupFlag = true;
					}
					
					array_push($aGroups, $aTmp);
				}
				
			}
			
		}
		else {
			if(!isset($_GET["noStateRecover"])) {
				foreach($this->aDefaultGroups as $sGroup) {
					$aTmp["name"]	= $sGroup;
					$aTmp["active"]	= 1;
					array_push($aGroups, $aTmp);
					$bIsGroupFlag = true;
				}
				
			}
			
		}
		
		
		$bIsGroupValueFlag	= false;
		if(isset($_POST["rs_groupValues"]))
		{
			foreach($_POST["rs_groupValues"] as $sKey=>$sValue)
			{
				$sGroupBy	= $sKey;
				$sValue 	= str_replace("&#43;", "+", urldecode($sValue));
				$sValue 	= self::toInput($sValue, $this->aColumnsTypes[$sGroupBy]);				
				foreach($aGroups as $sName => $aGroup) {
					if($aGroups[$sName]["name"] == $sKey) {
						$aGroups[$sName]["value"]	= $sValue;
						$bIsGroupValueFlag	= true;
					}
				}
			}
			
		}
			
		$sCurrentGroup		= "";
		if($bIsGroupValueFlag) {
			for($i = (count($aGroups) -1); $i>=0; $i--) {
				if(isset($aGroups[$i]["value"])) {
					$sCurrentGroup	= $aGroups[$i]["name"];
					break;
				}
				
			}
			
		}
		else {
			foreach ($aGroups as $aGroup) {
				if($aGroup["active"] == 1) {
					$sCurrentGroup	= $aGroup["name"];
					break;
				}
				
			}
			
		}
		
		$bFlag			= true;
		$sNextGroupName	= "";
		$sLastGroup		= "";
		$aDontSearch	= array();
		$aGroupValues	= array();
		
		foreach($aGroups as $aG) {
			//print_r($aG);
			if((int)$aG["active"]) {
				$sLastGroup		= $aG["name"];
				
				if($bFlag && $aG["name"] != $sCurrentGroup) {
					$bFlag			= false;
					$sNextGroupName	= $aG["name"];
				}
				
			}
			
			if(isset($aG["value"])) {
				array_push($aDontSearch, $aG["name"]);
				$aGroupValues[$aG["name"]]	= $aG["value"];
				
				if($aG["name"] == $sCurrentGroup) {
					$sNextGroupName	= "";
					$bFlag = true;
				}
				
			}
			
		}
		

		$this->aParams["showGroupChild"]	= (int) MSF_SL::getParam($_GET, "rs_showGroupChild", 0);
		$this->aParams["groupAction"]		= MSF_SL::getParam($_GET, "rs_groupAction", "load");
		$this->aParams["groupIdentity"]		= MSF_SL::getParam($_GET, "rs_group_identity", "");
		$this->aParams["isGroup"]			= $bIsGroupFlag;
		$this->aParams["aGroups"]			= $aGroups;
		$this->aParams["currentGroup"]		= $sCurrentGroup;
		$this->aParams["nextGroup"]			= $sNextGroupName;
		$this->aParams["lastGroup"]			= $sLastGroup;
		$this->aParams["aGroupDontSearch"]	= $aDontSearch;
		$this->aParams["aGroupValues"]		= $aGroupValues;
	}
	
	private function prepareTypes()
	{
		foreach($this->aColumns as $aColumn) {
			$this->aColumnsTypes[$aColumn["name"]]	= $aColumn["type"];
		}
	}

	private function grabEnumFromGet($sName, $aValids, $sDefault)
	{		
		$sParam	= MSF_SL::getParam($_GET, $sName, $sDefault);
		$sTmpParam = str_ireplace("\\'", "'", $sParam);					
		if(!in_array($sTmpParam, $aValids)) {											
			throw new MSF_RecordsetException("Wrong GET parameter '" . $sName . "'='" . $sParam . "'");
		}
		$aTmp = explode(" ", $sParam);
		$sParam = $aTmp[count($aTmp) - 1];
		
		return $sParam;
	}
	
	public function checkTotal($iTotal, $iPerPage)
	{
		if($this->aParams["limitFrom"] >= $iTotal) {
			//v cikle potomu, chto sostoyanie mozhet ne sohranyatsya
			while($this->aParams["limitFrom"] >= $iTotal) {
				$this->aParams["limitFrom"]	= $this->aParams["limitFrom"] - $iPerPage;
			}
			
			if($this->aParams["limitFrom"] < 0) {
				$this->aParams["limitFrom"]	= 0;
			}
			
		}
		
	}
	
}

?>
