<?php
if(!defined("__MSF")) {
	die("Direct access is not allowed");
}

class MSF_RS_SqlBuilder
{
	private $aColumns			= array();
	
	private $aTables			= array();
	
	private $aReferenceMap		= array();
	
	private $aConditions		= array();
	
	/**
	 * @var MSF_RS_Params
	 */
	private $Params				= null;
	
	private $iRecordPerPage		= 0;
	
	private $sSecondaryOrder	= null;
	
	/**
	 * @var MSF_DB
	 */
	private $Db 				= null;
	
	function __construct(MSF_RS_Params $Params, $iRecordsPerPage)
	{
		$this->Db 				= MSF_Registry::get("MSF_DB");
		$this->Params			= $Params;
		$this->iRecordPerPage	= $iRecordsPerPage;
	}
	
	public function setRecordsPerPage($iRecordsPerPage)
	{
		$this->iRecordPerPage	= $iRecordsPerPage;
	}
	
	public function setColumns($aInput)
	{
		$this->aColumns	= $aInput;
	}
	
	public function setTables($aInput)
	{
		$this->aTables	= $aInput;
	}
	
	public function setSecondaryOrder($sOrder)
	{
		$this->sSecondaryOrder	= $sOrder;
	}
	
	public function setConditions($aInput)
	{
		$this->aConditions	= $aInput;
	}
	
	public function addConditions($aInput)
	{
		$this->aConditions	= array_merge($this->aConditions, $aInput);
	}
	
	public function getPlainSql()
	{
		$sSQL	= 	$this->getSelect() .
					"\n" .
					$this->getFrom() .
					"\n" .
					$this->getWhere() .
					"\n" .
					$this->getGroup() .
					"\n" .
					$this->getOrder() .
					"\n" .
					$this->getLimit();
		
		return $sSQL;
	}
	
	public function getColor($sColorField)
	{
		$sSQL	=	"SELECT " . $sColorField . " AS color" .
					"\n" .
					$this->getFrom() .
					"\n" .
					$this->getWhere() .
					"\n" .
					$this->getOrder();
					
		return $sSQL; 
	}
			
	public function getTotalSql()
	{
		if($this->Params->get("isGroup")) {
			$sSQL	=	"SELECT count(*) AS total" .
						"\n" .
						"FROM (" . "\n" .
						$this->getSelect() .
						"\n" .
						$this->getFrom() .
						"\n" .
						$this->getWhere() .
						"\n" .
						$this->getGroup() .
						") AS a";
		}
		else {
			$sSQL	=	"SELECT count(*) AS total" .
						"\n" .
						$this->getFrom() .
						"\n" .
						$this->getWhere();
		}
		
		return $sSQL;
	}
	
	private function getFrom()
	{
		$sStr	= "FROM" . "\n";
		$sStr	.= implode(",\n" , $this->aTables);
				
		return $sStr;
	}
	
	private function getSelect()
	{
		$sCurrentGroup	= $this->Params->get("currentGroup");
		$sNextGroup		= $this->Params->get("nextGroup");
		$sLastGroup		= $this->Params->get("lastGroup");
		$aDontSearch	= $this->Params->get("aGroupDontSearch");
		$aGroupValues	= $this->Params->get("aGroupValues");
		
		
		$sStr	= "SELECT" . "\n";
		
		
		if($sCurrentGroup != "") {
			if(!empty($aGroupValues) && $sNextGroup != "") {
				$sStr 	.= $sNextGroup . ", COUNT(*) AS rs_total_in_group";
			}
			else {
				if($sLastGroup == $sCurrentGroup && !empty($aGroupValues)) {
					$i = 0;
					foreach($this->aColumns as $aColumn) {
						if(!in_array($aColumn["name"], $aDontSearch)) {
							if($i > 0) {
								$sStr .= ", ";
							}
							
							$sStr .= $aColumn["name"];
							$i++;
						}
					
					}
				}
				else {
					$sStr 	.= $sCurrentGroup . ", COUNT(*) AS rs_total_in_group";
				}
				
			}
			
		}
		else {
			$i = 0;
			foreach($this->aColumns as $aColumn) {
				if($i > 0) {
					$sStr .= ", ";
				}
				
				$sStr .= $aColumn["name"];
				$i++;
				
			}
			
		}
		
		return $sStr;
	}
	
	private function getLimit()
	{	/*
		if($this->Params->get("isGroup")) {
			$sStr	= "";
		}
		else {
			$sStr	=	"LIMIT" . "\n" .
							$this->iRecordPerPage . "\n" .
						"OFFSET" . "\n" .
							$this->Params->get("limitFrom");
		}*/
		$sStr = "";
		if($this->iRecordPerPage != 0) {
			$iOffset	= $this->Params->get("limitFrom");		
			if($iOffset % $this->iRecordPerPage) {
				$iOffset	= 0;
			}
			
			$sStr	=	"LIMIT" . "\n" .
							$this->iRecordPerPage . "\n" .
						"OFFSET" . "\n" .
							$iOffset;
		}
		
		return $sStr;
	}
	
	private function getOrder()
	{
		$sCurrentGroup	= $this->Params->get("currentGroup");
		$sNextGroup		= $this->Params->get("nextGroup");
		$sLastGroup		= $this->Params->get("lastGroup");
		$aGroupValues	= $this->Params->get("aGroupValues");
		$aTypes			= $this->Params->getTypes();
		
		if($sCurrentGroup != "") {
			if(!empty($aGroupValues) && $sNextGroup != "") {
				$sStr	= 	"ORDER BY" . "\n" .	$sNextGroup;
				if(in_array($aTypes[$sNextGroup], array("date", "timestamp"))) {
					$sStr	.= " DESC";
				}
				else {
					$sStr	.= " ASC";
				}
				
			}
			else {
				if($sLastGroup == $sCurrentGroup && !empty($aGroupValues)) {
					$sStr	= 	"ORDER BY" . "\n" .	$this->Params->get("orderBy") . " " . $this->Params->get("orderDirection");
					if($this->sSecondaryOrder) {
						$sStr	.= ", " . $this->sSecondaryOrder;
					}
					
				}
				else {
					$sStr	= 	"ORDER BY" . "\n" .	$sCurrentGroup;
									
					if(in_array($aTypes[$sCurrentGroup], array("date", "timestamp"))) {
						$sStr	.= " DESC";
					}
					else {
						$sStr	.= " ASC";
					}
					
				}
				
			}
						
		}
		else {
			$sStr	= 	"ORDER BY" . "\n" .	$this->Params->get("orderBy") . " " . $this->Params->get("orderDirection");
			if($this->sSecondaryOrder) {
				$sStr	.= ", " . $this->sSecondaryOrder;
			}
		}
		
		return $sStr;
	}
	
	private function getWhere()
	{
		$sStr			= "WHERE 1=1";
		$aSearch		= $this->Params->get("aFilters");
		$aGroupValues	= $this->Params->get("aGroupValues");
		
		foreach($this->aConditions as $sCondition) {
			$sStr	.= "\n" . $sCondition;
		}
		
		// kostyl
		if ((!MSF_IS_ADMIN) && (MSF_MODULE == 'workflow') && (count($aSearch) > 0)) {
			$sStr .= "\n" . "AND (\nlower(CAST(wf_documents.document_id AS text)) LIKE '00000'\nOR\n1=1";
		}
		
		foreach($aSearch as $sName=>$aField) {
			$sName = $this->Db->escapeString($sName);
			
			foreach ($aField as $sFieldKey => $sFieldValue) {
				$aField[$sFieldKey] = $this->Db->escapeString($sFieldValue);
			}
			
			if($aField["isMulti"] == 1) {
				if($aField["mFrom"] !== "" && $aField["mTo"] !== "") {
					$sStr	.= "\n" . "AND " . $sName . " BETWEEN '" . $aField["mFrom"] . "' AND '" . $aField["mTo"] . "'";
				}
			}
			else {
				if(in_array($aField["sType"], array("int", "float"))) {
					if (is_numeric($aField["mValue"])) {
						$sStr	.=	"\n" . "AND " . $sName . " " . $aField["sCondition"] . " '" . $aField["mValue"] . "'";
					}	
				}
				else if (in_array($aField["sType"], array("date", "timestamp"))){
					$sStr	.=	"\n" . "AND " . $sName . " " . $aField["sCondition"] . " '" . $aField["mValue"] . "'";
				}
				else {
					// костыль 
					//$sStr .= "\n" . "AND " . $sName . " = '" . $aField["mValue"] . "'";
					///*
					if(($aField["sType"] == "table") || ($aField["sCompare"] == "like" && !in_array($aField["sType"], array("int", "float", "date", "timestamp")))) {
						$sNotEqual = "";
						$sOrNull   = "";
						if (isset($aField["sCondition"]) && $aField["sCondition"] == "!=") {
							$sNotEqual 	= "NOT ";
							$sOrNull	= "OR lower(" . $sName . ") IS NULL";
						}
						$mValue = str_replace(array("!", "%"), array("!!", "!%"), mb_strtolower($aField["mValue"], "utf-8"));
						$sStr .= "\n" . "AND (lower(" . $sName . ") " . $sNotEqual . "LIKE '%" 
							  . $mValue . "%' ESCAPE '!'" . $sOrNull . ")";
					}
					else {
						$sStr .= "\n" . "AND " . $sName . " = '" . $aField["mValue"] . "'";
					}
					//*/
				}
				
			}
			
		}
		// kostyl
		if ((!MSF_IS_ADMIN) && (MSF_MODULE == 'workflow') && (count($aSearch) > 0)) {
			$sStr .= "\n)\n";
		}
		
		$aTypes	= $this->Params->getTypes();
		foreach($aGroupValues as $sKey=>$sValue) {
			$sKey 	= $this->Db->escapeString($sKey);
			//$sValue = $this->Db->escapeString($sValue);
			if($sValue === null) {
				$sStr .= "\n" . "AND " . $sKey . " IS NULL";
			}
			elseif($sValue == "") {
				switch($aTypes[$sKey]) {
					case "date"			:
					case "timestamp"	:
						$sStr .= "\n" . "AND " . $sKey . " IS NULL";
						break;
					case "int"			:
					case "float"		:
						$sStr .= (is_numeric($sValue)) ? "\n" . "AND (" . $sKey . " = '"  . $sValue . "' OR " . $sKey . " IS NULL)"
							 						   : "\n" . "AND " . $sKey . " IS NULL";
						break;	
					default				:
						$sStr .= "\n" . "AND (" . $sKey . " = '"  . $sValue . "' OR " . $sKey . " IS NULL)";					
						break;
				}
				
			}
			else {
				$sStr .= "\n" . "AND " . $sKey . " = '"  . $this->Db->escapeString($sValue) . "'";
			}
		
		}
		
		$sQuickFilter	= mb_strtolower($this->Params->get("sQuickFilter"), "utf-8");
		if($sQuickFilter != "") {
			$aTmp			= array();
			$sQuickFilter 	= str_replace(array("!", "%"), array("!!", "!%"), $this->Db->escapeString($sQuickFilter));
			
			// kostyl
			if ((!MSF_IS_ADMIN) && (MSF_MODULE == 'workflow')) {
				array_push($aTmp, "lower(CAST(wf_documents.document_id AS text)) LIKE '00000'");
			}
			
			foreach($this->aColumns as $aColumn) {
				foreach ($aColumn as $sKey => $sValue) {
					$aColumn[$sKey] = $this->Db->escapeString($sValue);
				}
				
				if($aColumn["visible"] && !strstr($aColumn["name"], "AS")) {
					switch ($aColumn["type"]) {
						case 'date':
						case 'timestamp':
							if (preg_match('@([0-9]{0,2})\.?([0-9]{0,2})\.?([0-9]{0,4})@si', $sQuickFilter)) {
								$sQuickFilter1 = preg_replace('@([0-9]{2})\.([0-9]{2})\.([0-9]{4})@si','\\3-\\2-\\1',$sQuickFilter);
								$sQuickFilter1 = preg_replace('@([0-9]{2})\.([0-9]{2})\.([0-9]{3})@si','\\3_-\\2-\\1',$sQuickFilter1);
								$sQuickFilter1 = preg_replace('@([0-9]{2})\.([0-9]{2})\.([0-9]{2})@si','\\3__-\\2-\\1',$sQuickFilter1);
								$sQuickFilter1 = preg_replace('@([0-9]{2})\.([0-9]{2})\.([0-9]{1})@si','\\3___-\\2-\\1',$sQuickFilter1);
								$sQuickFilter1 = preg_replace('@([0-9]{2})\.([0-9]{2})\.@si','____-\\2-\\1',$sQuickFilter1);
								$sQuickFilter1 = preg_replace('@([0-9]{2})\.([0-9]{4})@si','\\2-\\1',$sQuickFilter1);
								$sQuickFilter1 = preg_replace('@([0-9]{2})\.([0-9]{4})@si','\\2-\\1',$sQuickFilter1);
								$sQuickFilter1 = preg_replace('@([0-9]{2})\.([0-9]{2})@si','\\2-\\1',$sQuickFilter1);
								$sQuickFilter1 = preg_replace('@([0-9]{2})\.([0-9]{1})@si','\\2_-\\1',$sQuickFilter1);
								$sQuickFilter1 = preg_replace('@([0-9]{2})\.@si','\\__-\\1',$sQuickFilter1);
								array_push($aTmp, "CAST(" . $aColumn["name"] . " AS text) LIKE '%" . $sQuickFilter1 . "%' ESCAPE '!'");		
							}
							else {
								array_push($aTmp, "CAST(" . $aColumn["name"] . " AS text) LIKE '%" . $sQuickFilter . "%' ESCAPE '!'");
							}
							break;
						case 'int':
						case 'float':
							if (is_numeric($sQuickFilter)) {
								array_push($aTmp, "CAST(" . $aColumn["name"] . " AS text) LIKE '%" . $sQuickFilter . "%' ESCAPE '!'");
							}
							break;
						default:
							array_push($aTmp, "lower(" . $aColumn["name"] . ") LIKE '%" . $sQuickFilter . "%' ESCAPE '!'");
							break;

					}
				}
				
				
			}
			
			if(!empty($aTmp)) {
				$sStr	.= "\n" . "AND (" . implode(" OR ", $aTmp) . ")";
			}
			
		}
				
		return $sStr;
	}

	private function getGroup()
	{
		$bIsGroup		= $this->Params->get("isGroup");
		$sCurrentGroup	= $this->Params->get("currentGroup");
		$sNextGroup		= $this->Params->get("nextGroup");
		$sLastGroup		= $this->Params->get("lastGroup");
		$sStr			= "";
		$aGroups		= $this->Params->get("aGroups");
		$aDontSearch	= $this->Params->get("aGroupDontSearch");
		$aGroupValues	= $this->Params->get("aGroupValues");
		
		if($sCurrentGroup != "") {
			if(!empty($aGroupValues) && $sNextGroup != "") {
				$sStr .= "GROUP BY" . "\n" . $sNextGroup;
			}
			else {
				if($sLastGroup == $sCurrentGroup && !empty($aGroupValues)) {
					//echo $sLastGroup . " = " . $sCurrentGroup . "\n";
				}
				else {
					$sStr .= "GROUP BY" . "\n" . $sCurrentGroup;
				}
				
			}
			
		}
		else {
			
		}
		
		return $sStr;
	}
	
	public function getSum($aColumnSum) {
		$aSQL	= array();
		
		foreach ($aColumnSum AS $sKey => $sName) {
			$aSQL[] = "SUM(" . $sKey . ") as \"" . $sKey . "\"";
		}
		
		$sSQL	= 	"SELECT " .
					implode(", ", $aSQL) .
					"\n" .
					$this->getFrom() .
					"\n" .
					$this->getWhere();
		
		return $sSQL;
	}
}

?>
