<?php
if(!defined("__MSF")) {
	die("Direct access is not allowed");
}

include_once "MSF_RS_SqlBuilder.php";
include_once "MSF_RS_Params.php";
include_once "MSF_RS_Urls.php";
include_once "MSF_RecordsetException.php";
include_once "MSF_RS_State.php";
include_once "library/MSF_SearchUtil/MSF_SearchUtil.php";
include_once "MSF_AutoCompletor.php";
class MSF_Recordset
{
	/**
	 * @var MSF_DB
	 */
	private $DB					= null;
		
	/**
	 * @var MSF_RS_SqlBuilder
	 */
	private $SqlBuilder			= null;
	
	/**
	 * @var MSF_RS_Params
	 */
	private $Params				= null;
	
	/**
	 * @var MSF_RS_Urls
	 */
	private $Urls				= null;
	
	/**
	 * @var MSF_RS_State
	 */
	private $State				= null;
	
	/**
	 * @var array
	 */
	private $aActions			= array();
	
	/**
	 * @var array
	 */
	private $aLink				= array();
	
	/**
	 * @var array
	 */
	private $aMultiActions		= array();
	
	/**
	 * @var string
	 */
	private $sMultiActionField	= "";
	
	/**
	 * @var array
	 */
	private $aRecordset			= array();
	
	/**
	 * @var array
	 */
	private $aColumns			= array();
	
	/**
	 * @var array
	 */
	private $aMarkParams		= array();
	
	private $iRecordsOnPage		= -1;
	
	private $bExport 			= false;
	
	public $bCalculateSum		= true;
	
	public static $RECORDS_PER_PAGE		= 10;
	
	public static $GROUPS_PER_PAGE		= 10;
	
	public static $RECORDS_PER_GROUP	= 100;
	
	const PAGINATION_SIDE_SIZE			= 3;
	
	private $bLmsHookOn 				= false;
	
	private $bLmsHookOpts 				= array();
	
	private $sColorField				= "";
	
	public function __construct(MSF_DB $DB)
	{
		$this->DB			= $DB;
		$this->Params		= new MSF_RS_Params();
		$this->SqlBuilder	= new MSF_RS_SqlBuilder($this->Params, self::$RECORDS_PER_PAGE);
		$this->Urls			= new MSF_RS_Urls($this->Params);
		$this->State		= new MSF_RS_State($this->DB);
	}
	
		
	public function setActions($aInput)
	{
		$this->aActions	= $aInput;
	}
	
	public function setLink($aInput)
	{
		$this->aLink	= $aInput;
	}
	
	public function setMultiActions($aInput)
	{
		$this->aMultiActions	= $aInput;
	}

	public function setMultiActionField($sField)
	{
		$this->sMultiActionField	= $sField;
	}

	public function setBaseUrl($sUrl)
	{
		$this->Urls->setBaseUrl($sUrl);
	}
	
	public function setRpcUrl($sUrl)
	{
		$this->Urls->setRpcUrl($sUrl);
	}
	
	public function setStateParams($aInput)
	{
		$this->State->setStateParams($aInput);
	}
	
	public function setParam($sName, $Value) {
		$this->Params->set ( $sName, $Value );
	}
	
	public function setExport($bExport) {
		$this->bExport = $bExport;
	}
	
	public function setColumns($aInput)
	{
		$this->aColumns	= $aInput;
		$this->Params->setColumns($aInput);
		$this->SqlBuilder->setColumns($aInput);
		$this->State->setColumns($aInput);
	}
	
	public function setMarkParams($aInput)
	{
		$this->aMarkParams	= $aInput;
	}
	
	public function setRecordsOnPage($iCount)
	{
		$this->iRecordsOnPage = $iCount;
	}
	
	public function setTables($aInput)
	{
		$this->SqlBuilder->setTables($aInput);
	}
	
	public function setConditions($aInput)
	{
		$this->SqlBuilder->setConditions($aInput);
	}
	
	public function setSecondaryOrder($sOrder)
	{
		$this->SqlBuilder->setSecondaryOrder($sOrder);
	}
	
	public function setReferenceMap()
	{}
	
	public function setSessionUrlName()
	{}
	
	public function setDefaultOrder($sColumn)
	{
		$this->Params->setDefaultOrder($sColumn);
	}
	
	public function setDefaultDirection($sDirection)
	{
		$this->Params->setDefaultDirection($sDirection);	
	}
	
	public function setDefaultFilters($aInput)
	{
		$this->Params->setDefaultFilters($aInput);
	}
	
	public function setDefaultGroups($aInput)
	{
		$this->Params->setDefaultGroups($aInput);
	}
	
	public function setColorField($sField) {
		$this->sColorField = $sField;
	}
	
	private function buildRecords($bForArchive = false)
	{
		try 
		{
			if($this->bLmsHookOn && $aParams = $this->Params->get('aGroups', false))
			{
			    if( isset($aParams[0]['name']) &&
			        isset($aParams[0]['active']) &&
			        $this->bLmsHookOpts['field'] == $aParams[0]['name'] &&
			        $aParams[0]['active'] == 1)
			        $this->bLmsHookOn = true;
			    else
			        $this->bLmsHookOn = false;
			}
			
			if(!$bForArchive) {
				$aSearch 	= $this->DB->select($this->SqlBuilder->getPlainSql());
				//echo $this->SqlBuilder->getPlainSql();
				//die('hererer');
				$aColor		= array();
				if ($this->sColorField != "") {
					$aColor = $this->DB->select($this->SqlBuilder->getColor($this->sColorField));
				}
				
			}
			else {
				$oArchDB = new ARCH_DBRecord();			
				$aSearch = $oArchDB->select($this->SqlBuilder->getPlainSql());
			}
			$aRecords	= array();
			$aVisible	= $this->getVisibleFields();
		
			if (!empty($aSearch)) {
				$aLmsHooksRecords = array();
				
				foreach ($aSearch as $aRow){
					$aItem 		= array();
					$aColumns 	= array();
					$aActions 	= array();
	
					if($this->sMultiActionField != "" && isset($aRow[$this->sMultiActionField])) {
						$aItem["multiactionField"]	= $aRow[$this->sMultiActionField];
					}

					$iMark	= 0;
					if(!empty($this->aMarkParams) && isset($aRow[$this->aMarkParams["field"]])) {
						$mValue	= $aRow[$this->aMarkParams["field"]];
						$mArg	= $this->aMarkParams["argument"];
						
						switch ($this->aMarkParams["condition"]) {
							case "="	:
								$iMark = ($mValue == $mArg) ? 1 : 0;
								break;
							case "<"	:
								$iMark = ($mValue < $mArg) ? 1 : 0;
								break;
							case ">"	:
								$iMark = ($mValue > $mArg) ? 1 : 0;
								break;
							case "!="	:
								$iMark = ($mValue > $mArg) ? 1 : 0;
								break;
						}
						
					}
					
					foreach ($aRow as $sName => $sValue) {
						if(!isset($aVisible[$sName])) {
							continue;
						}
						
						$aColumn 				= array();
						$aColumn["value"]			= MSF_RS_Params::toOutput($sValue, $aVisible[$sName]["type"], $aVisible[$sName]["optional"], $sName);
						$aColumn["original"]		= $sValue;
						$aColumn["slash_original"]	= addslashes($sValue);
						$aColumn["mark"]			= $iMark;
						$aColumn["type"]			= $aVisible[$sName]["type"];
						$aColumn["name"]			= $sName;
						$aColumn['text_class'] 		= '';
						
						if($this->bLmsHookOn && $sName == $this->bLmsHookOpts['field']) {
							$aLmsHooksRecords[$sValue] = 0;
						}
	
						array_push($aColumns, $aColumn);
					}
	
	
					foreach ($this->aActions as $aAction) {
						$sAddToUrl = "";
	
						if (isset($aAction["bind"])) {
							foreach ($aRow AS $sKey => $sValue) {
								if (in_array($sKey, $aAction["bind"])) {
									$sAddToUrl .= "/" . $sValue;
								}
	
							}
	
						}
	
	
						$aElmAction = array();
						$aElmAction["title"]	= $aAction["title"];
						$aElmAction["image"]	= $aAction["image"];
						$aElmAction["href"]		= $aAction["href"] . $sAddToUrl;
						if (isset($_GET['hideMenu'])) {
							if (!strstr($aElmAction["href"], '?')) {
								$aElmAction["href"]		= $aElmAction["href"]. '?hideMenu';
							}
							else {
								$aElmAction["href"]		= $aElmAction["href"]. '&hideMenu';
							}
							
						}

						$aElmAction["export"] = (strpos($aAction["href"], "export")>0) ? 1 : 0;
						$aElmAction["confirm"]	= (int)$aAction["confirm"];
						$aElmAction["question"]	= isset($aAction["question"]) ? $aAction["question"] : MSF_Registry::get("MSF_Locale")->getWord("QUESTION1");
						$aElmAction["blank"]	= isset($aAction["blank"]) 	  ? $aAction["blank"]    : false;
						
						array_push($aActions, $aElmAction);
					}
					
					$sLink	= "";
					if(!empty($this->aLink)) {
						$sLink	= $this->aLink["href"];
						
						foreach ($aRow AS $sKey => $sValue) {
							if (in_array($sKey, $this->aLink["bind"])) {
								$sLink .= "/" . $sValue;
							}
	
						}
						
					}
	
					$aItem["columns"]			= $aColumns;
					$aItem["actions"]			= $aActions;
					$aItem["link"]				= $sLink;
					$aItem["total_in_group"]	= MSF_SL::getParam($aRow, "rs_total_in_group", 0);
					array_push($aRecords, $aItem);
				}
				
				try {
    				if($this->bLmsHookOn && sizeof($aLmsHooksRecords) > 0 && $this->operateWithData($aLmsHooksRecords))
    				{
    				    foreach($aRecords as $iKey => $dontuseit)
    				    {
    				        if(isset($aRecords[$iKey]['columns'][0]))
    				        {
    				            if($aLmsHooksRecords[$aRecords[$iKey]['columns'][0]['original']] > 0)
    				            {
    				                $aRecords[$iKey]['columns'][0]['text_class'] = 'rs-bold-font';
    				            }
    				        }
    				    }
    				}
    			}
    			catch(Exception $e){}
			}
			for ($i=0; $i<sizeof($aRecords); $i++) {
				if (isset($aColor[$i])) {
					$aRecords[$i]["color"] = (isset($aColor[$i]["color"])) ? $this->correctColor($aColor[$i]["color"]) : "#000";
				}
				else {
				  $aRecords[$i]["color"] = "#000";
				}
			}
			$this->aRecordset["records"]		= $aRecords;
			$this->aRecordset["groupValues"]	= $this->Params->get("aGroupValues");
			//$this->aRecordset["nextGroup"]		= $this->Params->get("nextGroup");
		}
		catch (Exception $e)
		{
			MSF_Logger::logTrace($e);
			$m = MSF_Registry::get("MSF_Messenger");
			$l = MSF_Registry::get("MSF_Locale");
			$m->addError($l->getWord("RS_STATE_RESETED"));
			if (!isset($_GET["rsStateReseted"])) {
				$m->redirect($_GET["msf_path"] . "?noStateRecover&rsStateReseted");
			}
			
		}

	}
	
	private function correctColor($sColor) {
		if (preg_match('/^#?+[0-9a-f]{3}(?:[0-9a-f]{3})?$/i', $sColor)) {
			return $sColor;
		}
		else {
			return "#000";
		}
	}
	
	//TODO: metod, pohozhe, bolshe ne nuzhen. Udalit potom
	private function buildRecordsGroups()
	{
		//echo $this->SqlBuilder->getPlainSql();
		//die("ololo");
		$aSearch 	= $this->DB->select($this->SqlBuilder->getPlainSql());
		$aVisible	= $this->getVisibleFields();
		
		$aColumns	= array();
		$i			= 0;
		
		if (!empty($aSearch)) {
			foreach ($aSearch as $aRow){
				$aReturnColumns	= array();
				$aReturnActions	= array();
				$iMark			= 0;

				if(!empty($this->aMarkParams) && isset($aRow[$this->aMarkParams["field"]])) {
					$mValue	= $aRow[$this->aMarkParams["field"]];
					$mArg	= $this->aMarkParams["argument"];
					
					switch ($this->aMarkParams["condition"]) {
						case "="	:
							$iMark = ($mValue == $mArg) ? 1 : 0;
							break;
						case "<"	:
							$iMark = ($mValue < $mArg) ? 1 : 0;
							break;
						case ">"	:
							$iMark = ($mValue > $mArg) ? 1 : 0;
							break;
						case "!="	:
							$iMark = ($mValue > $mArg) ? 1 : 0;
							break;
					}
					
				}
				
				foreach ($aRow as $sName => $sValue) {
					if(!isset($aVisible[$sName])) {
						continue;
					}
					
					$aTmp	= array();
					//$aTmp["value"]	= $this->toOutput($sValue, $this->aColumnsTypes[$sName]);
					//$aTmp["value"]	= $sValue;
					$aTmp["value"]	= MSF_RS_Params::toOutput($sValue, $aVisible[$sName]["type"], $aVisible[$sName]["optional"]);
					$aTmp["origin"]	= $sValue;
					$aTmp["mark"]	= $iMark;
					
					array_push($aReturnColumns, $aTmp);
				}


				foreach ($this->aActions as $aAction) {
					$sAddToUrl 		= "";
					$aReturnAction	= array();
					if (isset($aAction["bind"])) {
						foreach ($aRow AS $sKey => $sValue) {
							if (in_array($sKey, $aAction["bind"])) {
								$sAddToUrl .= "/" . $sValue;
							}

						}

					}

					$aReturnAction["title"]		= $aAction["title"];
					$aReturnAction["image"]		= $aAction["image"];
					$aReturnAction["href"]		= $aAction["href"] . $sAddToUrl;
					$aReturnAction["confirm"]	= (int)$aAction["confirm"];
					$aReturnAction["question"]	= isset($aAction["question"]) ? $aAction["question"] : MSF_Registry::get("MSF_Locale")->getWord("QUESTION1");
					$aReturnAction["blank"]		= isset($aAction["blank"]) 	  ? $aAction["blank"]    : false;
					
					array_push($aReturnActions, $aReturnAction);
				}

				$sLink	= "";
				if(!empty($this->aLink)) {
					$sLink	= $this->aLink["href"];
					
					foreach ($aRow AS $sKey => $sValue) {
						if (in_array($sKey, $this->aLink["bind"])) {
							$sLink .= "/" . $sValue;
						}

					}
					
				}
				
				$aColumns[$i]["columns"] 	= $aReturnColumns;
				$aColumns[$i]["actions"]	= $aReturnActions;
				$aColumns[$i]["link"]		= $sLink;
	
				
				if($this->sMultiActionField != "" && isset($aRow[$this->sMultiActionField])) {
					
					$aColumns[$i]["multiactionName"]	= $this->sMultiActionField;
					$aColumns[$i]["multiactionValue"]	= $aRow[$this->sMultiActionField];
				}

				$i++;
			}

		}
		//print_r($aColumns);
		$aReturn = array();
		$aReturn["columns"]		= $aColumns;
		$aReturn["groupValues"]	= $this->Params->get("aGroupValues");
		$aReturn["nextGroup"]	= $this->Params->get("nextGroup");
			

		//$aReturn = array();
		return $aReturn;
	}
	
	private function buildTitles()
	{
		$aColumns	= array();
		$aExists	= array();
		$aGroups	= $this->Params->get("aGroups");
		$sGroupMode	= $this->Params->get("groupMode");

		foreach($aGroups as $aGroup) {
			if($aGroup["active"] == 1) {
				foreach($this->aColumns as $aColumn) {
					if($aColumn["name"] == $aGroup["name"]) {
						if($sGroupMode == "table") {
							if(isset($aGroup["value"])) {
								array_push($aExists, $aColumn["name"]);
							}
							
						}
						else {
							array_push($aColumns, $aColumn);
							array_push($aExists, $aColumn["name"]);
							break;
						}
							
					}
										
				}
				
			}
			
		}
		foreach($this->aColumns as $aColumn) {
			if(!in_array($aColumn["name"], $aExists) && $aColumn["visible"]) {
				array_push($aColumns, $aColumn);
			}
			
		}
			
		
		$aTitles	= array();
		$aColumnSum = array();
		$isFilter	= 0;
		$aFilters	= $this->Params->get("aFilters");
							
		foreach($aColumns as $aColumn) {
			if(!$aColumn["visible"]) {
				continue;
			}
			
			$aTitle	= array();
			$aTmp = explode(" ", $aColumn["name"]);			
			if($aTmp[count($aTmp) - 1] == $this->Params->get("orderBy")) {
				$aTitle["orderBy"]	= $this->Params->get("orderDirection");
			}
			else {
				$aTitle["orderBy"]	= "none";
			}
	
	
			$bIsGroup	= false;
			
			foreach($aGroups as $aGroup) {
				if($aGroup["name"] == $aColumn["name"] && $aGroup["active"] == 1) {
					$bIsGroup	= true;
					break;
				}
			}
			
			
			$aTitle["isGroup"]	= (int)$bIsGroup;
			
			foreach($aColumn as $sKey=>$sVal) {
				$aTitle[$sKey]	= $sVal;
			}
			
			if(isset($aFilters[$aColumn["name"]])) {
				$aTitle["filterEnable"]		= 1;
				$aFilter	= $aFilters[$aColumn["name"]];
				if(isset($aFilter["sCondition"])) {
					$aTitle["condition"]	= $aFilter["sCondition"];
				}
				if($aFilter["isMulti"] == 0) {
					if ($aFilter["mValue_original"] !== "") {
						$aTitle["filterValue"]	= $aFilter["mValue_original"];
					}
					else if($aFilter["mValue"] !== "") {
						$aTitle["filterValue"]	= $aFilter["mValue"];
					}
					else {
						$aTitle["filterValue"]	= "";
					}
					
				}
				else {
					$aTitle["filterValue_from"]	= $aFilter["mFrom_original"];
					$aTitle["filterValue_to"]	= $aFilter["mTo_original"];
					if ($aTitle["filterValue_from"] == "" || $aTitle["filterValue_to"] == ""){
						$aTitle["filterEnable"]		= 0;
					}
				}
				
				$aTitle["isMulti"]			= $aFilter["isMulti"];
			}
			else {
				$aTitle["filterEnable"]		= 0;
			}
			
			array_push($aTitles, $aTitle);
			
			if ($aColumn["type"] == "int" || $aColumn["type"] == "float") {
				$aNameField 					= explode(".", $aColumn["name"]);
				$sName							= isset($aNameField[1]) ? $aNameField[1] : $aColumn["name"];
				$aColumnSum[$aColumn["name"]] 	= $sName;
			}
		}
		if (!empty($aColumnSum) && $this->bCalculateSum) {
			$this->aRecordset["columnSum"] = $this->DB->select($this->SqlBuilder->getSum($aColumnSum), true);
		}

		$this->aRecordset["isFilter"]	= $isFilter;
		$this->aRecordset["titles"]		= $aTitles;
	}
	
	private function buildPagination($iTotalRecords, $iPerPage)
	{
		//$iTotalRecords	= $this->DB->select($this->SqlBuilder->getTotalSql(), true, "total");
		$iTotalPages 	= $iTotalRecords / $iPerPage;
		$iLimitFrom		= $this->Params->get("limitFrom");
		
		if($iTotalPages > (int)$iTotalPages || $iTotalPages == 0) {
			$iTotalPages = (int)$iTotalPages + 1;
		}

		$aPagination							= array();

		$iCurrentPage	= $iLimitFrom / $iPerPage + 1;
		if($iCurrentPage > (int)$iCurrentPage || $iCurrentPage == 0) {
			$iCurrentPage = (int)$iCurrentPage + 1;
		}

		$iPreviusLimit	= $iLimitFrom - $iPerPage;
		if($iPreviusLimit < 0) {
			$iPreviusLimit = -1;
		}

		$iNextLimit		= $iLimitFrom + $iPerPage;
		if($iNextLimit >= $iTotalRecords) {
			$iNextLimit = -1;
		}

		$iPageFrom		= $iCurrentPage - self::PAGINATION_SIDE_SIZE;
		$iPageTo		= $iCurrentPage + self::PAGINATION_SIDE_SIZE;

		if($iPageFrom < 1) {
			while($iPageFrom < 1) {
				$iPageFrom++;
				if($iPageTo < $iTotalPages) {
					$iPageTo++;
				}

			}

		}

		if($iPageTo > $iTotalPages) {
			while($iPageTo > $iTotalPages) {
				$iPageTo--;
				if($iPageFrom > 1) {
					$iPageFrom--;
				}

			}

		}

		$aPagination["totalPages"]		= $iTotalPages;
		$aPagination["totalRecords"]	= $iTotalRecords;
		$aPagination["currentPage"]		= $iCurrentPage;
		$aPagination["previusPage"]		= $iPreviusLimit;
		$aPagination["nextPage"]		= $iNextLimit;
		$aPagination["lastPage"]		= ($iTotalPages - 1) * $iPerPage;

		$aPages					= array();
		$aPagination["pages"]	= &$aPages;

		for($i = $iPageFrom; $i <= $iPageTo; $i++) {
			$aPage	= array();


			$aPage["number"]		= $i;
			$aPage["limitFrom"]	= ($i - 1) * $iPerPage;

			if($i == $iCurrentPage) {
				$aPage["currentPage"]	= 1;
			}
			else {
				$aPage["currentPage"]	= 0;
			}

			array_push($aPages, $aPage);
		}

		$this->aRecordset["pagination"]	= $aPagination;
	}

	private function buildUrls()
	{
		$aUrls				= array();
		$aUrls["orderUrl"]	= $this->Urls->getOrder();
		$aUrls["limitUrl"]	= $this->Urls->getLimit();
		$aUrls["baseUrl"]	= $this->Urls->getBase();
		$aUrls["filterUrl"]	= $this->Urls->getFilters();
	//	echo $aUrls["filterUrl"] . "<br/>:)<br/>";
		$aUrls["rpcUrl"]	= $this->Urls->getRpc();
		$aUrls["groupUrl"]	= $this->Urls->getGroups();
		$this->aRecordset["urls"]	= $aUrls;
	}

	private function buildGroups()
	{
		$aGroups			= array();
		$aCurrentGroups		= $this->Params->get("aGroups");
		$aInGroup			= array();
		
		foreach($aCurrentGroups as $aCurr) {
			if($aCurr["active"] == 0) {
				continue;
			}
			
			foreach($this->aColumns as $aColumn) {
				if($aCurr["name"] == $aColumn["name"]) {
					$aTmp			= array();
					$aTmp["name"]	= $aColumn["name"];
					$aTmp["active"]	= 1;
				
					array_push($aGroups, $aTmp);
					array_push($aInGroup, $aColumn["name"]);
				}
				
			}
			
		}

		
		foreach($this->aColumns as $aColumn) {
			if((int)$aColumn["group"] == 1 && !in_array($aColumn["name"], $aInGroup)) {
				$aTmp			= array();
				$aTmp["name"]	= $aColumn["name"];
				$aTmp["active"]	= 0;
				
				array_push($aGroups, $aTmp);
						
			}
			
		}
		if (!isset($_REQUEST["clearGroupState"])) {
			$this->aRecordset["groupsToRestore"]	= $this->State->getGroupToRestore($this->Params->get("aGroupValues"));
		}
		else {
			$this->aRecordset["groupsToRestore"]	= array();
		}
		$this->aRecordset["showGroupChild"]		= $this->Params->get("showGroupChild");
		$this->aRecordset["isGroup"]			= (int)$this->Params->get("isGroup");
		$this->aRecordset["sCurrentGroup"]		= $this->Params->get("currentGroup");
		$this->aRecordset["groups"]				= $aGroups;
		$this->aRecordset["groupMode"]			= $this->Params->get("groupMode");
		$this->aRecordset["groupIdentity"]		= $this->Params->get("groupIdentity");
	}

	private function buildMultiActions()
	{
		
		if($this->sMultiActionField == "") {
			$this->aRecordset["isMultiAction"]	= 0;
			return false;
		}

		$this->aRecordset["isMultiAction"]		= 1;
		$this->aRecordset["multiActionField"]	= $this->sMultiActionField;

		$aMultiActions 						= array();
		
		foreach($this->aMultiActions as $aAction) {
			$aMultiAction = array();

			$aMultiAction["title"]		= $aAction["title"];
			if (isset($_GET['hideMenu'])) {
				if (!strstr($aAction["href"], '?')) {
					$aMultiAction["href"]		= $aAction["href"]. '?hideMenu';
				}
				else {
					$aMultiAction["href"]		= $aAction["href"]. '&hideMenu';
				}
				
			}
			else {
				$aMultiAction["href"]		= $aAction["href"];
			}

			$aMultiAction["image"]		= $aAction["image"];
			$aMultiAction["confirm"]	= (int)$aAction["confirm"];
			$aMultiAction["question"]	= isset($aAction["question"]) ? $aAction["question"] : MSF_Registry::get("MSF_Locale")->getWord("QUESTION1");

			array_push($aMultiActions, $aMultiAction);
		}
		
		$this->aRecordset["multiactions"]	= $aMultiActions;
	}
	
	private function buildQuickFilter()
	{
		$this->aRecordset["quickFilter"]	=  stripcslashes($this->Params->get("sQuickFilter"));
	}
	
	private function getVisibleFields()
	{
		$aFields	= array();

		foreach ($this->aColumns as $aColumn) {
			if($aColumn["visible"]) {
				$aTmp	= split(" ", $aColumn["name"]);
				$sName	= $aTmp[(count($aTmp) - 1)];
				$aTmp	= split("\.", $sName);
				$sName	= $aTmp[(count($aTmp) - 1)];
								
				$aFields[$sName]["type"]	= $aColumn["type"];
				if(isset($aColumn["optional"])) {
					$aFields[$sName]["optional"]	= $aColumn["optional"];
				}
				else {
					$aFields[$sName]["optional"]	= array();
				}
				
			}
			
		}
		
		return $aFields;
	}

	public function clearState()
	{
		$this->State->clearState();
	}

	public function getAllRecords($bForArchive = false)
	{
		$this->Params->grabParams ();
		$aPreParams = $this->Params->getParams ();
		
		$aParams = $this->State->checkParamsState ( $aPreParams );
		if ($this->bExport) {
			$aParams ["limitFrom"] = $aPreParams ["limitFrom"];
		}
		//$this->Urls = new MSF_RS_Urls($this->Params);		
		
		if($this->State->isClearGroupState()) {
			$this->State->clearGroupState();
		}
		
		$this->Params->setParams($aParams);
		$this->buildUrls();
		if (strlen($this->sTable) > 0) {
			$this->buildContextSearch();
		}
		$this->buildQuickFilter();		
		if (isset($aParams["rs_search"])) {
			$this->doContextSearch($aParams["rs_search"]);
		}

		if(!$bForArchive) {
			$iTotalRecords	= $this->DB->select($this->SqlBuilder->getTotalSql(), true, "total");
		}
		else {
			$oArchDB = new ARCH_DBRecord();
			$iTotalRecords = $oArchDB->select($this->SqlBuilder->getTotalSql(), true, "total");		
		}
		
		$this->Params->checkTotal($iTotalRecords, self::$RECORDS_PER_PAGE);
		$this->buildTitles();
		$this->buildRecords($bForArchive);
		$this->buildPagination($iTotalRecords, self::$RECORDS_PER_PAGE);
		$this->buildGroups();
		$this->buildMultiActions();
		if($this->Params->get("isGroup")) {
			$this->SqlBuilder->setRecordsPerPage(self::$RECORDS_PER_GROUP);
		}

		return $this->aRecordset;
	}
	
	public function getGroup($bForArchive = false)
	{
		$this->Params->grabParams();
		if($this->iRecordsOnPage == -1) {
			$this->SqlBuilder->setRecordsPerPage(self::$RECORDS_PER_GROUP);
		}
		else {
			$this->SqlBuilder->setRecordsPerPage($this->iRecordsOnPage);
		}
				
		$this->State->saveGroupsState($this->Params->getParams());
		
		if($this->Params->get("groupAction") == "load") {
			$this->buildQuickFilter();
			if ($this->Params->get("rs_search")) {
				$this->doContextSearch($this->Params->get("rs_search"));
			}
			
			if(!$bForArchive) {
				$iTotalRecords = $this->DB->select($this->SqlBuilder->getTotalSql(), true, "total");
			}
			else {
				$oArchDB	   = new ARCH_DBRecord();
				$iTotalRecords = $oArchDB->select($this->SqlBuilder->getTotalSql(), true, "total");				
			}
			
			$this->Params->checkTotal($iTotalRecords, self::$RECORDS_PER_GROUP);
			$this->buildTitles();			
			$this->buildRecords($bForArchive);
			$this->buildPagination($iTotalRecords, self::$RECORDS_PER_GROUP);
			$this->buildUrls();
			$this->buildGroups();
			$this->buildMultiActions();
			$this->aRecordset["sCurrentGroup"]	= $this->Params->get("nextGroup");
			$this->aRecordset["dataLoad"]		= true;
		}
		else {
			$this->aRecordset					= array();
			$this->aRecordset["dataLoad"]		= false;
		}
				
		//debug($_GET["msf_path"],MSF_Registry::get("MSF_DB")->getCount(), $_SERVER["REQUEST_URI"]);
		return $this->aRecordset;
	}
	/**
	 *
	 */
	private $sTable 		= "";
	
	private $sFieldname		= "";
	
	private $iContextId		= 0;
	
	public function enableContextSearch($sTable, $sFieldname, $iContextId) 
	{
		$this->sTable		= $sTable;
		$this->sFieldname 	= $sFieldname;
		$this->iContextId	= $iContextId;
	}
	
	private function buildContextSearch() 
	{
		$oMenu				= MSF_Registry::get("MSF_Menu")->getRoot();
		$oMenu->setAttribute("contextSearchEnabled", 1);
		$oMenu->setAttribute("contextSearchAction", $this->aRecordset["urls"]["filterUrl"]);
		//debug($this->aRecordset["urls"], $_GET);
	}
	
	private function makeSearchUrl()
	{
		
	}
	
	private function doContextSearch($sText) 
	{
		$oMenu				= MSF_Registry::get("MSF_Menu")->getRoot();
		if (strlen($sText) > 0) {
			$oMenu->setAttribute("contextSearchValue", $sText);
			//debug(MSF_SearchUtil::newInstance($sText)->getSqlInjection($this->sTable, $this->sFieldname, $this->iContextId));
			$this->SqlBuilder->addConditions(array(" AND " . MSF_SearchUtil::newInstance($sText)->getSqlInjection($this->sTable, $this->sFieldname, $this->iContextId)));
		}
		//debug(MSF_SearchUtil::newInstance($sText)->getSounds());
		//debug(MSF_SearchUtil::newInstance($sText)->getSqlInjection($this->sTable, $this->sFieldname, $this->iContextId));

	}
	
	public function setLmsHookOn($iUserId, $sTable, $sFieldName)
	{
    	$this->bLmsHookOn = true;
    	$this->bLmsHookOpts = array(
    	    'user_id' => $iUserId,
    		'table'   => $sTable,
    		'field'   => $sFieldName
    	);
	}
	
	private function operateWithData(&$aData)
	{
	    $aTmp = array();
	    foreach($aData as $k => $v)
	        $aTmp[] = "'".str_replace("'", "\\'", $k)."'";
        
	    $sql = 'SELECT '.$this->bLmsHookOpts['field'].', COUNT(*) as rs_new_records ';
	    $sql.= 'FROM '.$this->bLmsHookOpts['table']." WHERE status = 'new' AND user_id = ".$this->bLmsHookOpts['user_id'];
	    $sql.= ' AND '.$this->bLmsHookOpts['field'].' IN ('.join(',', $aTmp).') GROUP BY '.$this->bLmsHookOpts['field'];
	    
	    $aResultData = $this->DB->select($sql);
	    
	    if(sizeof($aResultData) > 0)
	    {
    	    foreach($aResultData as $aRow)
    	    {
    	        $aData[$aRow[$this->bLmsHookOpts['field']]] = $aRow['rs_new_records'];
    	    }
    	    return true;
	    }
	    
	    return false;
	}
}

