<?php
if(!defined("__MSF")) {
	die("Direct access is not allowed");
}

class MSF_RecordsetXml extends MSF_ViewSupportXml
{
	protected $Recordset;

	protected $aData;

	private $aRowIdentities	= array();
	
	public function buildRecordset($aData)
	{
		$this->Recordset	= $this->Template->createElement("recordset");
		$this->Recordset->setAttribute("isFilter", $aData["isFilter"]);
		$this->Recordset->setAttribute("isMultiaction", $aData["isMultiAction"]);
		$this->Recordset->setAttribute("isGroup", $aData["isGroup"]);

		if($aData["isMultiAction"] === 1) {
			$this->Recordset->setAttribute("multiactionField", $aData["multiActionField"]);
		}

		$this->Root->appendChild($this->Recordset);

		$this->aData	= $aData;
		
		$this->buildUrls();
		$this->buildTitles();
		
		$this->buildPagination();
		$this->buildRecords();
		$this->buildGroups();
		$this->buildMultiactions();
		$this->buildConditions();
		$this->buildQuickFilter();
		$this->attachResurses();
		
	}
	
	public function buildGroup($aData)
	{
		$this->Recordset = $this->Template->createElement("recordset");
		$this->Root->appendChild($this->Recordset);
		
		if (isset($aData["isMultiAction"])) {
			$this->Recordset->setAttribute("isMultiaction", $aData["isMultiAction"]);
		}
				
		if($aData["dataLoad"]) {
			$this->Recordset->setAttribute("dataLoad", 1);
			$this->aData	= $aData;
			$this->buildUrls();
			$this->buildTitles();
			$this->buildRecords();
			$this->buildGroups();
			$this->buildPagination();
			
			$this->buildMultiactions();
			$this->buildConditions();
			$this->attachResurses();
		}
		else {
			$this->Recordset->setAttribute("dataLoad", 0);
		}
		
		$this->Template->loadXsl(CNF_ABS_PATH . "/library/MSF_Recordset/resources/default/xslt/ajax.xsl");
	}

	private function buildUrls()
	{
		
		foreach($this->aData["urls"] as $sKey=>$sValue) {
			if (isset($_GET['hideMenu'])) {
				if (!strstr($sValue, '?')) {
					$sValue = $sValue . '?hideMenu';
				}
				else {
					$sValue = $sValue . '&hideMenu';
				}
				
			}
			
			$this->Recordset->setAttribute($sKey, $sValue);
		}

	}
	
	private function buildQuickFilter()
	{
		$this->Recordset->setAttribute("quickFilter", $this->aData["quickFilter"]);
	}

	private function buildTitles()
	{
		$Titles	= $this->Template->createElement("titles");
		$this->Recordset->appendChild($Titles);

		$Titles->setAttribute("isFilter", $this->aData["isFilter"]);
		$Titles->setAttribute("isMultiaction", $this->aData["isMultiAction"]);
		$Titles->setAttribute("isGroup", $this->aData["isGroup"]);
		
		//debug($this->aData["titles"]);
		$bColumnSumStatus 	= false;
		$aFilterPath		= array();
		foreach($this->aData["titles"] as $aTitle) {
			$Title	= $this->Template->createElement("title");
			$Titles->appendChild($Title);

			if($aTitle["filterEnable"] == 1) {
				array_push($aFilterPath, $aTitle["title"]);
			}
			
			foreach($aTitle as $sKey=>$sValue) {
//                                $sValue = $sValue;
                                if($sKey == "title"){
                                    $sValue = MSF_Registry::get("MSF_Locale")->getI18N($sValue);
                                }
				if(!is_array($sValue)) {
					$Title->setAttribute($sKey, $sValue);
				}
				else {
					$this->recConvert($Title, $sKey, $sValue);
				}
				
			}
			
			if (isset($this->aData["columnSum"][$aTitle["name"]])) {
				$bColumnSumStatus 	= true;
				$mColumnSum 		= "";
				if ($aTitle["type"] == "int") {
					$mColumnSum = number_format($this->aData["columnSum"][$aTitle["name"]], 0, ".", " ");
				}
				if ($aTitle["type"] == "float") {
					$mColumnSum = number_format($this->aData["columnSum"][$aTitle["name"]], 3, ".", " ");
					$mColumnSum = rtrim($mColumnSum, "0");
                                        $mColumnSum = rtrim($mColumnSum, ".");
				}

				$Title->setAttribute("columnSum", $mColumnSum);
			}
			
		}

		$Titles->setAttribute("filterPath", implode(" / ", $aFilterPath));
		$this->Recordset->setAttribute("columnSumStatus", $bColumnSumStatus);
	}

	private function recConvert($Node, $sName, $aData)
	{
		$New	= $this->Template->createElement($sName);
		$Node->appendChild($New);
		
		foreach($aData as $sKey=>$sValue) {
			if(is_array($sValue)) {
				$this->recConvert($New, $sKey, $sValue);
			}
			else {
				//"opt_" - esli budet chislo, to ne budet rabotat
				$Tmp	= $this->Template->createElement("item");
				$Tmp->setAttribute("key", $sKey);
				$Tmp->setAttribute("value", $sValue);
				$New->appendChild($Tmp);
			}
			
		}
		
	}
	
	private function buildPagination()
	{
		$Pagination	= $this->Template->createElement("pagination");
		$this->Recordset->appendChild($Pagination);

		$Pagination->setAttribute("totalPages", $this->aData["pagination"]["totalPages"]);
		$Pagination->setAttribute("totalRecords", $this->aData["pagination"]["totalRecords"]);
		$Pagination->setAttribute("currentPage", $this->aData["pagination"]["currentPage"]);
		$Pagination->setAttribute("previusPage", $this->aData["pagination"]["previusPage"]);
		$Pagination->setAttribute("nextPage", $this->aData["pagination"]["nextPage"]);
		$Pagination->setAttribute("lastPage", $this->aData["pagination"]["lastPage"]);

		$Pages	= $this->Template->createElement("pages");
		$Pagination->appendChild($Pages);

		foreach($this->aData["pagination"]["pages"] as $aPage) {
			$Page	= $this->Template->createElement("page");
			$Pages->appendChild($Page);

			foreach($aPage as $sKey=>$sValue) {
				$Page->setAttribute($sKey, $sValue);
			}

		}

	}

	private function buildRecords()
	{
		$Records	= $this->Template->createElement("records");
		$this->Recordset->appendChild($Records);
		$aIdentities	=	array();
		
		//debug($this->aData["groupsToRestore"]);
		
		foreach($this->aData["records"] as $aRecord) {
			$Item	= $this->Template->createElement("item");
			$Records->appendChild($Item);

			if($this->aData["isMultiAction"] && isset($aRecord["multiactionField"])) {
				$Item->setAttribute("multiactionFiled", $aRecord["multiactionField"]);
			}

			$Columns		= $this->Template->createElement("columns");
			$Item->appendChild($Columns);
			$sRowIdentity	= microtime();
			$sOriginalValue	= "";
			foreach($aRecord["columns"] as $aColumn) {
				$Column	= $this->Template->createElement("column");
				$Columns->appendChild($Column);
				if (($aColumn["type"] == "int") && is_numeric($aColumn["value"])) {
					$aColumn["value"] = number_format($aColumn["value"], 0, '.', ' ');	
				}
				if (($aColumn["type"] == "float") && is_numeric($aColumn["value"])) {
					$sFractionalPart 	= strstr($aColumn["value"], ".");
					$aColumn["value"] 	= number_format($aColumn["value"], strlen($sFractionalPart) - 1, '.', ' ');
				}				
//                                debug($aColumn);
//                                exit;
                                $msfLocale = MSF_Registry::get("MSF_Locale");
//                                debug($aColumn);
//                                exit;
                                $aColumn["value"] = $msfLocale->getI18N($aColumn["value"]);
//                                $aColumn["value"] = $msfLocale->getI18N("<russian>Системные</russian><english>System</english>");
				$Column->setAttribute("value", 			MSF_String::breakingLongWords($aColumn["value"]));
				$Column->setAttribute("original", 		$aColumn["original"]);
				$Column->setAttribute("slash_original", $aColumn["slash_original"]);
				$Column->setAttribute("mark", 			$aColumn["mark"]);
				$Column->setAttribute("type", 			$aColumn["type"]);
				$Column->setAttribute("text_class", 	$aColumn["text_class"]);
				if ($aRecord["color"]) {
					$Column->setAttribute("color", 			$aRecord["color"]);
				} else {
					$Column->setAttribute("color", 			"#000");
				}
				
				$sRowIdentity	.= $aColumn["original"];
				$sOriginalValue	= $aColumn["original"];
			}
			//getStateParams
			//debug($this->getStateParams($sOriginalValue));

			$sRowIdentity	= MSF_String::hash($sRowIdentity, 20);
			$Item->setAttribute("identity", $sRowIdentity);
			$Item->setAttribute("totalInGroup", $aRecord["total_in_group"]);
			$Item->setAttribute("getParams", $this->getStateParams($sOriginalValue));

			$aIdentities[MSF_String::hash($sOriginalValue, 20)]	= $sRowIdentity;
			
			$Actions	= $this->Template->createElement("actions");
			$Item->appendChild($Actions);

			foreach($aRecord["actions"] as $aAction) {
				$Action	= $this->Template->createElement("action");
				$Actions->appendChild($Action);

				foreach($aAction as $sKey=>$sValue) {
					$Action->setAttribute($sKey, $sValue);
				}
			}
			
			if($aRecord["link"]) {
				$Link	= $this->Template->createElement("link");
				$Link->setAttribute("value", $aRecord["link"]);
				$Item->appendChild($Link);
			}
			
		}
		
		$this->aRowIdentities	= $aIdentities;
	}
	
	private function getStateParams($sValue)
	{
		foreach($this->aData["groupsToRestore"] as $aGroup) {
			if($aGroup["value"] == $sValue) {
				return $aGroup["params"];
			}
		}
		
		return "";
	}

	private function buildGroups()
	{
		$Groups	= $this->Template->createElement("groups");
		$this->Recordset->appendChild($Groups);

		$Groups->setAttribute("currentGroup", $this->aData["sCurrentGroup"]);
		
		$aGroupPath	= array();
		foreach($this->aData["groups"] as $aGroup) {
			$Item	= $this->Template->createElement("item");
			$Groups->appendChild($Item);
			
			
			foreach($aGroup as $k=>$v) {
				$Item->setAttribute($k, $v);
			}
			
			if($aGroup["active"] == 1) {
				foreach($this->aData["titles"] as $aTitle) {
					if($aTitle["name"] == $aGroup["name"]) {
						array_push($aGroupPath, $aTitle["title"]);
					}
					
				}
				
			}
			
		}
		$Groups->setAttribute("groupPath", implode(" / ", $aGroupPath));
		
		$sGroupValues	= "";
		//debug($_POST);
		foreach($this->aData["groupValues"] as $sGroup=>$mValue) {
			$sGroupValues	.= "&rs_groupValues[" . $sGroup . "]=" . urlencode(str_replace("+", "&#43;", $mValue));
		}
				
		
		$Groups->setAttribute("groupValues", $sGroupValues);
		$Groups->setAttribute("groupMode", $this->aData["groupMode"]);
		$Groups->setAttribute("groupIdentity", $this->aData["groupIdentity"]);
		$Groups->setAttribute("showGroupChild", $this->aData["showGroupChild"]);

		//debug($this->aRowIdentities);
		$Restore	= $this->Template->createElement("restoreGroups");
		$this->Recordset->appendChild($Restore);
		

		//debug($this->aData["groupsToRestore"]);
		foreach($this->aData["groupsToRestore"] as $aGroup) {
			if(isset($this->aRowIdentities[MSF_String::hash($aGroup["value"], 20)]) && $aGroup["visible"] == 1) {
				$Item	= $this->Template->createElement("item");
				$Item->setAttribute("name", $aGroup["name"]);
				$Item->setAttribute("value", $aGroup["value"]);
				$Item->setAttribute("identity", $this->aRowIdentities[MSF_String::hash($aGroup["value"], 20)]);
				$Item->setAttribute("postParams", $sGroupValues);
				$Item->setAttribute("getParams", $aGroup["params"]);
				$Item->setAttribute("visible", $aGroup["visible"]);
				
				$Restore->appendChild($Item);
			}
			
		}
		
	}
	
	private function buildMultiactions()
	{
		if($this->aData["isMultiAction"]) {
			$Actions	= $this->Template->createElement("multiactions");
			$this->Recordset->appendChild($Actions);
			$Actions->setAttribute("field", $this->aData["multiActionField"]);

			foreach($this->aData["multiactions"] as $aAction) {
				$Action	= $this->Template->createElement("action");
				$Actions->appendChild($Action);

				foreach($aAction as $sKey=>$sVal) {
					$Action->setAttribute($sKey, $sVal);
				}

			}

		}

	}

	private function buildConditions()
	{
		
		$Conditions	= $this->Template->createElement("conditions");
		$this->Recordset->appendChild($Conditions);
		
		$aConditions	= array("=", "<", ">", "!=");
		
		foreach($aConditions as $sCondition) {
			$Item	= $this->Template->createElement("item");
			$Conditions->appendChild($Item);
			
			$Item->setAttribute("value", $sCondition);
		}

	}
	
		
	private function attachResurses()
	{
		$sPath 	= "/library/MSF_Recordset/resources/" . CNF_SKIN;

		$this->Template->addXslPath($sPath . "/xslt/recordset.xsl");
		$this->Template->addXslPath($sPath . "/xslt/table.xsl");
		$this->Template->addXslPath($sPath . "/xslt/group.xsl");
		$this->Template->addXslPath($sPath . "/xslt/groupTable.xsl");

		$this->Template->addLink($sPath . "/css/rs.css", "text/css");
		$this->Template->addLink($sPath . "/css/recordset.css", "text/css");
		$this->Template->addLink($sPath . "/css/RS_ContextMenu.css", "text/css");
		$this->Template->addLink($sPath . "/css/RS_GroupManager.css", "text/css");
		
		$this->Template->addLink($sPath . "/js/RS_Filter.js", "text/javascript");
		$this->Template->addLink($sPath . "/js/RS_RowHighlight.js", "text/javascript");
		$this->Template->addLink($sPath . "/js/RS_Actions.js", "text/javascript");
		$this->Template->addLink($sPath . "/js/RS_GroupManager.js", "text/javascript");
		$this->Template->addLink($sPath . "/js/RS_GroupPanel.js", "text/javascript");
		$this->Template->addLink($sPath . "/js/RS_ActionPanel.js", "text/javascript");
		$this->Template->addLink($sPath . "/js/RS_TreeQueue.js", "text/javascript");
		$this->Template->addLink($sPath . "/js/RS_EvalQueue.js", "text/javascript");
		$this->Template->addLink($sPath . "/js/RS_GroupSupply.js", "text/javascript");
		$this->Template->addLink($sPath . "/js/RS_HScroll.js", "text/javascript");
		
		$this->Template->addLink("/js/ext/calendar_date_select/js/calendar_date_select.js", "text/javascript");
		$this->Template->addLink("/js/ext/calendar_date_select/css/default.css", "text/css");
		$this->Template->addLink("/js/msf/date/DatePlaceholder.js", "text/javascript");
	}

}
?>