<?php
class MSF_Stemmer
{
    private $VERSION = "0.03";
    private $Stem_Caching = 0;
    private $Stem_Cache = array();
    private $VOWEL = '/(a|e|i|o|u|ju|ja)/i';
    private $PERFECTIVEGROUND = '/((iv|ivshi|ivshis|iv|ivshi|ivshis)|((?<=[aja])(v|vshi|vshis)))$/i';
    private $REFLEXIVE = '/(s[ja])$/i';
    private $ADJECTIVE = '/(ee|ie|ie|oe|imi|imi|ej|ij|ij|oj|em|im|im|om|ego|ogo|emu|omu|ih|ih|uju|juju|aja|jaja|oju|eju)$/i';
    private $PARTICIPLE = '/((ivsh|ivsh|ujush)|((?<=[aja])(em|nn|vsh|ush|ush)))$/i';
    private $VERB = '/((ila|ila|ena|ejte|jujte|ite|ili|ej|uj|il|il|im|im|en|ilo|ilo|eno|jat|uet|ujut|it|it|eni|it|it|ish|uju|ju)|((?<=[aja])(la|na|ete|jte|li|j|l|em|n|lo|no|et|jut|ni|t|esh|nno)))$/i';
    private $NOUN = '/(a|ev|ov|ie|e|e|ijami|jami|ami|ei|ii|i|iej|ej|oj|oj|j|ijam|jam|iem|em|am|om|o|u|ah|ijah|jah|i|iju|u|u|ija|ja|ja)$/i';
    private $RVRE = '/^(.*?[a|e|i|o|u|ju|ja])(.*)$/i';
    private $DERIVATIONAL = '/[^a|e|i|o|u|i|e|ju|ja][a|e|i|o|u|i|e|ju|ja]+[^a|e|i|o|u|i|e|ju|ja]+[a|e|i|o|u|i|e|ju|ja].*(?<=o)st?$/i';

    function s(&$s, $re, $to)
    {
        $orig = $s;
        $s = preg_replace($re, $to, $s);
        return $orig !== $s;
    }

    function m($s, $re)
    {
        return preg_match($re, $s);
    }

    function stem_word($word)
    {
       // $word = strtolower($word);
       try {
        $word = iconv('UTF-8', 'windows-1251', $word);
        $word = strtr($word, '', '');
       }
       catch (Exception $e) {
       	$word = "";
       }
        # Check against cache of stemmed words
        if ($this->Stem_Caching && isset($this->Stem_Cache[$word])) {
            return $this->Stem_Cache[$word];
        }
        $stem = $word;
        do {
          if (!preg_match($this->RVRE, $word, $p)) break;
          $start = $p[1];
          $RV = $p[2];
          if (!$RV) break;

          # Step 1
          if (!$this->s($RV, $this->PERFECTIVEGROUND, '')) {
              $this->s($RV, $this->REFLEXIVE, '');

              if ($this->s($RV, $this->ADJECTIVE, '')) {
                  $this->s($RV, $this->PARTICIPLE, '');
              } else {
                  if (!$this->s($RV, $this->VERB, ''))
                      $this->s($RV, $this->NOUN, '');
              }
          }

          # Step 2
          $this->s($RV, '/$/', '');

          # Step 3
          if ($this->m($RV, $this->DERIVATIONAL))
              $this->s($RV, '/?$/', '');

          # Step 4
          if (!$this->s($RV, '/$/', '')) {
              $this->s($RV, '/?/', '');
              $this->s($RV, '/$/', '');
          }

          $stem = $start.$RV;
        } while(false);
        if ($this->Stem_Caching) $this->Stem_Cache[$word] = $stem;
        return $stem;
    }

    function stem_caching($parm_ref)
    {
        $caching_level = @$parm_ref['-level'];
        if ($caching_level) {
            if (!$this->m($caching_level, '/^[012]$/')) {
                die(__CLASS__ . "::stem_caching() - Legal values are '0','1' or '2'. '$caching_level' is not a legal value");
            }
            $this->Stem_Caching = $caching_level;
        }
        return $this->Stem_Caching;
    }

    function clear_stem_cache()
    {
        $this->Stem_Cache = array();
    }
}