<?php
if(!defined("__MSF")) {
	die("Direct access is not allowed");
}

include_once "MSF_UniqueFieldDB.php";
include_once "MSF_UniqueFieldException.php";
include_once "modules/references/library/Reference/REF_Value.php";


class MSF_UniqueField
{
	/**
	 * @var MSF_UniqueFieldDB
	 */
	private $DB;
	
	const STRUCTURE_ID		= 9000000;
	
	const TYPE_REFERENCE	= "reference";	
	
	const TYPE_DOCTYPE		= "doctype";

	const TYPE_STRUCTURE	= "structure";
	
	/**
	 * @param MSF_DB $DB
	 * @return void
	 */
	public function __construct(MSF_DB $DB)
	{
		$this->DB	= new MSF_UniqueFiledDB($DB);
	}
	
	public function blockFields($aFields, $sType, $iId)
	{			
		foreach($aFields as $aField) {
			if($aField["type"] == "reference" && (int)$aField["optional"]["unique"] == 1) {				
				$this->block(
					$aField["value"], 
					$sType,
					$aField["optional"]["reference"], 
					$iId,
					$aField["name"]
				);
				
			}
			
		}
		
	}
	
	public function freeFields($aFields, $sType, $iId)
	{
		foreach($aFields as $aField) {
			if($aField["type"] == "reference" && (int)$aField["optional"]["unique"] == 1) {
				$this->free(
					$aField["value"], 
					$sType,
					$aField["optional"]["reference"], 
					$iId,
					$aField["name"]
				);			
			}
			
		}
	}
	
	/**
	 * @param string $sRefValue
	 * @param int $sType
	 * @param int $iSourceId
	 * @param int $iDestId
	 * @param string $sDestFieldName
	 */
	public function block($sRefValue, $sType, $iSourceId, $iDestId, $sDestFieldName)
	{
		//echo $sRefValue . " " . $sType . " " . $iSourceId . " " . $iDestId . " " . $sDestFieldName;
		try {
			$this->DB->block($this->parse($sRefValue), $sType, $iSourceId, $iDestId, $sDestFieldName);	
		}
		catch(Exception $e) {
			throw new MSF_UniqueFieldException("Can`t block value. Reason: " . $e->getMessage());
		}
					
	}
	
	
	/**
	 * @param string $sRefValue
	 * @param int $sType
	 * @param int $iSourceId
	 * @param int $iDestId
	 * @param string $sDestFieldName
	 */
	public function free($sRefValue, $sType, $iSourceId, $iDestId, $sDestFieldName)
	{
		$this->DB->free($this->parse($sRefValue), $sType, $iSourceId, $iDestId, $sDestFieldName);
	}
	
	/**
	 * @param string $sRefValue
	 * @param int $sType
	 * @param int $iSourceId
	 * @param int $iDestId
	 * @param string $sDestFieldName
	 */
	public function check($sRefValue, $sType, $iSourceId, $iDestId, $sDestFieldName)
	{
		return $this->DB->check($this->parse($sRefValue), $sType, $iSourceId, $iDestId, $sDestFieldName);
	}
	
	/**
	 * @param string $sBlockerType
	 * @param int $iBlockerId
	 * @param int $iDestId
	 * @param string $sDestFieldName
	 * @param string $sCurrentValues
	 * @return array
	 */
	public function getBlocked($sBlockerType, $iBlockerId, $iDestId, $sDestFieldName, $sCurrentValues = "")
	{
		if($sDestFieldName == "new_subdivision") {
			$sDestFieldName = "subdivision";
		}
		
		if($sDestFieldName == "move_subdivision") {
			$sDestFieldName = "subdivision";
		}
		
		$aResult	= $this->DB->getBlocked($sBlockerType, $iBlockerId, $iDestId, $sDestFieldName);
		$aReturn	= array();
		$aIds		= $this->parse($sCurrentValues);
						
		foreach($aResult as $aRow) {
			if(!in_array((int)$aRow["value"], $aIds)) {
				array_push($aReturn, (int)$aRow["value"]);	
			}
			
		}
		
		return $aReturn;
	}	
	
	private function parse($sRefValue)
	{
		$RV		= new REF_Value();
		$RV->parse($sRefValue);				
		$aIds	= $RV->getKeysList();
		
		return $aIds;
	}
	
}
?>