<?php
if(!defined("__MSF")) {
	die("Direct access is not allowed");
}
	
class MSF_WorkTime
{
	
	/**
	 * @var MSF_XmlWrapper
	 */
	private $Xml	= null;
	
	/**
	 * @var array
	 */
	private $aWorkWeek	= array();
	
	/**
	 * @var array
	 */
	private $aHolidays	= array();
	
	private $aWorkdays	= array();
	
	public function __construct() 
	{
		$Document	= new DOMDocument();
		$Document->load(CNF_ABS_PATH . "/" . CNF_CALENDAR_PATH);
		
		$this->Xml			= new MSF_XmlWrapper($Document->saveXML());
		$this->aWorkWeek	= (array) $this->Xml->get("workWeek")->getValue();
		
		$aHolidays	= (array) $this->Xml->get("holidays")->getValue();
		$aTmp		= array();
		foreach($aHolidays as $sHoliday) {
			array_push($aTmp, date("d.m.Y", strtotime($sHoliday . "-" . date("Y"))));
		}
		$this->aHolidays	= $aTmp;
		
		$aWorkdays	= (array) $this->Xml->get("workdays")->getValue();
		$aTmp		= array();
		foreach($aWorkdays as $sWorkday=>$aTime) {
			$aTmp[date("d.m.Y", strtotime($sWorkday . "-" . date("Y")))]	= $aTime;			
		}
		
		$this->aWorkdays	= $aTmp;			
	}

	public function getDate($iStart, $iDayNumbers, $bCheckHolidays, $fMulti = 1)
	{
		$aTimes	= $this->getWorkTime($iStart, $iDayNumbers, $bCheckHolidays);		
		$fTotalTime	= $this->getTotalTime($aTimes, $fMulti);
				
		if(empty($aTimes)) {
			$iSecond	= 0;
		}
		else {
			for($i=0; $i<count($aTimes); $i++) {			
				if($fTotalTime <= ($aTimes[$i]["end"] - $aTimes[$i]["start"])) {					
					$fHour	= $aTimes[$i]["start"] + $fTotalTime;								
					break;
				}
				
				$fTotalTime	-= $aTimes[$i]["end"] - $aTimes[$i]["start"];				
			}
			
			$iSecond	= strtotime($aTimes[$i]["date"] . " 00:00:00");			
			$iSecond	+= 60 * 60 * $fHour;			
		}
		
		return $iSecond;		
	}	
	
	private function getTotalTime($aTimes, $fMulti)
	{
		$fTotal	= 0.0;
		
		foreach($aTimes as $aTime)
		{
			$fTotal += $aTime["end"] - $aTime["start"];
		}
		
		return $fTotal * $fMulti;
	}
	
	private function getWorkTime($iStart, $iNumOfDays, $bCheckHolidays)
	{
		$iDate	= $iStart;		
		$aTimes	= array();
		$Date	= new MSF_Date($iStart);
		
		for($i=0; $i<=$iNumOfDays; $i++) {
			$iDay	= date("w", $iDate);
			if
			(					
				(
					!isset($this->aWorkWeek[$iDay]) || 
					in_array(date("d.m.Y", $iDate), $this->aHolidays)
					&& $bCheckHolidays
				)
				&& !isset($this->aWorkdays[date("d.m.Y", $iDate)])
			)
			{				
				$iNumOfDays++;						
			}
			else {
				$aTmp	= array();
				$aTmp["date"]	= date("d.m.Y", $iDate);
				if(isset($this->aWorkWeek[$iDay])) {
					$aTmp["start"]	= $this->aWorkWeek[$iDay]["start"];
					$aTmp["end"]	= $this->aWorkWeek[$iDay]["end"];
				}
				else {					
					$aTmp["start"]	= $this->aWorkdays[date("d.m.Y", $iDate)]["start"];
					$aTmp["end"]	= $this->aWorkdays[date("d.m.Y", $iDate)]["end"];					
				}
			
				array_push($aTimes, $aTmp);
			}
						
			$iDate	+= 60 * 60 * 24;
		}
		
		if(!empty($aTimes)) {
			$iBegin				= ($Date->getHour() + $Date->getMinute() / 60);
			$iFinalDay			= count($aTimes) - 1;				
			
			$aTimes[0]["start"] 		= $iBegin;
			if($iBegin >= $aTimes[$iFinalDay]["start"] && $iBegin <= $aTimes[$iFinalDay]["end"]) {				
				$aTimes[$iFinalDay]["end"]	= $iBegin;
			}
			elseif($iBegin < $aTimes[$iFinalDay]["start"]) {
				$aTimes[$iFinalDay]["end"]	= $aTimes[$iFinalDay]["start"];
			}
						
		}
		
		return $aTimes;
	}
	
}

?>