<?php
if (!defined("__MSF")) {
	exit("Direct access is not allowed");
}

class ARCH_Archive {
	
	private $Recordset;
	
	public function getRecords($iLimitFrom = -1, $bExport = false)
	{
		$this->buildRecordset($iLimitFrom, $bExport);		
		//$this->Recordset->enableContextSearch("ref_index", "id", $this->iRefId);		
		return $this->Recordset->getAllRecords(true);
	}
	
	public function getArchiveGroup()
	{
		$this->buildRecordset(true);		
		return $this->Recordset->getGroup(true);
	}
	
	private function buildRecordset($iLimitFrom = -1, $bExport = false)
	{
		$Locale			 = MSF_Registry::get("MSF_Locale");
		$this->Recordset = new MSF_Recordset(MSF_Registry::get("MSF_DB"), MSF_Registry::get("MSF_Template"));
		$this->Recordset->setExport ( $bExport );
		if ($iLimitFrom != - 1) {
			$this->Recordset->setParam ( "limitFrom", $iLimitFrom );
		}

		$this->Recordset->setBaseURL("/archive/?");
		$this->Recordset->setRpcURL("/archive/group/?");		
		$Locale	= MSF_Registry::get("MSF_Locale");

		$aActions = array(
			array (
				"title"		=> $Locale->getWord("WORKFLOW_ADMIN_LIST_ACTION_OPEN"),
				"href"		=> "/archive/view",
				"image"		=> "/templates/default/images/folder_page_white.png",
				"bind"		=> array("archive_id"),
				"confirm"	=> false
			),
			array (
				"title"		=> $Locale->getWord("WORKFLOW_ADMIN_LIST_ACTION_DELETE"),
				"href"		=> "/archive/remove",
				"image"		=> "/templates/default/images/red_cross.png",
				"bind"		=> array("archive_id"),
				"confirm"	=> true
			),
			
		);
		
		$aMultiActions	= array(
			array(
	 			"title"		=> $Locale->getWord("WORKFLOW_ADMIN_LIST_ACTION_DELETE"),
				"href"		=> "/archive/remove",
				"image"		=> "/templates/default/images/red_cross.png",
				"confirm"	=> true
	 		)
		);
		
		$this->Recordset->setActions($aActions);
		$this->Recordset->setMultiActions($aMultiActions);
 		$this->Recordset->setMultiActionField("archive_id");
 		
		$this->Recordset->setLink(
			array(
				"href"		=> "/archive/view",
				"bind"		=> array("archive_id")
			)
		);
		
		$this->Recordset->setStateParams(array(
			"module"		=> "archive",
			"action"		=> "show",
			"action_id"		=> 0,
			"structure_id"	=> MSF_Registry::get("MSF_UserBox")->getCurrentStructureId()
		));
		
		$this->setRecordsetSQL();
		$this->setRecordsetColumns();
		$this->setRecordsetConditions();
		$this->Recordset->setSecondaryOrder("archive_id");
	}
	
	private function setRecordsetSQL()
	{		
		$this->Recordset->setTables(array("wf_archive"));
	}
	
	private function setRecordsetColumns()
	{		
		$aColumns = array();
		$Locale	  = MSF_Registry::get("MSF_Locale");		

		$aColumnNames	= array(
			1 => array(
				"name" 	=> "archive_id",
				"title" => "ID",
				"type"	=> "int"
				),
			2 => array(
				"name"	=> "author_name",
				"title"	=> $Locale->getWord("WORKFLOW_ADMIN_LIST_AUTHOR"),
				"type"	=> "string"
				),
			3 => array(
				"name"	=> "subdivision_name",
				"title" => $Locale->getWord("STRUCTURE_MODULES_SUBDIV"),
				"type"	=> "string"
				),
			4 => array(
				"name"	=> "created_at",
				"title"	=> $Locale->getWord("WORKFLOW_ADMIN_LIST_CREATED"),
				"type"	=> "date"
				),
			5 => array(
				"name"	=> "modified_at",
				"title" => $Locale->getWord("WORKFLOW_ADMIN_LIST_CHANGED"),
				"type"	=> "timestamp"
				),
			6 => array(
				"name"	=> "archived_at",
				"title" => $Locale->getWord("WORKFLOW_ADMIN_LIST_ARCHIVED"),
				"type"	=> "timestamp"
				),
			7 => array(
				"name"	=> "doctype_title",
				"title" => $Locale->getWord("WORKFLOW_DOCTYPE_RECORDSET_TITLE"),
				"type"	=> "string"
				),
			8 => array(
				"name"	=> "registration_number",
				"title" => $Locale->getWord("WORKFLOW_FOLDERS_ADMIN_REGISTRATION_NUMBER"),
				"type"	=> "string"
				),
			9 => array(
				"name"	=> "document_title",
				"title"	=> $Locale->getWord("WORKFLOW_DOCTYPE_SPA_TITLE"),
				"type"	=> "string"
				)
		);
			
		$i = 1;
		$aExclusions = array("modified_at", "archived_at", "registration_number", "document_title");
		foreach($aColumnNames AS $aValue) {
			$aColumns[$i]["name"]  = $aValue["name"];
			$aColumns[$i]["title"] = $aValue["title"];
			$aColumns[$i]["type"] = $aValue["type"];
			if($aValue["name"] == "archive_id") {				
				$aColumns[$i]["sort"] = 1;
				$aColumns[$i]["filter"] = 1;
				$aColumns[$i]["group"] = 0;				
				$aColumns[$i]["visible"] = 0;
				$aColumns[$i]["compare"] = "equal";
			}
			else {
				$aColumns[$i]["sort"] = 1;
				$aColumns[$i]["filter"] = 1;
				if(!in_array($aValue["name"], $aExclusions)) {
					$aColumns[$i]["group"] = 1;									
				}
				else {
					$aColumns[$i]["group"] = 0;
				}				
				
				$aColumns[$i]["visible"] = 1;
				$aColumns[$i]["compare"] = "like";
			}
			
			$i++;
		}				
		
		$this->Recordset->setColumns($aColumns);	
	}
	
	private function setRecordsetConditions()
	{
		$UserBox = MSF_Registry::get("MSF_UserBox");
		$sPath	 = $UserBox->getCurrentStructurePath();				
		if (!empty($sPath)) {
			$sEmployeePath	= str_replace(".", ",", $sPath);			
		}
		else {
			$sEmployeePath	= "0";
		}		
		
		$aWhere = array();		
		$aWhere[] = 
		" AND 
		(
		(
		doctype_id IN 
		(
		SELECT DISTINCT doctype_id FROM wf_access_list
		WHERE structure_id IN (" . $sEmployeePath . ")
		)
		) 
		OR
		( 
		archive_id IN 
		(
		SELECT DISTINCT archive_id FROM wf_elements_access_list
		WHERE wf_elements_access_list.archive_id = archive_id AND
		structure_id IN (" . $sEmployeePath . ")
		)
		)
		)";
		
		$this->Recordset->setConditions($aWhere);
	}
	
}
?>