<?php
class ARCH_DBRecord {
	
	private $oDb;	
	
	public function __construct()
	{		
		$this->oDb = new MSF_DB();
		$aSettings = MSF_Registry::get("MSF_Settings")->getModule("archive");
		$sServer   = $aSettings["settings"]["arch_sql_host"];
		$sBase	   = $aSettings["settings"]["arch_sql_database"];
		$sUser	   = $aSettings["settings"]["arch_sql_user"];
		$sPassword = $aSettings["settings"]["arch_sql_password"];
		$sScheme   = $aSettings["settings"]["arch_sql_scheme"];
                $sPort     = $aSettings["settings"]["arch_sql_port"];
		$this->oDb->connect($sServer, $sBase, $sUser, $sPassword, $sScheme, $sPort);				
	}
	
	public function select($SQL, $bOneRow = false, $sRowName = null)
	{
		$mResult = array();
		
		$this->oDb->executeQuery($SQL);
		if ($bOneRow) {
			$mResult = $this->oDb->fetchArray();
			if($mResult === false) {
				$mResult = array();
			}
			
			if (!is_null($sRowName)) {
				if (array_key_exists($sRowName, $mResult)) {
					$mResult = $mResult[$sRowName];
				}
				else {
					throw new MSF_DBException("Index not found " . $sRowName);
				}

			}

		}
		else {
			while ($aRow = $this->oDb->fetchArray()) {
				array_push($mResult, $aRow);
			}

		}

		return $mResult;
	}
	
	public function findItem($iArchiveId) {
		
		$sSql = "SELECT * FROM wf_archive WHERE archive_id = ".$iArchiveId;
		$this->oDb->executeQuery($sSql);
		$aParams = $this->oDb->fetchArray();		
		return $aParams;
	}
	
	public function findItemByDocId($iDocumentId)
	{
		$sSql = "SELECT archive_id FROM wf_archive WHERE document_id = " . $iDocumentId;
		$this->oDb->executeQuery($sSql);
		return $this->oDb->fetchArray();
	}
	
	public function addItem($aArchiveData)
	{		
		$sSql = "INSERT INTO\n" .
					"wf_archive\n" .
				"(\n" .
					"structure_id,\n" .
					"author_name,\n" .
					"created_at,\n" .
					"modified_at,\n" .
					"archived_at,\n" .
					"doctype_id,\n" .
					"document_id,\n" .
					"registration_number, \n" .
					"document_title, \n" .
					"fields_xml, \n" .
					"sysfields_xml, \n" .
					"template_xml, \n" .
					"doctype_title," . "\n" .
					"subdivision_name," . "\n" .
					"media_id" . "\n" .
				")\n" .
				"values(\n" .
					$aArchiveData["structure_id"] . ",\n" .
					"'" . $aArchiveData["author_name"] . "',\n" .
					"'" . $aArchiveData["created_at"] . "',\n" .
					"'" . $aArchiveData["modified_at"] . "',\n" .
					"'" . $aArchiveData["archived_at"] . "',\n" .
					$aArchiveData["doctype_id"] . ",\n" .
					$aArchiveData["document_id"] . ",\n" .
					"'" . $this->escapeValue($aArchiveData["registration_number"]) . "',\n" .
					"'" . $this->escapeValue($aArchiveData["document_title"]) . "',\n" .
					"'" . $this->escapeValue($aArchiveData["fields_xml"]) . "',\n" .
					"'" . $this->escapeValue($this->replaceLangVar($aArchiveData["sysfields_xml"])) . "',\n" .
					"'" . pg_escape_string($this->escapeValue($aArchiveData["template_xml"])) . "',\n" .
					"'" . pg_escape_string($this->escapeValue($aArchiveData["doctype_title"])) . "',\n" .
					"'" . $this->escapeValue($aArchiveData["subdivision_name"]) . "',\n" .
					$aArchiveData["media_id"] . "\n" .					
				")";						
					
				$this->oDb->executeQuery($sSql);		
	}
	
	public function deleteItem($iArchiveId)
	{
		$sSql	=	"DELETE FROM" . "\n" . 
						"wf_archive" . "\n" .
					"WHERE" . "\n" .
						"archive_id = " . $iArchiveId;		
		
		$this->oDb->executeQuery($sSql);
	}
	
        private function replaceLangVar($sValue) {            
            $oLc = new MSF_Locale(CNF_ABS_PATH . '/languages/ru.ini');
            $sValue = str_replace("DIGITAL_SIGN_NOT_ACCEPTED", $oLc->getWord("DIGITAL_SIGN_NOT_ACCEPTED"), $sValue);
            $sValue = str_replace("DIGITAL_SIGN_ACCEPTED", $oLc->getWord("DIGITAL_SIGN_ACCEPTED"), $sValue);
            $sValue = str_replace("WFA_DECSION_ATTACH", $oLc->getWord("WFA_DECSION_ATTACH"), $sValue);
            
            return $sValue;
        }
        
	private function escapeValue($sValue)
	{
		$sValue = MSF_HtmlEntities::replace($sValue);
		$sValue = MSF_HtmlEntities::stripCommented($sValue);
		return $sValue;	
	}
	
	
}
?>