<?php
defined("__MSF") or die("Direct access is not allowed");

include_once "modules/lms/library/LMS_MailDB.php";
include_once "modules/lms/library/LMS_Mail.php";
include_once "modules/lms/library/LMS_MailList.php";
include_once "modules/lms/library/LMS_FoldersList.php";
include_once "modules/lms/library/LMS_FoldersDB.php";
include_once "modules/lms/library/LMS_Folders.php";
include_once "modules/lms/library/LMS_Addresses.php";
include_once "library/MSF_Utils/MSF_RteObjectsIntegrator.php";

include_once "modules/lms/view/xml/LMS_Xml.php";
include_once "modules/lms/view/xml/LMS_ListXml.php";
include_once "modules/lms/view/xml/LMS_FoldersListXml.php";
include_once "modules/lms/view/xml/LMS_FolderXml.php";
include_once "modules/lms/view/xml/LMS_AddressesXml.php";

include_once "library/MSF_Export/MSF_ExportToExcel.php";

class MSF_FrontController extends MSF_ControllerSupport
{
	private $Xml 		= null;

	private $ListXml 	= null;

	private $sAction 	= '';
	
	private $Mail 		= null;
	
	private $Folders 	= null;
	
	private $iMailId 	= 0;

	private $iFolderId 	= 0;

	private $iFileId 	= 0;
	
	private $aFolders 	= array();
	
	private $Ubox 		= null;
	
	public function __construct()
	{
		//echo MSF_String::decrypt("jOXF0ZF8flGhCOSg/y0q8w==");

		$this->Mail 	= new LMS_Mail(new LMS_MailDB(MSF_Registry::get("MSF_DB")));
		$this->Folders 	= new LMS_Folders(new LMS_FoldersDB(MSF_Registry::get("MSF_DB")));
		$this->Xml		= new LMS_Xml(MSF_Registry::get("MSF_Template"), MSF_Registry::get("MSF_Locale"));
		$this->ListXml 	= new LMS_ListXml(MSF_Registry::get("MSF_Template"), MSF_Registry::get("MSF_Locale"));
		$this->Ubox 	= MSF_Registry::get("MSF_UserBox");
		parent::__construct();
	}
	
	public function clearAction()
	{
		if (IS_REQUEST && MSF_SL::getParam($_POST, "clear_request") == "msf_clear_lms_tables") {
			$aTables 	= array("lms_inbox", "lms_outbox", "lms_sent", "lms_draft", "lms_trash", "lms_messages", "lms_emails");
			$DB 		= MSF_Registry::get("MSF_DB");
		
			for ($i = 0; $i < count($aTables); $i++) {
				$DB->executeQuery("TRUNCATE TABLE " . $aTables[$i]);
			}
			echo "Tables <" . implode(", ", $aTables) . "> cleared.";
		}
		else {
			echo "
			<form method=\"post\">
				<input name=\"clear_request\"/>
				<input type=\"submit\" value=\"OK\"/>
			</form>";
		}
	}

	public function indexAction()
	{
		$this->folderAction();
	}

	public function mailAction()
	{
		$this->sAction 		= (string) 	$this->getParam(0);
		$this->iFolderId 	= (int) 	$this->getIntParam(1);
		$this->iMailId 		= (int) 	$this->getIntParam(2);
		$this->iFileId 		= (int) 	$this->getIntParam(5);
		$this->aFolders 	= $this->addFolders($this->Folders->get(), array(), $this->iFolderId);
		$sTitle 			= $this->locale("LMS_TITLE_CREATE");
		$this->Mail->setFolderId($this->iFolderId);

		switch($this->sAction) {
			case "create":
			case "edit":
				$this->editMail();
				break;
			case "reply":
			case "forward":
				$this->replyMail();
				break;
			case "remove":
				$this->removeMail();
				break;
			case "moveto":
				$this->movetoMail();
				break;
			case "restore":
				$this->restoreMail();
				break;
			case "mark":
				$this->markMail();
				break;
			case "unmark":
				$this->unmarkMail();
				break;
			default:
				$sTitle = "";
				if ($this->getParam(3) == "file" && $this->getParam(4) == "remove" && $this->iFileId > 0) {
					$this->removeFile();
				}
				else {
					$this->viewMail();
				}
				break;
		}
		
		$this->addFolderTitle($this->iFolderId, $sTitle);
	}
	
	private function markMail()
	{
		if (IS_REQUEST) {
			$this->Mail->mark(MSF_SL::getParam($_POST, "message_id"));
			$this->redirect("/lms/folder/" . $this->iFolderId, "");
		}
	}
	
	private function unmarkMail()
	{
		if (IS_REQUEST) {
			$this->Mail->unmark(MSF_SL::getParam($_POST, "message_id"));
			$this->redirect("/lms/folder/" . $this->iFolderId, "");
		}
	}
	
	private function removeFile()
	{
		$this->Mail->removeFile($this->iMailId, $this->iFileId);
		$this->redirect("/lms/mail/view/" . $this->iFolderId . "/" . $this->iMailId);
	}
	
	private function movetoMail()
	{
		if (IS_REQUEST) {
			$this->moveTo($this->iMailId, $this->iFolderId, (int) MSF_SL::getParam($_POST, "folder_id"));
			$this->redirect("/lms/folder/" . $this->iFolderId, $this->locale("LMS_MESSAGE_EMAIL_MOVED"));
		}
	}
	
	private function restoreMail()
	{
		if (IS_REQUEST) {
			$this->restoreTo(MSF_SL::getParam($_POST, "message_id", array()), $this->iFolderId);
		}
		else {
			$this->restoreTo($this->iMailId, $this->iFolderId);
		}	
		
		$this->redirect("/lms/folder/" . $this->iFolderId);
	}

	private function editMail($bForw = false)
	{
		$aResult = array();
		$bCheck  = false;
		$bCreate = false;
		$bSmtp	 = true;

		if (IS_REQUEST) {
			$sErrorEmp 	= "";
			$sErrorMail = "";
			$sAction 	= (string) MSF_SL::getParam($_POST, "action");
			$sAddresses = trim(MSF_SL::getParam($_POST, "addresses"));
			$sCC		= trim(MSF_SL::getParam($_POST, "cc"));
			$sBCC 		= trim(MSF_SL::getParam($_POST, "bcc"));
			if (!empty($sCC)) {
				$sAddresses = empty($sAddresses) ? $sCC : $sAddresses . ", " . $sCC;
			}
			if (!empty($sBCC)) {
				$sAddresses = empty($sAddresses) ? $sBCC : $sAddresses . ", " . $sBCC;
			}
			$iForwId 	= ($bForw) ? $this->getIntParam(2) : 0;
			$bCheck 	= true;
			
			if ($sAction == "send") {				
				$bEmps 	= true;
				$bMails = true;
				$aCheck = $this->Mail->parseAddresses($sAddresses);
				
				if (empty($aCheck)) {
					$this->addError($this->locale("LMS_ERROR_EMAIL_NOT_GIVED"));
					$bCheck = false;
				}

				for ($i = 0; $i < count($aCheck); $i++) {
					switch ($aCheck[$i]["error_no"]) {
						case 1:
							$sErrorEmp .= $aCheck[$i]["name"] . ", ";
							$bCheck = false;
							$bEmps = false;
							break;
						case 2:
							$sErrorMail .= $aCheck[$i]["email"] . ", ";
							$bCheck = false;
							$bMails = false;
							break;
						case 3:
							$bSmtp = false;
							$bCheck = false;
							break;	
					}
				}
				
				if (!$bEmps) {
					$this->addError($this->locale("LMS_ERROR_EMPLOYEE_NOT_FOUND") . substr($sErrorEmp, 0, strlen($sErrorEmp) - 2));
				}
				if (!$bMails) {
					$this->addError($this->locale("LMS_ERROR_EMAIL_NOT_CORRECT") . substr($sErrorMail, 0, strlen($sErrorMail) - 2));
				}
				if (!$bSmtp) {
					$this->addError($this->locale("LMS_ERROR_SENT_SMTP_BODY_SMTP"));
				}
				
				
				if ($bForw) {
					$aResult = $this->Mail->open($iForwId);
				}
			}
			
			if ($bCheck) {
				$this->Mail->setAddresses((string) MSF_SL::getParam($_POST, "addresses"), true);
				$this->Mail->setAddresses((string) MSF_SL::getParam($_POST, "cc"), true, "cc");
				$this->Mail->setAddresses((string) MSF_SL::getParam($_POST, "bcc"), true, "bcc");
				$this->Mail->setSubject((string) MSF_SL::getParam($_POST, "subject"));
				$this->Mail->setBody((string) MSF_SL::getParam($_POST, "body"));
				
				switch ($sAction) {
					case "send":
						if ($this->iFolderId == 3) {
							$iForwId = $this->iMailId;
						}
				
						$this->Mail->save(0, true, false, $iForwId);
						if ($this->iFolderId == 3) {
							$this->Mail->remove($this->iMailId, true);
						}
						$this->redirect("/lms");
						exit();
					case "save":
						$iMailId = $this->Mail->save($this->iMailId, false, true, $iForwId);
						$this->redirect("/lms/mail/edit/" . LMS_Mail::DRAFT . "/" . $iMailId, $this->locale("LMS_MESSAGE_MAIL_SAVED"));
						exit();;
				}
			}
			
		}
		elseif ($this->iMailId > 0) {
			if ($this->iFolderId == 3) {
				$bCreate = true;
			}
			if ($this->Mail->exists($this->iMailId)) {
				$aResult = $this->Mail->open($this->iMailId);
				$aResult["to"]  		= addslashes($aResult["to"]);
				$aResult["cc"]  		= addslashes($aResult["cc"]);
				$aResult["bcc"]  		= addslashes($aResult["bcc"]);
			}
			else {
				$this->addError($this->Locale->getWord("LMS_ERROR_MAIL_NOT_EXISTS"));
			}
		}
		elseif ($this->iFolderId == 3) {
			 $bCreate = true;
		}
		else {
			 $bCreate = true;
		}
	
		if ($this->sAction == "create" || $this->sAction == "edit") {
			$bCreate = true;
		}
		if ($bCreate || !$bCheck) {
			
			if (empty($aResult)) {
				$aResult["to"]  		= addcslashes(MSF_SL::getParam($_POST, "addresses", "", true), '"');
				$aResult["cc"]  		= addcslashes(MSF_SL::getParam($_POST, "cc", "", true), '"');
				$aResult["bcc"]  		= addcslashes(MSF_SL::getParam($_POST, "bcc", "", true), '"');
				$aResult["subject"] 	= MSF_SL::getParam($_POST, "subject", "", true);
				$aResult["body"] 		= MSF_SL::getParam($_POST, "body", $this->Ubox->getUserAttribute("email_template"), true);
				$aResult["message_id"] 	= 0;
			}
			
			if ($bCreate) {
				$this->Xml->editOne($aResult, $this->iFolderId);
			}

		}
		else {
			$this->redirect("/lms/mail/view/" . $this->iFolderId . "/" . $this->iMailId);
		}
		
		$this->setView($this->Xml);
		return $aResult;
	}
	
	private function viewMail()
	{
		$aResult 	= array();
		$this->Mail->setFolderId($this->iFolderId);
		
		if ($this->Mail->exists($this->iMailId)) {
			if ($this->iFolderId == 3) {
				$this->redirect("/lms/mail/edit/" . $this->iFolderId . "/" . $this->iMailId);
			}
			$aResult = $this->Mail->open($this->iMailId);
			if (empty($aResult)) {
				MSF_Registry::get("MSF_Messenger")->addError(MSF_Registry::get("MSF_Locale")->getWord("LMS_ERROR_MAIL_NOT_EXISTS"));
			}
		}
		elseif ($this->iFolderId == -1 || $this->iMailId == -1) {
			$aResult = $this->Mail->openLast();
			if (empty($aResult)) {
				MSF_Registry::get("MSF_Messenger")->addError(MSF_Registry::get("MSF_Locale")->getWord("LMS_ERROR_MAIL_NOT_EXISTS"));
			}
		}
		else {
			MSF_Registry::get("MSF_Messenger")->addError(MSF_Registry::get("MSF_Locale")->getWord("LMS_ERROR_MAIL_NOT_EXISTS"));
		}
		
		$this->Xml->viewOne($aResult, $this->iFolderId, $this->aFolders, $this->iMailId);
		$this->setView($this->Xml);
	}
	
	private function replyMail()
	{
		$aResult = array();
		
		if (IS_REQUEST) {
			$bForw = false;
			if ($this->sAction == "forward") {
				$bForw = true;
			}
			$aResult = $this->editMail($bForw);
			$aResult["to"]  		= MSF_SL::getParam($_POST, "addresses");
			$aResult["cc"]  		= MSF_SL::getParam($_POST, "cc");
			$aResult["bcc"]  		= MSF_SL::getParam($_POST, "bcc");
			$aResult["subject"] 	= MSF_SL::getParam($_POST, "subject");
			$aResult["body"] 		= MSF_SL::getParam($_POST, "body");
			$aResult["message_id"] 	= 0;
		}
		else {
			$aResult = array();
			$this->Mail->setFolderId($this->iFolderId);
	
			if ($this->Mail->exists($this->iMailId)) {
				$aResult = $this->Mail->open($this->iMailId);
				$aResult["to"] = addcslashes($aResult["from"], '"') . ", ";
				
				if ($this->sAction == "reply") {
					$aResult["attachments"] = "N";
					$aResult["files"] 		= array();
				}
				else {
					$aResult["to"] 	= "";
					$aResult["cc"] 	= "";
					$aResult["bcc"] = "";
				}
				
				$this->Mail->formReplyAttributes($aResult, $this->sAction);
			}
			else {
				MSF_Registry::get("MSF_Messenger")->addError(MSF_Registry::get("MSF_Locale")->getWord("LMS_ERROR_MAIL_NOT_EXISTS"));
			}
		}
		
		$this->Xml->editOne($aResult, false, $this->iFolderId);
		$this->setView($this->Xml);
	}
	
	private function removeMail()
	{
		if (IS_REQUEST) {
			$this->Mail->remove(MSF_SL::getParam($_POST, "message_id", array()));
		}
		else {
			$this->Mail->remove($this->iMailId);
		}
		
		if ($this->iFolderId == LMS_Mail::TRASH) {
			$sMessage = $this->Locale->getWord("LMS_MESSAGE_MAILS_DELETED");
		}
		else {
			$sMessage = $this->Locale->getWord("LMS_MESSAGE_MAILS_DELETED_TO_TRASH");
		}
		
		$this->redirect("/lms/folder/" . $this->iFolderId, $sMessage);
	}
	
	
	
	

	public function foldersAction()
	{
		switch ($this->getParam(0)) {
			case "create":
			case "edit":
				$this->editFolder();
				break;
			case "remove":
				$this->removeFolder();
				break;
			default:
				$this->viewFolders();
				break;
		}
		
		$this->Template->setPageTitle(
			$this->Locale->getWord("LMS_MODULE_NAME") .
			$this->Locale->getWord("ROOT_TITLE_SEPARATOR") .
			$this->Locale->getWord("LMS_TITLE_MY_FOLDERS")
		);
	}
	
	public function folderAction()
	{
		$iFolderId = $this->getIntParam(0);
		$this->aFolders = $this->addFolders($this->Folders->get(), array(), $iFolderId);
		$this->addFolderTitle($iFolderId);
		MSF_AutoCompletor::setUri("/lms/folder/" . $iFolderId . "/autocomplete");
		if ($this->getParam(1) == "ajaxrequest") {
			$this->ajaxRequest($iFolderId);
		}
		else if ($this->getParam(1) == "autocomplete") {
			$this->autocomplete($iFolderId);
		}
		else {
			if (IS_REQUEST) {
				$sMessage 	= "";
				$aIds 		= MSF_SL::getParam($_POST, "message_id", array());
				$sIds 		= implode(",", $aIds);
				$iFolderId 	= (int) $this->getParam(0);
				if (!empty($sIds)) {
					$this->moveTo(
						$sIds,
						$iFolderId,
						(int) MSF_SL::getParam($_POST, "folder_id")
					);
				}
				if (count($aIds) > 1) {
					$sMessage = $this->locale("LMS_MESSAGE_EMAILS_MOVED");
				}
				elseif (count($aIds) == 1) {
					$sMessage = $this->locale("LMS_MESSAGE_EMAIL_MOVED");
				}
				$this->redirect("/lms/folder/" . $iFolderId, $sMessage);
			}
			
			if ($this->getParam(0) == "clear") {
				if ($this->getParam(1) == 4) {
					$aEmailIds = $this->Folders->getTrashEmailsIds();
					if (!empty($aEmailIds)) {
						$this->Mail->setFolderId(4);
						$this->Mail->remove($aEmailIds);
					}
					$this->redirect("/lms/folder/4", $this->locale("LMS_MESSAGE_TRASH_CLEARED"));
				}
				
				$this->redirect("/lms/folder/" . $this->getParam(1), "");
			}
			
			
			$List 		= new LMS_MailList();
			
			if ($List->exists($iFolderId)) {
				$this->ListXml->viewFolder(
					$List->open($iFolderId),
					$this->addFolders($this->Folders->get(), $this->Folders->getNewCount(),	$iFolderId),
					$iFolderId
				);
			}
			else {
				$this->ListXml->viewFolder(
					array(),
					$this->addFolders($this->Folders->get(), $this->Folders->getNewCount(),	$iFolderId),
					0
				);
				MSF_Registry::get("MSF_Messenger")->addError(MSF_Registry::get("MSF_Locale")->getWord("LMS_ERROR_FOLDER_NOT_EXISTS"));
			}
	
			$sInitPath = "/lms/exportRecordset/folder/" . $iFolderId;
			$this->Template->getModule ()->appendChild ( $this->Template->createElement ( "initPath", $sInitPath ) );
			$this->setView($this->ListXml);
		}
	}
	
	private function ajaxRequest($iFolderId)
	{
		$List = new LMS_MailList();
		
		$View = new MSF_RecordsetXml();
		$View->buildGroup($List->group($iFolderId));
		$this->setView($View);
	}
	
	public function autocomplete($iFolderId)
	{
		$List = new LMS_MailList();
		MSF_AutoCompletor::autoCompleteResponse($List->open($iFolderId, -1, false, false));
	}
	
	private function editFolder()
	{
		$View = new LMS_FolderXml();
		
		if (IS_REQUEST) {
			$this->Folders->save($this->getIntParam(1));
			$this->redirect("/lms/folders", $this->locale("LMS_MESSAGE_FOLDER_SAVED"));
			exit();
		}
		
		$View->view($this->Folders->getOne($this->getIntParam(1)));
		$this->setView($View);
	}

	private function viewFolders()
	{
		$List = new LMS_FoldersList();
		$View = new LMS_FoldersListXml();
		$View->viewFolder($List->get());
		$sInitPath = "/lms/exportRecordset/folders";
		$this->Template->getModule ()->appendChild ( $this->Template->createElement ( "initPath", $sInitPath ) );
		$this->setView($View);
	}
	
	private function moveTo($mIds, $iFromFolderId, $iToFolder)
	{
		if ($iToFolder == 0 || $iToFolder > 4) {
			$this->Mail->setFolderId($iFromFolderId);
			$this->Mail->move($mIds, $iToFolder);
		}
	}
	
	private function restoreTo($mIds, $iFolderId)
	{
		if ($iFolderId == LMS_Mail::TRASH) {
			$this->Mail->restore($mIds);
		}
	}
	
	private function removeFolder()
	{
		if (IS_REQUEST) {
			$this->Folders->remove(MSF_SL::getParam($_POST, "folder_id", array()));
		}
		else {
			$this->Folders->remove($this->getIntParam(1));
		}
		
		$this->redirect("/lms/folders", $this->locale("LMS_MESSAGE_FOLDERS_DELETED"));
	}
	
	private function addFolders($aFolders, $aCount, $iFolderId)
	{
		$iIndex 	= 0;
		$aResult 	= array();
		
		$aResult[$iIndex]["folder_id"] 		= 0;
		$aResult[$iIndex]["title"] 			= $this->locale("LMS_TITLE_INBOX");
		$aResult[$iIndex]["moveto"] 		= "yes";
		
		if (!empty($aCount) && $aCount["inbox_total"] > 0) {
			$aResult[$iIndex]["new_total"] = $aCount["inbox_total"];
		}
		
		$aResult[++$iIndex]["folder_id"] 	= 1;
		$aResult[$iIndex]["title"] 			= $this->locale("LMS_TITLE_OUTBOX");
		$aResult[$iIndex]["moveto"] 		= "no";
		
		if (!empty($aCount) && $aCount["outbox_total"] > 0) {
			$aResult[$iIndex]["new_total"] = $aCount["outbox_total"];
		}
		
		$aResult[++$iIndex]["folder_id"] 	= 2;
		$aResult[$iIndex]["title"] 			= $this->locale("LMS_TITLE_SENT");
		$aResult[$iIndex]["moveto"] 		= "no";
		$aResult[++$iIndex]["folder_id"] 	= 3;
		$aResult[$iIndex]["title"] 			= $this->locale("LMS_TITLE_DRAFT");
		$aResult[$iIndex]["moveto"] 		= "no";
		
		if (!empty($aCount) && $aCount["draft_total"] > 0) {
			$aResult[$iIndex]["new_total"] = $aCount["draft_total"];
		}
		$aResult[++$iIndex]["folder_id"] 	= 4;
		$aResult[$iIndex]["title"] 			= $this->locale("LMS_TITLE_TRASH");
		$aResult[$iIndex]["moveto"] 		= "no";
		
		for ($i = 0; $i < sizeof($aFolders); $i++) {
			$aResult[++$iIndex] 		= $aFolders[$i];
			$aResult[$iIndex]["moveto"] = "yes";
			if ($aResult[$iIndex]["new_total"] == 0) {
				unset($aResult[$iIndex]["new_total"]);
			}
		}
		
		for ($i = 0; $i < sizeof($aResult); $i++) {
			if ($aResult[$i]["folder_id"] == $iFolderId) {
				$aResult[$i]["selected"] = "yes";
			}
			else {
				$aResult[$i]["selected"] = "no";
			}
		}
		
		return $aResult;
	}
	
	private function addFolderTitle($iFolderId, $sExtra = "")
	{
		$sTitle = $this->Locale->getWord("LMS_MODULE_NAME") . $this->Locale->getWord("ROOT_TITLE_SEPARATOR");
		
		if (empty($sExtra)) {
			for ($i = 0; $i < count($this->aFolders); $i++) {
				if ($iFolderId == $this->aFolders[$i]["folder_id"]) {
					$sTitle .= $this->aFolders[$i]["title"];
					break;
				}
			}
		}
		else {
			$sTitle .= $sExtra;
		}
		
		$this->Template->setPageTitle($sTitle);
	}
	
	private $Addr;
	
	public function addressesAction()
	{
		$sTitle = 	$this->Locale->getWord("LMS_MODULE_NAME") .
					$this->Locale->getWord("ROOT_TITLE_SEPARATOR") .
					$this->Locale->getWord("LMS_TITLE_ADDRESS_BOOK");
		$this->Template->setPageTitle($sTitle);
		
		$this->Addr = new LMS_Addresses();
		
		switch ($this->getParam(0)) {
			case "add":
			case "edit":
				$this->editAddr();
				break;
			case "remove":
				$this->removeAddr();
				break;
			case "load":
				$this->ajaxReq();
				break;
			default:
				$this->viewAdrs();
				break;
		}
	}
	
	public function exportRecordsetAction() {
		$sType		= $this->getParam(0);
		if($sType == "folder") {
			$iFolderId 	= $this->getIntParam ( 1 );		
			$oList = new LMS_MailList ( );
			$aRecordset = $oList->open ( $iFolderId, 0, true );
			$iPages = $aRecordset ["pagination"] ["totalPages"];
			$oPHPExcel = new MSF_ExportToExcel ( );
			$oPHPExcel->exportTitles ( $aRecordset );
			for($i = 0; $i < $iPages; $i ++) {
				set_time_limit(MSF_Recordset::$RECORDS_PER_PAGE);
				$aRecordset = $oList->open ( $iFolderId, $i * MSF_Recordset::$RECORDS_PER_PAGE, true );
				$oPHPExcel->exportBody ( $aRecordset );
			}
			
			$sFileTitle	= $this->Locale->getWord("LMS_TITLE_MAIN");
		}
		else if($sType == "addresses") {
			$oAddresses	= new LMS_Addresses();
			$aRecordset	= $oAddresses->getList(0, true);
			$iPages = $aRecordset ["pagination"] ["totalPages"];
			$oPHPExcel = new MSF_ExportToExcel ( );
			$oPHPExcel->exportTitles ( $aRecordset );
			for($i = 0; $i < $iPages; $i ++) {
				set_time_limit(MSF_Recordset::$RECORDS_PER_PAGE);
				$aRecordset = $oAddresses->getList ( $i * MSF_Recordset::$RECORDS_PER_PAGE, true );
				$oPHPExcel->exportBody ( $aRecordset );
			}
			
			$sFileTitle	= $this->Locale->getWord("LMS_TITLE_ADDRESS_BOOK");
		}
		else if($sType == "folders") {
			$oList 		= new LMS_FoldersList();
			$aRecordset	= $oList->get(0, true);
			$iPages = $aRecordset ["pagination"] ["totalPages"];
			$oPHPExcel = new MSF_ExportToExcel ( );
			$oPHPExcel->exportTitles ( $aRecordset );
			for($i = 0; $i < $iPages; $i ++) {
				set_time_limit(MSF_Recordset::$RECORDS_PER_PAGE);
				$aRecordset = $oList->get ( $i * MSF_Recordset::$RECORDS_PER_PAGE, true );
				$oPHPExcel->exportBody ( $aRecordset );
			}

			$sFileTitle	= $this->Locale->getWord("LMS_TITLE_MY_FOLDERS");
		}
		
		$sLink = $oPHPExcel->saveSpreadsheet ();
		$aResult = array(
			"link"	=> $sLink,
			"ext"	=> "xls",
			"name"	=> urlencode($sFileTitle)
			);
			
		$View = new MSF_ViewSupportJson();
		$View->set("data", $aResult);
		$this->setView($View);	
	}	
	
	private function ajaxReq()
	{
		$View = new LMS_AddressesXml();
		$View->buildGroup($this->Addr->getGroup());
		$this->setView($View);
	}
	
	private function removeAddr()
	{
		if (IS_REQUEST) {
			$this->Addr->remove((array) MSF_SL::getParam($_POST, "id"));
		}
		else {
			$this->Addr->remove($this->getIntParam(1));
		}
		$this->redirect("/lms/addresses", $this->locale("LMS_LINK_ADDRESS_BOOK_DELETED"));
	}
	
	private function editAddr()
	{
		if (IS_REQUEST) {
			$iResult = $this->Addr->save($this->getIntParam(1), $_POST);
			if ($iResult == 1) {
				$this->redirect("/lms/addresses", $this->locale("LMS_LINK_ADDRESS_BOOK_ADDED"));
			}
			elseif ($iResult == 2) {
				$this->redirect("/lms/addresses", $this->locale("LMS_LINK_ADDRESS_BOOK_CHANGED"));
			}
			else {
				$this->redirect("/lms/addresses", $this->locale("LMS_LINK_ADDRESS_BOOK_NOT_ADDED"));
			}
		}
		
		$View = new LMS_AddressesXml();
		$this->Addr->showRef($this->getIntParam(1));
		$this->addAction($this->locale("LMS_LINK_CLOSE"), "/lms/addresses");
		//$View->edit($this->Addr->getRefParams($this->getIntParam(1)));
		$this->setView($View);
	}
	
	private function viewAdrs()
	{
		$View = new LMS_AddressesXml();
		$View->view($this->Addr->getList());
		$sInitPath = "/lms/exportRecordset/addresses";
		$this->Template->getModule ()->appendChild ( $this->Template->createElement ( "initPath", $sInitPath ) );		
		$this->setView($View);
	}
}

?>