<?php

defined("__MSF") or die("Direct access is not allowed");

include_once "modules/references/library/Reference/UserReferenceDB.php";
include_once "modules/references/library/Reference/UserReference.php";
include_once "modules/references/library/Reference/REF_Fields.php";
include_once "modules/references/library/Reference/REF_Template.php";

class LMS_Addresses
{
	private $Db;
	
	/**
	 * @var UserReference
	 */
	private $Ref;
	
	private $Locale;
	
	private $Ubox;
	
	private $Template;
	
	public function __construct()
	{
		$this->Db 		= MSF_Registry::get("MSF_DB");
		$this->Locale 	= MSF_Registry::get("MSF_Locale");
		$this->Ubox 	= MSF_Registry::get("MSF_UserBox");
		$this->Template = MSF_Registry::get("MSF_Template");
		$this->Ref 		= new UserReference(new UserReferenceDB($this->Db));
		$this->Ref->load(6);
	}
	
	public function exists($sEmail)
	{
		$aRef 		= $this->Ref->getAsArray();
		$bExists 	= false;
		
		foreach ($aRef as $aRow) {
			if ($aRow["user_name"] == $this->getUserValue()) {
				if (substr_count($aRow["contact_email"], $sEmail) > 0) {
					$bExists = true;
					break;
				}
			}
		}
		
		if (!$bExists) {
			$Ref = new UserReference(new UserReferenceDB($this->Db));
			$Ref->load(1);
			$aRef = $Ref->getAsArray();
			
			foreach ($aRef as $aRow) {
				if (substr_count($aRow["email"], $sEmail) > 0) {
					$bExists = true;
					break;
				}
			}
		}
		
		
		return $bExists;
	}
	
	public function save($iId, $aParams)
	{
		$iSave = 1;
		foreach ($this->Ref->row($iId)->show() as $aItem) {
			if ($aItem["name"] == "user_name" && !empty($aItem["value"])) {
				if ($aItem["value"] != $this->getUserValue()) {
					$iSave = 0;
					break;
				}
				else {
					$iSave = 2;
					break;
				}
			}
		}
		
		if ($iSave != 0) {
			foreach ($aParams as $sKey => $sValue) {
				$aItem[$sKey] = trim(trim(trim(trim(trim($sValue), ">"), "<"), "\""), "&quot;");
			}
		
			/*
			foreach ($aParams as $sKey => $sValue) {
				$aParams[$sKey] = MSF_SL::getParam($aParams, $sKey);
			}
			*/
			$aParams["user_name"] 	= $this->getUserValue();
			$aParams["info"] 		= MSF_SL::getParam($_POST, "info");
			
			
			$this->Ref->row($iId)->save($aParams);
		}
		
		return $iSave;
	}
	
	public function remove($mIds)
	{
		if (is_array($mIds)) {
			foreach ($mIds as $iId) {
				$this->Ref->row($iId)->remove();
			}
		}
		else {
			$this->Ref->row($mIds)->remove();
		}
	}
	
	public function getRefParams($iId)
	{
		$aTitles = array();
		$bValues = true;
		
		foreach ($this->Ref->row($iId)->show() as $aColumn) {
			if ($aColumn["name"] == "user_name" && $aColumn["value"] != $this->getUserValue()) {
				$bValues = false;
			}
			if ($aColumn["name"] == "id" || ($aColumn["name"] == "user_name")) {
				continue;
			}
			$aTmp 			= array();
			$aTmp["name"] 	= $aColumn["name"];
			$aTmp["title"] 	= $aColumn["title"];
			$aTmp["type"] 	= $aColumn["type"];
			$aTmp["value"] 	= ($bValues) ? $aColumn["value"] : "";
			$aTitles[] 		= $aTmp;
		}
		
		return $aTitles;
	}
	
	private function getUserValue()
	{
		$RefValue = new REF_Value();
		$RefValue->add($this->Ubox->getUserAttribute("id"), $this->Ubox->getUserAttribute("login"));
		return $RefValue->format();
	}
	
	public function getList($iLimitFrom = -1, $bExport = false)
	{
		$Recordset = $this->prepareRecs($iLimitFrom, $bExport);
 		return $Recordset->getAllRecords();
	}
	
	public function getGroup()
	{
		$Recordset = $this->prepareRecs();
		return $Recordset->getGroup();
	}
	
	private function prepareRecs($iLimitFrom = -1, $bExport = false)
	{
		$aColumns 	= $this->Ref->getColumns();
		$aToRS 		= array();
		foreach($aColumns as $aColumn) {
			$bGroup = true;
			if($aColumn["name"] == "id") {
				$aColumn["visible"] = 0;
			}
			if ($aColumn["name"] == "info") {
				$bGroup = false;
			}
			
			$aTmp = array(
				"name" 		=> $this->Ref->getTable() . "." . $aColumn["name"],
				"title" 	=> $aColumn["title"],
				"type" 		=> $aColumn["type"],
				"sort" 		=> true,
				"filter" 	=> true,
				"group" 	=> $bGroup,
				"visible" 	=> $aColumn["visible"],
				"compare" 	=> "like"
			);
			
			if(isset($aColumn["optional"])) {
				$aTmp["optional"] = $aColumn["optional"];
			}
			
			array_push($aToRS, $aTmp);
		}
		
		
		$Recordset = new MSF_Recordset(MSF_Registry::get("MSF_DB"));
		$Recordset->setExport($bExport);		
		if ($iLimitFrom != - 1) {
			$Recordset->setParam ("limitFrom", $iLimitFrom);
		}
		
		$Recordset->setRpcUrl("/lms/addresses/load/?");
		$Recordset->setBaseURL("/lms/addresses/?");
		$Recordset->setColumns($aToRS);
		$Recordset->setTables(array($this->Ref->getTable()));
		$Recordset->setConditions(array(
			"AND user_name = '" . $this->getUserValue() . "'"
		));
		$Recordset->setLink(
			array(
				"href"		=> "/lms/addresses/edit",
				"bind"		=> array("id")
			)
		);
		
		$aActions = array(
			array (
				"title"		=> $this->Locale->getWord("LMS_BTN_EDIT"),
				"href"		=> "/lms/addresses/edit",
				"image"		=> "/templates/default/images/pencil.png",
				"bind"		=> array("id"),
				"confirm"	=> false
			),
			array (
				"title"		=> $this->Locale->getWord("LMS_BTN_DELETE"),
				"href"		=> "/lms/addresses/remove",
				"image"		=> "/templates/default/images/red_cross.png",
				"bind"		=> array("id"),
				"confirm"	=> true
			),
			
		);
		
		$aMultiActions	= array(
			array(
	 			"title"		=> $this->Locale->getWord("LMS_BTN_DELETE"),
				"href"		=> "/lms/addresses/remove",
				"image"		=> "/templates/default/images/red_cross.png",
				"confirm"	=> true
	 		)
		);
		
		$Recordset->setStateParams(array(
			"module"		=> "lms_address_book",
			"action"		=> "show",
			"action_id"		=> 1,
			"structure_id"	=> MSF_Registry::get("MSF_UserBox")->getCurrentStructureId()
		));
		
		$Recordset->setActions($aActions);
		$Recordset->setMultiActions($aMultiActions);
 		$Recordset->setMultiActionField("id");
 		
 		return $Recordset;
	}
	
	public function showRef($iId)
	{
		$Fields 		= new REF_Fields($this->Ref->getFieldsXml());
		$REF_Template 	= new REF_Template(str_replace("<p>&#160;</p>", "",$this->Ref->getTemplate()));

		$Fields->setExtra("references", 6);
		$REF_Template->setExtra("references", 6);
		$REF_Template->setFields($Fields);
		
		$aRow = $this->Ref->row($iId)->show();
		foreach ($aRow as $aCol) {
			$Fields->set($aCol["name"], $aCol["value"]);
		}
		
		$this->Template->addLinksOf("modules/references/view/links.ini");
		$this->Template->getModule()->appendChild($this->Template->createElement("document"));
		$REF_Template->addEditTemplate($this->Template, "module/template");
	}
}

?>