<?php

defined("__MSF") or die("Direct access is not allowed");

include_once "library/MSF_SafeText/MSF_SafeText.php";
include_once "modules/references/library/Reference/UserReferenceDB.php";
include_once "modules/references/library/Reference/UserReference.php";
include_once "modules/media/library/MD_MediaDB.php";
include_once "modules/media/library/MD_Media.php";
include_once "modules/lms/library/LMS_MailDecode.php";
include_once "modules/lms/library/LMS_Addresses.php";

class LMS_Mail
{
	const  	INBOX 		= 0;
	
	const  	OUTBOX 		= 1;
	
	const 	SENT 		= 2;
	
	const 	DRAFT 		= 3;
	
	const 	TRASH 		= 4;
	
	private $DB 		= null;
	
	private $Media 		= null;
	
	private $Reference 	= null;
	
	private $UserBox 	= null;
	
	private $aParams 	= array();
	
	private $aFilesIds 	= array();
	
	private $aFileNames = array();
	
	private $aTmpFiles 	= array();
	
	private $aAttached 	= array();
	
	private $iFolderId 	= 0;
	
	private $iUserId 	= 0;
	
	private $sNewLine 	= "<br/>";
	
	private $bFCheck 	= false;
	
	private $Addrs 		= null;

	
	public function __construct(LMS_MailDB $DB)
	{
		$this->DB 			= $DB;
		$this->Media 		= new MD_Media(new MD_MediaDB(MSF_Registry::get("MSF_DB")));
		$this->UserBox 		= MSF_Registry::get("MSF_UserBox");
		$this->iUserId 		= (int) $this->UserBox->getUserAttribute("id");
		$this->Reference 	= new UserReference(new UserReferenceDB(MSF_Registry::get("MSF_DB")));
		$this->Addrs 		= new LMS_Addresses();
		$this->init();
		$this->Media->setMaxFileSize(100000000);
	}
	
	private function init()
	{
		$this->aParams["identifier"] 	= "MSF-" . MSF_String::random(20) . '-MSF';
		$this->aParams["to"] 			= array();
		$this->aParams["cc"] 			= array();
		$this->aParams["bcc"] 			= array();
		$this->aParams["time"] 			= date("r");
		$this->aParams["encode"] 		= "UTF-8";
		$this->aParams["type"] 			= "text/html";
		$this->aParams["subject"] 		= "";
		$this->aParams["body"] 			= "";
		$this->aParams["user_id"] 		= $this->iUserId;
		$this->aParams["from_name"] 	= $this->UserBox->getEmployeeAttribute("name");
		$this->aParams["from_email"] 	= $this->getUserEmail();
		
		$this->aAttached = MSF_SL::getParam($_POST, "lms_attached_files", array());
	}
	
	public function mark($aIds)
	{
		if (count($aIds) > 0) {
			$this->DB->updateMark(implode(",", $aIds), $this->iFolderId, "new");
		}
	}
	
	public function unmark($aIds)
	{
		if (count($aIds) > 0) {
			$this->DB->updateMark(implode(",", $aIds), $this->iFolderId, "viewed");
		}
	}
	
	public function setFrom($sEmail, $sName = "", $iUserId = 0)
	{
		if ($sEmail) {
			$this->aParams["from_email"] 	= (string) $sEmail;
		}
		$this->aParams["from_name"] 	= (string) $sName;
		if ($iUserId) {
			$this->iUserId = $iUserId;
			$this->aParams["user_id"]	 = $this->iUserId;
			$this->aParams["from_email"] = $this->getUserEmail();
		}
		else {
			$this->iUserId = $this->getSystemUserId();
			$this->aParams["user_id"]	 = $this->iUserId;
			$this->aParams["from_email"] = $this->getSystemEmail();
		}
		//$this->DB->isDirect(true);
	}
	
	public function exists($iMailId)
	{
		return ($this->DB->exists($iMailId, $this->iFolderId) == 1) ? true : false;
	}
	
	public function setAddresses($sAddresses, $bIsLms = false, $sKey = "to")
	{
		if ($bIsLms) {
			$this->aParams[$sKey] 	= $this->parseAddresses((string) $sAddresses);
		}
		else {
			$this->aParams["to"] 	= $this->parseAddresses((string) $sAddresses, "|^|row|^|");
		}
	}
	
	public function setStructureIds($aIds)
	{
		$aUserIds 	= array();
		$aAddresses = array();
		
		for ($i = 0; $i < count($aIds); $i++) {
			$this->UserBox->load($aIds[$i], "structure", true);
			$aTmp = $this->UserBox->getLoadedResult();
			
			for ($j = 0; $j < count($aTmp); $j++) {
				//if (isset($aTmp[$j]["user"]["email"])) {
					if (!in_array($aTmp[$j]["user"]["id"], $aUserIds)) {
						$aCopy 				= explode("|^|row|^|", $aTmp[$j]["user"]["email_send_copy"]);
						$aUserIds[] 		= $aTmp[$j]["user"]["id"];
						$aObj["structure"] 	= true;
						$aObj["id"] 		= $aTmp[$j]["user"]["id"];
						$aObj["email"] 		= $aTmp[$j]["user"]["email"];
						$aObj["name"] 		= $aTmp[$j]["employee"]["name"];
						$aObj["to_ext"] 	= (isset($aCopy[1]) && $aCopy[1] == 1) ? true : false;
						$aObj["is_smtp"] 	= (!empty($aTmp[$j]["user"]["email_smtp_address"]) || $this->isSystemSmtp()) ? true : false;
						if (empty($aTmp[$j]["user"]["email_smtp_address"]) && $this->isSystemSmtp()) {
							if (empty($this->aParams["from_email"]) || !$this->isSmtp()) {
								$this->aParams["from_email"] = $this->getSystemEmail();
							}
						}
						$aAddresses[] 		= $aObj;
					}
				//}
			}
		}
		
		$this->aParams["user_id"] 	= $aUserIds;
		$this->aParams["to"] 		= $aAddresses;
	}
	
	public function setTo($iUserId, $sEmail)
	{
		$aTmp["id"] = $iUserId;
		$aTmp["name"] = "";
		$aTmp["structure"] = false;
		$aTmp["email"] = $sEmail;
		$aTmp["to_ext"] = true;
		$aTmp["is_smtp"] = true;
		$this->aParams["to"] = array($aTmp);
	}
	
	public function setSubject($sSubject)
	{
		$this->aParams["subject"] = (string) $sSubject;
	}
	
	public function setBody($sBody)
	{
		$this->aParams["body"] = (string) $sBody;
	}
	
	public function setType($sType)
	{
		switch ($sType) {
			case "text/plain":
			case "text/html":
				$this->aParams["type"] = $sType;
				break;
		}
	}
	
	public function setFolderId($iFolderId)
	{
		$this->iFolderId = (int) $iFolderId;
	}

	public function getFilesIds()
	{
		return $this->aFilesIds;
	}
	
	public function getFileNames()
	{
		return $this->aFileNames;
	}
	
	public function save($iMailId = 0, $bAddToSent = false, $bSaveToDrafts = false, $iForwId = 0)
	{
		if (empty($this->aParams["subject"])) {
			$this->aParams["subject"] = MSF_HtmlEntities::strip(mb_substr(
				strip_tags(str_replace(array("<br>", "<br/>", "<br />"), " ", $this->aParams["body"])),
				0, 75, "UTF8"
			)) . "...";
		}
		
		$Tidy = new tidy();
		$Tidy->parseString($this->aParams["body"], array(
			'output-xhtml' => true,
			'show-body-only' => true,
			"word-2000" => true
		), "UTF8");
		$Tidy->cleanRepair();
		$this->aParams["body"] = isset($Tidy->value) ? $Tidy->value : "";
		
		$iFilesSize = 0;
		$aFiles 	= MSF_SL::getParam($_FILES, "lms_files", array());
		
		if (!empty($aFiles["name"][0])) {
			$this->aParams["attachments"] = "Y";
			
			for ($i = 0; $i < count($aFiles["size"]); $i++) {
				$iFilesSize += $aFiles["size"][$i];
			}
		}
		
		if ($iForwId > 0) {
			if ($this->exists($iForwId) && !empty($this->aAttached)) {
				for ($i = 0; $i < count($this->aAttached); $i++) {
					$aTmp = $this->Media->getFileInfo($this->aAttached[$i]);
					$this->aFilesIds[] = $aTmp["file_id"];
					$iFilesSize += $aTmp["size"];
					$this->aParams["attachments"] = "Y";
				}
			}
		}
		
		$sToEmailsString 			= $this->toHeaders($this->aParams["to"]);
		$this->aParams["headers"] 	= $this->formHeaders();
		$this->aParams["size"] 		= $this->getSize($iFilesSize);
		$bEmail 					= ($this->isSmtp() || $this->isSystemSmtp());
		
		if ($this->exists($iMailId) || $bSaveToDrafts) {
			$sTo 						= MSF_SL::getParam($_POST, "addresses");
			$this->aParams["to"] 		= empty($sTo) ? MSF_Registry::get("MSF_Locale")->getWord("LMS_LABEL_EMAIL_NOT_GIVEN") : $sTo;
			$this->aParams["headers"] 	= $this->formHeaders($sTo, trim(MSF_SL::getParam($_POST, "cc")), trim(MSF_SL::getParam($_POST, "bcc")));
			
			if ($this->iFolderId == self::DRAFT && $this->exists($iMailId)) {
				$this->DB->update($iMailId, $this->aParams);
			}
			else {
				$iEmailId 	= $this->DB->insert($this->aParams, self::DRAFT);
				$iMailId 	= $this->DB->getNewMessageId();
				$this->attachFiles($aFiles, $iEmailId, self::DRAFT);
			}
		}
		else {
			$this->Reference->load(1);
			$aRef 		= $this->Reference->getAsArray();
			$aEmails 	= $this->getEmailsFromRef($aRef);
			$aUnique 	= $this->getUniqueAddresses();
			$this->aParams["to"] = $sToEmailsString;
			
			foreach ($aUnique as $aItem) {
				if ($this->checkEmail($aItem["email"])) {
					if (!$this->Addrs->exists($aItem["email"])) {
						$aTmp = array();
						$aTmp["group_name"] 	= MSF_Registry::get("MSF_Locale")->getWord("LMS_ADDRESS_DEFAULT_GROUP");
						$aTmp["contact_name"] 	= $aItem["name"];
						$aTmp["contact_email"] 	= $aItem["email"];
						$this->Addrs->save(0, $aTmp);
					}
				}
			}
			
			$bNewOutbox		= false;
			
			if ($bAddToSent) {
				$bExternal 	= false;
				$bInternal 	= false;
				$bToExt 	= false;
				
				$aTmp = array();
				if (is_array($this->aParams["user_id"])) {
					$aTmp = $this->aParams["user_id"];
					$this->aParams["user_id"] = $this->iUserId;
				}
				
				for ($i = 0; $i < count($aUnique); $i++) {
					if (!empty($aUnique[$i]["email"]) && !in_array($aUnique[$i]["email"], $aEmails)) {
						$bExternal = true;
					}
					else {
						$bInternal = true;
					}
					if ($this->checkEmail($aUnique[$i]["email"]) && $aUnique[$i]["to_ext"] == 1) {
						$bToExt = true;
					}
				}
				if (!$this->isSmtp() && $this->isSystemSmtp()) {
					$this->aParams["from_email"] = $this->getSystemEmail();
				}
				if ((($bEmail && ($bExternal || $bToExt)) || (!$bEmail && $bExternal)) && !empty($this->aParams["from_email"])) {
					if (strlen(trim($this->aParams['to']))) {
						$this->aParams["blocked"] = 1;
						$iEmailId 	= $this->DB->insert($this->aParams, self::OUTBOX);
						$this->attachFiles($aFiles, $iEmailId, self::OUTBOX);
						$bNewOutbox = true;
						if ($bEmail) {
							$this->DB->unBlock($iEmailId);
						}
						
						unset($this->aParams["blocked"]);
					}

				}
				
				if ($bInternal && ($this->iUserId != $this->getSystemUserId())) {
					$iEmailId = $this->DB->insert($this->aParams, self::SENT);
					$this->attachFiles($aFiles, $iEmailId, self::SENT);
				}
				
				if (!empty($aTmp)) {
					$this->aParams["user_id"] = $aTmp;
				}
			}
			
			
			for ($i = 0; $i < count($aUnique); $i++) {
				if ($aUnique[$i]["id"] == 0) {
					continue;
				}
				$this->aParams["external"] 	= "N";
				$this->aParams["user_id"] 	= $aUnique[$i]["id"];
				
				if (!$bNewOutbox && $aUnique[$i]["to_ext"] && $aUnique[$i]["is_smtp"]) {
					$aParams 				= $this->aParams;
					
					$aParams["headers"] 	= $this->formHeaders($aUnique[$i]["email"], $aUnique[$i]["email"]);
					$aParams["external"] 	= ($aUnique[$i]["structure"]) ? "N" : "Y";
					$aParams["from_email"] 	= $aUnique[$i]["email"];
					$iEmailId = $this->DB->insert($aParams, self::OUTBOX);
					$this->attachFiles($aFiles, $iEmailId, self::OUTBOX);
				}
				
				$iEmailId = $this->DB->insert($this->aParams, self::INBOX);
				$this->attachFiles($aFiles, $iEmailId, self::INBOX);
			}
		}
		
		if (!empty($this->aTmpFiles)) {
			for ($i = 0; $i < count($this->aTmpFiles); $i++) {
				if (MSF_File::exists($this->aTmpFiles[$i])) {
					MSF_File::remove($this->aTmpFiles[$i]);
				}
			}
		}
		
		
		return $iMailId;
	}
	
	private function isSmtp()
	{
		$bResult = false;

		if ($this->checkEmail($this->getUserEmail())) {
				$sAddr 	= $this->getUserSmtpAddress();				
				if (!empty($sAddr)) {
					$bResult = true;
				}	
			}
		
		return $bResult;
	}
	
	public function isSystemSmtp()
	{
		$bResult = false;
		$aSettings 	= MSF_Registry::get("MSF_Settings")->getModule("lms");
		if (!empty($aSettings["settings"]["system_email_smtp_server"])) {
			$bResult = true;
		}
		
		return $bResult;
	}
	
	private function getSystemEmail()
	{
		$bResult = "";
		$aSettings 	= MSF_Registry::get("MSF_Settings")->getModule("lms");
		if (!empty($aSettings["settings"]["system_email"])) {
			$bResult = $aSettings["settings"]["system_email"];
		}
		
		return $bResult;
	}
	
	public function formReplyAttributes(&$aParams, $sAction)
	{
		$sBody = $aParams["body"];
		//$aBody = explode("\n", $sBody);
		//$sBody = "";

		if ($sAction == "reply") {
			$reply = "Re";
		}
		else {
			$reply = "Fw";
		}

		$sSubject 		= $aParams["subject"];
		$iReCount 		= substr_count($sSubject, $reply . ":");
		$iReReCount 	= substr_count($sSubject, $reply . "[");

		if ($iReCount > 0) {
			$sSubject = str_replace($reply . ":", $reply . "[2]:", $sSubject);
		}
		else if($iReReCount > 0) {
			$iReply 	= (int) $sSubject{3};
			$sSubject 	= str_replace($reply . "[" . $iReply . "]:", $reply . "[" . ($iReply + 1) . "]:", $sSubject);
		}
		else {
			$sSubject = $reply . ": " . $sSubject;
		}
		
		$sMessage 	= $this->sNewLine
					. $this->sNewLine
					. $this->sNewLine
					. "-----Original Message-----" . $this->sNewLine
					. "From: " . $aParams["from"] . $this->sNewLine
					. "To: " . $aParams["to_email"] . $this->sNewLine
					. "Date: " . $aParams["time"] . $this->sNewLine
					. "Subject: " . $aParams["subject"] . $this->sNewLine
					. $this->sNewLine
					. ">" . $this->sNewLine
					. ((!empty($sBody)) ? $sBody : ">") . $this->sNewLine
					. ">" . $this->sNewLine;
		
		$aParams["subject"] 	= $sSubject;
		$aParams["body"]		= $sMessage;
	}
	
	public function open($iMailId)
	{
		$aResult = array();
		
		if ($this->exists($iMailId)) {
			$aResult = $this->prepareOpen($this->DB->open($iMailId, $this->iFolderId));
		}
		
		return $aResult;
	}
	
	public function openLast()
	{
		return $this->prepareOpen($this->DB->selectLast($this->iFolderId));
	}
	
	public function remove($mIds, $bForce = false)
	{
		if ($bForce) {
			$aEmailIds = $this->DB->forceDelete($mIds, $this->iFolderId);
		}
		else {
			$aEmailIds = $this->DB->delete($mIds, $this->iFolderId);
		}
		
		for ($i = 0; $i < count($aEmailIds); $i++) {
			$aFiles 	= $this->Media->getFiles("lms_" . $this->iFolderId, $aEmailIds[$i]);
			$aFileIds 	= array();
			
			for ($j = 0; $j < count($aFiles); $j++) {
				try {
					if ($this->iFolderId == self::TRASH) {
						$this->Media->remove($aFiles[$j]["file_id"]);
					}
					else {
						$aFileIds[] = $aFiles[$j]["file_id"];
					}
				}
				catch (Exception $e) {
					
				}
			}
			
			if (!empty($aFileIds)) {
				$this->Media->saveModuleName($aFileIds, "lms_" . self::TRASH);
			}
		}
	}
	
	public function move($mId, $iToFolderId)
	{
		$this->updateFiles($mId, $iToFolderId, $this->iFolderId);
		$this->DB->move($mId, $this->iFolderId, $iToFolderId);
	}

	public function restore($mId)
	{
		$aFolderIds = $this->DB->restore($mId);
		
		for ($i = 0; $i < count($aFolderIds["folders"]); $i++) {
			$this->updateFiles("", $aFolderIds["folders"][$i], $this->iFolderId, $aFolderIds["emails"]);
		}
	}
	
	
	private function updateFiles($mId, $iToFolderId, $iFolderId, $aEmailIds = array())
	{
		if (!is_array($mId)) { 
			$aMId = explode(",", $mId);
		}
		else {
			$aMId = $mId;
		}
		if (!empty($aMId)) {
			if (empty($aEmailIds)) {
				$aEmailIds = $this->DB->getEmailId($this->iFolderId, $aMId);
			}
			if (!empty($aEmailIds)) {
				$aFiles = array();
				if (is_array($aMId)) {
					for ($i = 0; $i < count($aEmailIds); $i++) {
						$aFiles = array_merge($aFiles, $this->Media->getFiles("lms_" . $iFolderId, $aEmailIds[$i]["email_id"]));
					}
				}
				else {
					$aFiles = array_merge($aFiles, $this->Media->getFiles("lms_" . $iFolderId, $aEmailIds["email_id"]));
				}
				
				for ($i = 0; $i < count($aFiles); $i++) {
					$this->Media->saveModuleName($aFiles[$i]["file_id"], "lms_" . $iToFolderId);
				}
			}
		}
	}
	
	
	private function checkEmail($sEmail)
	{
		return preg_match("/^[-_a-zA-Z0-9\.]{1,}@[-_a-zA-Z0-9\.]{2,}\.[a-zA-Z]{2,}$/", $sEmail);
	}
	
	public function parseAddresses($sAddresses, $sDelimeter = ",")
	{
		$aResult = array();
		if (!empty($sAddresses)) {
			$aParsed 	= explode($sDelimeter, $sAddresses);
			$aEmps 		= array();
			
			$this->Reference->load(4);
			$aSubd 		= $this->Reference->getAsArray();
			$aSAddrs 	= array();

			foreach ($aParsed AS $Key => $Value) {
				for ($j = 0; $j < count($aSubd); $j++) {
					if (trim($Value) == trim($aSubd[$j]["subdivision"])) {
						
						$aStructue = $this->UserBox->load($aSubd[$j]["id"], MSF_UserBox::SUBDIVISION);
						for ($k = 0; $k < count($aStructue); $k++) {
							if (isset($aStructue[$k]["employee"]["name"])) {
								$aSAddrs[] = $aStructue[$k]["employee"]["name"];
							}
						}
						unset($aParsed[$Key]);
						break;
					}
				}
			}
			
			foreach ($aParsed as $sValue) {
				$sValue = trim($sValue);
				if (empty($sValue)) {
					continue;
				}
				$aSAddrs[] = $sValue;
			}
			
			$aParsed = array_unique($aSAddrs);
			
			$this->Reference->load(2);
			$aRef = $this->Reference->getAsArray();
			$iRef = count($aRef);
			$this->Reference->load(1);
			$aUsrs = $this->Reference->getAsArray();
			$iUsrs = count($aUsrs);
			
			foreach ($aParsed AS $sAddr) {
				$bIsEmp = false;
				foreach ($aRef AS $aItem) {
					if ($aItem["name"] == $sAddr) {
						$bIsEmp = true;
					}
				}
				
				if ($bIsEmp) {
					$aEmps[] = $sAddr;
				}
				elseif (strstr($sAddr, "@")) {
					$aTmp = explode("<", $sAddr);
					
					if (count($aTmp) > 1) {
						$sEmail 			= str_replace(">", "", trim($aTmp[1]));
						$aObj["id"] 		= 0;
						$aObj["email"] 		= $sEmail;
						$aObj["name"] 		= stripslashes(trim($aTmp[0]));
						$aObj["to_ext"] 	= true;
						$aObj["is_smtp"] 	= $this->isSmtp() || $this->isSystemSmtp();
						$aObj["structure"] 	= false;
						$aObj["error_msg"] 	= "";
						$aObj["error_no"] 	= 0;

						if (!$this->checkEmail($aObj["email"])) {
							$aObj["error_msg"] 	= "email not correct";
							$aObj["error_no"] 	= 2;
						}
						if (empty($this->aParams['from_email']) || !$this->isSmtp()) {
							$aObj["error_msg"] 	= "email or smtp error";
							$aObj["error_no"] 	= 3;
						}
						$aResult[] = $aObj;
					}
					else {
						for ($j = 0; $j < count($aTmp); $j++) {
							if (strstr($aTmp[$j], "@") != false) {
								$aObj["id"] 		= 0;
								$aObj["email"] 		= trim($aTmp[$j]);
								$aObj["name"] 		= "";
								$aObj["to_ext"] 	= true;
								$aObj["is_smtp"] 	= $this->isSmtp() || $this->isSystemSmtp();
								$aObj["structure"] 	= false;
								$aObj["error_msg"] 	= "";
								$aObj["error_no"] 	= 0;
								
								if (!$this->checkEmail($aObj["email"])) {
									$aObj["error_msg"] 	= "email not correct";
									$aObj["error_no"] 	= 2;
								}
								if (empty($this->aParams['from_email']) || !$this->isSmtp()) {
									$aObj["error_msg"] 	= "email or smtp error";
									$aObj["error_no"] 	= 3;
								}								
								$aResult[] = $aObj;
							}
						}
					}
				}
				else {
					$aEmps[] = $sAddr;
				}
			}
			
			for ($j = 0; $j < count($aEmps); $j++) {
				$bCheck = false;
				for ($i = 0; $i < $iRef; $i++) {

					$sTmp 		= str_replace(",","",$aRef[$i]["name"]);
					$aEmps[$j]  = str_replace(",","",$aEmps[$j]);
					if (!empty($sTmp) && $sTmp == $aEmps[$j]) {
						$aObj["id"] 		= 0;
						$aObj["name"] 		= stripslashes(trim($sTmp));
						$aObj["email"] 		= "";
						$aObj["structure"] 	= true;
						$aObj["to_ext"] 	= false;
						$aObj["is_smtp"] 	= false;
						$aObj["error_msg"] 	= "";
						$aObj["error_no"] 	= 0;
						
						for ($k = 0; $k < $iUsrs; $k++) {
							$aTmp = explode("/^/col/^/", $aUsrs[$k]["employee_ref_value"]);
							
							if (isset($aTmp[1]) && (int) $aTmp[1] == $aRef[$i]["id"]) {
								$aObj["id"] = $aUsrs[$k]["id"];
								if (strstr($aUsrs[$k]["email"], "@") != false) {
									$aCopy 				= explode("|^|row|^|", $aUsrs[$k]["email_send_copy"]);
									$aObj["email"] 		= trim($aUsrs[$k]["email"]);
									$aObj["to_ext"] 	= (isset($aCopy[1]) && $aCopy[1] == 1) ? true : false;
									$aObj["is_smtp"] 	= (!empty($aUsrs[$k]["email_smtp_address"]) || $this->isSystemSmtp()) ? true : false;
									if (empty($this->aParams['from_email']) || !$this->isSmtp()) {
										$this->aParams['from_email'] = $this->getSystemEmail();
									}
									break;
								}
							}
						}
						
						$aResult[] = $aObj;
						$bCheck = true;
					}
				}
				
				if (!$bCheck) {
					$aObj["id"] 		= 0;
					$aObj["name"] 		= stripslashes($aEmps[$j]);
					$aObj["email"] 		= "";
					$aObj["structure"] 	= true;
					$aObj["to_ext"] 	= false;
					$aObj["is_smtp"] 	= false;
					$aObj["error_msg"] 	= "employee not exists";
					$aObj["error_no"] 	= 1;
					$aResult[] 			= $aObj;
				}
			}
		}
		return $aResult;
		
	}
	
	private function formHeaders($sTo = "", $sFrom = "", $sCC = "", $sBCC = "")
	{
		if (empty($sTo)) {
			$sTo = $this->toHeaders($this->aParams["to"], true);
		}
		if (empty($sCC)) {
			$sCC = $this->toHeaders($this->aParams["cc"], true);
		}
		if (empty($sBCC)) {
			$sBCC = $this->toHeaders($this->aParams["bcc"], true);
		}
		if (empty($sFrom)) {
			$sFrom = $this->aParams["from_email"];
		}
		
		$sHeaders = "";
		$sHeaders .= "Return-path: <" . $sFrom . ">\r\n";
		$sHeaders .= "Date: " . $this->aParams["time"] . "\r\n";
		$sHeaders .= "From: " . $this->toUtf8($this->aParams["from_name"]) . (($this->checkEmail($sFrom)) ? " <" . $sFrom . ">" : "") . "\r\n";
		$sHeaders .= "To: " . $sTo . "\r\n";
		$sHeaders .= !empty($sCC) ? "Cc: " . $sCC . "\r\n" : "";
		$sHeaders .= !empty($sBCC) ? "Bcc: " . $sBCC . "\r\n" : "";
		$sHeaders .= "Subject: " . $this->toUtf8($this->aParams["subject"]) . "\r\n";
		$sHeaders .= "MIME-Version: 1.0\r\n";
		$sHeaders .= "Reply-to: " . $this->toUtf8($this->aParams["from_name"]) . " <" . $sFrom . ">\r\n";
		$sHeaders .= "Content-type: " . $this->aParams["type"] . "; charset=\"UTF-8\"\r\n";
		$sHeaders .= "X-Mailer: MSF_Framework\r\n";
		$sHeaders .= "\r\n\r\n";
		return base64_encode($sHeaders);
	}
	
	private function toHeaders($aTo, $bEncode = false)
	{
		$aResult 	= array();
		$iSize 		= count($aTo);
		
		for ($i = 0; $i < $iSize; $i++) {
			if (!empty($aTo[$i]["name"]) && !empty($aTo[$i]["email"])) {
				if ($bEncode) {
					$aResult[] = $this->toUtf8($aTo[$i]["name"])  . " <" . $aTo[$i]["email"] . ">";
				}
				else {
					$aResult[] = $aTo[$i]["name"] . " <" . $aTo[$i]["email"] . ">";
				}
			}
			elseif (!empty($aTo[$i]["email"])) {
				$aResult[] = $aTo[$i]["email"];
			}
			elseif (!empty($aTo[$i]["name"])) {
				if ($bEncode) {
					$aResult[] = $this->toUtf8($aTo[$i]["name"]);
				}
				else {
					$aResult[] = $aTo[$i]["name"];
				}
			}

		}
		
		return implode(', ', $aResult);
	}
	
	private function toUtf8($sString)
	{
		return !empty($sString) ? "=?UTF-8?B?" . base64_encode(stripslashes(stripslashes($sString))) . "?=" : "";
	}
	
	private function getUniqueAddresses()
	{
		$aResult 	= array();
		$aObj 		= array();
		
		if (isset($this->aParams["bcc"])) {
			for ($i = 0; $i < count($this->aParams["bcc"]); $i++) {
				if (empty($this->aParams["bcc"][$i]["name"]) && empty($this->aParams["bcc"][$i]["email"])) {
					continue;
				}
				if (isset($this->aParams["bcc"][$i]["id"])) {
					$aObj["id"] = $this->aParams["bcc"][$i]["id"];
				}
				$aObj["name"] 		= $this->aParams["bcc"][$i]["name"];
				$aObj["email"] 		= $this->aParams["bcc"][$i]["email"];
				$aObj["to_ext"] 	= $this->aParams["bcc"][$i]["to_ext"];
				$aObj["is_smtp"] 	= $this->aParams["bcc"][$i]["is_smtp"];
				$aObj["structure"] 	= $this->aParams["bcc"][$i]["structure"];
				$aResult[] 			= $aObj;
			}
		}
		
		if (isset($this->aParams["cc"])) {
			for ($i = 0; $i < count($this->aParams["cc"]); $i++) {
				if (empty($this->aParams["cc"][$i]["name"]) && empty($this->aParams["cc"][$i]["email"])) {
					continue;
				}
				if (!$this->uniqueExists($this->aParams["cc"][$i], $aResult)) {
					if (isset($this->aParams["cc"][$i]["id"])) {
						$aObj["id"] = $this->aParams["cc"][$i]["id"];
					}
					$aObj["name"] 		= $this->aParams["cc"][$i]["name"];
					$aObj["email"] 		= $this->aParams["cc"][$i]["email"];
					$aObj["to_ext"] 	= $this->aParams["cc"][$i]["to_ext"];
					$aObj["is_smtp"] 	= $this->aParams["cc"][$i]["is_smtp"];
					$aObj["structure"] 	= $this->aParams["cc"][$i]["structure"];
					$aResult[] 			= $aObj;
				}
			}
		}
		
		for ($i = 0; $i < count($this->aParams["to"]); $i++) {
			if (empty($this->aParams["to"][$i]["name"]) && empty($this->aParams["to"][$i]["email"])) {
				continue;
			}
			if (!$this->uniqueExists($this->aParams["to"][$i], $aResult)) {
				if (isset($this->aParams["to"][$i]["id"])) {
					$aObj["id"] = $this->aParams["to"][$i]["id"];
				}
				$aObj["name"] 		= $this->aParams["to"][$i]["name"];
				$aObj["email"] 		= $this->aParams["to"][$i]["email"];
				$aObj["to_ext"] 	= $this->aParams["to"][$i]["to_ext"];
				$aObj["is_smtp"] 	= $this->aParams["to"][$i]["is_smtp"];
				$aObj["structure"] 	= $this->aParams["to"][$i]["structure"];
				$aResult[] 			= $aObj;
			}
		}
		
		return $aResult;
	}
	
	private function uniqueExists($aToCheck, $aArray)
	{
		$bResult = false;
		for ($i = 0; $i < count($aArray); $i++) {
			if ($aToCheck["name"] == $aArray[$i]["name"] && $aToCheck["email"] == $aArray[$i]["email"]) {
				$bResult = true;
				break;
			}
		}
		
		return $bResult;
	}
	
	private function getEmailsFromRef($aRef)
	{
		$aResult = array();
		
		for ($i = 0; $i < count($aRef); $i++) {
			if (!empty($aRef[$i]["email"])) {
				$aResult[] = $aRef[$i]["email"];
			}
		}
		
		return $aResult;
	}
	
	private function getSize($iFilesSize)
	{
		return strlen($this->aParams["headers"]) + strlen($this->aParams["body"]) + $iFilesSize;
	}
	
	public function removeFile($iMailId, $iFileId)
	{
		$aEmailId 	= $this->DB->getEmailId($this->iFolderId, $iMailId);
		$iEmailId 	= isset($aEmailId["email_id"]) ? $aEmailId["email_id"] : 0;
		
		if ($iEmailId > 0) {
			$aFiles = $this->Media->getFiles("lms_" . $this->iFolderId, $iEmailId);
		
			if (empty($aFiles)) {
				$this->DB->updateAttachmets($this->iFolderId, $iMailId);
			}
		}
		
		try {
			$this->Media->remove($iFileId);
		}
		catch (Exception $e) {
			
		}
	}
	
	private function attachFiles($aFiles, $iEmailId, $iFolderId)
	{
		$aNames = array();
		
		if (!$this->bFCheck) {
			$this->bFCheck = true;
			if (!empty($aFiles["name"][0])) {
				$aFTmp = array();
				
				for ($i = 0; $i < count($aFiles["name"]); $i++) {
					if ($aFiles["error"][$i] == 0) {
						$aTmp["name"] 		= $aFiles["name"][$i];
						$aTmp["type"] 		= $aFiles["type"][$i];
						$aTmp["tmp_name"] 	= $aFiles["tmp_name"][$i];
						$aTmp["error"] 		= $aFiles["error"][$i];
						$aTmp["size"] 		= $aFiles["size"][$i];
						$aFTmp[] 			= $this->Media->upload($aTmp, "lms_" . $iFolderId, $iEmailId);
						$aNames[] 			= $aTmp["name"];
					}
				}
				
				if (!empty($this->aFilesIds)) {
					$this->attachFiles($aFiles, $iEmailId, $iFolderId);
				}
				$this->aFilesIds	= array_merge($this->aFilesIds, $aFTmp);
				$this->aFileNames	= array_merge($this->aFileNames, $aNames);
			}
			elseif (!empty($this->aFilesIds)) {
				$this->attachFiles($aFiles, $iEmailId, $iFolderId);
			}
		}
		else {
			for ($i = 0; $i < count($this->aFilesIds); $i++) {
				$aFile 		= $this->Media->getFileInfo($this->aFilesIds[$i]);
				$sPath 		= $aFile["directory"] . "/" . $aFile["file_id"];
				$sTmpFile 	= $aFile["directory"] . "/" . MSF_String::random(6) . "_" . $aFile["file_id"];
				
				if (!MSF_File::exists($sTmpFile)) {
					MSF_File::copy($sPath, $sTmpFile);
				}
				
				$aTmp["name"] 		= $aFile["name"];
				$aTmp["type"] 		= $aFile["type"];
				$aTmp["tmp_name"] 	= $sTmpFile;
				$aTmp["error"] 		= 0;
				$aTmp["size"] 		= $aFile["size"];
				
				$this->Media->upload($aTmp, "lms_" . $iFolderId, $iEmailId);
				
				$this->aTmpFiles[] = $sTmpFile;
				$aNames[] = $aTmp["name"];
			}
		}
		
		if (!empty($aNames)) {
			$this->DB->addFilesToSearch($aNames, $iFolderId, $this->DB->getNewMessageId());
		}
	}
	
	private function stripSlashes($aInput)
	{
		foreach ($aInput AS $k => $v) {
			if (is_string($v)) {
				$aInput[$k] = MSF_String::stripSlashes($v);
			}
		}
		
		return $aInput;
	}
	
	private function prepareOpen($aInput)
	{
		$aResult = array();
		
		if (!empty($aInput)) {
			$aResult["to"] 	= "";
			$aResult["cc"] 	= "";
			$aResult["bcc"] = "";
			
			$Decoded = LMS_MailDecode::process(base64_decode($aInput["headers"]));
			if (strtoupper($aInput["encode"]) != "UTF-8" && !empty($aInput["encode"])) {
				if (isset($Decoded->headers["cc"])) {
					$aResult["cc"] = iconv($aInput["encode"], "UTF-8", LMS_MailDecode::decodeValue($Decoded->headers["cc"]));
				}
				if (isset($Decoded->headers["bcc"])) {
					$aResult["bcc"] = iconv($aInput["encode"], "UTF-8", LMS_MailDecode::decodeValue($Decoded->headers["bcc"]));
				}
			}
			else {
				if (isset($Decoded->headers["cc"])) {
					$aResult["cc"] = LMS_MailDecode::decodeValue($Decoded->headers["cc"]);
				}
				if (isset($Decoded->headers["bcc"])) {
					$aResult["bcc"] = LMS_MailDecode::decodeValue($Decoded->headers["bcc"]);
				}
			}
			$sPattern = "<[-a-z0-9!#$%&\'*+/=?^_`{|}~]+@([.]?[a-zA-Z0-9_/->])*";
			$aResult["cc"] = ereg_replace($sPattern, "", $aResult["cc"]);
			$aResult["bcc"] = ereg_replace($sPattern, "", $aResult["bcc"]);
			if ($this->iFolderId == self::DRAFT) {
				if (!empty($aResult["cc"]) || !empty($aResult["bcc"])) {
					$this->Reference->load(1);
					$aRef = $this->Reference->getAsArray();
					$aResult["cc"] 	= $this->toRefValue($aResult["cc"], $aRef);
					$aResult["bcc"] = $this->toRefValue($aResult["bcc"], $aRef);
				}
			}
			
			$Date = new MSF_Date();
			
			$aResult["to"] 			= (empty($aResult["to"])) ? $aInput["to_email"] : $aResult["to"];
			$aResult["to"] 			= ($aResult["to"] == MSF_Registry::get("MSF_Locale")->getWord("LMS_LABEL_EMAIL_NOT_GIVEN")) ? "" : $aResult["to"];
			$aResult["to_email"] 	= $aResult["to"];
			$aResult["message_id"] 	= $aInput["message_id"];
			if (!$this->isLocalEmail($aInput["from_email"]) && $this->checkEmail($aInput["from_email"])) {
				$sFrom = $aInput["from_name"] . " <" . $aInput["from_email"] . ">";
			}
			else {
				$sFrom = $aInput["from_name"];
			}
			$aResult["from"]		= ($aInput["from_name"] == $aInput["from_email"])
											? $aInput["from_email"]
											: $sFrom;
			$aResult["subject"] 	= $aInput["subject"];
			$aResult["time"] 		= $Date->parseAndFormat($aInput["time"], CNF_DATE_TIME_FORMAT);
			$aResult["attachments"] = $aInput["attachments"];
			$aResult["files"] 		= array();
			$aResult["body"] 		= $this->replaceTags($aInput["body"], $aInput["type"]);
			
			if ($aInput["attachments"] == "Y") {
				$aFTmp 	= array();
				$aFiles = $this->Media->getFiles("lms_" . $this->iFolderId, $aInput["email_id"]);
				
				if (!empty($this->aAttached)) {
					for ($i = 0; $i < count($aFiles); $i++) {
						if (in_array($aFiles[$i]["file_id"], $this->aAttached)) {
							$aFTmp[] = $aFiles[$i];
						}
					}
				}
				else {
					if (IS_REQUEST) {
						$aFTmp = array();
					}
					else {
						$aFTmp = $aFiles;
					}
				}
				$aResult["files"] = $aFTmp;
			}
		}
		
		return $aResult;
	}
	
	private function isLocalEmail($sEmail) {

		$sysEmail = $this->getSystemEmail();
		if ($sysEmail == $sEmail) return true;
		
		$this->Reference->load(1);
		$aUsrs = $this->Reference->getAsArray();
		$iUsrs = count($aUsrs);
		
		for ($j = 0; $j < $iUsrs; $j++) {
			if ($sEmail==$aUsrs[$j]["email"]) return true;		
		}
		return false;

	}
	
	private function replaceTags($sBody, $sType)
	{
		if ($sType != "text/plain" && $sType != "") {
			$sBody = preg_replace("|<!--.*?-->|si", "", $sBody);
			$sBody = preg_replace("|<\?xml.*?\/>|si", "", $sBody);
			$sBody = preg_replace("|<!.*?>|si", "", $sBody);
			$sBody = preg_replace("|<style.*?<\/style>|si", "", $sBody);
		}
		if ($sType == "text/plain") {
			$sBody = strip_tags($sBody);
		}
		
		return (!in_array($sType, array("text/html", "text/richtext")) || $sType == "") ? nl2br(trim($sBody)) : $sBody;
	}
	
	private function toRefValue($sString, $aRef)
	{
		$sResult = "";
		$aEmails = explode(",", $sString);
		
		for ($i = 0; $i < count($aEmails); $i++) {
			for ($j = 0; $j < count($aRef); $j++) {
				if ($aEmails[$i] == $aRef[$j]["email"]) {
					$sResult .= "empty/" . $aRef[$j]["id"] . "|";
					break;
				}
			}
		}
		
		return $sResult;
	}
	
	private function getUserEmail()
	{
		$oUR = new UserReference(
			new UserReferenceDB(
			MSF_Registry::get("MSF_DB")
			)
		);
		return $oUR->getUserEmail($this->iUserId);
	}
	
	private function getSystemUserId()
	{
		$oUR = new UserReference(
			new UserReferenceDB(
			MSF_Registry::get("MSF_DB")
			)
		);
		return $oUR->getSystemUserId();
	}
	
	
	private function getUserSmtpAddress()
	{
		$oUR = new UserReference(
			new UserReferenceDB(
			MSF_Registry::get("MSF_DB")
			)
		);
		return $oUR->getUserSmtpAddress($this->iUserId);
	}

}

?>