<?php
// MSF_UserBox already loaded
include_once "modules/structure/library/Struct_Main.php";
include_once "modules/references/library/Reference/REF_Value.php";

/**
 *
 */
class MSF_OrgStructureFacade {
	
	const POST_REF_ID = 3;
	
	const SUBDIVISION_REF_ID = 4;
	
	/**
	 * @var MSF_OrgStructureFacade
	 */
	private static $oInstance = null;
	
	/**
	 * @var MSF_UserBox
	 */
	private $oUB;
	
	/**
	 * @var Struct_Main
	 */
	private $oStr;
	
	/**
	 * @var UserReference
	 */
	private $oP;
	
	/**
	 * @var UserReference
	 */
	private $oSub;
	
	/**
	 * @var REF_Value
	 */
	private $oRV;
	
	/**
	 * @return MSF_OrgStructureFacade
	 */
	public static function getInstance() {
		if (self::$oInstance == null) {
			self::$oInstance = new MSF_OrgStructureFacade();
		}
		
		return self::$oInstance;
	}
	
	private function __clone() {}
	
	private function __construct() {
		$this->oUB 	= MSF_Registry::get("MSF_UserBox");
		$this->oStr	= new Struct_Main();
		$this->oRV	= new REF_Value();
		$this->oP	= new UserReference(new UserReferenceDB(MSF_Registry::get("MSF_DB")));
		$this->oP->load(self::POST_REF_ID);
		$this->oSub	= new UserReference(new UserReferenceDB(MSF_Registry::get("MSF_DB")));
		$this->oSub->load(self::SUBDIVISION_REF_ID);
	}
	
	public function getCurrentStructureId()
	{
		return $this->oUB->getCurrentStructureId();
	}	
	
	public function getStructureIdsByAddress(WF_Document $oDoc, $sType, $sValue, $bWithPosts = true) {
		$oV = new WF_Values($oDoc, $oDoc->getCurrentVersion(), $oDoc->getDoctypeVersion());
		return $oV->toStructureIds($sValue, $bWithPosts);
	}
	
	public function getSubjectByStructureId($iStrId) {
		$aResult	= array();
		$aSubject 	= $this->oStr->getElement($iStrId);
		if (!empty($aSubject)) {
			$aResult = $aSubject;
		}

		return $aResult;
	}
	
	public function getSubdivisionBySubjectId($iSubId) {
		$aResult 	= array();
		if ($iSubId > 0) {
			$aResult = $this->oSub->row($iSubId)->show();
		}
		
		return $aResult;
	}
	
	public function getSubdivisionHashBySubjectId($iSubId)
	{
		$aResult 	= array();
		if ($iSubId > 0) {
			$aResult = $this->oSub->row($iSubId)->getAsHash();
		}
		
		return $aResult;
	}
	
	public function getSubdivisionByStructureId($iStrId) {
		$aSubject = $this->getSubjectByStructureId($iStrId);
		return $this->getSubdivisionBySubjectId($aSubject["subject_id"]);
	}
	
	public function getSubdivisionIdByStructureId($iStrId) {
		$aSubject = $this->getSubjectByStructureId($iStrId);
		return $aSubject["subject_id"];
	}
	
	public function getPostBySubjectId($iSubId) {
		$aResult 	= array();
		if ($iSubId > 0) {
			try {
				$aResult = $this->oP->row($iSubId)->show();
			}
			catch (Exception $oE) {
				if (CNF_MODE == "debug") {
					debug($oE);
				}
				MSF_Logger::logTrace($oE);
			}
			
		}
		
		return $aResult;
	}
	
	public function getEmailByStructureId($iStrId) {
		return $this->getAccountTypeAttr(
				 $this->getAccountByStructureId($iStrId),
				"user",
				"email"
				);
	}
	
	public function getPostByStructureId($iStrId) {
		$aSubject = $this->getSubjectByStructureId($iStrId);
		return $this->getPostBySubjectId(MSF_SL::getParam($aSubject, "subject_id", "0"));
	}
	
	public function getPostIdByStructureId($iStrId) {
		$aAccount = $this->getAccountByStructureId($iStrId);
		return (int)MSF_SL::getParam($aAccount, "currentPostId", 0);
	}
	
	public function getPostOrSubdivisionByStructureId($iStrId) {
		$aResult 	= array();
		$aSubject 	= $this->getSubjectByStructureId($iStrId);
		
		if (!empty($aSubject)) {
			if ($aSubject["type"] == "subdivision") {
				$aResult = $this->getSubdivisionBySubjectId($aSubject["subject_id"]);
			}
			else if ($aSubject["type"] == "post") {
				$aResult = $this->getPostBySubjectId($aSubject["subject_id"]);
			}
		}
		
		return $aResult;
	}
	
	public function getAccountByStructureId($iStrId) {
		$this->oUB->load($iStrId, "structure");
		$aResult = $this->oUB->getLoadedResult();
		if (isset($aResult[0])) {
			return $aResult[0];
		}
		else {
			return array();
		}
		
	}
	
	public function getEmployeeIdByStructureId($iStrId) {
		return $this->getAccountTypeAttr(
				 $this->getAccountByStructureId($iStrId),
				"employee",
				"id"
				);
	}
	
	public function getEmployeeNameByStructureId($iStrId) {
		return $this->getAccountTypeAttr(
				$this->getAccountByStructureId($iStrId),
				"employee",
				"name"
				);
	}
	
	private function getAccountTypeAttr($aAccount, $sType, $sAttr) {
		if (isset($aAccount[$sType])) {
			if (isset($aAccount[$sType][$sAttr])) {
				return $aAccount[$sType][$sAttr];
			}
			else {
				return "";
			}
			
		}
		else {
			return null;
		}
		
	}
	
	public function isStructureId($iStrId) {
		$sSql = "SELECT COUNT(*) AS total FROM structure WHERE node_id = " . (int) $iStrId;
		$iRes = MSF_Registry::get("MSF_DB")->select($sSql, true, 'total');
		return $iRes == 1;
	}
	
	/**
	 * @return string
	 */
	public function strIdsToRefValue(array $aStrIds, $mOpt = true) {
		$oRV = new REF_Value();
		if (is_bool($mOpt)) {
			$mOpt = array("postEnable" => $mOpt);
		}
		foreach ($aStrIds AS $iStrId) {
			$aSubject 			= $this->getSubjectByStructureId($iStrId);
			$aPostOrSubdivision = $this->getPostOrSubdivisionByStructureId($iStrId);
			$aIds 				= array($iStrId);
			
			if (!empty($aPostOrSubdivision)) {
				if ($aPostOrSubdivision[0]["name"] == "post") {
					$aAccount 	= $this->getAccountByStructureId($iStrId);
					
					if (isset($aAccount["employee"]["id"])) {
						$aIds[]		= $aAccount["employee"]["id"];
						foreach ($aAccount["posts"] as $aPost) {
							if ($aPost["id"] == $aSubject["subject_id"]) {
								$aIds[] = $aPost["id"];
								break;
							}
							
						}
						
						$sRes = RefFieldHelper::formatStructureValue(
								$aIds[1], 
								$aIds[2], 
								MSF_SL::getParam($mOpt, "employee", "name"), 
								MSF_SL::getParam($mOpt, "post", "post"), 
								MSF_SL::getParam($mOpt, "employeeEnable", 1), 
								MSF_SL::getParam($mOpt, "postEnable", 1)
								);
						$oRV->add($aIds, $sRes);
						
					}
				}
				else {
					$iSubdivision = 0;
					for ($i=0; $i<count($aPostOrSubdivision); $i++) {
						if ($aPostOrSubdivision[$i]["name"] == "subdivision") {
							$iSubdivision = $i;
						}
					}
					if ($aPostOrSubdivision[$iSubdivision]["name"] == "subdivision") { 
						$aIds[]	= $this->getSubdivisionIdByStructureId($iStrId);
						$oRV->add($aIds, $aPostOrSubdivision[$iSubdivision]["value"]);
					}
				}
			}
			
		}
		
		return $oRV->format();
	}
	
	private function getStrIdByEmployeeAndNodeId($iEmployeeId, $iNodeId){
		$iStrId = 0;
		
		foreach ($this->oStr->getList() AS $aItem) {
			if ($aItem["subject_id"] == $iEmployeeId && $aItem["parent_id"] == $iNodeId) {
				$iStrId = $aItem["node_id"];
				break;
			}
		}
		
		return $iStrId;
	}
	
	public function getBoss($iStrId) {
            
                $postId = "";
                $sValue 	= "";
		$aSubject 	= $this->getSubjectByStructureId($iStrId);
    		try {
			if (!empty($aSubject)) {
				if ($aSubject["type"] == "post"){
                                    $postId = $this->getPostIdByStructureId($iStrId);
                                    $aSubject = $this->getSubjectByStructureId($aSubject["parent_id"]);
    				}	
				$aSubdivision 	 	= $this->getSubdivisionHashBySubjectId($aSubject["subject_id"]);
                                //$aSubdivision = Array ( [subdivision] => отдел1 [employee_ref_value] => Иванов/^/col/^/26 [name_d5f9514318] => [author_name] => Павленко Павел Павлович (Зицпредседатель)/^/col/^/3/^/col/^/3/^/col/^/3 [editor_name] => Павленко Павел Павлович (Зицпредседатель)/^/col/^/3/^/col/^/3/^/col/^/3 [created_at] => 2013-06-24 18:57:05 [modified_at] => 2013-06-24 18:57:05 )                               
				$this->oRV->parse($aSubdivision["employee_ref_value"]);
				$aEmloyeeRefKey		= $this->oRV->getKeysList();                                       
    
                                if (!empty($aEmloyeeRefKey) && $aEmloyeeRefKey[0] == $postId) {
                                    //руководитель данного отдела и $iStrId - одно лицо
                                    $aSubject = $this->getSubjectByStructureId($aSubject["parent_id"]);
                                    $aSubdivision = $this->getSubdivisionHashBySubjectId($aSubject["subject_id"]);
                                    $this->oRV->parse($aSubdivision["employee_ref_value"]);
                                    $aEmloyeeRefKey = $this->oRV->getKeysList();
                                }
                                 
				if (!empty($aEmloyeeRefKey)) {
                                    
                                    
					$aEmployeeValue = $this->oRV->getValuesList();
					$sEmployeeValue = "";
					if (!empty($aEmployeeValue)) {
						$sEmployeeValue .= $aEmployeeValue[0];
					}
					$iNodeId = 0;
					$aResult 	= $this->oUB->load($aEmloyeeRefKey[0], "post");
					if (!empty($aResult)) {
						$aPosts = $aResult["posts"];
						foreach ($aPosts as $aKey) {
							if (($aKey["id"] == $aEmloyeeRefKey[0]) && isset($aKey["node_id"])) {
								$iNodeId = $aKey["node_id"]; 
							}
						}
					}
					if ($iNodeId) {
						$sValue .= $sEmployeeValue . "/^/col/^/" . $iNodeId;
					}
					else {
						return $sValue;
					}
					$aPost = $this->getPostBySubjectId($aEmloyeeRefKey[0]);
					$aPostRefKey = array();
					foreach ($aPost as $aKey) {
						if ($aKey["name"] == "employee_ref_value") {
							$this->oRV->parse($aKey["value"]);
							$aPostRefKey = $this->oRV->getKeysList();
						}
					}
					if (!empty($aPostRefKey)) {
						$sValue .= "/^/col/^/" . $aPostRefKey[0] . "/^/col/^/" . $aEmloyeeRefKey[0];
					}
				}
			}
		}
		catch (Exception $e) { }
		
		return $sValue;
	}
	

}
