<?php
if(!defined("__MSF")) {
	die("Direct access is not allowed");
}

require_once "modules/media/library/MD_Media.php";
require_once "modules/lms/library/security/LMS_MailSwitcher.php";
require_once "modules/references/library/security/REF_ReferencesSwitcher.php";
require_once "modules/workflow/library/security/WF_MediaSwitcher.php";
require_once "modules/workflow/library/security/WF_FoldersSwitcher.php";
require_once "library/MSF_Export/MSF_Export_Media.php";
require_once "modules/archive/library/ARCH_SecurityManager.php";

class MSF_FrontController extends MSF_ControllerSupport
{
	public function indexAction()
	{
		$View	= new MSF_ViewSupportXml();
		$View->addTemplate("/modules/media/view/templates/default/test_upload.xsl");
		$View->addLinksOf("references");

		$Temlate	= MSF_Registry::get("MSF_Template");

		$Temlate->addLink("/modules/media/view/js/media.js", "text/javascript");
		$this->setView($View);
	}

	public function uploadAction()
	{
		if(!isset($_FILES["upload_file"])) {
			throw new MD_CannotUploadException("File not uploaded");
		}

		$sModule	= (string)$this->getParam(0);
		$iId		= (int)$this->getParam(1);

		$Media	= new MD_Media(new MD_MediaDB(MSF_Registry::get("MSF_DB")));
		$Media->setUploadDirectory(CNM_MEDIA_UPLOAD_DIRECTORY);
		$Media->setMaxFileSize(CNM_MEDIA_MAX_FILE_SIZE);
		
		$iId	= $Media->upload($_FILES["upload_file"], $sModule, $iId);

		$aInfo	= $Media->getFileInfo($iId);

		echo "<script>parent.MD_UploadComplete(" . $iId . ", '" . $aInfo["name"] . "');</script>";
	}

	public function downloadAction()
	{
		$iId	= $this->getIntParam(0);

		$Media	= new MD_Media(new MD_MediaDB(MSF_Registry::get("MSF_DB")));
		$Media->setLinkDirectory(CNM_MEDIA_LINKS_DIRECTORY);
		
		
		try {
			$aInfo 		= $Media->getFileInfo($iId);
			$sPathFile 	= $aInfo["directory"] . "/" . $aInfo["file_id"];
			
			if (!file_exists($sPathFile)) {
				$oView = new MSF_ViewSupportXml();
    			$this->setView($oView);
				MSF_Registry::get("MSF_Messenger")->addError($this->Locale->getWord("WORKFLOW_MESSAGE_FILE_NOT_FOUND"));
			}
			else {
				if($this->hasAccess($aInfo)) {
					$Media->download($iId);
					die();
				}
				else {
					$this->notAccessAction();
					return;				
				}
			}
			
		}
		catch(MD_Exception $e) {
			header("HTTP/1.0 404 Not Found");
		}

	}

	public function imageAction()
	{
		$iId	= $this->getIntParam(0);
		$iW		= $this->getIntParam(1);
		$iH		= $this->getIntParam(2);

		$Media	= new MD_Media(new MD_MediaDB(MSF_Registry::get("MSF_DB")));
		$Media->setLinkDirectory(CNM_MEDIA_LINKS_DIRECTORY);
		$Media->setImageCacheDirectory(CNM_MEDIA_IMAGE_CACHE_DIRECTORY);
				
		try {
			if($this->hasAccess($Media->getFileInfo($iId))) {
				//header("Location: /" . $Media->resize($iId, $iW, $iH));
				$Media->resize($iId, $iW, $iH);
				die();
			}
			else {
				//MSF_AccessDenied::exitApp($this->Locale->getWord("ROOT_ERROR_NO_ACCESS"));
				$this->notAccessAction();
				return;
			}
			
		}
		catch(MD_Exception $e) {
			header("HTTP/1.0 404 Not Found");
		}

	}

	public function clearecacheAction()
	{
		$Media	= new MD_Media(new MD_MediaDB(MSF_Registry::get("MSF_DB")));
		$Media->setImageCacheDirectory(CNM_MEDIA_IMAGE_CACHE_DIRECTORY);
		$Media->setImageCacheLifeTime(-1);
		$Media->clearImageCache();
	}

	public function clearlinksAction()
	{
		/*
		$Media	= new MD_Media(new MD_MediaDB(MSF_Registry::get("MSF_DB")));
		$Media->setLinkDirectory(CNM_MEDIA_LINKS_DIRECTORY);
		$Media->clearLinks();*/
	}
	
	public function rsReportsAction()
	{
		$oUserBox  = MSF_Registry::get("MSF_UserBox");
		$iUserId   = (int)$oUserBox->getUserAttribute("id");
		$sHash	   = $this->getParam(0);
		$sExt	   = $this->getParam(1);
		$oMedia	   = new MSF_Export_Media();
		$oMedia->download($iUserId . "." . $sHash, $sExt);
		die();		
	}
	
	private function hasAccess($aInput)
	{
		$iId		= $aInput["user_id"];
		if(isset($aInput["module"])) {
			$sModule	= $aInput["module"];
		}
		else {
			$sModule	= "archive_document";
		}
		
		$aType		= explode("_", $sModule);
		$aType[1] 	= isset($aType[1]) ? $aType[1] : "";
		$sType		= $aType[0];
		$bReturn	= false;
		$bPass 		= (MSF_Registry::get("MSF_UserBox")->isAdmin() == 1 && $aType[1] == "back") ? true : false;
		
		switch($sType) {
			case "references"	:
				if ($bPass) {
					$bReturn = true;
				}
				else {
					$Switcher	= new REF_ReferencesSwitcher(MSF_Registry::get("MSF_DB"));
					$bReturn	= $Switcher->hasAccess("view", WF_Utils::getPath(), array("reference_id"=>$iId));
				}
				break;
			case "lms"			:
				$Switcher	= new LMS_MailSwitcher(MSF_Registry::get("MSF_DB"));
				$bReturn	= $Switcher->hasAccess("view", WF_Utils::getPath(), array("name"=>$sModule, "id"=>$iId));
				break;
			case "workflow"		:
				$Switcher	= new WF_MediaSwitcher(MSF_Registry::get("MSF_DB"));
				$bReturn	= $Switcher->hasAccess("view", WF_Utils::getPath(), array("name"=>$sModule, "id"=>$iId));
				break;
			case "folders"		:
				if ($bPass) {
					$bReturn = true;
				}
				else {
					$Switcher	= new WF_FoldersSwitcher(MSF_Registry::get("MSF_DB"));
					$bReturn	= $Switcher->hasAccess("view", WF_Utils::getPath(), $iId);
				}
				break;
			case "archive"		:
				$Switcher	= new ARCH_SecurityManager();
				$bReturn	= TRUE;//$Switcher->hasFileAccess($iId);				
				break;
			case "default"		:
				$bReturn 	= true;
		}
		
		return $bReturn;
	}
}
?>