<?php
if(!defined("__MSF")) {
	die("Direct access not allowed");
}

include_once "MD_MediaDB.php";
include_once "MD_Exception.php";
include_once "MD_CannotUploadException.php";
include_once "MD_CannotCreateLinkException.php";
include_once "MD_FileNotLoadException.php";
include_once "MD_FileNotImageException.php";
include_once "MD_CannotCreateImageException.php";
include_once "MD_Image.php";
include_once "modules/archive/library/ARCH_MediaDB.php";

class MD_Media
{
	private $aAllowedMimeTypes		= array();

	private $DB						= null;

	private $sUploadDirectory		= null;
	
	private $sLinkDirectory			= null;
	
	private $sImageCacheDirectory	= null;
	
	private $iLinkLifeTime			= 0;
	
	private $iImageCacheLifeTime	= 0;

	private $iMaxFileSize			= 0;

	private $aFileInfo				= array();
	
	public function __construct(MD_MediaDB $DB)
	{
		$this->DB	= $DB;
		$this->reset();
	}

	public function setUploadDirectory($sDir)
	{
		$this->sUploadDirectory	= $sDir;
	}
	
	public function setLinkDirectory($sDir)
	{
		$this->sLinkDirectory	= $sDir;
	}
	
	public function setImageCacheDirectory($sDir)
	{
		$this->sImageCacheDirectory	= $sDir;
	}
	
	public function setMaxFileSize($iSize)
	{
		$this->iMaxFileSize	= (int)$iSize;
	}
	
	public function setLinkLifeTime($iSeconds)
	{
		$this->iLinkLifeTime	= $iSeconds;
	}
	
	public function setImageCacheLifeTime($iSeconds)
	{
		$this->iImageCacheLifeTime	= $iSeconds;
	}
	
	public function setAllowedMimeTypes($aType)
	{
		$this->aAllowedMimeTypes	= $aType;
	}

	public function getUploadDirectory()
	{
		return $this->sUploadDirectory;
	}
	
	public function getLinkDirectory()
	{
		return $this->sLinkDirectory;
	}

	public function getImageCacheDiretory()
	{
		return $this->sImageCacheDirectory;
	}
	
	public function getMaxFileSize()
	{
		return $this->iMaxFileSize;
	}

	public function getImageCacheLifeTime()
	{
		return $this->iImageCacheLifeTime;
	}
	
	public function getLinkLifeTime() {
		return $this->iLinkLifeTime;
	}
	
	public function getAllowedMimeTypes()
	{
		return $this->aAllowedMimeTypes;
	}

	public function getFileInfo($iId)
	{
		$this->load($iId);
		return $this->aFileInfo;
	}

	private function reset()
	{
		$this->sUploadDirectory			= CNM_MEDIA_UPLOAD_DIRECTORY;
		$this->sLinkDirectory			= CNM_MEDIA_LINKS_DIRECTORY;
		$this->sImageCacheDirectory		= CNM_MEDIA_IMAGE_CACHE_DIRECTORY;
		$this->aAllowedMimes			= "*";
		$this->iMaxFileSize				= 1048576; // 1 Mb
		$this->iLinkLifeTime			= 60 * 60 * 24;
		$this->iImageCacheLifeTime		= 60 * 60 * 24 * 7;
	}

	private function load($iId)
	{
		if (!empty($iId)) {
			$this->aFileInfo =	$this->DB->load($iId);
			if(empty($this->aFileInfo)) {
				$oArchDB = new ARCH_MediaDB();
				$this->aFileInfo = $oArchDB->load($iId);
				if(empty($this->aFileInfo)) {
					throw new MD_FileNotLoadException("File not load. Id: " . $iId);
				}
				
			}
			
		}
		
		//echo $this->aFileInfo["directory"];
		//$this->aFileInfo["link"]	= $this->createLink(CNF_ABS_PATH . "/" . $this->aFileInfo["directory"], $this->aFileInfo["name"], $this->aFileInfo["md5"]);
		//$this->aFileInfo["link"]	= $this->createLink($this->aFileInfo["directory"], $this->aFileInfo["name"], $this->aFileInfo["md5"]);
	}

	public function isProperSize($aFile)
	{
		if($aFile["size"] > $this->iMaxFileSize) {
			return false;
		}
		return true;
	}
	
	public function upload($aFile, $sModule, $iUserId)
	{
		if(empty($aFile) || $aFile["error"] != 0) {
			throw new MD_CannotUploadException("File not uploaded.");
		}
		if($aFile["size"] > $this->iMaxFileSize) {
			throw new MD_CannotUploadException("File size > " . $this->iMaxFileSize);
		}

		if(!MSF_Dir::exists($this->sUploadDirectory)) {
			MSF_Dir::create($this->sUploadDirectory);
		}

		$sDir		= $this->sUploadDirectory . "/" . date("Y");
		if(!MSF_Dir::exists($sDir)) {
			MSF_Dir::create($sDir);
		}

		$sDir		.= "/" . date("m");
		if(!MSF_Dir::exists($sDir)) {
			MSF_Dir::create($sDir);
		}

		$sDir		.= "/" . date("d");
		if(!MSF_Dir::exists($sDir)) {
			MSF_Dir::create($sDir);
		}
		
		$sFullPath	= $sDir . "/" . date("H");
		if(!MSF_Dir::exists($sFullPath)) {
			MSF_Dir::create($sFullPath);
		}
		
		$sFullPath	= $sFullPath . "/" . MSF_String::random(10);
		MSF_Dir::create($sFullPath);	

		$sFullPath 	= str_replace(CNF_ABS_PATH . '/', "", $sFullPath);
		$iFileId	= $this->DB->upload(
				$aFile["name"],
				$sFullPath,
				md5_file($aFile["tmp_name"]),
				sha1_file($aFile["tmp_name"]),
				$sModule,
				$iUserId,
				$aFile["type"],
				$aFile["size"]
				);
				
		$sNewFile = CNF_ABS_PATH . '/' . $sFullPath ."/" . $iFileId;
                
		MSF_File::move($aFile["tmp_name"], $sNewFile);
		//
		chmod($sNewFile, 0777);
		//
	
		return $iFileId;
	}

	public function link($iId)
	{
		$this->load($iId);
		
		return $this->createLink($this->aFileInfo["directory"], $this->aFileInfo["name"], $this->aFileInfo["md5"]);
	}

	public function download($id)
	{
		$this->load($id);
				
		$sFile	= $this->aFileInfo["directory"] . "/" . $id;
		if (MSF_File::exists($sFile)) {	
			if($this->aFileInfo["md5"] != "") {
				if(md5_file($sFile) != $this->aFileInfo["md5"]) {
					throw new MD_CannotCreateLinkException("Controll check failed");
				}
			}
	
			$sFileName	= (strstr($_SERVER['HTTP_USER_AGENT'], "MSIE")) ? iconv("UTF-8", "windows-1251", $this->aFileInfo["name"]) 
																		: $this->aFileInfo["name"];
			header('Content-Type: text/html; ');
			header("Content-Type: " 								. $this->aFileInfo["type"]);
			header('Accept-Ranges: bytes');
			header("Content-disposition: attachment; filename=\"" 	. $sFileName . "\"");
			header('Pragma: public');
			header("Content-Length: " 								. $this->aFileInfo["size"]);
			header('Pragma: public');
			
			/*if (array_pop(explode(".", $this->aFileInfo["name"])) == "php"){*/
				readfile(CNF_ABS_PATH . "/" . str_replace(CNF_ABS_PATH, "", $sFile));
			/*}
			else {
				virtual("/" . str_replace(CNF_ABS_PATH, "", $sFile));
			}*/
		}
	}
	
	public function getFiles($sModule, $iId)
	{
		$aFiles		= $this->DB->getFiles($sModule, $iId);
		$aReturn	= array();
		
		foreach($aFiles as $aRow) {
			//$aRow["link"]	= $this->createLink(CNF_ABS_PATH . "/" . $aRow["directory"], $aRow["name"], $aRow["md5"]);
			$aRow["link"]	= "stub";
			array_push($aReturn, $aRow);
		}
		
		return $aReturn;
	}
	
	public function saveModuleName($mFileId, $sModuleName)
	{
		if (is_array($mFileId)) {
			$mFileId[] 	= 0;
			$mFileId 	= implode(",", $mFileId);
		}
		
		$this->DB->updateModule($mFileId, $sModuleName);
	}
	
	public function remove($iId)
	{
            
		$this->load($iId);
		$this->DB->remove($iId);
                $fileName = $this->aFileInfo["directory"];
                $pos = strpos(CNF_ABS_PATH, $fileName);                 
                if($pos === false) {
                    $fileName2 = CNF_ABS_PATH . "/" . $fileName;
                } else {
                    $fileName2 = $fileName;
                }
		MSF_Dir :: remove($fileName2);
	}

	public function resize($iId, $x=0, $y=0)
	{
		$this->load($iId);
		$ext		= split("\.", $this->aFileInfo["name"]);
		$ext		= $ext[count($ext) - 1];
		
		$sFilePath	= $this->aFileInfo["directory"] . "/" . $iId;
		
		$sName	= $iId . "_" . $x . "_" . $y . "." . $ext;
		if(!MSF_File::exists($this->sImageCacheDirectory . "/" . $sName)) {
			$Image	= new MD_Image($sFilePath . "." . $ext);
			$Image->resize($this->sImageCacheDirectory . "/". $sName . "." . $ext, $x, $y);
		}
		
		//debug("/" . $this->sImageCacheDirectory . "/" . $sName, $this->aFileInfo["type"]);
		//debug($this->sImageCacheDirectory, $sName);
		//debug("/" . $this->sImageCacheDirectory . "/" . $sName);
		header("Content-Type: " 								. $this->aFileInfo["type"]);
		header("Content-disposition: inline; filename=" 		. basename($this->aFileInfo["name"]));
		
		virtual("/" . $this->sImageCacheDirectory . "/" . $sName);
		//die();
		//return $this->createLink($this->sImageCacheDirectory, $sName);
	}
	
	public function clearImageCache()
	{
		$aImage	= MSF_Dir::read($this->sImageCacheDirectory);
		$iNow	= time();
		foreach($aImage as $sImage) {
			$sImage		= $this->sImageCacheDirectory . "/" . $sImage;
			$iCreatedAt	= filemtime($sImage);
			
			if($iNow > ($iCreatedAt + $this->iImageCacheLifeTime)) {
				MSF_File::remove($sImage);
			}

		}

	}
	
	public function getDB()
	{
		return $this->DB;
	}

}
?>