<?php


if(!defined("__MSF")) {
	die("Direct access not allowed");
}

include_once CNF_ABS_PATH . "/modules/media/library/MD_Media.php";

class Test__MD_Media extends MSF_TestUnit
{
	public function setUp()
	{
		try {
			
			$aInfo		= $this->getFileConf();
			
			
			$im = @imagecreatetruecolor(200, 400);
			$text_color = imagecolorallocate($im, 233, 14, 91);
			imagestring($im, 1, 5, 5,  "A Simple Text String", $text_color);
			imagepng($im, $aInfo["tmp_name"]);
			imagedestroy($im);
/*
			$handle	= fopen($aInfo["tmp_name"], "w");
			for($i=0; $i<1024; $i++) {
				fwrite($handle, "a");
			}
			fclose($handle);
*/
		}
		catch(Exception $e) {
			$this->fail("Can`t create test data. Reason: \n" . $e->getMessage());
		}
		
	}

	public function setDown()
	{

		try {
			
			$aInfo		= $this->getFileConf();
			if(file_exists($aInfo["tmp_name"])) {
				unlink($aInfo["tmp_name"]);
			}
			
		}
		catch(Exception $e) {
			$this->fail("Can`t delete test data. Reason: \n" . $e->getMessage());
		}

	}

	public function test()
	{
		try {
			$this->testSetters();
			$iId = $this->testUpload();
			$this->testDownload($iId);
			$this->testResize($iId);
			$this->testClearLinks();
			$this->testClearCache();
			$this->testRemove($iId);
		}
		catch(Exception $e) {
			$this->fail("Test failed. Reason: " . $e->getMessage());
		}

	}

	private function testSetters()
	{
		$M	=	 new MD_Media(new MD_MediaDB(MSF_Registry::get("MSF_DB")));

		$M->setUploadDirectory("media");
		$M->setLinkDirectory("testLink");
		$M->setImageCacheDirectory("testCache");
		$M->setMaxFileSize(5242880);
		$M->setLinkLifeTime(999);
		$M->setImageCacheLifeTime(777);
		$M->setAllowedMimeTypes(array("*"));


		$this->assertEquals($M->getUploadDirectory(), "media", "Media::getUploadDirectory()");
		$this->assertEquals($M->getLinkDirectory(), "testLink", "Media::getLinkDirectory()");
		$this->assertEquals($M->getImageCacheDiretory(), "testCache", "Media::getImageCacheDiretory()");
		$this->assertEquals($M->getMaxFileSize(), 5242880, "Media::getMaxFileSize()");
		$this->assertEquals($M->getLinkLifeTime(), 999, "Media::getLinkLifetTime()");
		$this->assertEquals($M->getImageCacheLifeTime(), 777, "Media::getImageCacheLifeTime()");
		$this->assertEquals($M->getAllowedMimeTypes(), array("*"), "Media::getAllowedMimeTypes()");
	}

	private function testUpload()
	{
		try {
			$M	=	 new MD_Media(new MD_MediaDB(MSF_Registry::get("MSF_DB")));

			$M->setUploadDirectory("files/uploads");
			$M->setAllowedMimeTypes(array("*"));
			$M->setMaxFileSize(500);

			try {
				$M->upload($this->getFileConf(), "test", "1");
				$this->fail("Max file size ignored");
			}
			catch(MD_Exception $e) {
				$this->assertEquals($e->getMessage(), "File size > 500", "Too big.");
			}

			$M->setMaxFileSize(10000000);
			$iId	= $M->upload($this->getFileConf(), "test", "1");
			$this->pass("file upload");

			return $iId;
		}
		catch(MD_Exception $e) {
			$this->fail("Upload failed. Reason: " . $e->getMessage());
		}

	}

	private function testDownload($iId)
	{
		$M =	new MD_Media(new MD_MediaDB(MSF_Registry::get("MSF_DB")));
		//$M->load();

		$aFileInfo	= $M->getFileInfo($iId);
		//$this->assertEquals($aFileInfo["name"],			"testName.png");
		$this->assertEquals($aFileInfo["directory"], 	"files/uploads/" . date("d.m.Y"));
		$this->assertEquals($aFileInfo["md5"],			"f2a80b18491168f22c6c35c0b074fa70");
		$this->assertEquals($aFileInfo["sha1"],			"b89e5c4391114d052042708b833c33b661992302");
		$this->assertEquals($aFileInfo["module"],		"test");
		$this->assertEquals($aFileInfo["user_id"],		1);
		$this->assertEquals($aFileInfo["type"], 		"text/plain");
		
		$link = $M->link($iId);
		
		$this->assertEquals(md5_file($link), "f2a80b18491168f22c6c35c0b074fa70", "Link is valid");
	}
	
	private function testResize($iId)
	{
		$M 		=	new MD_Media(new MD_MediaDB(MSF_Registry::get("MSF_DB")));
		$sImage	= $M->resize($iId, 100, 100);
		try {			
			$aInfo	= getimagesize($sImage);			
			
			if(!$aInfo) {
				throw new Exception;
			}
			$this->assertTrue(($aInfo[0]<=100), "Width resized.");
			$this->assertTrue(($aInfo[1]<=100), "Height resized.");
		}
		catch(Exception $e) {
			$this->fail("Media::resize() failed. Reason: " . $e->getMessage());	
		}
	}
	
	private function testClearLinks()
	{
		$M =	new MD_Media(new MD_MediaDB(MSF_Registry::get("MSF_DB")));
		$M->setLinkLifeTime(-1);
		$M->clearLinks();

		$aDirs	= MSF_Dir::read("files/links");
		
		$this->assertTrue(empty($aDirs), "Media::clearLinks()");		
	}
	
	private function testClearCache()
	{		
		$M =	new MD_Media(new MD_MediaDB(MSF_Registry::get("MSF_DB")));
		$M->setImageCacheLifeTime(-1);
		$M->clearImageCache();
		
		$aDirs	= MSF_Dir::read("files/image_cache");
		
		$this->assertTrue(empty($aDirs), "Media::clearImageCache()");
	}

	private function testRemove($iId) 
	{
		$M			= new MD_Media(new MD_MediaDB(MSF_Registry::get("MSF_DB")));
		$aFileInfo	= $M->getFileInfo($iId);
		$M->remove($iId);
		
		$sPath	= $aFileInfo["directory"] . "/" . $aFileInfo["name"];
		$this->assertFalse(file_exists($sPath), "File removed");
		
		try {
			$M->getFileInfo($iId);
			$this->fail("File record not removed");
		}
		catch(MD_Exception $e) {
			$this->pass("File record removed");
		}
		
	}
	
	private function getAllowedMime()
	{
		array(
			"image/gif",
			"image/jpg",
			"image/jpeg",
			"text/plain"
		);
	}

	private function getFileConf()
	{
		$aReturn				= array();
		$aReturn["name"]		= "testName.png";
		$aReturn["type"]		= "text/plain";
		$aReturn["size"]		= "1024";
		$aReturn["tmp_name"]	= CNF_ABS_PATH . "/files/daersdfasd.png";
		$aReturn["error"]		= 0;


		return $aReturn;
	}

}
?>