<?php
if(!defined("__MSF")) {
	die("Direct access is not allowed");
}

include_once "modules/references/library/ReferencesList/REF_ReferencesList.php";
include_once "modules/references/view/xml/REF_ReferencesListXml.php";
include_once "modules/references/view/xml/REF_ReferenceXml.php";
include_once "modules/references/library/Reference/UserReference.php";
include_once "modules/references/view/xml/REF_TemplateXml.php";
include_once "library/MSF_Utils/MSF_RteObjectsIntegrator.php";
include_once "modules/references/library/security/REF_ReferencesSecurityManager.php";
include_once "modules/references/library/security/REF_ReferencesSwitcherDB.php";
include_once "modules/references/library/security/REF_ReferencesSwitcher.php";

class REF_ReferenceBack extends REF_BackAbstract
{
	private $iReferenceId;

	/**
	 * @var UserReference
	 */
	private $Reference;

	private $sType;

	private $View;

	private function DB()
	{
		return MSF_Registry::get("MSF_DB");
	}
	
	private function insert()
	{
		$sSql = "CREATE OR REPLACE FUNCTION ref_groups_null_replace(group_name varchar) RETURNS varchar AS $$
					BEGIN
		    			IF group_name IS NULL THEN
		        			RETURN '". MSF_Registry::get("MSF_Locale")->getWord("REFERENCE_WITHOUT_GROUP") ."';
		    			END IF;
			    		RETURN group_name;
					END;
				$$ LANGUAGE plpgsql;";
		
		$this->DB()->executeQuery($sSql);
		
		$sSql = "CREATE OR REPLACE VIEW ref_references_and_groups_view AS
					SELECT
						ref.reference_id,
						ref.name AS ref_name,
						ref.description,
						ref_groups_null_replace(gr.name) AS group_name,
						ref.created_at,
						ref.modified_at
					FROM
						references_config ref
					LEFT JOIN
						ref_references_groups gr
					ON
						ref.group_id = gr.group_id";
		
		$this->DB()->executeQuery($sSql);
	}
	
	public function indexAction()
	{
		$this->insert();
		
		$List = $this->refList();
		$View = new REF_ReferencesListXml(MSF_Registry::get("MSF_Template"), MSF_Registry::get("MSF_Locale"));
		$View->buildRecordset($List->getListRecordset(true));
		$View->addTemplate("/modules/references/view/templates/default/referenceList.xsl");
		
		$this->Controller->setView($View);
		
		$this->Controller->getTemplate()->setPageTitle(
				$this->Controller->locale("ROOT_ADMIN_MENU") .
				$this->Controller->locale("ROOT_TITLE_SEPARATOR") .
				$this->Controller->locale("REF_REFERENCES")
			);
	}
	
	public function ajaxAction()
	{
		$this->insert();
		
		$List = $this->refList();
		
		$View = new REF_ReferencesListXml(MSF_Registry::get("MSF_Template"), MSF_Registry::get("MSF_Locale"));
		
		$View->buildGroup($List->getGroup(true));
		$this->Controller->setView($View);
	}
	
	private function refList()
	{
		$ReferenceList = new REF_ReferencesList(new REF_ReferencesListDB(MSF_Registry::get("MSF_DB")));

		$ReferenceList->addAction($this->Controller->locale("REF_OPEN"), "/admin/references/list/edit", "/templates/default/images/folder_page_white.png",  array("reference_id"), false);
		//$ReferenceList->addAction($this->Controller->locale("REF_ACCESS"), "/admin/references/list/access", "/templates/default/images/key.png",  array("reference_id"), false);
		$ReferenceList->addAction($this->Controller->locale("REF_DELETE"), "/admin/references/list/remove", "/templates/default/images/red_cross.png",  array("reference_id"), true);
		$ReferenceList->setLink("/admin/references/list/edit", array("reference_id"));
		
		$ReferenceList->setBaseUrl("/admin/references/list/?");
		$ReferenceList->setStateParams(
			array(
				"module"		=> "admin_references_references",
				"action"		=> "index",
				"action_id"		=> "0",
				"structure_id"	=> MSF_Registry::get("MSF_UserBox")->getUserAttribute("id")
			)
		);
		
		$AB	= MSF_Registry::get("MSF_ActionBar");
		$AB->addAction(
			$this->Controller->locale("REF_ADD"),
			"/admin/references/list/add/",
			""
		);
		
		return $ReferenceList;
	}
	
	public function editAction()
	{
		$this->setReference();

		if(IS_REQUEST) {
			//$aColumns	= $this->prepareFields();
			
			$sName 		= trim(MSF_SL::getParam($_POST, "refName", ""));
			$sDesc		= trim(MSF_SL::getParam($_POST, "refDescription", ""));
			$sFullDesc	= trim(MSF_SL::getParam($_POST, "refFullDescription", ""));
			$iGroupId	= (int)MSF_SL::getParam($_POST, "refGroupId", 0);
						
			$sTemplate	= $this->Reference->getTemplate();
				
			$aColumns			= array();
			$aColumns["config"]	= $this->Reference->getFieldsXml();
			$aColumns["add"]	= array();
			$aColumns["remove"]	= array();
			
			
			/*
			if($sTemplate == "") {
				$sTemplate	= $this->prepareDefaultTemplate($aColumns["config"]);
			}
			*/
			$this->Reference->saveReference($aColumns, $sName, $sTemplate, $sDesc, $sFullDesc, $iGroupId);

			$sBackUrl	= "/admin/references/list/edit/" . $this->iReferenceId;
			MSF_Registry::get("MSF_Messenger")->redirect($sBackUrl, MSF_Registry::get("MSF_Locale")->getWord("REFERENCE_SAVE"));
		}


		
		$Groups			= new REF_ReferenceGroupsList();
		
		$this->View->setGroupsList($Groups->getList());
		$this->View->editMain();
		$this->View->setReferenceName($this->Reference->getName());
		$this->View->setReferenceDescription($this->Reference->getDescription());
		$this->View->setReferenceFullDescription(stripslashes($this->Reference->getFullDescription()));
		$this->View->setGroupId($this->Reference->getGroupId());
		
		MSF_RteObjectsIntegrator::files($this->iReferenceId, "references_back");
		$this->buildActionPanel();
		$this->setPageTitle();
	}
	
	public function fieldsAction()
	{		
		$this->setReference();

		if(IS_REQUEST) {
			$aColumns	= $this->prepareFields();			
			if(!empty($aColumns["remove"])) {
				$this->Reference->clearState();
			}
			
			$sName 		= $this->Reference->getName();
			$sDesc		= $this->Reference->getDescription();
			$iGroupId	= $this->Reference->getGroupId();
			$sFullDesc	= $this->Reference->getFullDescription();
			
			$sTemplate	= $this->Reference->getTemplate();
			
			$this->Reference->saveReference($aColumns, $sName, $sTemplate, $sDesc, $sFullDesc, $iGroupId);

			$sBackUrl	= "/admin/references/list/fields/" . $this->iReferenceId;
			MSF_Registry::get("MSF_Messenger")->redirect($sBackUrl, MSF_Registry::get("MSF_Locale")->getWord("WORKFLOW_FIELDS_SAVED"));
		}

		$ReferenceList	= new REF_ReferencesList(new REF_ReferencesListDB(MSF_Registry::get("MSF_DB")));
		$DoctypeList = new WF_DoctypesList(new WF_DoctypesListDB(MSF_Registry::get("MSF_DB")));		
		
		$this->View->setDoctypesList($DoctypeList->getAll());
		$this->View->setReferencesList($ReferenceList->getList());
		$this->View->editFields($this->Reference->getFieldsXml());
		MSF_RteObjectsIntegrator::files($this->iReferenceId, "references_back");
		$this->buildActionPanel();
		$this->setPageTitle();

	}

	public function addAction()
	{
		$View		= new REF_ReferenceXml(MSF_Registry::get("MSF_Template"), MSF_Registry::get("MSF_Locale"));
		$Reference	= new UserReference(new UserReferenceDB(MSF_Registry::get("MSF_DB")));
		$this->Controller->setView($View);
		$View->add();
		//$View->editMain();
		
		if(IS_REQUEST) {
			$sName = MSF_SL::getParam($_POST, "referenceName");
			$sDesc = MSF_SL::getParam($_POST, "referenceDescription");
						
			if($sName == ""){
				$oMessenger		= MSF_Registry::get("MSF_Messenger");
				$oMessenger->addError(MSF_Registry::get("MSF_Locale")->getWord("REFERENCE_ENTER_NAME"));
			}
			else {
				if ($Reference->nameExists($sName)) {
					MSF_Registry::get("MSF_Messenger")->addError(
						MSF_Registry::get("MSF_Locale")->getWord("REFERENCE_MESSAGE_NAME_DULICATION")
					);
						
				}
				
				$iRefId = $Reference->create($sName, $sDesc);
				$sUrl	= "/admin/references/list/edit/" . $iRefId . "/";
				MSF_Registry::get("MSF_Messenger")->redirect($sUrl, MSF_Registry::get("MSF_Locale")->getWord("REFERENCE_CREATED"));
				exit();
			}
			
		}
		
		$View->addTemplate("/modules/references/view/templates/default/add.xsl");
		$AB	= MSF_Registry::get("MSF_ActionBar");
		$AB->addAction(
			$this->Controller->locale("REF_CLOSE"),
			"/admin/references/list/",
			""
		);
		
		$this->Controller->getTemplate()->setPageTitle(
				$this->Controller->locale("ROOT_ADMIN_MENU") .
				$this->Controller->locale("ROOT_TITLE_SEPARATOR") .
				$this->Controller->locale("REF_REFERENCES")
			);
	}

	public function templateAction()
	{
		$this->setReference();
		
		
		if(IS_REQUEST) {
			$aSave				= array();
			$aSave["config"]	= $this->Reference->getFieldsXml();
			$aSave["add"]		= array();
			$aSave["remove"]	= array();
			
			
			$sTemplateXml	= MSF_SL::getParam($_POST, "template", "<div></div>");
			$this->Reference->saveReference($aSave, $this->Reference->getName(), $sTemplateXml, $this->Reference->getDescription(), $this->Reference->getFullDescription(), $this->Reference->getGroupId());
			
			$sBackUrl	= "/admin/references/list/template/" . $this->iReferenceId;
			MSF_Registry::get("MSF_Messenger")->redirect($sBackUrl, MSF_Registry::get("MSF_Locale")->getWord("WORKFLOW_TEMPLATE_SAVED"));
		}
		
		$this->View	= new REF_TemplateXml();
		$sTemplateXml = MSF_String::stripSlashes($this->Reference->getTemplate());
		//$sTemplateXml = str_Replace("&nbsp;", "&amp;nbsp;", $sTemplateXml);
		$this->View->view($sTemplateXml,$this->getTemplateFields());
		
	
		MSF_RteObjectsIntegrator::files($this->iReferenceId, "references_back");
		
		$this->buildActionPanel();
		$this->setPageTitle();
	}
	
	public function accessAction()
	{
		$iId		= $this->Controller->getIntParam(1);
		
		$View		= new REF_ReferenceXml(MSF_Registry::get("MSF_Template"), MSF_Registry::get("MSF_Locale"));
		$Manager	= new REF_ReferencesSecurityManager(MSF_Registry::get("MSF_DB"), $iId);
		
		$this->setReference();
		
		if(IS_REQUEST) {
			$aAccess	= (array) MSF_SL::getParam($_POST, "access", array());
			
			$Manager->edit($aAccess);
			$sBackUrl	= "/admin/references/list/access/" . $iId;
			MSF_Registry::get("MSF_Messenger")->redirect($sBackUrl, MSF_Registry::get("MSF_Locale")->getWord("WORKFLOW_ACCESS_SAVED"));
		}
		
		$View->changeAccess($Manager->get());
		$this->Controller->setView($View);
		
		/*
		$AB	= MSF_Registry::get("MSF_ActionBar");
		$AB->addAction(
			$this->Controller->locale("REF_ACCESS_ADD"),
			"#"
		);
		$AB->addAction(
			$this->Controller->locale("REF_CLOSE"),
			"/admin/references/list/edit/" . $iId,
			""
		);
		*/
		$this->buildActionPanel();
		
		$this->setPageTitle();
	}
	
	public function removeAction()
	{
		$this->setReference();
		
		$sBackUrl	= "/admin/references/list/";
		if($this->Reference->isSystem()) {
			MSF_Registry::get("MSF_Messenger")->redirect($sBackUrl, MSF_Registry::get("MSF_Locale")->getWord("REFERENCE_DELETE_SYSTEM"), true);
		}
		else {
			$this->Reference->remove();
			MSF_Registry::get("MSF_Messenger")->redirect($sBackUrl, MSF_Registry::get("MSF_Locale")->getWord("REFERENCE_DELETED"));
		}
				
	}

	public function jsonfieldsAction()
	{
		$this->iReferenceId		= (int)$this->Controller->getParam(1);
		$this->Reference	= new UserReference(new UserReferenceDB(MSF_Registry::get("MSF_DB")));
		$this->Reference->load($this->iReferenceId, array());
		$aFields	= $this->Reference->getFields();
		$aReturn	= array();
		
		
		foreach($aFields as $aField) {
			$aReturn[] = $aField["title"];
		}
		
		$this->View	= new MSF_ViewSupportJson();
		$this->View->set("fields", $aReturn);
		
		$this->Controller->setView($this->View);
	}
	
	public function jsonsysfieldsAction()
	{
		$oSystemFieldsSettings = new REF_SystemFieldsSettings();
		$aReturn = $oSystemFieldsSettings->getSysFieldsTitles();
		
		$this->View	= new MSF_ViewSupportJson();
		$this->View->set("fields", $aReturn);
		
		$this->Controller->setView($this->View);
	}
	
	private function setReference()
	{
		$this->iReferenceId		= (int)$this->Controller->getParam(1);

		$this->View				= new REF_ReferenceXml(MSF_Registry::get("MSF_Template"), MSF_Registry::get("MSF_Locale"));

		$this->View->setReferenceId($this->iReferenceId);
		$this->Controller->setView($this->View);

		$this->Reference	= new UserReference(new UserReferenceDB(MSF_Registry::get("MSF_DB")));
		$this->Reference->load($this->iReferenceId, array());
	}

	private function prepareFields()
	{
		$aPostColumns	= MSF_SL::getParam($_POST, "fields", array());		
		$aOldColumns	= $this->Reference->getFields();
		$aToDelete		= array();
		$aToAdd			= array();
		$aOldColumns 	= $this->Reference->getFields();
		$XmlWrapper		= $this->buildWrapper();

		
		
		//proveryaem, chto tam so starimi stolbcami
		foreach($aOldColumns as $aOldColumn) {
			$flag 		= false;
			$aNewColumn = array();

			if($aOldColumn["name"] == "id") {
				$aNewColumn = $aOldColumn;
				$flag = true;
			}
			elseif(isset($aOldColumn["blocked"]) && $aOldColumn["blocked"] == 1) {
				$flag = true;
			}
			else {
				foreach($aPostColumns as $sPostColumnName=>$sPostColumnValue){
					if($aOldColumn["name"] == $sPostColumnName) {
						$flag = true;
					}

				}

			}

			if(!$flag) {
				$aToDelete[]	= $aOldColumn["name"];
			}

		}
		//debug($_POST);
		//a teper dobavlyaem novie i sohranyaem starie
		
		$aSearch = array("<", ">", "\"");
		
		foreach($aPostColumns as $sPostColumnName=>$aPostColumnValue) {

			$sTitle = str_replace($aSearch, "", $aPostColumnValue["title"]);
			
			if(strpos($sPostColumnName, "new") !== false) {
				$aValue				= (array)MSF_SL::getParam($aPostColumnValue, "value", array());
				$sPostColumnName	= "name_" . MSF_String::hash(microtime(), 10);
				$aTmp				= array();
				$aTmp["name"]		= $sPostColumnName;
				$aTmp["type"]		= MSF_SL::getParam($aValue, "type", "int");
				$aTmp["values"] 	= array();

				
				if($aTmp["type"] == "notChoose" || (
					$aTmp["type"] == "reference" && $aPostColumnValue["value"]["optional"]["reference"] == "-1"
				)) {
					continue;
				}
				else {
					$aToAdd[]	= $aTmp;
					$XmlWrapper->set($sPostColumnName, $aPostColumnValue["value"], $sTitle);
				}
				/*
				if($aTmp["type"] != "notChoose") {
					
				}*/
			}
			else {
				$XmlWrapper->set($sPostColumnName, $aPostColumnValue["value"], $sTitle);
			}
			
		}
		
		
		//die();

		$sXML	= str_replace("<?xml version=\"1.0\"?>", "", $XmlWrapper->getXml());
				
		$aReturn			= array();
		$aReturn["config"]	= $sXML;
		$aReturn["add"]		= $aToAdd;
		$aReturn["remove"]	= $aToDelete;
		//die("ololo");
		return $aReturn;
	}
	
	private function prepareDefaultTemplate($sFields)
	{
		$FieldsXML	= new MSF_XmlWrapper($sFields);
		
		$sTemplate	= "<table  class=\"form\">";
		foreach($FieldsXML as $Field) {
			if($Field->getTitle() != "ID") {
				$sTemplate .= "<tr>";
				$sTemplate .= "<td class=\"form-row-title\">" . $Field->getTitle() . "</td>";
				$sTemplate .= "<td class=\"form-row-value\">{" . $Field->getTitle() . "}</td>";
				$sTemplate .= "</tr>";
			}
			
		}
		
		$sTemplate .= "</table>";
		return $sTemplate;
	}
	
	private function buildWrapper()
	{
		$XmlWrapper	= new MSF_XmlWrapper();

		$aIdConf					= array();
		$aIdConf["type"]			= "int";
		$aIdConf["required"]		= 1;
		$aIdConf["defaultValue"]	= 0;
		$aIdConf["group"] 			= 0;
		$aIdConf["sort"] 			= 1;
		$aIdConf["filter"] 			= 1;

		$XmlWrapper->set("id", $aIdConf, "ID");

		return $XmlWrapper;
	}

	private function getTemplateFields()
	{
		$aFields = array();
		foreach (new MSF_XmlWrapper($this->Reference->getFieldsXml()) AS $Data) {
			if($Data->getTitle() != "ID") {
				$aFields[] = array(
					"title" 	=> $Data->getTitle(),
					"tag" 		=> "{" . $Data->getTitle() . "}"
					);
			}
			
		}
		
		return MSF_Json::encode($aFields);
	}
	
	private function buildActionPanel()
	{
		$AB	= MSF_Registry::get("MSF_ActionBar");
		
		$AB->addAction(
			$this->Controller->locale("REF_MAIN"),
			"/admin/references/list/edit/" . $this->iReferenceId . "/",
			""
		);
		
		$AB->addAction(
			$this->Controller->locale("REF_FIELDS"),
			"/admin/references/list/fields/" . $this->iReferenceId . "/",
			""
		);
		
		$AB->addAction(
			$this->Controller->locale("REF_TEMPLATE"),
			"/admin/references/list/template/" . $this->iReferenceId . "/",
			""
		);
		
		$AB->addAction(
			$this->Controller->locale("REF_ACCESS"),
			"/admin/references/list/access/" . $this->iReferenceId . "/",
			""
		);
		
		$AB->addAction(
			$this->Controller->locale("REF_CLOSE"),
			"/admin/references/list/",
			""
		);
		
		$AB->addAction(
			$this->Controller->locale("WORKFLOW_ACCESS_CAN_VIEW"),
			"/references/add/" . $this->iReferenceId . "/",
			"",
			"",
			false,
			"right"
		);
	}

	private function setPageTitle()
	{
		$sGroup = "";
		
		if ($this->Reference->getGroupId() > 0) {
			$Group 	= new REF_ReferenceGroup(new REF_ReferenceGroupDB($this->DB()));
			$Group->load($this->Reference->getGroupId());
			$aGroup = $Group->getConfig();
			
			if (!empty($aGroup["name"])) {
				$sGroup = $aGroup["name"] . $this->Controller->locale("ROOT_TITLE_SEPARATOR");
			}
		}
		
		$this->Controller->getTemplate()->setPageTitle(
				$this->Controller->locale("ROOT_ADMIN_MENU") .
				$this->Controller->locale("ROOT_TITLE_SEPARATOR") .
				$this->Controller->locale("REF_REFERENCES") .
				$this->Controller->locale("ROOT_TITLE_SEPARATOR") .
				MSF_Registry::get("MSF_Locale")->getI18N($sGroup) .
				MSF_Registry::get("MSF_Locale")->getI18N($this->Reference->getName())
			);
	}
}
?>