<?php
include_once "library/MSF_SafeText/MSF_SafeText.php";
require_once "modules/media/library/MD_Media.php";
require_once "library/MSF_EnumValue/MSF_EnumValue.php";
include_once "modules/workflow/library/actions/WF_DocFacade.php";
include_once "RefFieldHelper.php";



class REF_Fields
{
	const COLUMN_DELIMITER	  = "/^/col/^/";
	
	const INTEGER 			  = "int";

	const STRING 			  = "string";

	const FLOAT 			  = "float";
	
	const TEXT	 			  = "text";
	
	const REFERENCE	 		  = "reference";
	
	const DATE				  = "date";
	
	const TIMESTAMP			  = "timestamp";
	
	const BOOLEAN			  = "bool";
	
	const STRUCTURE			  = "structure";
	
	const ENUM				  = "enum";
	
	const PICTURE			  = "picture";
	
	const FILE				  = "file";
	
	const DOCUMENT    	      = "document";
	
	const PASSWORD    		  = "password";
	
	const TABLE    			  = "table";
	
	private $sXml			  = "";
		
	private $aFields 		  = array();
	
	private $aUnfilled 		  = array();
	
	private $aIncorrect		  = array();
	
	private $aIncorrectUnique = array();

	private $aChecks		  = array();
	
	private $iExtraId		  = 0;
	
	private $sExtraName		  = "trash";
	
	function __construct($sXml)
	{
		$this->sXml = $sXml;
		$this->init();
	}
	
	public function setExtra($sName, $iId)
	{
		$this->iExtraId 	= $iId;
		$this->sExtraName 	= $sName;
	}
	
	public function reset()
	{
		$this->init();
	}
	
	public function registerCheck($sType, $mCallback, $aArg)
	{
		$this->aChecks[$sType]	= array("callback"=>$mCallback, "args"=>$aArg);
	}
	
	private function init()
	{
		foreach (new MSF_XmlWrapper($this->sXml) AS $Data) {
			$this->aFields[$Data->getName()] = array (
				"title" 		=> $Data->getTitle(),
				"type" 			=> $Data->getValue("type"),
				"required"		=> (boolean) $Data->getValue("required"),
				"intitle"		=> (boolean) $Data->getValue("intitle"),
				"unique"		=> (boolean) $Data->getValue("unique"),
				"description"	=> $Data->getValue("description"),
				"value"			=> $Data->getValue("defaultValue"),
				"default" 		=> $Data->getValue("defaultValue"),
				"optional" 		=> $Data->getValue("optional")
			);
					
		}
	}
	
	public function getUnfilled()
	{
		return $this->aUnfilled;
	}
	
	public function getIncorrect()
	{
		return $this->aIncorrect;
	}
	
	public function getIncorrectUnique()
	{
		return $this->aIncorrectUnique;
	}
	
	public function getCount()
	{
		return count($this->aFields);
	}
	
	public function getField($sName) {
		if ($this->exists($sName)) {
			return $this->aFields[$sName];
		}
		else {
			return array();
		}
	}
	
	public function getFieldsFull()
	{
		return $this->aFields;
	}
	
	public function getAsList()
	{
		$i 			= 0;
		$aResult 	= array();
		foreach ($this->aFields AS $sName => $aValue) {
			$aResult[$i] = array(
					"name"			=> $sName,
					"type"			=> $aValue["type"],
					"value" 		=> $aValue["value"],
					"title"			=> $aValue["title"],
					"intitle"		=> (isset($aValue["intitle"])) ? $aValue["intitle"] : "",
					"description"	=> $aValue["description"],
					"tag" 			=> "{" . $aValue["title"] . "}",
					"optional"		=> $aValue["optional"]
					);
			$i++;
					
		}
		
		return $aResult;
	}
	
	public function getFieldType($sName) {
		if	(isset($this->aFields[$sName])) {
			return $this->aFields[$sName]["type"];
		}
	}
	
	public function getRequiredFieldsList()
	{
		$aResult = array();
		foreach ($this->aFields AS $sName => $aBody) {
			if ($aBody["required"]) {
				$aResult[] = array(
						"title" => $aBody["title"],
						"name"	=> $sName
						);
			}
						
		}
		
		return $aResult;
	}
	
	public function getAsHash()
	{
		$aResult 	= array();
		foreach ($this->aFields AS $sName => $aValue) {
			$aResult[$sName] = $aValue["value"];
		}

		return $aResult;
	}
	
	public function exists($sName)
	{
		return array_key_exists($sName, $this->aFields);
	}
	
	public function set($sName, $mValue)
	{
		if (!$this->exists($sName)) {
			return;
		}
		
		if (!is_array($mValue) && !is_object($mValue)) {
			$mValue = trim($mValue);
		}
		
		$aFields 		=& $this->aFields[$sName];

		$isRequired		= isset($aFields["required"]) ? (bool) $aFields["required"] : false;
		$isBinary		= ($aFields["type"] == "file") || ($aFields["type"] == "picture");
		$isTable		= ($aFields["type"] == "table");
		if ($isBinary && is_array($mValue) && isset($mValue["name"][0])) {
			$isEmpty = empty($mValue["name"][0]);
		}
		else if ($isBinary && isset($_POST[$sName . "_exists"])) {
			$isEmpty = empty($_POST[$sName . "_exists"]);
			if (empty($mValue)) {
				$mValue  = implode(",", $_POST[$sName . "_exists"]);
			}
		}
		else if ($isTable) {
			$isEmpty	= true;
			foreach (explode("^R^", $mValue) AS $iKey => $sTabValue) {	
				if ($iKey > 1) {
					$sRealTabValue = str_replace("^C^", "", $sTabValue);
					if (mb_strlen($sRealTabValue) != 0) {
						$isEmpty = false;
					}
				}
			}
		}
		else {
			if (!is_array($mValue) && !is_object($mValue)) {
				$isEmpty = (mb_strlen($mValue) == 0) ? true : false;
			}
			else {
				$isEmpty = empty($mValue);
			}
		}
		
		if (!$isBinary && $isRequired && $isEmpty) {
			$this->addToUnfilled($sName);
			$aFields["value"] = $mValue;
		}
		else if ($isBinary && $isRequired && $isEmpty && !is_array($mValue)) {
			$this->addToUnfilled($sName);
			$aFields["value"] = $mValue;
		}
		else if ($isBinary && !$isRequired && !$isEmpty && !is_array($mValue)) {
			$this->removeFromUnfilled($sName);
			$aFields["value"] = $mValue;
		}
		else {
			$this->removeFromUnfilled($sName);
			$aFields["value"] = $this->getValue($sName, $aFields, $mValue);
			$this->checkForIncorrect($sName);
		}
		
	}
	
	public function injectFields($aFields)
	{
		foreach($aFields AS $sKey => $sValue) {
			$this->aFields[$sKey] = $sValue;
		}
	}
	
	private function checkForIncorrect($sName)
	{
		if(empty($this->addIncorrect)) {
			foreach($this->aChecks as $sCheckType=>$aTmp) {
				if(isset($this->aFields[$sName][$sCheckType]) && $this->aFields[$sName][$sCheckType]) {
					if(!call_user_func($aTmp["callback"], $aTmp["args"], $sName, $this->aFields[$sName]["value"], $this->aFields[$sName])) {
						if($sCheckType == "unique") {
							$this->addIncorrectUnique($sName, $this->aFields[$sName]["title"]);
						}
						else {
							$this->addIncorrect($sName, $this->aFields[$sName]["title"]);
						}
						
					}
					
				}
				
			}
			
		}
		
	}

	private function getValue($sName, $aFields, $mValue)
	{
		switch ($aFields["type"]) {
			case self::INTEGER:
				return $this->setInteger($sName, $aFields, $mValue);
				break;
				
			case self::FLOAT:
				return $this->setFloat($sName, $aFields, $mValue);
				break;
			
			case self::ENUM:
				return $this->setEnum($aFields, $mValue);
				break;
			
			case self::TABLE:
				return $this->setTable($sName, $aFields, $mValue);
				break;
				
			case self::TEXT:
			case self::STRING:
				return $this->setString($sName, $aFields, $mValue);
				break;

			case self::PASSWORD:
				return $this->setPassword($sName, $aFields, $mValue);
				break;
				
			case self::BOOLEAN:
				return $this->setBoolean($sName, $aFields, $mValue);
				break;
				
			case self::REFERENCE:
				return $this->setReference($sName, $aFields, $mValue);
				break;
				
			case self::STRUCTURE:
				return $this->setStructure($sName, $aFields, $mValue);
				break;

			case self::TIMESTAMP:
				return $this->setTimestamp($sName, $aFields, $mValue);
				break;
				
			case self::DATE:
				return $this->setDate($sName, $aFields, $mValue);
				break;
				
			case self::PICTURE:
				return $this->setPicture($sName, $aFields, $mValue, true);
				break;

			case self::FILE:
				return $this->setPicture( $sName, $aFields, $mValue);
				break;
				
			case self::DOCUMENT:
            	return $this->setDocument( $sName, $aFields, $mValue);
                break;
					
			default:
				return $aFields["default"];
				break;
		}
		
	}
	
	public function get($sName)
	{
		return $this->exists($sName) ? $this->setDate($sName, $this->aFields[$sName], $this->aFields[$sName]["value"]) : null;
	}
	
	public function validateReference($aField1)
	{
		try {
			$aField		= $aField1;
			$oRef		= new UserReference(new UserReferenceDB(MSF_Registry::get("MSF_DB")));
			$oRef->load($aField["optional"]["reference"]);
			$aColumns	= $oRef->getColumns();
			$aTypes = array();
			foreach($aColumns AS $aColumn) {
				$aTypes[$aColumn["name"]] = $aColumn["type"];	
			}
			
			if(isset($aField["optional"]["defaultFilter"]) && !empty($aField["optional"]["defaultFilter"])) {
				foreach($aField["optional"]["defaultFilter"] AS $sKey => $sValue) {
					if ($aField["optional"]["defaultFilter"][$sKey] != "") {
						switch($aTypes[$sKey]) {
							case "int":
								$aField["optional"]["defaultFilter"][$sKey]	= (int)$sValue;
								break;
							case "float":
								$aField["optional"]["defaultFilter"][$sKey]	= (float)$sValue;
								break;
							case "file":
							case "picture":
								$aField["optional"]["defaultFilter"][$sKey]	= "";
								break;	
						}
					}
				}
				
			}
		
			return $aField;
		}
		catch (Exception $e) {
			return $aField1;
		}
		
		
		
	}
	
	private function addIncorrect($sName, $sTitle)
	{
		$this->aIncorrect[] = array(
			"name" 	=> $sName,
			"title" => $sTitle
			);
	}
	
	private function addIncorrectUnique($sName, $sTitle)
	{
		$this->aIncorrectUnique[] = array(
			"name" 	=> $sName,
			"title" => $sTitle
		);
	}
	
	private function setInteger($sName, $aField, $mValue)
	{
		if(strlen($mValue) != 0 && !is_numeric($mValue)) {
			$this->addIncorrect($sName, $aField["title"]);	
		}		
		
		return $mValue;
	}
	
	private function setFloat($sName, $aField, $mValue)
	{
		$mValue = str_replace(",", ".", $mValue);

		if (strlen($mValue) != 0 && !is_numeric($mValue)) {
			$this->addIncorrect($sName, $aField["title"]);
		}
		
		return $mValue;
	}
	
	private function setString($sName, $aField, $mValue)
	{
		if (is_object($mValue)) {
			$sValue = $this->convertToString($mValue);
		}
		else {
			$sValue = (string) $mValue;
		}

		if (isset($aField["optional"]["viewRTE"]) && $aField["optional"]["viewRTE"] == 1) {
			$sValue = str_replace("\n", " ", $sValue);
		}
		if (isset($aField["optional"]["viewRTE"]) && $aField["optional"]["viewRTE"] == 0) {
			$sValue = str_replace("<br>", "\n", $sValue);
		}
		return $this->escapeTemplate($sValue);
	}
	
	private function convertToString($mValue)
	{
		$aValue = array();
		foreach ($mValue as $sKey => $sValue) {
        	if (is_array($sValue) || is_object($sValue)) {
            	$aValue[$sKey] = $this->convertToString($sValue);
			}
			else {
            	return $sValue;
        	}
	    }
	} 
	
	
	private function setTable($sName, $aField, $mValue)
	{
		if (isset($aField["optional"]["fields"])) {
			foreach ($aField["optional"]["fields"] AS $aFieldCol) {
				if ($this->getFieldType($aFieldCol["type"]) == "file" && strstr($mValue, "-f-" . $aFieldCol["index"] . "-f-")) {
					foreach ($_FILES AS $sKey => $aValue) {
						if (strstr($sKey, $aFieldCol["index"])) {
							$sVal = "";
							$aFieldOptions				= $this->aFields[$aFieldCol["type"]];
							$aFieldOptions["required"] 	= 0;
							$sFiles 					= $this->setPicture($sKey, $aFieldOptions, $aValue);
							if ($sFiles != "") {							
								$aNewFile = array();
								foreach ($aValue["name"] AS $sNameFile) {
									if ($sNameFile != "") {
										$aNewFile[] = $sNameFile;
									}
								}
								$bNotMulti 	= ($this->aFields[$aFieldCol["type"]]["optional"]["multiselect"] == 0 && count($aNewFile) != 0);
								$aExists 	= (isset($_POST[$sKey . "_exists_name"]) && !$bNotMulti)
												? $_POST[$sKey . "_exists_name"]
												: array();
								$aResult 	= array_merge($aExists, $aNewFile);
								$aFiles	 	= explode(",", $sFiles);
								$sVal 	 	= implode(",", $aResult) . ":.:" . $sFiles;
							}
							$mValue = preg_replace("/-f-" . $aFieldCol["index"] . "-f-/", $sVal, $mValue, 1);
						}
					}
				}
			}
		}
		
		return (string) $mValue;
	}
	
	private function escapeTemplate($sTemplate)
	{
		$sTemplate	= str_replace("\r", "", $sTemplate);
		$sPattern	= '/\<(.+)\>/i';
		$sResult	= preg_replace_callback(
				$sPattern,
       			create_function(
        			   	'$matches',
       					'$sReplaced = str_replace("%5C", "/", $matches[0]);' .
       					'return $sReplaced;'
       					),
       			$sTemplate
   				);

		$sResult = preg_replace('@<script[^>]*?>.*?</script>@si', ' ', $sResult);
		$sResult = preg_replace('@<style[^>]*?>.*?</style>@si', ' ' , $sResult);
		return $sResult;
	}
	
	private function setBoolean($sName, $aField, $mValue)
	{
		$aValue		= MSF_EnumValue::parse($mValue);
		$sReturn	= "";
		if($aValue) {
			$sReturn	= $mValue;
		}
		else {
			if($mValue == "1") {
				$sReturn	= MSF_EnumValue::format($aField["optional"]["labelYes"], 1);
			}
			elseif ($mValue == "0") {
				$sReturn	= MSF_EnumValue::format($aField["optional"]["labelNo"], 0);
			} else {
				$sReturn	= "";
			}
		}
			    
		return $sReturn;
	}
	
	private function setReference($sName, $aField, $mValue)
	{
		$RefValue = new REF_Value();
		$RefValue->parse($mValue);
		
		return $RefValue->format();
	}
	
	private function setStructure($sName, $aField, $mValue)
	{
		$oResValue 	= new REF_Value();
		$RefValue 	= new REF_Value();
		if (strlen($mValue) > 0) {
			$RefValue->parse($mValue);
		}
		
		$aValues	= $RefValue->getValuesList();
		foreach ($RefValue->getKeysList() AS $iId => $aKeys) {
			if (count($aKeys) == 3) {
				$sRes = RefFieldHelper::formatStructureValue(
						$aKeys[1], 
						$aKeys[2], 
						MSF_SL::getParam($aField["optional"], "employee", "name"), 
						MSF_SL::getParam($aField["optional"], "post", "post"), 
						MSF_SL::getParam($aField["optional"], "employeeEnable", 1), 
						MSF_SL::getParam($aField["optional"], "postEnable", 1)
						);
				$oResValue->add($aKeys, $sRes);
			}
			else if (count($aKeys) == 2) {
				$oResValue->add($aKeys, $aValues[$iId]);
			}

		}

		return $RefValue->format();
	}
	
	private function setDate($sName, $aField, $mValue)
	{
		return MSF_DATE::toDbFormat($mValue, $aField['type']);
	}
	
	private function setTimestamp($sName, $aField, $mValue)
	{
		return MSF_DATE::toDbFormat($mValue, $aField['type']);
	}

	private function setPicture($sName, $aField, $mValue, $checkPicture	= false)
	{
		$aIds		= array();
		$iCount		= 0;
		if (is_array($mValue)) {
			$aOldIds	= array();
			$oMedia     = new MD_Media(new MD_MediaDB(MSF_Registry::get("MSF_DB")));
			$oMedia->setUploadDirectory(CNM_MEDIA_UPLOAD_DIRECTORY);
			$oMedia->setMaxFileSize(CNM_MEDIA_MAX_FILE_SIZE);
			if(isset($_POST[$sName . "_exists"])) {
				foreach($_POST[$sName . "_exists"] as $iKey => $iFileId) {
					array_push($aOldIds, $iFileId);
					$aInfo = $oMedia->getFileInfo($iFileId);
					$_POST[$sName . "_exists_name"][$iKey] = $aInfo["name"];
				}
				
			}
			
			if (count($mValue) == 1) {
				$sReturn = implode(',', $aOldIds);
			}
			else {
				if($aField["required"] == 1) {
					$bValidFlag	= false;
					if(count($aOldIds) > 0) {
						$bValidFlag	= true;
					}
					
					foreach($mValue["error"] as $iErrorNum) {
						if($iErrorNum == 0) {
							$bValidFlag	= true;
						}
						
					}
					
					if(!$bValidFlag) {
						$this->addToUnfilled($sName);
					}
					
				}
	
				$iFilled = 0;
				for($i = 0; $i < count($mValue["name"]); $i++) {
					if (empty($mValue["name"][$i]) && !isset($_POST[$sName . "_exists"])) {
						// stub
					}
					else if ($mValue["size"][$i] <= CNM_MEDIA_MAX_FILE_SIZE) {
						$iFilled++;
						if($checkPicture) {
							$aAllowedImage		= array("image/gif", "image/jpeg", "image/png", "image/bmp", "image/x-png", "image/pjpeg");
							if(!empty($mValue["name"][$i]) && !in_array($mValue["type"][$i], $aAllowedImage)) {
								$this->addIncorrect($sName, $aField["title"]);
								return 0;
							}
							
						}
						
						$aToMedia				= array();
						$aToMedia["name"]		= $mValue["name"][$i];
						$aToMedia["type"]		= $mValue["type"][$i];
						$aToMedia["tmp_name"]	= $mValue["tmp_name"][$i];
						$aToMedia["error"]		= $mValue["error"][$i];
						$aToMedia["size"]		= $mValue["size"][$i];
						try {
							$aIds[] = (int) $oMedia->upload($aToMedia, $this->sExtraName, $this->iExtraId);
						}
						catch (Exception $oE) {
							MSF_Logger::logTrace($oE);
						}
						
					}
					
				}

				if ($aField["required"] && !count($iFilled)) {
					$this->addToUnfilled($sName);
				}
				
				//$this->removeFromUnfilled($sName);
				if($aField["optional"]["multiselect"] == 1) {
					$sReturn	= implode(",", array_merge($aOldIds, $aIds));
				}
				else {
					if(count($aIds) > 0) {
						$sReturn	= $aIds[0];
					}
					else {
						if(isset($aOldIds[0])) {
							$sReturn	= $aOldIds[0];
						}
						else {
							$sReturn	= "";
						}
						
					}
					
				}	
			}
		
		}
		else {
			$sReturn = $mValue;
		}
		
		return $sReturn;
	}
	
	private function setDocument($sName, $aField, $mValue)
    {
    	if (is_array($mValue)) {
    		$RefValue 	= new REF_Value();
    		$NRefValue 	= new REF_Value();
    		$sTmp 		= "";
    		$iSize 		= count($mValue);
    		$aTmp 		= array();
    		$iCounter 	= 0;
    		for ($i = 0; $i < $iSize; $i++) {
    			if (!empty($mValue[$i])) {
    				$RefValue->parse($mValue[$i]);
    				$aTmp[$iCounter]["values"] 	= $RefValue->getValuesList();
    				$aTmp[$iCounter]["keys"] 	= $RefValue->getKeysList();
    				$iCounter++;
    			}
    		}
    		
    		for ($i = 0; $i < count($aTmp); $i++) {
    			for ($j = 0; $j < count($aTmp[$i]["values"]); $j++) {
    				$NRefValue->add($aTmp[$i]["keys"][$j], $aTmp[$i]["values"][$j]);
    			}
    		}
    		
    		$mValue = $NRefValue->format();
    	}
    	
		$sValue = (string) $mValue;
		
		return $sValue;
    }
    
    private function setPassword($sName, $aField, $mValue)
    {
    	if(is_array($mValue)) {
    		if (isset($aField["required"]) && $aField["required"] == 1 && strlen($mValue["main"]) == 0) {
    			$this->addToUnfilled($sName);
    			return "";
    		}
    		if($mValue["main"] != $mValue["confirm"]) {
    			$this->addIncorrect($sName, $aField["title"]);
    			return "";
    		}
    		else {
    		    if (!empty($mValue["main"]) && $mValue["main"] != ":!" && strlen($mValue["main"]) < 3) {
	    			$this->addIncorrect($sName, $aField["title"]);
	    			return "";
    			}
    			if((string)$mValue["main"] == "") {
    				return "";
    			}
    			else {
    				return MSF_String::encrypt((string)$mValue["main"]);
    			}
    			
    		}
    		  			
    	}
    	else {
    		return (string)$mValue;
    	}
    	
    }

    
    private function setEnum($aField, $mValue)
    {
    	if (!is_array($mValue)) {
	    	if(isset($aField["optional"]["enum"][$mValue])) {
	    		return MSF_EnumValue::format($aField["optional"]["enum"][$mValue], $mValue);
	    	}
	    	else {
	    		return $mValue;
	    	}
    	}
    	else {
    		if (empty($mValue)) {
    			return $mValue;
    		}
    		else {
    			if (isset($aField["optional"]["enum"])) {
    				$sValue = "";
	    			foreach ($mValue as $mKey) {
	    				if ($mKey>=0) {
		    				if (strlen($sValue)) $sValue .= self::COLUMN_DELIMITER;
		    				$sValue .= MSF_EnumValue::format($aField["optional"]["enum"][$mKey], $mKey);
	    				}
	    			}
    				return $sValue;
    			}
    			else {
    				return $mValue;
    			}
    		}
    	}
    }

	private function addToUnfilled($sName)
	{
		$bFlag	= true;
		foreach($this->aUnfilled as $aTmpField) {
			if($aTmpField["name"] == $sName) {
				$bFlag	= false;
			}
		}
		if($bFlag) {
			$this->aUnfilled[] = array(
				"name"	=> $sName,
				"title" => $this->aFields[$sName]["title"]
			);
			
		}
	}
	
	private function removeFromUnfilled($sName)
	{
		$aNewUnfilled = array();
		foreach($this->aUnfilled as $aTmpField) {
			if($aTmpField["name"] !== $sName) {
				$aNewUnfilled[] = $aTmpField;
			}
			
		}
		
		$this->aUnfilled = $aNewUnfilled;
	}
	
}

?>