<?php
if(!defined("__MSF")) {
	die("Direct access is not allowed");
}
require_once "system/mvc/MSF_ModelDBSupport.php";
include_once "library/MSF_Recordset/MSF_Recordset.php";
include_once "modules/references/view/xml/REF_ReferenceGroupsListXml.php";

class REF_ReferenceGroupsList
{
	/**
	 * @var MSF_Recordset
	 */
	private $Recordset	= null;

	/**
	 * Deistviya dlya recordseta
	 *
	 * @var array
	 */
	private $aActions	= array();

	/**
	 * @return void
	 * @access public
	 */
	public function __construct()
	{
		$this->Recordset	= new MSF_Recordset(MSF_Registry::get("MSF_DB"));
	}
	
	/**
	 * Vozvrashaet recordset gruppi
	 *
	 * @return MSF_Recordset
	 * @access public
	 */
	public function getListRecordset()
	{
 		$this->Recordset->setColumns($this->getColumns());
 		$this->Recordset->setTables(array("ref_references_groups"));
 		$this->Recordset->setActions($this->aActions);
 		
		return $this->Recordset->getAllRecords();
	}

	/**
	 * Vozvrashaet spisiok grupp
	 *
	 * @return array
	 * @access public
	 */
	public function getList()
	{
		$sSQL	= 	"SELECT" . "\n" .
						"group_id, name, description" . "\n" .
					"FROM" . "\n" .
						"ref_references_groups" . "\n" .
					"ORDER BY" . "\n" .
						"name";

		return MSF_Registry::get("MSF_DB")->select($sSQL);
	}

	/**
	 * Dobavlyaet deistvie v recordset
	 *
	 * @param string $sTitle
	 * @param string $sLink
	 * @param string $sImage
	 * @param array $aFields
	 * @param bool $bConfrm
	 * @return void
	 * @access public
	 */
	public function addAction($sTitle, $sLink, $sImage, $aFields, $bConfrm = false)
	{
		$this->aActions[]	= array(
				"title" 	=> $sTitle,
				"href" 		=> $sLink,
				"image" 	=> $sImage,
				"bind" 		=> $aFields,
				"confirm" 	=> $bConfrm
			);
	}

	/**
	 * ustanavlivae basoviy url v recordset
	 *
	 * @param string $sUrl
	 * @return void
	 * @access public
	 */
	public function setBaseUrl($sUrl)
	{
		$this->Recordset->setBaseURL($sUrl);
	}

	/**
	 * ustanavlivae rpc url v recordset
	 *
	 * @param string $sUrl
	 * @return void
	 * @access public
	 */
	public function setRpcUrl($sUrl)
	{
		$this->Recordset->setRpcURL($sUrl);
	}
	
	public function setLink($sHref, $aBind)
	{
		$this->Recordset->setLink(
			array(
				"href"		=> $sHref,
				"bind"		=> $aBind
			)
		);
	}

	public function setStateParams($aParams)
	{
		$this->Recordset->setStateParams(array(
			"module"		=> $aParams["module"],
			"action"		=> $aParams["action"],
			"action_id"		=> $aParams["action_id"],
			"structure_id"	=> $aParams["structure_id"]
		));
	}
	
	/**
	 * vozvrashaet stolbci dlya recordseta
	 *
	 * @return array
	 * @access private
	 */
	private function getColumns()
	{
		$aTypes		= array("int", "string", "string", "date", "timestamp");
 		$aTitles 	= array("ID", MSF_Registry::get("MSF_Locale")->getWord("REFERENCE_GROUP_NAME"), MSF_Registry::get("MSF_Locale")->getWord("REFERENCE_GROUP_DESCRIPTION"), MSF_Registry::get("MSF_Locale")->getWord("REFERENCE_GROUP_CREATED"), MSF_Registry::get("MSF_Locale")->getWord("REFERENCE_GROUP_CHANGED"));
 		$aColumns 	= array("group_id", "name", "description", "created_at", "modified_at");
 		$aRColumns 	= array();

 		for ($i = 0; $i < sizeof($aTitles); $i++) {
	 		$aRColumns[$i]["name"] 		= $aColumns[$i];
	 		$aRColumns[$i]["title"] 	= $aTitles[$i];
			$aRColumns[$i]["type"] 		= $aTypes[$i];
	 		$aRColumns[$i]["sort"] 		= true;
	 		$aRColumns[$i]["filter"] 	= true;
	 		$aRColumns[$i]["group"] 	= false;
	 		$aRColumns[$i]["compare"] 	= "like";
	 		$aRColumns[$i]["visible"] 	= ($aColumns[$i] == "group_id") ? false : true;
 		}

 		return $aRColumns;
	}
}
?>