<?php

class REF_SystemFieldsSettings {
	
	private $aFields = array(
		array (
				"name"	=> "author_name",
				"type"	=> "structure"
		),
		array (
				"name"	=> "editor_name",
				"type"	=> "structure"
		),
		array(
				"name"	=> "created_at",
				"type"	=> "timestamp"
		),
		array(
				"name"	=> "modified_at",
				"type"	=> "timestamp"
		)		
	);
	
	public static function getSqlType($sType)
	{
		switch ($sType) {
	 		case "int"			:
	 			$sSql = "bigint";
	 			break;
	 		case "date"			:
	 			$sSql = "date";
	 			break;
	 		case "timestamp"	:
	 			$sSql = "timestamp";
	 			break;
	 		case "time"			:
	 			$sSql = "time";
	 			break;
	 		case "float"		:
	 			$sSql = "double precision";
	 			break;
	 		case "table"		:
	 		case "picture"		:
			case "file"			:
	 		case "string"		:
	 		case "reference"	:
	 		case "structure"	:
	 		case "document"		:
	 		case "text"			:
	 		case "bool"			:
	 		case "enum"			:
	 			$sSql = "text";
	 			break;
		}
		
		return $sSql;
	}
	
	public function getSystemFields()
	{
		return $this->aFields;
	}
	
	public function getFieldsAndTypesAsString()
	{
		$sFields = "";
		if(isset($this->aFields[0])) {
			for($i = 0; $i < count($this->aFields); $i++) {
				$sFields .= $this->aFields[$i]["name"]." ".$this->getSqlType($this->aFields[$i]["type"]).", ";
 			}
			
		}
		
		return $sFields;
	}
	
	public function getRefSystemFields()
	{
		$RefFields = array();
		for($i = 0; $i < count($this->aFields); $i++) {
			$sKey = $this->aFields[$i]["name"];
			$RefFields[$sKey] = array(
				"title" 	  => $this->getTitle($this->aFields[$i]["name"]),
				"type"		  => $this->aFields[$i]["type"],
				"required"	  => "",
				"unique"	  => "",
				"description" => "",
				"value"		  => "",
				"default"	  => "",
				"optional"	  => ""
			);
			
			if($this->aFields[$i]["type"] == "structure") {
				$RefFields[$sKey]["optional"] = array(
					"postEnable" 	=> 1,
					"selectAuthor"  => 0,
					"multiselect"	=> 1);
			}
			
		}
		
		return $RefFields;
	}
	
	public function getSystemFieldsNames()
	{
		$aList = array();
		foreach($this->aFields AS $Value) {
			$aList[] = $Value["name"];
		}
		
		return $aList;
	}
	
	public function getSysFieldsTitles()
	{
		$aList = array();
		foreach($this->aFields AS $Value) {
			$aList[] = $this->getTitle($Value["name"]);
		}
		
		return $aList;
	}
	
	private function getTitle($sName)
	{
		$sTitle = "REF_" . strtoupper($sName);
		return MSF_Registry::get("MSF_Locale")->getWord($sTitle);
	}
	
	public function offsetGet($iKey, $aValues)
	{
		return array(
			"name" 		=> $this->aFields[$iKey]["name"],
			"title" 	=> $this->getTitle($this->aFields[$iKey]["name"]),
			"type"		=> $this->aFields[$iKey]["type"],
			"value" 	=> MSF_SL::getParam($aValues, $this->aFields[$iKey]["name"], '')
		);
	}
}
?>