<?php
/*
 * File: 	Ref_Value.php
 * Date: 	28.02.2008
 * Time:	11:48:02
 * Detrix
 */

// [include]
include_once "modules/references/library/Reference/REF_ParsingException.php";
// [/include]

/**
 * @package 	XXX
 * @copyright 	Copyright (c) 2007, Detrix
 */
class REF_Value
{
	const ROW_DELIMITER		= "|^|row|^|";

	const COLUMN_DELIMITER	= "/^/col/^/";
	
	/**
	 * @var 	type description
	 * @access 	private
	 */
	private $aValue = array();

	/////////////////////////////////////////////////////////////////////////////////////////////////
	//
	// Getters
	//
	/////////////////////////////////////////////////////////////////////////////////////////////////

	public function getSize()
	{
		return count($this->aValue);
	}

	/////////////////////////////////////////////////////////////////////////////////////////////////
	//
	// Public Doers
	//
	/////////////////////////////////////////////////////////////////////////////////////////////////

	/**
	 * @throws REF_ParsingException
	 */
	public function parse($sFormattedString)
	{
		//echo $sFormattedString . "<br/>";
		// $aParsedValue likes 	array[0][key]	= 1;
		//						   	 	[value]	= Almaty;
		// 							 [1][key]	= 2;
		//						   	 	[value]	= Astana;
		$aParsedValue 	= array();
		/*
		echo "@@@@@@@@@@@@@@@@@@@@@@@@\n";
		print_r($sFormattedString);
		echo "@@@@@@@@@@@@@@@@@@@@@@@@\n";
		*/

		if (strlen(trim($sFormattedString)) > 0) {
			$aRows 			= explode(self::ROW_DELIMITER, $sFormattedString);
			for ($i = 0; $i < count($aRows); $i++) {
				$aCell = explode(self::COLUMN_DELIMITER, $aRows[$i]);
				
				if (!is_array($aCell)) {
					throw new REF_ParsingException($aRows[$i]);
				}
				
				if((count($aCell) == 2)) {
					$aParsedValue[$i]["key"] = (int) $aCell[1];
				}
				else {
					$aTmp	= array();
					for($j=1; $j<count($aCell); $j++) {
						array_push($aTmp, (int)$aCell[$j]);
					}
					
					$aParsedValue[$i]["key"] = $aTmp;
				}
				
				$aParsedValue[$i]["value"] 	= (string) $aCell[0];
			}

		}


		$this->aValue = $aParsedValue;
	}

	public function format()
	{
		// $sFormattedString looks like Almaty/1|Astana/2
		
		$sFormattedString		= "";
		$i 						= 0;
		$iValueSize				= count($this->aValue);
		$iDelimiter				= $iValueSize - 1;
		foreach ($this->aValue AS $aValue) {
			$sFormattedString .= $aValue["value"] . self::COLUMN_DELIMITER;
			if(is_array($aValue["key"])) {
				$sFormattedString .= implode(self::COLUMN_DELIMITER, $aValue["key"]);
			}
			else {
				$sFormattedString .= $aValue["key"];
			}
			
			if ($i++ < $iDelimiter) {
				$sFormattedString .= self::ROW_DELIMITER;
			}

		}
		
		return $sFormattedString;
	}

	public function get($iRowId, $sKey = null)
	{
		if (($sKey == null) && (array_key_exists($iRowId, $this->aValue))) {
			$mResult = $this->aValue[$iRowId];
		}
		else if (($sKey == null) && (!array_key_exists($iRowId, $this->aValue))) {
			$mResult = null;
		}
		else if (array_key_exists($sKey, $this->aValue[$iRowId])) {
			$mResult = $this->aValue[$iRowId][$sKey];
		}
		else {
			$mResult = null;
		}

		return $mResult;
	}

	public function add($iKey, $sValue)
	{
		$this->aValue[] = array (
			"key" 	=> $iKey,
			"value"	=> (string) $sValue
		);
	}

	public function remove($iRowId)
	{
		unset($this->aValue[$iRowId]);
	}

	public function getValuesList()
	{
		return $this->getListOf("value");
	}

	public function getKeysList()
	{
		return $this->getListOf("key");
	}

	/////////////////////////////////////////////////////////////////////////////////////////////////
	//
	// Private Doers
	//
	/////////////////////////////////////////////////////////////////////////////////////////////////

	private function getListOf($sKey)
	{
		$aResult = array();
		foreach ($this->aValue AS $aValue) {
			$aResult[] = $aValue[$sKey];
		}

		return $aResult;
	}
	
	public function reset() {
		$this->aValue = array();
	}

}
?>