<?php

if (!defined("__MSF")) {
    exit("Direct access is not allowed");
}

include_once "system/mvc/MSF_ModelDBSupport.php";
include_once "ReferenceException.php";
include_once "UserReferenceDB.php";
include_once "UserReferenceRow.php";
include_once "UserReferenceRowDB.php";
include_once "REF_Value.php";
include_once "library/MSF_UniqueField/MSF_UniqueField.php";
include_once "library/MSF_Recordset/MSF_Recordset.php";
include_once "library/MSF_Validator/MSF_ValidatorFactory.php";
include_once "modules/eventlog/library/ELOG_EventLog.php";

class UserReference {

    /**
     * @var UserReferenceDB
     */
    private $DB;
    private $iRefId;
    private $aSelect;
    private $aConf;
    private $bIsLoaded;

    /**
     * @var MSF_Recordset
     */
    private $Recordset;
    private $aColumns;
    private $sTable;
    private $sName;
    private $sDescription;
    private $sFullDescription;
    private $iGroupId;
    private $Template;
    private $bIsSystem;

    public function __construct(UserReferenceDB $DB) {
        $this->DB = $DB;
    }

    public function nameExists($sName) {
        return $this->DB->nameExists($sName);
    }

    public function load($iId, $aSelect = array()) {
        $aConf = $this->DB->load($iId);
        if (empty($aConf)) {
            throw new ReferenceException("Reference not found");
        }

        $this->iRefId = $iId;
        $this->sTable = trim($aConf["table_name"]);
        $this->sName = $aConf["name"];
        $this->sDescription = $aConf["description"];
        $this->sFullDescription = $aConf["full_description"];
        $this->Template = $aConf["template"];
        $this->aSelect = $aSelect;
        $this->aColumns = self::parseColumns($aConf["columns"]);
        $this->sFieldsXml = $aConf["columns"];
        $this->iGroupId = $aConf["group_id"];
        $this->isSystem = $aConf["is_system"];
    }

    public function loadByName($sName, $aSelect = array()) {
        $aConf = $this->DB->loadByName($sName);
        if (empty($aConf)) {
            throw new ReferenceException("Reference not found");
        }

        $this->iRefId = $aConf["reference_id"];
        $this->sTable = trim($aConf["table_name"]);
        $this->sName = $aConf["name"];
        $this->Template = $aConf["template"];
        $this->aSelect = $aSelect;
        $this->aColumns = self::parseColumns($aConf["columns"]);
        $this->sFieldsXml = $aConf["columns"];
        $this->isSystem = $aConf["is_system"];
    }

    public function getName() {
        return $this->sName;
    }

    public function getId() {
        return $this->iRefId;
    }

    public function isSystem() {
        return (bool) $this->isSystem;
    }

    public function getDescription() {
        return $this->sDescription;
    }

    public function getFullDescription() {
        return $this->sFullDescription;
    }

    public function getColumns() {
        return $this->aColumns;
    }

    public function getTable() {
        return $this->sTable;
    }

    public function getGroupId() {
        return $this->iGroupId;
    }

    public function getRecordIdByFieldValue($sName, $sValue) {
        $aFields = $this->getAsArray();
        foreach ($aFields AS $recKey => $recValue) {
            foreach ($recValue AS $Key => $Value) {
                if ($Key == $sName && $Value == $sValue) {
                    return $aFields[$recKey]["id"];
                }
            }
        }

        return -1;
    }

    public function getTemplate() {
        return $this->Template;
    }

    public function getRecords($iLimitFrom = -1, $bExport = false) {
        $this->buildRecordset($iLimitFrom, $bExport);
        $this->Recordset->enableContextSearch("ref_index", "id", $this->iRefId);
        // ppc kostyl
        $this->Recordset->setDefaultOrder('id');
        return $this->Recordset->getAllRecords();
    }

    public function getGroup() {
        $this->buildRecordset();
        $this->Recordset->enableContextSearch("ref_index", "id", $this->iRefId);
        return $this->Recordset->getGroup();
    }

    public function getFields() {
        $aReturn = array();
        foreach ($this->aColumns as $aColumn) {
            if ($aColumn["name"] == "id") {
                continue;
            }

            if (isset($aColumn["values"])) {
                $aColumn["values"] = implode(",", $aColumn["values"]);
            }

            array_push($aReturn, $aColumn);
        }

        return $aReturn;
    }

    public function getFieldsXml() {
        return $this->sFieldsXml;
    }

    public function getAvaibleFieldTypes() {
        $aTypes = array("int", "float", "string", "text", "enum", "date", "reference");
        return $aTypes;
    }

    public function getMiniView($bIsUnique = 0, $sUniqueType = "", $iObjectId, $sFieldName, $sDefaultValues = "", $sOrderBy, $sOrderDirection, $sLimitFrom, $iRecordsLimit = -1, $aCurrentValue = array(-1)) {
        $aExcept = array(-1);
        if (($bIsUnique == 1) || ($bIsUnique == -1)) {
            $UF = new MSF_UniqueField(MSF_Registry::get("MSF_DB"));
            $aTmp = $UF->getBlocked(
                    $sUniqueType, $this->iRefId, $iObjectId, $sFieldName, $sDefaultValues
            );

            $aExcept = array_merge($aExcept, $aTmp);
        }

        $aColumns = array();
        $i = 1;
        foreach ($this->aColumns as $sKey => $aParam) {
            if (!in_array($aParam["name"], $this->aSelect) && $aParam["name"] != "id") {
                continue;
            }

            $aColumns[$i] = array();
            $aColumns[$i]["name"] = $aParam["name"];
            $aColumns[$i]["title"] = MSF_Registry::get("MSF_Locale")->getI18N($aParam["title"]);
            $aColumns[$i]["sort"] = true;
            $aColumns[$i]["filter"] = true;
            $aColumns[$i]["group"] = true;
            $aColumns[$i]["type"] = $aParam["type"];
            $aColumns[$i]["visible"] = true;

            if (isset($aParam["optional"])) {
                $aColumns[$i]["optional"] = $aParam["optional"];
            }

            if (in_array($aParam["type"], array("int", "float"))) {
                $aColumns[$i]["compare"] = "equal";
            } else {
                $aColumns[$i]["compare"] = "like";
            }

            $i++;
        }

        $aWhere = array();
        if ($bIsUnique == -1) {
            $aWhere = array(
                "AND id IN(" . implode(",", $aExcept) . ")"
            );
        } else if ($bIsUnique == 1) {
            $aWhere = array(
                "AND (id NOT IN(" . implode(",", $aExcept) . ") OR id IN(" . implode(",", $aCurrentValue) . "))"
            );
        }

        if ($this->iRefId == 6) {
            $RVal = new REF_Value();
            $Ubox = MSF_Registry::get("MSF_UserBox");
            $RVal->add($Ubox->getUserAttribute("id"), $Ubox->getUserAttribute("login"));

            $aWhere[] = "AND user_name = '" . $RVal->format() . "'";
        }

        $Recordset = new MSF_Recordset(MSF_Registry::get("MSF_DB"));
        if ($iRecordsLimit != -1) {
            $Recordset->setRecordsOnPage($iRecordsLimit);
        }

        $Recordset->setColumns($aColumns);
        $Recordset->setTables(array($this->sTable));
        $Recordset->setConditions($aWhere);
        $Recordset->setBaseURL("/references/viewref/" . $this->iRefId . "/?");
        $Recordset->setDefaultOrder("id");
        $Recordset->setSecondaryOrder("id");

        return $Recordset->getGroup();
    }

    public function getAsArray() {
        return $this->DB->getAllRecords($this->sTable, array());
    }

    public function saveReference($aParams, $sName, $sTemplate, $sDesc, $sFullDesc, $iGroupId) {
        $aOld = $this->aColumns;
        $aNew = self::parseColumns($aParams["config"]);


        $aConfig = $aParams["config"];
        $aToRemove = $aParams["remove"];
        $aToAdd = $aParams["add"];
        if ($this->needClearState($aOld, $aNew)) {
            $this->buildRecordset();
            $this->Recordset->clearState();
        }

        $this->DB->begin();
        foreach ($aToRemove as $sColumn) {
            $this->DB->removeColumn($this->sTable, $sColumn);
        }

        foreach ($aToAdd as $aColumn) {
            $this->DB->addColumn($this->sTable, $aColumn["name"], $aColumn["type"], $aColumn["values"]);
        }

        $this->DB->saveConfig($this->iRefId, $sName, $aConfig, $sTemplate, $sDesc, $sFullDesc, $iGroupId);
        $this->DB->commit();
    }

    public function create($sName, $sDescription = "") {
        $Document = new DOMDocument();
        $Root = $Document->createElement("document");
        $Document->appendChild($Root);
        $Element = $Document->createElement("element");
        $Element->setAttribute("name", "id");
        $Element->setAttribute("title", "ID");
        $Element->setAttribute("type", "array");
        $Root->appendChild($Element);
        $Item = $Document->createElement("item");
        $Item->setAttribute("name", "type");
        $Item->setAttribute("type", "string");
        $Item->setAttribute("value", "int");
        $Element->appendChild($Item);
        $Item = $Document->createElement("item");
        $Item->setAttribute("name", "required");
        $Item->setAttribute("type", "int");
        $Item->setAttribute("value", "1");
        $Element->appendChild($Item);
        $Item = $Document->createElement("item");
        $Item->setAttribute("name", "group");
        $Item->setAttribute("type", "int");
        $Item->setAttribute("value", "0");
        $Element->appendChild($Item);
        $Item = $Document->createElement("item");
        $Item->setAttribute("name", "sort");
        $Item->setAttribute("type", "int");
        $Item->setAttribute("value", "1");
        $Element->appendChild($Item);
        $Item = $Document->createElement("item");
        $Item->setAttribute("name", "filter");
        $Item->setAttribute("type", "int");
        $Item->setAttribute("value", "1");
        $Element->appendChild($Item);
        $Item = $Document->createElement("item");
        $Item->setAttribute("name", "defaultValue");
        $Item->setAttribute("type", "string");
        $Item->setAttribute("value", "0");
        $Element->appendChild($Item);
        $sTableName = "ref_user_" . MSF_String::hash(microtime(), 32);
        return $this->DB->createReference(
                        $sName, $sDescription, $Document->saveHTML(), $sTableName
        );
    }

    public function remove() {
        $this->removeReferenceEventLog();
        $this->DB->removeReferenceConfig($this->iRefId);
        $this->DB->removeReferenceTable($this->sTable);
    }

    private function removeReferenceEventLog() {
        $sEventTitle = $this->getName();
        $oEventLog = new ELOG_EventLog(MSF_Registry::get("MSF_DB"));
        $oEventLog->addItem(array("event_type" => "admin", "event_title" => $sEventTitle, "message_type" => "reference"));
    }

    public function clearState() {
        if (!$this->Recordset) {
            $this->buildRecordset();
        }

        $this->Recordset->clearState();
    }

    private function buildRecordset($iLimitFrom = -1, $bExport = false) {
        $this->Recordset = new MSF_Recordset(
                        MSF_Registry::get("MSF_DB"),
                        MSF_Registry::get("MSF_Template")
        );
        $this->Recordset->setExport($bExport);
        if ($iLimitFrom != - 1) {
            $this->Recordset->setParam("limitFrom", $iLimitFrom);
        }

        $this->Recordset->setBaseURL("/references/viewref/" . $this->iRefId . "/?");
        $this->Recordset->setRpcUrl("/references/groupreference/" . $this->iRefId . "/?");
        $Locale = MSF_Registry::get("MSF_Locale");

        $aActions = array(
            array(
                "title" => $Locale->getWord("REF_OPEN"),
                "href" => "/references/viewrow/" . $this->iRefId,
                "image" => "/templates/default/images/folder_page_white.png",
                "bind" => array("id"),
                "confirm" => false
            ),
            array(
                "title" => $Locale->getWord("REF_OPEN_BLANK"),
                "href" => "/references/viewrow/" . $this->iRefId,
                "image" => "/templates/default/images/folder_page_white_blank.png",
                "bind" => array("id"),
                "confirm" => false,
                "blank" => true
            ),
            array(
                "title" => $Locale->getWord("REF_DELETE"),
                "href" => "/references/remove/" . $this->iRefId,
                "image" => "/templates/default/images/red_cross.png",
                "bind" => array("id"),
                "confirm" => true
            ),
        );

        if ($this->checkAccess("edit_conf")) {
            $aMultiActions = array(
                array(
                    "title" => $Locale->getWord("REF_DELETE"),
                    "href" => "/references/remove/" . $this->iRefId,
                    "image" => "/templates/default/images/red_cross.png",
                    "confirm" => true
                )
            );
        } else {
            $aMultiActions = array();
        }

        $this->Recordset->setActions($aActions);
        $this->Recordset->setMultiActions($aMultiActions);
        $this->Recordset->setMultiActionField("id");

        $this->Recordset->setLink(
                array(
                    "href" => "/references/viewrow/" . $this->iRefId,
                    "bind" => array("id")
                )
        );

        $this->Recordset->setStateParams(array(
            "module" => "reference",
            "action" => "show",
            "action_id" => $this->iRefId,
            "structure_id" => MSF_Registry::get("MSF_UserBox")->getCurrentStructureId()
        ));

        $this->setRecordsetSQL();
        $this->setRecordsetColumns();
        $this->Recordset->setDefaultOrder("id");
        $this->Recordset->setSecondaryOrder("id");
    }

    private function checkAccess($sAction) {
        $Switcher = new REF_ReferencesSwitcher(MSF_Registry::get("MSF_DB"));
        return $Switcher->hasAccess($sAction, WF_Utils::getPath(), array("reference_id" => $this->iRefId));
    }

    private function setRecordsetSQL() {
        $this->Recordset->setTables(array($this->sTable));
    }

    private function setRecordsetColumns() {
        $aColumns = array();
        $i = 1;
        $sDefaultFilter = null;
        if (!is_array($this->aColumns)) {
            $this->aColumns = array();
        }

        foreach ($this->aColumns as $sKey => $aParam) {
            if ($aParam["name"] == "id") {
                $aParam["visible"] = 0;
            }


            $aColumns[$i] = array();
            $aColumns[$i]["name"] = $aParam["name"];
            $aColumns[$i]["title"] = $aParam["title"];
            $aColumns[$i]["type"] = $aParam["type"];
            if (((int) $aParam["visible"] == 1)) {
                if (array_key_exists('visible_on_locale', $aParam)) {
                    if ($aParam["visible_on_locale"] == CNF_LANGUAGE_2 || $aParam["visible_on_locale"] == "both") {
                        $aColumns[$i]["visible"] = 1;
                    } else {
                        $aColumns[$i]["visible"] = 0;
                    }
                } else {

                    $aColumns[$i]["visible"] = 1;
                }
            } else if ((int) $aParam["visible"] == 0) {
                $aColumns[$i]["visible"] = 0;
            }
            if ($aParam["type"] == "file" || $aParam["type"] == "picture") {
                $aColumns[$i]["sort"] = 0;
                $aColumns[$i]["filter"] = 0;
                $aColumns[$i]["group"] = 0;
            } else {
                $aColumns[$i]["sort"] = $aParam["sort"];
                $aColumns[$i]["filter"] = $aParam["filter"];
                $aColumns[$i]["group"] = $aParam["group"];
            }

            if (isset($aParam["optional"])) {
                $aColumns[$i]["optional"] = $aParam["optional"];
            }

            if (in_array($aParam["type"], array("int", "float"))) {
                $aColumns[$i]["compare"] = "equal";
            } else {
                $aColumns[$i]["compare"] = "like";
            }

            if (is_null($sDefaultFilter) && $aParam["name"] != "id") {
                $sDefaultFilter = $aParam["name"];
            }

            $i++;
        }

        $this->Recordset->setColumns($aColumns);
    }

    /**
     * @param int $iRowId
     * @return UserReferenceRow
     */
    public function row($iRowId = 0) {
        if ($iRowId === 0) {
            $bIsNew = true;
        } else {
            $bIsNew = false;
        }

        return new UserReferenceRow(
                        new UserReferenceRowDB(
                                MSF_Registry::get("MSF_DB"),
                                $this->iRefId,
                                $iRowId
                        ),
                        $this->aColumns, $bIsNew
        );
    }

    public static function parseColumns($sColumns) {
        $aColumns = array();
        foreach (MSF_XmlWrapper::xmlToArray($sColumns) AS $sName => $aBody) {
            $aColumn = array();
            $aColumn["name"] = $sName;
            $aColumn["title"] = $aBody["title"];
            foreach ($aBody["value"] as $sName => $mValue) {
                if (in_array($sName, array("name", "title"))) {
                    continue;
                }

                if (is_array($mValue)) {
                    $aColumn[$sName] = array();
                    foreach ($mValue as $k => $v) {
                        $aColumn[$sName][$k] = $v;
                    }
                } else {
                    $aColumn[$sName] = $mValue;
                }
            }

            array_push($aColumns, $aColumn);
        }

        return $aColumns;
    }

    private function toMiniView($aData) {
        $aColumnsType = array();
        foreach ($this->aColumns as $aColumn) {
            $aColumnsType[$aColumn["name"]] = $aColumn["type"];
        }

        foreach ($aData as &$aRow) {
            foreach ($aRow as $key => &$sValue) {
                $sValue = MSF_RS_Params::toOutput($sValue, $aColumnsType[$key]);
            }
        }

        return $aData;
    }

    public function checkUnique($aArgs, $sName, $mValue) {
        return !(bool) (int) $this->DB->checkUnique($this->sTable, $sName, $mValue, $aArgs["iRowId"]);
    }

    public function checkLicense($numOfLicense, $iId) {
        $bFlag = false;
        $aLicenses = $this->DB->getLicense(MSF_EnumValue::ROW_DELIMITER);
        if (count($aLicenses) < $numOfLicense) {
            $bFlag = true;
        } else {
            foreach ($aLicenses as $aLic) {
                if ($aLic["id"] == $iId) {
                    $bFlag = true;
                }
            }
        }

        return $bFlag;
    }

    private function needClearState($aOld, $aNew) {
        $bResult = false;

        foreach ($aOld as $aOldField) {
            if ($aOldField["name"] == "id") {
                continue;
            }

            $bIsField = false;
            foreach ($aNew as $aNewField) {
                if ($aOldField["name"] == $aNewField["name"]) {
                    $bIsField = true;

                    if ($aOldField["visible"] == 1 && $aOldField["visible"] == 0) {
                        $bIsField = false;
                    }

                    if (isset($aOldField["group"]) && $aOldField["group"] == 1 && $aOldField["group"] == 0) {
                        $bIsField = false;
                    }

                    if (isset($aOldField["sort"]) && $aOldField["sort"] == 1 && $aOldField["sort"] == 0) {
                        $bIsField = false;
                    }

                    if (isset($aOldField["filter"]) && $aOldField["filter"] == 1 && $aOldField["filter"] == 0) {
                        $bIsField = false;
                    }
                }
            }

            if (!$bIsField) {
                $bResult = false;
                break;
            }
        }

        return $bResult;
    }

    public function getUserId($sUserName) {
        $aUser = $this->DB->getUserId($sUserName);
        if (!empty($aUser)) {
            return $aUser[0]['id'];
        } else {
            return 0;
        }
    }

    public function getUserEmail($iUserId) {
        $aEmail = $this->DB->getUserEmail($iUserId);
        if (!empty($aEmail)) {
            return $aEmail[0]['email'];
        } else {
            return "";
        }
    }

    public function getUserSmtpAddress($iUserId) {
        $aEmail = $this->DB->getUserSmtpAddress($iUserId);
        if (!empty($aEmail)) {
            return $aEmail[0]['email_smtp_address'];
        } else {
            return "";
        }
    }

    public function getSystemUserId() {
        $aId = $this->DB->getSystemUserId();
        if (!empty($aId)) {
            return $aId[0]['id'];
        } else {
            return "";
        }
    }

}

?>