<?php

include_once "msfx/basket/MSF_Basket.php";
include_once "modules/main/library/orgStructure/MSF_OrgStructureFacade.php";

class UserReferenceRow
{
	private $DB;

	private $aColumns;

	private $bIsNew;

	public function __construct($DB, $aColumns, $bIsNew)
	{
		$this->DB		= $DB;
		$this->aColumns	= $aColumns;
		$this->bIsNew	= $bIsNew;
	}

	public function show($aInput = array())
	{
		if($this->bIsNew || !empty($aInput)) {
			$aRow = $this->prepareOutput($aInput);
		}
		else {
			$aRow	= $this->DB->getRow();			
		}
		
		if(empty($aRow)) {
			throw new ReferenceException("Row not found");
		}

		$aReturn	= array();
		foreach($this->aColumns as $aColumn) {
			if($aColumn["name"] != "id") {
				$aTmpColumn				= array();
				$aTmpColumn["name"]		= $aColumn["name"];
				$aTmpColumn["title"]	= $aColumn["title"];
				$aTmpColumn["type"]		= $aColumn["type"];

				$sValue = $aRow[$aColumn["name"]];
				
				if($aColumn["type"] == "enum") {
					$aTmpColumn["values"]	= $aColumn["optional"]["enum"];
					$aTmpColumn["value"]	= $sValue;
				}
				elseif($aColumn["type"] == "reference") {
					$aTmpColumn["value"]		= $sValue;
					$aTmpColumn["reference_id"]	= $aColumn["optional"]["reference"];
					$aTmpColumn["multiselect"]	= $aColumn["optional"]["multiselect"];
					$aTmpColumn["column"]		= MSF_SL::getParam($aColumn["optional"], "columns", array());
				}
				elseif($aColumn["type"] == "bool") {
					$aTmpColumn["labelYes"]	= $aColumn["optional"]["labelYes"];
					$aTmpColumn["labelNo"]	= $aColumn["optional"]["labelNo"];
					$aTmpColumn["value"]	= $sValue;
				}
				else {
					$aTmpColumn["value"]	= $sValue;
				}

				array_push($aReturn, $aTmpColumn);
			}

		}		
		
		$oSystemFieldsSettings  = new REF_SystemFieldsSettings();
		$aSysFields				= $oSystemFieldsSettings->getSystemFields();
		for($i = 0; $i < count($aSysFields); $i++) {			
			array_push($aReturn, $oSystemFieldsSettings->offsetGet($i, $aRow));			
		}		
		
		return $aReturn;
	}
	
	public function getAsHash()
	{
		$aFields = $this->show();		
		$aHash	 = array();
				
		foreach ($aFields AS $aValue) {
			$sKey = $aValue["name"];			
			$aHash[$sKey] = $aValue["value"];
		}
				
		return $aHash;		
	}

	public function save($aInput)
	{					
		$aInputs	= $this->prepareInput($aInput);
		
		if(!$this->validate($aInputs["toValidate"])) {
			
			return false;
		}	
		
		// Добавление системных полей				
		$oUserBox 						  = MSF_Registry::get("MSF_UserBox");
		$oStructureFacade				  = MSF_OrgStructureFacade::getInstance();
		$iStructureId 					  = $oUserBox->getCurrentStructureId();		
		if($iStructureId != NULL) {
			$sEmployeeName					  = $oStructureFacade->getEmployeeNameByStructureId($iStructureId);						
			$aPost = $oStructureFacade->getPostByStructureId($iStructureId);
			$sPost = "";
			if (isset($aPost[0]["value"])){
				$sPost = $aPost[0]["value"];
			}
			$iEmployeeId = $oStructureFacade->getEmployeeIdByStructureId($iStructureId);
			$iPostId = $oStructureFacade->getPostIdByStructureId($iStructureId);
			$RefValue 						  = new REF_Value();
			$RefValue->add(array($iStructureId, $iEmployeeId, $iPostId), $sEmployeeName." (".$sPost.")");
			$sAuthorName 					  = $RefValue->format();
		}
		else {
			$sAuthorName = "";
		}							
		
		$sCurrentDate = date("Y-m-d H:i:s");
		$aInputs["values"]["modified_at"] = $sCurrentDate;		
		$aInputs["values"]["editor_name"] = $sAuthorName;		
		if($this->bIsNew) {
			$aInputs["values"]["created_at"]  = $sCurrentDate;
			$aInputs["values"]["author_name"] = $sAuthorName;
			$iId	= $this->DB->addRow($aInputs["values"]);
		}
		else {
			$iId	= $this->DB->changeRow($aInputs["values"]);
		}			

		return $iId;
	}

	public function remove()
	{	
		$oBasket = new MSF_Basket();
		$oBasket->addDeletedRefElement($this->DB->getRefId(), $this->DB->getRowId());		
		$this->DB->remove();
	}

	private function validate($aConf)
	{
		$Factory		= new MSF_ValidatorFactory();
		$Messenger		= MSF_Registry::get("MSF_Messenger");
		$aValidators	= array();
		$bValidFlag		= true;

		foreach($aConf as $aRow) {
			$aValidators[] = $Factory->createValidator($aRow["type"], $aRow["aConf"]);
		}
		
		//vikluchil validaciu izza neponyatok v validacie v RED_Fields
		/*
		foreach($aValidators as $Validator) {
			if (!$Validator->isValid()) {
				$bValidFlag = false;

	            while ($sError = $Validator->getError()) {
	                $Messenger->addError($sError);
	            }

        	}

		}
		*/
		return $bValidFlag;
	}

	private function prepareInput($aInput)
	{
		$aNewValues		= array();
		$aToValidate	= array();
		$aReturn		= array();

		// Если приходит false вместо массива
		if(!$this->aColumns) {
			$this->aColumns = array();
		}		
		foreach($this->aColumns as $aColumn) {
			if($aColumn["name"] == "id") {
				continue;
			}

			$aValidatorConf 				= array();
			$aValidatorConf["sFieldName"]	= $aColumn["title"];

			if($aColumn["type"] == "reference") {
				$mRow 			= $aInput[$aColumn["name"]];

				$aValidatorConf["mValue"]		= $mRow;
				$aNewValues[$aColumn["name"]] 	= $mRow;
			}
			elseif($aColumn["type"] == "password") {
				$aValidatorConf["mValue"]		= MSF_SL::getParam($aInput, $aColumn["name"], "", false);
				$aValidatorConf["sConfirm"]		= MSF_SL::getParam($_POST, $aColumn["name"] . "_confirm", "", false);
				
				$aNewValues[$aColumn["name"]]	= $aValidatorConf["mValue"];
			}
			else {
				if (isset($aInput[$aColumn["name"]])) {
					$aValidatorConf["mValue"]		= $aInput[$aColumn["name"]];
					$aNewValues[$aColumn["name"]]	= $aValidatorConf["mValue"];
				}
				else {
					$aValidatorConf["mValue"]		= "";
					$aNewValues[$aColumn["name"]]	= "";
				}
			}

			if($aColumn["type"] == "enum") {
				$aValidatorConf["aEnum"] = $aColumn["optional"]["enum"];
			}
			


			$aTmp	= array(
								"type"	=> $aColumn["type"],
								"aConf"	=> $aValidatorConf
							);

			array_push($aToValidate, $aTmp);
		}

		$aReturn["values"]			= $aNewValues;
		$aReturn["toValidate"]		= $aToValidate;

		return $aReturn;
	}

	private function prepareOutput($aInput)
	{
		$aReturn = array();

		foreach($this->aColumns as $aColumn) {
			if(IS_REQUEST) {
				if(in_array($aColumn["type"], array("date", "timestamp", "time"))) {
					$aDate = MSF_SL::getParam($aInput, $aColumn["name"], array());

					$sDate	= "";

					if(in_array($aColumn["type"], array("date", "timestamp"))) {
						$sDate	.= (int) MSF_SL::getParam($aDate, "year", 1970)
								. "-" . (int) MSF_SL::getParam($aDate, "month", 1)
								. "-" . (int) MSF_SL::getParam($aDate, "day", 1);
					}

					if($aColumn["type"] == "timestamp") {
						$sDate .= " ";
					}

					if(in_array($aColumn["type"], array("time", "timestamp"))) {
						$sDate	.= (int) MSF_SL::getParam($aDate, "hour", 0)
								. ":" .(int) MSF_SL::getParam($aDate, "minute", 0);
					}

					$mValue = $sDate;
				}
				elseif($aColumn["type"] == "reference") {
					$mValue = MSF_SL::getParam($aInput, $aColumn["name"], 0);
				}
				else {
					$mValue = MSF_SL::getParam($aInput, $aColumn["name"], "");
				}

				$aReturn[$aColumn["name"]] = $mValue;
			}
			else {

				switch($aColumn["type"]) {
					case "date"			:
							$aReturn[$aColumn["name"]] 	= $aColumn["defaultValue"]["year"]
														. "-" . $aColumn["defaultValue"]["month"]
														. "-" . $aColumn["defaultValue"]["day"];
						break;
					case "timestamp"	:
							$aReturn[$aColumn["name"]] 	= $aColumn["defaultValue"]["year"]
														. "-" . $aColumn["defaultValue"]["month"]
														. "-" . $aColumn["defaultValue"]["day"]
														. " " . $aColumn["defaultValue"]["hour"]
														. ":" . $aColumn["defaultValue"]["minute"];
						break;
					case "time"			:
							$aReturn[$aColumn["name"]] 	= $aColumn["defaultValue"]["hour"]
														. ":" . $aColumn["defaultValue"]["minute"];
						break;
					case "int"			:
					case "float"		:
					case "string"		:
					case "reference"	:
					case "text"			:
					case "bool"			:
							$aReturn[$aColumn["name"]] = $aColumn["defaultValue"];
						break;
					case "enum"			:
							$aReturn[$aColumn["name"]] = $aColumn["optional"]["enum"][$aColumn["defaultValue"]];
						break;
					default				: $aReturn[$aColumn["name"]] = "";
				}

			}

		}

		return $aReturn;
	}
}
?>