<?php
if(!defined("__MSF")) {
	die("Direct access is not allowed");
}
require_once "system/mvc/MSF_ModelDBSupport.php";
include_once "library/MSF_Recordset/MSF_Recordset.php";
include_once "REF_ReferencesListDB.php";
include_once "modules/references/library/Reference/UserReference.php";

class REF_ReferencesList
{
	/**
	 * @var MSF_Recordset
	 */
	private $Recordset	= null;

	private $aActions	= array();

	private $DB			= null;

	public function __construct(REF_ReferencesListDB $DB)
	{
		$this->Recordset	= new MSF_Recordset(MSF_Registry::get("MSF_DB"));
		$this->DB			= $DB;
	}

	public function getListRecordset($bWithGroups = false, $iLimitFrom = -1, $bExport = false)
	{
		$this->setRecordsetParams($bWithGroups, $iLimitFrom, $bExport);
		return $this->Recordset->getAllRecords();
	}

	public function getList()
	{
		$aList	= $this->DB->getList();

		for($i=0, $total=count($aList); $i<$total; $i++) {
			try {
				$aList[$i]["columns"]	= UserReference::parseColumns($aList[$i]["columns"]);
			}
			catch(Exception $e) {
				$aList[$i]["columns"]	= array();
			}

		}

		return $aList;
	}
	
	public function getGroup($bWithGroups = false)
	{
		$this->setRecordsetParams($bWithGroups);
		return $this->Recordset->getGroup();
	}

	public function addAction($sTitle, $sLink, $sImage, $aFields, $bConfrm = false)
	{
		$this->aActions[] = array(
			"title"		=> $sTitle,
			"href"		=> $sLink,
			"image"		=> $sImage,
			"bind"		=> $aFields,
			"confirm"	=> $bConfrm
		);			
		
	}

	public function setBaseUrl($sUrl)
	{
		$this->Recordset->setBaseURL($sUrl);
	}

	public function setRpcUrl($sUrl)
	{
		$this->Recordset->setRpcURL($sUrl);
	}
	
	public function setLink($sHref, $aBind)
	{
		$this->Recordset->setLink(
			array(
				"href"		=> $sHref,				
				"bind"		=> $aBind
			)
		);	
	}
	
	public function setStateParams($aParams)
	{
		$this->Recordset->setStateParams(array(
			"module"		=> $aParams["module"],
			"action"		=> $aParams["action"],
			"action_id"		=> $aParams["action_id"],
			"structure_id"	=> $aParams["structure_id"]
		));
	}

	private function getColumns($bWithGroups)
	{
		$aRColumns = array();
		
		if (!$bWithGroups) {
			$aTypes		= array("int", "text", "text");
                        $aTitles 	= array("ID", MSF_Registry::get("MSF_Locale")->getWord("REFERENCE_NAME2"), 
                            MSF_Registry::get("MSF_Locale")->getWord("REFERENCE_DESCRIPTION2"));
	 		$aColumns 	= array("reference_id", "name", "description");
	
	 		for ($i = 0; $i < sizeof($aTitles); $i++) {
		 		$aRColumns[$i]["name"] 		= $aColumns[$i];
		 		$aRColumns[$i]["title"] 	= $aTitles[$i];
				$aRColumns[$i]["type"] 		= $aTypes[$i];
		 		$aRColumns[$i]["sort"] 		= true;
		 		$aRColumns[$i]["filter"] 	= true;
		 		$aRColumns[$i]["group"] 	= false;
		 		$aRColumns[$i]["compare"] 	= "like";
		 		
		 		$aColumns[$i] == "reference_id" ? $aRColumns[$i]["visible"] = 0 : $aRColumns[$i]["visible"] = 1;	 		
	 		}
		}
		else {
			$aTypes 	= array("int", "text", "text", "text", "date", "timestamp");
			$aTitles 	= array("ID", MSF_Registry::get("MSF_Locale")->getWord("REFERENCE_NAME2"), 
                            MSF_Registry::get("MSF_Locale")->getWord("REFERENCE_DESCRIPTION2"), 
                            MSF_Registry::get("MSF_Locale")->getWord("REFERENCE_GROUP"), 
                            MSF_Registry::get("MSF_Locale")->getWord("REFERENCE_CREATED2"), 
                            MSF_Registry::get("MSF_Locale")->getWord("REFERENCE_CHANGED"));
			$aColumns 	= array("reference_id", "ref_name", "description", "group_name", "created_at", "modified_at");
			
			for ($i = 0; $i < sizeof($aTitles); $i++) {
		 		$aRColumns[$i]["name"] 		= $aColumns[$i];
		 		$aRColumns[$i]["title"] 	= $aTitles[$i];
				$aRColumns[$i]["type"] 		= $aTypes[$i];
		 		$aRColumns[$i]["sort"] 		= true;
		 		$aRColumns[$i]["filter"] 	= true;
		 		$aRColumns[$i]["compare"] 	= "like";
		 		$aRColumns[$i]["group"] 	= ($aColumns[$i] == "group_name") ? true : false;
		 		
		 		$aColumns[$i] == "reference_id" ? $aRColumns[$i]["visible"] = 0 : $aRColumns[$i]["visible"] = 1;	 		
	 		}
		}
		
 		return $aRColumns;
	}
	
	private function setRecordsetParams($bWithGroups, $iLimitFrom = -1, $bExport = false)
	{
		$this->Recordset->setColumns($this->getColumns($bWithGroups));
		$this->Recordset->setSecondaryOrder("reference_id");
		$this->Recordset->setExport ( $bExport );
		if ($iLimitFrom != - 1) {
			$this->Recordset->setParam ( "limitFrom", $iLimitFrom );
		}
		
		if (!$bWithGroups) {			
			$this->Recordset->setTables(array("references_config"));
			$this->Recordset->setActions($this->aActions);
		}
		else {			
			$this->Recordset->setTables(array("ref_references_and_groups_view"));			
			$this->Recordset->setActions($this->aActions);
			$this->Recordset->setDefaultGroups(array("group_name"));
			$this->setRpcUrl("/admin/references/list/ajax/?");
		}
		
	}
	
}
?>