<?php
if(!defined("__MSF")) {
	die("Direct access is not allowed");
}
require_once "modules/structure/library/security/Struct_Switcher.php";
require_once "modules/workflow/library/WF_Utils.php";
require_once "modules/references/library/Reference/REF_ReferenceGroupsList.php";

class Reference_MenuService implements MSF_IService
{
	private $Menu;

	private $Locale;
	
	private $DB;
	
	private $Auth;

	public function __construct()
	{
		$this->Menu 	= MSF_Registry::get("MSF_Menu");
		$this->Locale	= MSF_Registry::get("MSF_Locale");
		$this->DB 		= MSF_Registry::get("MSF_DB");
		$this->Auth 	= MSF_Registry::get("MSF_Authorization");
	}

	public function execute()
	{
		if (MSF_Registry::get("MSF_UserBox")->isAdmin()) {
			//$this->buildBack();
		}
		$this->buildFront();
	}
	
	private function buildBack()
	{
		$this->Menu->setItem("admin", "ref_admin", $this->Locale->getWord("REF_REFERENCES_MAIN"), "#", "image");
		$this->Menu->setItem("ref_admin", "ref_admin_references", $this->Locale->getWord("REF_REFERENCES"), "/admin/references/list/", "image");
		$this->Menu->setItem("ref_admin", "ref_admin_groups", $this->Locale->getWord("REF_REF_GROUPS"), "/admin/references/groups/", "image");
	}

	private function buildFront()
	{
		$this->Menu->setItem(null, "ref_main", $this->Locale->getWord("REF_REFERENCES"), "#");
				
		$StrucureSW	= new Struct_Switcher($this->DB);
		if($StrucureSW->hasAccess("view", WF_Utils::getPath(), array())) {
			$this->Menu->setItem("ref_main", "struct_main", $this->Locale->getWord("STRUCTURE_MODULE_NAME"), "/structure/view");
		}
			
		$aRefs 		= $this->getRefs();
		$aGroups	= array();
		
		foreach($aRefs as $aRef) {
			if($aRef["group_id"] != 0) {
				if(!in_array($aRef["group_id"], $aGroups)) {
					array_push($aGroups, $aRef["group_id"]);
				}
			}
			
		}
		
		$Groups	= new REF_ReferenceGroupsList();
		$aList	= $Groups->getList();
				
		foreach($aList as $aGroup) {
			if(in_array($aGroup["group_id"], $aGroups)) {
				$this->Menu->setItem("ref_main", "reference_group_" . $aGroup["group_id"], $aGroup["name"], "#", "", "", $aGroup["description"]);
			}
		}
		
		foreach($aRefs as $aRef) {
			if($aRef["group_id"] != 0) {
				$this->Menu->setItem(
					"reference_group_" . $aRef["group_id"],
					"reference_" . $aRef["reference_id"],
					$aRef["name"],
					"/references/viewref/" . $aRef["reference_id"],
					"", "",
					$aRef["description"]
				);
			}
			else {
				$this->Menu->setItem(
					"ref_main",
					"reference_" . $aRef["reference_id"],
					$aRef["name"], "/references/viewref/" . $aRef["reference_id"],
					"", "",
					$aRef["description"]
				);
			}
		}
	}
	
	private function getRefs()
	{
		$aResult 	= array();
		$aPath 		= WF_Utils::getPath();
		$aDIds 		= array();
		$aAIds 		= array();
		
		if(!empty($aPath)) {
			$aTmp = $this->select($aPath);
			//debug($aTmp);
			for ($j = 0; $j < count($aTmp); $j++) {
				if ($aTmp[$j]["can_view"] == 2) {
					$aDIds[] = $aTmp[$j]["reference_id"];
				}
				elseif ($aTmp[$j]["can_view"] == 1 && (!in_array($aTmp[$j]["reference_id"], $aDIds) || !in_array($aTmp[$j]["reference_id"], $aAIds))) {
					//if ($aTmp[$j]["reference_id"] == 2) {
					//	debug($aTmp);
					//}
					if (!in_array($aTmp[$j]["reference_id"], $aAIds)) {
						$iIndex 			= count($aResult);
						$aResult[$iIndex] 	= $aTmp[$j];
						$aAIds[] 			= $aTmp[$j]["reference_id"];
					}
					
				}
				
			}
			
		}
		$aFinal = array();
		foreach ($aResult as $aItem) {
			if (!in_array($aItem["reference_id"], $aDIds) || in_array($aItem["reference_id"], $aAIds)) {
				$aFinal[] = $aItem;
			}
		}
		
		
		return $aFinal;
		//return $aResult;
	}
	
	private function select($aIds)
	{
		$sSql = "SELECT\n" .
					"ref.reference_id,\n" .
					"ref.name,\n" .
					"ref.description,\n" .
					"ref.group_id,\n" .
					"acs.can_view\n" .
				"FROM\n" .
					"references_config ref,\n" .
					"ref_references_access_list acs\n" .
				"WHERE\n" .
					"ref.reference_id = acs.reference_id\n" .
				"AND\n" .
					"structure_id IN (" . implode(", ", $aIds) . ")\n" .
				"ORDER BY\n" .
					"ref.name";
		
		
		return $this->DB->select($sSql);
	}
}

?>