<?php
defined("__MSF") 
	or die("Direct access is not allowed");

include_once "modules/references/library/Reference/UserReferenceDB.php";
include_once "modules/references/library/Reference/UserReference.php";
include_once "modules/structure/library/Struct_Utils.php";
include_once "modules/structure/library/Struct_Main.php";
include_once "modules/structure/view/xml/Struct_ModelXml.php";
include_once "modules/structure/library/Struct_ReferencesSecurityManager.php";
include_once "modules/structure/library/Struct_WorkflowsSecurityManager.php";
include_once "modules/structure/library/security/Struct_Switcher.php";
class MSF_FrontController extends MSF_ControllerSupport
{
	private $ModelXml 	= null;

	/**
	 * @var Struct_Main
	 */
	private $Struct 	= null;
	
	private $sColDelim 	= "/^/col/^/";

	public function __construct()
	{
		$this->Struct = new Struct_Main();
		$this->Struct->setConfig(MSF_Registry::get("MSF_Settings")->getXmlWrapper()->get("modules")->getValue("structure"));
		$this->ModelXml = new Struct_ModelXml(MSF_Registry::get("MSF_Template"), MSF_Registry::get("MSF_Locale"));
		parent::__construct();
		$this->Template->setPageTitle(
			$this->locale("STRUCTURE_REF_LIST") .
			$this->locale("ROOT_TITLE_SEPARATOR") .
			$this->locale("STRUCTURE_MODULE_NAME")
		);
	}

	public function indexAction()
	{
		$this->viewAction();
	}
	
	private function checkAccess($sAction)
	{
		$Switcher = new Struct_Switcher(MSF_Registry::get("MSF_DB"));
		return $Switcher->hasAccess($sAction, WF_Utils::getPath(), array());
	}

	public function viewAction()
	{
		switch ($this->getParam(0)) {
			case "subdivisions":
				$this->view("subdivisions");
				break;
			case "tree":
				if (!$this->getParam(1)) {
					$this->treeRequest();
				}
				else {
					$this->treeRequest($this->getParam(1));
				}
				break;
			case "ajaxrequest":
				$this->treeRequest();
				break;
			case "table":
				$this->tableRequest();
				break;
			default:
				$this->view();
				break;
		}
	}

	public function editAction()
	{


		if ($this->checkAccess("edit")) {
			if (IS_REQUEST) {

				$sType 				= (string) 	$this->getParam(0);
				$iId 				= (int) 	$this->getParam(1);
				$sValue 			= (string) 	MSF_SL::getParam($_POST, $sType);
				$sValue2 			= (string) 	MSF_SL::getParam($_POST, "post2");				
				$aRes 				= $this->Struct->getElement($iId);				
				$aExtra 			= array();
				$RefValue 			= new REF_Value();
				$RefValue->parse($sValue);
				$aId 				= $RefValue->get(0);
				if (!empty($sValue2)) {
					$RefValue->parse($sValue2);
					
					for ($i = 0; $i < $RefValue->getSize(); $i++) {
						$aTmp = $RefValue->get($i);
						$aExtra[] = $aTmp["key"][0];
					}

				}
				
				if ($aId["key"] > 0) {
					$this->Struct->edit($iId, $aId["key"], implode(",", $aExtra));
				}
				else {
					$this->Struct->edit($iId, 0, implode(",", $aExtra));
				}
			
				if (!empty($aRes)) {
					$this->uniqueFree("empty" . $this->sColDelim . $aRes["subject_id"], $sType);					
				}

				if (isset($_POST["new_subdivision"])) {
					$sNewSubdivision 	= (string) 	MSF_SL::getParam($_POST, "new_subdivision");
					if (strlen($sNewSubdivision)) {
						$oNewSubdivision	= new REF_Value();
						$oNewSubdivision->parse($sNewSubdivision);
						$aNewSubdivision	= $oNewSubdivision->get(0);
						$this->Struct->replace($aNewSubdivision["key"], $aId["key"]);	
						if (!$this->uniqueCheck("empty" . $this->sColDelim . $aId["key"], $sType)) {
							$this->uniqueBlock("empty" . $this->sColDelim . $aId["key"], $sType);
						}
					}
					else {
						MSF_Registry::get("MSF_Messenger")->addError($this->locale("STRUCTURE_NULL_SUBDIVISION_MESSAGE"));
						$this->redirect("/structure/view");
						return;
					}
	
				}
				if (isset($_POST["move_subdivision"])) {
					$sNewSubdivision 	= (string) 	MSF_SL::getParam($_POST, "move_subdivision");
					if (strlen($sNewSubdivision)) {
						$oNewSubdivision	= new REF_Value();
						$oNewSubdivision->parse($sNewSubdivision);
						$aNewSubdivision	= $oNewSubdivision->get(0);
						$this->Struct->replaceSubdivision($aNewSubdivision["key"], $aId["key"]);
						if (!$this->uniqueCheck("empty" . $this->sColDelim . $aId["key"], $sType)) {
							$this->uniqueBlock("empty" . $this->sColDelim . $aId["key"], $sType);
						}
					}
					else {
						MSF_Registry::get("MSF_Messenger")->addError($this->locale("STRUCTURE_NULL_SUBDIVISION_MESSAGE"));
						$this->redirect("/structure/view");
						return;
					}
					
				}
				
				if($sType == "subdivision" && isset($aId["key"])) {
					if (!$this->uniqueCheck("empty" . $this->sColDelim . $aId["key"], $sType)) {
						$this->uniqueBlock("empty" . $this->sColDelim . $aId["key"], $sType);
					}		
				}
				
				if (isset($aId[1])) {
					if (!$this->uniqueCheck("empty" . $this->sColDelim . $aId[1], $sType)) {
						$this->uniqueBlock("empty" . $this->sColDelim . $aId[1], $sType);
					}
					
				}
				
			}

			$this->redirect("/structure/view", "");
		}
		else {
			$this->notAccessAction();
			return;
		}
	}

	public function removeAction()
	{
		// proverka dostupa
		if ($this->checkAccess("add_delete")) {
			$iId 		= $this->getIntParam(1);
			$oUserBox 	= MSF_UserBox::getInstance();
			$aIds 		= array();
			$iNewId 	= (int) MSF_SL::getParam($_POST, "structure_id", 0);
			switch ($this->getParam(0)) {
				case "post":
					$aIds[] = $iId;
					// proverka na udalenie sebya
					if ($iId == $oUserBox->getCurrentStructureId()) {
						MSF_Registry::get("MSF_Messenger")->addError($this->locale("STRUCTURE_REMOVING_YOURSELF_CANNOT"));
						$this->redirect("/structure/view");
						return;
					}
					break;
				default:
					// dlya podrazdeleniya vibrat vseh ego sotrudnikov
					$aIds 	= Struct_Utils::getPostsInSubdivision($iId);
					// proverka na udalenie podrazdeleniya v kotorom nahodishsia
					if (in_array($oUserBox->getCurrentStructureId(), $aIds)){
						MSF_Registry::get("MSF_Messenger")->addError($this->locale("STRUCTURE_REMOVING_YOURSELF_CANNOT"));
						$this->redirect("/structure/view");
						return;
					}
					break;
			}

			// Esli u sotrudnika ili podrazdeleniya
			// net sozdannih im documentov, to 
			// perepisat' pod novim avtorom			
			if (!Struct_Utils::isDocumentsExists($aIds) || Struct_Utils::didReplaceAuthorDocs($iId, $iNewId, $aIds)) {
				$this->findTreeElement($this->Struct->getTree(), $iId);
				$this->Struct->remove($iId);
				$this->rebuildHook();
				$this->redirect("/structure/view", $this->locale("STRUCTURE_REMOVED"));
			}
			else {
				$aStrList 	= $this->Struct->getList();
				//$WF 		= new Struct_WorkflowsSecurityManager($this->DB, $aStrList);
				//$Refs 		= new Struct_ReferencesSecurityManager(MSF_Registry::get("MSF_DB"));
				$aResult 	= $this->Struct->getTree();
				$this->ModelXml->viewTree(
						$this->toJson($this->unsetDeletingElement($aResult, $iId)),
						$this->Struct->getConfig(),
						MSF_Json::encode(array()),
						MSF_Json::encode(array()),
						MSF_Json::encode(array()),
						true
						);
				$this->setView($this->ModelXml);
			}			

		}
		else {
			$this->notAccessAction();
			return;
		}
	}
	
	private function unsetDeletingElement($aTree, $iId)
	{
		$aResult 	= array();
		$iCount 	= 0;
		
		foreach ($aTree as $aItem) {
			if ($aItem["node_id"] == $iId) {
				continue;
			}
			else {
				$aResult[$iCount] = $aItem;
			}
			
			if (isset($aItem["nodes"])) {
				$aResult[$iCount]["nodes"] = $this->unsetDeletingElement($aItem["nodes"], $iId);
			}
			$iCount++;
		}
		
		return $aResult;
	}
	
	private function findTreeElement($aStructure, $iId)
	{
		for ($i = 0; $i < count($aStructure); $i++) {
			if ($aStructure[$i]["node_id"] == $iId) {
				$this->uniqueFreeTree(array($aStructure[$i]));
				break;
			}
			elseif ($aStructure[$i]["type"] == "subdivision" && isset($aStructure[$i]["nodes"])) {
				$this->findTreeElement($aStructure[$i]["nodes"], $iId);
			}
		}
	}
	
	private function uniqueFreeTree($aStructure)
	{
		for ($i = 0; $i < count($aStructure); $i++) {
			if ($aStructure[$i]["type"] == "subdivision" && isset($aStructure[$i]["nodes"])) {
				$this->uniqueFreeTree($aStructure[$i]["nodes"]);
			}
			$this->uniqueFree("empty" . $this->sColDelim . $aStructure[$i]["subject_id"], $aStructure[$i]["type"]);
		}
	}

	public function moveAction()
	{
		if ($this->checkAccess("edit")) {
			$iWhat 	= (int) $this->getParam(0);
			$iWhere = (int) $this->getParam(1);
	
			$this->Struct->move($iWhat, $iWhere);
			$this->redirect("/structure/view", $this->locale("STRUCTURE_CHANGED"));
		}
		else {
			//MSF_AccessDenied::exitApp("moveAction()");
			$this->notAccessAction();
			return;
		}
	}
	
	public function moveBeginAction()
	{
		if ($this->checkAccess("edit")) {
			$iThisId  = (int) $this->getParam(0);
			$iFirstId = (int) $this->getParam(1);
	
			$this->Struct->moveBegin($iThisId, $iFirstId);
			$this->redirect("/structure/view", $this->locale("STRUCTURE_CHANGED"));
		}
		else {
			//MSF_AccessDenied::exitApp("moveAction()");
			$this->notAccessAction();
			return;
		}
	}
	
	public function moveEndAction()
	{
		if ($this->checkAccess("edit")) {
			$iThisId = (int) $this->getParam(0);
			$iLastId = (int) $this->getParam(1);
	
			$this->Struct->moveEnd($iThisId, $iLastId);
			$this->redirect("/structure/view", $this->locale("STRUCTURE_CHANGED"));
		}
		else {
			//MSF_AccessDenied::exitApp("moveAction()");
			$this->notAccessAction();
			return;
		}
	}
	
	private function view($sType = "")
	{
		if ($this->checkAccess("view")) {
			$Actionbar 	= MSF_Registry::get("MSF_ActionBar");
			$Actionbar->addAction(MSF_Registry::get("MSF_Locale")->getWord("STRUCTURE_EXPAND_BUTTON"), "#", "", "return false");
			$Actionbar->addAction(MSF_Registry::get("MSF_Locale")->getWord("STRUCTURE_COLLAPSE_BUTTON"), "#", "", "return false");
			
			$aStrList 	= $this->Struct->getList();
			//$WF 		= new Struct_WorkflowsSecurityManager($this->DB, $aStrList);
			//$Refs 		= new Struct_ReferencesSecurityManager(MSF_Registry::get("MSF_DB"));
			$aResult 	= array();
			
			if ($sType == "subdivisions") {
				$aResult = $this->Struct->getSubdivisions();
			}
			else {
				$aResult = $this->Struct->getTree();
			}
			$this->ModelXml->viewTree(
				$this->toJson($aResult),
				$this->Struct->getConfig(),
				MSF_Json::encode(array()),
				MSF_Json::encode(array()),
				MSF_Json::encode(array())
			);
			$this->setView($this->ModelXml);
		}
		else {
			//MSF_AccessDenied::exitApp("viewAction()");
			$this->notAccessAction();
			return;
		}
	}

	public function addAction()
	{
		if ($this->checkAccess("add_delete")) {
			$sType 		= (string) 	$this->getParam(0);
			$iParentId  = (int) 	$this->getParam(1);
			$sValues 	= (string) 	MSF_SL::getParam($_POST, $sType);
			$RefValue = new REF_Value();
			$RefValue->parse($sValues);
			$aId1 = $RefValue->get(0);
			$bAdded 	= false;
			$aValues = explode("|", $sValues);
			
			for ($i = 0; $i < sizeof($aValues); $i++) {
				$aId = explode($this->sColDelim, $aValues[$i]);
				if (!empty($aId[1]) && !$this->uniqueCheck("empty" . $this->sColDelim . $aId[1], $sType)) {
					$this->uniqueBlock("empty" . $this->sColDelim . $aId[1], $sType);
					$this->Struct->add($iParentId, $sType, $aId[1]);
					$bAdded = true;
				}

			}

			if (isset($_POST["new_subdivision"])) {
				$sNewSubdivision 	= (string) 	MSF_SL::getParam($_POST, "new_subdivision");
				$oNewSubdivision	= new REF_Value();
				$oNewSubdivision->parse($sNewSubdivision);
				$aNewSubdivision	= $oNewSubdivision->get(0);
				$this->Struct->replace($aNewSubdivision["key"], $aId1["key"]);	
			}

			if ($bAdded) {
				$this->rebuildHook();
				$this->redirect("/structure/view", $this->locale("STRUCTURE_ADDED"));
			}
			else {
				$this->redirect("/structure/view");
			}
		}
		else {
			//MSF_AccessDenied::exitApp("addAction()");
			$this->notAccessAction();
			return;
		}
	}
	
	private function treeRequest($sFieldName = "")
	{
		$View 		= new MSF_ViewSupportJson();
		$sFilter	= MSF_SL::getParam($_GET, "rs_quickFilter", "");
		if ($this->checkAccess("view")/* || MSF_Registry::get("MSF_UserBox")->isAdmin()*/) {
			$aResult["tree"]	= $this->Struct->getSubdivisions($sFilter, $sFieldName);
			$aResult["columns"] = $this->Struct->getConfig();
		}
		else {
			$aResult = array(
				"denied"		=> 1,
				"accessDenied"	=> $this->locale("STRUCTURE_MINI_ACCESS_DENIED"),
				"changeDenied"	=> $this->locale("STRUCTURE_MINI_CHANGE_DENIED")
			);
		}

		$View->set("data", $aResult);
		$this->setView($View);
	}
	
	private function tableRequest()
	{
		$View 	= new MSF_ViewSupportJson();
		if ($this->checkAccess("view")/* || MSF_Registry::get("MSF_UserBox")->isAdmin()*/) {
			$iId			= (int) MSF_SL::getParam($_GET, "id", 0);
			$iSkipNodeId 	= (int) MSF_SL::getParam($_GET, "skipNodeId", 0);
			$sEmployee 		= MSF_String::encodeSpecialChars($this->getParam(1)); 
			$sPost 			= MSF_String::encodeSpecialChars($this->getParam(2));
			if (empty($sEmployee)) {
				$sEmployee = "name";
			}
			else if (empty($sPost)) {
				$sPost = "post";
			}
			$View->set("data", $this->Struct->getPosts($iId, $sEmployee, $sPost, $iSkipNodeId));
		}
		else {
			$aResult = array(
				"denied"		=> 1,
				"accessDenied"	=> $this->locale("STRUCTURE_MINI_ACCESS_DENIED"),
				"changeDenied"	=> $this->locale("STRUCTURE_MINI_CHANGE_DENIED")
			);
			
			$View->set("data", $aResult);
			
		}
		
		$this->setView($View);
	}

	private function toJson($aResult)
	{
		$bEdit 	= $this->checkAccess("edit");
		$bAdd 	= $this->checkAccess("add_delete");
		$iConf 	= 50;
		
		if ($bEdit && $bAdd) {
			// edit/add_delete access
			$iConf = 100;
		}
		elseif (!$bEdit && $bAdd) {
			// add_delete access
			$iConf = 125;
		}
		elseif ($bEdit && !$bAdd) {
			// edit access
			$iConf = 150;
		}
		
		return
			!empty($aResult[0])
				?
					MSF_Json::encode(
						array(
							"tree" 		=> $aResult,
							"iConfig" 	=> $iConf
						)
					)
				:
					null;
	}
	
	private function uniqueBlock($sValue, $sRefType)
	{
		$UF	= new MSF_UniqueField(MSF_Registry::get("MSF_DB"));
		$UF->block(
			$sValue,
			MSF_UniqueField::TYPE_STRUCTURE,
			$this->getRefId($sRefType),
			MSF_UniqueField::STRUCTURE_ID,
			$sRefType
		);

	}
	
	private function uniqueFree($sValue, $sRefType)
	{
		$UF	= new MSF_UniqueField(MSF_Registry::get("MSF_DB"));
		$UF->free(
			$sValue,
			MSF_UniqueField::TYPE_STRUCTURE,
			$this->getRefId($sRefType),
			MSF_UniqueField::STRUCTURE_ID,
			$sRefType
		);
	}
	
	private function uniqueCheck($sValue, $sRefType)
	{
		$UF	= new MSF_UniqueField(MSF_Registry::get("MSF_DB"));
		return $UF->check(
			$sValue,
			MSF_UniqueField::TYPE_STRUCTURE,
			$this->getRefId($sRefType),
			MSF_UniqueField::STRUCTURE_ID,
			$sRefType
		);
	}
	
	private function getRefId($sRefType)
	{
		$iRefId = 4;
		if ($sRefType == "post") {
			$iRefId = 3;
		}
		
		return $iRefId;
	}
	
	private function rebuildHook()
	{
		$oHook = new MSF_Hook();
		$oHook->rebuild();
	}

}

?>