<?php

defined("__MSF") or die("Direct access is not allowed");

include_once "library/MSF_DBTree/MSF_DBTree.php";

class Struct_Tree
{	
	private $Tree = null;

	public function __construct(MSF_DB $DB)
	{
		$this->Tree = new MSF_DBTree($DB, "structure", "node_id", "parent_id", "path");
	}

	public function get($bTree = true, $iId = 1, $sType = null)
	{
		try {
			return $this->Tree->select(
				$iId,
				array("subject_id", "type", "path", "additional_subject_ids", "position"),
				(($sType !== null) ? array("type" => $sType) : array()),
				array(),
				$bTree
			);
		}
		catch (Exception $e) {
			return array();
		}
	}
	
	public function replace($iNode, $iElem) 
	{
		$this->Tree->replace($iNode, $iElem);
		$this->updatePosition(array($this->get()));
	}
	
	public function search($mColumns, $mValues)
	{
		return $this->Tree->searchByValues($mColumns, $mValues);
	}

	public function add($iParentId, $sType, $iSubjectId)
	{
		$aFields = array (
			"type"			=> $sType,
			"subject_id" 	=> $iSubjectId
		);
		$sResult = $this->Tree->insert($iParentId, $aFields);
		$this->updatePosition(array($this->get()));
		
		return $sResult;
	}

	public function edit($iNodeId, $iSubjectId, $sExtraIds)
	{
		$aFields = array (
			"subject_id" => $iSubjectId,
			"additional_subject_ids" => $sExtraIds
		);
		
		$sResult = $this->Tree->update($iNodeId, $aFields);
		
		return $sResult;
	}

	public function remove($iNodeId)
	{
		$this->Tree->remove($iNodeId, true);
		$this->updatePosition(array($this->get()));
	}

	public function move($iWhat, $iWhere)
	{
		$this->Tree->replacePath($iWhat, $iWhere);
		$this->updatePosition(array($this->get()));
	}
	
	public function moveBegin($iThisId, $iFirstId)
	{
		$this->Tree->moveBegin($iThisId, $iFirstId);
		$this->updatePosition(array($this->get()));
	}
	
	public function moveEnd($iThisId, $iLastId)
	{
		$this->Tree->moveEnd($iThisId, $iLastId);
		$this->updatePosition(array($this->get()));
	}

	public function getPosts($iId)
	{
		return $this->Tree->select(
			1,
			array("subject_id"),
			array("parent_id" => $iId, "type" => "post"),
			array(),
			false
		);
		
	}
	
	public function updatePosition($aTree) {
		static $Position = 0;
		foreach ($aTree AS $Item) {
			$this->Tree->updatePositionQuery($Item['node_id'], ++$Position);
			if (isset($Item['nodes'])){
				$this->updatePosition($Item['nodes']);
			}	
		}
	}
}
?>
