<?php
//   список всех групп журналов - WF_FoldersGroupsList::getList();
//   все жруналы в группе - WF_FolderGroup::getFolders()
//   все типы документа в журнале - WF_DoctypesList::getLinked($iId)

include_once "modules/workflow/library/folders/WF_FoldersGroupsList.php";
include_once "modules/workflow/library/folders/WF_FolderGroup.php";
include_once "modules/workflow/library/doctype/WF_DoctypesList.php";
include_once "modules/workflow/library/doctype/WF_DoctypesListDB.php";
include_once "modules/structure/library/Struct_Tree.php";
include_once "Struct_WorkflowsSecurityManagerDB.php";
include_once "modules/references/library/Reference/UserReferenceDB.php";
include_once "modules/references/library/Reference/UserReference.php";

class Struct_WorkflowsSecurityManager
{
	private $aStructureList = array();
	
	private $aDoctypes		= array();
	
	private $aFolders 		= array();
	
	private $AccessDB 		= null;
	
	private $aRefPosts 		= array();
	
	private $aRefSub 		= array();
	
	private $sColDelim 		= "/^/col/^/";

	public function __construct(MSF_DB $oDb, $aStructureList)
	{
		$this->aFolders			= $this->getFoldersList($oDb);
		$this->aDoctypes		= $this->getDoctypesList($oDb);
		$this->AccessDB 		= new Struct_WorkflowsSecurityManagerDB($oDb);
		$this->aStructureList 	= $aStructureList;
		
		$Ref = new UserReference(new UserReferenceDB($oDb));
		$Ref->load(3);
		$this->aRefPosts = $Ref->getAsArray();
		$Ref->load(4);
		$this->aRefSub = $Ref->getAsArray();
	}

	public function getFolders()
	{
		$aResult = array();
		$aAccess = $this->getAccessFolders();
		
		foreach ($this->aFolders as $iFolderId => $sTitle) {
			foreach ($aAccess as $iStructureId => $aValue) {
				if (isset($aValue[$iFolderId])) {
					$aResult[$iStructureId][$iFolderId]["title"] 	= $sTitle;
					$aResult[$iStructureId][$iFolderId]["can_view"] = $aValue[$iFolderId]["can_view"];
				}
			}
		}
		
		return $aResult;
	}
	
	public function getDoctypes()
	{
		$aResult = array();
		$aAccess = $this->getAccessDoctypes();
		
		foreach ($this->aDoctypes as $iDoctypeId => $sTitle) {
			foreach ($aAccess as $iStructureId => $aValue) {
				foreach($aValue AS $iKey => $aAccessInfo) {				
					if ($aAccessInfo["doctype_id"] == $iDoctypeId) {
						$sType 	= "";
						$iId 	= 0;
						$aRef 	= array();
						$sAuth 	= "";
						$sName	= "";
						
						for ($i = 0; $i < count($this->aStructureList); $i++) {
							if ($this->aStructureList[$i]["node_id"] == $aAccessInfo["author_id"]) {
								$sType 	= $this->aStructureList[$i]["type"];
								$iId 	= $this->aStructureList[$i]["subject_id"];
								break;
							}
						}
						
						if ($sType == "post") {
							$aRef = $this->aRefPosts;
						}
						else {
							$aRef = $this->aRefSub;
						}
						
						for ($i = 0; $i < count($aRef); $i++) {
							if ($aRef[$i]["id"] == $iId) {
								$sAuth = $aRef[$i][$sType];
								$sName = $aRef[$i]["employee_ref_value"];
								break;
							}
						}
						
						$aAuth = explode($this->sColDelim, $sAuth);
						$aName = explode($this->sColDelim, $sName);
						if($sType == "post") {
							$sAuth = $aName[0] . " (" . $aAuth[0] . ")";	
						}
						else {
							$sAuth = $aAuth[0];
						}
											
						$aResult[$iStructureId][$iKey]["title"] 		= $sTitle;
						$aResult[$iStructureId][$iKey]["author"] 		= $sAuth;
						$aResult[$iStructureId][$iKey]["can_view"] 		= $aAccessInfo["can_view"];
						$aResult[$iStructureId][$iKey]["can_remove"] 	= $aAccessInfo["can_remove"];						
					}
				}
			}
		}
		
		return $aResult;
	}

	private function getFoldersList($oDb)
	{
		$aResult 			= array();
		$aFolderGroupsList 	= $this->getFolderGroupsList();
		$oFolderGroup 		= new WF_FolderGroup(new WF_FolderGroupDb($oDb));
		
		for ($i = 0; $i < count($aFolderGroupsList); $i++) {
			$oFolderGroup->load($aFolderGroupsList[$i]);
			$aList = $oFolderGroup->getFolders();
			
			for ($j = 0; $j < count($aList); $j++) {
				$aResult[$aList[$j]["folder_id"]] = $aList[$j]["name"];
			}
		}
		
		return $aResult;
	}
	
	private function getFolderGroupsList()
	{
		$aResult 	= array(0);
		$oList 		= new WF_FoldersGroupsList();
		$aList 		= $oList->getList();
		foreach ($aList AS $aItem) {
			$aResult[] = $aItem["group_id"];
		}
		
		return $aResult;
	}

	private function getDoctypesList(MSF_DB $oDb)
	{
		$aResult 		= array();
		$oDoctypesList 	= new WF_DoctypesList(new WF_DoctypesListDB($oDb));
		$aDoctypes 		= $oDoctypesList->getAll();
		
		for ($i = 0; $i < count($aDoctypes); $i++) {
			$aResult[$aDoctypes[$i]["doctype_id"]] = $aDoctypes[$i]["title"];
		}
		
		return $aResult;
	}
	
	private function getAccessFolders()
	{
		$aResult = array();
		$aAccess = $this->AccessDB->selectFolders();
		
		foreach ($aAccess as $aItem) {
			$iStructureId 	= $aItem["structure_id"];
			$iFolderId 		= $aItem["folder_id"];
			
			if (!isset($aResult[$iStructureId])) {
				$aResult[$iStructureId] = array();
			}
			if (!isset($aResult[$iStructureId][$iFolderId])) {
				$aResult[$iStructureId][$iFolderId] = array();
			}
			
			$aResult[$iStructureId][$iFolderId]["can_view"] = $aItem["can_view"];
		}
		
		$aCorrectResult = array();		
		$oStruct  = new Struct_Main();
		$aStrList = $oStruct->getList();
		foreach($aStrList AS $iKey => $aValue) {
			$iId = $aValue["node_id"];
			if(!isset($aResult[$iId])) {
				$aResult[$iId] = array();			
			}
			
		}
				
		foreach($aResult AS $iStrId => $aValue) {
			$aParents = Struct_Utils::getParents($iStrId);
			$aIds 	  = array();
			for($i = 0; $i < count($aParents); $i++) {
				$aIds[] = $aParents[$i]["node_id"];	
			}
			
			$aCorrectResult[$iStrId] = $aResult[$iStrId];
			for($i = 0; $i < count($aIds); $i++) {				
				foreach($aResult[$aIds[$i]] AS $iKey => $aResValue) {
					$aCorrectResult[$iStrId][$iKey] = $aResValue;						
				}			
				
			}
			
		}
		
		return $aCorrectResult;
	}
	
	private function getAccessDoctypes()
	{
		$aResult = array();
		$aAccess = $this->AccessDB->selectDoctypes();
		
		foreach ($aAccess as $aItem) {
			$iStructureId 	= $aItem["structure_id"];			
			if (!isset($aResult[$iStructureId])) {
				$aResult[$iStructureId] = array();
			}
			
			$iCount = count($aResult[$iStructureId]) + 1;
			$iIndex = $this->findDoctype($aItem["doctype_id"], $aItem["author_id"], $aResult[$iStructureId]);
			if($iIndex == -1) {
				$aResult[$iStructureId][$iCount]["doctype_id"]	= $aItem["doctype_id"];
				$aResult[$iStructureId][$iCount]["author_id"] 	= $aItem["author_id"];
				$aResult[$iStructureId][$iCount]["can_view"] 	= $aItem["can_view"];
				$aResult[$iStructureId][$iCount]["can_remove"] 	= $aItem["can_remove"];
			}
			else {
				if($aItem["can_view"] == 2) {
					$aResult[$iStructureId][$iIndex]["can_view"] 	= $aItem["can_view"];
				}
				
				if($aItem["can_remove"] == 2) {
					$aResult[$iStructureId][$iIndex]["can_remove"] 	= $aItem["can_remove"];
				}
				
			}
			
		}		
		
		$aCorrectResult = array();		
		$oStruct  = new Struct_Main();
		$aStrList = $oStruct->getList();
		foreach($aStrList AS $iKey => $aValue) {
			$iId = $aValue["node_id"];
			if(!isset($aResult[$iId])) {
				$aResult[$iId] = array();			
			}
			
		}
				
		foreach($aResult AS $iStrId => $aValue) {
			$aParents = Struct_Utils::getParents($iStrId);
			$aIds 	  = array();
			for($i = 0; $i < count($aParents); $i++) {
				$aIds[] = $aParents[$i]["node_id"];	
			}
			
			$aCorrectResult[$iStrId] = $aResult[$iStrId];
			for($i = 0; $i < count($aIds); $i++) {				
				foreach($aResult[$aIds[$i]] AS $iKey => $aResValue) {
					$iIndex = $this->findDoctype($aResValue["doctype_id"], $aResValue["author_id"], $aCorrectResult[$iStrId]);
					if($iIndex == -1 && !$this->checkExRights($aResValue["doctype_id"], $aResValue["author_id"], $aValue)) {						
						$aCorrectResult[$iStrId][] = $aResValue;
					}
											
				}			
				
			}
			
		}
		
		return $aCorrectResult;
	}
	
	private function findDoctype($iDoctypeId, $iAuthorId, $aSource)
	{
		$iIndex = -1;
		foreach($aSource AS $iKey => $aValue) {
			if($aValue["doctype_id"] == $iDoctypeId && $aValue["author_id"] == $iAuthorId) {
				$iIndex = $iKey;
				break;			
			}
			
		}
		
		return $iIndex;		
	}
	
	private function checkExRights($iDoctypeId, $iAuthorId, $aSource)
	{
		$bHasRights = false;
		$aParents 	= Struct_Utils::getParents($iAuthorId);		
		$aIds = array();
		foreach($aParents AS $aValue) {
			if($aValue["node_id"] != $iAuthorId) {
				$aIds[] = $aValue["node_id"];		
			}
			
		}
		
		foreach($aSource AS $aValue) {			
			if($aValue["doctype_id"] == $iDoctypeId && in_array($aValue["author_id"], $aIds)) {
				$bHasRights = true;
				break;		
			}
			
		}
		
		return $bHasRights;		
	}
	
}

?>