<?php
if(!defined("__MSF")) {
	die("Direct access is not allowed");
}
require_once "WF_Control.php";
include_once 'modules/workflow/actions/view/WFA_AbstractViewPort.php';
include_once 'WFA_ControlViewPort.php';
include_once 'system/MSF_Locale.php';
include_once 'system/utils/MSF_XmlWrapper.php';
include_once 'modules/main/library/orgStructure/MSF_OrgStructureFacade.php';
include_once 'modules/references/library/Reference/REF_Value.php';

class WFA_Control extends WF_ActionSupport implements WF_IAction
{
	
	private $oControl;
	
	private $iDocumentId;
	
	private $isControlled;
	
	public function initialize()
	{
		$oDoc = $this->getDocument();
		$this->iDocumentId 	= ($oDoc) ? $oDoc->getDocumentId():0;
		
		$this->oControl    	= new WF_Control();
		$this->isControlled = $this->oControl->isControlled($this->iDocumentId);
	}
	
	public function processExecute()
	{
		$aResultParam = $this->emulateRequestContextParams();

		if ($this->getParam('post', 'status') == 'enabled') {
			$aResultParam = $this->getControlArray();
			
			if (!isset($aResultParam["finish_at"]) || $aResultParam["finish_at"] == "") {
				$this->getLog()->display($this->getActionTitle(), 
										   MSF_Registry::get('MSF_Locale')->getWord("WF_CONTEXT_EXECUTION_NOT_CORRECT_PARAMS"),
										   MSF_Registry::get('MSF_Locale')->getWord("WF_CONTEXT_EXECUTION_NOT_CORRECT_PARAMS") . "\n" 
										 . MSF_Registry::get('MSF_Locale')->getWord("WORKFLOW_CONTROL_NOT_DATE")
										 );
				return self::SUCCESS;
			}
			if (isset($aResultParam["finish_at"]) && $aResultParam["finish_at"] < $aResultParam["start_at"]) {
				$this->getLog()->display($this->getActionTitle(), 
										   MSF_Registry::get('MSF_Locale')->getWord("WF_CONTEXT_EXECUTION_NOT_CORRECT_PARAMS"),
										   MSF_Registry::get('MSF_Locale')->getWord("WF_CONTEXT_EXECUTION_NOT_CORRECT_PARAMS") . "\n" 
										 . MSF_Registry::get('MSF_Locale')->getWord("WORKFLOW_CONTROL_EXPIRED_DATE")
										 );
				return self::SUCCESS;
			}
		}
		return $this->processRequest();
	}
	
	private function emulateRequestContextParams()
	{
		$aCfg 				= $this->getSettings()->getPureActions($this->getActionId());
		$oView 				= new WFA_ControlViewPort(
				$this->getDoctype(),
				$this->getDoctypeVersion(),
				$this->getDocument(),
				$this->getDocumentVersion(),
				$this->getStructureId(),
				null
				);
		$aParams 			= $oView->prepareBlank($aCfg['options']);
		$aParams['status']	= MSF_SL::getParam($aCfg['options'], 'status');
	
		$this->setParam('post', $aParams);
		
		return $aParams;
	}
	
	private function getControlArray()
	{
		$aControl 						= array();
		$aControl['document_id']		= $this->iDocumentId;
		$aControl['action_id']			= $this->getActionId();
		$aControl['number_of_points']	= $this->getNumberOfPoints();
		$aControl['document_position'] 	= $this->getDocumentPosition();
		$aControl['start_at']			= time();
		$aControl['finish_at']			= $this->getTimeByDatetime($this->getParam('post', 'datetime'));//
		$aControl['is_expired']			= 0;
		$aControl['controller']			= $this->getParam('post', 'controller');
		$aControl['number_of_mails']	= (int) $this->getParam('post', 'number_of_mails', 0);
		$aControl['executor']			= $this->getParam('post', 'executor');
		return $aControl;
	}
	
	public function processRequest()
	{	
		$Locale 						= MSF_Registry::get("MSF_Locale");
		$aControl 						= $this->getControlArray();
		if ($this->getParam('post', 'status') == 'enabled') {
			$aAction = $this->getAction();				
			$oStructureFacade = MSF_OrgStructureFacade::getInstance();
			$oRefValue = new REF_Value();
			$oRefValue->parse($aControl['executor']);
			$aKeysList = $oRefValue->getKeysList();
			$sExecutor = $aControl['executor'];
			if ($aControl['executor'] != "") {
				$Fields = new MSF_XmlWrapper($this->getDoctypeVersion()->getFieldsXml());
				foreach ($Fields as $Data) {
					if (isset($aAction["options"]["executor_value"])) {
						if ($Data->getName() == $aAction["options"]["executor_value"]) {
							$aValue = $Data->getValue("optional");
							if ($aValue["postEnable"] == 0) {
								foreach ($aKeysList AS $KeyValue) {
									$aPost = $oStructureFacade->getPostByStructureId($KeyValue[0]);
									$sExecutor = str_replace('(' . $aPost[0]["value"] . ')', '', $sExecutor);	
								}

							}

							break;
						}

					}
					
				}
				
			}
			
			if ($aControl['number_of_mails'] > 0) {
				$aControl['mailing_timeout']	= $this->getMailingTimeout($aControl['finish_at'], $aControl['start_at'], $aControl['number_of_mails']);
				$aControl['notify_at']			= $this->getNotifyAt($aControl['finish_at'], $aControl['start_at'], $aControl['mailing_timeout'], $aControl['number_of_mails']);
			}
			else {
				$aControl['mailing_timeout']	= 0;
				$aControl['notify_at']			= $aControl['finish_at'];
			}
		
			$aIncorrectFields = array();
			if($aControl['finish_at'] == "" && ((isset($aAction["options"]["executor_necessary"]) && $aControl['executor'] != "")
				|| (!isset($aAction["options"]["executor_necessary"])))) {
				$aIncorrectFields[] = $Locale->getWord("WFA_CONTROL_DATE");				
			}
			else if(isset($aAction["options"]["executor_necessary"]) && $aControl['executor'] == "" && $aControl['finish_at'] != "") {
				$aIncorrectFields[] = $Locale->getWord("WFA_CONTROL_EXECUTOR");				
			}
			else if(isset($aAction["options"]["executor_necessary"]) && $aControl['executor'] == "" && $aControl['finish_at'] == "") {
				$aIncorrectFields[] = $Locale->getWord("WFA_CONTROL_DATE");
				$aIncorrectFields[] = $Locale->getWord("WFA_CONTROL_EXECUTOR");				
			}

			if(!empty($aIncorrectFields)) {				
				MSF_Registry::get("MSF_Messenger")->addError($Locale->getWord("WORKFLOW_FULFILL_FIELDS_CORRECTLY") . implode(", ", $aIncorrectFields));
				return self::NULL;
			}

			//debug(789);
			//$oControl->removeByDocumentId($iDocumentId);	
			if ($aControl['finish_at'] > $aControl['start_at']) {								
				$this->logEnabled($aControl['start_at'], $aControl['finish_at'], $aControl['controller'], $aControl['executor'], $aControl['number_of_mails']);
				if ($this->isControlled) {
					$this->oControl->removeByDocumentId($this->iDocumentId);
				}				
				$this->oControl->add($aControl);
				
				if (isset($aAction["options"]["rewrite_fields"]) && ($aControl['executor'] != "")) {
					$this->saveStructureFieldValue($aAction["options"]["executor_value"], $sExecutor);
				}
				
				if (isset($aAction["options"]["rewrite_data_fields"])){
					$aRewriteField = $this->getDocumentVersion()->getRefFields()->getField($aAction["options"]["deadline_value"]);
					$sRewriteValue = $this->getParam('post', 'datetime');
					if (!empty($aRewriteField)) {
						$this->getDocumentVersion()->setFields(array($aAction["options"]["deadline_value"] => $sRewriteValue));
						$this->getDocumentVersion()->save();
					}
				}
				
				if (isset($aAction["options"]["record_data_field"])){
					$aDataField = $this->getDocumentVersion()->getRefFields()->getField($aAction["options"]["record_data_field"]);
					$sValue = $this->getParam('post', 'datetime');
					if (!empty($aDataField)) {
						$this->getDocumentVersion()->setFields(array($aAction["options"]["record_data_field"] => $sValue));
						$this->getDocumentVersion()->save();
					}
				} 
				$this->callMove();
				return self::SUCCESS;
			}
			else {
				MSF_Registry::get("MSF_Messenger")->addError($Locale->getWord("WORKFLOW_CONTROL_EXPIRED_DATE"));
				return self::NULL;
			}
			
		}
		else {
			if ($this->isControlled) {
				$this->oControl->removeByDocumentId($this->iDocumentId);
			}
			$this->logDisabled($aControl['start_at'], $aControl['finish_at'], $aControl['controller'], $aControl['executor']);
			$this->callMove();		
			return self::SUCCESS;	
		}
					
		
	}

	private function callMove()
	{
		$aRoute		= $this->getSettings()->getActions($this->getActionId());
		if (isset($aRoute["options"])) {
			if (isset($aRoute["options"]["move_points"])) {		
				$this->call("move", array("iNewPoint" => $aRoute["options"]["move_points"]));
			}
		}
	}

	private function getNumberOfPoints()
	{
		$aSettings = $this->getSettings()->getPureActions($this->getActionId());
		return (int)MSF_SL::getParam($aSettings['options'], 'number_of_points');
	}
	
	private function getDocumentPosition()
	{
		$iDocumentPosition 	= $this->getDocument()->getPointId();
		if ($this->isControlled) {
			$aControl = $this->oControl->findOneByDocumentId($this->iDocumentId);
			
			if ($aControl['action_id'] == $this->getActionId()) {
				$iDocumentPosition = $aControl['document_position'];
			}	
		}
		
		return $iDocumentPosition;
	}
	
	private function getMailingTimeout($iFinishAt, $iStartAt, $iNumberOfMails)
	{
		$aSettings 			= $this->getSettings()->getPureActions($this->getActionId());
		$fInactivePeriod	= $aSettings['settings']['inactive_period'];
		return (int)(($iFinishAt - $iStartAt) * $fInactivePeriod / $iNumberOfMails);
		
	}
	
	public function getNotifyAt($iFinishAt, $iStartAt, $iTimeout, $iNumberOfMails)
	{
		$aSettings 			= $this->getSettings()->getPureActions($this->getActionId());
		$fInactivePeriod	= $aSettings['settings']['inactive_period'];
		if ($iNumberOfMails == 1) {
			return (int)(($iFinishAt - $iStartAt) * $fInactivePeriod + $iStartAt);
		}
		else {
			return (int)(($iFinishAt - $iStartAt) * $fInactivePeriod + $iStartAt + $iTimeout) - 60;	
		}

	}
	
	private function getTimeByDatetime($sDatetime)
	{
		$oDate = new MSF_Date();
		$oDate->parse($sDatetime);
		return $oDate->getTime();
	}
	
	private function getAction()
	{
		return $this->getSettings()->getActions($this->getActionId());
	}

	public function processResponse()
	{
		return self::FAILURE;
	}
	
	private function logEnabled($iStartAt, $iFinishAt, $sController, $sExecutor, $iNumberOfMails)
	{
		$this->getLog()->display(
				$this->getActionTitle(), 
				$this->getShortTextControlEnabled(), 
				$this->getFullTextControlEnabled($iStartAt, $iFinishAt, $sController, $sExecutor, $iNumberOfMails)
				);
	}
	
	private function logDisabled()
	{
		$this->getLog()->display(
				$this->getActionTitle(), 
				$this->getShortTextControlDisabled()
				);
	}
	

	private function getFullTextControlEnabled($iStartAt, $iFinishAt, $sController, $sExecutor, $iNumberOfMails)
	{
		$oStartDate 	 = new MSF_Date($iStartAt);
		$oFinishDate 	 = new MSF_Date($iFinishAt);
		$sResult 		 = $this->getShortTextControlEnabled() . "\n";
		$sResult 		.= MSF_Registry::get('MSF_Locale')->getWord("WFA_CONTROL_START") . ' ' . $oStartDate->format('d.m.Y H:i') . "\n";
		// Если поля "Контролер" и "Исполнитель" не заполнены, то не пишем их в лог
		if($sController != "") {
			$sResult 		.= MSF_Registry::get('MSF_Locale')->getWord("WFA_CONTROL_CONTROLLER") . ' ' . $this->structureValueToTitle($sController) . "\n";
		}
		
		if($sExecutor != "") {
			$sResult 		.= MSF_Registry::get('MSF_Locale')->getWord("WFA_CONTROL_EXECUTOR") . ': ' . $this->structureValueToTitle($sExecutor) . "\n";
		}
		
		$sResult 		.= MSF_Registry::get('MSF_Locale')->getWord("WFA_CONTROL_DATE") . ': ' . $oFinishDate->format('d.m.Y H:i') . "\n";
		$sResult 		.= MSF_Registry::get('MSF_Locale')->getWord("WFA_CONTROL_REM_KOL_LOG") . ' ' . $iNumberOfMails . "\n";

		return $sResult;
	}
	

	private function getFullTextControlDisabled($iStartAt, $iFinishAt, $sController, $sExecutor)
	{
		$oStartDate 	 = new MSF_Date($iStartAt);
		$oFinishDate 	 = new MSF_Date($iFinishAt);
		$sResult 		 = $this->getShortTextControlDisabled() . "\n";
		return $sResult;
	}
	
	private function structureValueToTitle($sStructureValue)
	{
		$oRv = new REF_Value();
		$oRv->parse($sStructureValue);
		return implode(', ', $oRv->getValuesList());
	}
	
	private function getShortTextControlEnabled()
	{
		return MSF_Registry::get('MSF_Locale')->getWord("WFA_CONTROL_ON");	
	}

	private function getShortTextControlDisabled()
	{
		return MSF_Registry::get('MSF_Locale')->getWord("WFA_CONTROL_OFF");
	}
	
}

?>