<?php
require_once "modules/workflow/actions/control/WF_Control.php";
require_once "modules/workflow/actions/control/WF_ControlDateFormatter.php";
require_once "modules/workflow/actions/control/WF_WorkDateUtil.php";

 class WFA_ControlViewPort extends WFA_AbstractViewPort
{
	/**
	 * @return void
	 */
	public function port(array $aActionParams, array $aCurrentDelegated)
	{
		try {
			$iDocumentId	= $this->oDocument->getDocumentId();
			$oControl 		= new WF_Control();
			$aOptions		= $aActionParams['options'];
			if (!isset($aOptions['show_controler'])) {
				$aOptions['show_controler'] = 'off';
			}
			if (!isset($aOptions['show_mails'])) {
				$aOptions['show_mails'] = 'off';
			}
			
			if ($aOptions["executor_type"] == "fields") {
				$oFields 						= new REF_Fields($this->oDoctypeVersion->getFieldsXml());
				$aExecutorField 				= $oFields->getField($aOptions["executor_value"]);
				$aOptions["executor_optional"]	= $aExecutorField["optional"];
			}
			
			if ($oControl->isControlled($iDocumentId)) {
				$aControl 			= $oControl->findOneByDocumentId($iDocumentId);
				if ($aControl['action_id'] == $aActionParams['spa_id']) {
					$aOptions = $this->prepareFilled($aOptions, $oControl->findOneByDocumentId($iDocumentId));
				}
				else {
					$aOptions = $this->prepareBlank($aOptions);
				}			
			}
			else {
				$aOptions = $this->prepareBlank($aOptions);			
			}			
			
		}
		catch (Exception $e) {
			
		}

		
		return $aOptions;
	}
	
	private function prepareFilled(array $aOptions, array $aControl)
	{
		$aOptions['status']				= 'enabled';
		// Проверка, надо ли показывать поле "Контролер"
		$aOptions['controller']			= $aControl['controller'];	
		$aOptions['executor']			= $aControl['executor'];
		// Проверка, надо ли показывать поле "Кол. предупреждений"
		$aOptions['number_of_mails']	= $aControl['number_of_mails'];
		
		$oDate 							= new MSF_Date($aControl['finish_at']);
		$aOptions['datetime']			= $oDate->format('d.m.Y H:i');
		return $aOptions;
	}
	
	public function prepareBlank(array $aOptions)
	{		
		// Проверка, надо ли показывать поле "Контролер"
		$aOptions['controller'] 	= $this->getStructureValue(
												$this->oDocument, 
												$aOptions['controller_type'],
												MSF_SL::getParam($aOptions, 'controller_value')
												);
												
		$aOptions['executor'] 		= $this->getStructureValue(
												$this->oDocument, 
												$aOptions['executor_type'], 
												MSF_SL::getParam($aOptions, 'executor_value')
												);
		
		if ($aOptions['deadline_type'] == 'after') {
			$oDate = new MSF_Date(
					WF_WorkDateUtil::getRealTimeByAfter(
							time(),   
							MSF_SL::getParam($aOptions, 'deadline_value_days'),
							MSF_SL::getParam($aOptions, 'deadline_value_hours'), 
							MSF_SL::getParam($aOptions, 'deadline_value_minuts')
							)
					);
					
			$aOptions['datetime'] = $oDate->format('d.m.Y H:i');
		}
		else {
			$oDf = new WF_ControlDateFormatter();
			$aOptions['datetime'] = $oDf->getDatetimeByDeadline(
					$this->oDocument->getCurrentVersion(),
					MSF_SL::getParam($aOptions, 'deadline_type'),
					MSF_SL::getParam($aOptions, 'deadline_value')
					);
		}
		
		$aOptions['status']			= 'disabled';
		// Проверка, надо ли показывать поле "Кол. предупреждений"		
		return $aOptions;
	}
	
	private function getStructureValue(WF_Document $oDoc, $sType, $sValue) {
		if (!empty($sValue)){
			$oV = new WF_Values($oDoc, $oDoc->getCurrentVersion(), $oDoc->getDoctypeVersion());
			$oV->toStructureIds($sValue, true);
			return $oV->getStructureValues();
		}
		else {
			return $sValue;
		}
	}
	
}
?>