<?php
defined("__MSF")
	or die("Direct access is not allowed");
	
include_once "modules/workflow/library/actions/WF_Points.php";
include_once "modules/workflow/library/variables/WF_Vars.php";
include_once "modules/workflow/actions/create/WFA_CreateDb.php";
include_once "modules/workflow/library/security/WF_DoctypeSecurityManagerDb.php";
include_once "modules/workflow/library/security/WF_DoctypeSecurityManager.php";
include_once "modules/workflow/library/security/WF_DocumentSecurityManagerDB.php";
include_once "modules/workflow/library/security/WF_DocumentSecurityManager.php";

include_once "modules/workflow/library/actions/WF_DocCache.php";

/**
 * Creating of a document
 */
class WFA_Create extends WF_ActionSupport implements WF_IAction
{
	/**
	 * @var array
	 */
	private $aFields 		= array();
	
	/**
	 * @var WF_Doctype
	 */
	private $Doctype		= null;
	
	/**
	 * @var WF_DoctypeVersion
	 */
	private $DoctypeVersion = null;
	
	/**
	 * @var WFA_CreateDb
	 */
	private $oDb			= null;
	
	/**
	 * @var MSF_BackUrl
	 */
	private $BackUrl 		= null;
	
	/**
	 * @var int
	 */
	private $iParentId 		= false;
	
	/**
	 * @var boolean
	 */
	private $isExection 	= false;
	
	private $DBRecord		= null;
	
	private $sError			= null;
	
	private $aActionOptions	= array();
	
	/**
	 * Initialization of the action
	 * @return void
	 */
	public function initialize()
	{
		$this->oDb = new WFA_CreateDb(MSF_Registry::get("MSF_DB"));
		if ($this->getDocument() !== null) {
			$aConfig 			= $this->getConfig();
			if (isset($aConfig["doctype_id"])) {
				$this->iParentId 	= $this->getDocument()->getDocumentId();
				try {
					$this->Doctype 		= new WF_Doctype($aConfig["doctype_id"]);
				}
				catch (Exception $e) {
					
				}
			}

		}
		else {
			if ($this->getDoctype() == null) {
				$this->Doctype = new WF_Doctype($this->getInitParam("subject_id"));
			}
			else {
				$this->Doctype = $this->getDoctype();
			}
			
		}
		
		if ($this->Doctype) {
			$this->DoctypeVersion 	= $this->Doctype->getCurrentVersion();
			$this->prepareDoctypesList();
		}

	}
	
	/**
	 * Processing of the "Execution" context
	 * @return integer
	 */
	public function processExecute() {		
		if (self::isCycled()) {
			MSF_Registry::get("MSF_Messenger")->addError(
					MSF_Registry::get("MSF_Locale")->getWord("WORKFLOW_CYCLE_10")
					);
		}
		else {
			$this->isExection = true;
			$this->processRequest();
		}

		return self::SUCCESS;
	}
	
	/**
	 * Processing of the Delegation context in the request mode
	 * @return integer
	 */
	public function processRequest()
	{			
		try {			
			$CurrentDocument 	= $this->getDocument();
			
			if ($CurrentDocument == null) {
				$iStructureId	= MSF_Registry::get("MSF_UserBox")->getCurrentStructureId();
				$iEmployeeId	= MSF_Registry::get("MSF_UserBox")->getEmployeeAttribute("id");
				$sEmployee		= MSF_Registry::get("MSF_UserBox")->getEmployeeAttribute("name");
				$sPost			= MSF_Registry::get("MSF_UserBox")->getEmployeeAttribute("post");			
								
				$Document 		= $this->DoctypeVersion->newDocument();
				$Document->setParentId(0);
				$this->createDocument($CurrentDocument, $Document, $iStructureId, $iEmployeeId, $sEmployee, $sPost);
				return self::NULL;
			}
			else {		
				$aStructureIds = $this->getStructureIdFromConfig(
						$this->getConfig(),
						$CurrentDocument
						);		
				if($this->getParam("get", "execut_doc") != ""){
					$oExecutDoc = $this->getDoctype()->getDocument($this->getParam("get", "execut_doc"));
					$oExecutDoc->getLog()->setDelegated($this->getParam("get", "execut_point"), 
														$this->getParam("get", "execut_action"), 
														array(1)
														);
					$oExecutDoc->save();		
				}
				if (empty($aStructureIds)) {
					$CurrentDocument->getLog()->display(
							$this->getActionTitle(),
							MSF_Registry::get("MSF_Locale")->getWord("WFA_DOC_NOT_CREATED")
							);
					$CurrentDocument->save();		
				}
				else {
					$this->callMove();
					$CurrentDocument->save();
					foreach ($aStructureIds AS $iStructureId) {
						if ($this->Doctype != null && $this->Doctype->exists()) {
							$Document = $this->DoctypeVersion->newDocument();
							$this->prepareFields($CurrentDocument, $Document);
							$Document->setParentId($CurrentDocument->getDocumentId());	
							$this->createDocument($CurrentDocument, $Document, $iStructureId);
						}
						else {
							$sMessage = MSF_Registry::get("MSF_Locale")
									->getWord("WORKFLOW_CREATE_NONE_DOCTYPE");
							$sTitle	= MSF_Registry::get("MSF_Locale")
									->getWord("WF_CONTEXT_EXECUTION_NOT_CORRECT_PARAMS");
							if ($this->isExection) {
								$this->getLog()->setEmployeeName($this->sEmployeeName);
								$this->getLog()->display($this->getActionTitle(), $sTitle, $sTitle . "\n" . $sMessage);
							}
							else {
								MSF_Registry::get("MSF_Messenger")->addError($sMessage);
							}

						}
					}
					return self::SUCCESS;	
				}


			}
			
			//

			
		}
		catch (Exception $oE) {
			//if (CNF_MODE == "debug") {
				throw $oE;
			//}
			///else {
			//	MSF_Registry::get("MSF_Messenger")->addError(MSF_Registry::get("MSF_Locale")->getWord("WORKFLOW_CANT_CREATE_DOC"));
			//	return self::NULL;				
			//}
			
			
		}
		
		return self::NULL;
	}

	private function callMove()
	{
		$aRoute		= $this->getSettings()->getActions($this->getActionId());
		if (isset($aRoute["options"])) {
			if (isset($aRoute["options"]["move_points"])) {		
				$this->call("move", array("iNewPoint" => $aRoute["options"]["move_points"]));
			}
		}
	}

	private function createDocument($CurrentDocument, $Document, $iStructureId, $iEmployeeId = null, $sEmployee = null, $sPost = null)
	{
		$oF 			= MSF_OrgStructureFacade::getInstance();
		$sEmployee		= ($sEmployee == null) ? $oF->getEmployeeNameByStructureId($iStructureId) : $sEmployee;
		$iEmployeeId	= ($iEmployeeId == null) ? $oF->getEmployeeIdByStructureId($iStructureId) : $iEmployeeId;
		if ($sPost == null) {
			$aPost 		= $oF->getPostByStructureId($iStructureId);
			if (isset($aPost[0]["value"])){
				$sPost 	= $aPost[0]["value"];
			}
		}
		
		$this->fieldsDefault($Document, $iStructureId, $sEmployee, $iEmployeeId, $sPost);
	
		$MediaId = new WF_MediaId();
		if ($this->getParam("post", "media_id") == 0) {
			$MediaId->generate();
			$Document->setMediaId($MediaId->get());
		}
		else {
			$Document->setMediaId($this->getParam("post", "media_id"));
		}					
		
		$Document->setStructureId($iStructureId);
		$isNewVersion = $this->saveDocumentVersion(
				$Document,
				$iStructureId,
				$sEmployee,
				$iEmployeeId,
				$sPost
				);				
				
		
		if ($isNewVersion) {
			if ($CurrentDocument !== null) {
				if ($this->isExection) {
					$CurrentDocument->getLog()->display(
							$this->getActionTitle(),
							$Document->getDoctype()->getTitle()
							);					
				}
				else {
					$CurrentDocument->getLog()->display(
							$this->getActionTitle(),
							MSF_Registry::get("MSF_Locale")
									->getWord("WFA_CHILD_DOCUMNET_CREATED")
							);						
				}
			}
			
			$Document->save();				
			
			if ($this->isExection) {
				$this->zeroPointInit($Document);
			}			
		}
		else {
			$Document->remove();
			$MediaId->save();
			if ($this->isExection && !empty($this->sError)) {
				$CurrentDocument->getLog()->display(
							$this->getActionTitle(),
							$this->sError
							);
			}
			
		}
		
	}
	
	private static $iLaunchCount = 0;
	
	private static function isCycled() {
		return self::$iLaunchCount++ > 10;
	}
	
	/**
	 * Processing of the Delegation context in the response mode
	 * @return integer
	 */
	public function processResponse()
	{	
		// Zaplatka, kogda net dostupa i nazhata knopka nazad
		if (!$this->DoctypeVersion) {
			MSF_Registry::get("MSF_Messenger")
			->redirect("/workflow/document/view/" . $this->getDocument()->getDocumentId());
			exit(); 
		}
		
		$WFTemplate 	= new REF_Template($this->DoctypeVersion->getTemplateXml());
		$iStructureId 	= MSF_OrgStructureFacade::getInstance()->getCurrentStructureId();
		$WFTemplate->setEmployeeName(MSF_OrgStructureFacade::getInstance()->getEmployeeNameByStructureId($iStructureId));
		$WFTemplate->setStructureId($iStructureId);
		$aPost = MSF_OrgStructureFacade::getInstance()->getPostByStructureId($iStructureId);
		if (isset($aPost[0]["value"])){
			$WFTemplate->setPost($aPost[0]["value"]);
		}
		$Fields 	= new REF_Fields($this->DoctypeVersion->getFieldsXml());
		$Template 	= $this->getTemplate();
		$WFVars 	= new WF_Vars();
		if ($this->getDocument() !== null) {
			$this->prepareFields(
					$this->getDocument(),
					$this->DoctypeVersion
							->getDoctype()
							->newDocument()
					);
		}
		
		$aFields 	= (array)$this->getParam("post");
		if (!empty($aFields)) {
			foreach ($aFields AS $sName => $mValue) {
				//if (!is_array($mValue)) {
					$Fields->set($sName, $mValue);
				//}

			}
		
		}

		$WFTemplate->setFields($Fields);
		$WFTemplate->setVars($WFVars);
		$WFTemplate->setPoints(new WF_Points(
				$this->getDocument(),
				count(MSF_XmlWrapper::xmlToArray(
						$this->DoctypeVersion->getRouteXml()
						))
				));
		$WFTemplate->setUniqueId($this->Doctype->getDoctypeId());
		$WFTemplate->setUniqueType(MSF_UniqueField::TYPE_DOCTYPE);
		
		$sPath 	= (string) 	$this->getParam("get", "msf_path");
		$iId 	= (int) 	substr($sPath, strrpos($sPath, "/") + 1);
		
		$Template->getModule()->setAttribute("id", $iId);
		$Template->addLink("/js/ext/calendar_date_select/js/calendar_date_select.js", "text/javascript");
		$Template->addLink("/js/ext/calendar_date_select/css/default.css", "text/css");

		$this->addLinksOf("references");
		$this->addLinksOf("structure");
		$oMediaId = new WF_MediaId();
		$WFTemplate->setExtra("workflow_doctype", $oMediaId->get());
		$WFTemplate->addEditTemplate($this->Template, "module/template");
		if ($this->Doctype->getFolderId() > 0) {
			MSF_Registry::get("MSF_ActionBar")->addAction(
				MSF_Registry::get("MSF_Locale")->getWord("WORKFLOW_TO_FOLDER"),
				"/workflow/folders/" . $this->Doctype->getFolderId()
			);
		}
		if ($Fields->getCount() > 0 && $WFTemplate->existFieldsInTemplate()) {
			MSF_Registry::get("MSF_ActionBar")->addAction(
				MSF_Registry::get("MSF_Locale")->getWord("WORKFLOW_BTN_SAVE"),
				"", CNX_ACTIONBAR_IMG, '$("save_button").click(); return false;'
			);
		}

		return self::SUCCESS;
	}
	
	private function getConfig() {
		$aAction = $this->getSettings()->getPureActions($this->getActionId());
		$this->aActionOptions = $aAction["options"];
		return $aAction["options"];
		/*
		//$iPointId	= $this->getDocument()->getPointId();
		//$aConfig 	= $this->getDocument()->getLog()->getDelegated(
		//		$iPointId,
		//		$this->getActionId()
		//		);
		if (empty($aConfig)) {
			$aTemp = array_merge(
					$this->getSettings()->getExecutionAtPoint($iPointId),
					$this->getSettings()->getDelegatedAtPoint($iPointId)
					);
			foreach ($aTemp AS $aActionCfg) {
				if ($aActionCfg["spa_id"] == $this->getActionId()) {
					$aConfig = $aActionCfg["options"];
					break;
				}
				
			}
			
		}
		
		if(empty($aConfig)) {
			$aTemp = array_merge(
					$this->getSettings()->getExecution(),
					$this->getSettings()->getDelegated()
					);
			
			foreach ($aTemp AS $aActionCfg) {
				if ($aActionCfg["spa_id"] == $this->getActionId()) {
					$aConfig = $aActionCfg["options"];
					break;
				}
				
			}		
			
		}
		
		return $aConfig;
		*/
	}

	private function getStructureIdFromConfig(array $aConfig, WF_Document $oDocument) {
		$oValues = new WF_Values(
				$oDocument,
				$oDocument->getCurrentVersion(),
				$oDocument->getDoctypeVersion()
				);

		return array_unique($oValues->toStructureIds($aConfig["address"]["value"]));
	}

	
	private function saveDocumentVersion(WF_Document $Document, $iStructureId, $sEmployeeName, $sEmployeeId, $sPost)
	{
		$Locale 			= MSF_Registry::get("MSF_Locale");
		$Date 				= new MSF_Date();
		$oUserBox 			= MSF_Registry::get("MSF_UserBox");
		$oMessenger 		= MSF_Registry::get("MSF_Messenger");
		$DocumentVersion 	= $Document->newVersion();
		$DocumentOldVersion	= $Document->getCurrentVersion();
		//debug($DocumentVersion->getFields(), $this->getParam("post"));
		///exit();
		$aFields = array_merge($DocumentVersion->getFields(), $this->getParam("post"));
		$DocumentVersion->setFields($aFields);
		
		foreach ($aFields AS $sName => $mValue) {
			if ($DocumentVersion->getRefFields()->getFieldType($sName) == "document") {
				$aDocument = $DocumentVersion->getRefFields()->getField($sName);
				if (isset($aDocument["optional"]["selectThisDoc"]) && $aDocument["optional"]["selectThisDoc"] == 1) {
					$sDocValue = htmlspecialchars_decode($DocumentVersion->buildTitle()) . "/^/col/^/"
									   . $DocumentVersion->getDocumentId() . "/^/col/^/"
									   . $DocumentVersion->getDocument()->getDoctype()->getFolderId();
									   
					$DocumentVersion->setField($sName, $sDocValue);
				}
			}		
		}

		$this->setParam("post", $DocumentVersion->getFields());
		$aUnfilledFields 	= $DocumentVersion->getUnfilledFields();
		$aIncorrectFields 	= $DocumentVersion->getIncorrectFields();
		$bCheck 			= false;

		if (empty($aIncorrectFields) || $this->isExection) {
			if (empty($aUnfilledFields) || $this->isExection) {
				$UF	= new MSF_UniqueField(MSF_Registry::get("MSF_DB"));
				$UF->freeFields(
						$DocumentOldVersion->getFieldsAsList(),
						MSF_UniqueField::TYPE_DOCTYPE,
						$this->Doctype->getDoctypeId()
						);
				$UF->blockFields(
						$DocumentVersion->getFieldsAsList(),
						MSF_UniqueField::TYPE_DOCTYPE,
						$this->Doctype->getDoctypeId()
						);
				$RefValue = new REF_Value();
				$iPostId   = MSF_Registry::get("MSF_UserBox")->getCurrentPostId();
				$RefValue->add(
						array($iStructureId, $sEmployeeId, $iPostId),
						$sEmployeeName . " (" . $sPost . ")"
						);
				$sAuthorName = $RefValue->format();
				$RefValue = new REF_Value();
				$RefValue->add(
						$oUserBox->getStructureAttribute("subdivision_id"),
						$oUserBox->getStructureAttribute("subdivision")
						);
				$sSubdivision = $RefValue->format();
				$DocumentVersion->setSystemField("author_name", $sAuthorName);
				$DocumentVersion->setSystemField("subdivision", $sSubdivision);
				$Date = new MSF_Date();
				$DocumentVersion->setSystemField("created_at", $Date->formatTimestamp());
				$DocumentVersion->setSystemField("modified_at", $Date->formatTimestamp());
				$DocumentVersion->setSystemField("doctype", $this->Doctype->getTitle());
				$aRoute = MSF_XmlWrapper::xmlToArray($this->DoctypeVersion->getRouteXml());
				if (isset($aRoute[0])) {
					$DocumentVersion->setSystemField("status", $aRoute[0]["title"]);
				}

				$DocumentVersion->setStructureId($iStructureId);
				$DocumentVersion->setEmployeeName($sEmployeeName);
				$DocumentVersion->setCreatedAt($Date->format());
				$DocumentVersion->save();
				$this->updateDocFamily(
						$Document->getDoctypeId(),
						$Document->getParentId(),
						$Document->getDocumentId(),
						$Document->getFamilyTree()
						);
				
				if ($this->iParentId > 0) {
					$DocumentSM = new WF_DocumentSecurityManager(
							MSF_Registry::get("MSF_DB"),
							$this->getDocument()->getDocumentId()
							);
					$aAccess 	= $DocumentSM->get();
					$DocumentSM = new WF_DocumentSecurityManager(
							MSF_Registry::get("MSF_DB"),
							$DocumentVersion->getDocumentId()
							);
					
					for ($i = 0; $i < sizeof($aAccess); $i++) {
						$DocumentSM->edit(array("structure_id" =>$aAccess[$i]["structure_id"]));
					}
					$DocumentSM->edit(array("structure_id" => $this->getDocument()->getStructureId()));
				}
				
				$bCheck = true;
			}
			else {
				if ($this->isExection) {
					$this->sError = $Locale->getWord("ERROR_TITLE") . ": " 
									. $Locale->getWord("WORKFLOW_FULFILL_FIELDS") . implode(", ", $aUnfilledFields);
				}
				else {
					$oMessenger->addError($Locale->getWord("WORKFLOW_FULFILL_FIELDS") . implode(", ", $aUnfilledFields));
				}
			}

		}
		else {
			if ($this->isExection) {
				$this->sError	= $Locale->getWord("ERROR_TITLE") . ": " 
								  . $Locale->getWord("WORKFLOW_FULFILL_FIELDS_CORRECTLY") . implode(", ", $aIncorrectFields);
			}
			else {
				$oMessenger->addError(
						$Locale->getWord("WORKFLOW_FULFILL_FIELDS_CORRECTLY") . implode(", ", $aIncorrectFields)
						);
			}
			/*$aFields	= $DocumentVersion->getFieldsAsList();
			$Media		= new MD_Media(new MD_MediaDB(MSF_Registry::get("MSF_DB")));
			foreach($aFields as $aField) {
				
				if(in_array($aField["type"], array("file", "picture"))) {
					if((string)$aField["value"] != ""){
						try {
							$Media->remove($aField["value"]);
							$DocumentVersion->setField($aField["name"], 0);
						}
						catch (Exception  $E) {

						}

					}

					if(isset($aSession[$aField["name"]])) {
						unset($aSession[$aField["name"]]);
					}
					
				}

			}

			$this->setParam("post", $DocumentVersion->getFields());*/
		}
		
		if($bCheck) {
			$this->setRedirectUrl(
					$this->getRedirectUrlToDocument($DocumentVersion->getDocumentId())
					);
			return true;
		}
		else {
			$this->isRedirectable(false);
			return false;
		}

	}

	
	private function prepareDoctypesList()
	{
		$aValue 	= $this->getParam("value");
		$aDoctypes 	= array();
		if (isset($aValue["doctypes"])) {
			for ($i = 0; $i < sizeof($aValue["doctypes"]); $i++) {
				$Doctype = new WF_Doctype($aValue["doctypes"][$i]);
				$aDoctypes[$i]["id"] 	= $Doctype->getDoctypeId();
				$aDoctypes[$i]["title"] = $Doctype->getTitle();
			}

			$this->setParam(
				"value",
				array("doctypes" => $aDoctypes)
			);

		}

	}
	
	private function updateDocFamily($iDoctypeId, $iParentId, $iCurrentDocumentId, $sFamilyTreeXml)
	{
		$aCurrentDocument	= array(
				"parent_id" 	=> $iParentId,
				"document_id"	=> $iCurrentDocumentId,
				"family_tree"	=> $sFamilyTreeXml,
				"doctype_id"	=> $iDoctypeId,
				);
		
		$aDocuments	= $this->getDocuments($aCurrentDocument);
		if (isset($aDocuments[1])) {
			$iOldParentId	= $aDocuments[1]["parent_id"];
			$iOldDocumentId	= $aDocuments[1]["document_id"];
			$sOldFamilyTree	= $aDocuments[1]["family_tree"];
		}
		else {
			$iOldParentId	= 0;
			$iOldDocumentId	= 0;
			$sOldFamilyTree	= "<document/>";
		}

		$aOldFamilyTree  = MSF_XmlWrapper::xmlToArray($sOldFamilyTree);

		if (empty($aOldFamilyTree)) {
			$aNewFamilyTree = array(
					array(
							"id" 	  => $aCurrentDocument["document_id"],
							"visible" => (isset($this->aActionOptions["family_tree_insert"])) 
										  ? $this->aActionOptions["family_tree_insert"] : 1,
							"node" 	  => array()
							)
							
					);
	
		}
		else {
			$aOldFamilyTree 		= $aOldFamilyTree["root"]["value"];
			$aAddedDocuments 		= array();
			$aDocumentsInTreeIds 	= array();
			$aDocumentsInTreeIds	= $this->treeToList($aOldFamilyTree);
			foreach ($aDocuments AS $aDocument) {
				$aAddedDocuments[] = $aDocument["document_id"];
			}

			foreach ($aDocumentsInTreeIds AS $iDocumentId) {
				if (!in_array($iDocumentId, $aAddedDocuments)) {
					$aDocuments[] = $this->oDb->selectDocument($iDocumentId);
				}

			}

			$aNewFamilyTree = $this->makeNewFamilyTree(
					$aOldFamilyTree,
					$aCurrentDocument["parent_id"],
					$aCurrentDocument["document_id"]
					);
		}

		$oXw = new MSF_XmlWrapper();
		$oXw->set("root", $aNewFamilyTree);
		$sNewFamilyTree = $oXw->getXml();

		foreach ($aDocuments AS $aDocument) {
			if (empty($aDocument)) {
				continue;
			}
			
			$this->oDb->updateDocument($aDocument["document_id"], $sNewFamilyTree);
			$oCache = new WF_DocCache($aDocument["doctype_id"], $aDocument["document_id"]);
			$oCache->remove();
		}
		
		if ($LoadedDocument = $this->getDocument()) {
			$LoadedDocument->setFamilyTree($sNewFamilyTree);
		}

	}
	
	private function treeToList($aNodeList)
	{
		$aList = array();
		foreach ($aNodeList AS $aNode) {
			$aList[] = $aNode["id"];
			foreach ($this->treeToList($aNode["node"]) AS $aTempList) {
				$aList[] = $aTempList;
			}

		}

		return $aList;
	}
	
	private function makeNewFamilyTree($aOldFamilyTree, $iParentId, $iDocumentId)
	{
		$aResultTree = array();
		foreach ($aOldFamilyTree AS $aNode) {
			$aTempNode = array();
			$aTempNode["id"] 	  = $aNode["id"];
			$aTempNode["visible"] = (isset($aNode["visible"])) ? $aNode["visible"] : 1;
			$aTempNode["node"]	  = $this->makeNewFamilyTree($aNode["node"], $iParentId, $iDocumentId);
			if ($aNode["id"] == $iParentId) {
				$aTempNode["node"][] = array(
						"id" 	  => $iDocumentId,
						"visible" => (isset($this->aActionOptions["family_tree_insert"])) ? $this->aActionOptions["family_tree_insert"] : 1,
						"node" 	  => array()
						);
			}

			
			$aResultTree[] = $aTempNode;

		}

		return $aResultTree;
	}
	
	private function getDocuments($aDocument)
	{
		$aDocuments 	= array();
		$aDocuments[]	= $aDocument;
		do {
	
			$aDocument 		= $this->getParent($aDocument["parent_id"]);
			if (!empty($aDocument)) {
				$aDocuments[] 	= $aDocument;
			}

		}
		while (!empty($aDocument["document_id"]));
		
		return $aDocuments;
	}
	
	private function getParent($iNodeId)
	{
		return $this->oDb->selectDocument($iNodeId);
	}
	
	private function getTree($aOldFamilyTree, $aDocuments, $iParentId)
	{
		$aResultTree = array();
		foreach ($aDocuments AS $aDocument) {
			if ($aDocument["parent_id"] == $iParentId) {
				$aResultTree[] = array(
						"id" 	=> $aDocument["document_id"],
						"node"	=> $this->getTree($aDocuments, $aDocument["document_id"])
						);

			}

		}

		return $aResultTree;
	}
	
	private function zeroPointInit(WF_Document $oDocument) {
		$oAb 		= WF_ActionBox::newInstance();
		$oAb->loadAction("view", 0, $oDocument->getDocumentId(), null, true);
		$oAction	= $oAb->getAction();
		$oEc		= new WF_ExecutionContext($oAction);
		$oEc->execute();
		$oAction->getDocument()->save();
	}
	
	
	
	private function fieldsDefault(WF_Document $Document, $iStructureId, $sEmployee, $sEmployeeId, $sPost)
	{
		$Fields 	= new MSF_XmlWrapper($Document->getDoctype()->getFieldsXml());
		$aFields 	= $this->getParam("post");
		foreach ($Fields as $Data) {
			if (!array_key_exists($Data->getName(), $aFields)) {
				$aValue = $Data->getValue();
				switch ($aValue["type"]) {
					case "structure":
						if (isset($aValue["optional"])) {
							if (isset($aValue["optional"]["selectAuthor"]) && $aValue["optional"]["selectAuthor"] == 1) {
								$iPostId   = MSF_Registry::get("MSF_UserBox")->getCurrentPostId();
								$sEmployee = RefFieldHelper::formatStructureValue(
												$sEmployeeId, 
												$iPostId, 
												MSF_SL::getParam($aValue["optional"], "employee", "name"), 
												MSF_SL::getParam($aValue["optional"], "post", "post"), 
												MSF_SL::getParam($aValue["optional"], "employeeEnable", 1), 
												MSF_SL::getParam($aValue["optional"], "postEnable", 1),
												true
										   );
								$RefValue = new REF_Value();
								$RefValue->add(array($iStructureId, $sEmployeeId, $iPostId), $sEmployee);
								$aFields[$Data->getName()] = $RefValue->format();
							}
							else {
								$aFields[$Data->getName()] = $aValue["defaultValue"];
							}
						}
						break;
					case "timestamp":
						if (isset($aValue["optional"]["setCurrentDate"]) && $aValue["optional"]["setCurrentDate"] == 1) {
							$Date = new MSF_Date();
							$aFields[$Data->getName()] = $Date->format(CNF_DATE_TIME_FORMAT);
						}
						break;
					case "date":
						if (isset($aValue["optional"]["setCurrentDate"]) && $aValue["optional"]["setCurrentDate"] == 1) {
							$Date = new MSF_Date();
							$aFields[$Data->getName()] = $Date->format(CNF_DATE_FORMAT);
						}
						break;
				}
			}
			
			if ($Data->getValue("type") == "date") {
				//$sValue = $Data->getValue("defaultValue");
				if (!empty($sValue)) {
					$Date = new MSF_Date();
					$Date->parse($sValue);
					$aFields[$Data->getName()] = $Date->format(CNF_DATE_FORMAT);
				}
			}
			if ($Data->getValue("type") == "timestamp") {
				//$sValue = $Data->getValue("defaultValue");
				if (!empty($sValue)) {
					$Date = new MSF_Date();
					$Date->parse($sValue);
					$aFields[$Data->getName()] = $Date->format(CNF_DATE_TIME_FORMAT);
				}
			}
		}

		$this->setParam("post", $aFields);
	}
	
	
	private function prepareFields(WF_Document $CurrentDocument, WF_Document $Document) {
		$aPost 				= (array)$this->getParam("post");
		//
		$sPrntFieldsXml 	= $CurrentDocument->getDoctype()->getFieldsXml();
		$aPrntFileds		= MSF_XmlWrapper::xmlToArray($sPrntFieldsXml);
		$aPrntValues		= $CurrentDocument->getCurrentVersion()->getFields();
		//
		$sChldFieldsXml 	= $Document->getDoctype()->getFieldsXml();
		$aChldFileds		= MSF_XmlWrapper::xmlToArray($sChldFieldsXml);
		//
		$aRslt				= array();
		$aFConfig			= $this->getFieldsConfig();
		
		//debug($aPost, $aPrntFileds, $aChldFileds);
		foreach ($aChldFileds AS $sFieldname => $aField) {
			if (isset($aFConfig[$sFieldname]) && isset($aPrntValues[$aFConfig[$sFieldname]])) {
				$mValue = $this->getFieldValue(
						$sFieldname,
						$aPost,
						$aPrntValues[$aFConfig[$sFieldname]]
						);
				switch ($aField["value"]["type"]) {
					case "structure":
						if ($aField["value"]["optional"]["multiselect"]) {
							$aRslt[$sFieldname] = $mValue;
						}
						else {
							$sExtract = $this->extractStructValue($mValue);
							if ($sExtract) $aRslt[$sFieldname] = $sExtract;
						}
						/*$aRslt[$sFieldname] = $this->getStructureValue(
								$this->parseValue($mValue),
								(bool) $aField["value"]["optional"]["postEnable"]
								);*/
						break;
					case "reference":
						if (isset($aField["value"]["optional"]["fieldsValue"])){
							$FieldsValue = $aField["value"]["optional"]["fieldsValue"];
						}
						else {
							$FieldsValue = "";
						}
						$aRslt[$sFieldname] = $this->getReferenceValue(
								$aField["value"]["optional"]["reference"],
								$FieldsValue,
								$this->parseValue($mValue),
								$aField["value"]["optional"]["separator"]
								);
						break;
					case "table":
						if ($mValue != "") {
							$aVar 		= explode("^R^", $mValue); 
							$aVar2		= explode("^C^", $aVar[1]);
							$aVar[0]	= $aVar2[0];
							$i = 0;
							foreach ($aChldFileds[$sFieldname]["value"]['optional']['fields'] AS $Key => $Value){
								$aVar2[$i++] = $Key;
							}
							$aVar[1] 				= implode("^C^", $aVar2);
							$aRslt[$sFieldname] 	= implode("^R^", $aVar);
						}
						else {
							$aRslt[$sFieldname] = $mValue;
						}
						break;
					default:
						$aRslt[$sFieldname] = $mValue;
						break;
				}
				
			}

		}
		
		foreach ($aPost AS $sFieldname => $mValue) {
			if (!isset($aRslt[$sFieldname])) {
				$aRslt[$sFieldname] = $mValue;
			}
			
		}

		$this->setParam("post", $aRslt);
	}
	private function extractStructValue($mValue) {
		$sResult = "";
		$aRow = explode("|^|row|^|", $mValue);
		$aCol = explode("/^/col/^/", $aRow[0]);
		if (isset($aCol[3])) {
			$sResult = $aRow[0]; 
		}
		return $sResult;
	}
	
	private function getFieldValue($sFieldname, $aPost, $sDefault) {
		$mValue = null;
		if (array_key_exists($sFieldname, $aPost)) {
			$mValue = $aPost[$sFieldname];
		}
		else {
			$mValue = $sDefault;
		}
		
		//debug($sFieldname, $aPost, $sDefault, $mValue);
		return $mValue;
	}
	
	private function parseValue($sRv) {
		$oRv = new REF_Value();
		$oRv->parse($sRv);
		return $oRv->getKeysList();
	}
	
	private function getFieldsConfig() {
		$aTemp = $this->getConfig();
		if (!isset($aTemp["fields"])) {
			return array();
		}
		return $aTemp["fields"];
	}
	
	private function getStructureValue($aIdsList, $bWithPost) {
		$aStrIds = array();
		foreach ($aIdsList AS $aIds) {
			$aStrIds[] = $aIds[0];
		}

		$oOs = MSF_OrgStructureFacade::getInstance();
		return $oOs->strIdsToRefValue($aStrIds, $bWithPost);
	}

	private function getReferenceValue($iRefId, $aClmns, $aValue, $sDelimiter) {
		$oUr = new UserReference(
				new UserReferenceDB(
						MSF_Registry::get("MSF_DB")
						)
						
				);
				
		$oUr->load($iRefId);
		$oRv = new REF_Value();
		foreach ($oUr->getAsArray() AS $aValues) {
			foreach ($aValue AS $iRowId) {
				if ($iRowId == $aValues["id"]) {
					$aRowValue = array();
					foreach ((array)$aClmns AS $sColumn) {
						if (isset($aValues[$sColumn])) {
							$aRowValue[] = $aValues[$sColumn];
						}
						
					}
					// nujno esli v spravo4nike est' pole tipa spravo4nik
					// potomu 4to ina4e id polya(tip spravo4nik) i id spravo4nika nakladivayutsia
					$oRv->parse(implode($sDelimiter, $aRowValue));
					$aResult = implode($sDelimiter, $oRv->getValuesList());
					$oRv->reset();
					$oRv->add($iRowId, $aResult);
				}
				
			}

		}

		return $oRv->format();
	}
	
	
}

?>