<?php

defined('__MSF') or die('Direct access is not allowed');

include_once "modules/media/library/MD_MediaDB.php";
include_once "modules/media/library/MD_Media.php";

class WFA_Decision extends WF_ActionSupport implements WF_IAction
{
	private $bMoved = false;
	
	private function getTitle()
	{
		$aSpa = $this->getSettings()->getPureActions($this->getActionId());
		//return !empty($aSpa["options"]["window_title"]) ? $aSpa["options"]["window_title"] : $aSpa["title"];
		return $aSpa["title"];
	}
	
	private function getDisplayParams($aParams)
	{
		$aSpa = $this->getDelegate();
		if (!empty($aParams["selected_variant"])) {
			foreach ($aSpa["options"] as $k => $v) {
				if (substr_count($k, "variant_") > 0 && $k == $aParams["selected_variant"]) {
					$aParams["text"] = $v["text"];
					break;
				}
			}
		}
		
		$sSmall = isset($aParams["text"]) ? $aParams["text"] : $aParams["window_title"];
		$sFull 	= "";
		
		if (!empty($aParams["comment"])) {
			$sFull .= $sSmall;
			$sFull .= "<br/>" . str_replace("\r\n", "", nl2br(str_replace("  ", "&#160; ", htmlspecialchars($aParams["comment"]))));			
			//$sFull .= "\n" . $aParams["comment"];
		}
		if (!empty($aParams["files"])) {
			if (empty($sFull)) {
				$sFull .= $sSmall;
			}
			foreach ($aParams["files"] as $aFile) {
				try {
					$sFull .= 	"<br/>" .
					//$sFull .= 	"\n" .
								MSF_Registry::get("MSF_Locale")->getWord("WORKFLOW_DOCTYPE_SPA_FILE") . ": " .
								"<a href=\"" . CNF_HOST_URL . "/media/download/" . $aFile["value"] . "\" target=\"_blank\">" . $aFile["name"] . "</a>";
				}
				catch (Exception $e) {}
			}
		}
		
		return array($sSmall, $sFull);
	}
	
	public function processRequest()
	{
		if (!$this->isMovable()) {
			$sMessage = MSF_Registry::get("MSF_Locale")
					->getWord("WF_CONTEXT_EXECUTION_NOT_FINISHED_CORRECTLY");
			MSF_Registry::get("MSF_Messenger")->addError($sMessage);
			return self::NULL;
		}
		
		$Media 	= new MD_Media(new MD_MediaDB(MSF_Registry::get("MSF_DB")));
		$aFIds 	= array();
		$aSpa 	= $this->getDelegate();
		$aSIds 	= array();
		if (isset($aSpa["employeeIds"])) {
			$aSIds = $aSpa["employeeIds"];
		}
		
		if (isset($_POST["decision_added_file"])) {
			$aFiles = $_POST["decision_added_file"];
			for ($i = 0; $i < count($aFiles["name"]); $i++) {
				$aTmp 			= array();
				$aTmp["name"] 	= $aFiles["name"][$i];
				$aTmp["value"] 	= $aFiles["value"][$i];
				$aFIds[] 		= $aTmp;
			}
		}
                $useEDS = "";
                $sign = "";
		if(isset($_POST["use_eds"])){
                    $useEDS = $_POST["use_eds"];
                    $sign = $_POST["fields_value"] . "||" . $_POST["eds_sign_cert"];
                }
		if (isset($_FILES["decision_file"]["name"])) {
			$aTmpFiles = array();
			$Media->setMaxFileSize(1000000000);
			
			for ($i = 0; $i < count($_FILES["decision_file"]["name"]); $i++) {
				if ($_FILES["decision_file"]["error"][$i] == 0) {
					$aTmp 				= array();
					$aTmp["name"] 		= $_FILES["decision_file"]["name"][$i];
					$aTmp["type"] 		= $_FILES["decision_file"]["type"][$i];
					$aTmp["tmp_name"] 	= $_FILES["decision_file"]["tmp_name"][$i];
					$aTmp["error"] 		= $_FILES["decision_file"]["error"][$i];
					$aTmp["size"] 		= $_FILES["decision_file"]["size"][$i];
					$aTmpFiles[] 		= $aTmp;
				}
			}
			
			for ($i = 0; $i < count($aTmpFiles); $i++) {
				$aTmp 			= array();
				$aTmp["name"] 	= $aTmpFiles[$i]["name"];
				$aTmp["value"] 	= $Media->upload($aTmpFiles[$i], "workflow_document", $this->getDocument()->getMediaId());
				$aFIds[] 		= $aTmp;
			}
		}
		
		$Date 	= new MSF_Date();
		$aSave 	= $this->getLog()->getDelegated($this->getDocument()->getPointId(), $this->getActionId());
		if (!isset($aSpa["options"]["window_title"])) $aSpa["options"]["window_title"] = "";
		$aLog 	= array(
			"time" 				=> time(),
			"window_title" 		=> $aSpa["options"]["window_title"],
			"comment" 			=> $this->getParam("post", "decision_comment"),
			"selected_variant" 	=> $this->getParam("post", "decision_variant"),
			"files" 			=> $aFIds,
			"struct_id"			=> $this->getStructureId(),
			"employeeName" 		=> MSF_Registry::get("MSF_UserBox")->getEmployeeAttribute("name"),
			"post" 				=> $this->getPostParam(),
			"created_at" 		=> $Date->format(CNF_DATE_TIME_FORMAT),
			"ids" 				=> $aSIds,
			"sign"				=> ($useEDS != "" && $useEDS == "1") ? $sign : self::sign($this->getDelegate(), $this->getDocumentVersion())
		);
		if (MSF_SL::getParam($aSpa["options"], "comment1", 1) == 0) {
			$aSave["recs"][time()] = $aLog;
		}
		else {
			if (isset($aSave["recs"])) {
				foreach ($aSave["recs"] AS $Key=>$aComment) {
					if ((isset($aComment["struct_id"]) && $aComment["struct_id"] == $this->getStructureId()) ||
						($aLog["post"] == $aComment["post"] && $aLog["employeeName"] == $aComment["employeeName"])) {
						unset($aSave["recs"][$Key]);
					}
				}
			}
			$aSave["recs"][time()] = $aLog;
		}
		
		$aDisplay = $this->getDisplayParams($aLog);
		$this->getLog()->setDelegated($this->getDocument()->getPointId(), $this->getActionId(), $aSave);
		$this->getLog()->display($this->getTitle(), $aDisplay[0], $aDisplay[1], true);
		$this->sendNotify($this->getParam("post", "decision_variant"), $this->validateText($this->getParam("post", "decision_comment")));
		$this->stopDocument();
		if($this->bMoved) {
			$this->save();
			return self::NULL;
		}
		else {
			return self::SUCCESS;
		}
		
	}
	
	public function sign($aDelegate, $oDocumentVersion)
	{
                //$aFieldsList	 	= $this->getDocument()->getCurrentVersion()->getFieldsAsList();
                //$version	 	= $this->getDocument()->getCurrentVersion();
                
		$aDelegate 	= $aDelegate;
		$sDsf 		= MSF_SL::getParam($aDelegate["options"], "dsf_content", "");
		$sSign		= "";
		if ($sDsf !== "") {
			$aDsfFields = explode(",", $sDsf);
			$aDocFields	= $oDocumentVersion->getFields();
			$sPrepared	= "";
			foreach ($aDsfFields AS $sFieldname) {
				if (isset($aDocFields[$sFieldname])) {
                                    $MediaDB 	= new MD_MediaDB(MSF_Registry::get("MSF_DB"));
                                    $isFileOrPicture = $MediaDB->isTypeFileOrPicture($sFieldname, $oDocumentVersion->getFDocumentId());
                                    if($isFileOrPicture){
                                        $fileId = $aDocFields[$sFieldname];
                                        $fileIdArray = explode(",", $fileId);
                                        foreach($fileIdArray as $valId){
                                            $sPrepared .= $MediaDB->getMD5File($valId);                                            
                                        }
                                    } else {
                                        $sPrepared .= $aDocFields[$sFieldname];
                                    }
                                    /*foreach ($aFieldsList as $Field) {
                                        if($sFieldname == $Field['name']){
                                            switch($Field['type']){
                                                case 'file':
                                                    break;
                                                case 'picture':
                                                    break;
                                            }
                                        }
                                    }*/					
				}
			}
			
			$sSign = md5($sPrepared) . sha1($sPrepared);
                        
		}

		return $sSign;
	}
	
	private function getPostParam()
	{
		$sPost 	= "";
		$UBox 	= MSF_Registry::get("MSF_UserBox");
		$aPosts = $UBox->getPosts();
		
		for ($i = 0; $i < count($aPosts); $i++) {
			if ($aPosts[$i]["id"] == $UBox->getCurrentPostId()) {
				$sPost = $aPosts[$i]["post"];
				if (isset($aPosts[$i]["boss"]) && $aPosts[$i]["boss"] == 1) {
					$sPost .= " " . MSF_Registry::get("MSF_Locale")->getWord("ROOT_POST_IO");
				}
				break;
			}
		}
		
		return $sPost;
	}
	
	public function processResponse()
	{
		return self::FAILURE;
	}
	
	private function stopDocument()
	{
		$aSpa = $this->getDelegate();
		$aSpa = $aSpa["options"];
		$sVar = $this->getParam("post", "decision_variant");
		
		if (isset($aSpa[$sVar])) {
			if (isset($aSpa[$sVar]["stop"]) && $aSpa[$sVar]["stop"] != "") {
				$iStop = (int) $aSpa[$sVar]["stop"];
				$this->call("move", array("iNewPoint" => $iStop));
				if ($iStop !== 0) {
					$this->bMoved = true;		
				}
			
			}
			
		}
		
	}
	
	private function isMovable()
	{
		$aSpa = $this->getDelegate();
		$aSpa = $aSpa["options"];
		$sVar = $this->getParam("post", "decision_variant");
		
		if (isset($aSpa[$sVar])) {
			if (isset($aSpa[$sVar]["stop"]) && $aSpa[$sVar]["stop"] != "") {
				$iStep		= (int) $aSpa[$sVar]["stop"];
				$iPoint 	= $this->getDocument()->getPointId();
				$iSize		= count(MSF_XmlWrapper::xmlToArray($this->getDoctypeVersion()->getRouteXml()));

				if ($iStep > 0) {
					return $iSize - $iPoint > $iStep; 
				}
				else {
					return $iPoint - abs($iStep) >= 0; 
				}
				
			}
			else {
				return true;
			}
			
		}
		else {
			return true;
		}
		
	}
	
	private function getDelegate()
	{
		/*
		$aResult 	= array();
		$aDelegate 	= $this->getSettings()->getDelegatedAtPoint($this->getPointId());
		
		for ($i = 0; $i < count($aDelegate); $i++) {
			if ($aDelegate[$i]["spa_id"] == $this->getActionId()) {
				$aResult = $aDelegate[$i];
				break;
			}
		}

		return $aResult;
		*/
		
		return $this->getSettings()->getPureActions($this->getActionId());
	}
	
	private function sendNotify($sVariant, $sCommment)
	{	
		$aSpa = $this->getSettings()->getPureActions($this->getActionId());
		$aSpa = $aSpa["options"];
		
		if (!empty($aSpa)) {
			$iCurrentStrId 	= MSF_Registry::get("MSF_UserBox")->getCurrentStructureId();
			$iAuthorStrId 	= $this->getDocument()->getStructureId();
			if (isset($aSpa["send_notify"]) && $aSpa["send_notify"] == 1 && $iCurrentStrId != $iAuthorStrId) {
				$Locale = MSF_Registry::get("MSF_Locale");
				$sTitle = $this->getDocument()->getTitle();
		
				$sLink = " <a href=\"" . CNF_HOST_URL . "/workflow/document/view/" . $this->getDocument()->getDocumentId() . "\">" . $sTitle . "</a>";
				$sBody = str_replace(
					"{document}",
					$sLink,
					$Locale->getWord("WORKFLOW_DOCTYPE_SPA_DECISION_NOTIFY")
				) .
				" " . $aSpa["window_title"] .
				"<br/><br/>" .
				MSF_Registry::get("MSF_UserBox")->getEmployeeAttribute("name") . "<br/>" .
				((isset($aSpa[$sVariant])) ? $aSpa[$sVariant]["text"] : "") . "<br/><br/>";
				$sCommentWord 	= MSF_Registry::get("MSF_Locale")->getWord("WORKFLOW_DOCTYPE_SPA_COMMENT");
				$sBody			.= $sCommentWord . (($sCommentWord{strlen($sCommentWord) - 1} != ":") ? ":" : "") . "<br/>" . $sCommment;				
				$this->call(
					"notify",
					array(
						"aStructure" 	=> array($iAuthorStrId),
						//"sSubject" 		=> $Locale->getWord("WORKFLOW_DOCUMENT") . " " . $sTitle,
						"sSubject" 		=> $sTitle,
						"sText" 		=> $sBody
					)
				);
			}
		}
	}
	
	private function validateText($sText)
	{
		$sResult 	= "";		 
		$sResult	= str_replace("\n", "<br>", htmlspecialchars($sText,  ENT_NOQUOTES, "UTF-8"));
		return $sResult;	
	}
}

?>