<?php
if (!defined('__MSF')) {
	exit('Direct access is not allowed');
}

include_once "modules/workflow/library/actions/WF_Points.php";
include_once "modules/references/library/Reference/REF_Template.php";
include_once "modules/workflow/library/variables/WF_Vars.php";

/**
 * 
 */
class WFA_Edit extends WF_ActionSupport implements WF_IAction
{
	/**
	 * @var array
	 */
	private $aFields = array();
	
	public function initialize() {
		if ($this->getInitParam("subject_version_id") == 0) {
			$sUrl = $_REQUEST["msf_path"] . "/" . $this->getDocument()->getCurrentVersion()->getDocumentVersionId();
			if ($this->getParam("get", "execut_doc") != "") {
				$sUrl .= "?&execut_doc="   . $this->getParam("get", "execut_doc") 
					  .  "&execut_point="  . $this->getParam("get", "execut_point")
					  .  "&execut_action=" . $this->getParam("get", "execut_action");
			}
			MSF_Registry::get("MSF_Messenger")
				->redirect($sUrl);
			exit();
		}
	}
	
	/**
	 * @return integer 
	 */
	public function processRequest()
	{
		$iResult = self::FAILURE;
		if (IS_REQUEST) {
			$oOld 		= $this->getDocumentVersion();
			$oNew 		= $this->getDocument()->newVersion();
			$aFields 	= array_merge($oOld->getFields(), $this->getParam("post"));
			$oNew->setFields($aFields);
			if ($this->edit($oNew, $oOld)) {
				$this->log();
				$iResult		= self::SUCCESS;
			}
			else {
				$this->aFields 	= $oNew->getFields();
				$iResult 		= self::NULL;
			}

			try {
				$aRoute		= $this->getSettings()->getActions($this->getActionId());
				if (isset($aRoute["options"])) {
					if (isset($aRoute["options"]["move_points"])) {
						$this->call("move", array("iNewPoint" => $aRoute["options"]["move_points"]));				
					}
				}
			}
			catch(Exception $e) {
				if (CNF_MODE == "debug") {
					debug($e->getMessage());
					exit();
				}	
				return self::FAILURE;
			}
		}
		
		return $iResult;
	}
	
	/**
	 * @return integer 
	 */
	public function processResponse()
	{
		$oFields 		= new REF_Fields($this->getDocument()->getDoctypeVersion()->getFieldsXml());
    		$this->aFields 	= array_merge($this->getDocument()->getCurrentVersion()->getFields(), $this->aFields);
		foreach ($this->aFields AS $sName => $sValue) {
			$oFields->set($sName, $sValue);
		}
		
		$WFTemplate = $this->prepareTemplate();
		$WFTemplate->setFields($oFields);
		$WFTemplate->addEditTemplate($this->getTemplate(), "module/template");
		
		return self::SUCCESS;
	}
	
	/**
	 * @return void
	 */
	private function addErrMsg($sMsg, array $aFields = array())
	{
		$oM = MSF_Registry::get("MSF_Messenger");
		$oM->addError($this->getWord($sMsg) . implode(", ", $aFields));
	}
	
	/**
	 * @return void
	 */
	private function addErrMsgForEditor()
	{
		$oM = MSF_Registry::get("MSF_Messenger");
		$sL = $this->getWord("WORKFLOW_MESSAGE_DOCUMENT_WAS_EDITED");
		$sM = str_replace("EDITER", $this->getDocsEmployeeName(), $sL);
		$oM->addError($sM);
	}

	/**
	 * @return void
	 */
	private function sendErrNotify($sNotify, $iStructureId)
	{
		$sMessage	= $this->getWord($sNotify);
		$sTitle		= $this->getDocument()->getTitle();
		$sMessage  .= "<br /><br />" . $this->getWord("WORKFLOW_DOCUMENT") . ": <br />" . 
					  "<a href=\"" . CNF_HOST_URL . "/workflow/document/view/" . $this->getDocument()->getDocumentId() . "\">" . $sTitle . "</a>";
		$this->call(
			"notify",
			array(
				"aStructure" 	=> array($iStructureId),
				"sSubject"		=> $sTitle,
				"sText" 		=> $sMessage
			)
		);
	}
	
	/**
	 * @return boolean
	 */
	private function edit(WF_DocumentVersion $oNew, WF_DocumentVersion $oOld)
	{
		$bRedirect 			= false;
		$aUnfilledFields 	= $oNew->getUnfilledFields();
		$aIncorrectFields 	= $oNew->getIncorrectFields();
		if (!empty($aIncorrectFields)) {
			$this->addErrMsg("WORKFLOW_FULFILL_FIELDS_CORRECTLY", $aIncorrectFields);
		}
		else if (!empty($aUnfilledFields)){
			$this->addErrMsg("WORKFLOW_FULFILL_FIELDS", $aUnfilledFields);
		}
		else {
			if ($this->equal($oNew, $oOld)) {
				$bRedirect = true;
			}
			else {
				$oPrepared = $this->prepareVersion($oNew, $oOld);
				if ($this->getInitParam("subject_version_id") != $this->getDocsVersionId()) {
					$this->sendErrNotify("WORKFLOW_DOCUMENT_LAST_EDITOR_NOTIFY", $this->getDocsStructureId());
					$this->getDocument()->setStructureId($this->getStructureId());
					$this->sendErrNotify("WORKFLOW_DOCUMENT_EDITOR_NOTIFY", $this->getDocsStructureId());
					$this->addErrMsgForEditor();
				}
				
				$oPrepared->save();
				$bRedirect = true;
			}
			if($this->getParam("get", "execut_doc") != ""){
				$oExecutDoc = $this->getDoctype()->getDocument($this->getParam("get", "execut_doc"));
				$oExecutDoc->getLog()->setDelegated($this->getParam("get", "execut_point"), 
													$this->getParam("get", "execut_action"), 
													array(1)
													);
				$oExecutDoc->save();		
			}
		}
		
		$this->isRedirectable($bRedirect);
		return $bRedirect;
	}
	
	/**
	 * @return void
	 */
	private function prepareVersion(WF_DocumentVersion $oNew, WF_DocumentVersion $oOld)
	{
		$oDate	= new MSF_Date();
		$oUb 	= MSF_Registry::get("MSF_UserBox");
		$oUf	= new MSF_UniqueField(MSF_Registry::get("MSF_DB"));
		$aSys	= $this->getDocument()->getCurrentVersion()->getSystemFields();
		$oDt	= $this->getDoctype();
		$oNew->setSystemField("modified_at", $oDate->formatTimestamp());
		$oOld->setSystemField("modified_at", $oDate->formatTimestamp());
		$oUf->freeFields(
				$oOld->getFieldsAsList(), 
				MSF_UniqueField::TYPE_DOCTYPE, 
				$oDt->getDoctypeId()
				);
		$oUf->blockFields(
				$oNew->getFieldsAsList(), 
				MSF_UniqueField::TYPE_DOCTYPE, 
				$oDt->getDoctypeId()
				);
		$oNew->setStructureId($oUb->getCurrentStructureId());
		$oNew->setEmployeeName($oUb->getEmployeeAttribute("name"));
		$oNew->setCreatedAt($oDate->format());
		$oNew->setSystemFields($aSys);
		return $oNew;
	}
	
	/**
	 * @return int 
	 */
	private function getDocsVersionId()
	{
		return $this->getDocument()->getCurrentVersion()->getDocumentVersionId();
	}
	
	/**
	 * @return boolean 
	 */
	private function equal(WF_DocumentVersion $oNew, WF_DocumentVersion $oOld)
	{
		return $oOld->getFields() == $oNew->getFields();
	}
	
	/**
	 * @return REF_Template
	 */
	private function prepareTemplate()
	{
		$this->addResources();
		$WFTemplate = new REF_Template($this->getDocsTemplate());
		$WFTemplate->setEmployeeName($this->getDocsEmployeeName());
		$WFTemplate->setStructureId($this->getDocsStructureId());
		$WFTemplate->setPost($this->getDocsPost());
		$WFTemplate->setPostId($this->getDocsPostId());
		$WFTemplate->setEmployeeId($this->getDocsEmployeeId());
		$WFTemplate->setUniqueId($this->getDoctype()->getDoctypeId());
		$WFTemplate->setUniqueType(MSF_UniqueField::TYPE_DOCTYPE);
		$WFTemplate->setVars(new WF_Vars($this));
		$WFTemplate->setPoints(new WF_Points($this->getDocument(), $this->getCountOfPoints()));
		$WFTemplate->setExtra("workflow_document", $this->getDocument()->getMediaId());
		return $WFTemplate;
	}
	
	/**
	 * @return void 
	 */
	private function addResources()
	{
		$this->addLinksOf("references");
		$this->addLinksOf("structure");
		$this->getTemplate()->addLink("/js/ext/calendar_date_select/js/calendar_date_select.js", "text/javascript");
		$this->getTemplate()->addLink("/js/ext/calendar_date_select/css/default.css", "text/css");
		
		$this->getActionBar()->addAction(
			$this->getWord("WORKFLOW_ACTION_CLOSE"),
			"/workflow/document/view/" . $this->getDocument()->getDocumentId()
		);
		
		$this->getActionBar()->addAction(
			$this->getWord("WORKFLOW_BTN_SAVE"),
			"", 
			CNX_ACTIONBAR_IMG, 
			'$("save_button").click(); if (!window.opera) return false;'
		);
	}
	
	/**
	 * @return string 
	 */
	private function getDocsPost()
	{
		$oF = MSF_OrgStructureFacade::getInstance();
		$aPost = $oF->getPostByStructureId($this->getDocsStructureId());
		if (count($aPost)) {
			return $aPost[0]["value"];
		}
		else {
			return '';
		}
		
	}
	
	/**
	 * @return integer 
	 */
	private function getCountOfPoints()
	{
		$sXml = $this->getDocument()->getDoctypeVersion()->getRouteXml();
		$aXml = MSF_XmlWrapper::xmlToArray($sXml);
		return count($aXml);
	}
	
	/**
	 * @return string 
	 */
	private function getWord($sWord)
	{
		return MSF_Registry::get("MSF_Locale")->getWord($sWord);
	}
	
	/**
	 * @return MSF_ActionBar
	 */
	private function getActionBar()
	{
		return MSF_Registry::get("MSF_ActionBar");
	}
	
	/**
	 * @return integer 
	 */
	private function getDocsPostId()
	{
		$oF = MSF_OrgStructureFacade::getInstance();
		return $oF->getPostIdByStructureId($this->getDocsStructureId());
	}
	
	/**
	 * @return integer 
	 */
	private function getDocsEmployeeId()
	{
		$oF = MSF_OrgStructureFacade::getInstance();
		return $oF->getEmployeeIdByStructureId($this->getDocsStructureId());
	}
	
	/**
	 * @return string 
	 */
	private function getDocsEmployeeName()
	{
		return $this->getDocument()->getCurrentVersion()->getEmployee();
	}
	
	/**
	 * @return string 
	 */
	private function getDocsTemplate()
	{
		return $this->getDocument()->getDoctypeVersion()->getTemplateXml();
	}
	
	/**
	 * @return integer 
	 */
	private function getDocsStructureId()
	{
		return $this->getDocument()->getStructureId();
	}
	
	/**
	 * @return void 
	 */
	private function log()
	{
		$oLog = $this->getDocument()->getLog();
		$oLog->display(
				$this->getActionTitle(), 
				$this->getWord("WORKFLOW_DOCUMENT_EDITED")
				);
	}

}