<?php
if(!defined("__MSF")) {
	die("Direct access is not allowed");
}

include_once "modules/workflow/actions/record/WFA_RecordUtils.php";

class WFA_Execution extends WF_ActionSupport implements WF_IAction
{
	private $aOptions;

	/**
	 * @var MSF_Locale
	 */
	private $oLocale;
	
	/**
	 * @var MSF_Messenger
	 */
	private $oMessenger;
	
	private $aExtDocumentsIds = array();
	
	private $aPost = array();
	
	public function initialize()
	{
		$this->oLocale	 	= MSF_Registry::get("MSF_Locale");
		$this->oMessenger 	= MSF_Registry::get("MSF_Messenger");		
		$this->initCfg();
		$this->initExtDocumentIds();		
	}
	
	public function processExecute()
	{
		return $this->process("execute");
	}
	
	public function processRequest()
	{
		return $this->process("request");
	}
	
	public function processResponse()
	{}	

	private function initCfg()
	{
		$aCfg 			= $this->getSettings()->getActions($this->getActionId());
		$this->aOptions = $aCfg["options"];
	}
	
	private function initExtDocumentIds()
	{
		switch ($this->aOptions["document_type"]) {
			case "variable":
				switch ($this->aOptions["document_value"]) {
					case "parent":
						$this->aExtDocumentsIds[] = $this->getDocument()->getParentId();
						break;
					case "descendants":
						$oWrapper = new MSF_XmlWrapper();
						$sTree	  = $this->getDocument()->getFamilyTree();						
						$aTree	  = $oWrapper->xmlToArray($sTree);
						$aDocs	  = WF_Utils::getFromFamilyRecursive($aTree["root"]["value"], $this->getDocument()->getDocumentId());
						for($i = 0; $i < count($aDocs); $i++) {
							$this->aExtDocumentsIds[] = $aDocs[$i]["id"];
						}						
						
						break;
				}
				 
				break;
			case "exdocument":				
				$oValue  = new REF_Value();
				$aFields = $this->getDocument()->getCurrentVersion()->getFields();
				$sDocs	 = $aFields[$this->aOptions["document_value"]];
				$oValue->parse($sDocs);								
				$aDocs	= $oValue->getKeysList();								
				for($i = 0; $i < count($aDocs); $i++) {
					if(isset($aDocs[$i][0])) {
						$this->aExtDocumentsIds[] = $aDocs[$i][0];
					}
					else {
						$this->aExtDocumentsIds[] = $aDocs[$i];
					}
										
				}
				
				break;
		}		
		
	}
	
	private function process($sType)
	{		
		$oFacade 		= new WF_DocFacade();				
		if (($this->isSingleMode() && $this->isFirstRun()) || !$this->isSingleMode()) {						
			if ($this->aOptions["operation_type"] == "execution") {
				for($i = 0; $i < count($this->aExtDocumentsIds); $i++) {
					if($this->aExtDocumentsIds[$i] != $this->getDocument()->getDocumentId()) {
						$oExtDocument = $oFacade->getDocumentById($this->aExtDocumentsIds[$i]);
						if($oExtDocument != null) {						 
							if ($this->aOptions["doctype_id"] == $oExtDocument->getDoctypeId()) {
								if ($this->moveToPoint($oExtDocument, $this->aOptions["operation_value"])) {
									//echo ":)))";
									$oFacade->initContextExecution($oExtDocument);
									//exit();
								}
								else {
									$sOutOfLength = $this->oLocale->getWord("WORKFLOW_OUT_OF_LENGTH");
									$this->oMessenger->addError($sOutOfLength);
									$this->log($sOutOfLength, $this->getLinkToExtDocument($oExtDocument));
								}
								
							}
						
						}
					}
					else {						
						$this->oMessenger->addError($this->oLocale->getWord("WFA_EXECUTION_EXEC_DENIED"));	
					}
					
				}
				
				$this->firstRun();
				return self::SUCCESS;
			}
			else if ($this->aOptions["operation_type"] == "delegation") {
				$sExtActionName = $this->getExtActionName();
				$this->aPost	= $this->getParam("post");								
				if ($sExtActionName !== null) {					
					for($i = 0; $i < count($this->aExtDocumentsIds); $i++) {						
						$oExtDocument = $oFacade->getDocumentById($this->aExtDocumentsIds[$i]);
						if($oExtDocument != null) {
							if ($this->aOptions["doctype_id"] == $oExtDocument->getDoctypeId()) {											
					        	$this->execAction($oExtDocument, $sExtActionName, $sType, $this->aOptions["operation_value"]);
							}
							
						}
										        
					}					
					
					$iResult = self::SUCCESS;
				}
				else {
					$iResult = self::SUCCESS;
					if ($sType == "request") {
						$this->oMessenger->addError($this->oLocale->getWord("WORKFLOW_EXT_ACTION_NOT_FOUND"));
					}
				}

		        $this->firstRun();
				return $iResult;
			}
			else if ($this->aOptions["operation_type"] == "extrecord") {
				$sDestField	= substr($this->aOptions["operation_value"], 4);
				$aParams	= array(
					"options" => array()
				);
				
				$oExtDoctype 	= new WF_Doctype($this->aOptions["doctype_id"]);
				$aExtDocFields  = MSF_XmlWrapper::xmlToArray($oExtDoctype->getFieldsXml());
				$aFields 		= $this->getDocumentVersion()->getFields();
				$sValue			= $aFields[$this->aOptions["docfield_name"]];

				if ($aExtDocFields[$sDestField]["value"]["type"] == "string") {
					$oRefFields 	= $this->getDocumentVersion()->getRefFields();
					$sValue = WFA_RecordUtils::castToString($sValue, $oRefFields->getFieldType($this->aOptions["docfield_name"]));
				}
				
				if($sType == "request") {					
					$this->aPost = $this->getParam("post");
					$this->aPost["replace_field"] = $sDestField;
					$this->aPost[$sDestField] = $sValue;
					unset($this->aPost[$this->aOptions["docfield_name"]]);				
					$_POST = $this->aPost;
				}
				else if($sType == "execute") {
					$aParams = array(
						"name"	  	=> "record",
						"title"	  	=> MSF_Registry::get("MSF_Locale")->getWord("ACTION_EXECUTION_RECORD_TITLE"),
						"options" 	=> array(
							"replace_field" => $sDestField,
							"field"			=> "value",
							"value"			=> $sValue
						),
						"settings"	=> array()
					);
															
				}
				
				for($i = 0; $i < count($this->aExtDocumentsIds); $i++) {						
					$oExtDocument = $oFacade->getDocumentById($this->aExtDocumentsIds[$i]);
					if($oExtDocument != null) {										
						if ($this->aOptions["doctype_id"] == $oExtDocument->getDoctypeId()) {																	
				        	$this->execAction($oExtDocument, "record", $sType, 0, $aParams);
						}
						
					}
									        
				}
				
				$this->firstRun();
				return self::SUCCESS;
			}
			else {
				return self::FAILURE;
			}
			
		}
		else {
			// $this->oMessenger->addError($this->oLocale->getWord("WORKFLOW_CANT_LAUNCH_ACTION_TWICE"));
			return self::SUCCESS;
		}


	
	}
	
	private function isSingleMode()
	{
		return $this->aOptions["number_of_operations"] == "single";
	}
	
	private function isFirstRun()
	{
		return !(count((array)$this->getLog()->getDelegated($this->getDocument()->getPointId(), $this->getActionId())) > 0);
	}
	
	private function firstRun()
	{
		$this->getLog()->setDelegated($this->getDocument()->getPointId(), $this->getActionId(), array(1));
	}
	
	private function log($sShort, $sFull = "")
	{
		if ($sFull !== "") {
			$sFull = $sShort . "<br/>" . $sFull;
		}
		
		$this->getLog()->display($this->getActionTitle(), $sShort, $sFull, true);
	}
	
	private function execAction($oExtDocument, $sExtActionName, $sType, $sExtActionId, $aParams = array())
	{	
		$oAb = WF_ActionBox::newInstance();
        $oAb->loadAction($sExtActionName, $sExtActionId, $oExtDocument->getDocumentId(), null, true);
        $oAction = $oAb->getAction();		
        $oAction->setParam("post", $this->aPost);        
        if(!empty($aParams)) {
        	$oAction->getSettings()->injectAction(0, $aParams);
        }
		
        $oAction->getSettings()->setFromExecution();
        $iResult = $oAb->process($sType);              	
        $oAction->getDocument()->save();
        if ($iResult == self::SUCCESS) {
	        if($this->aOptions["operation_type"] == "delegation") {        
	        	$sMessage = MSF_Registry::get("MSF_Locale")->getWord("ACTION_EXECUTION_LOG_EXECUTE") . $oAction->getActionTitle();
	        }
	        else {
	        	$sMessage = MSF_Registry::get("MSF_Locale")->getWord("ACTION_EXECUTION_LOG_RECORD");
	        }
        	$this->log($sMessage, $this->getLinkToExtDocument($oExtDocument));
        }
        else {
        	$this->log($this->oLocale->getWord("WF_CONTEXT_EXECUTION_NOT_CORRECT_PARAMS"));
        }
        
        return $iResult;
	}
	
	private function getLinkToExtDocument(WF_Document $oExtDocument)
	{
		$sHref = CNF_HOST_URL . "/workflow/document/view/" . $oExtDocument->getDocumentId();
		return "<a href=\"" . $sHref . "\">" .$oExtDocument->getTitle(). "</a>";
	}
	
	private function getExtActionName()
	{
		$oDoctype 	= new WF_Doctype($this->aOptions["doctype_id"]);
		$aFields 	= MSF_XmlWrapper::xmlToArray($oDoctype->getCurrentVersion()->getSpaXml());
		if (isset($aFields[$this->aOptions["operation_value"]])) {
			return $aFields[$this->aOptions["operation_value"]]["value"]["action"];
		}
		else {
			return null;
		}
		
	}
	
	private function moveToPoint(WF_Document $oExtDocument, $iNumberOfSteps)
	{
		$sRouteXml	= $oExtDocument->getDoctypeVersion()->getRouteXml();
		$aRoute		=  MSF_XmlWrapper::xmlToArray($sRouteXml);
		$iPointId 	= $oExtDocument->getPointId() + $iNumberOfSteps;
		if (($iPointId < count($aRoute)) && ($iPointId >= 0)) {
		//	$Log	= $this->getLog();
		//	$Log->clearExecuted($iPointId);
		//	$Log->clearWaited($iPointId);				
			$oExtDocument->setPointId($iPointId);
			$oExtVersion = $oExtDocument->getCurrentVersion();
			$oExtVersion->setSystemField("status", $aRoute[$iPointId]["title"]);
			$oExtVersion->save();
	        $sMessage 	= MSF_Registry::get("MSF_Locale")->getWord("ACTION_EXECUTION_LOG_MOVE");
	        $this->log($sMessage, $this->getLinkToExtDocument($oExtDocument));
			return true;
		}
		else {
			return false;
		}
	
	}
	
	public function cancel()
	{
		$Log = $this->getLog();		
		$Log->clearDelegatedAction($this->getDocument()->getPointId(), $this->getActionId());
	}

	

}

?>