<?php
class WFA_ExecutionViewPortBuilder extends WFA_AbstractViewPortBuilder 
{	
	
	/**
	 * @return array|null
	 */	
	public function build(array $aAction)
	{			
		$aCurrLog 				= $this->oDocument->getLog()->getDelegated($this->oDocument->getPointId());
		$aOptions				= $aAction["options"];
		if(empty($aOptions["document_type"])) {
			return null;			
		}
		
		if(empty($aOptions["document_value"])) {
			return null;	
		}
		
		$oExtDocument			= $this->getExtDocument($aOptions["document_type"], $aOptions["document_value"], $aOptions["doctype_id"]);		
		if (isset($aOptions["operation_value"]) && $aOptions["operation_value"] != "") {
			$aAction["ext_spa_id"]	= $aOptions["operation_value"];			
		}
		else {
			return null;			
		}		
		if ($oExtDocument == null) {			
			return null;			
		}		
		else if (($aOptions["number_of_operations"] == "single") && isset($aCurrLog[$aAction["spa_id"]]) && !empty($aCurrLog[$aAction["spa_id"]])) {
			return null;			
		}
		else if ($aOptions["operation_type"] == "delegation") {
			$oDoctype 				= new WF_Doctype($aOptions["doctype_id"]);
			$oDoctypeVersion		= $oDoctype->getCurrentVersion();
			$aAllSpa				= MSF_XmlWrapper::xmlToArray($oDoctypeVersion->getSpaXml());
			if (!isset($aAllSpa[$aOptions["operation_value"]])) {
				return null;
			}
			$aSpa					= $aAllSpa[$aOptions["operation_value"]];
			$aAction["name"]		= $aSpa["value"]["action"];
			$aAction["options"]		= $aSpa["value"]["value"];
			switch ($aAction["name"]) {
				case "create":
					$aAction["url"] 	= "/workflow/document/spa/create/" . $aOptions["operation_value"] . "/" . $oExtDocument->getDocumentId() 
										. "?&execut_doc=" . $this->oDocument->getDocumentId() . "&execut_action=" . $aAction["spa_id"] 
										. "&execut_point=" . $this->oDocument->getPointId();
					break;
				case "edit":
					$aAction["url"] 	= "/workflow/document/spa/edit/" . $aOptions["operation_value"] . "/" . $oExtDocument->getDocumentId()
										. "?&execut_doc=" . $this->oDocument->getDocumentId() . "&execut_action=" . $aAction["spa_id"] 
										. "&execut_point=" . $this->oDocument->getPointId();
					break;
				default:
					$aAction["url"] 	= "/workflow/document/spa/execution/" . $aAction["spa_id"] . "/" . $this->oDocument->getDocumentId();
					break;
			
			}
			
			$this->oDoctype 			= $oDoctype;
			$this->oDoctypeVersion 		= $oDoctypeVersion;
			$this->oDocument 			= $oExtDocument;
			$this->oDocumentVersion 	= $oExtDocument->getCurrentVersion();
			
			return $aAction;
		}
		else if ($aOptions["operation_type"] == "execution") {			
			return $aAction;			
		}
		else if ($aOptions["operation_type"] == "extrecord") {			
			if($aOptions["docfield_name"] != "" && $aOptions["operation_value"] != "") {
				$oFacade = new WF_DocFacade();
				$oDoc	 = $oFacade->getDocumentById($this->getDocument()->getDocumentId());						
				$aFields	 = $oDoc->getCurrentVersion()->getFields();						
				$aAction = array(
					"ext_spa_id"  => 0,
					"spa_id"  	  => $aAction["spa_id"],
					"title"	  	  => $aAction["title"],
					"name"	  	  => "record",
					"description" => "",
					"url"	  => $aAction["url"] 	= "/workflow/document/spa/execution/" . $aAction["spa_id"] . "/" . $this->oDocument->getDocumentId(),
					"options" => array(
						"visible" 	    => 0,
						"label_text"    => "",
						"replace_field" => $aOptions["docfield_name"],
						"field"			=> "value",
						"value"			=> $aFields[$aOptions["docfield_name"]],
						"object_id"		=> 0
					),
					"settings" => array()				
				);
				
				return $aAction;
			}
			else {
				return null;
			}
			
		}
		
	}
	
	private function getExtDocument($sDocumentType, $sDocumewntValue, $iDoctypeId)
	{
		switch ($sDocumentType) {
			case "variable":
				switch ($sDocumewntValue) {
					case "parent":
						$oFacade = new WF_DocFacade();
						return $oFacade->getDocumentById($this->oDocument->getParentId());
						break;
					case "descendants":
						$oWrapper = new MSF_XmlWrapper();
						$sTree	  = $this->oDocument->getFamilyTree();						
						$aTree	  = $oWrapper->xmlToArray($sTree);												
						$aDocs	  = WF_Utils::getFromFamilyRecursive($aTree["root"]["value"], $this->getDocument()->getDocumentId());						
						$oDoc	  = null;
						if(isset($aDocs[0])) {
							$oFacade = new WF_DocFacade();
							for($i = 0; $i < count($aDocs); $i++) {
								$iDocId		= $aDocs[$i]["id"];
								$oTmpDoc	= $oFacade->getDocumentById($iDocId);								
								if($oTmpDoc != null && $oTmpDoc->getDoctypeId() == $iDoctypeId) {
									$oDoc = $oTmpDoc;			
								}								
								
							}	
												 
						}						
						
						return $oDoc;
						break;
				}
				 
				break;
			case "exdocument":				
				$oValue  = new REF_Value();
				$aFields = $this->getDocument()->getCurrentVersion()->getFields();
				$sDocs	 = $aFields[$sDocumewntValue];												
				$oValue->parse($sDocs);				
				$aDocs	 = $oValue->getKeysList();
				$oFacade = new WF_DocFacade();
				$oDoc	 = null;				
				for($i = 0; $i < count($aDocs); $i++) {
					if(isset($aDocs[$i][0])) {
						$iDocId	= $aDocs[$i][0];						 
					}
					else if(isset($aDocs[$i])){					
						$iDocId	= $aDocs[$i];					
					}

					$oTmpDoc	= $oFacade->getDocumentById($iDocId);					
					if($oTmpDoc != null && $oTmpDoc->getDoctypeId() == $iDoctypeId) {
						$oDoc = $oTmpDoc;			
					}					
					
				}
				
				return $oDoc;
				break;
			default:
				return null;
				break;
		}
		
	}
}
?>