<?php

defined('__MSF') or die('Direct access is not allowed');

include_once "modules/lms/library/LMS_MailDB.php";
include_once "modules/lms/library/LMS_Mail.php";
include_once "modules/workflow/library/security/WF_DocumentSecurityManager.php";
include_once "modules/media/library/MD_MediaDB.php";
include_once "modules/media/library/MD_Media.php";

class WFA_Notify extends WF_ActionSupport implements WF_IAction
{
	const N_SUCC 	= 0;
	
	const N_USER 	= 1;
	
	const N_EMP 	= 2;
	
	const N_POST 	= 2;
	
	private static $iExec 		= 0;
	
	private static $aUBoxCache 	= array();
	
	private $sRowDelim 			= "|^|row|^|";
	private $sColDelim 			= "/^/col/^/";
	
	private $sActionType		= "";
		
	private $aFIds				= array();
	
	
	private function getUBox($iId)
	{
		$UBox = MSF_Registry::get("MSF_UserBox");
		$aTmp = array();
		
		if (!isset(self::$aUBoxCache[$iId])) {
			$UBox->load($iId, "structure", true);
			$aTmp = $UBox->getLoadedResult();
			self::$aUBoxCache[$iId] = $aTmp;
		}
		else {
			$aTmp = self::$aUBoxCache[$iId];
		}
		return $aTmp;
	}
	
	private function getDisplayParams($aStructureIds, $sText)
	{
		$sFull 	= "";
		$sAddrs = "";
		$aETmp 	= array();
		for ($i = 0; $i < count($aStructureIds); $i++) {
			$aTmp = $this->getUBox($aStructureIds[$i]);
			
			for ($j = 0; $j < count($aTmp); $j++) {
				if (!in_array($aTmp[$j]["user"]["id"], $aETmp)) {
					$aETmp[] = $aTmp[$j]["user"]["id"];
					if (isset($aTmp[$j]["employee"]["name"])) {
						$sAddrs .= $aTmp[$j]["employee"]["name"] . ", ";
					}
					elseif (isset($aTmp[$j]["user"]["login"])) {
						$sAddrs .= $aTmp[$j]["user"]["login"] . ", ";
					}
				}
			}
		}
		
		$sSmall = substr($sAddrs, 0, strlen($sAddrs) - 2);
		if (!empty($sText)) {
			$sText = str_replace("<wbr/>", "\n", MSF_String::breakingLongWords($sText));
			$sFull .= $sSmall;
			$sFull .= "\n" . $sText;
		}
		if (!empty($this->aFIds)) {
			if (empty($sFull)) {
				$sFull .= $sSmall;
			}

			foreach ($this->aFIds as $aFile) {
				try {
					$sFull .= 	"<br/>" . MSF_Registry::get("MSF_Locale")->getWord("WORKFLOW_DOCTYPE_SPA_FILE") . ": " .
								"<a href=\"" . CNF_HOST_URL . "/media/download/" . $aFile["value"] . "\" target=\"_blank\">" . $aFile["name"] . "</a>";								
				}
				catch (Exception $e) {}
			}
		}
		return array($sSmall, $sFull);
	}
	
	private function getAction()
	{
		return $this->getSettings()->getPureActions($this->getActionId());
	}
	
	public function toStrIds($mValue)
	{
		return $this->getValues()->toStructureIds($mValue);
	}
	
	public function processCall()
	{
		$this->sActionType 	= "call";
		$this->sendNotify(
			$this->getParam("aStructure"),
			$this->getDocument()->getRowTitle(),
			$this->getParam("sText"),
			"system",
			false
		);
		
		return self::SUCCESS;
	}
	
	public function processExecute()
	{
		$this->sActionType 	= "execute";
		$aSpa 				= $this->getAction();
		$sType 				= $this->getNotifyType();
		$sTitle 			= $aSpa["title"];
		$aSpa 				= $aSpa["options"];
		$aRes 				= array();
		$aAddrs 			= array();
		$aExtra 			= array();
		$bSend 				= false;
		$aHas 				= array();
		$aLogs				= array();
		// Action options not eq
		if (isset($aSpa["addresses"])) {
			$aSpa["notify_address_value"] = $aSpa["addresses"];
		}
		if (isset($aSpa["text"])) {
			$aSpa["notify_text"] = $aSpa["text"];
		}

		if (isset($aSpa["notify_address_value"])) {
			if ($sType == "serial") {
				$aLogs 		= $this->getLog()->getDelegated($this->getDocument()->getPointId(), $this->getActionId());
				$aAddrs 	= $this->toStrIds($aSpa["notify_address_value"]);
				$aRes 		= $this->parseSerial($aAddrs, $aLogs);
				$aMerge 	= $aRes;
				$aUserBox	= (isset($aRes[0])) ? $this->getUBox($aRes[0]) : array();
				$aExtra 	= (isset($aUserBox[0]["extraStructureIds"])) ? $aUserBox[0]["extraStructureIds"] : array();
				if (!isset($aLogs["structureIds"])) {
					$bSend = true;
				}
				elseif (IS_REQUEST) {
					$aWait 		= $this->getSettings()->getWaitedAtPoint($this->getDocument()->getPointId());
					$aExecuted  = array_unique($this->getLog()->getWaited($this->getDocument()->getPointId()));

					if (isset($aRes[0]) && $this->iRequestId == $aWait["spa_id"]) {
						$bSend 	= true;
						$aMerge = array_merge($aLogs["structureIds"], $aRes);
						$aHas 	= $aLogs["structureIds"];	
					}
				}
			}
			else {
				$bSend 	= true;
				$aRes 	= $this->toStrIds($aSpa["notify_address_value"]);
				$aExtra = $this->getValues()->getExtraStructureIds();
			}
		}
		if (empty($aRes)) {
				$oLocale = MSF_Registry::get("MSF_Locale");
				$this->getLog()->display($this->getActionTitle(), 
				$oLocale->getWord("WFA_NOTIFY_ADRESS_NONE"));
				return self::SUCCESS;				 
		}
		$bResult = self::NULL;
		if ($bSend) {
			if (!isset($aSpa["notify_author"])) {
				$aSpa["notify_author"] = "";
			}
			if ($sType == "serial") {
				if ($aAddrs === $aMerge && $aRes[0] != array_pop($aAddrs)) {
					$bResult = self::SUCCESS;
				}
				else {
					$this->sendNotify(
						$aRes,
						$this->getDocument()->getRowTitle(),
						$aSpa["notify_text"],
						$aSpa["notify_author"],
						true
					);
					$aDisplay = $this->getDisplayParams($aRes, $aSpa["notify_text"]);
					$this->getLog()->display($sTitle, $aDisplay[0], $aDisplay[1]);
					$this->getLog()->setDelegated($this->getDocument()->getPointId(), $this->getActionId(), array("structureIds" => $aMerge));

					if ($aRes[0] == array_pop($aAddrs)) {
						$bResult = self::SUCCESS;
					}
				}
			}
			else {
				$bResult = self::SUCCESS;
				$this->sendNotify(
					$aRes,
					$this->getDocument()->getRowTitle(),
					$aSpa["notify_text"],
					$aSpa["notify_author"], 
					true
				);
				$aDisplay = $this->getDisplayParams($aRes, $aSpa["notify_text"]);
				$this->getLog()->display($sTitle, $aDisplay[0], $aDisplay[1]);
			}
			
			if (!empty($aExtra)) {
				$this->sendNotify(
					$aExtra,
					$this->getDocument()->getRowTitle(),
					$aSpa["notify_text"],
					$aSpa["notify_author"],
					true, true
				);
			}
		}
		$this->callMove();
		return $bResult;
	}
	
	public function processRequest()
	{	
		$iResult 	= self::SUCCESS;
		$Messenger 	= MSF_Registry::get("MSF_Messenger");
		$Locale 	= MSF_Registry::get("MSF_Locale");
		try {
			$sNotifyText 	= (string) $this->getParam("post", "notify_text");
			$sAddresses 	= (string) $this->getParam("post", "notify_addresses");
			$sNotifyAuthor 	= (string) $this->getParam("post", "notify_author");
			if (empty($sAddresses)) {
				$iResult = self::NULL;
				$Messenger->addError($Locale->getWord("WFA_NOTIFY_ADRESS_NONE"));
			}
			else {
				$aAction = $this->getAction();
				if (isset($aAction["options"]["rewrite_field"])) {
					$this->saveStructureFieldValue($aAction["options"]["addresses"], $sAddresses);
				}
				
				$aAddresses = $this->getValues()->toStructureIds($sAddresses);
				$aExtra 	= $this->getValues()->getExtraStructureIds();
				$aAddresses = $this->parseAddresses($aAddresses);
				$aVerify 	= $this->checkAddresses($aAddresses["ids"]);
				$sEmployee 	= "";
				
				for ($i = 0; $i < count($aAddresses); $i++) {
					if ($aAddresses["ids"][$i] == $aVerify["iStructureId"]) {
						$sEmployee = $aAddresses["names"][$i];
						break;
					}
				}
				switch ($aVerify["iVerify"]) {
					case self::N_USER:
						$Messenger->addError(str_replace('{employee}', $sEmployee, $Locale->getWord("WORKFLOW_DOCTYPE_SPA_NOTIFY_ERROR_N_USER")));
						$iResult = self::NULL;
						break;
					case self::N_EMP:
						$Messenger->addError(str_replace('{employee}', $sEmployee, $Locale->getWord("WORKFLOW_DOCTYPE_SPA_NOTIFY_ERROR_N_EMP")));
						$iResult = self::NULL;
						break;
					case self::N_POST:
						$Messenger->addError(str_replace('{employee}', $sEmployee, $Locale->getWord("WORKFLOW_DOCTYPE_SPA_NOTIFY_ERROR_N_POST")));
						$iResult = self::NULL;
						break;
					default:
						if (isset($_FILES["notify_files"]["name"])) {
							$aTmpFiles 	= array();
							$Media 		= new MD_Media(new MD_MediaDB(MSF_Registry::get("MSF_DB")));
							
							for ($i = 0; $i < count($_FILES["notify_files"]["name"]); $i++) {
								if ($_FILES["notify_files"]["error"][$i] == 0) {
									$aTmp 				= array();
									$aTmp["name"] 		= $_FILES["notify_files"]["name"][$i];
									$aTmp["type"] 		= $_FILES["notify_files"]["type"][$i];
									$aTmp["tmp_name"] 	= $_FILES["notify_files"]["tmp_name"][$i];
									$aTmp["error"] 		= $_FILES["notify_files"]["error"][$i];
									$aTmp["size"] 		= $_FILES["notify_files"]["size"][$i];
									$aTmpFiles[] 		= $aTmp;
								}
							}
							
							for ($i = 0; $i < count($aTmpFiles); $i++) {
								$aTmp 			= array();
								$aTmp["name"] 	= $aTmpFiles[$i]["name"];
								if ($Media->isProperSize($aTmpFiles[$i])) {
									$aTmp["value"] 	= $Media->upload($aTmpFiles[$i], "workflow_document", $this->getDocument()->getMediaId());
									$this->aFIds[] 	= $aTmp;
								}
								else {
									$Messenger->addError($Locale->getWord("WFA_NOTIFY_TOO_BIG_FILE"));
									$iResult = self::NULL;
									break;
								}
							}
						}
						
						$this->sendNotify(
							$aAddresses["ids"],
							$this->getDocument()->getRowTitle(),
							$sNotifyText,
							$sNotifyAuthor
						);
						if (!empty($aExtra)) {
							$this->sendNotify(
								$aExtra,
								$this->getDocument()->getRowTitle(),
								$sNotifyText,
								$sNotifyAuthor,
								true,
								true
							);
						}

						$aDisplay = $this->getDisplayParams($aAddresses["ids"],  $this->prepareText($sNotifyText));
						$this->getLog()->setEmployeeName($this->sEmployeeName);
						$this->getLog()->display($aAction["title"], $aDisplay[0], $aDisplay[1], true);
						break;
				}
			}
		}
		catch (Exception $e) {
			$iResult = self::FAILURE;
		}
		$this->callMove();
		return $iResult;
	}

	private function callMove()
	{
		$aRoute		= $this->getSettings()->getActions($this->getActionId());
		if (isset($aRoute["options"])) {
			if (isset($aRoute["options"]["move_points"])) {		
				$this->call("move", array("iNewPoint" => $aRoute["options"]["move_points"]));
			}
		}
	}

	private function prepareText($sText) 
	{
		if (!empty($sText)) {
			$sText = htmlspecialchars($sText);
			$sText = MSF_String::breakingLongWords($sText);
			$sText = "<br />" . nl2br($sText);
		}
		return $sText;
	}

	private function checkAddresses($aAddresses)
	{
		$iResult 	= self::N_SUCC;
		$iStruct 	= 0;
		$sUsers 	= "";
		$iSize 		= count($aAddresses);
		$UBox 		= MSF_Registry::get("MSF_UserBox");
		
		for ($i = 0; $i < $iSize; $i++) {
			$aTmp 		= $this->getUBox($aAddresses[$i]);
			if (empty($aTmp)) {
				continue;
			}
			$iStruct 	= $aAddresses[$i];
			if (!isset($aTmp[0]["user"])) {
				$iResult = self::N_USER;
				break;
			}
			elseif (!isset($aTmp[0]["employee"])) {
				$iResult = self::N_EMP;
				break;
			}
			elseif (!isset($aTmp[0]["posts"])) {
				$iResult = self::N_POST;
				break;
			}
			elseif (!empty($aTmp[0]["user"])) {
				$sUsers .= $aTmp[0]["user"]["employee_ref_value"] . $this->sColDelim . $aTmp[0]["user"]["id"];
				
				if ($i < $iSize - 1) {
					$sUsers .= $this->sRowDelim;
				}
			}
		}
		
		return array("iVerify" => $iResult, "iStructureId" => $iStruct);
	}
	
	public function processResponse()
	{
		return self::FAILURE;
	}
	
	private function getNotifyType()
	{
		$aSpa 	= $this->getAction();
		$sType 	= "parallel";
		
		if (isset($aSpa["options"]["notify_type"])) {
			$sType 	= $aSpa["options"]["notify_type"];
			
			if ($sType == "doc_field") {
				$Fields 	= new MSF_XmlWrapper($this->getDoctypeVersion()->getFieldsXml());
				$aFields 	= $this->getDocumentVersion()->getFields();
				
				if (isset($aSpa["options"]["doc_field"])) {
					foreach ($Fields as $Data) {
						if ($Data->getName() == $aSpa["options"]["doc_field"] && isset($aFields[$Data->getName()])) {
							$Value 	= new MSF_EnumValue();
							$aValue = $Value->parse($aFields[$Data->getName()]);
							
							switch ((int) $aValue["key"]) {
								case 0:
									$sType = "parallel";
									break;
								case 1:
									$sType = "serial";
									break;
							}
						}
					}
				}
			}
		}
		
		return $sType;
	}
	
	private function parseAddresses($mAddresses)
	{
		$aResult = array();
		if (is_array($mAddresses)) {
			for ($i = 0; $i < count($mAddresses); $i++) {
				$aTmp = $this->getUBox($mAddresses[$i]);
				if (empty($aTmp)) {
					continue;
				}
				$aResult["ids"][] 	= $mAddresses[$i];
				$aResult["names"][] = $aTmp[0]["employee"]["name"];
			}
		}
		else {
			$aElements = explode($this->sRowDelim, $mAddresses);
			
			for ($i = 0; $i < sizeof($aElements); $i++) {
				$aTmp = explode($this->sColDelim, $aElements[$i]);
				$aResult["ids"][] 	= $aTmp[1];
				$aResult["names"][] = $aTmp[0];
			}
		}
		
		if (empty($aResult)) {
			$aResult["ids"] 	= array(0);
			$aResult["names"] 	= array("");
		}
		return $aResult;
	}
	
	private function parseSerial($aAddresses, $aLog)
	{
		$aResult = array();
		
		if (isset($aLog["structureIds"])) {
			for ($i = 0; $i < count($aAddresses); $i++) {
				$bExists = false;
				
				for ($j = 0; $j < count($aLog["structureIds"]); $j++) {
					if ($aLog["structureIds"][$j] == $aAddresses[$i]) {
						$bExists = true;
						break;
					}
				}
				
				if (!$bExists) {
					$aResult[] = $aAddresses[$i];
					break;
				}
			}
		}
		else {
			$aResult = array(((isset($aAddresses[0])) ? $aAddresses[0] : ""));
		}
		
		return $aResult;
	}
	
	private function sendNotify($aAddresses, $sSubject, $sText, $sAuthor = "system", $isCurrent = true, $bExtra = false)
	{
		if ($isCurrent) {
			$sText = $this->formNotifyBody($this->validateText($sText));
		}
		
		if ($bExtra) {
			$sSubject = MSF_Registry::get("MSF_Locale")->getWord("ROOT_POST_IO") . " " . $sSubject;
		}
		else {
			$this->delegateAccess($aAddresses);
		}
		try {
			$Mail = new LMS_Mail(new LMS_MailDB(MSF_Registry::get("MSF_DB")));
			if ($this->sActionType == "execute" || $this->sActionType =="call") {
				switch ($sAuthor) {
				case "author":
					$sAuthorName = $this->getAuthor();
					$sAuthorId = $this->getUserId($sAuthorName);
					$Mail->setFrom("", $sAuthorName, $sAuthorId);
					break;
				case "user":
					break;
				default:
					$Mail->setFrom("", "Detrix");
					break;
				}
			}
			$Mail->setStructureIds($aAddresses);
			$Mail->setSubject($sSubject);
			$Mail->setType("text/html");
			$Mail->setBody($sText);
			$Mail->save(0, true);
		}
		catch (Exception $e) {

		}
	}
	
	private function formNotifyBody($sNotifyText)
	{
		$sDocument 	= MSF_Registry::get("MSF_Locale")->getWord("WORKFLOW_DOCUMENT");
		$sUrl 		= $sDocument . (($sDocument{strlen($sDocument) - 1} != ":") ? ":" : "") . "<br/>" .
					"<a href=\"" . CNF_HOST_URL . "/workflow/document/view/" . $this->getDocument()->getDocumentId() . "\">" .
						$this->getDocument()->getTitle() .
					"</a><br/><br/>";
			
		if (!empty($sNotifyText)) {
			$sCommentWord 	= MSF_Registry::get("MSF_Locale")->getWord("WORKFLOW_DOCTYPE_SPA_COMMENT");
			$sNotifyText 	= $sCommentWord . (($sCommentWord{strlen($sCommentWord) - 1} != ":") ? ":" : "") . "<br/>" . $sNotifyText;
		}
		
		if (!empty($this->aFIds)) {
			$sNotifyText .= "<br/><br/>" . MSF_Registry::get("MSF_Locale")->getWord("WFA_DECSION_ATTACH");
			foreach ($this->aFIds AS $aFile) {
				$sNotifyText   .= "<br/><a href=\"" . CNF_HOST_URL . "/media/download/" . $aFile["value"] . "\" target=\"_blank\">" . $aFile["name"] . "</a>";
			}
		}
		
		return $sUrl . $sNotifyText;
	}
	
	private function delegateAccess(array $aAddresses)
	{
		$aFamilyList 	= array();
		$oDb 			= MSF_Registry::get("MSF_DB");
		$sFamilyTree	= $this->getDocument()->getFamilyTree();
		$iDocumentId	= $this->getDocument()->getDocumentId();
		$aFamilyTree 	= MSF_SL::getParam(
				MSF_XmlWrapper::xmlToarray($sFamilyTree),
				"root",
				array()
				);
		
		if (!empty($aFamilyTree)) {
			$aFamilyList 	= $this->treeToList(
					$aFamilyTree["value"][0],
					$iDocumentId
					);
		}

		foreach ($aFamilyList AS $iDocumentId) {
			$oSM = new WF_DocumentSecurityManager($oDb, $iDocumentId);
			foreach ($aAddresses AS $iAddressId) {
				$oSM->edit(
						array(
								"structure_id" => $iAddressId
								)
						);
				
			}
			
			unset($oSM);

		}

	}
	
	private function validateText($sText)
	{
		$sResult 	= "";		 
		$sResult	= str_replace("\n", "<br>", htmlspecialchars($sText,  ENT_NOQUOTES, "UTF-8"));
		return $sResult;	
	}
	
	private function treeToList(array $aTree, $iDocumentId, array $aPath = array())
	{
		$isRouted 	= false;
		$aPath[] 	= $aTree["id"];
		if ($aTree["id"] != $iDocumentId) {
			for ($i = 0; $i < count($aTree["node"]); $i++) {
				$aTemp = $this->treeToList($aTree["node"][$i], $iDocumentId, $aPath);
				if (!empty($aTemp)) {
					$isRouted 	= true;
					$aPath 		= $aTemp;
					break;
				}
				
			}
		
		}
		else {
			$isRouted = true;
		}
		
		return $isRouted ? $aPath : array();

	}

	private function getAuthor() {
		$sEmployeeName = $this->getDocumentVersion()->getSystemField("author_name");
		if (!empty($sEmployeeName)) {
			$RefValue 		= new REF_Value();
			$RefValue->parse($sEmployeeName);
			$sEmployeeName 	= $RefValue->get(0, "value");
			$aEmployeeName	= explode("(", $sEmployeeName);
			$sEmployeeName	= trim($aEmployeeName[0]); 
		}		
		return $sEmployeeName;
	}

	private function getUserId($sUserName) {
		$oUR = new UserReference(
					new UserReferenceDB(
					MSF_Registry::get("MSF_DB")
					)
				);
		return $oUR->getUserId($sUserName);
	}

}

?>