<?php

defined('__MSF') or die('Direct access is not allowed');

include_once "modules/references/library/Reference/UserReferenceDB.php";
include_once "modules/references/library/Reference/UserReference.php";
include_once "modules/media/library/MD_Media.php";
include_once "modules/1cv81/library/V81_DocFieldChanger.php";
include_once "modules/workflow/library/variables/WF_Vars.php";
include_once "WFA_RecordUtils.php";
include_once "WFA_RecordViewPort.php";

class WFA_Record extends WF_ActionSupport implements WF_IAction
{
	public function processRequest()
	{				
		try {						
			$sField = (string) MSF_SL::getParam($_POST, "replace_field");
			if(empty($sField)){
				MSF_Registry::get("MSF_Messenger")->addError(MSF_Registry::get("MSF_Locale")->getWord("WFA_RECORD_FIELD_NULL"));
				return self::NULL;
			}			
			$sValue = $this->doAction(
				$sField,
				(string) MSF_SL::getParam($_POST, "field"),
				MSF_SL::getParam($_POST, $sField),
				true
			);
						
			$bSpecial 	= false;
			$sType 		= "";
			$sTitle 	= "";
			$Wrapper 	= new MSF_XmlWrapper($this->getDoctypeVersion()->getFieldsXml());
			
			foreach ($Wrapper as $Data) {
				if ($Data->getName() == $sField) {
					$sType 	= $Data->getValue("type");
					$sTitle = $Data->getTitle();
				}
			}
			if (!empty($sValue)) {
				switch ($sType) {
					case "enum":
						$aValues = $Data->getValue("optional");				
						if (isset($aValues["enum"])) {
							foreach ($aValues["enum"] as $iKey => $sText) {
								if ($sValue == $iKey) {
									$sValue = $sText;
									break;
								}
							}
						}
						break;
					case "bool":
						$aValues = $Data->getValue("optional");
						if (isset($aValues["labelYes"]) && (int) $sValue == 1) {
							$sValue = $aValues["labelYes"];
						}
						elseif (isset($aValues["labelNo"]) && (int) $sValue == 0) {
							$sValue = $aValues["labelNo"];
						}
						break;
					case "picture":
					case "file":
					case "table":
					case "text":
						$bSpecial = true;
						break;
					case "document":
					case "reference":
					case "structure":
						$RefValue = new REF_Value();
//                                            print_r($sValue);
//                                            exit();
                                                if(is_array($sValue)) {
                                                    for($i = 0; $i < count($sValue); $i++) {
                                                        $RefValue->parse($sValue[$i]);
                                                    }
                                                } else {
                                                    $RefValue->parse($sValue);
                                                }    
						$aValues = $RefValue->getValuesList();
						$iSize = count($aValues);
						$sValue = "";
						
						for ($i = 0; $i < $iSize; $i++) {
							$sValue .= $aValues[$i];
							
							if ($i < $iSize - 1) {
								$sValue .= ", ";
							}
						}
						$sValue = htmlspecialchars_decode($sValue);
						break;
					case "date":
						$Date = new MSF_Date();
						$Date->parse($sValue);
						$sValue = $Date->format(CNF_DATE_FORMAT);
						break;
					case "timestamp":
						$Date = new MSF_Date();
						$Date->parse($sValue);
						$sValue = $Date->format(CNF_DATE_TIME_FORMAT);
						break;
				}
			}
			$aRoute		= $this->getSettings()->getActions($this->getActionId());
			if (isset($aRoute["options"])) {
				if (isset($aRoute["options"]["move_points"])) {
					$this->call("move", array("iNewPoint" => $aRoute["options"]["move_points"]));				
				}
			}
			
 			$sSmall = MSF_Registry::get("MSF_Locale")->getWord("WORKFLOW_DOCTYPE_SPA_RECORD_LOG");
 			
 			if ($bSpecial) {
 				$sFull 	= $sSmall . "<br/>" . $sTitle . " = " . $sValue;
 			}
 			else {
 				$sFull 	= $sSmall . "\n" . $sTitle . " = " . $sValue;
 			}
 			
 			$sSmall = preg_replace('@<script[^>]*?>.*?</script>@si', ' ', $sSmall);
 			$sFull 	= preg_replace('@<script[^>]*?>.*?</script>@si', ' ', $sFull);
 			$sFull 	= preg_replace('@<(ul|li)[^>]*?>|</(ul|li)>@si', ' ', $sFull);
			$this->getLog()->display($this->getActionTitle(), $sSmall, $sFull, $bSpecial);
		}
		catch (Exception $e) {
			if (CNF_MODE == "debug") {
				debug($e->getMessage());
				exit();
			}	
			return self::FAILURE;
		}

		return self::SUCCESS;
	}
	
	public function processExecute()
	{
		$aAction = $this->getSettings()->getActions($this->getActionId());
		
		if (!isset($aAction["options"])) {
			return self::FAILURE;
		}
		
		$sType  = (string) MSF_SL::getParam($aAction["options"], "field");
		if (empty($sType)) {
			return self::FAILURE;
		}
		
		$sField = (string) MSF_SL::getParam($aAction["options"], "replace_field");
		if(empty($sField)){
			MSF_Registry::get("MSF_Messenger")->addError(MSF_Registry::get("MSF_Locale")->getWord("WFA_RECORD_FIELD_NULL"));
			return self::NULL;
		}
		
		$oViewPort 		= new WFA_RecordViewPort(	
													$this->getDoctype(), 
													$this->getDoctypeVersion(), 
													$this->getDocument(), 
													$this->getDocumentVersion(), 
													$this->getStructureId(), 
													$this
												 );
		$aViewOptions 	= $oViewPort->port($aAction, array());
		
		$this->doAction(
			$sField,
			$sType,
			$aViewOptions["value"],
			false
		);
		
		try {
			$aRoute		= $this->getSettings()->getActions($this->getActionId());
			if (isset($aRoute["options"])) {
				if (isset($aRoute["options"]["move_points"])) {
					$this->call("move", array("iNewPoint" => $aRoute["options"]["move_points"]));				
				}
			}		
		}
		catch(Exception $e) {
			if (CNF_MODE == "debug") {
				debug($e->getMessage());
				exit();
			}	
			return self::FAILURE;
		}
		
		return self::SUCCESS;
	}
	
	
	private function doAction($sField, $sType, $mValue, $isDelegation)
	{
		$sReturnValue 	= $mValue;
		$bPass 			= true;	
		$aFields 		= $this->getDocumentVersion()->getFields();
		$Fields 		= new MSF_XmlWrapper($this->getDoctypeVersion()->getFieldsXml());
		$aAction 		= $this->getSettings()->getActions($this->getActionId());
		$oRefFields 	= $this->getDocument()->getCurrentVersion()->getRefFields();
		switch ($sType) {
			case "field":
				$aOptions 	= array();
				$sType 		= "";
				
				foreach ($Fields as $Data) {
					if ($Data->getName() == $sField) {
						$aOptions 	= $Data->getValue("optional");
						$sType 		= $Data->getValue("type");
						break;
					}
				}
				foreach ($aFields as $k => $v) {
					if ($k == $mValue) {
						$mValue = $v;
						break;
					}
				}

				
				switch ($sType) {
					case "int":
						if (strlen($mValue) > 0)  {
							$mValue = round($mValue);
						}
						break;
					case "string":
					case "text":
						$sValueType   = $oRefFields->getFieldType($aAction["options"]["value"]);
						break;
					case "structure":
						$mValue = $this->writeStructureType($mValue, $aOptions);
						$sReturnValue = $mValue;
						break;
					case "reference":
						$bRewrite = false;
						foreach ($Fields as $Data) {
							if ($Data->getName() == $sField) {
								$bRewrite = true;
								break;
							}
						}
						
						if ($bRewrite) {
							$mValue = $this->writeRefType($mValue, $aOptions);
						}

						break;
					case "document":
						$mValue 		= $this->writeDocType($mValue, $sReturnValue);
						$sReturnValue 	= $mValue;
						break;
					case "file":
					case "picture":
						/*
						foreach ($aFields as $sKey => $sItem) {
							if ($sKey == $sField) {
								$Media 	= new MD_Media(new MD_MediaDB(MSF_Registry::get("MSF_DB")));
								$aTmp 	= explode(",", $sItem);
								$aNTmp 	= explode(",", $sTmpFiled);
								
								foreach ($aTmp as $iFileId) {
									if (!empty($iFileId) && !in_array($iFileId, $aNTmp)) {
										//$Media->remove($iFileId);
									}
								}
								
								break;
							}
						}
						*/
						$aReturn 		= $this->writeAttachType($mValue, $isDelegation, $sField, $aOptions);					
						$mValue 		= $aReturn["mValue"];
						$sReturnValue 	= $aReturn["sReturnValue"];
						break;
					case "table":
						if (isset($aAction["options"]["value"])) {
							$mValue			= $this->tableValueInsert($mValue, $aFields[$sField], $aAction["options"]["value"]);
						}
						$sReturnValue = $this->writeTableReturn(htmlspecialchars($mValue), $aOptions);
						break;
				}
				break;
			case "var":
				foreach ($Fields as $Data) {
					if ($sField == $Data->getName()) {
						switch ($Data->getValue("type")) {
							case "structure":
								$mValue = $this->writeStructureType($mValue, $Data->getValue("optional"));
								$sReturnValue = $mValue;
								break;
							case "date":
								$Date = new MSF_Date();
								$Date->parse($mValue);
								$mValue = $Date->format(CNF_DATE_FORMAT);
								break;
							case "timestamp":								
								$Date = new MSF_Date();
								$Date->parse($mValue);
								$mValue = $Date->format(CNF_DATE_TIME_FORMAT);
								break;
						}
						break;
					}
				}
				break;
			case "formula":
				$aAction["options"]["optional"]["type"] = $Fields->get($sField)->getValue();
				
				if ($aAction["options"]["optional"]["type"] == "int") {
					$mValue = round($mValue);
				}
				if (!is_numeric($mValue) && strlen($mValue) != 0) {
					$bPass = false;
				}
				break;
			default:
				foreach ($Fields as $Data) {
					if ($sField == $Data->getName()) {
						switch ($Data->getValue("type")) {
							case "int":
								$mValue = str_replace(",", ".", $mValue);
								if (!is_numeric($mValue) && strlen($mValue) != 0) {
									$bPass = false;
								}				
								if (strlen($mValue) > 0)  {
									$mValue = round($mValue);
								}
								
								break;
							case "float":
								$mValue = str_replace(",", ".", $mValue);
								if (!is_numeric($mValue) && strlen($mValue) != 0) {
									$bPass = false;
								}
								break;
							case "document":
								$mValue = $this->writeDocType($mValue, $sReturnValue);
								break;
							case "reference":
								$mValue = $this->writeRefType($mValue, $Data->getValue("optional")); 
								break;
							case "table":
								$sReturnValue = $this->writeTableReturn($mValue, $Data->getValue());
								break;
							case "structure":
								$mValue = $this->writeStructureType($mValue, $Data->getValue("optional"));
								$sReturnValue = $mValue;
								break;
							case "file":
							case "picture":
								$aReturn 		= $this->writeAttachType($mValue, $isDelegation, $sField, $Data->getValue("optional"));					
								$mValue 		= $aReturn["mValue"];
								$sReturnValue 	= $aReturn["sReturnValue"];
							break;
						}
						break;
					}
				}
				break;
		}
		if (isset($aOptions["multiselect"]) && $aOptions["multiselect"] == 0) {
			$mValue = $this->extractFirstValue($mValue, $sType);
		}
		
		if ($bPass) {
			foreach ($aFields as $k => $v) {
				if ($k == $sField) {
					$aFields[$k] = $mValue;
				}
			}
		}
		
		$Date = new MSF_Date();		
		if ($isDelegation) {
			$Doctype	= $this->getDoctype();
			$NVersion 	= $this->getDocument()->newVersion();
			$OVersion 	= $this->getDocumentVersion();
			$oUserBox 	= MSF_Registry::get("MSF_UserBox");			
			//$UF			= new MSF_UniqueField(MSF_Registry::get("MSF_DB"));			
			//$UF->freeFields($OVersion->getFieldsAsList(), MSF_UniqueField::TYPE_DOCTYPE, $Doctype->getDoctypeId());			
			//$UF->blockFields($OVersion->getFieldsAsList(), MSF_UniqueField::TYPE_DOCTYPE, $Doctype->getDoctypeId());			
			//$OVersion->setSystemField("modified_at", $Date->formatTimestamp());
			$NVersion->setStructureId($oUserBox->getCurrentStructureId());
			$NVersion->setEmployeeName($oUserBox->getEmployeeAttribute("name"));
			$NVersion->setCreatedAt($Date->format());
			$NVersion->setSystemFields($OVersion->getSystemFields());
			$NVersion->setFields($aFields);
			$UF			= new MSF_UniqueField(MSF_Registry::get("MSF_DB"));			
			$UF->freeFields($OVersion->getFieldsAsList(), MSF_UniqueField::TYPE_DOCTYPE, $Doctype->getDoctypeId());			
			$UF->blockFields($NVersion->getFieldsAsList(), MSF_UniqueField::TYPE_DOCTYPE, $Doctype->getDoctypeId());
			$NVersion->save();
			$OVersion->save();
			$this->setDocumentVersion($this->getDocument()->getCurrentVersion());
		}
		else {
			$this->getDocumentVersion()->setSystemField("modified_at", $Date->formatTimestamp());
			$this->getDocumentVersion()->setFields($aFields);
			$this->getDocumentVersion()->save();
		}
		
		
		// Пример записи реквизита в документ 1С
		/*$sNeedDoctypeId = 784;
		$sNeedField = "ЗаявкаЗакрыта";
		$sNeedDoc = "ЗаявкаИндивидуальная";
		if($this->getDocument()->getDoctypeId() == $sNeedDoctypeId) {
			$sDocLink = "";
			$aDocFields = $this->getDocument()->getCurrentVersion()->getFieldsAsList();			
			foreach($aDocFields AS $Value) {
				if($Value["title"] == "ДокументСсылка") {
					$sDocLink = $Value["value"];
				}
				
			}			
			
			$oDocFieldChanger = new V81_DocFieldChanger;
			$oDocFieldChanger->changeDocField($sNeedDoc, $sDocLink, $sNeedField, "true");		
		}*/		
		
		return $sReturnValue;
	}
	private function extractFirstValue($mValue, $sType = "") {
		$sResult = "";
		$aRow = explode("|^|row|^|", $mValue);
		$aCol = explode("/^/col/^/", $aRow[0]);
		switch ($sType) {
			case "structure":
				if (isset($aCol[3])) {
					$sResult = $aRow[0]; 
				}
				break;
			default:
				$sResult = $aRow[0];
				break;
		}
		return $sResult;
	}
	
	private function writeTableReturn($mValue, $aOptions)
	{
		$aFields = array();
		if (isset($aOptions["fields"])) {
			foreach ($aOptions["fields"] AS $sFielname => $aFieldbody) {
	            $aFields[$sFielname]["index"] 		= "";
	            $aFields[$sFielname]["title"] 		= MSF_SL::getParam($aFieldbody, "title", "");
	            $aFields[$sFielname]["type"]  		= MSF_SL::getParam($aFieldbody, "type", "");
	            $aFields[$sFielname]["optional"] 	= array();
	            $aFields[$sFielname]["default"] 	= ""; 
			}

		}

		$sRandom = "record_table_log_" . MSF_String::random(5);
		$sReturn = "
		<style>
			div." . $sRandom . " table th {
				font-size: 12px;
				border: 1px solid #000;
			}
			div." . $sRandom . " table td {
				border: 1px solid #000;
			}
		</style>
		<div id=\"" . $sRandom . "\" class=\"" . $sRandom . "\"></div>
		<script>
			new REF_Table({
				oContainer		: \"" . $sRandom . "\",
				sWidth 			: \"100%\",
				sCellpadding 	: \"3\",
				sCellspacing 	: \"0\",
				sBorderWidth 	: \"1\",
				sDisplayInRS 	: \"" . ((isset($aOptions["displayInRS"])) ? $aOptions["displayInRS"] : "") . "\",
				sMode 			: \"view\",
				aFields 		: " . MSF_Json::encode($aFields) . ",
				sName 			: \"\",
				sDefaultValue 	: \"" . str_replace("\"", "\\\"", $mValue) . "\"
			});
		</script>";
		
		return $sReturn;
	}
	
	private function tableValueInsert($mValue, $mOldValue, $sInsertType) {
		if ($mValue == "top" || $mValue == "bottom") {
			return $mOldValue;
		}
		if (!empty($mOldValue)) {
			$aOldValue 	= explode("^R^", $mOldValue);
			$aValue		= explode("^R^", $mValue);
			$i = count($aOldValue);
			$j = count($aValue);
			$aNewValue = array();
			if ($sInsertType == "top") {			
				for ($k = 2; $k < $i; $k++) {
					$aNewValue[$j++] = $aOldValue[$k];
				}
				$mValue = implode("^R^", array_merge($aValue, $aNewValue));
			}
			if ($sInsertType == "bottom") {
				$aNewValue = $aOldValue;
				for ($k = 2; $k < $j; $k++) {
					$aNewValue[] = $aValue[$k];
				}
				$mValue = implode("^R^", $aNewValue);
			}
		}
		
		return $mValue;
	}
	
	private function writeRefType($mValue, $aOptions)
	{		
		if (isset($aOptions["reference"])) {
			$Ref = new UserReference(new UserReferenceDB(MSF_Registry::get("MSF_DB")));
			$Ref->load($aOptions["reference"]);
			$RefValue = new REF_Value();
			$RefValue->parse($mValue);
			$aKeys = $RefValue->getKeysList();
			$aCols = $Ref->getAsArray();

			$aType = array();
			$aVals = array();
			
			foreach ($Ref->getFields() AS $aField) {
				$aType[$aField["name"]] = $aField["type"];
			}
			
			for ($i = 0; $i < count($aCols); $i++) {
				$iSearchKey = array_search($aCols[$i]["id"], $aKeys);
				if ($iSearchKey !== false) {
					$aVals[$iSearchKey] = $aCols[$i];
				}
			}
			
			$mValue 		= "";
			ksort($aVals);
			$RefValue->reset();
			foreach ($aVals AS $aRef) {
				$iId 		= 0;
				$aRValue 	= array();
				foreach ($aRef as $sKey => $sValue) {
					if ($sKey == "id") {
						$iId = $sValue;
						continue;
					}
					if (isset($aOptions["fieldsValue"]) && !in_array($sKey, $aOptions["fieldsValue"])) continue;
				
					if (strlen($sValue) == 0) continue;
					
					switch ($aType[$sKey]) {
						case "date":
							$oDate = new MSF_Date();
							$oDate->parse($sValue);
							$sValue = $oDate->format(CNF_DATE_FORMAT);
							break;
						case "timestamp":
							$oDate = new MSF_Date();
							$oDate->parse($sValue);
							$sValue = $oDate->format(CNF_DATE_TIME_FORMAT);
							break;
						case "enum":
						case "bool":
							$aTmpEnumValue = explode(MSF_EnumValue::ROW_DELIMITER, $sValue);
							$sValue = $aTmpEnumValue[0];
							break;
						case "file":
							$sValue     = 'загрузить';
							break;
						case "picture":
							$sValue     = "<img src='/media/image/" . $sValue . "/100/100'>";
							break;
					}
					
					$aRValue[] = $sValue;
				}
								
				$RefValue->add($iId, implode($aOptions["separator"], $aRValue));
			}
			
			$mValue 		= $RefValue->format();
		}
		
		return $mValue;
	}
	
	private function writeDocType($mValue, &$sReturnValue)
	{
		/*
		$sReturnValue = "";
		$RefValue 	= new REF_Value();
		$RefValue2 	= new REF_Value();
		$iSize 		= count($mValue);
		for ($i = 0; $i < $iSize; $i++) {
			$RefValue->parse($mValue[$i]);
			$RefValue->parse($mValue);
			$aValues = $RefValue->getValuesList();
			
			foreach ($RefValue->getKeysList() as $iKey => $aKeys) {
				$RefValue2->add($aKeys, $aValues[$iKey]);
				$sReturnValue .= $aValues[$iKey];
			}
			
			if ($i < $iSize - 1) {
				$sReturnValue .= "; ";
			}
		}
		
		return $RefValue2->format();
		*/
		
		/*
		if (is_array($mValue)) {
			$sReturnValue 	= "";
			$RefValue 		= new REF_Value();
			
			foreach ($mValue as $sDoc) {
				$RefValue->parse($sDoc);
				$aValues 	= $RefValue->getValuesList();
				$iSize 		= count($aValues);
				$iCount 	= 0;
				
				foreach ($aValues as $sValue) {
					$sReturnValue .= $sValue;
					
					if ($iCount < $iSize - 1) {
						$sReturnValue .= "; ";
					}
					$iCount++;
				}
			}
		}
		else {
			$RefValue = new REF_Value();
			$RefValue->parse($mValue);
			
			$sReturnValue 	= "";
			$aValues 		= $RefValue->getValuesList();
			$iSize 			= count($aValues);
			$iCount 		= 0;
			
			foreach ($aValues as $sValue) {
				$sReturnValue .= $sValue;
				
				if ($iCount < $iSize - 1) {
					$sReturnValue .= "; ";
				}
				$iCount++;
			}
		}
		*/
		if (is_array($mValue)) {
			$mValue = implode("|^|row|^|", $mValue);
		}
		
		return $mValue;
	}
	
	private function writeStructureType($mValue, $aValue)
	{
		$bShow 	= false;
		$UBox 	= MSF_Registry::get("MSF_UserBox");
		if ($aValue["postEnable"] == 1) {
			$bShow = true;
		}
		
		switch ($mValue) {
			case "author":
				$RefValue 	= new REF_Value();
				$mValue 	= $this->getDocumentVersion()->getSystemField("author_name");
				$RefValue->parse($mValue);
				$aKeys 		= $RefValue->getKeysList();
				$sRes 		= RefFieldHelper::formatStructureValue(
						$aKeys[0][0], 
						$aKeys[0][1], 
						MSF_SL::getParam($aValue, "employee", "name"), 
						MSF_SL::getParam($aValue, "post", "post"), 
						MSF_SL::getParam($aValue, "employeeEnable", 1), 
						MSF_SL::getParam($aValue, "postEnable", 1)
						);
				$ResValue = new REF_Value();
				$ResValue->add($aKeys[0], $sRes);
				$mValue = $ResValue->format();
				break;
			case "user":
				$RefValue 	= new REF_Value();
				$aIds 		= array(
						$UBox->getCurrentStructureId(), 
						$UBox->getEmployeeAttribute("id"), 
						$UBox->getCurrentPostId()
						);
				$sRes 		= RefFieldHelper::formatStructureValue(
						$aIds[1], 
						$aIds[2], 
						MSF_SL::getParam($aValue, "employee", "name"), 
						MSF_SL::getParam($aValue, "post", "post"), 
						MSF_SL::getParam($aValue, "employeeEnable", 1), 
						MSF_SL::getParam($aValue, "postEnable", 1)
						);
				$RefValue->add($aIds, $sRes);
				$mValue = $RefValue->format();
				break;
			default:
				$RefValue 	= new REF_Value();
				$RefValue->parse($mValue);
				$aValues 	= $RefValue->getValuesList();
				$aKeys 		= $RefValue->getKeysList();
				if (isset($aKeys[0]) && count($aKeys[0]) > 2) {
					$RefValue = new REF_Value();
					for ($i = 0; $i < count($aValues); $i++) {
						if (count($aKeys[$i]) == 3) {
							$sRes = RefFieldHelper::formatStructureValue(
									$aKeys[$i][1], 
									$aKeys[$i][2], 
									MSF_SL::getParam($aValue, "employee", "name"), 
									MSF_SL::getParam($aValue, "post", "post"), 
									MSF_SL::getParam($aValue, "employeeEnable", 1), 
									MSF_SL::getParam($aValue, "postEnable", 1)
									);
							$RefValue->add($aKeys[$i], $sRes);
						}
						else {
							$RefValue->add($aKeys[$i], $aValues[$i]);
						}
						
					}
					
					$mValue = $RefValue->format();
				}
				
				break;
		}
		
		return $mValue;
	}
	
	private function writeAttachType($mValue, $isDelegation, $sField, $aOptions) {
		$sReturnValue 	= "";
		$Media 			= new MD_Media(new MD_MediaDB(MSF_Registry::get("MSF_DB")));
		if (!empty($mValue)) {
			foreach (explode(", ", $mValue) AS $iId) {
				$aFileInfo 		= $Media->getFileInfo($iId);
				$sReturnValue  .= "<a href=\"/media/download/" . $iId . "\">" . $aFileInfo["name"] . "</a>";
			}
		}
		
		if ((!empty($_FILES)) && ($isDelegation)) {								
			if (isset($_POST["record_added_file"]) && !empty($_POST["record_added_file"]["value"])) {
				$iSize 	= count($_POST["record_added_file"]["value"]);
				$iCount = 0;
				$aIds 	= array();
				
				foreach ($_POST["record_added_file"]["value"] as $iId) {
					$aIds[] = $iId;
					$mValue .= $iId;
					
					if ($iCount < $iSize - 1) {
						$mValue .= ",";
					}
					
					$iCount++;
				}
				
				$iCount = 0;
				foreach ($_POST["record_added_file"]["name"] as $sName) {
					$sReturnValue .= "<a href=\"/media/download/" . $aIds[$iCount] . "\">" . $sName . "</a>";
					
					if ($iCount < $iSize - 1) {
						$sReturnValue .= ", ";
					}
					
					$iCount++;
				}
			}
			
			$aFiles = (array) MSF_SL::getParam($_FILES, $sField);
			$iSize 	= count(isset($aFiles["name"]) ? $aFiles["name"] : array());
			
			$Media->setMaxFileSize(99999999999999);
			for ($i = 0; $i < $iSize; $i++) {
				if ($aFiles["error"][$i] == 4) {
					continue;
				}
				else if (!strlen($aFiles["name"][$i])) {
					continue;
				}
				
				if (!empty($mValue)) {
					$mValue .= ",";
				}
				if (!empty($sReturnValue)) {
					$sReturnValue .= ", ";
				}
				
				$aTmpFile 				= array();
				$aTmpFile["name"] 		= $aFiles["name"][$i];
				$aTmpFile["size"] 		= $aFiles["size"][$i];
				$aTmpFile["error"] 		= $aFiles["error"][$i];
				$aTmpFile["type"] 		= $aFiles["type"][$i];
				$aTmpFile["tmp_name"] 	= $aFiles["tmp_name"][$i];
				$iFileId 				= $Media->upload($aTmpFile, "workflow_document", $this->getDocument()->getMediaId());
				$mValue 				.= $iFileId;
				$sReturnValue 			.= "<a href=\"/media/download/" . $iFileId . "\">" . $aFiles["name"][$i] . "</a>";
			}
		}
		
		if (isset($aOptions["multiselect"]) && $aOptions["multiselect"] == 0) {
			$aValue = explode(",", $mValue);
			$mValue = array_shift($aValue);
		}
		
		return array("mValue" => $mValue, "sReturnValue" => $sReturnValue);
	}
	
	public function processResponse()
	{
		MSF_RteObjectsIntegrator::files($this->getDoctype()->getDoctypeId(), "workflow_document");
		return self::FAILURE;
	}

	
}

?>