<?php

include_once "modules/media/library/MD_Media.php";
include_once "modules/workflow/library/variables/WF_Vars.php";
include_once "modules/workflow/actions/view/WFA_AbstractViewPort.php";
include_once "WFA_RecordUtils.php";

class WFA_RecordViewPort extends WFA_AbstractViewPort
{
	private $aOptions 	= array();
	
	private $Fields		= array();
	
	private $aDocFields = array();
	
	private $oRefFields = array();
	/**
	 * @return void
	 */
	public function port(array $aActionParams, array $aCurrentDelegated)
	{	
		$this->aOptions = $aActionParams["options"];
		
		if (isset($this->aOptions)) {
			if (!isset($this->aOptions["value"])) {
				$this->aOptions["value"] = "";
			}
			$this->Fields 		= new MSF_XmlWrapper($this->oDoctypeVersion->getFieldsXml());
			$this->aDocFields 	= $this->oDocumentVersion->getFields();

			$sFieldName 	= "";
			$sFieldValue 	= "";
			$this->oRefFields   = $this->oDocumentVersion->getRefFields();
			foreach ($this->Fields as $Data) {
				if ($Data->getName() == $this->aOptions["replace_field"]) {
					$this->aOptions["optional"] = $Data->getValue();
				}

				if ($this->aOptions["field"] == "field") {
					foreach ($this->aDocFields as $sField => $sValue) {
						if ($sField == $this->aOptions["value"]) {
							$sReplaceType = $this->oRefFields->getFieldType($this->aOptions["replace_field"]);
							if ($sReplaceType == "string" || $sReplaceType == "text") {
								$sValueType = $this->oRefFields->getFieldType($this->aOptions["value"]);
								$sValue 	= WFA_RecordUtils::castToString($sValue, $sValueType);
							}
							$this->aOptions["value"] = $sValue;
							break;
						}
					}
				}		
			}

			if (isset($this->aOptions["optional"])) {
				switch ($this->aOptions["field"]) {
					case "var":
						$this->getVar();
						break;
					case "field":
						$this->getField();
						break;
					case "formula":
						$this->getFormula();
						break;
					default:
						$this->getDefault();
						break;
				}
			}
		}
		else {
			$this->aOptions = array();
		}
		
		MSF_RteObjectsIntegrator::files($this->oDocument->getMediaId(), "workflow_document");
						
		return $this->aOptions;
	}
	
	private function getVar()
	{
		$oVars		   = new WF_Vars($this->oAction);
		switch ($this->aOptions["optional"]["type"]) {
			case "date":
				$Date = new MSF_Date();
				$Date->parse($oVars->getHtml($this->aOptions["value"]));
				$this->aOptions["value"] = $Date->format(CNF_DATE_FORMAT);
				break;
			case "timestamp":
				$Date = new MSF_Date();
				$Date->parse($oVars->getHtml($this->aOptions["value"]));
				$this->aOptions["value"] = $Date->format(CNF_DATE_TIME_FORMAT);
				break;
			case "text":
			case "string":
				$aValue = array();
				$sValue = $oVars->getHtml($this->aOptions["value"]);
				switch ($this->aOptions["value"]){
					case "aktenplan":
						$aAktenplan = $oVars->getValue("aktenplan");
						if (!empty($aAktenplan)) {
							foreach ($aAktenplan["titles"] AS $aTitle) {
								$aValue[] = "<b>" . MSF_Registry::get("MSF_Locale")->getWord("AKTENPLAN_INDEX") . ":</b> " . htmlspecialchars($aTitle["index"]) . ", " .
									"<b>" . MSF_Registry::get("MSF_Locale")->getWord("AKTENPLAN_TITLE") .":</b> " . htmlspecialchars($aTitle["title"]) . ", " .
									"<b>" . MSF_Registry::get("MSF_locale")->getWord("AKTENPLAN_SUBDIVISION") .":</b> " . htmlspecialchars($aTitle["department"]);
							}
						}
						$sValue = implode("; ", $aValue);
						break;
					case "control":
						$sValue = WFA_RecordUtils::getVarControl($this->oDocument->getDocumentId());
						break;
					case "decision":
						$sValue = WFA_RecordUtils::getVarDecision($this->oAction);
						break;
					case "documents_family":
						preg_match_all("|<a.*>(.*)</a>|", $sValue, $aValueSplit);
						$sValue = (!empty($aValueSplit[1])) ? implode("<br/>", $aValueSplit[1]) : $sValue;
						break;
					case "author":
					case "subdivision":
						$sValue = preg_replace("|<a.*>(.*)</a>|", "$1", $sValue);
						break;
					case "title":
						$sValue = $sValue;
						break;
				}							
				$this->aOptions["value"] = WFA_RecordUtils::stringMerge($this->aOptions, $this->aDocFields, $sValue);						
				break;
			case "structure":
				switch ($this->aOptions["value"]) {
					case "author":
						$RefValue 	= new REF_Value();
						$NRefValue 	= new REF_Value();
						$RefValue->parse($this->oDocumentVersion->getSystemField("author_name"));
						$aKeys 		= $RefValue->getKeysList();						
						if (isset($aKeys[0][2])) {
							$NRefValue->add($aKeys[0], RefFieldHelper::formatStructureValue(
													   $aKeys[0][1], 
													   $aKeys[0][2], 
													   MSF_SL::getParam($this->aOptions["optional"]["optional"], "employee", "name"), 
													   MSF_SL::getParam($this->aOptions["optional"]["optional"], "post", "post"), 
													   MSF_SL::getParam($this->aOptions["optional"]["optional"], "employeeEnable", 1), 
													   MSF_SL::getParam($this->aOptions["optional"]["optional"], "postEnable", 1),
													   true
												   )
											);
							$this->aOptions["value"] = $NRefValue->format();
							
						}
						else {
							$this->aOptions["value"] = $this->oDocumentVersion->getSystemField("author_name");
						}
						break;
					case "user":
						$UBox 		= MSF_Registry::get("MSF_UserBox");					
						if ($UBox->getCurrentStructureId() != "") {
							$RefValue 	= new REF_Value();
							$aIds 		= array($UBox->getCurrentStructureId(), $UBox->getEmployeeAttribute("id"), $UBox->getCurrentPostId());						
							$RefValue->add($aIds,  RefFieldHelper::formatStructureValue(
													   $UBox->getEmployeeAttribute("id"), 
													   $UBox->getCurrentPostId(), 
													   MSF_SL::getParam($this->aOptions["optional"]["optional"], "employee", "name"), 
													   MSF_SL::getParam($this->aOptions["optional"]["optional"], "post", "post"), 
													   MSF_SL::getParam($this->aOptions["optional"]["optional"], "employeeEnable", 1), 
													   MSF_SL::getParam($this->aOptions["optional"]["optional"], "postEnable", 1)
												   )
										   );
							$this->aOptions["value"] = $RefValue->format();
						}
						else {
							$this->aOptions["value"] = "";
						}
						
						break;
				}
				$this->getDefault();
				break;
			default:
				$this->getDefault();
				break;
		}
	}
	
	private function getField()
	{
		switch ($this->aOptions["optional"]["type"]) {
			case "table":
				$this->setEmployeeName(MSF_OrgStructureFacade::getInstance()->getEmployeeNameByStructureId($this->oDocument->getStructureId()));
				$this->setStructureId($this->oDocument->getStructureId());
				$aPost = MSF_OrgStructureFacade::getInstance()->getPostByStructureId($this->oDocument->getStructureId());
				$this->setPost(isset($aPost[0]["value"]) ? $aPost[0]["value"] : "");
				$this->setPostId(MSF_OrgStructureFacade::getInstance()->getPostIdByStructureId($this->oDocument->getStructureId()));
				$this->setEmployeeId(MSF_OrgStructureFacade::getInstance()->getEmployeeIdByStructureId($this->oDocument->getStructureId()));
				$aOptions 	= $this->aOptions["optional"]["optional"];
				$aCurrentFields = $aOptions["fields"];
				$oFields		= new REF_Fields($this->oDoctypeVersion->getFieldsXml());
				foreach ($aCurrentFields AS $sKey => $aBody) {
					if (is_string($aBody)) {
						continue;
					}
					else if ($aBody["type"] !== "TEXT") {
						$aExtField 					= $oFields->getField($aBody["type"]);
						if (!empty($aExtField)) {
							$UBox 		= MSF_Registry::get("MSF_UserBox");
							$aCurrentFields[$sKey]["type"] 		= $aExtField["type"];
							$aCurrentFields[$sKey]["default"] 	= $aExtField["default"];
							$aCurrentFields[$sKey]["optional"]  = $aExtField["optional"];
							if ($aCurrentFields[$sKey]["type"] == "structure") {
								$RefValue = new REF_Value();
								$sRes = RefFieldHelper::formatStructureValue(
										$UBox->getEmployeeAttribute("id"), 
										$UBox->getCurrentPostId(), 
										MSF_SL::getParam($aCurrentFields[$sKey]["optional"], "employee", "name"), 
										MSF_SL::getParam($aCurrentFields[$sKey]["optional"], "post", "post"), 
										MSF_SL::getParam($aCurrentFields[$sKey]["optional"], "employeeEnable", 1), 
										MSF_SL::getParam($aCurrentFields[$sKey]["optional"], "postEnable", 1)
										);
								$RefValue->add(array(
									$UBox->getCurrentStructureId(),
									$UBox->getEmployeeAttribute("id"),
									$UBox->getCurrentPostId()
								), $sRes);
								$aCurrentFields[$sKey]["optional"]["author"] = $RefValue->format();
							}
							if ($aCurrentFields[$sKey]["type"] == "text") {
								$aCurrentFields[$sKey]["default"] = str_replace(array("</p> ", "<br>"), "\n", $aCurrentFields[$sKey]["default"]);
								$aCurrentFields[$sKey]["default"] = strip_tags($aCurrentFields[$sKey]["default"]);
							}		
						}
						else {
							$aCurrentFields[$sKey]["optional"] = null;
							$aCurrentFields[$sKey]["default"] 	= "";
						}
		
					}
		
				}
				
				$aValue = array();
				$aValue["type"] 				= $this->aOptions["optional"]["type"];
				$aValue["fields"] 				= MSF_Json::encode($aCurrentFields);
				$aValue["displayInRS"] 			= isset($aOptions["displayInRS"]) ? $aOptions["displayInRS"] : "";
				$aValue["width"] 				= $aOptions["width"];
				$aValue["cellpadding"] 			= $aOptions["cellpadding"];
				$aValue["borderWidth"] 			= $aOptions["borderWidth"];
				$this->aOptions["value"] 		= addslashes($this->aOptions["value"]);				
				$this->aOptions["optional"] 	= $aValue;
				break;
			case "document":
				$aValue 	= array();
				$RefValue 	= new REF_Value();
				if (isset($this->aOptions["record_param"]) && $this->aOptions["record_param"] != "") {
					$this->aOptions["value"] = WFA_RecordUtils::specialRecordParam($this->aOptions, $this->aDocFields);
				}
				$RefValue->parse($this->aOptions["value"]);
				$aKeys 		= $RefValue->getKeysList();
				$aValues 	= $RefValue->getValuesList();
				
				foreach ($aKeys as $iKey => $aItems) {
					$RefValue = new REF_Value();
					$RefValue->add($aItems, $aValues[$iKey]);
					$aValue[] = $RefValue->format();
				}
				
				$this->aOptions["value"] = $aValue;
				break;
			case "file":
			case "picture":
				if (!empty($this->aOptions["value"])) {
					$Media 	= new MD_Media(new MD_MediaDB(MSF_Registry::get("MSF_DB")));
					$aIds 	= explode(",", $this->aOptions["value"]);
					$this->aOptions["files"] = array();
					
					foreach ($aIds as $iId) {
						$aFile 	= $Media->getFileInfo($iId);
						$aTmp 	= array();
						$aTmp["id"] 	= $aFile["file_id"];
						$aTmp["name"] 	= $aFile["name"];
						$this->aOptions["files"][] = $aTmp;
					}
				}
				break;
			default:
				$this->getDefault();
				break;
		}
	}
	
	private function getDefault()
	{
		switch ($this->aOptions["optional"]["type"]) {
			case "table":
				$this->setEmployeeName(MSF_OrgStructureFacade::getInstance()->getEmployeeNameByStructureId($this->oDocument->getStructureId()));
				$this->setStructureId($this->oDocument->getStructureId());
				$aPost = MSF_OrgStructureFacade::getInstance()->getPostByStructureId($this->oDocument->getStructureId());
				$this->setPost(isset($aPost[0]["value"]) ? $aPost[0]["value"] : "");
				$this->setPostId(MSF_OrgStructureFacade::getInstance()->getPostIdByStructureId($this->oDocument->getStructureId()));
				$this->setEmployeeId(MSF_OrgStructureFacade::getInstance()->getEmployeeIdByStructureId($this->oDocument->getStructureId()));
				$aOptions 	= $this->aOptions["optional"]["optional"];
				$aCurrentFields = $aOptions["fields"];
				$oFields		= new REF_Fields($this->oDoctypeVersion->getFieldsXml());
				foreach ($aCurrentFields AS $sKey => $aBody) {
					if (is_string($aBody)) {
						continue;
					}
					else if ($aBody["type"] !== "TEXT") {
						$aExtField 					= $oFields->getField($aBody["type"]);
						if (!empty($aExtField)) {
							$UBox 		= MSF_Registry::get("MSF_UserBox");
							$aCurrentFields[$sKey]["type"] 		= $aExtField["type"];
							$aCurrentFields[$sKey]["default"] 	= $aExtField["default"];
							$aCurrentFields[$sKey]["optional"]  = $aExtField["optional"];
							if ($aCurrentFields[$sKey]["type"] == "structure") {
								$RefValue = new REF_Value();
								$sRes = RefFieldHelper::formatStructureValue(
										$UBox->getEmployeeAttribute("id"), 
										$UBox->getCurrentPostId(), 
										MSF_SL::getParam($aCurrentFields[$sKey]["optional"], "employee", "name"), 
										MSF_SL::getParam($aCurrentFields[$sKey]["optional"], "post", "post"), 
										MSF_SL::getParam($aCurrentFields[$sKey]["optional"], "employeeEnable", 1), 
										MSF_SL::getParam($aCurrentFields[$sKey]["optional"], "postEnable", 1)
										);
								$RefValue->add(array(
									$UBox->getCurrentStructureId(),
									$UBox->getEmployeeAttribute("id"),
									$UBox->getCurrentPostId()
								), $sRes);
								$aCurrentFields[$sKey]["optional"]["author"] = $RefValue->format();
							}
							if ($aCurrentFields[$sKey]["type"] == "text") {
								$aCurrentFields[$sKey]["default"] = str_replace(array("</p> ", "<br>"), "\n", $aCurrentFields[$sKey]["default"]);
								$aCurrentFields[$sKey]["default"] = strip_tags($aCurrentFields[$sKey]["default"]);
							}		
						}
						else {
							$aCurrentFields[$sKey]["optional"] = null;
							$aCurrentFields[$sKey]["default"] 	= "";
						}
		
					}
		
				}
				
				$aValue = array();
				$aValue["type"] 		= $this->aOptions["optional"]["type"];
				$aValue["fields"] 		= MSF_Json::encode($aCurrentFields);
				$aValue["displayInRS"] 	= isset($aOptions["displayInRS"]) ? $aOptions["displayInRS"] : "";
				$aValue["width"] 		= "900"; //$aOptions["width"];
				$aValue["cellpadding"] 	= $aOptions["cellpadding"];
				$aValue["borderWidth"] 	= $aOptions["borderWidth"];
				$this->aOptions["value"] 		= str_replace("\"", "\\\"", $this->aOptions["value"]);
				$this->aOptions["optional"] = $aValue;
				break;
			case "date":
				if (!empty($this->aOptions["value"])) {
					$Date = new MSF_Date();
					$this->aOptions["value"] = $Date->parseAndFormat($this->aOptions["value"], CNF_DATE_FORMAT);
				}
				break;
			case "timestamp":
				if (!empty($this->aOptions["value"])) {
					$Date = new MSF_Date();
					$this->aOptions["value"] = $Date->parseAndFormat($this->aOptions["value"], CNF_DATE_TIME_FORMAT);
				}
				break;
			case "reference":
				$sColumns 	= "{";
				$sGroups 	= "[";
				$sFilters 	= "{";
				$iSize 		= count($this->aOptions["optional"]["optional"]["columns"]);
				$iCount 	= 0;
				$bGroup 	= ($this->aOptions["optional"]["optional"]["groupEnable"] == 1) ? "true" : "false";
				$bFilter 	= ($this->aOptions["optional"]["optional"]["filterEnable"] == 1) ? "true" : "false";
				$aFields 	= isset($this->aOptions["optional"]["optional"]["fieldsValue"])
								   ? $this->aOptions["optional"]["optional"]["fieldsValue"]
								   : array();
				
				if (isset($this->aOptions["optional"]["optional"]["columns"])) {
					foreach ($this->aOptions["optional"]["optional"]["columns"] as $sValue) {
						$bSelect = "false";
						if (in_array($sValue, $aFields)) {
							$bSelect = "true";
						}
						$sColumns .= 	$sValue . ": {'select': " .
										$bSelect . ", 'group': " .
										$bGroup . ", 'filter': " .
										$bFilter . ", 'sort': true}";
						
						if ($iCount < $iSize - 1) {
							$sColumns .= ",";
						}
						
						$iCount++;
					}
				}
				
				if (isset($this->aOptions["optional"]["optional"]["defaultGroup"])) {
					$iSize 	= count($this->aOptions["optional"]["optional"]["defaultGroup"]);
					$iCount = 0;
					foreach ($this->aOptions["optional"]["optional"]["defaultGroup"] as $sValue) {
						$sGroups .= "'" . $sValue . "'";
						
						if ($iCount < $iSize - 1) {
							$sGroups .= ",";
						}
					}
				}
				
				if (isset($this->aOptions["optional"]["optional"]["defaultFilter"])) {
					$iSize 	= count($this->aOptions["optional"]["optional"]["defaultFilter"]);
					$iCount = 0;
					foreach ($this->aOptions["optional"]["optional"]["defaultFilter"] as $sKey => $sValue) {
						if (strlen($sValue) == 0) {
							continue;
						}
						$sFilters .= $sKey . ": '" . $sValue . "'";
						
						if ($iCount < $iSize - 1) {
							$sFilters .= ",";
						}
					}
				}
				
				$sColumns .= "}";
				$sGroups .= "]";
				$sFilters .= "}";
				$this->aOptions["optional"]["optional"]["groupEnable"] 	= $bGroup;
				$this->aOptions["optional"]["optional"]["filterEnable"] 	= $bFilter;
				$this->aOptions["optional"]["optional"]["columns"] 		= $sColumns;
				$this->aOptions["optional"]["optional"]["defaultGroup"] 	= $sGroups;
				$this->aOptions["optional"]["optional"]["defaultFilter"] 	= $sFilters;
				if (isset($this->aOptions["record_param"]) && $this->aOptions["record_param"] != "") {
					$this->aOptions["value"] = WFA_RecordUtils::specialRecordParam($this->aOptions, $this->aDocFields);
				}
				if ($this->aOptions["optional"]["optional"]["multiselect"] == 1) {
					$this->aOptions["optional"]["optional"]["multiselect"] = "true";
				}
				else {
					$this->aOptions["optional"]["optional"]["multiselect"] = "false";
					if (!empty($this->aOptions["value"])){
						$aMultiValue = explode("|^|row|^|", $this->aOptions["value"]);
						$this->aOptions["value"] = $aMultiValue[0];
					}								
				}
				$this->aOptions["value"] = addslashes($this->aOptions["value"]);
				break;
			case "structure":
				if (isset($this->aOptions["record_param"]) && $this->aOptions["record_param"] != "") {
					$this->aOptions["value"] = WFA_RecordUtils::specialRecordParam($this->aOptions, $this->aDocFields);
				}
				if ($this->aOptions["optional"]["optional"]["multiselect"] == 0) {
					$this->aOptions["value"] = $this->setStructureValue($this->aOptions["value"]);
				}
				$this->aOptions["value"] = addslashes($this->aOptions["value"]);
				break;
			case "text":
			case "string":
				$this->aOptions["value"] = WFA_RecordUtils::stringMerge($this->aOptions, $this->aDocFields, $this->aOptions["value"]);						
				break;
		}
	}
	
	private function setStructureValue($sValue) {
		$sResult = "";
		if (!empty($sValue)) {
			$aMultiValue = explode("|^|row|^|", $sValue);
			$aValue = explode("/^/col/^/", $aMultiValue[0]);
			if (isset($aValue[3])) $sResult = $aMultiValue[0];
		}
		return $sResult;
	}
	
	private function getFormula() {
		$this->aOptions["value"] = WFA_RecordUtils::recordFormula($this->aOptions, $this->Fields, 
												  				  $this->oDocumentVersion->getFields(), 
												 				  $this->aOptions["value"]);
	}
	
	private $sEmployeeName 	= "";
	
	private $sPost 			= "";
	
	private $iStructureId1 	= 0;
	
	private $iPostId		= 0;
	
	private $iEmployeeId	= 0;
	
	public function setEmployeeId($iEmployeeId)
	{
		$this->iEmployeeId = $iEmployeeId;
	}
	
	public function setPostId($iPostId) 
	{
		$this->iPostId = $iPostId;
	}
	
	public function setEmployeeName($sEmployeeName)
	{
		$this->sEmployeeName = $sEmployeeName;
	}
	
	public function setPost($sPost)
	{
		$this->sPost = $sPost;
	}
		
	public function setStructureId($iStructureId)
	{
		$this->iStructureId1 = $iStructureId;
	}
	
	public function getAuthor($hasPost = false)
	{
		if ($hasPost) {
			return $this->sEmployeeName . "(" . $this->sPost . ")" . 
					REF_Value::COLUMN_DELIMITER . $this->iStructureId1 . 
					REF_Value::COLUMN_DELIMITER . $this->iEmployeeId. 
					REF_Value::COLUMN_DELIMITER . $this->iPostId;
		}
		else {
			return $this->sEmployeeName . 
					REF_Value::COLUMN_DELIMITER . $this->iStructureId1 . 
					REF_Value::COLUMN_DELIMITER . $this->iEmployeeId.
					REF_Value::COLUMN_DELIMITER . 0;
		}
		
	}
}
?>