<?php
if(!defined("__MSF")) {
	die("Direct access is not allowed");
}

include_once 'WF_Remind.php';
include_once 'modules/main/library/orgStructure/MSF_OrgStructureFacade.php';
include_once 'WF_DateFormatter.php';
include_once 'system/MSF_Locale.php';

class WFA_Remind extends WF_ActionSupport implements WF_IAction
{
			
	public function initialize()
	{
		if (IS_REQUEST) {
			$aP 			= $this->getParam('post');
			$aP['before'] 	= $aP['before_days'] . '/' . $aP['before_hours'] . '/' . $aP['before_minuts'];
			$this->setParam('post', $aP);
		}
		
		
	}
	

	public function processExecute() // ok
	{
		$this->emulateRequestContextParams();
		$this->getLog()->setEmployeeName("Detrix");
		return $this->processRequest("execute");
	}
	
	/**
	 * Array
	 * (
	 * 	   [status] 			=> enabled
	 *     [address] 			=> user
	 *     [status] 			=> enabled
	 *     [datetime] 			=> 31.12.2009 23:59
	 *     [numberOfBeforeDays] => 7
	 *     [frequency]			=> weekly
 	 * )
 	 * @return void
 	 */
	private function emulateRequestContextParams()
	{
		$oDate 					= new WF_DateFormatter();
		$aCfg 					= $this->getSettings()->getPureActions($this->getActionId());
		$aParams 				= array();
		if (isset($aCfg['options']['status'])) {
			$aParams['status']		= $aCfg['options']['status'];
		}
		else {
			$aParams['status']		= "enabled";
		}
		
		if(isset($aCfg['options']['only_workdays'])) {
			$aParams['only_workdays'] = $aCfg['options']['only_workdays'];	
		}
		else {
			$aParams['only_workdays'] = 0;
		}
		
		$aParams['address'] 	= (empty($aCfg['options']['address_type'])) ? null : $aCfg['options']['address_value'];
		$aParams['before'] 		= $aCfg['options']['before_days'] . '/' . $aCfg['options']['before_hours'] . '/' . $aCfg['options']['before_minuts'];
		$aParams['frequency'] 	= $aCfg['options']['frequency'];
		
		switch ($aCfg['options']['deadline_type']) {
			case '':
				$aParams['datetime'] = $oDate->getDatetimeByDeadline(
						$this->getDocument()->getCurrentVersion(), 
						$aCfg['options']['deadline_type'],
						null
						);
				break;
				
			case 'after':
				$aParams['datetime'] = $oDate->getDatetimeByDeadline(
						$this->getDocument()->getCurrentVersion(), 
						$aCfg['options']['deadline_type'],
						$aCfg['options']['deadline_value_days'] . '/' . $aCfg['options']['deadline_value_hours'] . '/' . $aCfg['options']['deadline_value_minuts']
						);
				break;
				
			default:
				$aParams['datetime'] = $oDate->getDatetimeByDeadline(
						$this->getDocument()->getCurrentVersion(), 
						$aCfg['options']['deadline_type'],
						$aCfg['options']['deadline_value']
						);
				break;
			
		}

		$this->setParam('post', $aParams);
	}
	
	public function processRequest($sType = "request")
	{
		if ($this->getParam('post', 'status') == 'enabled') {
			if (strlen(trim($this->getParam('post', 'datetime'))) > 0) {
				$this->logEnabled();
			}
			else {
				if($sType == "request") {
					MSF_Registry::get("MSF_Messenger")->addError(
						MSF_Registry::get('MSF_Locale')->getWord("WFA_REMIND_TIME_NONE")
					);
				}
				else {
					$this->getLog()->display(
											   $this->getActionTitle(), 
											   MSF_Registry::get('MSF_Locale')->getWord("WF_CONTEXT_EXECUTION_NOT_CORRECT_PARAMS"),
											   MSF_Registry::get('MSF_Locale')->getWord("WF_CONTEXT_EXECUTION_NOT_CORRECT_PARAMS") . "\n" 
											 . MSF_Registry::get('MSF_Locale')->getWord("WFA_REMIND_TIME_NONE")
											 );
				}
				
				return self::NULL;
			}
			
		}
		else {
			$this->logDisabled();
		}
		try {
			$aRoute		= $this->getSettings()->getActions($this->getActionId());
			if (isset($aRoute["options"])) {
				if (isset($aRoute["options"]["move_points"])) {
					$this->call("move", array("iNewPoint" => $aRoute["options"]["move_points"]));				
				}
			}		
		}
		catch(Exception $e) {
			if (CNF_MODE == "debug") {
				debug($e->getMessage());
				exit();
			}	
			return self::FAILURE;
		}
		return self::SUCCESS;
	}
	
	private function logEnabled() {
		$oOs 		= MSF_OrgStructureFacade::getInstance();
		$oDf		= new WF_DocFacade();
		$aAdress	= array_unique($oDf->addressValueToStructureIds($this->getDocument(), $this->getParam('post', 'address')));
		if (!empty($aAdress)) {
			$aEmployees = array();
			foreach ($aAdress AS $iStrId) {
				$this->saveEnabled($iStrId);
				$aEmployees[] = $oOs->getEmployeeNameByStructureId($iStrId);
			}
			$this->rewriteDataFields();
			$oLc 		= new MSF_Locale('modules/workflow/languages/russian.ini');
			$sShortText = MSF_Registry::get('MSF_Locale')->getWord("WORKFLOW_REMIND_ON") . implode(', ', $aEmployees);
			$this->getLog()->display(
					$this->getActionTitle(),
					$sShortText,
					$this->prepareFullText($sShortText)
					);
					
		}
		else {
			$this->getLog()->display(
					$this->getActionTitle(),
					MSF_Registry::get('MSF_Locale')->getWord("WF_CONTEXT_EXECUTION_NOT_CORRECT_PARAMS"),
					MSF_Registry::get('MSF_Locale')->getWord("WF_CONTEXT_EXECUTION_NOT_CORRECT_PARAMS") . "\n" .
					MSF_Registry::get('MSF_Locale')->getWord("WFA_REMIND_ADRESS_NONE")
					);
		}

	}

	private function saveEnabled($iStrId) {
		$oRemind = new WF_Remind(
				$this->getDocument()->getDocumentId(),
				$iStrId,
				$this->getActionId()
				);
		$oRemind->setDatetime($this->getParam('post', 'datetime'));
		$oRemind->setBefore($this->getParam('post', 'before'));
		$oRemind->setFrequency($this->getParam('post', 'frequency'));
		$oRemind->setDocumentTitle($this->getDocument()->getTitle());
		$oRemind->save();			
	}
	
	private function logDisabled() {
		$oOs 		= MSF_OrgStructureFacade::getInstance();
		$oDf		= new WF_DocFacade();
		$aAdress	= array_unique($oDf->addressValueToStructureIds($this->getDocument(), $this->getParam('post', 'address')));
		if (!empty($aAdress)) {
			$aEmployees = array();
			foreach ($aAdress AS $iStrId) {
				$oRemind = new WF_Remind(
					$this->getDocument()->getDocumentId(),
					$iStrId,
					$this->getActionId()
				);

				if ($oRemind->remove()) {
					$aEmployees[] = $oOs->getEmployeeNameByStructureId($iStrId);
				}
			}
			$oLc 		= new MSF_Locale('modules/workflow/languages/russian.ini');
			$sShortText = MSF_Registry::get('MSF_Locale')->getWord("WORKFLOW_REMIND_OFF") . implode(', ', $aEmployees);
			$this->getLog()->display(
					$this->getActionTitle(),
					$sShortText
					);	
		}
	}
	
	private function prepareFullText($sShortText) {
		$aBefore	 = explode('/', $this->getParam('post', 'before'));
		$sFullText	 = $sShortText . "\n";
		
		$sFullText 	.= MSF_Registry::get('MSF_Locale')->getWord("WORKFLOW_REMIND_DATE_TIME") . $this->getParam('post', 'datetime') . "\n";

		$sFullText 	.= MSF_Registry::get('MSF_Locale')->getWord("WORKFLOW_REMIND_SPA_BEFORE"). ' ' 
					. $aBefore[0] . MSF_Registry::get('MSF_Locale')->getWord("WORKFLOW_REMIND_DAYS"). ' ' 
					. $aBefore[1] . MSF_Registry::get('MSF_Locale')->getWord("WORKFLOW_REMIND_HOUR"). ' ' 
					. $aBefore[2] . MSF_Registry::get('MSF_Locale')->getWord("WORKFLOW_REMIND_MINUT") . "\n";
		$aCfg 					= $this->getSettings()->getPureActions($this->getActionId());
		$aRemindSeconds			= explode(";", $aCfg["settings"]["remindSeconds"]);
		$iFrequency				= $this->getParam('post', 'frequency');
		if(in_array($iFrequency, $aRemindSeconds)) {
			$sFullText	.= MSF_Registry::get('MSF_Locale')->getWord("WFA_REMIND_OVER") . round(($iFrequency / 60) * 100) / 100 . " " . MSF_Registry::get('MSF_Locale')->getWord("WORKFLOW_REMIND_MINUT");
		}
		else {
			$sFullText	.= MSF_Registry::get('MSF_Locale')->getWord('WFA_REMIND_' . strtoupper($iFrequency));
		}
		
		return $sFullText;
	}

	private function rewriteDataFields()
	{
		$aAction = $this->getSettings()->getActions($this->getActionId());		
		if (isset($aAction["options"]["rewrite_data_fields"])){
			$aRewriteField = $this->getDocumentVersion()->getRefFields()->getField($aAction["options"]["deadline_value"]);
			$sRewriteValue = $this->getParam('post', 'datetime');
			if (!empty($aRewriteField)) {
				$this->getDocumentVersion()->setFields(array($aAction["options"]["deadline_value"] => $sRewriteValue));
				$this->getDocumentVersion()->save();
			}
		}
		
		if (isset($aAction["options"]["record_data_field"])){
			$aDataField = $this->getDocumentVersion()->getRefFields()->getField($aAction["options"]["record_data_field"]);
			$sValue = $this->getParam('post', 'datetime');
			if (!empty($aDataField)) {
				$this->getDocumentVersion()->setFields(array($aAction["options"]["record_data_field"] => $sValue));
				$this->getDocumentVersion()->save();
			}
		}
	}

	public function processSchedule()
	{
		return self::SUCCESS;
	}
	
	public function processResponse()
	{
		return self::FAILURE;
	}
	
		
	public function processCall()
	{
		return self::SUCCESS;
	}
	
	public function cancel()
	{
		return self::SUCCESS;
	}
	
}