<?php
defined('__MSF') or die('Direct access is not allowed');

include_once "modules/media/library/MD_MediaDB.php";
include_once "modules/media/library/MD_Media.php";

abstract class WFA_AbstractViewPort
{
    /**
     * @var WF_Doctype
     */
    protected $oDoctype;

    /**
     * @var WF_DoctypeVersion
     */
    protected $oDoctypeVersion;

    /**
     * @var WF_Document
     */
	
    protected $oDocument;

    /**
     * @var WF_DocumentVersion
     */
	
    protected $oDocumentVersion;

    /**
     * @var integer
     */
    protected $iStructureId;

    /**
     * @var WF_Values
     */
	
    protected $oValues;
	
    protected $oAction;

	final public function __construct(WF_Doctype $oDoctype, WF_DoctypeVersion $oDoctypeVersion, 
									  WF_Document $oDocument, WF_DocumentVersion $oDocumentVersion, 
									  $iStructureId, $oViewAction)
	{
		$this->oDoctype 		= $oDoctype;
		$this->oDoctypeVersion 	= $oDoctypeVersion;
		$this->oDocument		= $oDocument;
		$this->oDocumentVersion	= $oDocumentVersion;
		$this->iStructureId		= $iStructureId;
		$this->oValues 			= new WF_Values($oDocument, $oDocumentVersion, $oDoctypeVersion);
		$this->oAction 			= $oViewAction;
		
    }

	final public function addActionToXml(MSF_Template $Template, $aAction)
	{
		$SpaNode	= $Template->createElement("spa");
        $Template->getModule()->appendChild($SpaNode);
		$ElmAction	= $Template->createElement("action");
        $SpaNode->appendChild($ElmAction);
		$ElmAction->setAttribute("id", 		$aAction["spa_id"]);
		$ElmAction->setAttribute("title",	$aAction["title"]);
		$ElmAction->setAttribute("type", 	$aAction["name"]);
		$sPath	= "/modules/workflow/actions/" . $aAction["name"] . "/default/spa.xsl";
                if($aAction["name"] == "decision"){
                    if(array_key_exists("use_eds",$aAction["options"]) == 1 && $aAction["options"]["use_eds"] == 1){
                $sDsf = $aAction["options"]["dsf_content"];
                    if($sDsf != ""){
                    $aDsfFields = explode(",", $sDsf);
                    $aDocFields = $this->oDocumentVersion->getFields();
                        $sPrepared	= "";
                        $sFieldType     = "";
                    foreach ($aDsfFields AS $sFieldname) {
                        if (isset($aDocFields[$sFieldname])) {
                                    $MediaDB 	= new MD_MediaDB(MSF_Registry::get("MSF_DB"));
                            $isFileOrPicture = $MediaDB->isTypeFileOrPicture($sFieldname, $this->oDocumentVersion->getFDocumentId());
                                    if($isFileOrPicture){
                                $sFieldType .= "*$sFieldname;";
                                $fileId = $aDocFields[$sFieldname];
                                $fileIdArray = explode(",", $fileId);
                                        foreach($fileIdArray as $valId){
                                    $sPrepared .= $MediaDB->getMD5File($valId);
                                }
                            } else {
                                $sFieldType .= "$sFieldname;";
                                $sPrepared .= $aDocFields[$sFieldname];
                            }
                        }
                    }
                    $aAction["options"]["hash_for_sign"] = md5($sPrepared);
                    $aAction["options"]["fields_for_hash"] = $sFieldType;
                }
            }
        }
        $Template->addXslPath($sPath);
        $sUrl = "";
        if (!isset($aAction["url"])) {
			$sUrl = "/workflow/document/spa/" . $aAction["name"]. "/" . $aAction["spa_id"] . "/" . $this->oDocument->getDocumentId();
		}
		else {
            $sUrl = $aAction["url"];
        }

        if (isset($_GET['hideMenu'])) {
            $sUrl .= "?hideMenu";
        }

		$ElmAction->setAttribute("url", 	$sUrl);
        foreach ($aAction["settings"] AS $sName => $sValue) {
            $ElmAction->setAttribute($sName, $sValue);
        }

		$aOptions	= array_merge($this->oDocument->getLog()->getDelegated($this->oDocument->getPointId(), $aAction["spa_id"]), $aAction["options"]);
        $this->recursiveBuildXml($Template, $ElmAction, $aOptions);
        $dom = $Template->getDomDocument();
        $xmldata = $dom->saveXML();
    }

	final private function recursiveBuildXml(MSF_Template $Template, $ParentNode, $aData)
	{
        foreach ($aData AS $sKey => $mValue) {
            $ElmItem = $Template->createElement("item");
            $ParentNode->appendChild($ElmItem);
            $ElmItem->setAttribute("name", $sKey);

            if (is_array($mValue)) {
                $this->recursiveBuildXml($Template, $ElmItem, $mValue);
			}
			else {
                $ElmItem->setAttribute("value", $mValue);
            }
			
        }
		
    }

    abstract public function port(array $aActionParams, array $aCurrentDelegated);
}
?>