<?php
include_once "library/MSF_EnumValue/MSF_EnumValue.php";
include_once "modules/workflow/library/common/WF_Exception.php";
include_once "modules/workflow/library/actions/WF_ExecutionContext.php";
include_once "modules/workflow/library/actions/WF_InfiniteLoopChecker.php";
include_once "modules/workflow/library/folders/WF_Folder.php";
include_once "modules/workflow/library/security/WF_FoldersSwitcherDB.php";
include_once "modules/workflow/library/security/WF_FoldersSwitcher.php";
include_once "modules/workflow/library/doctype/WF_DoctypesList.php";
include_once "modules/workflow/library/document/WF_Document.php";
include_once "modules/references/library/Reference/REF_Template.php";
include_once "modules/workflow/library/WF_Utils.php";
include_once "modules/workflow/library/folders/WF_Aktenplan.php";
include_once "modules/workflow/library/security/WF_AktenplanSwitcher.php";

include_once "modules/workflow/library/security/WF_DoctypeSwitcher.php";
include_once "modules/workflow/view/xml/WF_DoctypesListXml.php";

include_once "modules/workflow/library/actions/WF_DocFacade.php";
include_once "modules/workflow/library/actions/WF_ActionBox.php";
include_once "modules/archive/library/ARCH_ArchiveElement.php";
include_once "library/MSF_Export/MSF_ExportToExcel.php";
include_once "library/MSF_Export/MSF_HtmlExporter.php";
include_once "modules/eventlog/library/ELOG_EventLog.php";
include_once "modules/media/library/MD_MediaDB.php";
include_once "modules/media/library/MD_Media.php";
// [/include]

/**
 * @package 	workflow
 * @copyright 	Copyright (c) 2007, Detrix
 */
class MSF_FrontController extends MSF_ControllerSupport
{
	//
	// Base URI-pattern for documentAction()
	// http://dev.msf/workflow/document/{URI_STATE}
	// or
	// http://dev.msf/workflow/folders/{URI_STATE}
	//
	const URI_STATE						= 0;
	
	//
	// First URI-pattern for documentAction()
	// http://dev.msf/workflow/document/{URI_STATE = create}/{URI_DOCTYPE_ID}
	//
	const URI_DOCTYPE_ID				= 1;
	
	//
	// Second URI-pattern for documentAction()
	// http://dev.msf/workflow/document/{URI_STATE = spa}/{URI_SPA_ID}/{URI_DOCUMENT_ID}/{URI_DOCUMENT_VERSION_ID}
	//
	const URI_SPA_ID					= 1;
	
	const URI_DOCUMENT_ID				= 2;
	
	const URI_DOCUMENT_VERSION_ID		= 3;
	
	//
	// Second URI-pattern for documentAction()
	// http://dev.msf/workflow/document/{URI_STATE = view}/{URI_SPA_ID}/{URI_VIEW_DOCUMENT_ID}/{URI_VIEW_DOCUMENT_VERSION_ID}
	//
	const URI_VIEW_DOCUMENT_ID			= 1;
	
	const URI_VIEW_DOCUMENT_VERSION_ID	= 2;
	

	
	private $iDoctypeId					= 0;
	
	private $iDoctypeVersionId			= 0;
	
	private $iDocumentId				= 0;
	
	private $iDocumentVersionId			= 0;
	
	/////////////////////////////////////////////////////////////////////////////////////////////////
	//
	// Public Doers
	//
	/////////////////////////////////////////////////////////////////////////////////////////////////

	public function indexAction()
	{
		echo "Welcome! Its workflow module.";
	}

	public function documentAction()
	{
		WF_Utils::appendDate($this->Template, $this->Template->getModule());
		switch ($this->getParam(self::URI_STATE)) {
			case "edit":
				$this->documentAction__edit();
				break;
			case "create":
				$this->documentAction__create();
				break;
			case "view":
				$this->documentAction__view();
				break;
			case "spa":
				$this->documentAction__spa();
				break;
			case "remove":
				$this->documentAction__remove();
				break;
			case "setversion":
				$this->documentAction__setversion();
				break;
		}

	}
	
	public function documentAction__setversion() {
		$iDocId 	= $this->getIntParam(1);
		$iDocVerId 	= $this->getIntParam(2);
		$oF 		= new WF_DocFacade();
		$oDoc		= $oF->getDocumentById($iDocId);
		$oDoc->setDocumentVersionId($iDocVerId);
		$oVer		= $oDoc->getCurrentVersion();
		$oDate		= new MSF_Date();
		$oDate->parse($oVer->getCreatedAt());
		$aSysFields	= $oVer->getSystemFields();
		$oRefValue	= new REF_Value();
		$oRefValue->parse($aSysFields["author_name"]); 
		$aValues	= $oRefValue->getValuesList();
		$sDescr		= MSF_Registry::get("MSF_Locale")->getWord("DOCUMENT_VERSION") . ": " . $oDate->format(CNF_DATE_TIME_FORMAT) . ", " . MSF_Registry::get("MSF_Locale")->getWord("DOCUMENT_VERSION_AUTHOR") . ": " . $aValues[0];
		$oLog		= $oDoc->getLog();
		$oO			= MSF_OrgStructureFacade::getInstance();
		$iStrId		= $oO->getCurrentStructureId();
		$oLog->setEmployeeName($oO->getEmployeeNameByStructureId($iStrId));
		$oLog->display(
                                MSF_Registry::get("MSF_Locale")->getWord("DOCUMENT_VERSION_CHANGED"),
				$sDescr
				);
		$oDoc->save();
		$this->redirect("/workflow/document/view/" . $iDocId);
	}
	
	
	public function foldersAction()
	{		
		$Switcher 	= new WF_FoldersSwitcher(MSF_Registry::get("MSF_DB"));
		$sAction 	= $this->getParam(self::URI_STATE);
		$iId 		= (($sAction == "group") || $sAction == "autocomplete") ? $this->getIntParam(1) : $this->getIntParam(0);			
		
		// Проверка на существование журнала		
		try {
			$TmpFolder	= new WF_Folder(new WF_FolderDB(MSF_Registry::get("MSF_DB")));
			if($sAction != "removedocs" && $sAction != "group") {
				$TmpFolder->load($iId);
			}				
		}
		catch(WF_FolderNotFoundException $oException) {
			$View = new WF_DoctypesListXml();
			$this->setView($View);
			MSF_Registry::get("MSF_Messenger")->addError($this->Locale->getWord("ROOT_404_PAGE_FOLDER"));
			return;			
		}
		
		switch ($sAction) {
			case "removedocs":
				$iFolderId = $this->getIntParam(1);
				if (IS_REQUEST) {
					$this->removeDocs($iFolderId, (array) MSF_SL::getParam($_POST, "document_id"));
				}

				$this->redirect("/workflow/folders/" . $iFolderId);
				break;
		}
		
		if ($Switcher->hasAccess("view", WF_Utils::getPath(), $iId)) {
			switch($sAction) {
				case "group"	:
					$this->groupFolders();
					break;
				case "autocomplete"	:
					$this->autocompleteFolders($iId);
					break;
				default			:
					$this->indexFolders((int) $sAction);
					break;
			}

		}
		else {
			//MSF_AccessDenied::exitApp('You are not allowed');
			$this->notAccessAction();
			return;
		}	

	}
	
	private function removeDocs($iFolderId, $aIds)
	{
		$sDocs 		= "";
		$bPass 		= true;
		$WF_FDoc 	= new WF_DocFacade();
		$iCount 	= 0;
		
		for ($i = 0; $i < count($aIds); $i++) {
			$Document = $WF_FDoc->getDocumentById($aIds[$i]);
			
			if ($Document === null) {
				continue;
			}
			
			$iCount++;
			$bRemove 			= false;
			$aAllowedDoctypes	= WF_Utils::getAllowedAuthorIds($iFolderId);
			$aAccessData		= isset($aAllowedDoctypes[$Document->getDoctypeId()]) ? $aAllowedDoctypes[$Document->getDoctypeId()] : array();
			$iCurrStrId 		= $Document->getStructureId();
			$aStrIds 			= array();
			
			for ($j = 0; $j < count($aAccessData); $j++) {
				$aStrIds = array_merge($aStrIds, $aAccessData[$j]["author_ids"]);
				if ($aAccessData[$j]["remove"] == 2) {
					$aStrIds = array_diff($aStrIds, $aAccessData[$j]["author_ids"]);
				}
			}
			
			if (in_array($iCurrStrId, $aStrIds)) {
				$bRemove = true;
			}
			elseif ($iCurrStrId == MSF_Registry::get("MSF_UserBox")->getCurrentStructureId() && $Document->getPointId() == 0) {
				$bRemove = true;
			}
			
			if (!$bRemove) {
				$bPass = false;
				$sDocs .= $Document->getTitle() . ", ";
			}
			else {
				$DVersion			= $Document->getDoctypeVersion();
				$Doctype			= $DVersion->getDoctype();
				$DocumentVersion	= $Document->getCurrentVersion();
				$this->documentRemoveEventLog($Document->getTitle());
				$UF	= new MSF_UniqueField(MSF_Registry::get("MSF_DB"));
				$UF->freeFields($DocumentVersion->getFieldsAsList(), MSF_UniqueField::TYPE_DOCTYPE, $Doctype->getDoctypeId());
							
				$Document->remove();
			}
		}
		
		if (!$bPass) {
			MSF_Registry::get("MSF_Messenger")->addError(
				MSF_Registry::get("MSF_Locale")->getWord("WORKFLOW_DOCUMENTS_REMOVE_DENIED") .
				" " . substr($sDocs, 0, strlen($sDocs) - 2)
			);
		}
		elseif ($iCount > 0) {
			MSF_Registry::get("MSF_Messenger")->addMessage(MSF_Registry::get("MSF_Locale")->getWord("WORKFLOW_DOCUMENT_DELETED"));
		}
	}
	
	private function documentRemoveEventLog($sEventTitle) {
		$oEventLog = new ELOG_EventLog(MSF_Registry::get("MSF_DB"));
		$oEventLog->addItem(array("event_type"=>"detrix", "event_title"=>$sEventTitle));
	}
	
	private function isGrouped()
	{
		foreach ((array) MSF_SL::getParam($_GET, "rs_groups") AS $iGroup) {
			if ($iGroup == 1) {
				return true;
			}
			
		}
		
		return false;
	}
	
	public function dossierAction()
	{
		$Switcher = new WF_AktenplanSwitcher(MSF_Registry::get("MSF_DB"));
		if ($Switcher->hasAccess("view", WF_Utils::getPath(), 0)) {
			$Aktenplan	= new WF_Aktenplan();
			$View 		= new WF_AktenplanXml();
			if(IS_REQUEST && $this->isGrouped()) {
				$View->buildGroup($Aktenplan->getGroup());
			}
			else {
				$View->addTemplate("/modules/workflow/view/templates/default/xsl/aktenplan.xsl");
				$View->buildRecordset($Aktenplan->getRecordset());
			}

			$this->setView($View);
			$this->Template->setPageTitle(
				$this->locale("WORKFLOW_DOCUMENTS") .
				$this->locale("ROOT_TITLE_SEPARATOR") .
				$this->locale("WORKFLOW_FOLDER_AKTENPLAN")
			);
			
		}
		else {
			//MSF_AccessDenied::exitApp("Bite my shiny metal ass, piece of meat!");
			$this->notAccessAction();
			return;
		}
			
	}
	
	public function ajaxfoldersAction()
	{
		$iCounter 			= 0;
		$aResult 			= array();
		$aSelectedFolders	= MSF_SL::getParam($_POST, "folders", array());
		$aDocTypes			= explode(",", (string) MSF_SL::getParam($_POST, "docTypes", ""));
		if ($aDocTypes[0] == "undefined") {
			$aDocTypes = array();
		}
		
		$aGroups 			= array(array("group_id" => 0, "name" => $this->locale("WORKFLOW_ADMIN_LIST_NONGROUP_TITLE")));
		$LGroups 			= new WF_FoldersGroupsList();
		$FGroups 			= new WF_FolderGroup(new WF_FolderGroupDB($this->DB));
		$aGroups 			= array_merge($aGroups, $LGroups->getList());
		$FS 				= new WF_FoldersSwitcher(MSF_Registry::get("MSF_DB"));
		$aFWD				= $FGroups->getFoldersWithDoctypes($aDocTypes);
		
		for ($i = 0; $i < count($aGroups); $i++) {
			$FGroups->load($aGroups[$i]["group_id"]);
			$aFolders 	= $FGroups->getFolders();
			$bCheck 	= false;
//                        $aGroups[$i]["name"] = MSF_Registry::get("MSF_Locale")->getI18N($aGroups[$i]["name"]);
//                        $aGroups[$i]["description"] = MSF_Registry::get("MSF_Locale")->getI18N($aGroups[$i]["description"]);
			if (!empty($aFolders)) {
				for ($j = 0; $j < count($aFolders); $j++) {
					if ($FS->hasAccess("view", WF_Utils::getPath(), $aFolders[$j]["folder_id"])) {
						if((empty($aSelectedFolders) || in_array($aFolders[$j]["folder_id"], $aSelectedFolders)) && 
							in_array($aFolders[$j]["folder_id"], $aFWD)) {
                                                        $aFolders[$j]["name"] = MSF_Registry::get("MSF_Locale")->getI18N($aFolders[$j]["name"]);
                                                        $aFolders[$j]["description"] = MSF_Registry::get("MSF_Locale")->getI18N($aFolders[$j]["description"]);
							$aResult[$iCounter]["folders"][] = $aFolders[$j];
							$bCheck = true;
						}
												
					}
					
				}
				
				if ($bCheck) {
					$aResult[$iCounter]["group_id"] = $aGroups[$i]["group_id"];
					$aResult[$iCounter]["name"] 	= MSF_Registry::get("MSF_Locale")->getI18N($aGroups[$i]["name"]);
					$iCounter++;
				}
				
			}
			
		}

		$View = new MSF_ViewSupportJson();
		$View->set("data", $aResult);
		$this->setView($View);
	}
	public function ajaxHashForSignAction(){
            $MediaDB 	= new MD_MediaDB(MSF_Registry::get("MSF_DB"));            
            $iDocumentId = (int)MSF_SL::getParam($_POST, "documentId", 0);
            $iDocumentVersionId = (int)  MSF_SL::getParam($_POST, "documentVersionId", 0);
            $aFields     = (string)MSF_SL::getParam($_POST, "fields", "");      
            $decId       = (string)MSF_SL::getParam($_POST, "decId", "");
            
            $hash = $MediaDB->getFieldValueAndFilePath($iDocumentId, $iDocumentVersionId, $aFields);            
            
            $View = new MSF_ViewSupportJson();
		$View->set("data", $hash);
                $View->set("decId", $decId);
		$this->setView($View);
        }
	public function ajaxdocumentsAction()
	{
		$aToOutput = array();
		$iFolderId	= (int)MSF_SL::getParam($_GET, "folderId", 0);
		$aDocTypes	= explode(",", (string) MSF_SL::getParam($_POST, "docTypes", ""));
		$Folder		= new WF_Folder(new WF_FolderDB(MSF_Registry::get("MSF_DB")));
		$Folder->load($iFolderId);		
		$aRecordset = $Folder->getDocumentsRecordset(true, $aDocTypes);
		// Добавление в минипредставление типа документа...
		if(isset($aRecordset["titles"]) && isset($aRecordset["records"])) {
                        for($idx = 1; $idx < count($aRecordset["titles"]); $idx ++){
                            $aRecordset["titles"][$idx]["title"] = MSF_Registry::get("MSF_Locale")->getI18N($aRecordset["titles"][$idx]["title"]);
                        }
			$oDocFacade = new WF_DocFacade();
			$aRecordset["titles"][] = array(
				"orderBy" => "none",
                "isGroup" => 0,
                "name" 	  => "doctype",
                "title"   => MSF_Registry::get("MSF_Locale")->getWord("WORKFLOW_DOCTYPE_RECORDSET_TITLE"),
               	"type" 	  => "string",
                "sort"	  => 0,
                "filter"  => 0,
                "group"	  => 0,
                "visible" => 0,
                "compare" => "equal",
                "filterEnable" => 0,
				"hidden"  => 1			
				);			
			for($i = 0; $i < count($aRecordset["records"]); $i++) {
				$iDoctypeId = $aRecordset["records"][$i]["columns"][0]["value"];
				$oDoctype 	= $oDocFacade->getDoctypeByDocumentId($iDoctypeId);
				if ($oDoctype !== null) {
					$sDoctypeTitle = $oDoctype->getTitle();
					$aRecordset["records"][$i]["columns"][] = array(
						"value" 	=> $sDoctypeTitle,
						"original"  => $sDoctypeTitle,
						"mark" 		=> 0,
						"type"		=> "string",
						"name"		=> "doctype",
						"hidden"	=> 1);							
				}
	
			}		

		}
		
		$View = new MSF_ViewSupportJson();
		$View->set("data", $aRecordset);
		$this->setView($View);
	}

	public function exportRecordsetAction() {		
		$iFolderId = $this->getIntParam ( 0 );
		$Folder = new WF_Folder ( new WF_FolderDB ( MSF_Registry::get ( "MSF_DB" ) ) );
		$Folder->load ( $iFolderId );
		$aRecordset = $Folder->getDocumentsRecordset ( false, array (), 0, true );
		
		$iPages = $aRecordset ["pagination"] ["totalPages"];
		$oPHPExcel = new MSF_ExportToExcel ( );
		
		$oPHPExcel->exportTitles ( $aRecordset );
		for($i = 0; $i < $iPages; $i ++) {
			set_time_limit(MSF_Recordset::$RECORDS_PER_PAGE);
			$aRecordset = $Folder->getDocumentsRecordset ( false, array (), $i * MSF_Recordset::$RECORDS_PER_PAGE, true );
			$oPHPExcel->exportBody ( $aRecordset );
		}
		
		$sLink		= $oPHPExcel->saveSpreadsheet ();
		$aConfig	= $Folder->getConfig();
		$aResult 	= array(
			"link"	=> $sLink,
			"ext"	=> "xls",
			"name"	=> urlencode($aConfig["name"])
			);
			
		$View = new MSF_ViewSupportJson();
		$View->set("data", $aResult);
		$this->setView($View);		
	} 
	
	public function exportAction()
	{
		$iDocId			= $this->getIntParam(0);
		$iDocVersionId	= $this->getIntParam(1);
		$oExporter		= new MSF_HtmlExporter($iDocId, "document", $iDocVersionId);
		$oExporter->exportToWord();	
	}
	
	
	/////////////////////////////////////////////////////////////////////////////////////////////////
	//
	// Private Doers
	//
	/////////////////////////////////////////////////////////////////////////////////////////////////
	private function documentAction__view()
	{
		try {
			$aInitParams = array(
					"spa_name" 					=> "view",
					"spa_id" 					=> 0,
					"subject_id" 				=> $this->getIntParam(self::URI_VIEW_DOCUMENT_ID),
					"subject_version_id" 		=> $this->getIntParam(self::URI_VIEW_DOCUMENT_VERSION_ID),
					"initializing"				=> true
					);
			//debug($aInitParams);
			//exit();
			$this->documentAction__spa($aInitParams);
		}
		catch(WF_DocumentNotExistedException $oException) {
			$oArchive	  = new ARCH_ArchiveElement();
			$iArchiveId = $oArchive->findItemByDocId($this->getIntParam(self::URI_VIEW_DOCUMENT_ID));			
			if($iArchiveId != -1) {				
				$this->redirect("/archive/view/" . $iArchiveId);			
			}
			else {
				$View = new WF_DoctypesListXml();
				$this->setView($View);
				MSF_Registry::get("MSF_Messenger")->addError($this->Locale->getWord("ROOT_404_PAGE_MESSAGE"));
				return;
			}
						
		}
		catch(WF_DoctypeNotExistedException $oException) {
			throw $oException;	
		}

	}
	
	
	//
	// URI-pattern for documentAction__spa()
	// http://dev.msf/workflow/document/{URI_STATE = spa}/{URI_SPA_ACTION}/{URI_SPA_ID}/{URI_VIEW_DOCUMENT_ID}/{URI_VIEW_DOCUMENT_VERSION_ID}
	//
	const URI_SPA_ACTION_NAME			= 1;
	
	const URI_SPA_ACTION_ID				= 2;
	
	const URI_SPA_DOCUMENT_ID			= 3;
	
	const URI_SPA_DOCUMENT_VERSION_ID	= 4;
	
	private function documentAction__spa($aInitParams = null)
	{	
		$oUb = MSF_Registry::get("MSF_UserBox");
                //debug($oUb);
                //exit;
		if ($oUb->getCurrentStructureId() > 0) {
			try {			
				if ($aInitParams == null) {
					$sSpaAction 		= $this->getParam(self::URI_SPA_ACTION_NAME);
					$iSpaActionId 		= $this->getIntParam(self::URI_SPA_ACTION_ID);
					$iSubjectId			= $this->getIntParam(self::URI_SPA_DOCUMENT_ID);
					$iSubjectVersionId 	= $this->getIntParam(self::URI_SPA_DOCUMENT_VERSION_ID);
					$sInitializing		= true;
				}
				else {
					$sSpaAction 		= $aInitParams["spa_name"];
					$iSpaActionId 		= $aInitParams["spa_id"];
					$iSubjectId			= $aInitParams["subject_id"];
					$iSubjectVersionId 	= $aInitParams["subject_version_id"];
					$sInitializing		= $aInitParams["initializing"];
				}
				
				$Ab = WF_ActionBox::newInstance();
				$Ab->loadAction($sSpaAction, $iSpaActionId, $iSubjectId, $iSubjectVersionId, $sInitializing);
				$Action	= $Ab->getAction();
				$Action->setParam("get", $_GET);
				$iStrId	= MSF_Registry::get("MSF_UserBox")->getCurrentStructureId() ;
				if($iStrId!== null && $Action->hasAccess()) {
					$oInfChecker 		= new WF_InfiniteLoopChecker();
					$aNotVerifyAction	= array("remove", "create");			
					if (!$Action->isInitialized() && $Action->isExisted() && !in_array($Action->getActionName(), $aNotVerifyAction)) {
						$oInfChecker->step();					
						if(!$oInfChecker->isInfinite()) {
							try {
								$oExecutionContext = new WF_ExecutionContext($Action);
								if ($oExecutionContext->execute()) {
									$Action->save();
									if (!IS_REQUEST) {
										$this->redirect($Action->getRedirectUrl() . "?" . $oInfChecker->getSteps());
									}
									
								}
		
							}
							catch (Exception $E) {
								if (CNF_MODE == "debug") {
									throw $E;
								}
								else {
									MSF_Registry::get("MSF_Messenger")->addError(MSF_Registry::get("MSF_Locale")->getWord("WF_CONTEXT_EXECUTION_NOT_FINISHED_CORRECTLY"));
									if ($Action->getDocument() !== null) {
										$this->redirect($Action->getRedirectUrl() . "?" . $oInfChecker->getSteps());
									}
								}
								
								
								
	
								
							}
		
						}
		
						$oInfChecker->clear();
					}
							
					if (IS_REQUEST && !$oInfChecker->isInfinite()) {					
						try {						
							$Action->setParam("post", array_merge($_POST, $_FILES));												
							switch ($Ab->process("request")) {
								case WF_IAction::SUCCESS:				
									$Action->save();
									if ($Action->isWaiting()) {
										$Action->addWaited();
										$Action->next();
									}
									$Action->save();
								case WF_IAction::NULL:								
									$sUrl = $Action->getRedirectUrl();
									if (!empty($sUrl)) {
										$this->redirect($sUrl);
									}
									else {
										$Ab->process("response");
										$this->setView($Action->getView());
									}
								
									break;
								case WF_IAction::FAILURE:																
									if (CNF_MODE == "debug") {
										throw new Exception($Action->getActionName() . ": Execution of context process is failed");
									}
									else {
										MSF_Registry::get("MSF_Messenger")->addError(MSF_Registry::get("MSF_Locale")->getWord("WF_CONTEXT_EXECUTION_NOT_FINISHED_CORRECTLY").'['.$Action->getActionTitle().']');
										if ($Action->getDocument() !== null) {
											$this->redirect($Action->getRedirectUrl() . "?" . $oInfChecker->getSteps());
										}
								
									}								
									
									
									
									
	
							}
		
						}
						catch (Exception $E) {
							if (CNF_MODE == "debug") {
								throw $E;
							}
							else {
								MSF_Registry::get("MSF_Messenger")->addError(MSF_Registry::get("MSF_Locale")->getWord("WF_CONTEXT_DELEGATION_NOT_FINISHED_CORRECTLY"));
								if ($Action->getDocument() !== null) {
									$this->redirect($Action->getRedirectUrl() . "?" . $oInfChecker->getSteps());
								}
							
							}
		
						}
		
					}
					else {
						if($oInfChecker->isInfinite()) {
							MSF_Registry::get("MSF_Messenger")->addError(MSF_Registry::get("MSF_Locale")->getWord("WORKFLOW_DOCUMENT_IN_INFINITE_CYCLE"));
						}
					
						switch ($Ab->process("response")) {
							case WF_IAction::SUCCESS:
								$this->setView($Action->getView());
								break;
							case WF_IAction::FAILURE:
								$sRedirectUrl = $Action->getRedirectUrl();
								if (strlen($sRedirectUrl) > 0) {
									$this->redirect($sRedirectUrl);
								} 
								
								break;
						}
	
					}
		
				}
				else if($iStrId!== null && $Action->hasAccess("view")) {
					$this->redirect("/workflow/document/view/" . $iSubjectId);
					return;	
					//MSF_AccessDenied::exitApp($this->Locale->getWord("ROOT_ERROR_NO_ACCESS"));
	
				}
				else {
					$this->notAccessAction();
					return;
				}
				
			}
			catch (WF_DoctypeNotExistedException $oException) {			
				throw $oException;
			}
			catch (WF_DocumentNotExistedException $oException) {
				throw $oException;
			}
			catch(Exception $oException) {	
				MSF_Logger::logTrace($oException);			
				throw $oException;								
			}
			
			/*
			catch (MSF_PageNotFoundException $oException) {
				throw $oException;
	//			/MSF_AccessDenied::FileNotFound($this->Locale->getWord("ROOT_DOCUMENT_NOT_FOUND"), $this->Locale->getWord("ROOT_404_PAGE_MESSAGE"));
			}
			catch (Exception $oException) {
				//$this->setView();
				//echo $oException->getMessage();
				//exitApp($oException->getMessage());
				//MSF_Registry::get("MSF_Messenger")->addError($oException->getMessage());
				throw new MSF_PageNotFoundException($oException->getMessage());
				
			}
	*/
		}
		else {
			$this->setView(new MSF_ViewSupportXml());
		}
		

	}
       
	const URI_CREATE_SUBJECT_ID			= 1;
	

	private function documentAction__create()
	{
		$aInitParams = array (
				"spa_name" 				=> "create",
				"spa_id" 				=> 0,
				"subject_id" 			=> $this->getIntParam(self::URI_CREATE_SUBJECT_ID),
				"subject_version_id" 	=> 0,
				"initializing"			=> false
				);

		$this->documentAction__spa($aInitParams);
	}
	
	private function indexDocument()
	{
		echo "indexDocument";
	}
	
	private function indexFolders($iId)
	{
		
		$Folder		= new WF_Folder(new WF_FolderDB(MSF_Registry::get("MSF_DB")));
		$View		= new WF_FolderXml(MSF_Registry::get("MSF_Template"), MSF_Registry::get("MSF_Locale"));
		$List		= new WF_DoctypesList(new WF_DoctypesListDB(MSF_Registry::get("MSF_DB")));
		$AB			= MSF_Registry::get("MSF_ActionBar");
		MSF_AutoCompletor::setUri("/workflow/folders/autocomplete/" . $iId);
				
		$aList		= $List->getLinked($iId);
		$aDropDowns	= array();
		foreach($aList as $aDoctype) {
			$aTmp				= array();
			$aTmp["title"]		= $aDoctype["title"];
			$aTmp["description"]= $aDoctype["description"];
			$aTmp["url"]		= "/workflow/document/create/" . $aDoctype["doctype_id"] . "";
			$aTmp["image"]		= "";
			$aTmp["onclick"]	= "";
			array_push($aDropDowns, $aTmp);
		}
		
		$AB->addDropDown($this->locale("WORKFLOW_DOCUMENT_CREATE"), $aDropDowns, "return false");		
		$Folder->load($iId);
		$data	= $Folder->getDocumentsRecordset();

        //var_dump($data['records'][1]);
		//die('here');
		$View->buildRecordset($Folder->getDocumentsRecordset());
		$View->addTemplate("/modules/workflow/view/templates/default/xsl/folders.xsl");

		$aConfing	= $Folder->getConfig();
                
                $msfLocale = MSF_Registry::get("MSF_Locale");
                $aConfing["name"] = $msfLocale->getI18N($aConfing["name"]);

		if($aConfing["group_id"] == 0) {
			$sTitle	=	$this->locale("WORKFLOW_DOCUMENTS") .
						$this->locale("ROOT_TITLE_SEPARATOR") .
						$aConfing["name"];
		}
		else {
			$FodlerGroup	= new WF_FolderGroup(new WF_FolderGroupDB(MSF_Registry::get("MSF_DB")));
			$FodlerGroup->load($aConfing["group_id"]);
			$aGroupConfig	= $FodlerGroup->getConfig();
			
			$sTitle	=	$this->locale("WORKFLOW_DOCUMENTS") .
						$this->locale("ROOT_TITLE_SEPARATOR") .
						MSF_Registry::get("MSF_Locale")->getI18N($aGroupConfig["name"]) .
						$this->locale("ROOT_TITLE_SEPARATOR") .
						MSF_Registry::get("MSF_Locale")->getI18N($aConfing["name"]);
		}
		
		$sInitPath = "/workflow/exportRecordset/" . $iId;
		$this->Template->getModule()->appendChild($this->Template->createElement("initPath", $sInitPath));	
		$this->Template->setPageTitle($sTitle);
		
		$this->setView($View);
	}
	
	private function documentAction__remove()
	{
		$aInitParams = array (
				"spa_name" 				=> "remove",
				"spa_id" 				=> 0,
				"subject_id" 			=> $this->getIntParam(2),
				"subject_version_id" 	=> 0,
				"initializing"			=> true
				);

		$this->documentAction__spa($aInitParams);
		$this->redirect("/workflow/folders/" . $this->getIntParam(1));
	}
	
	private function autocompleteFolders($iId) {
		$Folder		= new WF_Folder(new WF_FolderDB(MSF_Registry::get("MSF_DB")));
		$Folder->load($iId);
		MSF_AutoCompletor::autoCompleteResponse($Folder->getDocumentsRecordset(true));
	}
	
	private function groupFolders()
	{
		$Folder	= new WF_Folder(new WF_FolderDB(MSF_Registry::get("MSF_DB")));
		$iId	= $this->getIntParam(1);
		$View	= new WF_FolderXml(MSF_Registry::get("MSF_Template"), MSF_Registry::get("MSF_Locale"));
		$Folder->load($iId);

		$View->buildGroup($Folder->getDocumentsGroup());
		$this->setView($View);
	}
	
}

?>