<?php

defined("__MSF") or die("Direct access is not allowed");

include_once "modules/workflow/view/xml/WF_WorkflowListXml.php";
include_once "library/MSF_Recordset/MSF_Recordset.php";

class WFC_MainList extends WFC_Abstract
{
	private $Recordset;
	
	private $Locale;
	
	private function DB()
	{
		return MSF_Registry::get("MSF_DB");
	}
	
	private function createView()
	{
		$Locale = MSF_Registry::get("MSF_Locale");
		
		$sSql = "CREATE OR REPLACE FUNCTION wf_folders_null_replace(group_name varchar) RETURNS varchar AS $$
					BEGIN
		    			IF group_name IS NULL THEN
		        			RETURN '" . $Locale->getWord("WORKFLOW_ADMIN_LIST_NONGROUP_TITLE") . "';
		    			END IF;
			    		RETURN group_name;
					END;
				$$ LANGUAGE plpgsql;";
		
		$this->DB()->executeQuery($sSql);
		
		$sSql = "CREATE OR REPLACE VIEW wf_folders_doctypes_view AS\n" .
					"SELECT\n" .
						"wff.group_id,\n" .
						"wf_folders_null_replace(wfg.name) AS group_name,\n" .
						"wff.folder_id,\n" .
						"wff.name AS folder_name\n" .
					"FROM\n" .
						"wf_folders wff\n" .
					"LEFT JOIN\n" .
						"wf_folders_groups wfg\n" .
					"ON\n" .
						"wfg.group_id = wff.group_id";
		
		$this->DB()->executeQuery($sSql);
		
		$sSql = "CREATE OR REPLACE RULE wf_folders_doctypes_view_insert AS
       				ON INSERT TO
       					wf_folders_doctypes_view
       				DO INSTEAD (
       					INSERT INTO
       						wf_folders
       					(
       						folder_id,
       						name
       					)
     					VALUES (
     						new.folder_id,
     						new.folder_name
     					);
     					INSERT INTO
     						wf_folders_groups
       					(
       						group_id,
       						name
       					)
     					VALUES (
     						new.group_id,
     						new.group_name
     					)
     				)";
		
		$this->DB()->executeQuery($sSql);
		
		$sSql = "INSERT INTO\n" .
					"wf_folders_doctypes_view\n" .
				"(\n" .
					"group_id,\n" .
					"group_name,\n" .
					"folder_id,\n" .
					"folder_name\n" .
				")\n" .
				"VALUES(\n" .
					0 . ",\n" .
					"'" . $Locale->getWord("WORKFLOW_ADMIN_LIST_NONGROUP_TITLE") . "',\n" .
					0 . ",\n" .
					"'" . $Locale->getWord("WORKFLOW_ADMIN_LIST_NONFOLDER_TITLE") . "'\n" .
				")";
		
		$this->DB()->executeQuery($sSql);
	}
	
	private function delete()
	{
		$sSql = "SELECT
					COUNT(*) AS total
				FROM
					wf_folders
				WHERE
					folder_id = 0";
		
		if ($this->DB()->select($sSql, true, "total") > 0) {
			$sSql = "DELETE FROM
						wf_folders
					WHERE
						folder_id = 0";
			$this->DB()->executeQuery($sSql);
			
			$sSql = "DELETE FROM
						wf_folders_groups
					WHERE
						group_id = 0";
			$this->DB()->executeQuery($sSql);
		}
	}
	
	public function view()
	{
		$this->Recordset 	= new MSF_Recordset(MSF_Registry::get("MSF_DB"));
		$this->Locale 		= MSF_Registry::get("MSF_Locale");
		$aActions 			= array();
		
		$aTypes 	= array("int", "string", "string", "string", "string", "date", "timestamp");
		$aTitles 	= array(
			"id",
			$this->Locale->getWord("WORKFLOW_ADMIN_LIST_TITLE"),
			$this->Locale->getWord("WORKFLOW_ADMIN_LIST_GROUP"),
			$this->Locale->getWord("WORKFLOW_ADMIN_LIST_FOLDER"),
			$this->Locale->getWord("WORKFLOW_ADMIN_LIST_DESCRIPTION"),
			$this->Locale->getWord("WORKFLOW_ADMIN_LIST_CREATED"),
			$this->Locale->getWord("WORKFLOW_ADMIN_LIST_CHANGED")
		);
		$aColumns 	= array(
			"wfd.doctype_id",
			"wfd.title",
			"wfv.group_name",
			"wfv.folder_name",
			"wfd.description",
			"wfd.created_at",
			"wfd.modified_at"
		);
		$aGroups 	= array("wfv.group_name", "wfv.folder_name");
		$aRSColumns = array();
		
		for ($i = 0; $i < sizeof($aColumns); $i++) {
			$aRSColumns[$i]["title"] 	= $aTitles[$i];
			$aRSColumns[$i]["type"] 	= $aTypes[$i];
			$aRSColumns[$i]["name"] 	= $aColumns[$i];
			$aRSColumns[$i]["sort"] 	= true;
			$aRSColumns[$i]["group"] 	= false;
			$aRSColumns[$i]["filter"] 	= true;
			$aRSColumns[$i]["compare"] 	= "like";
			$aRSColumns[$i]["visible"] 	= true;
			
			if(in_array($aColumns[$i], $aGroups)) {
				$aRSColumns[$i]["group"] = true;
			}
			if ($aColumns[$i] == "wfd.doctype_id") {
				$aRSColumns[$i]["visible"] = false;
			}
			if ($aColumns[$i] == "wff.name") {
				$aRSColumns[$i]["group"] = true;
			}
		}
		
		$aActions[] = array(
			"title" 	=> $this->Locale->getWord("WORKFLOW_ADMIN_LIST_ACTION_OPEN"),
			"href" 		=> "/admin/workflow/doctype/edit",
			"image" 	=> "/templates/" . CNF_SKIN . "/images/folder_page_white.png",
			"bind" 		=> array("doctype_id"),
			"confirm" 	=> false
		);
		/*
		$aActions[] = array(
			"title" 	=> $this->Locale->getWord("WORKFLOW_ADMIN_LIST_ACTION_ACCESS"),
			"href" 		=> "/admin/workflow/doctype/access",
			"image" 	=> "/templates/" . CNF_SKIN . "/images/key.png",
			"bind" 		=> array("doctype_id"),
			"confirm" 	=> false
		);
		*/
		$aActions[] = array(
			"title" 	=> $this->Locale->getWord("WORKFLOW_ADMIN_MENU_DOCTYPE_COMPARE"),
			"href" 		=> "/admin/workflow/doctype/compare",
			"image" 	=> "/templates/" . CNF_SKIN . "/images/3_docs.png",
			"bind" 		=> array("doctype_id"),
			"confirm" 	=> false
		);
		$aActions[] = array(
			"title" 	=> $this->Locale->getWord("WORKFLOW_ADMIN_LIST_ACTION_VERSIONS"),
			"href" 		=> "/admin/workflow/doctype/versions",
			"image" 	=> "/templates/" . CNF_SKIN . "/images/3_docs.png",
			"bind" 		=> array("doctype_id"),
			"confirm" 	=> false
		);
		$aActions[] = array(
			"title" 	=> $this->Locale->getWord("WORKFLOW_ADMIN_MENU_DOCTYPE_EXPORT"),
			"href" 		=> "/admin/workflow/doctype/export",
			"image" 	=> "/templates/" . CNF_SKIN . "/images/3_docs.png",
			"bind" 		=> array("doctype_id"),
			"confirm" 	=> false
		);
		$aActions[] = array(
			"title" 	=> $this->Locale->getWord("WORKFLOW_ADMIN_LIST_ACTION_DELETE"),
			"href" 		=> "/admin/workflow/doctype/remove",
			"image" 	=> "/templates/" . CNF_SKIN . "/images/red_cross.png",
			"bind" 		=> array("doctype_id"),
			"confirm" 	=> true
		);
		
		//$this->Recordset->setMultiActionField("doctype_id");
		$this->Recordset->setActions($aActions);
		$this->Recordset->setTables(array(
			"wf_folders_doctypes_view AS wfv",
			"wf_doctypes AS wfd"
		));
		$this->Recordset->setConditions(array(
			"AND wfv.folder_id = wfd.folder_id"
		));
		$this->Recordset->setDefaultGroups($aGroups);
		$this->Recordset->setBaseURL("/admin/workflow/?");
		$this->Recordset->setRpcUrl("/admin/workflow/mainajax/?");
		$this->Recordset->setColumns($aRSColumns);
		$this->Recordset->setSecondaryOrder("wfd.doctype_id");
		$this->Recordset->setLink(
			array(
				"href"		=> "/admin/workflow/doctype/edit",
				"bind"		=> array("doctype_id")
			)
		);
		
		$this->Recordset->setStateParams(
			array(
				"module"		=> "admin_workflow_main",
				"action"		=> "index",
				"action_id"		=> "0",
				"structure_id"	=> MSF_Registry::get("MSF_UserBox")->getUserAttribute("id")
			)
		);
		
		/*
		$aMultiActions[] = array(
 			"title"		=> MSF_Registry::get("MSF_Locale")->getWord("LMS_BTN_DELETE"),
			"href"		=> "/lms/mail/remove/" . $iFolderId . ((isset($_GET["rs_limitFrom"])) ? "&rs_limitFrom=" . (int) $_GET["rs_limitFrom"] : ""),
			"image"		=> "/templates/" . CNF_SKIN . "/img/remove.gif",
			"confirm"	=> true
 		);
		*/
 		
 		
	}
	
	private function actionbar()
	{
		/*
		MSF_Registry::get("MSF_ActionBar")->addDropDown(
			$this->Locale->getWord("WORKFLOW_ACTIONBAR_ADD"),
			array(
				array(
					"url" 	=> "/admin/workflow/folders_groups/add",
					"title" => $this->Locale->getWord("WORKFLOW_ACTIONBAR_ADD_GROUP")
				),
				array(
					"url" 	=> "/admin/workflow/folders/add",
					"title" => $this->Locale->getWord("WORKFLOW_ACTIONBAR_ADD_FOLDER")
				),
				array(
					"url" 	=> "/admin/workflow/doctype/create",
					"title" => $this->Locale->getWord("WORKFLOW_ACTIONBAR_ADD_DOCTYPE")
				)
			),
			"return false"
		);
		*/
		
		MSF_Registry::get("MSF_ActionBar")->addAction(
			$this->Locale->getWord("WORKFLOW_ACTIONBAR_ADD") . " " .
			$this->Locale->getWord("WORKFLOW_ACTIONBAR_ADD_DOCTYPE"),
			"/admin/workflow/doctype/create"
		);
		$sTitle	= $this->Locale->getWord("WORKFLOW_ADMIN_MENU_DOCTYPE_IMPORT");
		$sAction		= "WF_DoctypeImport(this, '$sTitle')";
		MSF_Registry::get("MSF_ActionBar")->addAction(
			$sTitle,
			"#",
			CNX_ACTIONBAR_IMG,
			$sAction,
			false,
			"left");
	}
	
	public function getView()
	{
		$this->delete();
		$this->createView();
		$this->view();
		$this->actionbar();
		$View = new WF_WorkflowListXml();
		$View->set($this->Recordset->getAllRecords());
		$this->delete();
		return $View;
	}
	
	public function getAjaxView()
	{
		$this->delete();
		$this->createView();
		$this->view();
		//$View = new MSF_ViewSupportJson();
		//$View->set("group", $this->Recordset->getGroup());
		$View = new WF_WorkflowListXml();
		$View->buildGroup($this->Recordset->getGroup());
		$this->delete();
		return $View;
	}
}

?>