<?php
defined("__MSF") or die("Direct access is not allowed");

include_once "modules/workflow/view/xml/WF_RouteXml.php";

class WFC_Route extends WFC_Abstract
{
	/**
	 * Returns instance of WF_RouteXml
	 * @return WF_RouteXml
	 */
	public function view($iDoctypeId)
	{
		if ($this->isOpera()) {
			MSF_Registry::get("MSF_Messenger")
			->addError("Внимание! Использование данной версии браузера Opera может привести к потере данных");
		}
		
		$oDoctype 			= new WF_Doctype($iDoctypeId);
		$oDoctypeVersion 	= $oDoctype->getCurrentVersion();
		$aSpa				= MSF_XmlWrapper::xmlToArray($oDoctypeVersion->getSpaXml());
		$oView = new WF_RouteXml();
		$oView->initLinks();
		try {
			$oView->initRoute($oDoctypeVersion->getRouteXml());
		}
		catch (Exception $oException) {
			$this->Controller->addError($this->Controller->locale("WORKFLOW_TEMPLATE_NOT_SAVED"));
			$oView->initRoute("<document/>");
		}
	
		$oView->injectAllDoctypes();
		$oView->initSpaActions($aSpa);
		$oView->initRouteActions();
		$oView->initAddress($oDoctypeVersion);
		$oView->initFields($oDoctypeVersion);
		$oView->remindSettings();
		$oView->formSystemFields($oDoctypeVersion->getSystemFieldsXml());		
		$oView->initTemplate();			

		return $oView;
	}
	
	private function isOpera()
	{
		if (strstr($_SERVER["HTTP_USER_AGENT"], 'Version/10.52')) {
			return true;
		}
		else if (strstr($_SERVER["HTTP_USER_AGENT"], 'Version/10.53')) {
			return true;
		}
		else if (strstr($_SERVER["HTTP_USER_AGENT"], 'Version/10.54')) {
			return true;
		}
		else if (strstr($_SERVER["HTTP_USER_AGENT"], 'Version/10.6')) {
			return true;
		}
		else {
			return false;
		}
		
	}
	

	
	/**
	 * Returns doctype id
	 * @return integer
	 */
	public function edit($iId)
	{
		$Doctype 		= new WF_Doctype($iId);
		$DoctypeVersion = $Doctype->getCurrentVersion();
		if (IS_REQUEST) {
			if ($this->isOpera()) {
				MSF_Registry::get("MSF_Messenger")
				->addError("Внимание! Данные не сохранены");
				$this->Controller->redirect("/admin/workflow/doctype/route/" . $iId, "");
				exit();
			}
			$bNewVersion 			= false;
			$aParams 				= array();
			$aParams["fields"] 		= $DoctypeVersion->getFieldsXml();;
			$aParams["spa"] 		= $DoctypeVersion->getSpaXml();
			$aParams["template"] 	= $DoctypeVersion->getTemplateXml();
			$aParams["archive"]		= $DoctypeVersion->getArchiveXml();
			$aParams["route"] 		= $this->prepareRouteXml($_POST);
                        ;
                        MSF_Logger::logMessage(var_export($this->prepareRouteXml($_POST)));
			if (MSF_SL::getParam($_POST, "save_in_version") == 1) {
				$bNewVersion = false;
			}
			else {
				$bDocs = true;
				if($DoctypeVersion->exists() && ($DoctypeVersion->getCountOfDocuments() == 0)) {
	       			$DoctypeVersion->remove(true);
	       			$bDocs = false;
	       		}

				if(isset($_POST["route"])) {
					$bNewVersion = $this->didRouteChange($DoctypeVersion->getRouteXml(), $aParams["route"]);					
				}
				else {
					if($bDocs) {
						$bNewVersion = true;
					}
					
				}
				
			}			
			
			WF_Utils::saveAsNewVersion($Doctype, $aParams, $bNewVersion);
			$this->Controller->redirect("/admin/workflow/doctype/route/" . $iId, $this->Controller->locale("WORKFLOW_ROUTE_SAVED"));
		}
		
		$this->addDoctypeActions($iId);
		return $this->view($iId);
	}
	
	private function prepareRouteXml($aRoute)
	{
		$aPost = array();

		if (isset($aRoute["route"])) {
			foreach ($aRoute["route"] as $k => $v) {
				$aResult 		= array();
				$aResult[$k] 	= $aRoute["route"][$k];
				if (is_string($aResult[$k])) {
					continue;
				}
				
				$aPost[$k]["title"] 	= $v["title"];
				$aPost[$k]["delegate"] 	= array();
				$aPost[$k]["execute"] 	= array();
				$aPost[$k]["wait"] 		= array();
				
				if (isset($aResult[$k]["delegate"])) {
					$iCount = 0;
					foreach ($aResult[$k]["delegate"] as $aValue) {
						if (!empty($aValue["spa_id"])) {
							if (!isset($aValue["value"])) {
								$aValue["value"] = "";
							}
							$aPost[$k]["delegate"][$iCount++] = $aValue;
						}
					}
				}
				if (isset($aResult[$k]["execute"])) {
					$iCount = 0;
					
					foreach ($aResult[$k]["execute"] as $aValue) {
						if (!empty($aValue["name"])) {
							if (!isset($aValue["value"])) {
								$aValue["value"] = "";
							}
							
							$aPost[$k]["execute"][$iCount++] = $aValue;
						}
					}
				}
				if (isset($v["wait"])) {
					$aPost[$k]["wait"] = $v["wait"];
				}
			}
		}
		
		$i 			= 0;
		$Wrapper 	= new MSF_XmlWrapper();
		
		foreach ($aPost AS $k => $aElement) {
			$sTitle = $aElement["title"];
			unset($aElement["title"]);
			
			$Wrapper->set($i++, $aElement, $sTitle);
		}
		
		
		return $Wrapper->getXml();
	}
	
	private function didRouteChange($sOldRoute, $sNewRoute)
	{
		$isChanged = false;		
		$sOldRoute = str_replace('=""', '="0"', $sOldRoute);
		$sNewRoute = str_replace('=""', '="0"', $sNewRoute);		
		$oOldRoute = new MSF_XmlWrapper($sOldRoute);
		$oNewRoute = new MSF_XmlWrapper($sNewRoute);		
		if($oOldRoute->getLength() != $oNewRoute->getLength()) {
			$isChanged = true;
		}
		
		if(!$isChanged) {
			$aOldRoute = $oOldRoute->getAsArray();
			$aNewRoute = $oNewRoute->getAsArray();
			if($aOldRoute != $aNewRoute) {
				$isChanged = true;
			}
			
		}
		
		return $isChanged;		
	}
}

?>