<?php
/*
 * File: 	WFC_Template.php
 * Date: 	15.02.2008
 * Time:	16:09:15
 * Detrix
 *
 */

// [include]
include_once "modules/workflow/library/doctype/WF_Doctype.php";
include_once "modules/workflow/view/xml/WF_TemplateXml.php";
include_once "modules/workflow/library/variables/WF_Vars.php";
include_once "library/MSF_Utils/MSF_RteObjectsIntegrator.php";
// [/include]

/**
 * @package 	workflow
 * @copyright 	Copyright (c) 2007, Detrix
 */
class WFC_Template extends WFC_Abstract
{
	// @abstract
	// protected $View = null;

	private $DoctypeVersion = null;

	private $iDoctypeId		= 0;

	public function __construct(MSF_BackController $Controller, $iDoctypeId)
	{
		parent::__construct($Controller);
		$this->Doctype 			= new WF_Doctype($iDoctypeId);
		$this->DoctypeVersion 	= $this->Doctype->getCurrentVersion();
		$this->View 			= new WF_TemplateXml();
		$this->iDoctypeId		= $iDoctypeId;
	}

	public function edit()
	{
		$sTemplate 	= MSF_SL::getParam($_POST, "template", "<br/>", true);
		$sTemplate 	= $this->escapeTemplate($sTemplate);
		
		$aSearch	= array("<!--[", "]-->");
		$aReplace	= array("<!-- [", "] -->");
		
		$sTemplate = str_replace($aSearch, $aReplace, $sTemplate);
		
		try {
			if ($this->DoctypeVersion->getCountOfDocuments() > 0) {
				$bNewVersion = false;
				if (MSF_SL::getParam($_POST, "save_in_version") == 1) {
					$bNewVersion = false;
				}
				else {
					$bNewVersion = ($this->DoctypeVersion->getTemplateXml() != $sTemplate) ? true : false;
				}			
				WF_Utils::saveAsNewVersion($this->Doctype, array("template" => $sTemplate), $bNewVersion);
				$this->Controller->redirect("/admin/workflow/doctype/template/" . $this->iDoctypeId,
											$this->Controller->locale("WORKFLOW_TEMPLATE_SAVED")
											);
			}
			else {
				if ($this->DoctypeVersion->exists()) {
					$Date = new MSF_Date();
					$this->DoctypeVersion->setTemplateXml($sTemplate);
					$this->DoctypeVersion->save();
					$this->Doctype->setModifiedAt($Date->formatTimestamp());
					WF_Utils::saveAsNewVersion($this->Doctype, array(), false);
					$this->Doctype->save();
					$this->Controller->redirect("/admin/workflow/doctype/template/" . $this->iDoctypeId,
												$this->Controller->locale("WORKFLOW_TEMPLATE_SAVED")
												);
				}
				else {
					$this->Controller->addError($this->Controller->locale("WORKFLOW_DOCTYPE_VERSION_NOT_EXISTS"));
				}
			}
		}
		catch (WF_Exception $E) {
			$this->Controller->addError($this->Controller->locale("WORKFLOW_TEMPLATE_NOT_SAVED"));
		}

	}
	
	private function escapeTemplate($sTemplate)
	{
		$sTemplate	= str_replace("\r", "", $sTemplate);
		$sTemplate	= str_replace("\n", " ", $sTemplate);
		$sPattern	= '/\<(.+)\>/i';
		$sResult	= preg_replace_callback(
				$sPattern,
       			create_function(
        			   	'$matches',
       					'$sReplaced = str_replace("\\\", "/", $matches[0]);' .
       					'$sReplaced = str_replace("%5C", "/", $sReplaced);' .
       					'return $sReplaced;'
       					),
       			$sTemplate
   				);
   		//$sResult	= str_replace(" ", "\n", $sResult);
		return $sTemplate;
	}
	
	private function prepareTemplateXml($sTemplateXml)
	{
		//$sTemplateXml = str_Replace("&nbsp;", "&amp;nbsp;", $sTemplateXml);
		return $sTemplateXml;
	}
	
	public function view()
	{
		$this->addDoctypeActions($this->iDoctypeId);
		if ($this->DoctypeVersion->exists()) {
			$sTemplateXml = $this->prepareTemplateXml($this->DoctypeVersion->getTemplateXml());
			$this->View->view(
				$sTemplateXml,
				$this->getFields(),
				$this->getVars()
			);
		}
		else {
			$this->Controller->addError($this->Controller->locale("WORKFLOW_DOCTYPE_VERSION_NOT_EXISTS"));
		}

		MSF_RteObjectsIntegrator::files($this->iDoctypeId, "workflow_doctype");
		//MD_Utils::appendFilesToModule("workflowdoctype", $this->iDoctypeId);
		$this->Controller->setView($this->View);
	}
	
	private function getFields()
	{
		$aFields = array();
		foreach (new MSF_XmlWrapper($this->DoctypeVersion->getFieldsXml()) AS $Data) {
			$aFields[] = array(
					"title" 	=> $Data->getTitle(),
					"tag" 		=> "{" . $Data->getTitle() . "}"
					);
		}
		
		return MSF_Json::encode($aFields);
	}
	
	private function getVars()
	{
		$WFVars = new WF_Vars();
		//debug($WFVars->getList());
		return MSF_Json::encode($WFVars->getList());
	}

}
?>