<?php

class WF_DocumentWSDLGenerator
{
    protected $oDb;
    protected $aDocsMap;
    
    const SOAP_CREATE_PROC = 'Создать_%s_%d';
    const SOAP_UPDATE_PROC = 'Записать_%s_%d';
    const SOAP_MOVE_PROC   = 'Переместить_%s_%d';
    
    public function __construct()
    {
        $this->oDb = MSF_Registry::get('MSF_DB');
    }
    
    public function handle()
    {
        $this->oDb->executeQuery('SELECT doctype_id as doc_id, title, fields_xml FROM wf_doctypes ORDER BY title');
        //header('content-type: text/plain');
        while($go = $this->oDb->fetchArray())
        {
            $this->aDocsMap[$go['doc_id']]['title'] = $go['title'];
            $this->aDocsMap[$go['doc_id']]['proc_name'] = $this->prepare_name($go['title']);
            /*foreach($this->aSoapMethods as $m) {
                printf($m, $this->aDocsMap[$go['doc_id']]['proc_name'], $go['doc_id']); echo ' <br> ';
            }*/
            $xml = new SimpleXMLElement($go['fields_xml']);
            $this->aDocsMap[$go['doc_id']]['fields'] = array();
            //var_dump($xml);
            
            $iterator = 0;
            foreach($xml->element as $el)
            {
                $attrs = $el->attributes();
                $field['title'] = $this->prepare_name($attrs['title']);
                $this->aDocsMap[$go['doc_id']]['fields'][] = $field;
            }
        }
        
    }
    
    public function generateWSDL()
    {
        $result =  '<?xml version="1.0" encoding="UTF-8"?>';
        $result .= '<definitions name="Detrix"';
        $result .= ' targetNamespace="urn:Detrix"';
        $result .= ' xmlns:tns="http://'.$_SERVER['HTTP_HOST'].'/webservice/workflow/wsdl"';
        $result .= ' xmlns:soap="http://schemas.xmlsoap.org/wsdl/soap/"';
        $result .= ' xmlns="http://schemas.xmlsoap.org/wsdl/">'."\n";
        
    //-- message
        foreach($this->aDocsMap as $doc_id => $aDocParams)
        {
            
        //-- create
            $result .= '<message name="'.sprintf(self::SOAP_CREATE_PROC, $aDocParams['proc_name'], $doc_id).'Request">';
            foreach($aDocParams['fields'] as $field)
            {
                $result .= '<part name="'.$field['title'].'" type="xsd:string"/>';
            }
            
            $result .= '</message>';
            $result .= '<message name="'.sprintf(self::SOAP_CREATE_PROC, $aDocParams['proc_name'], $doc_id).'Response">';
            $result .= '<part name="result" type="xsd:string"/>';
            $result .= '</message>';
            
        //-- update
            $result .= '<message name="'.sprintf(self::SOAP_UPDATE_PROC, $aDocParams['proc_name'], $doc_id).'Request">';
            $result .= '<part name="ИдентификаторДокумента" type="xsd:string"/>';
            foreach($aDocParams['fields'] as $field)
            {
                $result .= '<part name="'.$field['title'].'" type="xsd:string"/>';
            }
            
            $result .= '</message>';
            $result .= '<message name="'.sprintf(self::SOAP_UPDATE_PROC, $aDocParams['proc_name'], $doc_id).'Response">';
            $result .= '<part name="result" type="xsd:string"/>';
            $result .= '</message>';
        //-- move
            $result .= '<message name="'.sprintf(self::SOAP_MOVE_PROC, $aDocParams['proc_name'], $doc_id).'Request">';
            $result .= '<part name="ИдентификаторДокумента" type="xsd:string"/>';
            $result .= '<part name="ТочкаДокумента" type="xsd:string"/>';
            $result .= '</message>';
            $result .= '<message name="'.sprintf(self::SOAP_MOVE_PROC, $aDocParams['proc_name'], $doc_id).'Response">';
            $result .= '<part name="result" type="xsd:string"/>';
            $result .= '</message>';
        }
        
    //-- portType
        $result .= "\n".'<portType name="DetrixPortType">';
        
        foreach($this->aDocsMap as $doc_id => $aDocParams)
        {   
            // create
            $result .= '<operation name="'.sprintf(self::SOAP_CREATE_PROC, $aDocParams['proc_name'], $doc_id).'">';
            $result .= '<input message="tns:'.sprintf(self::SOAP_CREATE_PROC, $aDocParams['proc_name'], $doc_id).'Request"/>';
            $result .= '<output message="tns:'.sprintf(self::SOAP_CREATE_PROC, $aDocParams['proc_name'], $doc_id).'Response"/>';
            $result .= '</operation>';
            
            // update
            $result .= '<operation name="'.sprintf(self::SOAP_UPDATE_PROC, $aDocParams['proc_name'], $doc_id).'">';
            $result .= '<input message="tns:'.sprintf(self::SOAP_UPDATE_PROC, $aDocParams['proc_name'], $doc_id).'Request"/>';
            $result .= '<output message="tns:'.sprintf(self::SOAP_UPDATE_PROC, $aDocParams['proc_name'], $doc_id).'Response"/>';
            $result .= '</operation>';
            
            // move
            $result .= '<operation name="'.sprintf(self::SOAP_MOVE_PROC, $aDocParams['proc_name'], $doc_id).'">';
            $result .= '<input message="tns:'.sprintf(self::SOAP_MOVE_PROC, $aDocParams['proc_name'], $doc_id).'Request"/>';
            $result .= '<output message="tns:'.sprintf(self::SOAP_MOVE_PROC, $aDocParams['proc_name'], $doc_id).'Response"/>';
            $result .= '</operation>';
        }
        $result .= '</portType>'."\n";
        
        
    //-- binding
        $result .= '<binding name="DetrixBinding" type="tns:DetrixPortType">';
        $result .= '<soap:binding style="rpc" transport="http://schemas.xmlsoap.org/soap/http"/>';
        
        foreach($this->aDocsMap as $doc_id => $aDocParams)
        {
            $create = sprintf(self::SOAP_CREATE_PROC, $aDocParams['proc_name'], $doc_id);
            $update = sprintf(self::SOAP_UPDATE_PROC, $aDocParams['proc_name'], $doc_id);
            $move = sprintf(self::SOAP_MOVE_PROC, $aDocParams['proc_name'], $doc_id);
            
            // create
            $result .= '<operation name="'.$create.'">';
            $result .= '<soap:operation soapAction="http://'.$_SERVER['HTTP_HOST'].'/webservice/workflow/server#'.$create.'"/>';
            $result .= '<input>';
            $result .= '<soap:body use="encoded" namespace="urn:Detrix" encodingStyle="http://schemas.xmlsoap.org/soap/encoding/"/>';
            $result .= '</input>';
            $result .= '<output>';
            $result .= '<soap:body use="encoded" namespace="urn:Detrix" encodingStyle="http://schemas.xmlsoap.org/soap/encoding/"/>';
            $result .= '</output>';
            $result .= '</operation>';
            
            // update
            $result .= '<operation name="'.$update.'">';
            $result .= '<soap:operation soapAction="http://'.$_SERVER['HTTP_HOST'].'/webservice/workflow/server#'.$update.'"/>';
            $result .= '<input>';
            $result .= '<soap:body use="encoded" namespace="urn:Detrix" encodingStyle="http://schemas.xmlsoap.org/soap/encoding/"/>';
            $result .= '</input>';
            $result .= '<output>';
            $result .= '<soap:body use="encoded" namespace="urn:Detrix" encodingStyle="http://schemas.xmlsoap.org/soap/encoding/"/>';
            $result .= '</output>';
            $result .= '</operation>';
            
            // move
            $result .= '<operation name="'.$move.'">';
            $result .= '<soap:operation soapAction="http://'.$_SERVER['HTTP_HOST'].'/webservice/workflow/server#'.$move.'"/>';
            $result .= '<input>';
            $result .= '<soap:body use="encoded" namespace="urn:Detrix" encodingStyle="http://schemas.xmlsoap.org/soap/encoding/"/>';
            $result .= '</input>';
            $result .= '<output>';
            $result .= '<soap:body use="encoded" namespace="urn:Detrix" encodingStyle="http://schemas.xmlsoap.org/soap/encoding/"/>';
            $result .= '</output>';
            $result .= '</operation>';
            
        }
        
        $result .= '</binding>'."\n";
        
    //-- service
        
        $result .= '<service name="DetrixService">';
        $result .= '<port name="DetrixPort" binding="DetrixBinding">';
        $result .= '<soap:address location="http://'.$_SERVER['HTTP_HOST'].'/webservice/workflow/server"/>';
        $result .= '</port>';
        $result .= '</service>'."\n";
        
        $result .= '</definitions>';
        
        return $result;
    }
    
    protected function prepare_name($source)
    {
        $literal = preg_replace('/[^a-z,а-я,ё,A-Z,А-Я,Ё]/uis', ' ', $source);
        $items = explode(' ', $literal);
        
        $result = '';
        foreach($items as $item)
        {
            $token = trim($item);
            if($token != '')
            {
                $token = mb_strtolower($token);
                $result .= mb_strtoupper(mb_substr($token, 0, 1)).mb_substr($token, 1);
            }
        }
        
        return $result;
    }
}


?>