<?php
/*
 * File: 	WF_ActionBox.php
 * Date: 	12.02.2008
 * Time:	15:42:35
 * Detrix
 *
 */

include_once "modules/workflow/library/actions/WF_ActionSupport.php";
include_once "modules/workflow/library/actions/WF_ActionNotFoundException.php";
include_once "modules/workflow/library/actions/WF_ActionNotLoadedException.php";
include_once "modules/workflow/library/actions/WF_ActionRuntimeException.php";
include_once "modules/workflow/library/actions/WF_NullAction.php";

/**
 * @package 	workflow
 * @copyright 	Copyright (c) 2007, Detrix
 */
class WF_ActionBox
{
	/**
	 * @var string
	 */
	private $sActionPath		= "";
	
	/**
	 * @var WF_IAction
	 */
	private $Action 			= null;

	/**
	 * @var int
	 */
	private $iDocumentId		= 0;
	
	/**
	 * @var int
	 */
	private $iDocumentVersionId	= 0;

	/**
	 * @var array
	 */
	private $aActions 			= array();

	/**
	 * @var bool
	 */
	private $bIsActionLoaded	= false;

	public function getActionsList()
	{
		$aList = array();
		for ($i = 0; $i < count($this->aActions); $i++) {
			$aList[$i] = $this->aActions[$i]["name"];
		}

		return $aList;
	}
	
	public function getAction()
	{
		return $this->Action;
	}

	/**
	 * Vozvrashaet config deistviya
	 *
	 * @param string $sActionName
	 * @return array
	 * @access public
	 */
	public function getActionCfg($sActionName)
	{
		$aActionCfg = null;
		for ($i = 0; $i < count($this->aActions); $i++) {
			if ($sActionName == $this->aActions[$i]["name"]) {
				$aActionCfg = (array) $this->aActions[$i];
				break;
			}

		}

		if ($aActionCfg == null) {
			throw new WF_ActionNotFoundException($sActionName);
		}

		return $aActionCfg;
	}

	/**
	 * Vozvrashaet massiv deystiy.
	 *
	 * @return array
	 * @access public
	 */
	public function getCfg()
	{
		return $this->aActions;
	}

	/**
	 * Proveryaet, zagruzheno li deistvie
	 *
	 * @return bool
	 * @access public
	 */
	public function isActionLoaded()
	{
		return $this->bIsActionLoaded;
	}

	/**
	 * Zozdaet i vozvrashet eczemplyaer classa
	 *
	 * @return WF_ActionBox
	 * @access public
	 */
	public static function newInstance()
	{
		$aWorkflowCfg = MSF_Registry::get("MSF_Settings")->getXmlWrapper()->get("modules")->getValue("workflow");
		//debug($aWorkflowCfg);
		return new self($aWorkflowCfg["actions"]);
	}

	/**
	 * @param array $aActionsCfg
	 * @access public
	 */
	public function __construct($aActionsCfg)
	{
		$this->unloadAction();
		$this->sActionPath 	= CNF_ABS_PATH . "/modules/workflow/actions";
		$this->aActions 	= $aActionsCfg;
		$Locale				= MSF_Registry::get("MSF_Locale");
		for ($i = 0; $i < count($this->aActions); $i++) {
			$sTitle = "WFA_" . strtoupper($this->aActions[$i]["name"]);
			$this->aActions[$i]["title"] = $Locale->getWord($sTitle);
			if (isset($this->aActions[$i]["spaJsClassname"])) {
				$this->aActions[$i]["spaJsClasspath"] = "/modules/workflow/actions/" . $this->aActions[$i]["name"] . "/" . $this->aActions[$i]["spaJsClassname"] . ".js";
			}
			
			if (!isset($this->aActions[$i]["hasRoute"])) {
				$this->aActions[$i]["hasRoute"] = false;
			}

			if ($this->aActions[$i]["hasRoute"]) {
				$this->aActions[$i]["routePath"] = "/modules/workflow/actions/" . $this->aActions[$i]["name"] . "/" . CNF_SKIN . "/back_route.xsl";
			}

		}
		
	}

	/**
	 * Provaeryaet est li takoe deistvie
	 *
	 * @param string $sActionName
	 * @return bool
	 * @access public
	 */
	public function isActionExists($sActionName)
	{
		$bIsActionExists = false;
		for ($i = 0; $i < count($this->aActions); $i++) {
			if ($sActionName == $this->aActions[$i]["name"]) {
				$bIsActionExists = true;
				break;
			}

		}

		return $bIsActionExists;
	}

	/**
	 * Zagruzhaet deistive
	 *
	 * @param string $sActionName
	 * @param int $iActionId
	 * @param int $iDocumentId
	 * @param int $iDocumentVersionId
	 * @param string $sInitializing
	 * @param WF_IAction $CallerAction
	 * @return void
	 * @access public
	 */
	public function loadAction($sActionName, $iActionId, $iDocumentId, $iDocumentVersionId, $sInitializing, $CallerAction = null)
	// throws WF_ActionNotFoundException
	{
		$aActionCfg				= $this->getActionCfg($sActionName); // throws WF_ActionNotFoundException
		include_once $this->sActionPath . "/" . $aActionCfg["name"] . "/" . $aActionCfg["classname"] . ".php"; // throws MSF_ThrowableException
		$Class 					= new ReflectionClass($aActionCfg["classname"]);
		$this->Action 			= $Class->newInstance($sActionName, $iActionId, $iDocumentId, $iDocumentVersionId, $sInitializing, $CallerAction);
		$this->bIsActionLoaded	= true;
		
		if ($this->Action instanceof WF_IAction) {
			if (isset($aActionCfg["settings"])) {
				foreach ($aActionCfg["settings"] AS $sName => $sValue) {
					$this->Action->setInitParam($sName, $sValue);
				}

			}

			$this->Action->setInitParam("spa_name", 			$sActionName);
			$this->Action->setInitParam("spa_id", 				$iActionId);
			$this->Action->setInitParam("subject_id", 			$iDocumentId);
			$this->Action->setInitParam("subject_version_id", 	$iDocumentVersionId);
			$this->Action->setInitParam("initializing", 		$sInitializing);
		}
		else {
			$this->unloadAction();
			throw new MSF_NotValidClassException("Action " . $aActionCfg["classname"] . " is not instance of WF_IAction");
		}
	}

	/**
	 * Vigruzhaet deistvie
	 *
	 * @return void
	 * @access public
	 */
	public function unloadAction()
	{
		$this->Action 			= new WF_NullAction(0, 0, 0);
		$this->bIsActionLoaded 	= false;
	}


	/**
	 * Zapuskaet deistvie
	 *
	 * @param string $sType
	 * @return int
	 * @access public
	 */
	public function process($sType)
	{		
		$this->Action->initialize();

		switch ($sType) {
			case "object":
				$mResult = $this->Action->processObject();
				break;
			case "schedule":
				$mResult = $this->Action->processSchedule();
				break;
			case "call":
				$mResult = $this->Action->processCall();
				break;
			case "execute":
				$mResult = $this->Action->processExecute();
				break;
			case "request":
				$mResult = $this->Action->processRequest();
				break;
			case "response":
				$mResult = $this->Action->processResponse();
				break;
			case "cancel":
				$this->Action->cancel();
				$mResult = WF_IAction::SUCCESS;
				break;
			default:
				$mResult = WF_IAction::FAILURE;
				break;
		}

		if ($mResult === null) {
			throw new WF_ActionRuntimeException("Called action returned <null> value. [Action " . $this->Action->getActionName() . "][Mode " . $sType . "]");			
		}
		
		return $mResult;


	}
	/**
	 * Vizivaet deistvie
	 *
	 * @param string $sAction
	 * @param array $aParams
	 * @return void
	 * @access public
	 */
	public function call($sAction, $aParams)
	{
		$this->Action->call($sAction, $aParams);
	}
		
}
?>