<?php
include_once "WF_AbstractAction.php";
include_once "WF_Settings.php";
include_once "WF_ActionBuilderDb.php";
include_once "modules/workflow/library/doctype/WF_Doctype.php";
include_once "modules/workflow/library/doctypeVersion/WF_DoctypeVersion.php";
include_once "modules/workflow/library/document/WF_Document.php";
include_once "modules/workflow/library/documentVersion/WF_DocumentVersion.php";
include_once "modules/workflow/library/security/WF_DoctypeSwitcher.php";
include_once "modules/workflow/library/security/WF_DocumentSwitcher.php";
include_once "modules/workflow/library/doctypeVersion/WF_DelegateValues.php";

abstract class WF_ActionBuilder extends WF_AbstractAction
{
	
	private $aEmployeePath		= array();
	
	private $isAllowed			= true;
	
	private $isInitialized 		= false;

	private $isMain				= false;
	
	private $isVersion			= false;
	
	private $iActionId			= 0;
	
	private $iAuthorId			= 0;
	
	private $iStructureId		= 0;
	
	private $iEmployeeId		= 0;
	
	private $iPostId			= 0;
	
	private $sPost				= '';
	
	private $Db					= null;
	
	private $oDelegator			= null;
	
	private $Doctype			= null;
	
	private $DoctypeVersion		= null;
	
	private $Document			= null;
	
	private $DocumentVersion 	= null;
	
	private $Settings			= null;
	
	private $Values				= null;
		
	private $sActionName		= "";
	
	protected  $sEmployeeName	= "";
	
	private $sRedirectUrl		= "";
	
	private $isRedirectable		= true;
	
	final public function getEmployeePath()
	{
		return $this->aEmployeePath;
	}
	
	final public function isRedirectable($isRedirectable = null)
	{
		if ($isRedirectable !== null) {
			$this->isRedirectable = $isRedirectable;
		}

		return $this->isRedirectable;
	}
	
	/**
	 * @return WF_Values
	 */
	final public function getValues()
	{
		return $this->Values;
	}

	final public function getActionName()
	{
		return $this->sActionName;
	}

	/**
	 * @return boolean
	 */
	final public function isVersion()
	{
		return $this->isVersion;
	}
	
	final public function getActionId()
	{
		return $this->iActionId;
	}
	
	public function isExisted()
	{
		return ($this->Document !== null) ? true : false;
	}
	
	final public function isInitialized()
	{
		return $this->isInitialized;
	}

	/**
	 * @return WF_Doctype
	 */
	final public function getDoctype()
	{
		return $this->Doctype;
	}

	/**
	 * @return WF_DoctypeVersion
	 */
	final public function getDoctypeVersion()
	{
		return $this->DoctypeVersion;
	}
	
	/**
	 * @return WF_Document
	 */
	final public function getDocument()
	{
		return $this->Document;
	}
	
	/**
	 * @return WF_DocumentVersion
	 */
	final public function getDocumentVersion()
	{
		return $this->DocumentVersion;
	}
	
	/**
	 * @return WF_Settings
	 */
	final public function getSettings()
	{
		return $this->Settings;
	}
	
	final public function isDelegated($iActionId)
	{
		return $this->oDelegator->isDelegated($iActionId);
	}
	
	final public function __construct($sActionName, $iActionId, $iDocumentId, $iDocumentVersionId, $bInitializing = true, $CallerAction = null)
	{
		try {
			$this->initEmployee();
			$this->init($sActionName, $iActionId, $iDocumentId, $iDocumentVersionId, $bInitializing, $CallerAction);
			$this->setInitParam("initialized", 	$this->isInitialized);
			if (!$this->isInitialized) {
				$this->initSecurity();
			}
	
			if ($this->Document !== null) {
				$this->initDelegator();
			}	
								
		}
		catch (Exception $oException) {
			throw $oException;
		}


		

		//debug($this->getSettings()->getRoute());
		
		
		//debug($this->getSettings()->getDelegatedAtPoint(0));
		//exit;
		
	}
	
	/**
	 * @return WF_Log
	 */
	final public function getLog()
	{
		$oLog = $this->getDocument()->getLog();
		return $oLog;
	}
	
	final public function hasAccess($sActionName = null)
	{
		if ($sActionName == null) {
			$sActionName = $this->sActionName;
		}
		$oDoc = $this->getDocument();
		if (!$oDoc || MSF_OrgStructureFacade::getInstance()->isStructureId($oDoc->getStructureId())) {
			return $this->isAllowed && (in_array($sActionName, array("view", "create", "remove")) || $this->isDelegated($this->getActionId()));
		}
		else {
			$oDa 						= new WF_DoctypeAccess();
			$aCompanyAccess 			= $oDa->get($this->Document->getDoctypeId(), 1);
			$aCurrentStructureIdAccess 	= $oDa->get($this->Document->getDoctypeId(), $this->getStructureId());
			if (isset($aCompanyAccess[0])) {
				return $aCompanyAccess[0]["view"] == 1;
			}
			else if (isset($aCurrentStructureIdAccess[0])){
				return $aCurrentStructureIdAccess[0]["view"] == 1;
			}
			else {
				return false;
			}

		}

	}
	
	final public function getRedirectUrl()
	{
		if (($this->sRedirectUrl == "") && $this->isRedirectable()) {
			if ($this->Document !== null) {
				$this->sRedirectUrl = $this->getRedirectUrlToDocument($this->Document->getDocumentId());
			}
			
		}
		
		return $this->sRedirectUrl;
	}
	
	final public function save()
	{
		if (($this->getDocument() !== null) && ($this->getDocument()->exists())) {
			$this->Document->getCurrentVersion()->save();
			$this->Document->save();
		}

	}

	final public function getRedirectUrlToDocument($iDocumentId)
	{
		return "/workflow/document/view/" . $iDocumentId;
	}

	final public function setRedirectUrl($sRedirectUrl)
	{
		$this->sRedirectUrl = $sRedirectUrl;
	}
	
	public function setDocument($oDocument)
	{
		$this->Document = $oDocument;
	}
	
	final public function isWaiting()
	{
		if ($this->Settings !== null) {
			return ($this->Settings->isWaiting($this->Document->getPointId(), $this->iActionId) 
					&& $this->Settings->isPointExists($this->Document->getPointId() + 1));
		}
		else {
			return false;
		}

	}
	
	final public function addWaited()
	{
		$this->getLog()->addWaited($this->getDocument()->getPointId(), $this->iStructureId);
	}
	
	final public function next()
	{
		$oLog = $this->getLog();
		$oLog->getWaited($this->getDocument()->getPointId());
		if ($this->isFool()) {
			$aWait = $this->getSettings()->getWaitedAtPoint($this->getDocument()->getPointId());
			if (isset($aWait['options']['overwrite']) && !empty($aWait['options']['field'])) {
				$this->updateField($aWait['options']['field']);
			}
			$oDate = new MSF_Date();
			//debug($this->getPointsDate(), $oDate->formatDate());
			//$this->getDocumentVersion()->setSystemField($this->getPointsDate(), $oDate->formatDate());
			if (!$this->Settings->isLast($this->getDocument()->getPointId())) {
				$this->call("move", array("iNewPoint" => 1), "call", true);
				if ($this->sActionName == "create") {
					$oAb 		= WF_ActionBox::newInstance();
					$oAb->loadAction("view", 0, $this->getDocument()->getDocumentId(), null, true);
					$oAction	= $oAb->getAction();
					$oEc		= new WF_ExecutionContext($oAction);
					$oEc->execute();
				}
			}			
		}

	}
	
	private function getPointsDate() {
		return "point_" . $this->getDocument()->getPointId();
	}
	
	private function updateField($sFieldname)
	{
		$oRefFields		= new REF_Fields($this->DoctypeVersion->getFieldsXml());
		$aFields		= $oRefFields->getField($sFieldname);
		$UBox 			= MSF_Registry::get("MSF_UserBox");
		
		$sDisplayName = RefFieldHelper::formatStructureValue(
				$UBox->getEmployeeAttribute("id"), 
				$UBox->getCurrentPostId(), 
				MSF_SL::getParam($aFields["optional"], "employee", "name"), 
				MSF_SL::getParam($aFields["optional"], "post", "post"), 
				MSF_SL::getParam($aFields["optional"], "employeeEnable", 1), 
				MSF_SL::getParam($aFields["optional"], "postEnable", 1)
				);
		
		$oRefValue	= new REF_Value();
		$oRefValue->add(
				array(
						$this->iStructureId,
						$this->iEmployeeId,
						$this->iPostId
						),
				$sDisplayName
				);
		$this->getDocumentVersion()->setFields(
				array(
						$sFieldname => $oRefValue->format()
						)
				);
		$this->getDocumentVersion()->save();
	}
	
	private function isFool()
	{
		$isFool	= true;
		$aWait 	= $this->Settings->getWaitedAtPoint($this->getDocument()->getPointId());
		$oLog 	= $this->getLog();
		$iCount	= count($aWait["employeeIds"]);
		if ($iCount > 0) {
			for ($i = 0; $i < $iCount; $i++) {
				print($aWait["employeeIds"][$i]);
				if (!$oLog->isWaited($this->getDocument()->getPointId(), $aWait["employeeIds"][$i])) {
					$isFool = false;
					break;
				}
				
			}
		}
		else {
			$isFool = false;
		}
		
		return $isFool;
	}
	
	final public function call($sAction, $aParams, $sType = "call", $bInitializing = true,
						 $iActionId = 0, $iSubjectId = 0, $iSubjectVersionId = 0)
	{
		$Ab 				= WF_ActionBox::newInstance();
		$iActionId 			= ($iActionId == 0) ? $this->getActionId() : $iActionId;
		$iSubjectId			= ($iSubjectId == 0) ? $this->getDocument()->getDocumentId() : $iSubjectId;
		$iSubjectVersionId	= ($iSubjectVersionId == 0) ? $this->getDocumentVersion()->getDocumentVersionId() : $iSubjectVersionId;
		$Ab->loadAction(
				$sAction,
				$iActionId,
				$this->getInitParam("subject_id"),
				$this->getInitParam("subject_version_id"),
				$bInitializing,
				$this
				);
				
		$Action = $Ab->getAction();
		$Action->iRequestId = $this->iRequestId;
		foreach ($aParams AS $sName => $mValue) {
			$Action->setParam($sName, $mValue);
		}

		return $Ab->process($sType);
	}
	
	private function init($sActionName, $iActionId, $iDocumentId, $iDocumentVersionId, $bInitializing = true, $CallerAction = null)
	{
		if ($iDocumentVersionId > 0) {
			$this->isVersion = true;
		}
		
		$this->Db 				= MSF_Registry::get("MSF_DB");
		$this->Dil 				= new WF_ActionBuilderDb($this->Db);
		$this->Template 		= MSF_Registry::get("MSF_Template");
		$this->sActionName		= $sActionName;
		$this->iActionId		= $iActionId;
		
		if ($bInitializing) {
			if ($CallerAction === null) {
				//try {
					if(!$this->Dil->documentExists($iDocumentId)) {
						throw new WF_DocumentNotExistedException("Document not found");	 
					}
					
					$iDoctypeId = $this->Dil->getDoctypeId($iDocumentId);
					if ($iDoctypeId > 0) {
						$this->Doctype			= new WF_Doctype($iDoctypeId);
						$this->DoctypeVersion	= $this->Doctype->getVersion($this->Dil->getDoctypeVersionId($iDocumentId));
						$this->Document 		= $this->DoctypeVersion->getDocument($iDocumentId);
						$this->DocumentVersion 	= ($iDocumentVersionId == 0) ? $this->Document->getCurrentVersion() : $this->Document->getVersion($iDocumentVersionId);
						$this->Values			= new WF_Values($this->Document, $this->DocumentVersion, $this->DoctypeVersion);
						$this->Settings			= new WF_Settings(
								$this->Values,
								$this->DoctypeVersion->getSpaXml(),
								$this->DoctypeVersion->getRouteXml(),
								$this->iStructureId
								);
	
						$this->iActionId 		= $iActionId;
						$this->sActionName		= $sActionName;
						$this->isInitialized 	= false;
						$this->isMain			= true;
						$this->Document->getLog()->setStructureId($this->iStructureId);
						$this->Document->getLog()->setEmployeeName($this->sEmployeeName);

						//debug($this->Settings->getRoute());
					}
				//}
				//catch (Exception $oE) {
					//throw new MSF_PageNotFoundException("Document " . $iDocumentId);
				//}
								
			}
			else {
					$this->Doctype			= $CallerAction->getDoctype();
					$this->DoctypeVersion	= $CallerAction->getDoctypeVersion();
					$this->Document 		= $CallerAction->getDocument();
					$this->DocumentVersion 	= $CallerAction->getDocumentVersion();
					$this->Values			= $CallerAction->getValues();
					$this->Settings			= $CallerAction->getSettings();
					$this->iActionId 		= $iActionId;
					$this->sActionName		= $sActionName;
					$this->isInitialized 	= true;
			}
			
		}
		
		if (CNF_MODE == "debug") {
			$this->debug();
		}

	}
	
	public function debug()
	{
		if ($this->Document !== null) {
			echo "<div style='background-color:white;'>";
			echo "<b> doctype id:</b> " 				. $this->Doctype->getDoctypeId();
			echo "<b> doctype version id:</b> " 		. $this->DoctypeVersion->getDoctypeVersionId();
			echo "<b> document id:</b> " 				. $this->Document->getDocumentId();
			echo "<b> document version id:</b> " 		. $this->DocumentVersion->getDocumentVersionId();
			echo "<b> doctype version extname:</b> " 	. $this->DoctypeVersion->getExtname();
			echo "</div>";
			echo "<div style='background-color:white;'>";
			echo "<b> point id:</b> " 					. $this->Document->getPointId();
			echo "<b> route:</b> " 						. (count((array)$this->Settings->getRoute()) == 0 ? "none" : "filled");
			echo "<b> registration number:</b> " 		. (strlen($this->getDocumentVersion()->getSystemField("registration_number") == 0) ? "none" : $this->Document->getRegistrationNumber());
			echo "<b> author:</b> " 						. $this->getDocumentVersion()->getSystemField("author_name")."(". $this->Document->getStructureId().")";
			echo "<b> employee:</b> " 					. $this->sEmployeeName . "(" . $this->iStructureId . ")" . "(". implode(".", $this->aEmployeePath) . ")";
			echo "</div>";
		}

	}
	
	protected function setDocumentVersion(WF_DocumentVersion $Version)
	{
		$this->DocumentVersion = $Version;
	}
	
	private function initSecurity()
	{
		$aReversed 	= array_reverse($this->aEmployeePath);
		$aAllowed	= array("view", "remove");
		if (in_array($this->sActionName, $aAllowed)) {
			$oDSwitcher = new WF_DocumentSwitcher($this->Db);
			if ($oDSwitcher->hasAccess("view", $aReversed, array("document_id"=>$this->Document->getDocumentId()))) {
				$this->isAllowed = true;
			}
			else {
				$oDtSwitcher 	= new WF_DoctypeSwitcher($this->Db);
				$aParams 		= array(
						"doctype_id" 	=> $this->Doctype->getDoctypeId(),
						"author_id"		=> $this->Document->getStructureId()
						);
				$this->isAllowed	 = $oDtSwitcher->hasAccess("view", $aReversed, $aParams);
			}
			
			if ($this->isMain) {
				if ($this->iStructureId == $this->Document->getStructureId()) {
					$this->isAllowed = true;
				}
				
			}

		}
		
	}
	
	private function initEmployee()
	{
		$UserBox 					= MSF_Registry::get("MSF_UserBox");
		$this->iStructureId			= $UserBox->getCurrentStructureId();
		$this->iEmployeeId			= $UserBox->getEmployeeAttribute("id");
		$this->iPostId				= $UserBox->getCurrentPostId();
		foreach ((array)$UserBox->getPosts() AS $aPost) {
			if ($aPost['id'] == $this->iPostId) {
				$this->sPost = $aPost['post'];
			}
			
		}

		$this->sEmployeeName		= $UserBox->getEmployeeAttribute("name");
		$sPath						= $UserBox->getCurrentStructurePath();
		if (!empty($sPath)) {
			$this->aEmployeePath	= explode(".", $sPath);
		}
		else {
			$this->aEmployeePath	= array(0);
		}
		
	}
	
	private function initDelegator()
	{
		$this->oDelegator = new WF_DelegateValues(
				$this->getDocument(),
				$this->getDocumentVersion(),
				$this->getDoctypeVersion(),
				$this->iStructureId
				);
	}
	
	public function getStructureId()
	{
		return $this->iStructureId;
	}
	
	public function getEmployeeId()
	{
		return $this->iEmployeeId;
	}
	
	public function getPostId()
	{
		return $this->iPostId;
	}
	
	public function saveStructureFieldValue($sFieldName, $sValue)
	{
		$aResult = array();
		$aField  = $this->getDocumentVersion()->getRefFields()->getField($sFieldName);
		if (!empty($aField)) {
			$oRefValue 	= new REF_Value();
			$this->getValues()->toStructureIds($sValue);
			$oRefValue->parse($this->getValues()->getStructureValues());
			$oResValue	= new REF_Value();
			$aValue		= $oRefValue->getValuesList();
			foreach ($oRefValue->getKeysList() AS $i => $aKeys) {
				switch (count($aKeys)) {
					case 3:
						$oResValue->add($aKeys, RefFieldHelper::formatStructureValue(
								$aKeys[1], 
								$aKeys[2], 
								MSF_SL::getParam($aField["optional"], "employee", "name"), 
								MSF_SL::getParam($aField["optional"], "post", "post"), 
								MSF_SL::getParam($aField["optional"], "employeeEnable", 1), 
								MSF_SL::getParam($aField["optional"], "postEnable", 1)
								)
						);
						break;
					case 2:
						$oResValue->add($aKeys, $aValue[$i]);
						break;
				}


			}

			$aResult[$sFieldName] = $oResValue->format();;	
		}

		$this->getDocumentVersion()->setFields($aResult);
		$this->getDocumentVersion()->save();
	}	

}
?>