<?php
include_once 'WF_ActionBuilderDb.php';
include_once 'modules/workflow/library/doctypeVersion/WF_Values.php';
include_once 'modules/workflow/library/actions/WF_ActionBox.php';
/**
 * @access public
 * @version 0.1
 * @copyright (c) 2008-2009 Detrix
 */
final class WF_DocFacade {
	
	/**
	 * @var MSF_DB
	 */
	private $oDb	= null;
	
	/**
	 * @var WF_ActionBuilderDb
	 */
	private $oABDb 	= null;
	
	/**
	 * @var string
	 */
	private $sError = null;

	/**
	 * Return new instance of WF_DocFacade
	 */
	public function __construct() {
		$this->oDb		= MSF_Registry::get('MSF_DB');
		$this->oABDb 	= new WF_ActionBuilderDb($this->oDb);
	}
	
	/**
	 * Returns instance of document or null
	 * @param int $iDocumentId
	 * @return WF_Document|null
	 */
	public function getDocumentById($iDocumentId) {
		try {
			$this->sError		= null;
			$iDoctypeId 		= $this->oABDb->getDoctypeId($iDocumentId);
			$iDoctypeVersionId 	= $this->oABDb->getDoctypeVersionId($iDocumentId);
			$oDoctype 			= new WF_Doctype($iDoctypeId);
			$oDoctypeVersion	= $oDoctype->getVersion($iDoctypeVersionId);
			$oDocument			= $oDoctypeVersion->getDocument($iDocumentId);
		}
		catch(Exception $oException) {
			$oDocument 			= null;
			$this->sError 		= $oException->getMessage();
		}
		
		return $oDocument;
	}
	
	public function initContextExecution(WF_Document $oDocument) 
	{					      		
		$oDocument->getLog()->clearExecuted($oDocument->getPointId());
		$oDocument->getLog()->clearWaited($oDocument->getPointId());
		$oDocument->save();
		$oAb 		= WF_ActionBox::newInstance();
		$oAb->loadAction("view", 0, $oDocument->getDocumentId(), null, true);
		$oAction	= $oAb->getAction();
		$oEc		= new WF_ExecutionContext($oAction);
		$oEc->execute();
		$oAction->getDocument()->save();
	}
	
	/**
	 * Returns instance of doctype or null
	 * @param int $iDocumentId
	 * @return WF_Doctype|null
	 */
	public function getDoctypeByDocumentId($iDocumentId) {
		try {
			$this->sError		= null;
			$iDoctypeId 		= $this->oABDb->getDoctypeId($iDocumentId);
			$oDoctype 			= new WF_Doctype($iDoctypeId);
		}
		catch(Exception $oException) {
			$oDoctype 			= null;
			$this->sError 		= $oException->getMessage();
		}

		return $oDoctype;
	}
	
	/**
	 * Input $sAddressValue:
	 * 1. user
	 * 2. author
	 * 3. all
	 * 4. any
	 * 5. field_XX
	 * 6. значение структуры
	 * @return array
	 */
	public function addressValueToStructureIds(WF_Document $oDocument, $sAddressValue) {
		$oValues = new WF_Values($oDocument, $oDocument->getCurrentVersion(), $oDocument->getDoctypeVersion());
		return $oValues->toStructureIds($sAddressValue);
	}
	
	/**
	 * Returns error message
	 * @return string|null
	 */
	public function getError() {
		return $this->sError;
	}
	
	public function getActionByDocumentId($iDocumentId)
	{
		$oAb 		= WF_ActionBox::newInstance();
		$oAb->loadAction("view", 0, $iDocumentId, null, true);
		$oAction	= $oAb->getAction();
		return $oAction;
	}

}