<?php
/**
 * Public interface for an actions
 * 
 * @package 	workflow
 */
interface WF_IAction
{
	const FINISH_POINT			= 500;

	const NEXT					= 1;
	
	/**
	 * The value which will be returned on success
	 */
	const SUCCESS				= 2;
	
	/**
	 * Very similar to SUCCESS. 
	 * There is only 1 defference. 
	 * Executed action in context execution will be executed again
	 */
	const NULL					= 4;
	
	/**
	 * The value which will be returned on failure
	 */
	const FAILURE				= 0;

	/**
	 * Initialize action
	 * @return void 
	 */
	public function initialize();

	/**
	 * Prepares SPA
	 */
	public function prepareSpa($aSpaValue);
	
	/**
	 * @return string 
     */
	public function getLogTitle();
	
	/**
	 * @return array() 
     */
	public function getLogParams();
	
	/**
	 * Process schedule
	 * @return self::SUCCESS || self::FAILURE
     */
	public function processSchedule();

	/**
	 * Process call
	 * @return self::SUCCESS || self::FAILURE
	 */
	public function processCall();
	
	/**
	 * Process execution
	 * @return self::SUCCESS || self::FAILURE
	 */
	public function processExecute();
	
	/**
	 * Process request
	 * @return self::SUCCESS || self::FAILURE
	 */
	public function processRequest();
	
	/**
	 * Process response
	 * @return self::SUCCESS || self::FAILURE
	 */
	public function processResponse();
	
	/**
	 * Cancel action
     */
	public function cancel();
}
?>
