<?php

class WF_Settings
{
	private $aRoute			= array();

	private $aActions		= array();
	
	private $aPureActions	= array();
	
	private $iStructureId	= 0;
	
	private $bFromExecution = false;
	
	private $oValues		= null;
	
	public function __construct($oValues, $sSpaXml, $sRouteXml, $iStructureId)
	{
		$this->iStructureId = $iStructureId;
		$this->oValues		= $oValues;
		$aSettings 			= $this->buildSettings();
		$this->aPureActions	= $this->buildSpa($sSpaXml, $aSettings);
		$this->aRoute		= $this->buildRoute($sRouteXml, $this->aPureActions, $aSettings);
		$this->aActions		= $this->buildActions($this->aRoute);
	}
	
	public function getPureActions($iActionId = null)
	{
		if ($iActionId == null) {
			$aResult = $this->aPureActions;	
		}
		else if (isset($this->aPureActions[$iActionId])){
			$aResult = $this->aPureActions[$iActionId];	
		}
		else {
			$aResult = array();
		}

		return $aResult;
	}
	
	public function isWaiting($iPointId, $iAcitonId)
	{
		if (isset($this->aRoute[$iPointId])) {
			return ($this->aRoute[$iPointId]["wait"]["spa_id"] == $iAcitonId);
		}
		else {
			return false;
		}
		
	}
	
	public function isLast($iPointId)
	{
		return (count($this->aRoute) - 1) == $iPointId;
	}
	
	public function getRoute()
	{
		return $this->aRoute;
	}
	
	public function getActions($iActionId = null)
	{
		if ($iActionId === null) {
			return $this->aActions;
		}
		else if (array_key_exists($iActionId, $this->aActions)) {
			return $this->aActions[$iActionId];
		}
		else if ($this->bFromExecution && array_key_exists($iActionId, $this->aPureActions)) {
			return $this->aPureActions[$iActionId];
		}
		else {
			return array();
		}

	}
	
	public function isPointExists($iPointId)
	{
		return array_key_exists($iPointId, $this->aRoute);
	}
	
	public function getTitleOfPoint($iPointId)
	{
		return (string) $this->getContextAtPoint("title", $iPointId);
	}
	
	public function getDelegated()
	{
		$aDelegated = array();
		for($i = 0; $i < count($this->aRoute); $i++) {
			$aDelegated = array_merge($aDelegated, $this->aRoute[$i]["delegate"]);	
		}
		
		return $aDelegated;
	}
	
	public function getDelegatedAtPoint($iPointId)
	{
		return $this->getContextAtPoint("delegate", $iPointId);
	}
	
	public function getExecution()
	{
		$aExecution = array();
		for($i = 0; $i < count($this->aRoute); $i++) {
			$aExecution = array_merge($aExecution, $this->aRoute[$i]["execute"]);
		}
		
		return $aExecution;
		
	}
	
	public function getExecutionAtPoint($iPointId)
	{
		return $this->getContextAtPoint("execute", $iPointId);
	}
	
	public function getWaitedAtPoint($iPointId)
	{
		return $this->getContextAtPoint("wait", $iPointId);
	}
	
	public function injectAction($iActionId, $aParams)
	{
		$this->aActions[$iActionId] = $aParams;
	}
	
	public function setFromExecution()
	{
		$this->bFromExecution = true;
	}

	private function getContextAtPoint($sContext, $iPointId)
	{
		if ($this->isPointExists($iPointId)) {
			return $this->aRoute[$iPointId][$sContext];
		}
		else {
			return array();
		}

	}
		
	private function buildSettings()
	{
		$aWFSettings 		= MSF_Registry::get("MSF_Settings")->getModule("workflow");
		$aResultSettings 	= array();
		foreach ($aWFSettings["actions"] AS $aWFAppSettings) {
			$aSettings = (isset($aWFAppSettings["settings"])) ? $aWFAppSettings["settings"]: array();
			$aResultSettings[$aWFAppSettings["name"]] = $aSettings;
			
		}

		return $aResultSettings;
	}
	
	private function buildSpa($sActionsXml, $aSettings) {
		$aDtAction 		= MSF_XmlWrapper::xmlToArray($sActionsXml);
		$aActions 		= array();
		foreach ($aDtAction AS $iSpaId => $aOptions) {
			$aActions[$iSpaId]					= array();
			$aActions[$iSpaId]["title"]			= $aOptions["title"];
			$aActions[$iSpaId]["name"]			= $aOptions["value"]["action"];
			$aActions[$iSpaId]["description"]	= isset($aOptions["value"]["description"]) ? $aOptions["value"]["description"] : "";
			$aActions[$iSpaId]["options"] = array();
			if (isset($aOptions["value"]["value"])) {
				$aActions[$iSpaId]["options"] = $this->getSpaOptionValues($aOptions["value"]["value"], false);
			}

			$aActions[$iSpaId]["settings"]	= array();
			foreach ($aSettings[$aOptions["value"]["action"]] AS $sKey => $sValue) {
				$aActions[$iSpaId]["settings"][$sKey] = $sValue;
			}
			
		}

		/*
		$aActions[0]	= array (
						"title" => "__VIEW__",
						"value" => array(
								"action" 	=> "view",
								"value"		=> array()
								)
						)
		*/
		return $aActions;
	}
	
	private function getSpaOptionValues($aOptions, $bExecution) {
		$aValue = array();
		if (isset($aOptions["field"])) {
			$aOptions = $this->setOptionArray($aOptions, "field", "value");
		}
		if (isset($aOptions["notify_address_type"])) {
			if ($bExecution) {
				$aOptions = $this->setOptionArray($aOptions, "notify_address_type", "notify_address_value");
			}
			else {
				$aOptions = $this->setOptionArray($aOptions, "notify_address_type", "addresses");
			}
		} 
		if (isset($aOptions["notify_address_type_if"])) {
			$aOptions = $this->setOptionArray($aOptions, "notify_address_type_if", "notify_address_value_if");
		}
		if (isset($aOptions["notify_address_type_else"])) {
			$aOptions = $this->setOptionArray($aOptions, "notify_address_type_else", "notify_address_value_else");
		}
		if (isset($aOptions["address_value"])) {
			$aOptions = $this->setOptionArray($aOptions, "address_type", "address_value");
		}
		foreach ($aOptions AS $sName => $sValue) {
			$aValue[$sName] = $sValue;
		}
		return $aValue;
	}
	
	private function setOptionArray($aOptions, $sType, $sOption) {
		$bGroup = ($aOptions[$sType] == "groups") ? true : false;
		if ($bGroup) {
			$aOptions[$sOption] = $this->oValues->getRefGroupNames($aOptions[$sOption]);
			$aOptions[$sType] = "value";
		} 
		return $aOptions;				
	}

	private function buildActions($aRoute)
	{
		$aActions = array();
		for ($i = 0; $i < count($aRoute); $i++) {
			$aDelegate = &$aRoute[$i]["delegate"];
			for ($j = 0; $j < count($aDelegate); $j++) {
				if (!array_key_exists($aDelegate[$j]["spa_id"], $aActions)) {
					$aActions[$aDelegate[$j]["spa_id"]] = $aDelegate[$j];
					unset($aActions[$aDelegate[$j]["spa_id"]]["spa_id"]);
				}
				
			}
			
			$aExecute	= &$aRoute[$i]["execute"];
			for ($j = 0; $j < count($aExecute); $j++) {
				if (!array_key_exists($aExecute[$j]["spa_id"], $aActions)) {
					$aActions[$aExecute[$j]["spa_id"]] = $aExecute[$j];
					unset($aActions[$aExecute[$j]["spa_id"]]["spa_id"]);
				}
				
			}
			
		}
		return $aActions;
	}

	private function buildRoute($sRouteXml, $aSpa, $aSettings)
	{
		$aDtRoute 		= MSF_XmlWrapper::xmlToArray($sRouteXml);
		$aRoute			= array();
		for ($i = 0; $i < count($aDtRoute); $i++) {
			$aDtPoint				= $aDtRoute[$i];
			$aRoute[$i] 			= array();
			$aRoute[$i]["title"] 	= $aDtPoint["title"];
			$aRoute[$i]["delegate"]	= array();
			$k 						= 0;
			$aDelegatedIds			= array();
			foreach ($aDtPoint["value"]["delegate"] AS $j => $aValue ) {
				$aDelegation 							= array();
				$aDelegation["spa_id"] 					= $aValue["spa_id"];
				$aDelegation["employeeIds"]				= $this->oValues->getPostIds($aValue["type"], $aValue["value"]);
				if (in_array($aDelegation["spa_id"], $aDelegatedIds) || (empty($aDelegation["spa_id"]))) {
					// do nothing					
				}
				else if (in_array($this->iStructureId, $aDelegation["employeeIds"])) {
					if (isset($aDelegation["spa_id"])) {
						$aRoute[$i]["delegate"][$k]				= array_merge($aDelegation, MSF_SL::getParam($aSpa, $aDelegation["spa_id"], array()));
						$aRoute[$i]["delegate"][$k]["settings"]	= MSF_SL::getParam($aSettings, $aRoute[$i]["delegate"][$k]["name"], array());
						$aDelegatedIds[] = $aDelegation["spa_id"];		
						$k++;		
					}

				}

			}
			
			$aRoute[$i]["execute"]	= array();
			for ($j = 0; $j < count($aDtPoint["value"]["execute"]); $j++) {
				$aRoute[$i]["execute"][$j] 				= array();
				$aRoute[$i]["execute"][$j]["spa_id"]	= $this->getDynamicActionId($i, $j);
				$aRoute[$i]["execute"][$j]["name"]		= $aDtPoint["value"]["execute"][$j]["name"];
				$aRoute[$i]["execute"][$j]["title"]		= MSF_Registry::get("MSF_Locale")->getWord("WFA_" . strtoupper($aRoute[$i]["execute"][$j]["name"]));
				$aRoute[$i]["execute"][$j]["options"]	= array();
				if (isset($aDtPoint["value"]["execute"][$j]["value"])) {
					$aRoute[$i]["execute"][$j]["options"] = $this->getSpaOptionValues($aDtPoint["value"]["execute"][$j]["value"], true);
				}

				$aRoute[$i]["execute"][$j]["settings"]	= MSF_SL::getParam($aSettings,$aRoute[$i]["execute"][$j]["name"], array());
				$this->aPureActions[$aRoute[$i]["execute"][$j]["spa_id"]] = $aRoute[$i]["execute"][$j];
				unset($this->aPureActions[$aRoute[$i]["execute"][$j]["spa_id"]]["spa_id"]);

			}

			$aWait = array();
			$aWait["spa_id"] = isset($aDtPoint["value"]["wait"]["spa_id"]) ? $aDtPoint["value"]["wait"]["spa_id"] : null;
			if (isset($aDtPoint["value"]["wait"]["value"])) {
				if (is_string($aDtPoint["value"]["wait"]["value"])) {
					$aWait["employeeIds"] = $this->oValues->getPostIds($aDtPoint["value"]["wait"]["type"], $aDtPoint["value"]["wait"]["value"]);
				}
				else {
					$aWait["employeeIds"] = $this->oValues->getPostIds($aDtPoint["value"]["wait"]["type"], $aDtPoint["value"]["wait"]["value"]["name"]);
					$aWait['options'] = array();
					
					foreach ($aDtPoint["value"]["wait"]["value"] AS $sKey => $mValue) {
						if ($sKey !== "name") {
							$aWait['options'][$sKey] = $mValue;
						}
					
					}

				}
			}
			else {
				$aWait["employeeIds"] = array();
			}
			
			if (isset($aDelegation)) {
				$aWait["settings"] 	= MSF_SL::getParam($aSettings,$aDelegation["spa_id"], array());
			}
			
			$aRoute[$i]["wait"] = array_merge($aWait, MSF_SL::getParam($aActions, $aWait["spa_id"], array()));
		}
		return $aRoute;
	}
	
	private function getDynamicActionId($iPointId, $iSystemId) {
		if ($iPointId > 9) {
			return (int) $iPointId . $iSystemId;
		}
		else {
			return (int) "7" . $iPointId . $iSystemId;
		}
		
	}	
	
}
?>