<?php

defined("__MSF") or die("Direct access is not allowed");

include_once "/library/MSF_Utils/MSF_Collection.php";
include_once "modules/workflow/library/documentLog/WF_DocumentLog.php";
include_once "modules/workflow/library/documentLog/WF_DocumentLogValue.php";
include_once "modules/workflow/library/documentLog/WF_DocumentLogSetDB.php";
include_once "modules/workflow/library/documentLog/WF_DocumentLogSet.php";
include_once "WF_Employee.php";

abstract class WF_AbstractAction
{
	protected $DB 			= null;

	protected $LogSet 		= null;

	protected $aUserParams 	= array();

	protected $Document 	= null;

	protected $bStatus 		= false;

	protected $Template 	= null;

	protected $iRouteLength = 0;

	protected $iActionId 	= 0;
	
	protected $aSystemParams 	= array();

	protected $sRedirectUrl 	= "";

	protected $sRedirectMessage = "";

	protected $iStructureId 		= 0;

	public function __construct()
	{
		$this->DB 				= MSF_Registry::get("MSF_DB");
		$this->Template 		= MSF_Registry::get("MSF_Template");
		$this->LogSet 			= new WF_DocumentLogSet(new WF_DocumentLogSetDB($this->DB));
		$this->iStructureId 	= MSF_Registry::get("MSF_UserBox")->getEmployeeAttribute("id");
	}

	public function setSystemParams($aSysParams)
	{
		$this->aSystemParams = $aSysParams;
	}

	public function setUserParams($aUserParams)
	{
		$this->aUserParams = $aUserParams;
	}

	public function setDocument(WF_IDocument $Document)
	{
		$this->Document = $Document;
	}

	public function getStatus()
	{
		return $this->bStatus;
	}

	public function setRouteLength($iLength)
	{
		$this->iRouteLength = $iLength;
	}

	public function setActionId($iActionId)
	{
		$this->iActionId = $iActionId;
	}

	public function getRedirectUrl()
	{
		return $this->sRedirectUrl;
	}

	public function getRedirectMessage()
	{
		return $this->sRedirectMessage;
	}

	protected function saveLog($sValue, $sDescription, $bDisplayable, $iStructureId = -1)
	{
		$Log 	= new WF_DocumentLog();
		$Value 	= new WF_DocumentLogValue();
		$Coll 	= new MSF_Collection();

		if ($iStructureId == -1) {
			$iStructureId = $this->iStructureId;
		}
		
		$Value->setCreatedAtDate(MSF_Date::getDateInt());
		$Value->setCreatedAtTime(MSF_Date::getDateTimeInt());
		$Value->setDescription($sDescription);
		$Value->setEmployee(new WF_Employee($iStructureId, MSF_Registry::get("MSF_User")->getFullName()));
		$Value->setValue($sValue);

		$Coll->add($Value);

		$Log->setActionId($this->iActionId);
		$Log->setDisplayable($bDisplayable);
		$Log->setDocumentId($this->Document->getDocumentId());
		$Log->setPointId($this->Document->getPointId());
		$Log->setValues($Coll);

		$this->LogSet->add($Log);
	}

	abstract public function processSystem();

	abstract public function processUserRequest();

	abstract public function processUserResponse(DOMElement $Com);
}

?>