<?php

defined("__MSF") or die("Direct access is not allowed");

include_once "modules/references/library/Reference/UserReferenceDB.php";
include_once "modules/references/library/Reference/UserReference.php";

class WF_DoctypeCompare
{
	private $Doctype;
	
	private $Ref;
	
	private $aDoctype;
	
	private $aDtVersions;
	
	public function __construct(WF_Doctype $Doctype)
	{
		$this->Doctype 		= $Doctype;
		$this->Ref 			= new UserReference(new UserReferenceDB(MSF_Registry::get("MSF_DB")));
		$this->aDoctype 	= $this->defineComparedResult();
		$this->aDtVersions 	= $this->Doctype->getVersionsData();
	}
	
	private function defineComparedResult()
	{
		$aDoctype 	= array();
		$Wrapper 	= new MSF_XmlWrapper($this->Doctype->getFieldsXml());
		
		foreach ($Wrapper AS $Data) {
			$aDoctype[] = $this->returnNeeded($Data, $this->Ref);
		}
		
		return $aDoctype;
	}
	
	public function getComparedResult()
	{
		return $this->aDoctype;
	}
	
	public function saveComparedResult($aPost, $isPost = true)
	{
		
		$aResult 	= array();
		$j 			= 0;
		$m 			= 0;
		
		if (!$isPost) {
			$aPost = $this->formDataToSave($aPost);
		}

		for ($i = 0; $i < count($this->aDtVersions); $i++) {
			$this->aDtVersions[$i]["fields_xw"] =  new MSF_XmlWrapper($this->aDtVersions[$i]["fields_xml"]);
		}

		foreach ($aPost AS $k => $v) {
			if (is_array($v)) {
				$aExploded 	= explode("::", $v[0]);
				$aTmp 		= explode("::", $k);
				$sTable 	= $aTmp[2];
				$aTmp 		= explode("_", $aTmp[0]);
				$sType 		= $aTmp[1];
				//
				for ($i = 0; $i < sizeof($this->aDtVersions); $i++) {
					foreach ($this->aDtVersions[$i]["fields_xw"] AS $Data) {
						if ($Data->getName() == $aExploded[1] && $sTable == $this->aDtVersions[$i]["extname"] && $sType == $Data->getValue("type")) {
							$aResult[$j][$m]["data"] 		= $Data;
							$aResult[$j][$m]["table"] 		= $this->aDtVersions[$i]["extname"];
							$aResult[$j][$m]["compared"] 	= $aExploded[0];

							$aValues = $Data->getValue();
							if (isset($aValues["reference"])) {
								$aResult[$j][$m]["reference"] = $aValues["reference"];
							}
							$m++;
							break;
						}
					}

				}

				$m = 0;
				
				if (isset($aResult[$j])) {
					$j++;
				}
			}
		}
		//debug(mt("stop"));
		
		$this->Doctype->setComparedXml($this->prepareDoctypeCompareXml($aResult));
		$this->Doctype->save();
		
	}
	
	private function formDataToSave($aPost)
	{
		$aResult 	= array();
		$aVersions 	= $this->getVersions();
		$aTmp 		= $aPost;
		
		for ($i = 0; $i < sizeof($aPost); $i++) {
			if (!isset($aPost[$i]["compared"])) {
				continue;
			}
			
			for ($m = 0; $m < sizeof($aPost[$i]["compared"]); $m++) {
				
				for ($j = 0; $j < sizeof($aVersions); $j++) {
					foreach ($aVersions[$j] as $aValue) {
						if ((isset($aValue["name"])) && ($aTmp[$i]["name"] == $aValue["name"])) {
							$aValue["version"] = $aVersions[$j]["version"];
							$aTmp[$i]["compared"][] = $aValue;
						}
					}
				}
				
			}
		}
		
		for ($i = 0; $i < sizeof($aTmp); $i++) {
			if (!isset($aPost[$i]["compared"])) {
				continue;
			}
			
			for ($j = 0; $j < sizeof($aTmp[$i]["compared"]); $j++) {
				$aResult["fields_" . $aTmp[$i]["type"] . "::" . $aTmp[$i]["name"] . "::" . $aTmp[$i]["compared"][$j]["version"]][] = $aTmp[$i]["name"] . "::" . $aTmp[$i]["compared"][$j]["name"];
			}
			
		}
		
		return $aResult;
	}
	
	private function prepareDoctypeCompareXml($aCompareResult)
	{
		$DOMDocument 	= new DOMDocument();
		$Document 		= $DOMDocument->createElement("document");
		
		for ($i = 0; $i < sizeof($aCompareResult); $i++) {
			$Element = $DOMDocument->createElement("element");
			$FirstItem = $DOMDocument->createElement("item");
			$Element->appendChild($FirstItem);
			$FieldsItem = $DOMDocument->createElement("item");
			$FieldsItem->setAttribute("name", "fields");
			$FieldsItem->setAttribute("type", "array");

			for ($j = 0; $j < sizeof($aCompareResult[$i]); $j++) {
				$Element->setAttribute("title", $aCompareResult[$i][$j]["data"]->getTitle());
				$Element->setAttribute("type", "array");
				$Element->setAttribute("name", $i);

				$FirstItem->setAttribute("name", "type");
				$FirstItem->setAttribute("type", "string");
				$FirstItem->setAttribute("value", $aCompareResult[$i][$j]["data"]->getValue("type"));

				$Item = $DOMDocument->createElement("item");
				$Item->setAttribute("name", $j);
				$Item->setAttribute("type", "array");

				$SubItem = $DOMDocument->createElement("item");
				$SubItem->setAttribute("name", "tablename");
				$SubItem->setAttribute("type", "string");
				$SubItem->setAttribute("value", $aCompareResult[$i][$j]["table"]);
				$Item->appendChild($SubItem);

				$SubItem = $DOMDocument->createElement("item");
				$SubItem->setAttribute("name", "fieldname");
				$SubItem->setAttribute("type", "string");
				$SubItem->setAttribute("value", $aCompareResult[$i][$j]["data"]->getName());
				$Item->appendChild($SubItem);

				$SubItem = $DOMDocument->createElement("item");
				$SubItem->setAttribute("name", "compared_with");
				$SubItem->setAttribute("type", "string");
				$SubItem->setAttribute("value", $aCompareResult[$i][$j]["compared"]);
				$Item->appendChild($SubItem);

				if (isset($aCompareResult[$i][$j]["reference"])) {
					$SubItem = $DOMDocument->createElement("item");
					$SubItem->setAttribute("name", "reference");
					$SubItem->setAttribute("type", "int");
					$SubItem->setAttribute("value", $aCompareResult[$i][$j]["reference"]);
					$Item->appendChild($SubItem);
				}

				$FieldsItem->appendChild($Item);
				$Element->appendChild($FieldsItem);
			}

			$Document->appendChild($Element);
		}
		
		$DOMDocument->appendChild($Document);
		return $DOMDocument->saveXML();
	}
	
	public function getVersions()
	{
		$aVersions = array();
		$WCompared = new MSF_XmlWrapper($this->Doctype->getComparedXml());

		foreach ($WCompared AS $Data) {
			for ($i = 0; $i < sizeof($this->aDoctype); $i++) {
				$aTmp 		= $this->aDoctype[$i];
				$aValues 	= $Data->getValue();

				for ($j = 0; $j < sizeof($aValues["fields"]); $j++) {
					if ($this->aDoctype[$i]["name"] == $aValues["fields"][$j]["compared_with"]) {
						$a = array();
						$a["name"] = $aValues["fields"][$j]["fieldname"];
						$a["type"] = $aValues["type"];

						if (isset($aValues["fields"][$j]["reference"])) {
							$Ref->load($aValues["fields"][$j]["reference"]);
							$a["ref_title"] = $Ref->getName();
						}

						$a["title"] 		= $Data->getTitle();
						$a["version"] 		= $aValues["fields"][$j]["tablename"];
						$aTmp["compared"][] = $a;
					}
				}
				$this->aDoctype[$i] = $aTmp;
			}
		}

		for ($i = 0; $i < sizeof($this->aDtVersions); $i++) {
			$Wrapper 		= new MSF_XmlWrapper($this->aDtVersions[$i]["fields_xml"]);
			$aTmp 			= array();
			$aVersions[$i] 	= array();

			foreach ($Wrapper AS $Data) {
				$aData = $this->returnNeeded($Data, $this->Ref);

				for ($j = 0; $j < sizeof($this->aDoctype); $j++) {
					if (isset($this->aDoctype[$j]["compared"])) {
						for ($m = 0; $m < sizeof($this->aDoctype[$j]["compared"]); $m++) {
							if ($this->aDoctype[$j]["compared"][$m]["name"] == $aData["name"] && $this->aDtVersions[$i]["extname"] == $this->aDoctype[$j]["compared"][$m]["version"]) {
								$aTmp[] = $aData["name"];
							}
						}
					}
				}

				if (!in_array($aData["name"], $aTmp)) {
					$aVersions[$i][] = $aData;
				}
			}
			$aVersions[$i]["version"] = $this->aDtVersions[$i]["extname"];
		}

		return $aVersions;
	}
	
	private function returnNeeded($Data, $Ref)
	{
		$aResult = array();

		$aResult["name"] = $Data->getName();
		$aResult["title"] = $Data->getTitle();
		$aResult["type"] = $Data->getValue("type");

		$aValues = $Data->getValue();

		if (isset($aValues["reference"]) && $Ref) {
			$Ref->load($aValues["reference"]);
			$aResult["ref_title"] = $Ref->getName();
		}
		
		return $aResult;
	}
}

?>