<?php

defined("__MSF") or die("Direct access is not allowed");

include_once "modules/workflow/library/logs/WF_Log.php";
include_once "WF_Values.php";

class WF_DelegateValues
{
	private $Document;
	
	private $DoctypeVersion;
	
	private $iStructureId;
	
	private $Route;
	
	private $isAuthor = false;
	
	private $Values;
	
	private $Logs;
	
	private $sRowDelim = "|^|row|^|";
	
	private $sColDelim = "/^/col/^/";
	
	public function __construct($Document, $DocumentVersion, $DoctypeVersion, $iStructureId)
	{
		$this->Document 		= $Document;
		$this->DoctypeVersion 	= $DoctypeVersion;
		$this->iStructureId 	= $iStructureId;
		$this->Route 			= new MSF_XmlWrapper($this->DoctypeVersion->getRouteXml());
		$this->Values 			= new WF_Values($Document, $DocumentVersion, $DoctypeVersion);
		if ($Document->getStructureId() == $iStructureId) {
			$this->isAuthor = true;
		}
	}
	
	public function getAtPoint($iPointId)
	{
		$aResult = array();
		
		foreach ($this->Route as $iPoint => $Data) {
			$aValue = $Data->getValue();
			if ($iPoint == $iPointId && isset($aValue["delegate"])) {
				$aResult = $aValue["delegate"];
				break;
			}
		}
		
		return $aResult;
	}
	
	public function get()
	{
		$aResult = array();
		
		foreach ($this->Route as $iPoint => $Data) {
			$aValue = $Data->getValue();
			if ($this->Document->getPointId() <= $iPoint && isset($aValue["delegate"])) {
				$aResult[] = $aValue["delegate"];
			}
		}
		
		return $aResult;
	}
	
	public function isDelegated($iActionId)
	{
		$bResult 	= false;
		$aDelegate 	= $this->getAtPoint($this->Document->getPointId());
		$sType 		= "";
		$mValue 	= "";
		
		foreach ($aDelegate as $aVal) {
			if (!$bResult && $aVal["spa_id"] == $iActionId) {
				$sType 	= $aVal["type"];
				$mValue = $aVal["value"];
		
				switch ($sType) {
					case "fields":
						$aEmployees = $this->Values->getFromFields($mValue);
						
						for ($i = 0; $i < count($aEmployees); $i++) {
							if ($this->iStructureId == $aEmployees[$i]) {
								$bResult = true;
								break;
							}
						}
						break;
					case "vars":
						$sValue = (is_array($mValue) && isset($mValue["name"])) ? $mValue["name"] : $mValue;
						switch ($sValue) {
							case "author":
								$bResult = $this->isAuthor;
								break;
							case "all":
								$aEmployees = $this->getAtPoints();
								for ($i = 0; $i < sizeof($aEmployees); $i++) {
									if ($this->iStructureId == $aEmployees[$i]) {
										$bResult = true;
										break;
									}
								}
								break;
						}
						break;
					case "groups":
						$aPath 		= WF_Utils::getPath();
						$aEmployees = $this->Values->getRefGroupIds($mValue, $this->iStructureId);
						for ($i = 0; $i < sizeof($aEmployees); $i++) {
							if (!empty($aEmployees[$i])) {
								if (in_array($aEmployees[$i], $aPath) && in_array($this->iStructureId, $aPath)) {
									$bResult = true;
									break;
								}
							}
						}
					default:
						$aPath 		= WF_Utils::getPath();
						$aEmployees = $this->Values->toStructureIds($mValue);
						for ($i = 0; $i < sizeof($aEmployees); $i++) {
							if (!empty($aEmployees[$i])) {
								if (in_array($aEmployees[$i], $aPath) && in_array($this->iStructureId, $aPath)) {
									$bResult = true;
									break;
								}
							}
						}
						break;
				}
			}
		}
		
		return $bResult;
	}
	
	private function getAtPoints()
	{
		$aResult 	= array();
		$aDelegate 	= $this->get();
		
		for ($i = 0; $i < sizeof($aDelegate); $i++) {
			for ($j = 0; $j < sizeof($aDelegate[$i]); $j++) {
				$aResult = array_merge($aResult, $this->Values->toStructureIds($aDelegate[$i][$j]["value"]));
			}
		}
		
		return $aResult;
	}
	
	public function isInWait($iPointId, $iActionId)
	{
		$Logs 		= WF_Log::newInstance(MSF_Registry::get("MSF_DB"), $this->Document->getDocumentId());
		$Logs->load();
		$aLogs 		= $Logs->get($iPointId, $iActionId);
		$bResult 	= false;
		$aLogValues = array();
		$aCheck 	= array();
		
		foreach ($aLogs as $aValues) {
			$aLogValues[] = $aValues["structureId"];
		}
		
		foreach ($this->Route as $Data) {
			if ($Data->getName() == $iPointId) {
				$aValue = $Data->getValue();
				
				if ($iActionId == $aValue["wait"]["spa_id"] && isset($aValue["wait"]["value"])) {
					$aResult = $this->Values->toStructureIds($aValue["wait"]["value"]);
					
					switch ($aValue["wait"]["type"]) {
						case "groups":
							$aResult = $this->Values->getRefGroupIds($aValue["wait"]["value"], $this->iStructureId);
						case "structure":
							for ($i = 0; $i < sizeof($aResult); $i++) {
								$bCheck = false;
								for ($j = 0; $j < sizeof($aLogValues); $j++) {
									if ($aResult[$i] == $aLogValues[$j]) {
										$bCheck = true;
										break;
									}
								}
								
								if ($bCheck) {
									$aCheck[] = true;
								}
								else {
									$aCheck[] = false;
								}
							}
							
							$bResult = true;
							for ($i = 0; $i < sizeof($aCheck); $i++) {
								if (!$aCheck[$i]) {
									$bResult = false;
									break;
								}
							}
							break;
						case "vars":
							switch ($aValue["wait"]["value"]) {
								case "any":
									for ($i = 0; $i < sizeof($aResult); $i++) {
										if (!in_array($aResult[$i], $aLogValues)) {
											$aCheck[] = false;
										}
										else {
											$aCheck[] = true;
										}
									}
									
									for ($i = 0; $i < sizeof($aCheck); $i++) {
										if ($aCheck[$i]) {
											$bResult = true;
											break;
										}
									}
									break;
								case "author":
									$bResult = in_array($this->iStructureId, $aResult);
									break;
								case "all":
									for ($i = 0; $i < sizeof($aResult); $i++) {
										$bCheck = false;
										for ($j = 0; $j < sizeof($aLogValues); $j++) {
											if ($aResult[$i] == $aLogValues[$j]) {
												$bCheck = true;
												break;
											}
										}
										
										if ($bCheck) {
											$aCheck[] = true;
										}
										else {
											$aCheck[] = false;
										}
									}
									
									$bResult = true;
									for ($i = 0; $i < sizeof($aCheck); $i++) {
										if (!$aCheck[$i]) {
											$bResult = false;
											break;
										}
									}
									break;
							}
							break;
						case "fields":
							for ($i = 0; $i < sizeof($aResult); $i++) {
								$bCheck = false;
								for ($j = 0; $j < sizeof($aLogValues); $j++) {
									if ($aResult[$i] == $aLogValues[$j]) {
										$bCheck = true;
										break;
									}
								}
								
								if ($bCheck) {
									$aCheck[] = true;
								}
								else {
									$aCheck[] = false;
								}
							}
							
							$bResult = true;
							for ($i = 0; $i < sizeof($aCheck); $i++) {
								if (!$aCheck[$i]) {
									$bResult = false;
									break;
								}
							}
							break;
					}
					break;
				}
			}
		}
		
		return $bResult;
	}
}

?>