<?php
/**
 * 
 */
class WF_DoctypeVersionCleaner {
	
	const FIELD_PREFIX		= "field_";
	
	private $aFields 		= array();
	
	private $aSpa 	 		= array();
	
	private $aRoute  		= array();
	
	private $aFieldsList;
	
	private $aSpaList;
	
	public function setFieldsXml($sFieldsXml) {
		$this->aFields 	= MSF_XmlWrapper::xmlToArray($sFieldsXml);
	}
	
	public function setSpaXml($sSpaXml) {
		$sSpaXml = str_replace("\n","&#10;", $sSpaXml);
		$this->aSpa = MSF_XmlWrapper::xmlToArray($sSpaXml);
	}
	
	public function setRouteXml($sRouteXml) {
		$this->aRoute 	= MSF_XmlWrapper::xmlToArray($sRouteXml);
	}
	
	public function getFieldsXml() {
		return $this->arrayToXml($this->aFields);
	}
	
	public function getSpaXml() {
		$sSpaXml = str_replace("\n","&#10;", $this->arrayToXml($this->aSpa));
		return $sSpaXml;
	}
	
	public function getRouteXml() {
		return $this->arrayToXml($this->aRoute);
	}
	
	private function arrayToXml($aArray) {
		$oXw = new MSF_XmlWrapper();
		foreach ($aArray AS $sName => $aBody) {
			$oXw->set($sName, $aBody["value"], $aBody["title"]);
		}

		return $oXw->getXml();
	}
	
	public function clean() {
		$this->prepareFields();
		$this->prepareSpa();
		$this->cleanSpa();
		$this->cleanRoute();
	}
	
	private function cleanSpa() {
		foreach ($this->aSpa AS $iSpaId => $aAction) {
			$this->aSpa[$iSpaId] = $this->seekForUnlinkedFields($aAction);
		}

	}
	
	private function cleanRoute() {
		foreach ($this->aRoute AS $iPointId => $aPoint) {
			$aPoint 					= $this->seekForUnlinkedFields($aPoint);
			$this->aRoute[$iPointId] 	= $this->seekForUnlinkedSpa($aPoint);
			//$this->aRoute[$iPointId] = $this->seekForUnlinkedFields($aPoint);
		}
		
	}
	
	/**
	 * @return void
	 */
	private function prepareFields() {
		$this->aFieldsList 		= array();
		$this->aFieldsTitles 	= array();
		foreach ($this->aFields AS $sName => $aBody) {
			$this->aFieldsList[] 			= $sName;
		}
		
	}
	
	/**
	 * @return void
	 */
	private function prepareSpa() {
		$this->aSpaList 	= array();
		$this->aSpaTitles 	= array();
		foreach ($this->aSpa AS $sName => $aBody) {
			$this->aSpaList[] 			= $sName;
		}
		
	}
	
	/**
	 * @return array
	 */
	private function seekForUnlinkedFields($aAction) {
		foreach ($aAction AS $sKey => $mValue) {
			if (is_array($mValue)) {
				$aAction[$sKey] = $this->seekForUnlinkedFields($mValue);
			}
			else if (substr($mValue, 0, (strlen(self::FIELD_PREFIX) - 1)) == self::FIELD_PREFIX) {
				if  (!in_array($mValue, $this->aFieldsList)) {
					$aAction[$sKey] = null;
				}
				else {
					$aAction[$sKey] = $mValue;
				}
			
			}
			else {
				$aAction[$sKey] = $mValue;
			}
			
		}
		
		return $aAction;
	}
	
	private function seekForUnlinkedSpa($aPoint) {
		foreach ($aPoint AS $sKey => $mValue) {
			if (is_array($mValue)) {
				$aPoint[$sKey] = $this->seekForUnlinkedSpa($mValue);
			}
			else if ($sKey == "spa_id") {

				if  (!in_array($mValue, $this->aSpaList)) {
					$aPoint[$sKey] = null;
				}
				else {
					$aPoint[$sKey] = $mValue;
				}
			
			}
			else {
				$aPoint[$sKey] = $mValue;
			}
			
		}
		
		return $aPoint;
	}

	public static function example() {
		echo "<b>WF_DoctypeVersionCleaner</b><br/>\n";
		echo "<hr/>\n";
		$sFieldsXml =  '<document>
							<element title="1" name="field_b0af523a9b" type="array">
								<item name="description" type="text" value=""></item>
								<item name="type" type="text" value="int"></item>
								<item name="required" type="text" value="0"></item>
								<item name="defaultValue" type="text" value=""></item>
								</element>
							</document>';
		$sSpaXml	=  '<document>
							<element title="notify" name="2760101" type="array">
								<item name="action" type="text" value="notify"></item>
								<item name="description" type="text" value=""></item>
								<item name="value" type="array">
									<item name="visible" type="text" value="1"></item>
									<item name="notify_address_type" type="text" value="fields"></item>
									<item name="addresses" type="text" value="field_a0bec36fc0"></item>
									<item name="text" type="text" value=""></item>
									<item name="type" type="text" value="parallel"></item>
									</item>
								</element>
							</document>';
		$sRouteXml	=  '<document>
							<element title="point" name="0" type="array">
								<item name="delegate" type="array">
									<item name="0" type="array">
										<item name="spa_id" type="text" value="2760100"></item>
										<item name="type" type="text" value="fields"></item>
										<item name="value" type="text" value="field_a0bec36fc0"></item>
										</item>
									</item>
								<item name="execute" type="array">
									<item name="0" type="array">
										<item name="name" type="text" value="notify"></item>
										<item name="value" type="array">
											<item name="notify_address_type" type="text" value="fields"></item>
											<item name="notify_address_value" type="text" value="field_a0bec36fc0"></item>
											<item name="notify_text" type="text" value=""></item>
											<item name="notify_type" type="text" value="parallel"></item>
											</item>
										</item>
									</item>
								<item name="wait" type="array">
									<item name="spa_id" type="text" value="2760100"></item>
									<item name="type" type="text" value="fields"></item>
									<item name="value" type="text" value="field_a0bec36fc0"></item>
									</item>
								</element>
							</document>';
		$oCleaner = new WF_DoctypeVersionCleaner();
		$oCleaner->setFieldsXml($sFieldsXml);
		$oCleaner->setSpaXml($sSpaXml);
		$oCleaner->setRouteXml($sRouteXml);
		$oCleaner->clean();
		
		echo "WF_DoctypeVersionCleaner. Cleaning SPA...";
		$aSpa = MSF_XmlWrapper::xmlToArray($oCleaner->getSpaXml());
		if ($aSpa[2760101]["value"]["value"]["addresses"] == null) {
			echo "success<br/>\n";
		}
		else {
			echo "failure<br/>\n";
		}
		
		$aRoute = MSF_XmlWrapper::xmlToArray($oCleaner->getRouteXml());

		echo "WF_DoctypeVersionCleaner. Cleaning context execute in the route...";
		if ($aRoute[0]["value"]["delegate"][0]["value"]["notify_address_value"] == null) {
			echo "success<br/>\n";
		}
		else {
			echo "failure<br/>\n";
		}
		
		echo "WF_DoctypeVersionCleaner. Cleaning context delegation in the route...";
		if ($aRoute[0]["value"]["delegate"][0]["value"] == null) {
			echo "success<br/>\n";
		}
		else {
			echo "failure<br/>\n";
		}
		
		echo "WF_DoctypeVersionCleaner. Cleaning context waiting in the route...";
		if ($aRoute[0]["value"]["wait"]["value"] == null) {
			echo "success<br/>\n";
		}
		else {
			echo "failure<br/>\n";
		}
		
		echo "<hr/>";
	}

	
}

//WF_DoctypeVersionCleaner::example();