<?php
include_once "modules/media/library/MD_Media.php";
class WF_Fields
{
	const INTEGER 	= "int";

	const STRING 	= "string";

	const FLOAT 	= "float";
	
	const TEXT	 	= "text";
	
	const REFERENCE = "reference";
	
	const DATE		= "date";
	
	const BOOLEAN	= "bool";
	
	const STRUCTURE	= "structure";
	
	const ENUM		= "enum";
	
	const PICTURE	= "picture";
	
	const FILE	= "file";
	
	private $aFields = null;

	function __construct($sXml)
	{
		foreach (new MSF_XmlWrapper($sXml) AS $Data) {
			$this->aFields[$Data->getName()] = array (
					"title" 	=> $Data->getTitle(),
					"type" 		=> $Data->getValue("type"),
					"required"	=> (boolean) $Data->getValue("required"),
					"value"		=> $Data->getValue("defaultValue"),
					"default" 	=> $Data->getValue("defaultValue"),
					"optional" 	=> $Data->getValue("optional")
					);
					
		}
		
	}
	
	public function getCount()
	{
		return count($this->aFields);
	}
	
	public function getAsList()
	{
		$i 			= 0;
		$aResult 	= array();
		foreach ($this->aFields AS $sName => $aValue) {
			$aResult[$i] = array(
					"name"		=> $sName,
					"type"		=> $aValue["type"],
					"value" 	=> $aValue["value"],
					"title"		=> $aValue["title"],
					"tag" 		=> "{" . $aValue["title"] . "}",
					"optional"	=> $aValue["optional"]
					);
			$i++;
					
		}

		return $aResult;
	}
	
	public function getRequiredFieldsList()
	{
		$aResult = array();
		foreach ($this->aFields AS $sName => $aValue) {
			if ($aValue["required"]) {
				$aResult[] = $sName;		
			}
						
		}
		
		return $aResult;
	}
	
	public function getAsHash()
	{
		$aResult 	= array();
		foreach ($this->aFields AS $sName => $aValue) {
			$aResult[$sName] = $aValue["value"];					
		}

		return $aResult;
	}
	
	public function exists($sName)
	{
		return array_key_exists($sName, $this->aFields);
	}
	
	public function set($sName, $mValue) {		
		if ($this->exists($sName)) {
			switch ($this->aFields[$sName]["type"]) {
				case self::INTEGER:
					$this->aFields[$sName]["value"] = $this->setInteger($this->aFields[$sName], $mValue);
					break;
					
				case self::FLOAT:
					$this->aFields[$sName]["value"] = $this->setFloat($this->aFields[$sName], $mValue);
					break;
				
				case self::ENUM:
				case self::TEXT:
				case self::STRING:
					$this->aFields[$sName]["value"] = $this->setString($this->aFields[$sName], $mValue);
					break;
					
				case self::BOOLEAN:
					$this->aFields[$sName]["value"] = $this->setBoolean($mValue);
					break;
					
				case self::REFERENCE:
					$this->aFields[$sName]["value"] = $this->setReference($this->aFields[$sName], $mValue);
					break;
					
				case self::STRUCTURE:
					$this->aFields[$sName]["value"] = $this->setStructure($this->aFields[$sName], $mValue);
					break;
					
				case self::DATE:
					$this->aFields[$sName]["value"] = $this->setDate($this->aFields[$sName], $mValue);
					break;	
					
				case self::PICTURE:
					// Проверка, загружен файл через интерфейс или нет
					if(is_array($mValue)) {
						$this->aFields[$sName]["value"] = $this->setPicture($this->aFields[$sName], $mValue);
					}
					else {
						$this->setInteger($this->aFields[$sName], $mValue);
					}
					break;
	
				case self::FILE:
					// Проверка, загружен файл через интерфейс или нет
					//if(is_array($mValue)) {
						$this->aFields[$sName]["value"] = $this->setPicture($this->aFields[$sName], $mValue);
					//}
					//else {
					//	$this->setInteger($this->aFields[$sName], $mValue);
					//}
					
					break;
						
				default:
					$mValue = $this->aFields[$sName]["default"];
					break;
				
			}

		}
		
	}
	
	public function get($sName)
	{
		return $this->exists($sName) ? $this->aFields[$sName]["value"] : null;
	}
	
	private function setInteger($aField, $mValue)
	{
		$iValue = (int) $mValue;
		if ($iValue == 0) {
			$iValue = $aField["default"];
		}
		
		return $iValue;
	}
	
	private function setFloat($aField, $mValue)
	{
		$fValue = (float) $mValue;
		if (strlen($fValue) == 0) {
			$fValue = $aField["default"];
		}
		
		return $fValue;
	}
	
	private function setString($aField, $mValue)
	{
		$sValue = (string) $mValue;
		if (strlen($sValue) == 0) {
			$sValue = $aField["default"];
		}
		
		return $sValue;
	}
	
	private function setBoolean($mValue)
	{
		return (int) (boolean) $mValue;
	}
	
	private function setReference($aField, $mValue)
	{
		$RefValue = new REF_Value();
		$RefValue->parse($mValue);
		return $RefValue->format();
	}
	
	private function setStructure($aField, $mValue)
	{
		$RefValue = new REF_Value();
		if (strlen($mValue) > 0) {
			$RefValue->parse($mValue);
		}
		
		return $RefValue->format();
	}
	
	private function setDate($aField, $mValue)
	{
		if (is_array($mValue)) {
			 $iDate = 	$mValue["year"] . 
						($mValue["month"] < 10 ? "0" . $mValue["month"] : $mValue["month"]) .
						($mValue["day"] < 10 ? "0" . $mValue["day"] : $mValue["day"]);
		 
		}
		else {
			$iDate = $mValue;
		}
		
		$oDf = new MSF_Date();
		$oDf->parse($iDate);
		return $oDf->formatDate();
	}

	private function setPicture($aField, $mValue)
	{
		if (is_array($mValue)) {
			if ($mValue["size"] > 0) {
				$oMedia     = new MD_Media(new MD_MediaDB(MSF_Registry::get("MSF_DB")));
				$oMedia->setUploadDirectory(CNM_MEDIA_UPLOAD_DIRECTORY);
				$oMedia->setMaxFileSize(CNM_MEDIA_MAX_FILE_SIZE);
				$iId = (int) $oMedia->upload($mValue, "workflow/documents", 0);
			}
			else {
				$iId = 0;
			}

		}
		else {
			$iId = (int) $mValue;
		}

		return $iId;
	}

}

?>