<?php

defined("__MSF") or die("Direct access is not allowed");

class WF_Values
{
	private $Document;
	
	private $DocumentVersion;
	
	private $DoctypeVersion;
	
	private $Route;
	
	private $iPointId;
	
	private $sStructure;
	
	private $UBox;
	
	private $sRowDelim = "|^|row|^|";
	
	private $sColDelim = "/^/col/^/";
	
	private static $aCache = array();
	
	private $aExtraStrIds = array();
	
	public function __construct(WF_Document $Document, WF_DocumentVersion $DocumentVersion, WF_DoctypeVersion $DoctypeVersion)
	{
		$this->UBox 			= MSF_Registry::get("MSF_UserBox");
		$this->Document 		= $Document;
		$this->DocumentVersion 	= $DocumentVersion;
		$this->DoctypeVersion 	= $DoctypeVersion;
		$this->Route 			= new MSF_XmlWrapper($DoctypeVersion->getRouteXml());
		$this->iPointId 		= $Document->getPointId();
	}
	
	public function getStructureValues()
	{
		return $this->sStructure;
	}
	
	public function toStructureIds($sTypeValue, $bWithPosts = false)
	{
		if (is_array($sTypeValue)) {
			$sTypeValue = $sTypeValue["name"];
		}
		$aTmp 		= array();
		$aResult 	= array();
		$this->aExtraStrIds = array();
		$this->sStructure 	= "";
		switch ($sTypeValue) {
			case "user":
				$aTmp = $this->getUser($bWithPosts);
				break;
			case "author":
				$aTmp = $this->getAuthor($bWithPosts);
				break;
			case "all":
				$aTmp = $this->getAll($bWithPosts);
				break;
			case "any":
			case "current":
				$aTmp = array($this->UBox->getCurrentStructureId());
				break;
			case "null":
				break;
			default:
				if (substr_count($sTypeValue, "field_") == 0) {
					$aTmp = $this->getStructure($sTypeValue, $bWithPosts);
				}
				else {
					$aTmp = $this->getFromFields($sTypeValue, $bWithPosts);
				}
				break;
		}
		
		for ($i = 0; $i < count($aTmp); $i++) {
			if ($aTmp[$i] != 0) {
				$aResult[] = $aTmp[$i];
			}
		}
		
		return $aResult;
	}
	
	//for delegate begin
	public function getPostIds($sType, $mValue)
	{
		if(is_array($mValue)) {
			$mValue = $mValue['name'];
		}
		
		$aResult 	= array();
		switch ($sType) {
			case 'structure':
				$aResult = $this->calculatePostIds($mValue);
				break;
				
			case 'groups':
				$aResult = $this->calculateGroups($mValue);
				break;
			
			case 'vars':
				if($mValue == 'author') {
					$aResult = array($this->Document->getStructureId());
				}
				else if($mValue == 'any' || $mValue == 'current') {
					$aResult = array($this->UBox->getCurrentStructureId());
				}
				else if($mValue == 'all') {
					$aResult = $this->getAll(false);
				}
				
				break;
				
			case 'fields':
				$aResult = $this->calculateFields($mValue);
				break;
				
			default:
				break;
		}
		
		return $aResult;
	}
	
	private function calculatePostIds($sValue)
	{
		$oDbCache = new MSF_DbCache();
		$aPosts = $oDbCache->get('posts');
		$oValue = new REF_Value();
		$oValue->parse($sValue);
		$aResult = array();
		foreach ($oValue->getKeysList() as $iIndex => $aKeys) {
			foreach (MSF_SL::getParam($aPosts, $aKeys[0], array()) AS $sV2) {
				if(!in_array($sV2, $aResult)) {
					$aResult[] = $sV2;
				}
				
			}
			
		}
		
		return $aResult;
	}
	
	public function calculateFields($sFieldName)
	{
		$aResult = array();
		$aFields = $this->DocumentVersion->getFields();
		if (array_key_exists($sFieldName, $aFields)) {
			$aResult = $this->calculatePostIds($aFields[$sFieldName]);
		}
		
		return $aResult;
	}
	
	private function calculateGroups($mValue)
	{
		$aResult = array();
		$oDoctype = new UserReference(new UserReferenceDB(MSF_Registry::get("MSF_DB")));
		$oDoctype->load(316);
		$oValue = new REF_Value();
		$oValue->parse($mValue);
		foreach ($oValue->getKeysList() AS $sV) {
			$aRows = $oDoctype->row($sV)->getAsHash();
			foreach ($this->calculatePostIds($aRows['employee']) AS $sV2) {
				if(!in_array($sV2, $aResult)) {
					$aResult[] = $sV2;
				}
				
			}
			
		}
		
		return $aResult;
	}
	//for delegate end
	
	private function getAuthor($bWithPosts)
	{
		$this->sStructure = $this->DocumentVersion->getSystemField("author_name");
		
		return array($this->Document->getStructureId());
	}
	
	private function getUser($bWithPosts)
	{
		$oFacade			= MSF_OrgStructureFacade::getInstance();
		$iStructureId		= $this->UBox->getCurrentStructureId();
		$this->sStructure	= $this->DocumentVersion->getSystemField("author_name");
		if($this->UBox->getEmployeeAttribute("name") != $oFacade->getEmployeeNameByStructureId($iStructureId)) {
			$iUserId				= $this->UBox->getUserAttribute("id");
			$aUser					= $this->UBox->getUser($iUserId);			
			$aInfo					= $this->UBox->load($iUserId);
			foreach($aInfo["posts"] AS $aItem) {
				if($aItem["employee_ref_value"] == $aUser["employee_ref_value"]) {
					$iStructureId =	$aItem["node_id"];
					break;
				}
				
			}
						
		}
		
		if ($bWithPosts) {
			$RefValue = new REF_Value();
			$RefValue->parse($this->sStructure);
			$this->sStructure = $this->UBox->getEmployeeAttribute("name");		
			$aPost = $this->getPost($this->UBox->getCurrentStructureId());
			$this->sStructure .= " (" . $aPost["value"] . ")";
			$aKey = $RefValue->getKeysList();
			if (!empty($aKey)) {
				$EmployeeId = $aKey[0][1];			
				$this->sStructure .= $this->sColDelim . $iStructureId . $this->sColDelim . $EmployeeId . $this->sColDelim . $aPost["id"];
			}
		}
		
		return array($iStructureId);
	}
	
	private function getPost($iStructureId)
	{
		$aRet = array();
		$aSubj = MSF_OrgStructureFacade::getInstance()->getSubjectByStructureId($iStructureId);
		if (!empty($aSubj)) {
			$aRes = MSF_OrgStructureFacade::getInstance()->getPostBySubjectId($aSubj["subject_id"]);
			
			if (!empty($aRes)) {
				$aRet["id"] 	= $aSubj["subject_id"];
				$aRet["value"] 	= $aRes[0]["value"];
			}
		}
		
		return $aRet;
	}
	
	private function getUBox($iId)
	{
		$aResult = array();
		
		if (!isset(self::$aCache[$iId])) {
			$this->UBox->load($iId, "structure");
			$aResult = $this->UBox->getLoadedResult();
			self::$aCache[$iId] = $aResult;
		}
		else {
			$aResult = self::$aCache[$iId];
		}
		
		return $aResult;
	}
	public function getRefGroupNames($sTypeValue) {
		$oUr = new UserReference(
				new UserReferenceDB(
						MSF_Registry::get("MSF_DB")
						)
				);
		$oUr->load(316); //Groups Reference Id

		$RefValue = new REF_Value();
		$RefValue->parse($sTypeValue);
		$aKeys = $RefValue->getKeysList();
		$aCols = $oUr->getAsArray();

		$aType = array();
		$aVals = array();
		
		for ($i = 0; $i < count($aCols); $i++) {
			$iSearchKey = array_search($aCols[$i]["id"], $aKeys);
			if ($iSearchKey !== false) {
				$aVals[$iSearchKey] = $aCols[$i];
			}
		}
		foreach ($aVals as $aValue) {
			if (isset($aValue["employee"])) {
				$aTmp[] = $aValue["employee"];
			}
		}
		$sResult = "";
		if (isset($aTmp)) {
			$sResult = implode($this->sRowDelim, $aTmp);
		}
		return $sResult;
	}
	public function getRefGroupIds($sTypeValue, $iStructureId) {
		$oUr = new UserReference(
				new UserReferenceDB(
						MSF_Registry::get("MSF_DB")
						)
				);
		$oUr->load(316); //Groups Reference Id

		$RefValue = new REF_Value();
		$RefValue->parse($sTypeValue);

		$aKeys = $RefValue->getKeysList();

		$aCols = $oUr->getAsArray();

		$aType = array();
		$aVals = array();
		
		for ($i = 0; $i < count($aCols); $i++) {
			$iSearchKey = array_search($aCols[$i]["id"], $aKeys);
			if ($iSearchKey !== false) {
				$aVals[$iSearchKey] = $aCols[$i];
			}
		}
		$aResult = array();
		foreach ($aVals as $aValue) {
			$aTmp = array();
			if (isset($aValue["employee"])) {
				$aTmp = $this->getStructure($aValue["employee"]);
			}
			for ($i = 0; $i < count($aTmp); $i++) {
				if ($iStructureId) {
					if ($aTmp[$i] == $iStructureId) {
						if (!in_array($aTmp[$i], $aResult)) $aResult[] = $aTmp[$i];
					}
				}
				else {
					if ($aTmp[$i] != 0) {
						if (!in_array($aTmp[$i], $aResult)) $aResult[] = $aTmp[$i];
					}			
				}
			}
		}
		return $aResult;		
	}
	
	public function getStructure($sValue, $bWithPosts = false)
	{
		$RefValue = new REF_Value();
		$RefValue->parse($sValue);
		$aResult = array();
		$aVals = $RefValue->getValuesList();
		
		$NewRefValue = new REF_Value();
		foreach ($RefValue->getKeysList() as $iIndex => $aKeys) {
			if (empty($aKeys)) {
				continue;
			}
			$aTmp = $this->getUBox($aKeys[0]);
			foreach ($aTmp as $aItem) {
				if (is_array($aItem)) {
					if (!isset($aItem["employee"]["name"])) {
						continue;
					}
					$aResult[] = $aItem["currentStructureId"];
					
					if (!empty($aItem["extraStructureIds"])) {
						$this->aExtraStrIds = array_merge($this->aExtraStrIds, $aItem["extraStructureIds"]);
					}
					
					if (count($aKeys) == 3) {
						$sToRefValue = $aItem["employee"]["name"];
						
						if ($bWithPosts) {
							foreach ($aItem["posts"] as $aValue) {
								if ($aValue["id"] == $aKeys[2]) {
									$sToRefValue .= " (" . $aValue["post"] . ")";
									break;
								}
							}
						}
						
						$NewRefValue->add($aKeys, $sToRefValue);
					}
				}
			}
			
			if (count($aKeys) != 3) {
				$NewRefValue->add($aKeys, $aVals[$iIndex]);
			}
		}
		
		$this->sStructure .= $NewRefValue->format();
		
		return $aResult;
	}
	
	public function getExtraStructureIds()
	{
		return $this->aExtraStrIds;
	}
	
	public function getFromFields($sFieldName, $bWithPosts = false)
	{
		$aResult = array();
		$aFields = $this->DocumentVersion->getFields();
		if (isset($aFields[$sFieldName])) {
			$this->sStructure	= $aFields[$sFieldName];
			$aValues 			= explode($this->sRowDelim, $aFields[$sFieldName]);
		}
		else {
			$aValues = array();
		}
		
		//$this->sStructure = $aFields[$sFieldName];
		
		for ($i = 0; $i < count($aValues); $i++) {
			if (empty($aValues[$i])) {
				continue;
			}
			
			//$aTmp = explode($this->sColDelim, $aValues[$i]);
			//$aResult[] = trim($aTmp[1]);
			$aResult = array_merge($aResult, $this->getStructure($aValues[$i], $bWithPosts));
		}
		
		return $aResult;
	}
	
	private function getAll($bWithPosts)
	{
		$aResult = array();
		
		foreach ($this->Route as $iPoint => $Data) {
			if ($iPoint == $this->iPointId) {
				$aValues 	= $Data->getValue();
				$aDelegate 	= $aValues["delegate"];
				$aWait 		= $aValues["wait"];
				
				if (isset($aWait["spa_id"])) {
					for ($i = 0; $i < count($aDelegate); $i++) {
						if ($aWait["spa_id"] == $aDelegate[$i]["spa_id"]) {
							if (is_array($aDelegate[$i]["value"])) {
								$aDelegate[$i]["value"] = $aDelegate[$i]["value"]["name"];
							}
							switch ($aDelegate[$i]["value"]) {
								case "author":
									$aResult = array_merge($aResult, $this->getAuthor($bWithPosts));
									break;
								default:
									if (substr_count($aDelegate[$i]["value"], "field_") == 0) {
										$aResult = array_merge($aResult, $this->getStructure($aDelegate[$i]["value"], $bWithPosts));
									}
									else {
										$aResult = array_merge($aResult, $this->getFromFields($aDelegate[$i]["value"], $bWithPosts));
									}
									break;
							}
						}
					}
				}
			}
		}
		
		return $aResult;
	}
}

?>