<?php
defined('__MSF')
	or die("Direct access is not allowed");

include_once "modules/workflow/library/document/WF_DocumentNotCreatedException.php";
include_once "modules/workflow/library/document/WF_DocumentNotSavedException.php";
include_once "modules/workflow/library/document/WF_DocumentNotExistedException.php";
include_once "modules/workflow/library/document/WF_DocumentNotRemovedException.php";
include_once "modules/workflow/library/logs/WF_Log.php";

include_once "modules/workflow/library/documentVersion/WF_DocumentVersionDB.php";
include_once "modules/workflow/library/documentVersion/WF_DocumentVersion.php";

include_once "modules/workflow/library/actions/WF_DocCache.php";

/**
 * Document
 * @copyright Copyright (c) 2007, Detrix
 */
class WF_Document
{
	private $DB;

	private $DoctypeVersion;

	private $iDoctypeVersion;

	private $iDocumentId;

	private $iDocumentVersionId;

	private $iParentId;

	private $iPointId;
	
	private $iStructureId;
	
	private $sFamilyTree;
	
	private $sRegistrationNumber;
	
	private $sTitle;

	private $iMediaId				= 0;
	
	/**
	 * @var WF_Log
	 */
	private $Log					= null;

	public function setRegistrationNumber($sRegistrationNumber)
	{
		$this->sRegistrationNumber = $sRegistrationNumber;
	}
	
	public function getRegistrationNumber()
	{
		return $this->sRegistrationNumber;
	}
	/////////////////////////////////////////////////////////////////////////////////////////////////
	//
	// Setters
	//
	/////////////////////////////////////////////////////////////////////////////////////////////////

	public function setFamilyTree($sFamilyTree)
	{
		$this->sFamilyTree = $sFamilyTree;
	}
	
	public function getFamilyTree()
	{
		return $this->sFamilyTree;
	}
	
	public function setTitle($sTitle)
	{
		$this->sTitle = $sTitle;
	}
	
	
	public function getTitle()
	{	
		return MSF_Registry::get("MSF_Locale")->getI18N($this->sTitle);
	}
	
	public function getRowTitle()
	{	
		return html_entity_decode(MSF_Registry::get("MSF_Locale")->getI18N($this->sTitle), ENT_COMPAT, "UTF-8");
	}	
	
	
	public function setDocumentVersionId($iDocumentVersionId)
	{
		$this->iDocumentVersionId = (int) $iDocumentVersionId;
	}
	
	public function setStructureId($iStructureId)
	{
		$this->iStructureId = (int) $iStructureId;
	//	debug($this->iStructureId);
	}

	public function setPointId($iPointId)
	{
		$this->iPointId = (int) $iPointId;
	}

	public function setParentId($iParentId)
	{
		$this->iParentId = (int) $iParentId;
	}
	
	public function setMediaId($iMediaId)
	{
		$this->iMediaId = (int) $iMediaId;
	}
	
	/////////////////////////////////////////////////////////////////////////////////////////////////
	//
	// Getters
	//
	/////////////////////////////////////////////////////////////////////////////////////////////////
	public function getDoctype()
	{
		return $this->DoctypeVersion->getDoctype();
	}
	public function getDoctypeVersion()
	{
		return $this->DoctypeVersion;
	}

	public function getDocumentId()
	{
		return $this->iDocumentId;
	}

	public function getDoctypeId()
	{
		return $this->DoctypeVersion->getDoctypeId();
	}
	
	public function getDoctypeVersionId()
	{
		return $this->DoctypeVersion->getDoctypeVersionId();
	}
	
	public function getStructureId()
	{
		return $this->iStructureId;
	}

	public function getPointId()
	{
		return $this->iPointId;
	}

	public function getDocumentVersionId()
	{
		return $this->iDocumentVersionId;
	}

	public function getParentId()
	{
		return $this->iParentId;
	}

	public function getDocumentVersionIds()
	{
		$aResult = array();
		if ($this->exists()) {
			$aVersions = $this->DB->selectDocumentVersionsIds($this->iDocumentId);
			for ($i = 0; $i < count($aVersions); $i++) {
				$aResult[] = $aVersions[$i]["document_version_id"];
			}

		}

		return $aResult;
	}
		
	public function getMediaId()
	{
		return $this->iMediaId;
	}
	
	/////////////////////////////////////////////////////////////////////////////////////////////////
	//
	// Public Doers
	//
	/////////////////////////////////////////////////////////////////////////////////////////////////

	public function __construct($DoctypeVersion, $iDocumentId = 0)
	{
		$this->DoctypeVersion		= $DoctypeVersion;
		$this->reset();
		$this->iDocumentId 			= $iDocumentId;
		$this->iDoctypeId			= $DoctypeVersion->getDoctypeId();
		$this->iDoctypeVersionId	= $DoctypeVersion->getDoctypeVersionId();
		$this->DB 					= new WF_DocumentDB(MSF_Registry::get("MSF_DB"));
		if ($this->iDocumentId > 0) {
			if ($this->exists()) {
				$aDocument = $this->DB->select(
						$this->DoctypeVersion->getDoctypeId(),
						$this->DoctypeVersion->getDoctypeVersionId(),
						$this->iDocumentId
						);

				$this->iStructureId			= $aDocument["structure_id"];
				$this->iDocumentVersionId	= $aDocument["document_version_id"];
				$this->iParentId			= $aDocument["parent_id"];
				$this->iPointId				= $aDocument["point_id"];
				$this->iMediaId				= $aDocument["media_id"];
				$this->sFamilyTree			= $aDocument["family_tree"];
				$this->sTitle				= $aDocument["title"];
				$this->sRegistrationNumber	= $aDocument["registration_number"];
				$this->Log					= new WF_Log($this->iDocumentId, $aDocument["log"]);
			}
			else {
				throw new WF_DocumentNotExistedException($this->iDocumentId);
			}

		}
		else {
			$this->Log					= new WF_Log($this->iDocumentId, array());
		}

	}

	public function exists()
	// throws MSF_DBException
	{
		return ($this->DB->selectCount($this->iDocumentId) == 1) ? true : false;
	}

	public function save()
	// MSF_DBExceprion, WF_DocumentNotExistedException
	{		
		$oCache = new WF_DocCache($this->getDoctypeId(), $this->getDocumentId());
		$oCache->remove();
		
		if ($this->exists()) {
			$this->DB->update(
					$this->iDocumentId,
					$this->iDocumentVersionId,
					$this->iPointId,
					$this->sTitle,
					$this->sRegistrationNumber,
					$this->Log,
					$this->iStructureId
					);

		}
		else if ($this->DoctypeVersion->exists()) {
			// debug("a");
			// exit();
			$this->iDocumentId = $this->DB->insert(
					$this->DoctypeVersion->getDoctypeId(),
					$this->DoctypeVersion->getDoctypeVersionId(),
					$this->iDocumentVersionId,
					$this->iParentId,
					$this->iPointId,
					$this->iStructureId,
					$this->iMediaId,
					$this->sFamilyTree,
					$this->sTitle,
					$this->sRegistrationNumber,
					$this->Log
					);

			if ($this->iDocumentId == 0) {
				$this->reset();
				throw new WF_DocumentNotCreatedException();
			}
			

		}
		else {
			throw new WF_DoctypeNotCreatedException("Doctype not created");
		}

	}

	public function remove()
	{
		$oCache = new WF_DocCache($this->getDoctypeId(), $this->getDocumentId());
		$oCache->remove();
		set_time_limit(30);
		foreach ($this->getDocumentVersionIds() AS $iVersionId) {
			$this->getVersion($iVersionId)->remove();
		}
		
		if (($this->DB->delete($this->iDocumentId) <> 1)) {
			throw new WF_DocumentNotRemovedException($this->iDocumentId);
		}

	}

	/////////////////////////////////////////////////////////////////////////////////////////////////
	//
	// Private Doers
	//
	/////////////////////////////////////////////////////////////////////////////////////////////////

	private function reset()
	{
		$this->iDocumentId 			= 0;
		$this->iDocumentVersionId	= 0;
		$this->iParentId			= 0;
		$this->iPointId				= 0;
		$this->iStructureId 		= 0;
	}
	
	/////////////////////////////////////////////////////////////////////////////////////////////////
	//
	// UNSAFE CODE
	//
	/////////////////////////////////////////////////////////////////////////////////////////////////

	/**
	 * @return WF_DocumentVersion
	 */
	public function newVersion()
	{
		return $this->getVersion(0);
	}

	/**
	 * @return WF_DocumentVersion
	 */
	public function getCurrentVersion()
	{
		$this->aVersions[$this->iDocumentVersionId] = new WF_DocumentVersion(
						$this,
						$this->DoctypeVersion->getExtname(),
						$this->DoctypeVersion->getFieldsXml(),
						$this->DoctypeVersion->getSystemFields()->__toString(),
						$this->iDocumentVersionId
						);
						
		return $this->aVersions[$this->iDocumentVersionId];
	}
	
	private $aVersions = array();

	/**
	 * @return WF_DocumentVersion
	 */
	public function getVersion($iDocumentVersionId)
	{
		if ($this->DoctypeVersion->exists()) {
			if (!array_key_exists($iDocumentVersionId, $this->aVersions)) {
				$this->aVersions[$iDocumentVersionId] = new WF_DocumentVersion(
						$this,
						$this->DoctypeVersion->getExtname(),
						$this->DoctypeVersion->getFieldsXml(),
						$this->DoctypeVersion->getSystemFields()->__toString(),
						$iDocumentVersionId
						);
			}

			return $this->aVersions[$iDocumentVersionId];
		}
		else {
			throw new WF_Exception("Cant getVersion() of a document");
		}

	}
	
	public function hasVersion()
	{
		return $this->DoctypeVersion->exists();
	}
	
	/**
	 * @return WF_Log
	 */
	public function getLog()
	{
		return $this->Log;
	}
	
	public function getDB()
	{
		return $this->DB;
	}

}
?>