<?php

defined("__MSF")
	or die("Direct access is not allowed");

include_once "modules/workflow/library/common/WF_Exception.php";
include_once "modules/workflow/library/document/WF_DocumentNotExistedException.php";
include_once "WF_DocumentLogNotCreatedException.php";
include_once "WF_DocumentLogValueNotCreatedException.php";
include_once "WF_DocumentLogNotRemovedException.php";
include_once "WF_DocumentLogValueNotRemovedException.php";
include_once "WF_IDocumentLogSet.php";
include_once "WF_DocumentLogSetDB.php";
include_once "WF_DocumentLog.php";
include_once "WF_DocumentLogValue.php";
include_once "library/MSF_Utils/MSF_Collection.php";

class WF_DocumentLogSet
{
	private $DB;

	public function __construct(WF_DocumentLogSetDB $DB)
	{
		$this->DB = $DB;
	}

	public function add(WF_IDocumentLog $DocumentLog)
	{
		$iDocumentId 	= $DocumentLog->getDocumentId();
		$iDocumentLogId = $this->DB->existsLog($iDocumentId, $DocumentLog->getActionId());

		if ($iDocumentLogId == 0) {
			$iDocumentLogId = $this->DB->insertLog(
				array(
					"document_id" 	=> $DocumentLog->getDocumentId(),
					"action_id" 	=> $DocumentLog->getActionId(),
					"point_id" 		=> $DocumentLog->getPointId(),
					"displayable" 	=> ($DocumentLog->getDisplayable()) ? "true" : "false"
				)
			);

			if ($iDocumentLogId == 0) {
				throw new WF_DocumentLogNotCreatedException("Document log not created");
			}
		}

		$Iterator = $DocumentLog->getValues()->iterator();

		while ($Iterator->hasNext()) {
			$Values = $Iterator->next();
			$bRows = $this->DB->insertLogValue(
				array(
					"document_log_id" 	=> $iDocumentLogId,
					"employee_id" 		=> $Values->getEmployeeId(),
					"employee_name" 	=> $Values->getEmployeeName(),
					"value" 			=> $Values->getValue(),
					"description" 		=> $Values->getDescription(),
					"created_at" 	=> $Values->getCreatedAt()
				)
			);

			if (!$bRows) {
				throw new WF_DocumentLogValueNotCreatedException("Document log value no created");
			}
		}
	}

	public function load($iDocumentId)
	{
		$aLogs 		= $this->DB->selectLogs($iDocumentId);
		$aValues 	= $this->DB->selectValues($iDocumentId);
		$Log 		= null;
		$Values 	= null;
		$CollValues = null;
		$CollLogs 	= new MSF_Collection();;

		for ($i = 0; $i < sizeof($aLogs); $i++) {
			$Log 		= new WF_DocumentLog();
			$CollValues = new MSF_Collection();

			$Log->setActionId($aLogs[$i]["action_id"]);
			$Log->setDisplayable($aLogs[$i]["displayable"]);
			$Log->setDocumentId($iDocumentId);
			$Log->setDocumentLogId($aLogs[$i]["document_log_id"]);
			$Log->setPointId($aLogs[$i]["point_id"]);

			for ($j = 0; $j < sizeof($aValues); $j++) {
				if ($aValues[$j]["document_log_id"] == $aLogs[$i]["document_log_id"]) {
					$Values = new WF_DocumentLogValue();

					$Values->setDocumentLogId($aLogs[$i]["document_log_id"]);
					$Values->setId($aValues[$j]["document_log_value_id"]);
					$Values->setEmployeeId($aValues[$j]["employee_id"]);
					$Values->setEmployeeName($aValues[$j]["employee_name"]);
					$Values->setValue($aValues[$j]["value"]);
					$Values->setDescription($aValues[$j]["description"]);
					$Values->setCreatedAt($aValues[$j]["created_at"]);

					$CollValues->add($Values);
				}
			}

			$Log->setValues($CollValues);
			$CollLogs->add($Log);
		}

		return $CollLogs;
	}

	public function update($iDocumentId)
	{
		$this->DB->update($iDocumentId);
	}

	public function remove($mDocumentLogIds, $mDocumentLogValueIds = null)
	{
		if ($mDocumentLogValueIds != null) {
			if (!$this->DB->deleteLogValues($mDocumentLogValueIds)) {
				throw new WF_DocumentLogValueNotRemovedException("Document log value not removed");
			}
		}
		else {
			if (!$this->DB->deleteLogs($mDocumentLogIds)) {
				throw new WF_DocumentLogNotRemovedException("Document log not removed");
			}
		}
	}
}

?>