<?php
defined("__MSF")
		or die("Direct access is not allowed");

include_once "modules/workflow/library/documentVersion/WF_DocumentVersionNotCreatedException.php";
include_once "modules/workflow/library/documentVersion/WF_DocumentVersionNotRemovedException.php";
include_once "modules/workflow/library/documentVersion/WF_DocumentVersionNotExistedException.php";
include_once "modules/workflow/library/doctypeVersion/WF_SystemFieldsSettings.php";
include_once "library/MSF_SearchUtil/MSF_SearchUtil.php";
include_once "modules/references/library/Reference/REF_Fields.php";
include_once "modules/workflow/library/document/WF_DocumentNotExistedException.php";
include_once 'library/MSF_SearchUtil/MSF_CsIndexedDB.php';

class WF_DocumentVersion
{
	private $DB					= null;
	
	private $Document			= null;
	
	private $Fields				= null;
	
	/**
	 * @var REF_Fields
	 */
	private $SystemFields 		= null;
	
	private $iDoctypeVersionId	= 0;

	private $iDocumentId		= 0;

	private $iDocumentVersionId	= 0;

	private $iStructureId		= 0;

	private $sEmployeeName		= "Noname Employee";

	private $sCreatedAt			= "1970-01-01 00:00:00";
	
	private $sExtname			= "";
	
	/////////////////////////////////////////////////////////////////////////////////////////////////
	//
	// Setters
	//
	/////////////////////////////////////////////////////////////////////////////////////////////////

	public function setFields($aFields)
	{
		foreach ($aFields AS $sName => $mValue) {
			$this->Fields->set($sName, $mValue);
		}
	}
	
	public function setSystemFields($aFields)
	{
		foreach ($aFields AS $sName => $mValue) {
			$this->SystemFields->set($sName, $mValue);
		}
	}
	
	public function setStructureId($iStructureId)
	{
		$this->iStructureId = $iStructureId;
	}
	
	public function setEmployeeName($sEmployeeName)
	{
		$this->sEmployeeName = $sEmployeeName;
	}

	public function setCreatedAt($sCreatedAt)
	{
		$DateFormat = new MSF_Date();
		$this->sCreatedAt = $DateFormat->parseAndFormat($sCreatedAt);
	}
	
	public function setSystemField($sFieldName, $mValue)
	{
		$this->checkSystemParam($sFieldName);
		$this->SystemFields->set($sFieldName, $mValue);
	}
	
	/////////////////////////////////////////////////////////////////////////////////////////////////
	//
	// Getters
	//
	/////////////////////////////////////////////////////////////////////////////////////////////////
	
	public function getDoctypeId()
	{
		return $this->DoctypeVersion->getDoctypeId();
	}
	
	public function getDoctypeVersionId()
	{
		return $this->iDoctypeVersionId;
	}
	
	public function getDocumentVersionId()
	{
		return $this->iDocumentVersionId;
	}
	
	public function getDocumentId()
	{
		return $this->Document->getDocumentId();
	}
        
        public function getFDocumentId(){
            return $this->iDocumentId;
        }
	
	public function getRefFields()
	{
		return $this->Fields;
	}

	public function getFields()
	{
		return $this->Fields->getAsHash();
	}
	
	public function getSystemFields()
	{
		return $this->SystemFields->getAsHash();
	}
	
	public function getCreatedAt()
	{
		return $this->sCreatedAt;
	}
	
	private function getExtName()
	{
		return $this->sExtName;
	}
	
	public function getSystemField($sFieldName)
	{
		$this->checkSystemParam($sFieldName);
		$aHash = $this->SystemFields->getAsHash();
		return $aHash[$sFieldName];
	}

	/////////////////////////////////////////////////////////////////////////////////////////////////
	//
	// Public Doers
	//
	/////////////////////////////////////////////////////////////////////////////////////////////////

	public function __construct($Document, $sExtname, $sFieldsXml, $sSystemFields, $iDocumentVersionId)
	{
		if (!$Document->exists()) {
			$Document->save();
		}
		
		$SystemFields 				= new WF_SystemFieldsSettings();
		
		$SystemFields->bindPoints($Document->getDoctypeVersion()->getRouteXml());
		//debug(MSF_XmlWrapper::xmlToArray($sSystemFields));
		$DB							= MSF_Registry::get("MSF_DB");
		$this->DB 					= new WF_DocumentVersionDB($DB);
		$this->Document 			= $Document;
		$this->Fields				= new REF_Fields($sFieldsXml);
		$this->SystemFields 		= new REF_Fields($sSystemFields);
		$this->Fields->setExtra("workflow_document", $Document->getMediaId());
		$this->reset();
		$this->iDocumentVersionId 	= $iDocumentVersionId;
		$this->sExtname				= $sExtname;
		if ($this->iDocumentVersionId > 0) {
			if ($this->exists()) {
				$aDocumentVersion 			= $this->DB->select($iDocumentVersionId);
				$this->iDocumentVersionId 	= $aDocumentVersion["document_version_id"];
				$this->iDocumentId 			= $aDocumentVersion["document_id"];
				$this->iStructureId 		= $aDocumentVersion["structure_id"];
				$this->sEmployeeName 		= $aDocumentVersion["employee_name"];
				$this->sCreatedAt 			= $aDocumentVersion["created_at"];
	
				$aTmpFields = $this->DB->selectDocument($this->sExtname, $this->iDocumentVersionId);
				unset($aTmpFields["document_version_id"]);
				
				$aFields 		= array();
				$aSystemFields 	= array();
				
				foreach ($aTmpFields as $sKey => $sValue) {
					$bSystem = false;
					for ($j = 0; $j < $SystemFields->size(); $j++) {
						if ($sKey == $SystemFields[$j]["name"]) {
							$bSystem = true;
							break;
						}
					}
					
					if ($bSystem) {
						$aSystemFields[$sKey] = $sValue;
					}
					else {
						$aFields[$sKey] = $sValue;
					}
				}
				
				$this->setFields($aFields);
				$this->setSystemFields($aSystemFields);
			}
			else {
				throw new WF_DocumentVersionNotCreatedException($this->iDocumentVersionId);
			}
			
		}
		
	}
	
	public function exists()
	{
		return ($this->DB->selectCount($this->iDocumentVersionId) == 1) ? true : false;
	}
	
	public function save()
	{
		$DB 	= MSF_Registry::get("MSF_DB");
		if ($this->exists()) {			
			$this->DB->updateDocument(
				$this->sExtname,
				$this->iDocumentVersionId,
				$this->Fields->getAsList(),
				$this->SystemFields->getAsList()
			);
			$this->Document->setTitle($this->buildTitle());
			$this->Document->save();
			// index doc
			$DBRecord = new MSF_CsIndexedDB('cs_indexed', 0);
			if (!$DBRecord->findOperation($this->iDocumentId, 'workflow', 'index', $this->Document->getDoctype()->getDoctypeId())) {
				$DBRecord->insert(array('subject_id' => $this->iDocumentId,
										'module' => 'workflow', 
										'operation' => 'index', 
										'context_id' => $this->Document->getDoctype()->getDoctypeId()
								  ));
			}
		}
		else if ($this->Document->exists()) {
			$DB->begin();
			$this->iDocumentVersionId = $this->DB->insert (
					$this->iDocumentId,
					$this->iStructureId,
					$this->sEmployeeName,
					$this->sCreatedAt
					);
	
			if ($this->iDocumentVersionId > 0) {
				//$this->Document->setTitle();
				$this->Document->setTitle($this->buildTitle());
				$this->DB->insertDocument(
						$this->sExtname,
						$this->iDocumentVersionId,
						$this->Fields->getAsList(),
						$this->SystemFields->getAsList()
						);
						
				$DB->commit();
				$this->Document->setDocumentVersionId($this->iDocumentVersionId);
				$this->Document->save();
				// index doc
				$DBRecord = new MSF_CsIndexedDB('cs_indexed', 0);
				if (!$DBRecord->findOperation($this->iDocumentId, 'workflow', 'index', $this->Document->getDoctype()->getDoctypeId())) {
					$DBRecord->insert(array('subject_id' => $this->iDocumentId,
											'module' => 'workflow', 
											'operation' => 'index', 
											'context_id' => $this->Document->getDoctype()->getDoctypeId()
									  ));
				}		
			}
			else {
				$DB->rollBack();
				$this->reset();
				throw new WF_DocumentVersionNotCreatedException("Document version not created");
			}
			
		}
		else {
			throw new WF_DocumentNotExistedException("Document not exists");
		}

	}
	

	
	public function getUnfilledFields()
	{
		$aResult 	= array();
		$aUnfilled 	= $this->Fields->getUnfilled();
		foreach ($aUnfilled AS $aBody) {
			$aResult[] = $aBody["title"];
		}

		return $aResult;
	}
	
	public function getIncorrectFields()
	{
		$aResult 		= array();
		$aIncorrect 	= $this->Fields->getIncorrect();
		foreach ($aIncorrect AS $aBody) {
			$aResult[] = $aBody["title"];
		}

		return $aResult;
	}
	
	public function getFieldsAsList()
	{
		return $this->Fields->getAsList();
	}
	
	public function getSystemFieldsAsList()
	{
		return $this->SystemFields->getAsList();
	}
	
	public function buildTitle()
	{
		$sDoctypeTitle	= $this->Document->getDoctypeVersion()->getDoctype()->getTitle();
		//$sCreatedAt		= $this->getSystemField("created_at");
		//$sAuthor		= $this->getSystemField("author_name");
		//$Date 			= new MSF_Date();
		//$RefValue 		= new REF_Value();
		//$RefValue->parse($sAuthor);
		//$aAuthor 		= $RefValue->get(0);
		//$sAuthor 		= $aAuthor["value"];
		//$sTitle 		= $sDoctypeTitle . " (" . $sAuthor . ", " . $Date->parseAndFormat($sCreatedAt, CNF_DATE_FORMAT) . ")";
		
		$aTitle = array();
		
		foreach ($this->Fields->getAsList() as $aItem) {
			if (isset($aItem["intitle"]) && $aItem["intitle"]) {
				if (mb_strlen($aItem["value"]) > 0) {
					switch ($aItem["type"]) {
						case "reference":
						case "structure":
						case "document":
							$RefValue = new Ref_Value();
							$RefValue->parse($aItem["value"]);
							$aItem["value"] = htmlspecialchars(implode(", ", $RefValue->getValuesList()));
							break;
						case "enum":
						case "bool":
						case "boolean":
							$Enum 	= new MSF_EnumValue();
							$aEnum 	= $Enum->parse($aItem["value"]);
							
							if (isset($aEnum["value"])) {
								$aItem["value"] = htmlspecialchars($aEnum["value"]);
							}
							else {
								$aItem["value"] = "";
							}
							break;
						case "date":
							$Date = new MSF_Date();
							$Date->parse($aItem["value"]);
							$aItem["value"] = $Date->format(CNF_DATE_FORMAT);
							break;
						case "datetime":
						case "timestamp":
							$Date = new MSF_Date();
							$Date->parse($aItem["value"]);
							$aItem["value"] = $Date->format(CNF_DATE_TIME_FORMAT);
							break;
						case "string":
							$aItem["value"] = htmlspecialchars($aItem["value"]);
							break;
					}
					$aTitle[]= $aItem["value"];
				}
			}
		}
		$sTitle = implode($aTitle, "; ");
		$sTitle = ($sTitle == "") ? $sDoctypeTitle : $sDoctypeTitle . " (" . $sTitle . ")";
		
		return $sTitle;
	}
	
	public function remove($bArchiving = false)
	{
		$DB = MSF_Registry::get("MSF_DB");
		$oMedia = new MD_Media(new MD_MediaDB($DB));
		$oDir	= new MSF_Dir();
		$DB->begin();
		
		foreach ($this->Fields->getAsList() AS $aBody) {
			if (($aBody["type"] == "file") || ($aBody["type"] == "picture") || ($aBody["type"] == "text")) {
				switch ($aBody["type"]) {
					case "file":
					case "picture":
						$aFiles = explode(',', $aBody['value']);
						break;
					case "text":
						preg_match_all('!"/media/(image|download)/(\d{1,})"!', $aBody['value'], $aSearch);
						$aFiles = $aSearch[2];
						break;
				}
				if (!empty($aFiles) && !$bArchiving) {
					foreach ($aFiles AS $sFile) {
						try {
							$aInfo		= $oMedia->getFileInfo($sFile);
							$oMedia->remove($sFile);
							$aDir		= $oDir->read($aInfo["directory"]);
							if (empty($aDir)) {
								$aInfoHour	= pathinfo($aInfo["directory"]);
								$oDir->remove($aInfo["directory"]);
								$aDirHour	= $oDir->read($aInfoHour["dirname"]);
								if (empty($aDirHour)) {
									$aInfoDay	= pathinfo($aInfoHour["dirname"]);
									$oDir->remove($aInfoHour["dirname"]);
									$aDirDay	= $oDir->read($aInfoDay["dirname"]);
									if (empty($aDirDay)) {
										$oDir->remove($aInfoDay["dirname"]);
									}
								}
							}
						}
						catch (Exception $e) {
							
						}
					}
				}
			}
		}
		if (!$this->DB->delete($this->iDocumentVersionId)) {
			$DB->rollBack();
			throw new WF_DocumentVersionNotRemovedException("Document version not removed");
			
		}
		else if (!$this->DB->deleteFields($this->sExtname, $this->iDocumentVersionId)) {
			$DB->rollBack();
			throw new WF_DocumentVersionNotRemovedException("Document version table rows not removed");
		}
		else {
			$DB->commit();
			$DBRecord = new MSF_CsIndexedDB('cs_indexed', 0);
			if (!$DBRecord->findOperation($this->getDocumentId(), 'workflow', 'remove', $this->Document->getDoctype()->getDoctypeId())) {
				$DBRecord->insert(array('subject_id' => $this->getDocumentId(),
										'module' => 'workflow', 'operation' => 'remove', 'context_id' => $this->Document->getDoctype()->getDoctypeId()));
			}
			//MSF_SearchUtil::newInstance()->removeIndex("wf_index", $this->Document->getDoctype()->getFolderId(), $this->getDocumentId());
		}
				
	}
	
	/////////////////////////////////////////////////////////////////////////////////////////////////
	//
	// Private Doers
	//
	/////////////////////////////////////////////////////////////////////////////////////////////////

	private function reset()
	{
		$this->iDocumentVersionId	= 0;
		$this->iDocumentId			= $this->Document->getDocumentId();
		$this->iStructureId			= 0;
		$this->sEmployeeName		= "Noname Employee";
		$this->iCreatedAt			= 0;
	}
	
	private function checkSystemParam($sKey)
	{
		$aFields = $this->SystemFields->getAsHash();
		
		if (!array_key_exists($sKey, $aFields)) {
			$sMessage = "Key \"" . $sKey . "\" not defined in system fields.\n" .
						"Avaliable keys:\n";
			
			foreach ($aFields as $sKey => $sValue) {
				$sMessage .= $sKey . "\n";
			}
			
			
			throw new WF_Exception($sMessage);
		}
	}
	
	/////////////////////////////////////////////////////////////////////////////////////////////////
	//
	// UNSAFE CODE
	//
	/////////////////////////////////////////////////////////////////////////////////////////////////

	public function setDoctypeVersion(WF_IDoctypeVersion $DoctypeVersion)
	{
		$this->DoctypeVersion = $DoctypeVersion;
	}

	public function getDocument()
	{
		return $this->Document;
	}





	public function getEmployee()
	{
		return $this->sEmployeeName;
	}
	
	public function setField($sName, $mValue) {
        $this->Fields->set($sName, $mValue);
    }
	


}

?>