<?php
if(!defined("__MSF")) {
	die("Direct access is not allowed");
}

require_once "WF_AktenplanDB.php";
require_once "modules/workflow/view/xml/WF_AktenplanXml.php";

class WF_Aktenplan
{
	/**
	 * @var MSF_Recordset
	 */
	private $Recordset;
	
	/**
	 * @var WF_AktenplanDB
	 */
	private $DB;
	
	const	AKTENPLAN_REFERENCE_ID	= 14;
	
	public function __construct()
	{
		$this->DB	= new WF_AktenplanDB(MSF_Registry::get("MSF_DB"));
	}
	
	private function buildRecordset()
	{

		
		$aAllowedDoctypes	= WF_Utils::getAllowedAuthorIds(56);
		$aDocumentsIds		= WF_Utils::getAllowedDocumentIds(56);
		$aStrIds			= array();

		foreach($aAllowedDoctypes as $aAccessData) {
			$aView 	= array();
			$aNView = array();			
			for ($i = 0; $i < count($aAccessData); $i++) {
				if ($aAccessData[$i]["view"] == 2) {
					$aNView = array_merge($aNView, $aAccessData[$i]["author_ids"]);
				}
				else {
					$aView = array_merge($aView, $aAccessData[$i]["author_ids"]);
				}
			}
			
			$aStrIds = array_diff($aView, $aNView);
		}
		
		array_push($aStrIds, MSF_Registry::get("MSF_UserBox")->getCurrentStructureId());
		
		$this->Recordset	= new MSF_Recordset(MSF_Registry::get("MSF_DB"));
		$this->Recordset->setColumns($this->prepareColumns());
		$this->Recordset->setTables(array("wf_aktenplan", "wf_documents"));
		$this->Recordset->setDefaultGroups(array("wf_aktenplan.year", "wf_aktenplan.department", "wf_aktenplan.title"));
		$aDocumentsIds[] = 0;
		$this->Recordset->setConditions(array(
			"AND wf_documents.document_id = wf_aktenplan.document_id",
			"AND (wf_documents.structure_id IN (" . implode(",", $aStrIds) . ") " . 
			"OR wf_documents.document_id IN (" . implode(",", $aDocumentsIds) . "))"
		));
		
		$this->Recordset->setBaseURL("/workflow/dossier/?");
 		$this->Recordset->setRpcURL("/workflow/dossier/?");
		
 		$this->Recordset->setStateParams(array(
			"module"		=> "workflow",
			"action"		=> "aktenplan",
			"action_id"		=> "-999",
			"structure_id"	=> MSF_Registry::get("MSF_UserBox")->getCurrentStructureId()
		));
 		
		
		$this->Recordset->setActions(array(
			array(
				"title" 	=> MSF_Registry::get("MSF_Locale")->getWord("WORKFLOW_ADMIN_LIST_ACTION_VIEW"),
				"href" 		=> "/workflow/document/view",
				"image" 	=> "/templates/" . CNF_SKIN . "/images/folder_page_white.png",
				"bind" 		=> array("document_id"),
				"confirm" 	=> false
			),
			array(
				"title" 	=> MSF_Registry::get("MSF_Locale")->getWord("WORKFLOW_ADMIN_LIST_ACTION_VIEW_BLANK"),
				"href" 		=> "/workflow/document/view",
				"image" 	=> "/templates/" . CNF_SKIN . "/images/folder_page_white_blank.png",
				"bind" 		=> array("document_id"),
				"confirm" 	=> false,
				"blank"		=> true
			),
			array(
				"title" 	=> MSF_Registry::get("MSF_Locale")->getWord("WORKFLOW_ADMIN_LIST_ACTION_DELETE"),
				"href" 		=> "/workflow/document/remove",
				"image" 	=> "/templates/" . CNF_SKIN . "/images/red_cross.png",
				"bind" 		=> array("document_id"),
				"confirm" 	=> true
			)
		));
		
		$this->Recordset->setLink(array(
			"href" 	=> "/workflow/document/view",
			"bind" 	=> array("document_id")
		));
	}
	
	public function getRecordset()
	{
		$this->buildRecordset();
		return $this->Recordset->getAllRecords();
	}
	
	public function getGroup()
	{
		$this->buildRecordset();
		return $this->Recordset->getGroup();
	}
	
	private function prepareColumns()
	{
		$aTypes		= array("int", "string", "string", "string", "string", "string", "date");
 		$aTitles 	= array("id", 
                                MSF_Registry::get("MSF_Locale")->getWord("AKTENPLAN_SUBDIVISION"), 
                                MSF_Registry::get("MSF_Locale")->getWord("AKTENPLAN_TITLE"), 
                                MSF_Registry::get("MSF_Locale")->getWord("AKTENPLAN_INDEX"),
                                MSF_Registry::get("MSF_Locale")->getWord("AKTENPLAN_YEAR"),
                                MSF_Registry::get("MSF_Locale")->getWord("AKTENPLAN_REGISTRATION_NUMBER"),
                                MSF_Registry::get("MSF_Locale")->getWord("AKTENPLAN_DATE_OF_REGISTRATION"));
 		$aColumns 	= array("wf_aktenplan.document_id", "wf_aktenplan.department", "wf_aktenplan.title", "wf_aktenplan.index", "wf_aktenplan.year", "wf_documents.registration_number", "wf_aktenplan.registration_date");
 		$aRColumns 	= array();

 		for ($i = 0; $i < sizeof($aTitles); $i++) {
	 		$aRColumns[$i]["name"] 		= $aColumns[$i];
	 		$aRColumns[$i]["title"] 	= $aTitles[$i];
			$aRColumns[$i]["type"] 		= $aTypes[$i];
	 		$aRColumns[$i]["sort"] 		= true;
	 		$aRColumns[$i]["filter"] 	= true;
	 		$aRColumns[$i]["group"] 	= true;
	 		$aRColumns[$i]["compare"] 	= "like";
	 		$aRColumns[$i]["visible"] 	= $aColumns[$i] == "wf_aktenplan.document_id" ? false : true;
 		}

 		return $aRColumns;
	}
	
	public function addDocument($iDocumentId, $sRegNumber, $sAktenplan, $sQuestion)
	{
		$RefValue	= new REF_Value();
		$Date		= new MSF_Date();
		
		$Reference	= new UserReference(new UserReferenceDB(MSF_Registry::get("MSF_DB")));
		$Reference->load(self::AKTENPLAN_REFERENCE_ID);
		
		$RefValue->parse($sAktenplan);
		
		$aKeys		= $RefValue->getKeysList();
		$aValues	= $RefValue->getValuesList();
		
		
		for($i=0; $i<$RefValue->getSize(); $i++) {
			$aRow	= $Reference->row($aKeys[$i])->show();
			$this->DB->addDocument(
				$aRow[1]["value"],
				$aRow[2]["value"],
				$aRow[3]["value"],
				$aRow[0]["value"],
				$sRegNumber,
				$iDocumentId,
				$Date->format("Y-m-d")
			);
		}
				
	}
	
	public function removeDocument($iDocumentId)
	{
		$this->DB->removeDocument($iDocumentId);
	}
	
}

?>