<?php
if(!defined("__MSF")) {
	die("Direct access is not allowed");
}

require_once "WF_FolderDB.php";
require_once "WF_FolderNotFoundException.php";
require_once "modules/workflow/view/xml/WF_FolderXml.php";
require_once "modules/workflow/library/doctypeVersion/WF_DoctypeVersionsList.php";
require_once "modules/eventlog/library/ELOG_EventLog.php";

class WF_Folder
{
	/**
	 * @var WF_FolderDB
	 * @access private
	 */
	private $DB				= null;

	/**
	 * @var array
	 * @access private
	 */
	private $aConfig		= array();

	/**
	 * @var int
	 * @access private
	 */
	private $iFolderId		= 0;

	/**
	 * Elsi true - znachit zhurnal noviy i tolko chto sozdan
	 *
	 * @var bool
	 * @access private
	 */
	private $bIsNew			= false;

	/**
	 * Name of Color Field for Workflow Recordset
	 * 
	 * @var string
	 * @access private
	 */
	private $sColorField	= "";

	/**
	 * @param WF_FolderDB $DB
	 * @access public
	 */
	public function __construct(WF_FolderDB $DB)
	{
		$this->DB	= $DB;
	}

	/**
	 * Sozdaet noviy zhurnal
	 * @return void
	 * @access public
	 */
	public function create()
	{
		$this->bIsNew					= true;
		$this->aConfig["name"]			= "";
		$this->aConfig["description"]	= "";
		$this->aConfig["full_description"]	= "";
		$this->aConfig["group_id"]		= 0;
		$this->aConfig["fields_xml"]	= "<document></document>";
		$this->aConfig["compared_xml"]	= "<document></document>";
		$this->aDoctypes				= array();
	}

	/**
	 * Zagruzhaet sushestvuyushiy zhurnal
	 *
	 * @param int $iFolderId
	 * @return void
	 * @access public
	 */
	public function load($iFolderId)
	{
		$this->iFolderId	= $iFolderId;
		$aConfig	= $this->DB->getConfig($iFolderId);
		if(empty($aConfig)) {
			throw new WF_FolderNotFoundException("Folder not found");
		}

		//$aConfig["name"] 	= MSF_String::decodeSpecialCharsFull($aConfig["name"]);
		$this->aConfig		= $aConfig;
	}

	/**
	 * Vozrashaet config
	 *
	 * @return array
	 * @access public
	 */
	public function getConfig()
	{
		return $this->aConfig;
	}

	/**
	 * Ustanavlivaet config
	 *
	 * @param array $aConfig
	 * @return void
	 * @access public
	 */
	public function setConfig($aConfig)
	{
		$this->aConfig["name"]				= MSF_SL::getParam($aConfig, "name", "");
		$this->aConfig["description"]		= MSF_SL::getParam($aConfig, "description", "");
		$this->aConfig["full_description"]	= MSF_SL::getParam($aConfig, "full_description", "");
		$this->aConfig["group_id"]			= MSF_SL::getParam($aConfig, "group_id", 0);
		$this->aConfig["fields_xml"]		= MSF_SL::getParam($aConfig, "fields_xml", "<document></document>", true);

		if(isset($aConfig["compared_xml"]))	{
			$this->aConfig["compared_xml"]	= MSF_SL::getParam($aConfig, "compared_xml", "<document></document>", true);
		}

	}

	/**
	 * Sohranyaet zhurnal i vosrashaet ego id
	 *
	 * @return int
	 * @access public
	 */
	public function save($bClearState = false)
	{
		if($this->bIsNew) {
			//$this->DB->begin();
			$this->iFolderId = $this->DB->create($this->aConfig);
			
			$this->load($this->iFolderId);
			
			$this->build();
			//$this->DB->commit();
		}
		else {
			//$this->DB->begin();
			$this->DB->saveConfig($this->aConfig, $this->iFolderId);
			$this->build();
			//$this->DB->commit();
		}

		if($bClearState) {
			$Recordset	= $this->buildRecordset();
			$Recordset->clearState();
		}
		
		
		return $this->iFolderId;
	}

	/**
	 * Sozdaet tablicu dlya zhurnala i naveshivaet trigeri na tablici
	 * documentov. Esli zhurnal uzhe sushestvuet - snachala snosit tablicu
	 * i trigeri.
	 * Pered vsem etim sopastavlyaet XML zhurnala s sopastavlennim XML doctypa.
	 *
	 * @return void
	 * @access public
	 */
	public function build()
	{
		$aDoctypes	= $this->prepareDoctypeComparedData();
		
		
		$aVersions	= $this->prepareDoctypeVersionComparedData($aDoctypes);
		
		$this->DB->createView($this->aConfig["view_table"], $this->prepareFields());
		$this->DB->associateFolder($this->aConfig["view_table"], $aVersions);
	}

	/**
	 * Udalyaet zhurnal. Snosit tablicu i udalyaet trigeri s tablic documentov/
	 *
	 * @return void
	 * @access public
	 */
	public function remove()
	{
		$this->removeFolderEventLog($this->aConfig["name"]);
		$this->removeTriggers();

		$this->DB->removeView($this->aConfig["view_table"]);
		$this->DB->removeFolder($this->iFolderId);
	}
	
	private function removeFolderEventLog($sEventTitle) {
		$oEventLog = new ELOG_EventLog(MSF_Registry::get("MSF_DB"));
		$oEventLog->addItem(array("event_type"=>"admin", "event_title"=>$sEventTitle, "message_type"=>"folder"));
	}
	
	public function removeTriggers()
	{
		$aDoctypes	= $this->prepareDoctypeComparedData();
		$aVersions	= $this->prepareDoctypeVersionComparedData($aDoctypes);
		
		//$this->DB->begin();
		foreach($aVersions as $sTable=>$aFields) {
			$this->DB->removeTrigger($sTable);
		}
	}

	/**
	 * Parsit XML poley i vosvrashaet massiv.
	 *
	 * @return array
	 * @access private
	 */
	private function prepareFields()
	{
		$Fields		= new MSF_XmlWrapper($this->aConfig["fields_xml"]);
		$aFields	= array();

		foreach($Fields as $Field) {
			$aValues					= $Field->getValue();
			if($aValues['static'] != 1) {
				$aFields[$Field->getName()]	= $aValues["type"];
			}
			
		}
		
		return $aFields;
	}

	/**
	 * Parsit XML sopastavlennih poley doctypa i vosvrashaet v vide massiva
	 *
	 * @return array
	 * @access public
	 */
	private function prepareDoctypeComparedData()
	{
		/* return examples
			array("docytpe_id"=>array("doctype_field"=>"folder_field"));
		*/
		$DoctypeList		= new WF_DoctypesList(new WF_DoctypesListDB(MSF_Registry::get("MSF_DB")));
		$LinkdedDoctypes	= $DoctypeList->getLinked($this->iFolderId);
		
		
		$FolderXml			= new MSF_XmlWrapper($this->aConfig["compared_xml"]);
		
		$aDoctypes	= array();
		foreach($LinkdedDoctypes as $aTmp) {
			$aDoctypes[$aTmp["doctype_id"]]	= array();
		}
		
		foreach($FolderXml as $Element) {
			$sFolderField	= $Element->getName();
			if(($sType = $this->fieldExists($sFolderField)) !== null) {
				$Items	= $Element->getValue();
				foreach($Items as $Item) {
					if (isset($aDoctypes[$Item["doctype_id"]])/* && isset($Item["type"]) && ($Item["type"] == $sType)*/) {
						$aDoctypes[$Item["doctype_id"]][$Item["name"]]	= $sFolderField;
					}

				}
				
			}

		}
		
		return $aDoctypes;
	}

	/**
	 * Vozvrashaet massiv talbic versiy, gde pole versii sopastavlenno
	 * s polem zhurnala
	 * array("table_name"=>array("version_field"=>"folder_field"));
	 *
	 * @param array $aDoctypes
	 * @return array
	 * @access private
	 */
	private function prepareDoctypeVersionComparedData($aDoctypes)
	{
		$aVersions	= array();
		foreach($aDoctypes as $iDoctypeId=>$aFields) {
			$Doctype		= new WF_Doctype($iDoctypeId);
			$ComparedXml	= new MSF_XmlWrapper($Doctype->compareAll());
			
			
			foreach($ComparedXml as $Element) {
				$aValue			= $Element->getValue();
				$aVersionFields	= $aValue["fields"];

				if(!is_array($aVersionFields)) {
					continue;
				}

				foreach($aVersionFields as $aVersionField) {
					if(!isset($aVersions[$aVersionField["tablename"]])) {
						$aVersions[$aVersionField["tablename"]]	= array();
					}

					if(isset($aFields[$aVersionField["compared_with"]])) {
						$aVersions[$aVersionField["tablename"]][$aVersionField["fieldname"]] = $aFields[$aVersionField["compared_with"]];
					}

				}

			}

		}
		
		//die("ololo");
		return $aVersions;
	}
	
	private function attachSystemFields($iDoctypeId, $sFieldsXml, $sSystemFieldsXml)
	{
		$VersionsList	= new WF_DoctypeVerionsList(new WF_DoctypeVerionsListDB(MSF_Registry::get("MSF_DB")));
		$aVersionsList	= $VersionsList->getVersion($iDoctypeId);
		$ComparedFields	= new MSF_XmlWrapper($sFieldsXml);				
		$SystemFields	= new MSF_XmlWrapper($sSystemFieldsXml);
		foreach($SystemFields as $SystemField) {
			$aValue	= $SystemField->getValue();
			$aTmp			= array();
			$aTmp["type"]	= $aValue["type"];
			$aTmp["fields"]	= array();
			
			foreach($aVersionsList as $aVersion) {
				$aFieldCompare	= array();
				$aFieldCompare["tablename"]		= $aVersion["extname"];
				$aFieldCompare["fieldname"]		= $SystemField->getName();
				$aFieldCompare["compared_with"]	= $SystemField->getName();
				
				array_push($aTmp["fields"], $aFieldCompare);
			}
			
			$ComparedFields->set($SystemField->getName(), $aTmp, $SystemField->getTitle());
		}
		
		return $ComparedFields->getXml();
	}

	/**
	 * Vozrashaet recordset zhurnala
	 *
	 * @return array
	 * @access public
	 */
	public function getDocumentsRecordset($bMiniView = false, $aDoctypes = array(), $iLimitFrom = -1, $bExport = false)
	{
		$Recordset	= $this->buildRecordset($bMiniView, $aDoctypes, $iLimitFrom, $bExport);
		return $Recordset->getAllRecords();
	}

	/**
	 * Vozrashaet gruppu recordseta. Nuzhno dlya gruppirovki recordseta
	 *
	 * @return array
	 * @access public
	 */
	public function getDocumentsGroup()
	{
		$Recordset	= $this->buildRecordset();
		return $Recordset->getGroup();
	}

	/**
	 * Vozrashet massiv vseh documentov
	 *
	 * @return unknown
	 * @access public
	 */
	public function getDocumentsList($aStructureIds = array(), $aDocumentIds = array())
	{
		$aColumns = $this->prepareRecordsetFields();
		return $this->DB->getDocumentsList($this->aConfig, $aColumns, $aStructureIds, $aDocumentIds);
	}
	
	/**
	 * Sobiraet recordset
	 *
	 * @return MSF_Recordset
	 * @access public
	 */
	public function buildRecordset($bMiniView = false, $aDoctypes = array(), $iLimitFrom = -1, $bExport = false)
	{		
		foreach($aDoctypes AS $iKey => $Value) 
		{
			if(!is_numeric($Value)) {
				$aDoctypes = array();			
			}			
		}		
		
		$aStrIds 			= array();
		$aActions 			= array();
		$Recordset 			= new MSF_Recordset(MSF_Registry::get("MSF_DB"));
		$Recordset->setExport($bExport);		
		if($iLimitFrom != -1) {
			$Recordset->setParam("limitFrom", $iLimitFrom);	
		}		
		
		$aColumns 			= $this->prepareRecordsetFields($bMiniView);
		$aAllowedDoctypes	= WF_Utils::getAllowedAuthorIds($this->iFolderId);
		$aDocs 				= WF_Utils::getAllowedDocumentIds($this->iFolderId);
		$Recordset->enableContextSearch("wf_index", "wf_documents.document_id", $this->iFolderId);
		
		//
		$aWhere = array("AND wf_documents.document_version_id=" . $this->aConfig["view_table"] . ".document_version_id AND (");
		$aDocs[] 	= 0;
		$iCount 	= 0;
		//
		foreach($aAllowedDoctypes as $iDoctypeId => $aAccessData) 
		{
			$aView 	= array();
			$aNView = array();
			for ($i = 0; $i < count($aAccessData); $i++) {
				if ($aAccessData[$i]["view"] == 2) {
					$aNView = array_merge($aNView, $aAccessData[$i]["author_ids"]);
				}
				else {
					$aView = array_merge($aView, $aAccessData[$i]["author_ids"]);
				}				
			}

			//
			$aView = array_diff($aView, $aNView);
			$aView[] = MSF_Registry::get("MSF_UserBox")->getCurrentStructureId();
			
			$aWhere[] = (($iCount > 0) ? "OR " : "") . "(wf_documents.structure_id IN (" . implode(",", $aView) . ")";
			$aWhere[] = "AND wf_documents.doctype_id = " . $iDoctypeId . ")";
			
			$iCount++;
		}
		
		//
		asort($aDocs);
		//debug(count($aDocs), count(array_unique($aDocs)));
		$aWhere[] = ((empty($aAllowedDoctypes)) ? "" : "OR ") . "wf_documents.document_id IN(" . implode(",", $aDocs) . "))";
		//	
		
		if($bMiniView && !empty($aDoctypes)) {
			$aWhere[] = "AND wf_documents.doctype_id IN(" . implode(",", $aDoctypes) . ")";
		}
		
		$Recordset->setColumns($aColumns);
		$Recordset->setTables(array($this->aConfig["view_table"], "wf_documents"));
		$Recordset->setConditions($aWhere);
		$Recordset->setColorField($this->sColorField);
		/*$Recordset->setConditions(array(
			"AND wf_documents.document_version_id=" . $this->aConfig["view_table"] . ".document_version_id",
			"AND (wf_documents.structure_id IN (" . implode(",", $aStrIds) . ")",
			"OR wf_documents.document_id IN (" . implode(",", $aDocs) . "))"
		));*/
		
		$aActions[] = array(
				"title" 	=> MSF_Registry::get("MSF_Locale")->getWord("WORKFLOW_ADMIN_LIST_ACTION_VIEW"),
				"href" 		=> "/workflow/document/view",
				"image" 	=> "/templates/" . CNF_SKIN . "/images/folder_page_white.png",
				"bind" 		=> array("document_id"),
				"confirm" 	=> false
		);
		$aActions[] = array(
				"title" 	=> MSF_Registry::get("MSF_Locale")->getWord("WORKFLOW_ADMIN_LIST_ACTION_VIEW_BLANK"),
				"href" 		=> "/workflow/document/view",
				"image" 	=> "/templates/" . CNF_SKIN . "/images/folder_page_white_blank.png",
				"bind" 		=> array("document_id"),
				"confirm" 	=> false,
				"blank"		=> true
		);
		$aActions[] = array(
			"title" 	=> MSF_Registry::get("MSF_Locale")->getWord("WORKFLOW_ADMIN_LIST_ACTION_DELETE"),
			"href" 		=> "/workflow/document/remove/" . $this->iFolderId,
			"image" 	=> "/templates/" . CNF_SKIN . "/images/red_cross.png",
			"bind" 		=> array("document_id"),
			"confirm" 	=> true
		);
		
		
		$aMultiActions	= array(
			array(
	 			"title"		=> MSF_Registry::get("MSF_Locale")->getWord("WORKFLOW_ADMIN_LIST_ACTION_DELETE"),
				//"href"		=> "/workflow/document/remove/" . $this->iFolderId,
				"href"		=> "/workflow/folders/removedocs/" . $this->iFolderId,
				"image"		=> "/templates/" . CNF_SKIN . "/images/red_cross.png",
				"confirm"	=> true
	 		)
		);
		
		
		$Recordset->setActions($aActions);
		$Recordset->setMultiActions($aMultiActions);
 		$Recordset->setMultiActionField("document_id");
		$Recordset->setLink(array(
			"href" 	=> "/workflow/document/view",
			"bind" 	=> array("document_id")
		));
		
		if(!$bMiniView) {
			$Recordset->setStateParams(array(
				"module"		=> "workflow",
				"action"		=> "folders_index",
				"action_id"		=> $this->iFolderId,
				"structure_id"	=> MSF_Registry::get("MSF_UserBox")->getCurrentStructureId()
			));
			
		}
		
		
		$Recordset->setBaseURL("/workflow/folders/" . $this->iFolderId . "/?");
 		$Recordset->setRpcURL("/workflow/folders/group/" . $this->iFolderId . "/?");
 		$Recordset->setSessionUrlName("foldersRecordset");
 		$Recordset->setDefaultOrder("wf_documents.document_id");
 		$Recordset->setSecondaryOrder("wf_documents.document_id");
		
		return $Recordset;
	}

	public function nameExists($sName, $iFolderId = 0)
	{
		if($this->DB->nameExists($sName, $iFolderId) > 0) {
			return true;
		}
		else {
			return false;
		}
	}
	
	/**
	 * Parsit XML poley dlya recirdseta
	 *
	 * @return array
	 * @access private
	 */
	public function prepareRecordsetFields($bMiniView = false)
	{
		$Fields			= new MSF_XmlWrapper($this->aConfig["fields_xml"]);
		$aColumns		= array();
		
		
		$aColumn["name"]	= "wf_documents.document_id";
		$aColumn["title"]	= "ID";
		$aColumn["type"] 	= "int";
 		$aColumn["sort"] 	= true;
 		$aColumn["filter"] 	= true;
 		$aColumn["group"] 	= false;
 		$aColumn["compare"] = "equal";
 		
 		if($bMiniView) {
 			$aColumn["visible"]	= true;
 		}
 		else {
 			$aColumn["visible"]	= false;
 		}
 		
 		array_push($aColumns, $aColumn);


		foreach($Fields as $Field) {
			$aColumn			= array();
			$aValues			= $Field->getValue();
			
			if($aValues["static"] == 1) {
				$aColumn["name"]	= "wf_documents." . $Field->getName();
			}
			else {
				$aColumn["name"]	= $this->aConfig["view_table"] . "." . $Field->getName();
			}
			
			$aColumn["title"]	= $Field->getTitle();
			$aColumn["type"] 	= $aValues["type"];	 		
	 		$aColumn["visible"]	= true;
                        if(array_key_exists('visible_on_locale', $aValues)){
                            $aColumn["visible_on_locale"] = $aValues["visible_on_locale"];
                            if($aValues["visible_on_locale"] == CNF_LANGUAGE_2 || $aValues["visible_on_locale"] == "both"){
                                $aColumn["visible"]	= true;
                            } else {
                                $aColumn["visible"]	= false;
                            }
                        } else {
                            $aColumn["visible_on_locale"] = "both";
                        }
                        
	 		if($aValues["type"] == "file" || $aValues["type"] == "picture") {
		 		$aColumn["sort"] 	= 0;
		 		$aColumn["filter"] 	= 0;
		 		$aColumn["group"] 	= 0;
	 		}
	 		else {
	 			$aColumn["sort"] 	= true;
		 		$aColumn["filter"] 	= true;
		 		$aColumn["group"] 	= $aValues["group"];
	 		}

	 		if(in_array($aValues["type"], array("int", "float"))) {
	 			$aColumn["compare"] 	= "equal";
	 		}
	 		else {
	 			$aColumn["compare"] 	= "like";
	 		}
			if ($aColumn["title"] != ".#color") {
				array_push($aColumns, $aColumn);
			}
			else {
				$this->sColorField = $Field->getName();
			}
		}
		
		return $aColumns;
	}
	/*
	public function fieldExists($sFieldName, $sType)
	{
		$Fields	= new MSF_XmlWrapper($this->aConfig["fields_xml"]);
		$bFlag	= false;
		
		foreach($Fields as $Field) {
			if(($Field->getName() == $sFieldName) && ($Field->getValue("type") == $sType)) {
				$bFlag	= true;
			}
			 
		}
		
		return $bFlag;
	}
	*/
	public function fieldExists($sFieldName)
	{
		$Fields	= new MSF_XmlWrapper($this->aConfig["fields_xml"]);
		$sType	= null;
		
		foreach($Fields as $Field) {
			if($Field->getName() == $sFieldName) {
				$sType	= $Field->getValue("type");
			}
			 
		}
		
		return $sType;
	}
	
	public static function reindex($iFolderId, array $aDoctypes) {
		$oDb = MSF_Registry::get("MSF_DB");
		$sSql  = "DELETE FROM wf_index WHERE context_id = " . $iFolderId;
		$oDb->executeQuery($sSql);
		if (!empty($aDoctypes)) {
			$sSql  = "SELECT document_id, doctype_id FROM wf_documents "
				   . "WHERE doctype_id IN (".implode(", ", $aDoctypes).")"; 
			foreach ($oDb->select($sSql) AS $aRow) {
				$sSql  = "INSERT INTO cs_indexed(subject_id, module, operation, context_id) "
					   . "VALUES (".$aRow["document_id"].", 'workflow', 'index', ".$aRow["doctype_id"].")";
				$oDb->executeQuery($sSql);
			}
			
		}

	}

}
?>