<?php
if(!defined("__MSF")) {
	die("Direct access is not allowed");
}

include_once CNF_ABS_PATH . "/modules/workflow/view/xml/WF_FolderGroupXml.php";
include_once CNF_ABS_PATH . "/modules/workflow/library/folders/WF_FolderGroupDB.php";
include_once "WF_FolderGroupNotFoundException.php";

class WF_FolderGroup
{
	/**
	 * @var WF_FolderGroupDB
	 */
	private $DB				= null;

	/**
	 * @var array
	 */
	private $aConfig		= array();

	/**
	 * @var int
	 */
	private $iGroupId		= 0;

	/**
	 * Esli true - znachit gruppa tolko chto sozdana
	 *
	 * @var unknown_type
	 */
	private $bIsNew			= false;

	/**
	 * @param WF_FolderGroupDB $DB
	 */
	public function __construct(WF_FolderGroupDB $DB)
	{
		$this->DB	= $DB;
	}

	/**
	 * Sozdaet novuyu gruppu
	 * 
	 * @return void
	 * @access public
	 */
	public function create()
	{
		$this->bIsNew					= true;
		$this->aConfig["name"]			= "";
		$this->aConfig["description"]	= "";
	}

	/**
	 * Zagruzhaet gruppu
	 *
	 * @param int $iGroupId
	 * @return void
	 * @access public
	 */
	public function load($iGroupId)
	{
		$this->iGroupId	= $iGroupId;
		if($iGroupId != 0) {
			$aConfig	= $this->DB->getConfig($this->iGroupId);
			if(empty($aConfig)) {
				throw new WF_FolderGroupNotFoundException("FolderGroup not found");
			}

			$this->aConfig		= $aConfig;
		}

	}
	
	public function nameExists($sName) {
		$sSQL =	"SELECT" . "\n" .
					"COUNT(*) AS total" . "\n" .
				"FROM" . "\n" .
					"wf_folders_groups" . "\n" .
				"WHERE" . "\n" .
					"name = '" . MSF_Registry::get('MSF_DB')->escapeString($sName) . "'";


		return $this->DB->DB->select($sSQL, true, 'total') > 0;
	}


	/**
	 * Poluchit cofig
	 *
	 * @return array
	 * @access public
	 */
	public function getConfig()
	{
		return $this->aConfig;
	}

	/**
	 * Ustanovit config
	 *
	 * @param array $aConfig
	 * @return void
	 * @access public
	 */
	public function setConfig($aConfig)
	{
		$this->aConfig["name"]			= MSF_SL::getParam($aConfig, "name", "");
		$this->aConfig["description"]	= MSF_SL::getParam($aConfig, "description", "");
	}

	/**
	 * Sohranyaet gruppu i vozvrashaet ee id
	 *
	 * @return int
	 * @access public
	 */
	public function save()
	{
		if($this->bIsNew) {
			$this->iGroupId = $this->DB->create($this->aConfig);
		}
		else {
			$this->DB->saveConfig($this->aConfig, $this->iGroupId);
		}

		return $this->iGroupId;
	}

	/**
	 * Udalyaet gruppu
	 * 
	 * @return void
	 * @access public
	 */
	public function remove()
	{
		$this->DB->removeLinks($this->iGroupId);
		$this->DB->removeGroup($this->iGroupId);
	}

	/**
	 * Vozvrashaet spisok papok
	 *
	 * @return array
	 * @access public
	 */
	public function getFolders()
	{
		return $this->DB->getFolders($this->iGroupId);
	}
	
	public function getFoldersWithDoctypes($aDoctypes)
	{
		foreach($aDoctypes AS $iKey => $Value) {
			if(!is_numeric($Value)) {
				$aDoctypes = array();
				break;
			}
		}		
		
		$aFolders = $this->DB->getFoldersWithDoctypes(implode(",", $aDoctypes));
		$aResult  = array();
		foreach($aFolders AS $Value) {
			$aResult[] = $Value["folder_id"];
		}
		return $aResult;
	}

}
?>