<?php
if(!defined("__MSF")) {
	die("Direct access is not allowed");
}
require_once CNF_ABS_PATH . "/system/mvc/MSF_ModelDBSupport.php";
include_once CNF_ABS_PATH . "/library/MSF_Recordset/MSF_Recordset.php";

class WF_FoldersList
{
	/**
	 * @var MSF_Recordset
	 */
	private $Recordset	= null;

	/**
	 * Deistviya dlya recordseta
	 *
	 * @var array
	 */
	private $aActions	= array();
	
	private $Locale 	= null;
	
	private $DB 		= null;

	/**
	 * @return void
	 * @access public
	 */
	public function __construct()
	{
		$this->Recordset 	= new MSF_Recordset(MSF_Registry::get("MSF_DB"));
		$this->Locale 		= MSF_Registry::get("MSF_Locale");
		$this->DB 			= MSF_Registry::get("MSF_DB");
	}

	/**
	 * Vozvrashaet recordset zhurnala
	 *
	 * @return MSF_Recordset
	 * @access public
	 */
	public function getListRecordset()
	{
 		$this->prepare();
		return $this->Recordset->getAllRecords();
	}
	
	private function prepare()
	{
		$this->Recordset->setColumns($this->getColumns());
 		//$this->Recordset->setTables(array("wf_folders"));
 		$this->Recordset->setTables(array("wf_folders_groups_view"));
 		$this->Recordset->setActions($this->aActions);
 		$this->Recordset->setSecondaryOrder("folder_id");
 		
 		$this->insert();
 		$this->Recordset->setDefaultGroups(array("group_name"));
	}
	
	public function getGroup()
	{
		$this->prepare();
		return $this->Recordset->getGroup();
	}
	
	public function setLink($aLink)
	{
		$this->Recordset->setLink($aLink);
	}
	
	
	/**
	 * Dobavlyaet deistvie v recordset
	 *
	 * @param string $sTitle
	 * @param string $sLink
	 * @param string $sImage
	 * @param array $aFields
	 * @param bool $bConfrm
	 * @return void
	 * @access public
	 */
	public function addAction($sTitle, $sLink, $sImage, $aFields, $bConfrm = false)
	{
		//$this->Recordset->addAction($sTitle, $sLink, $sImage, $aFields, $bConfrm);
		$this->aActions[]	= array(
				"title" 	=> $sTitle,
				"href" 		=> $sLink,
				"image" 	=> $sImage,
				"bind" 		=> $aFields,
				"confirm" 	=> $bConfrm
			);
	}
	
	public function setStateParams($aInput) {
		$this->Recordset->setStateParams($aInput);
	}

	/**
	 * ustanavlivae basoviy url v recordset
	 *
	 * @param string $sUrl
	 * @return void
	 * @access public
	 */
	public function setBaseUrl($sUrl)
	{
		$this->Recordset->setBaseURL($sUrl);
	}

	/**
	 * ustanavlivae rpc url v recordset
	 *
	 * @param string $sUrl
	 * @return void
	 * @access public
	 */
	public function setRpcUrl($sUrl)
	{
		$this->Recordset->setRpcURL($sUrl);
	}

	/**
	 * vozvrashaet stolbci dlya recordseta
	 *
	 * @return array
	 * @access private
	 */
	/*
	private function getColumns()
	{
		$aTypes		= array("int", "string", "text");
 		$aTitles 	= array("ID", $this->Locale->getWord("WORKFLOW_ADMIN_LIST_TITLE"), $this->Locale->getWord("WORKFLOW_ADMIN_LIST_DESCRIPTION"));
 		$aColumns 	= array("folder_id", "name", "description");
 		$aRColumns 	= array();

 		for ($i = 0; $i < sizeof($aTitles); $i++) {
	 		$aRColumns[$i]["name"] 		= $aColumns[$i];
	 		$aRColumns[$i]["title"] 	= $aTitles[$i];
			$aRColumns[$i]["type"] 		= $aTypes[$i];
	 		$aRColumns[$i]["sort"] 		= true;
	 		$aRColumns[$i]["filter"] 	= true;
	 		$aRColumns[$i]["group"] 	= false;
	 		$aRColumns[$i]["compare"] 	= "like";
	 		$aRColumns[$i]["visible"] 	= ($aTitles[$i] == "ID") ? false : true;
 		}

 		return $aRColumns;
	}
	*/
	private function getColumns()
	{
		$aTypes		= array("int", "string", "string", "string", "date", "timestamp");
 		$aTitles 	= array(
 			"ID",
 			$this->Locale->getWord("WORKFLOW_ADMIN_LIST_TITLE"),
 			$this->Locale->getWord("WORKFLOW_ADMIN_LIST_DESCRIPTION"),
 			$this->Locale->getWord("WORKFLOW_ADMIN_LIST_GROUP"),
 			$this->Locale->getWord("WORKFLOW_ADMIN_LIST_CREATED"),
 			$this->Locale->getWord("WORKFLOW_ADMIN_LIST_CHANGED")
 		);
 		$aColumns 	= array("folder_id", "folder_name", "description", "group_name", "created_at", "modified_at");
 		$aRColumns 	= array();

 		for ($i = 0; $i < sizeof($aTitles); $i++) {
	 		$aRColumns[$i]["name"] 		= $aColumns[$i];
	 		$aRColumns[$i]["title"] 	= $aTitles[$i];
			$aRColumns[$i]["type"] 		= $aTypes[$i];
	 		$aRColumns[$i]["sort"] 		= true;
	 		$aRColumns[$i]["filter"] 	= true;
	 		$aRColumns[$i]["compare"] 	= "like";
	 		$aRColumns[$i]["group"] 	= ($aColumns[$i] == "group_name") ? true : false;
	 		$aRColumns[$i]["visible"] 	= ($aColumns[$i] == "folder_id") ? false : true;
 		}

 		return $aRColumns;
	}
	
	private function insert()
	{
		$sSql = "CREATE OR REPLACE FUNCTION ref_groups_null_replace(group_name varchar) RETURNS varchar AS $$
					BEGIN
		    			IF group_name IS NULL THEN
		        			RETURN '" . $this->Locale->getWord("WORKFLOW_ADMIN_LIST_NONGROUP_TITLE") . "';
		    			END IF;
			    		RETURN group_name;
					END;
				$$ LANGUAGE plpgsql;";
		
		$this->DB->executeQuery($sSql);
		
		$sSql = "CREATE OR REPLACE VIEW wf_folders_groups_view AS
					SELECT\n" .
						"wff.group_id,\n" .
						"wf_folders_null_replace(wfg.name) AS group_name,\n" .
						"wff.folder_id,\n" .
						"wff.name AS folder_name,\n" .
						"wff.description,\n" .
						"wff.created_at,\n" .
						"wff.modified_at\n" .
					"FROM\n" .
						"wf_folders wff\n" .
					"LEFT JOIN\n" .
						"wf_folders_groups wfg\n" .
					"ON\n" .
						"wfg.group_id = wff.group_id";
		
		$this->DB->executeQuery($sSql);
	}
}
?>