<?php
if(!defined("__MSF")) {
	die("Direct access is not allowed");
}
require_once "WF_LogDB.php";
class WF_Log
{
	private $iStructureId;
	
	private $sEmployeeName;
	
	private $aLog;
	
	private $iDocumentId;
	
	/**
	 * @var WF_LogDB
	 */
	private $DB;
	
	private static $iCount	= 0;
	
	public function __construct($iDocumentId, $sSerialized)
{
		$this->iStructureId		= 0;
		$this->sEmployeeName	= "";
		$this->iDocumentId		= $iDocumentId;
		$this->DB				= new WF_LogDB(MSF_Registry::get("MSF_DB"));
		try {
			$this->aLog			= unserialize(stripslashes($sSerialized));
		}
		catch (Exception $oE) {
			
		}

		if($this->aLog === false) {
			$this->aLog	= array();
		}
		
	}
	
	public function setStructureId($iStructureId)
	{
		$this->iStructureId		= $iStructureId;
	}
	
	public function setEmployeeName($sEmployeeName)
	{
		$this->sEmployeeName	= $sEmployeeName;
	}
	
	public function getStructureId()
	{
		return $this->iStructureId;
	}
	
	public function getEmployeeName()
	{
		return $this->sEmployeeName;
	}
	
	public function report($iPointId, $sAction, $sResult)
	{
		$Date	= new MSF_Date();
		
		
		$this->DB->report(
			$this->iDocumentId,
			$iPointId,
			$sAction,
			$this->iStructureId,
			$sResult,
			$Date->formatTimestamp()
		);
	}
	
	public function display($sActionTitle, $sSmallText, $sFullText = "", $bSpecialChars = false, $iActionId = -1)
	{
		$this->checkPart("display");
				
		$this->aLog["display"][count($this->aLog["display"])]	= array(
			"timestamp"		=> time(),
			"structureId"	=> $this->iStructureId,
			"employeeName"	=> $this->sEmployeeName,
			"actionTitle"	=> MSF_Registry::get("MSF_Locale")->getI18N($sActionTitle),
			"smallText"		=> MSF_Registry::get("MSF_Locale")->getI18N($sSmallText),
			"fullText"		=> MSF_Registry::get("MSF_Locale")->getI18N($sFullText),
			"specialchars" 	=> $bSpecialChars,
			"action_id" 	=> $iActionId
		);
		
	}
	
	public function getDisplay()
	{
		$this->checkPart("display");
		
		return $this->aLog["display"];
	}
	
	public function addExecuted($iPointId, $iActionId)
	{
		$this->addToPart("execution", $iPointId, $iActionId);
		/*
		$this->checkPart("execution");
		
		if(!isset($this->aLog["execution"][$iPointId])) {
			$this->aLog["execution"][$iPointId]	= array();
		}
		
		array_push($this->aLog["execution"][$iPointId], $iActionId);*/
	}
	
	public function isExecuted($iPointId, $iActionId)
	{
		/*
		$this->checkPart("execution");
		$bResult	= false;
		if(isset($this->aLog["execution"][$iPointId])) {
			if(in_array($iActionId, $this->aLog["execution"][$iPointId])) {
				$bResult	= true;
			}
			
		}
		
		return $bResult;*/
		return $this->isInPart("execution", $iPointId, $iActionId);
	}
	
	public function clearExecuted($iPointId)
	{
		/*
		$this->checkPart("execution");
		$this->aLog["execution"][$iPointId]	= array();*/
		$this->clearPart("execution", $iPointId);
	}
	
	public function getExecuted($iPointId)
	{
		return $this->getPart("execution", $iPointId);
	}
	
	public function addWaited($iPointId, $iStructureId)
	{
		$this->addToPart("waiting", $iPointId, $iStructureId);
	}
	
	public function isWaited($iPointId, $iStructureId)
	{
		return $this->isInPart("waiting", $iPointId, $iStructureId);
	}
	
	public function clearWaited($iPointId)
	{
		$this->clearPart("waiting", $iPointId);
	}
	
	public function getWaited($iPointId)
	{
		return $this->getPart("waiting", $iPointId);
	}
	
	private function addToPart($sPart, $iPointId, $iActionId)
	{
		$this->checkPart($sPart);
		
		if(!isset($this->aLog[$sPart][$iPointId])) {
			$this->aLog[$sPart][$iPointId]	= array();
		}
		
		array_push($this->aLog[$sPart][$iPointId], $iActionId);
	}
	
	private function getPart($sPart, $iPointId)
	{
		$this->checkPart($sPart);
		$aReturn	= array();
		
		if(isset($this->aLog[$sPart][$iPointId])) {
			$aReturn	= $this->aLog[$sPart][$iPointId];
		}
		
		return $aReturn;
	}
	
	private function isInPart($sPart, $iPointId, $iActionId)
	{
		$this->checkPart($sPart);
		$bResult	= false;
		if(isset($this->aLog[$sPart][$iPointId])) {
			if(in_array($iActionId, $this->aLog[$sPart][$iPointId])) {
				$bResult	= true;
			}
			
		}
		
		return $bResult;
	}
	
	private function clearPart($sPart, $iPointId)
	{
		$this->checkPart($sPart);
		$this->aLog[$sPart][$iPointId]	= array();
	}
	
	public function clearDelegatedAction($iPointId, $iActionId)
	{				
		$this->checkPart("delegation");
		if($iPointId !== null) {
			if(isset($this->aLog["delegation"][$iPointId])) {
				if($iActionId !== null) {
					if(isset($this->aLog["delegation"][$iPointId][$iActionId])) {
						unset($this->aLog["delegation"][$iPointId][$iActionId]);
					}
				}				
				
			}
			
		}
		
	}
	
	public function setDelegated($iPointId, $iActionId, array $aValue)
	{
		$this->checkPart("delegation");
		$this->aLog["delegation"][$iPointId][$iActionId]	= $aValue;
	}
	
	public function getDelegated($iPointId = null, $iActionId = null)
	{
		$this->checkPart("delegation");
		$aResult	= array();
		
		//debug($iPointId, $iActionId, $this->aLog);
		if($iPointId !== null) {
			if(isset($this->aLog["delegation"][$iPointId])) {
				if($iActionId !== null) {
					if(isset($this->aLog["delegation"][$iPointId][$iActionId])) {
						$aResult	= $this->aLog["delegation"][$iPointId][$iActionId];
					}
				}
				else {
					$aResult	= $this->aLog["delegation"][$iPointId];
				}
				
			}
			
		}
		else {
			$aResult	= $this->aLog["delegation"];
		}
		
		
		return $aResult;
	}
	
	public function getFullLog()
	{
		return $this->aLog;
	}
	
	public function __toString()
	{
		return addslashes(serialize($this->aLog));
	}
	
	private function checkPart($sPart)
	{
		if(!isset($this->aLog[$sPart])) {
			$this->aLog[$sPart]	= array();
		}
		
	}
}
?>