<?php
if(!defined("__MSF")) {
	die("Direct access is not allowed");
}

include_once "system/security/MSF_ISecurityManager.php";
include_once "system/security/MSF_SecurityMatrixNotSavedException.php";
include_once "WF_DoctypeSecurityManagerDb.php";
include_once "modules/references/library/Reference/REF_Value.php";
include_once "modules/main/library/orgStructure/MSF_OrgStructureFacade.php";


class WF_DoctypeSecurityManager implements MSF_ISecurityManager
{
	/**
	 * @var WF_FoldersSecurityManagerDB
	 */
	private $DB;
	
	/**
	 * @var int
	 */
	private $iId;
	
	/**
	 *
	 * @param MSF_DB $DB
	 * @param int $iId
	 * @return void
	 * @access public
	 */
	public function __construct(MSF_DB $DB, $iId)
	{
		$this->DB	= new WF_DoctypeSecurityManagerDb($DB);
		$this->iId	= $iId;
	}
	
	/**
	 * Izmenyaet tablicu dostupa
	 *
	 * @param array $aPost
	 * @return void
	 * @access public
	 */
	public function edit($aPost)
	{
		try {
			$aToSave	= array();
			$iGroup		= 1;
			$RV			= new REF_Value();
			foreach($aPost as $aRow) {
				$RV->parse($aRow["structure"]);
				$aStructureIds	= $RV->getKeysList();
				$RV->parse($aRow["author"]);
				$aAuthorIds		= $RV->getKeysList();
				foreach($aStructureIds as $mStructureId) {
					if (is_array($mStructureId)) {
						$mStructureId = $mStructureId[0];
					}
					foreach($aAuthorIds as $mAuthorId) {
						if (is_array($mAuthorId)) {
							$mAuthorId = $mAuthorId[0];
						}
						$tmp	= array(
							"structure_id"		=> $mStructureId,
							"author_id"			=> $mAuthorId,
							"can_view"			=> $aRow["can_view"],
							"can_remove"		=> $aRow["can_remove"],
							"group_id"			=> $iGroup
						);

						array_push($aToSave, $tmp);
					}

				}

				$iGroup++;
			}

			$this->DB->edit($aToSave, $this->iId);
		}
		catch(MSF_DBException $e) {
			throw new MSF_SecurityMatrixNotSavedException($e->getMessage());
		}
		
	}
	
	/**
	 * Daet tablicu dostupa
	 *
	 * @return array
	 * @access public
	 */
	public function get()
	{
		$aData		=  $this->DB->get($this->iId);
		//print_r($aData);
		$aTmpData	= array();
		$aReady		= array();
		foreach($aData as $aRow) {
			if(!isset($aTmpData[$aRow["group_id"]])) {
				$aTmpData[$aRow["group_id"]] = array(
					"structure"		=> array(),
					"author"		=> array(),
					"can_view"		=> $aRow["can_view"],
					"can_remove"	=> $aRow["can_remove"]
				);
			}
			
			if(!in_array($aRow["structure_id"], $aTmpData[$aRow["group_id"]]["structure"])) {
				array_push($aTmpData[$aRow["group_id"]]["structure"], $aRow["structure_id"]);
			}
			
			if(!in_array($aRow["author_id"], $aTmpData[$aRow["group_id"]]["author"])) {
				array_push($aTmpData[$aRow["group_id"]]["author"], $aRow["author_id"]);
			}
			
		}
		
		foreach($aTmpData as $aRow) {
			$aTmp = array(
				"structure"		=> MSF_OrgStructureFacade::getInstance()->strIdsToRefValue($aRow["structure"]),
				"author"		=> MSF_OrgStructureFacade::getInstance()->strIdsToRefValue($aRow["author"]),
				"can_view"		=> $aRow["can_view"],
				"can_remove"	=> $aRow["can_remove"]
			);
			
			array_push($aReady, $aTmp);
		}
		
		return $aReady;
	}
	

	
}

?>