<?php

include_once "modules/structure/library/Struct_Tree.php";
include_once "system/security/MSF_Access.php";
include_once "WF_DoctypeSwitcherDb.php";

class WF_DoctypeSwitcher extends MSF_Access
{
	private $oDb 			= null;
	
	private $aAccessTypes	= array(
			"view" 		=> "can_view",
			"remove"	=> "can_remove"
			);
	
	private $aStructure 	= array();
	
	protected function initialize()
	{
		$Struct 			= new Struct_Tree($this->getDb());
		$this->oDb 			= new WF_DoctypeSwitcherDb($this->getDb());
		$this->aStructure 	= $Struct->get(false);
	}
	
	protected function getAccessValue($sAccessType, $iStructureId, $aParams)
	{
		$iResult = self::INHERIT;
		$aRights = $this->oDb->select($aParams["doctype_id"], $iStructureId);
		$aDeny 	= array();			
		$aAllow = array();
		$aNodes = array();
		
		for ($i = 0; $i < count($aRights); $i++) {
			$aTmp = array();
			if ($this->checkIdType($aRights[$i]["author_id"]) == "subdivision") {
				$aTmp = array_merge($aTmp, $this->getSubdivNodes($aRights[$i]["author_id"]));
			}
			else {
				$aTmp[] = $aRights[$i]["author_id"];
			}
			
			if ($aRights[$i][$this->aAccessTypes[$sAccessType]] == 2) {
				$aDeny = array_merge($aDeny, $aTmp);
			}
			else {
				$aAllow = array_merge($aAllow, $aTmp);
			}
			
		}
		
		$iAllowCount 	= count($aAllow);
		$iDenyCount 	= count($aDeny);
		if ($iAllowCount > 0) {
			for ($i = 0; $i < $iAllowCount; $i++) {
				if ($iDenyCount > $iAllowCount) {
					if (in_array($aAllow[$i], $aDeny)) {
						$aNodes[] = $aAllow[$i];
					}
					
				}
				else {
					if (!in_array($aAllow[$i], $aDeny)) {
						$aNodes[] = $aAllow[$i];
					}
					
				}
				
			}
		}
		else {
			$aNodes = $aDeny;
		}
		
		for ($i = 0; $i < count($aRights); $i++) {
			for ($j = 0; $j < count($aNodes); $j++) {
				if ($aRights[$i]["author_id"] == $aNodes[$j] && in_array($aParams["author_id"], $aNodes)) {
					$iResult = $aRights[$i][$this->aAccessTypes[$sAccessType]];
					break;
				}
				
			}
			
		}
		
		return $iResult;
	}
	
	private function getSubdivNodes($iStructureId)
	{
		$aResult = array();

		for ($i = 0; $i < count($this->aStructure); $i++) {
			if ($this->aStructure[$i]["parent_id"] == $iStructureId) {
				if ($this->aStructure[$i]["type"] == "post") {
					$aResult[] = $this->aStructure[$i]["node_id"];
				}
				elseif ($this->aStructure[$i]["type"] == "subdivision") {
					$aResult[] 	= $this->aStructure[$i]["node_id"];
					$aResult 	= array_merge($aResult, $this->getSubdivNodes($this->aStructure[$i]["node_id"]));
				}
			}
		}
		
		$aResult[] = $iStructureId;
		return $aResult;
	}
	
	private function getParentId($iStructureId)
	{
		$iResult = 0;
		
		for ($i = 0; $i < count($this->aStructure); $i++) {
			if ($this->aStructure[$i]["node_id"] == $iStructureId) {
				if ($this->aStructure[$i]["type"] == "subdivision") {
					$iResult = $iStructureId;
					break;
				}
				else {
					$iResult = $this->aStructure[$i]["parent_id"];
					break;
				}
			}
		}
		
		return $iResult;
	}
	
	private function checkIdType($iStructureId)
	{
		$sType = "";
		
		for ($i = 0; $i < count($this->aStructure); $i++) {
			if ($this->aStructure[$i]["node_id"] == $iStructureId) {
				$sType = $this->aStructure[$i]["type"];
				break;
			}
		}
		
		return $sType;
	}
}

?>