<?php

defined("__MSF") or die("Direct access is not allowed");

include_once "system/security/MSF_ISecurityManager.php";
include_once "WF_DocumentSecurityManagerDB.php";
include_once "system/security/MSF_SecurityMatrixNotSavedException.php";

class WF_DocumentSecurityManager implements MSF_ISecurityManager
{
	/**
	 * @var WF_DocumentSecurityManagerDB
	 */
	private $DB;
	
	/**
	 * @var int
	 */
	private $iDocumentId;
	
	/**
	 * @param MSF_DB $DB
	 * @param int $iDocumentId
	 * @return void
	 * @access public
	 */
	public function __construct(MSF_DB $DB, $iDocumentId)
	{
		$this->DB 			= new WF_DocumentSecurityManagerDB($DB);
		$this->iDocumentId 	= $iDocumentId;
	}
	
	/**
	 * Izmenyaet matricu dostupa
	 *
	 * @param array $aPost
	 * @return void
	 * @access public
	 */
	public function edit($aPost)
	{
		try {
			if ($this->DB->exists($aPost, $this->iDocumentId) == 0) {
				$this->DB->insert($aPost, $this->iDocumentId);
			}
		}
		catch(MSF_DBException $e) {
			throw new MSF_SecurityMatrixNotSavedException($e->getMessage());
		}
	}
	
	/**
	 * Vozvrashaet matricu dostupa na document
	 *
	 * @return array
	 * @access public
	 */
	public function get()
	{
		return $this->DB->select($this->iDocumentId);
	}
	
	/**
	 * Udalyaet matricu dostupa documenta
	 * 
	 * @return void
	 * @access public
	 */
	public function remove()
	{
		$this->DB->delete($this->iDocumentId);
	}
}

?>