<?php
include_once "WF_IVariable.php";
include_once "WF_VarsSupport.php";
/**
 */
class WF_Vars
{
	/**
	 * @var array
	 */
	private $aVars 			= array();
	
	/**
	 * @var WF_ActionSupport
	 */
	private $ActionSupport 	= null;
	
	private $oLocale		= null;
	
	/**
	 * @param WF_ActionSupport $ActionSupport
	 * @return void
	 * @access public
	 */
	public function __construct(WF_ActionSupport $ActionSupport = null)
	{
		$this->oLocale			= MSF_Registry::get("MSF_Locale");
		$this->ActionSupport 	= $ActionSupport;
		$this->initVarsDescription();
		if ($ActionSupport !== null) {			
			$this->initVarsValues();
		}
		
	}
	
	/**
	 * Vozvrashaet peremennie
	 *
	 * @param string $sType
	 * @return array
	 * @access public
	 */
	public function getList($sType = "both")
	{
		$aVars = array();
		foreach ($this->aVars AS $aVar) {			
			if (($aVar["type"] == $sType) || ($aVar["type"] == "both")) {
				$aVars[] = $aVar;
			}

		}
		
		return $aVars;
	}

	/**
	 * Vozvrashaet [html] systemnogo polya, po nazvaniyu
	 * Esli $Key ne peredaetsya, to vozvrashyaet array so vsemi polyami
	 * Klychi = nazvanie polia, znacheniya = html polya
	 */
	public function getHtml($Key = "")
	{
		if ($Key == "") {
			$aVarsHtml = array();
			foreach ($this->aVars AS $aVar) {
				$aVarsHtml[$aVar["name"]] = $aVar["html"];
			}
		
			return $aVarsHtml;
		}
		else {
			foreach ($this->aVars AS $aVar) {
				if ($aVar["name"] == $Key) {
					return $aVar["html"];
				}
			}
		}
	}
	
	public function setHtml($Key, $Value)
	{		
		$this->aVars[$Key]["html"] = $Value;
	}
	
	/**
	 * @return void
	 * @access private
	 */
	private function initVarsDescription()
	{
		foreach (WF_ActionBox::newInstance()->getCfg() AS $aWFAppCfg) {
			if (array_key_exists("variables", $aWFAppCfg)) {
				foreach ($aWFAppCfg["variables"] AS $aVar) {
					$sType 			= isset($aVar["type"]) ? $aVar["type"] : "both";
					$this->aVars[] 	= array(
							"name"		=> $aVar["name"],
							"type"		=> $sType,
							"mode"		=> $aVar["mode"],
							"title"		=> $this->getTitle($aVar["name"]),
							"tag"		=> $this->getTag($this->getTitle($aVar["name"])),
							"classname"	=> $aVar["classname"],
							"classpath"	=> $this->getClasspath($aWFAppCfg["name"], $aVar["classname"]),
							"wfapp"	 	=> $aWFAppCfg["name"],
							"value"		=> null,
							"html"		=> "&#160;"
							);
				}

			}
			
		}
		
	}
	
	
	/**
	 * @return void
	 * @access private
	 */
	private function initVarsValues()
	{
		for ($i = 0; $i < count($this->aVars); $i++) {
			$Var = $this->loadVar($this->aVars[$i]);
			$this->aVars[$i]["value"]	= $Var->getValue();
			$this->aVars[$i]["html"]	= $Var->getHtml();
		}
	}
	
	/**
	 * Enter description here...
	 *
	 * @param array $aVar
	 * @return WF_IVariable
	 * @access private
	 */
	private function loadVar($aVar)
	{
		include_once $aVar["classpath"];
		$Class	= new ReflectionClass($aVar["classname"]);
		return $Class->newInstance($this->ActionSupport);
	}
	
	/**
	 * @param string $sVarname
	 * @return string
	 * @access private
	 */
	private function getTitle($sVarname)
	{
		return $this->oLocale->getWord("WORKFLOW_VAR_" . strtoupper($sVarname));
	}
	
	/**
	 * @param string $sWFApp
	 * @param string $sClassname
	 * @return string
	 * @access private
	 */
	private function getClasspath($sWFApp, $sClassname)
	{
		return CNF_ABS_PATH . "/modules/workflow/actions/" . $sWFApp . "/variables/" . $sClassname . ".php"; 
	}
	
	/**
	 * @param string $sVarname
	 * @return string
	 * @access private
	 */
	private function getTag($sVarname)
	{
		return "[" . /*strtoupper(*/$sVarname/*)*/ . "]";
	}
	
	public function getValue($Key)
	{
		foreach ($this->aVars AS $aVar) {
			if ($aVar["name"] == $Key) {
				return $aVar["value"];
			}
		}
	}

}

?>