<?php
/*
 * File: 	MSF_Actions.php
 * Date: 	25.01.2008
 * Time:	15:43:03
 * Detrix
 *
 */

// [include]
// include_once XXX;
// [/include]

/**
 * @package 	MSF_Actions
 * @copyright 	Copyright (c) 2007, Detrix
 */
class MSF_ActionBar
{
	private static $Instance = null;

	public static function getInstance($Tpl = null)
	{
		if (self::$Instance == null) {
			self::$Instance = new self($Tpl);
		}

		return self::$Instance;
	}

	private function __clone()
	{}

	private function __construct(MSF_Template $Template)
	{
		$this->Template 		= $Template;
		$this->ElmActions		= $this->Template->createMSFX("actionbar");
		$this->ElmLinksL  		= $this->Template->createElement("links-left");
		$this->ElmDropDownsL 	= $this->Template->createElement("dropdowns-left");
		$this->ElmLinksR 		= $this->Template->createElement("links-right");
		$this->ElmDropDownsR 	= $this->Template->createElement("dropdowns-right");

		$this->ElmActions->appendChild($this->ElmLinksL);
		$this->ElmActions->appendChild($this->ElmLinksR);
		$this->ElmActions->appendChild($this->ElmDropDownsL);
		$this->ElmActions->appendChild($this->ElmDropDownsR);
	}

	public function addAction($sValue, $sUrl, $sImage = CNX_ACTIONBAR_IMG, $sOnClick = "", $bNotSelect = false, $sSide = "left", $sHelp="")
	{
		static $i = 0;
		$Link = $this->Template->createElement("link");
		$Link->setAttribute("id", 		"actionbar_item_" . ++$i);
		$Link->setAttribute("title", MSF_Registry::get("MSF_Locale")->getI18N($sValue));
		$Link->setAttribute("url", 		$sUrl);
		$Link->setAttribute("image", 	$sImage);
		$Link->setAttribute("onclick", 	$sOnClick);
		$Link->setAttribute("help", 	MSF_Registry::get("MSF_Locale")->getI18N($sHelp));
		
		$sCurrentUrl = isset($_GET["msf_path"]) ? $_GET["msf_path"] : "";
		
		if ($this->getParsedUrl($sUrl) == $this->getParsedUrl($sCurrentUrl) && !$bNotSelect) {
			$Link->setAttribute("selected", "true");
			//$Link->setAttribute("selected", "false");
		}
		else {
			$Link->setAttribute("selected", "false");
		}
		
		if ($sSide == "left") {
			$this->ElmLinksL->appendChild($Link);
		}
		else {
			$this->ElmLinksR->appendChild($Link);
		}
	 }

	public function addDropDown($sValue, $aDropDowns, $sOnClick = "", $sSide = "left", $sHelp = "")
	{
	 	static $i 	= 0;
	 	$DropDown	= $this->Template->createElement("dropdown");
	 	$DropDown->setAttribute("title", MSF_Registry::get("MSF_Locale")->getI18N($sValue));
	 	$DropDown->setAttribute("onclick", $sOnClick);
	 	$DropDown->setAttribute("id", "actionbar_dropdown_item_" . ++$i);
	 	$DropDown->setAttribute("help", MSF_Registry::get("MSF_Locale")->getI18N($sHelp));
	 	
	 	if ($sSide == "left") {
	 		$this->ElmDropDownsL->appendChild($DropDown);
	 	}
	 	else {
	 		$this->ElmDropDownsR->appendChild($DropDown);
	 	}

	 	foreach($aDropDowns as $aDropDown) {
			$Element	= $this->Template->createElement("element");
			$DropDown->appendChild($Element);

			foreach($aDropDown as $sKey=>$sValue) {
				$Element->setAttribute($sKey, MSF_Registry::get("MSF_Locale")->getI18N($sValue));
			}

	 	}

	 }
	 
	public function addDropDropDown($aDropDowns)
	 {
	 	static $i 	= 0;
	 	$i++;
	 	$DropDropDown = $this->Template->createElement("dropdropdown");
	 	$DropDropDown->setAttribute("id", "actionbar_dropdropdown_item_" . $i);	 	
	 	$DropDropDown->setAttribute("itembar_id", "actionbar_dropdropdown_itembar_" . $i);	 		 		 		 	
	 	$this->ElmDropDownsL->appendChild($DropDropDown);
	 	foreach($aDropDowns AS $aDropDown) {
	 		$Dropdown	= $this->Template->createElement("dropdown");
	 		$DropDropDown->appendChild($Dropdown);
	 		
	 		foreach($aDropDown AS $Key => $Value) {
	 			if(!is_array($Value)) {
	 				$Dropdown->setAttribute($Key, $Value);
	 			}
	 			
	 		}
	 		
	 		if(isset($aDropDown["elements"])) {
		 		foreach($aDropDown["elements"] AS $aElement) {
		 			$Element = $this->Template->createElement("element");
		 			$Dropdown->appendChild($Element);
	
		 			foreach($aElement AS $sKey => $sValue) {
		 				$Element->setAttribute($sKey, $sValue);	
		 			}
		 			
		 		}
		 		
	 		}
	 		 	
	 	}
	 	
	 }

	 private function getParsedUrl($sUrl)
	 {
	 	if (!empty($sUrl)) {
	 		$iLength 	= strlen($sUrl);
	 		$sUrl 		= ($sUrl{$iLength - 1} == "/") ? substr($sUrl, 0, $iLength - 1) : $sUrl;
	 	}
	 	
	 	return $sUrl;
	 }
}
?>