<?php
if (!defined("__MSF")) {
	exit("Direct access is not allowed");
}
require_once "modules/eventlog/library/ELOG_EventLog.php";

abstract class MSF_AbstractAuth
{
	/**
	 * @var MSF_DB
	 */
	protected $oDb 			= null;
	
	/**
	 * @var MSF_UserBox
	 */
	protected $oUb			= null;
	
	/**
	 * @var MSF_Template
	 */
	protected $oTemplater 	= null;
	
	/**
	 * @var MSF_Messenger
	 */
	protected $oMessenger	= null;
	
	protected $aConfig		= 0;
	
	protected $sUri			= "";
        
        protected $oAuthDb          = null;
	
	public function __construct($aConfig)
	{
		$this->sUri			= $_SERVER["REQUEST_URI"];
		$this->aConfig		= $aConfig;
		$this->oDb 			= MSF_Registry::get("MSF_DB");
		$this->oUb 			= MSF_Registry::get("MSF_UserBox");
		$this->oTemplater	= new MSF_Template();
		$this->oMessenger 	= new MSF_Messenger($this->oTemplater);
                $this->oAuthDb          = new MSF_AuthenticationDB($this->oDb);
	}
	
	public function getUri()
	{
		return $this->sUri;
	}
	
	/**
	 */
	public function getConfigPage($iId, $sUrl)
	{
		$sTmp 	= "";
		if ($this->oUb->getUserAttribute("start_page_type") == 0) {
			$sTmp 		= $this->oUb->getUserAttribute("page_url");
		}
		else {
			$LastUrl 	= new MSF_LastUrl(new MSF_LastUrlDB($this->oDb), $iId);
			$sTmp 	 	= $LastUrl->getUrl();
		}
		
		$sTmp = trim(trim($sTmp), "/");
		$sUrl = trim(trim($sUrl), "/");
		if (!empty($sTmp)) {
			$sUrl = $sTmp;
		}
		elseif (empty($sUrl)) {
			$sUrl = CNF_DEFAULT_MODULE;
		}
		
		return $sUrl;
	}
	
	public function logout($sUrl = "/")
	{
		$iCurrentUserId 	= $this->oUb->getUserAttribute("id");
		$this->oUb->destroy();
		unset($_SESSION["SID"]);
		unset($_SESSION["User"]);
		$_SESSION = array();
		//session_destroy();
		session_regenerate_id();
		$oUDb = new MSF_AuthenticationDB($this->oDb);
		$oUDb->delete(strlen($iCurrentUserId) ? $iCurrentUserId : 0);
		HEADER("Location: " . $sUrl);
		exit();
	}
	
	public function generateSession()
	{
		$sSessionId = md5(MSF_String::random(10) . microtime());
		$aUser["unique_id"] 	= $sSessionId;
		$aUser["session_time"] 	= $this->getTime();
		//ini_set('session.gc_maxlifetime', /*60*$this->aConfig["session_time"]*/1);
		$_SESSION["User"] 		= $aUser;
		
		return $sSessionId;
	}

	public function recordSession()
	{
                $rUserId 	= $this->oUb->getUserAttribute("id");
                $sSessionID = $this->oAuthDb->getSessionId($rUserId);
		$aUser["unique_id"] 	= $sSessionID;
		$aUser["session_time"] 	= $this->getTime();
		$_SESSION["User"] 		= $aUser;		
		//return $sSessionId;
	}
        
        
	public function getTime()
	{
		return time() + 60 * $this->aConfig["session_time"];
	}
	
	public function redirectByUri() 
	{
		$iId = $this->oUb->getUserAttribute("id");
		$this->redirect($this->getConfigPage($iId, $this->sUri));
	}
	
	public function redirect($sUrl) 
	{
		HEADER("Location: " . $sUrl);
		exit();
	}

	public function redirectById($iId)
	{
		$sUrl = $_SERVER["REQUEST_URI"];
		if ($sUrl == "/") {
			$sUrl = $this->getConfigPage($iId, $sUrl);
		}
		elseif (strpos($sUrl, 'login') == 1) {
			$sUrl = "/";
		}
		$this->redirect($sUrl);
	}
	
    public function isLogout()
    {
		return strstr($this->sUri, "/logout") != "";
    }
    
    protected function loginById($iUserId, $isRedirectable = true)
    {
    	if ($iUserId > 0) {
			$this->oUb->load($iUserId);
			if ($this->oUb->getUserAttribute("is_active") == 1) {
				$sSessionId = $this->generateSession();
				if (!$this->oAuthDb->exists($iUserId)) {
					
                                        $this->oAuthDb->insert($iUserId, $sSessionId);
				}
				else {
					$this->recordSession();
                                        //$this->oAuthDb->update($iUserId, $sSessionId);
				}
				
				$this->oAuthDb->insertLog($iUserId, $_SERVER["REMOTE_ADDR"], time());
				$oEventLog = new ELOG_EventLog($this->oDb);
				$oEventLog->addItem(array("event_type"=>"login"));
				
			}
			else {
				unset($_SESSION["MSF_UserBox"]);
				$this->oMessenger->addError(MSF_Registry::get("MSF_Locale")->getWord("ROOT_MESSAGE_USER_BLOCKED"));
			}
		}
		else {
			if($isRedirectable) {
				$this->sendMessage();
				$this->oMessenger->addError(MSF_Registry::get("MSF_Locale")->getWord("ROOT_MESSAGE_USER_INCORRECT"));
			}
		}
		
    }
    
    abstract public function getLoginAndPassword();
    	
	abstract function isLogin();
	
	abstract public function login($sLogin, $sPassword);
	
	abstract public function render();
	
	abstract public function verify();
	
	abstract function sendMessage(); 
	
}

?>