<?php
if (!defined("__MSF")) {
	exit("Direct access is not allowed");
}

include_once "MSF_AbstractAuth.php";
include_once "MSF_LdapAuthentication.php";
include_once "MSF_NtlmPrompt.php";
class MSF_NtlmAuthentication extends MSF_LdapAuthentication
{
	/**
	 * @var MSF_NtlmPrompt
  	 */
	private $oPrompt;
	
	public function __construct($aConfig)
	{
		parent::__construct($aConfig);
		$this->oPrompt = new MSF_NtlmPrompt();
	}
	
	public function ntlmLogin($sLogin)
	{
		$iUserId = $this->oAuthDb->getUserIdByLogin($sLogin);
		$this->loginById($iUserId);
		return $iUserId;
	}
	
	private function authUser($sSubjectUser)
	{
		$sUser 			= $this->aConfig['ldap_user'] . '@' . $this->aConfig['ldap_domain'];
		$sPassword 		= $this->aConfig["ldap_password"];
		$rConnection 	= $this->ldapConnect();
		$bResult 		= false;
		if ($rConnection) {
			if ($this->ldapBind($rConnection, $sUser, $sPassword)) {
				$sFilter	= '(userPrincipalName=' . $sSubjectUser . ')';
				$rSearch 	= ldap_search($rConnection, $this->getDc(), $sFilter);
				$rEntry		= ldap_first_entry($rConnection, $rSearch);
				$bResult 	= $rEntry ? true : false; 
			} 

			ldap_close($rConnection);
		}

		return $bResult;
	}
	
	private function getDc()
	{
		$aDc = array();
		foreach (explode(".", $this->aConfig["ldap_domain"]) AS $sDc) {
			$aDc[] = "dc=" . $sDc;
		}
		
		return implode(",", $aDc);
	}
	
	public function render()
	{
		
		switch (MSF_SL::getParam($_GET, "sso", "on")) {
			case "off":
				parent::render();
				break;
			case "who":
				$oPrompt = new MSF_NtlmPrompt();
				if ($oPrompt->isAuthorized()) {
					echo "Request from " . $oPrompt->getUser() . "/" . $oPrompt->getWorkstation() . "<br/>";
					echo "Accept " . $this->getUser($oPrompt->getUser(), $oPrompt->getUsername()) . "<br/>";
					echo "NetBIOS: " . $this->aConfig["ldap_nbios"] . "<br/>";
					echo "Domain: " . $this->aConfig["ldap_domain"] . "<br/>";
				}
				else {
					echo "NTLM prompt error";
				}
				
				exit();
				break;
			case "on":
				default:
				$sUser 		= $this->getUser($this->oPrompt->getUser(), $this->oPrompt->getUsername());
				if (!$this->oPrompt->isAuthorized()) {
					parent::render();
				}
				else if ($this->authUser($sUser)) {
					$sUser = str_replace(strstr($sUser, "@"), "", $sUser);
					$iUser = $this->ntlmLogin($sUser);
					$this->oPrompt->sendOk();
					if ($iUser) {
						$this->redirectById($iUser);
					}
					else {
						$this->redirect('/?sso=off');
					}
					
				}
				else {
					$this->redirect('/?sso=off');
				}
				
				break;
			
		}
		
	}
	
	public function getUser($sUser, $sUsername)
	{
		$sNbios = $sUsername . "@" . $this->aConfig["ldap_nbios"];
		if ($sUser == $sNbios) {
			$sUser = $sUsername . "@" . $this->aConfig["ldap_domain"];
		}
		
		return $sUser;
	}
	
	public function logout($sUrl = "/")
	{
		if ($sUrl == "/") {
			parent::logout($sUrl . "?sso=off");
		}
		else {
			parent::logout($sUrl);
		}
		
	}
	
}

?>