<?php

defined("__MSF") or die("Direct access is not allowed");

class MSF_Authorization
{
	private static $Instance 	= null;
	
	private $Router 			= null;
	
	private $aModules 			= array();
	
	private $Settings 			= null;
	
	private $aServices 			= array();
	
	private function __construct()
	{
		$this->Router 		= MSF_Registry::get("MSF_Router");
		$UserBox 			= MSF_Registry::get("MSF_UserBox");
		$this->Settings 	= MSF_Registry::get("MSF_Settings");
		$this->aModules 	= $UserBox->getStructureAttribute("modules");
		$this->aAllModules 	= $this->Settings->getModulesList();
	}
	
	private function __clone()
	{}
	
	public static function getInstance()
	{
		if (self::$Instance == null) {
			self::$Instance = new self();
		}
		
		return self::$Instance;
	}
	
	public function getCurrentModule()
	{
		return $this->Router->getModule();
	}
	
	public function getModules()
	{
		return $this->aModules;
	}
	
	public function hasAccess($sModule = "")
	{
		$bAccess = false;
		
		if (empty($sModule)) {
			$sModule = $this->getCurrentModule();
		}
		
		for ($j = 0; $j < sizeof($this->aAllModules); $j++) {
			$aSettings = $this->Settings->getXmlWrapper()->get("modules")->getValue($this->aAllModules[$j]);
			
			if (isset($aSettings["settings"]["visible"])) {
				if ($sModule == $this->aAllModules[$j] && $aSettings["settings"]["visible"] == 0) {
					$bAccess = true;
					break;
				}
				else {
					for ($i = 0; $i < sizeof($this->aModules); $i++) {
						if (($this->aModules[$i]["module_name"] == $this->aAllModules[$j])
							&& ($this->aModules[$i]["module_name"] == $sModule)) {
							if ($this->Router->isAdminMode() && $this->aModules[$i]["back"] == 1) {
								$bAccess = true;
								break;
							}
							if (!$this->Router->isAdminMode() && $this->aModules[$i]["front"] == 1) {
								$bAccess = true;
								break;
							}
						}
					}
				}
			}
		}
		
		return $bAccess;
	}
	
	public function hasAccessMode($sMode, $sModule)
	{
		if ($sMode != "back" && $sMode != "front") {
			throw new MSF_ThrowableException("Mode value must be \"front\" or \"back\"");
		}
		
		$bAccess = false;
		
		if (empty($sModule)) {
			$sModule = $this->getCurrentModule();
		}
		
		for ($i = 0; $i < sizeof($this->aModules); $i++) {
			if ($sModule == $this->aModules[$i]["module_name"]) {
				if ($this->aModules[$i][$sMode] == 1) {
					$bAccess = true;
				}
			}
		}
		
		return $bAccess;
	}
	
	public function hasAdminAccess()
	{
		$bAccess = false;
		
		for ($i = 0; $i < sizeof($this->aModules); $i++) {
			if ($this->aModules[$i]["back"] == 1) {
				$bAccess = true;
			}
		}
		
		return $bAccess;
	}
}

?>