<?php

defined("__MSF") or die("Direct access is not allowed");

class MSF_UserBox
{
	const USER 					= "user";
	
	const EMPLOYEE 				= "employee";
	
	const POST 					= "post";
		
	const SUBDIVISION 			= "subdivision";
	
	const STRUCTURE 			= "structure";

	private static $Instance 	= null;
	
	private static $aCache 		= array();
	
	private $sSessionName 		= "MSF_UserBox";
	
	/**
	 * @var MSF_DB
	 */
	private $Db 				= null;
	
	/**
	 * @var MSF_DBTree
	 */
	private $Tree 				= null;
	
	/**
	 * @var UserReference
	 */
	private $Reference 			= null;
	
	private $bSession 			= false;
	
	private $aUsers 			= array();
	
	private $aEmployees 		= array();

	private $aPosts 			= array();
	
	private $aSubdivs 			= array();
	
	private $aTree 				= array();
	
	private $aUboxBlank 		= array(
		"user" 					=> array(),
		"employee" 				=> array(),
		"posts" 				=> array(),
		"structure" 			=> array(),
		"currentPostId" 		=> 0,
		"currentStructureId" 	=> 0,
		"extraStructureIds" 	=> array(),
		"currentStructurePath" 	=> ""
	);
	
	private $aUBox 		= array(
		"user" 					=> array(),
		"employee" 				=> array(),
		"posts" 				=> array(),
		"structure" 			=> array(),
		"currentPostId" 		=> 0,
		"currentStructureId" 	=> 0,
		"extraStructureIds" 	=> array(),
		"currentStructurePath" 	=> ""
	);
	
	
	public function find($sLogin, $sPassword)
	{
		$iId = 0;
		$sLogin = $this->loginLow($sLogin);

		for ($i = 0; $i < count($this->aUsers); $i++) {
			if ($this->loginLow($this->aUsers[$i]["login"]) == $sLogin && $this->aUsers[$i]["password"] == $sPassword) {
				$iId = $this->aUsers[$i]["id"];
				break;
			}
			
		}
		return $iId;
	}
	
	
	private function loginLow($sLogin) 
	{
		$aLow	   = array('а','б','в','г','д','е','ё','ж','з','и','й','к','л','м','н','о','п','р',
						   'с','т','у','ф','х','ц','ч','ш','щ','ъ','ы','ь','э','ю','я');
		$aUp	   = array('А','Б','В','Г','Д','Е','Ё','Ж','З','И','Й','К','Л','М','Н','О','П','Р',
						   'С','Т','У','Ф','Х','Ц','Ч','Ш','Щ','Ъ','Ы','Ь','Э','Ю','Я');
		
		
		return strtolower(str_replace($aUp, $aLow, $sLogin));
	}
	
	
	public function load($iId, $sType = self::USER, $notEmptyUsers = false)
	{
		$this->aUBox = $this->aUboxBlank;
		switch ($sType) {
			case self::USER:
				$this->loadUser($iId);
				break;
			case self::EMPLOYEE:
				$this->loadEmployee($iId);
				break;
			case self::POST:
				$this->loadPost($iId);
				break;
			case self::SUBDIVISION:
				$this->loadSubdivision($iId);
				break;
			case self::STRUCTURE:
				$this->loadStructure($iId);
				break;
		}
		
		if ($notEmptyUsers) {
			$aFinal = array();
			foreach ($this->aUBox as $aItem) {
				if (!empty($aItem["user"])) {
					$aFinal[] = $aItem;
				}
				
			}
			
			$this->aUBox = $aFinal;
		}
		
		if (!$this->bSession) {
			$_SESSION[$this->sSessionName] = $this->aUBox;
		}

		return $this->aUBox;
	}
	
	public function getUser($iUserId)
	{
		foreach ($this->aUsers as $aUser) {
			if ($aUser["id"] == $iUserId) {
				unset($aUser["password"]);
				return $aUser;
			}
			
		}
		
		return null;
	}
	
	public function getEmployee($iEmployeeId)
	{
		foreach ($this->aEmployees as $aEmployee) {
			if ($aEmployee["id"] == $iEmployeeId) {
				return $aEmployee;
			}
			
		}
		
		return null;
	}
	
	public function getPost($iPostId)
	{
		foreach ($this->aPosts as $aPost) {
			if ($aPost["id"] == $iPostId) {
				return $aPost;
			}
			
		}
		
		return null;
	}
	
	
	public function getRefValues($sType)
	{
		switch ($sType) {
			case self::USER:
				return $this->aUsers;
			case self::EMPLOYEE:
				return $this->aEmployees;
			case self::POST:
				return $this->aPosts;
			case self::SUBDIVISION:
				return $this->aSubdivs;
		}
		
		return null;
	}
	
	public function getLoadedResult()
	{
		return $this->aUBox;
	}
	
	public function getUserAttribute($sAttribute)
	{
		$sReturn = $this->getSessionAttribute("user", $sAttribute);
		
		if(!empty($sReturn) && in_array($sAttribute, array("is_active", "is_admin", "email_remove_on_recieve", "email_send_copy", "start_page_type"))) {
			$aValue 	= MSF_EnumValue::parse($sReturn);
			$sReturn 	= 0;
			if($aValue) {
				$sReturn = $aValue["key"];
			}
			
		}
		
		return $sReturn;
	}
	
	public function getEmployeeAttribute($sAttribute)
	{
		return $this->getSessionAttribute("employee", $sAttribute);
	}
	
	public function getPosts()
	{
		return $this->getSessionAttribute("posts");
	}
	
	public function getCurrentPostId()
	{
		return $this->getSessionAttribute("currentPostId");
	}
	
	public function getCurrentStructureId()
	{
		return $this->getSessionAttribute("currentStructureId");
	}
	
	public function getCurrentStructurePath()
	{
		return $this->getSessionAttribute("currentStructurePath");
	}
	
	public function getStructureAttribute($sAttrubute)
	{
		return $this->getSessionAttribute("structure", $sAttrubute);
	}
	
	public function searchTreePath($sPath)
	{
		$this->Tree->searchByValues(array("path" => $sPath));
	}
	
	public function isAdmin()
	{
		return $this->getUserAttribute("is_admin");
	}
	
	public function destroy()
	{
		unset($_SESSION[$this->sSessionName]);
	}

	private function loadUser($iId)
	{
		$this->aUBox["user"] = $this->getUser($iId);
		if (!empty($this->aUBox["user"])) {
			$this->aUBox["employee"] = $this->getEmployeeByUserRefValue($this->aUBox["user"]["employee_ref_value"]);
		}
		
		if (!empty($this->aUBox["employee"])) {
			$this->aUBox["posts"] = $this->getPostsByEmployeeId($this->aUBox["employee"]["id"]);
		}
		
		if (!empty($this->aUBox["posts"])) {
			$this->aUBox["structure"] = $this->getStructureByPosts($this->aUBox["posts"]);
			$this->defineDefaultParams();
		}

		//debug($this->aUBox["structure"]);
		//exit();
	}
	
	private function loadEmployee($iId)
	{
		$this->aUBox["user"] 		= $this->getUserByEmployeeId($iId);
		$this->aUBox["employee"] 	= $this->getEmployee($iId);
		$this->aUBox["posts"] 		= $this->getPostsByEmployeeId($iId);
		
		if (!empty($this->aUBox["posts"])) {
			$this->aUBox["structure"] = $this->getStructureByPosts($this->aUBox["posts"]);
			$this->defineDefaultParams();
		}
		
	}

	private function loadSubdivision($iId)
	{
		foreach ($this->aTree as $aItem) {
			if ($aItem["type"] == "subdivision" && $aItem["subject_id"] == $iId) {
				$this->loadStructure($aItem["node_id"]);
				break;
			}
			
		}
		
	}

	private function getStructureContent($iId)
	{
		foreach ($this->aTree as $aItem) {
			if ($aItem["node_id"] == $iId) {
				return $aItem;
			}
			
		}
		
		return array();
	}
	
	private function getEmployeeByPostId($iId)
	{
		$RefValue = new REF_Value();
		
		foreach ($this->aPosts as $aItem) {
			if ($iId == $aItem["id"]) {
				$RefValue->parse($aItem["employee_ref_value"]);
				$aIds = $RefValue->getKeysList();
				foreach ($this->aEmployees as $aEmp) {
					if (in_array($aEmp["id"], $aIds)) {
						return $aEmp;
					}
					
				}
				
			}
			
		}
		
		return array();
	}
	
	private function getEmployeeByUserRefValue($sRefValue)
	{
		$RefValue = new REF_Value();
		$RefValue->parse($sRefValue);
		$aIds = $RefValue->getKeysList();
		
		foreach ($this->aEmployees as $aItem) {
			if (in_array($aItem["id"], $aIds)) {
				return $aItem;
			}
			
		}
		
		return array();
	}
	
	private function getUserByEmployeeId($iId)
	{
		$RefValue = new REF_Value();
		
		foreach ($this->aUsers as $aItem) {
			$RefValue->parse($aItem["employee_ref_value"]);
			$aIds = $RefValue->getKeysList();
			if (in_array($iId, $aIds)) {
				return $aItem;
			}
			
		}
		
		return array();
	}
	
	public function getPostsByEmployeeId($iId)
	{
		$aResult 	= array();
		$RefValue 	= new REF_Value();
		$aIds 		= array();
		
		foreach ($this->aPosts as $aPost) {
			$RefValue->parse($aPost["employee_ref_value"]);
			if (in_array($iId, $RefValue->getKeysList())) {
				foreach ($this->aTree as $aItem) {
					if ($aItem["type"] == "post" && $aItem["subject_id"] == $aPost["id"]) {
						$aTmp 				= $aPost;
						$aTmp["node_id"] 	= $aItem["node_id"];
						$aResult[] 			= $aTmp;
						$aIds[] 			= $aPost["id"];
						break;
					}
					
				}
				
			}
			
		}
		
		foreach ($this->aTree as $aItem) {
			if (!empty($aItem["additional_subject_ids"])) {
				$aExtTreeIds = explode(",", $aItem["additional_subject_ids"]);
				foreach ($this->aTree as $aTree) {
					if ($aTree["type"] == "post" && in_array($aTree["node_id"], $aExtTreeIds) && in_array($aTree["subject_id"], $aIds)) {
						foreach ($this->aPosts as $aPost) {
							if ($aPost["id"] == $aItem["subject_id"]) {
								$aTmp 					= $aPost;
								$aTmp["node_id"] 		= $aItem["node_id"];
								$aTmp["boss"] 			= 1;
								$aTmp["last_selected"] 	= $aPost["last_selected"];
								$aResult[] 				= $aTmp;
								break;
							}
							
						}
						 
					}
					
				}
				
			}
			
		}
		
		return $aResult;
	}
	
	private function getSubdivisionById($iId)
	{
		foreach ($this->aSubdivs as $aItem) {
			if ($iId == $aItem["id"]) {
				return $aItem;
			}
		}
		
		return array();
	}
	
	private function getStructureByPosts($aPosts)
	{
		$aResult 	= array();
		$aIds 		= array();
		$aNIds 		= array();
		$aSelected 	= array();
		$bEx 		= false;
		
		foreach ($aPosts as $aItem) {
			$aIds[] 	= $aItem["id"];
			$aNIds[] 	= $aItem["node_id"];
			if (!$bEx && $aItem["last_selected"]) {
				$aSelected 	= $aItem;
				$bEx 		= true;
			}
			
		}
		
		if (!$bEx) {
			$aSelected = MSF_SL::getParam($aPosts, 0);
		}

		$aSIds 	= array();
		$aPList = array();
		//debug($aSelected,$aPosts);
		foreach ($this->aTree as $aItem) {
			if (in_array($aItem["subject_id"], $aIds) && in_array($aItem["node_id"], $aNIds)) {
				$aSIds[] 	= $aItem["node_id"];
				$aPList[] 	= $aItem["path"];
			}
			
			if ($aItem["type"] == "subdivision") {
			
				foreach ($this->aTree as $aEl) {
					if ($aEl["node_id"] == $aSelected["node_id"] && $aItem["node_id"] == $aEl["parent_id"]) {
						$aSubd = $this->getSubdivisionById($aItem["subject_id"]);
						if (isset($aSubd["id"]) && isset($aSubd["subdivision"])) {
							$aResult["subdivision_id"] 	= $aSubd["id"];
							$aResult["subdivision"] 	= $aSubd["subdivision"];							
						}
						else {
							$aResult["subdivision_id"] 	= 0;
							$aResult["subdivision"] 	= "?";	
						}

						break;
					}
					
				}
				
			}
			
		}

		foreach ($aPosts as $aItem) {
			foreach ($aSIds as $iIndex => $sValue) {
				if ($aItem["node_id"] == $sValue) {
					$aResult["id_list"][] 		= $sValue;
					$aResult["paths_list"][] 	= $aPList[$iIndex];
				}
				
			}
			
		}
		
		return $aResult;
	}
	
	private function getSessionAttribute($sKey, $sAttribute = "")
	{
		$mResult = null;
		
		if(isset($_SESSION[$this->sSessionName][0])) {
			$aData = $_SESSION[$this->sSessionName][0];
		}
		else {
			if(isset($_SESSION[$this->sSessionName])) {
				$aData = $_SESSION[$this->sSessionName];
			}
			
		}
		
		if (strlen($sAttribute) > 0) {
			$mResult = isset($aData[$sKey][$sAttribute]) ? $aData[$sKey][$sAttribute] : null;
		}
		else {
			$mResult = isset($aData[$sKey]) ? $aData[$sKey] : null;
		}
		
		return $mResult;
	}

	public static function getInstance()
	{
		if (self::$Instance == null) {
			self::$Instance = new self(isset($_SESSION["MSF_UserBox"]));
		}
		
		return self::$Instance;
	}
	
	private function __construct($bSession)
	{
		//debug($_SESSION);
		$this->Db 			= MSF_Registry::get("MSF_DB");
		$this->Tree 		= new MSF_DBTree($this->Db, "structure", "node_id", "parent_id", "path");
		$this->Reference 	= new UserReference(new UserReferenceDB($this->Db));
		$this->bSession 	= $bSession;
		
		$this->initTree();
		$this->initRefs();
	}
	
	private function initTree()
	{
		$this->aTree = $this->Tree->select(
			1, array("subject_id", "path", "type", "additional_subject_ids","position"),
			array(), array(), false
		);
	}
	
	private function initRefs()
	{
		$this->Reference->load(1);
		$this->aUsers 		= $this->Reference->getAsArray();
		$this->Reference->load(2);
		$this->aEmployees 	= $this->Reference->getAsArray();
		$this->Reference->load(3);
		$this->aPosts 		= $this->Reference->getAsArray();
		$this->Reference->load(4);
		$this->aSubdivs 	= $this->Reference->getAsArray();
	}
	
	private function __clone() {}
	
	public function isStructureId($iStrId)
	{
		return $this->Tree->exists($iStrId);
	}

	private function loadPost($iId, $iStrId = 0, $sStrPath = '0')
	{
		$aPost = $this->getPost($iId);
		
		if (!empty($aPost)) {
			$this->aUBox["employee"] = $this->getEmployeeByPostId($aPost["id"]);
			if (!empty($this->aUBox["employee"])) {
				$this->aUBox["user"] 		= $this->getUserByEmployeeId($this->aUBox["employee"]["id"]);
				$this->aUBox["posts"] 		= $this->getPostsByEmployeeId($this->aUBox["employee"]["id"]);
				$this->aUBox["structure"] 	= $this->getStructureByPosts($this->aUBox["posts"]);
				$this->defineDefaultParams($iStrId, $sStrPath);
			}
			
		}

	}
	
	private function loadStructure($iId)
	{
		$aStr = $this->getStructureContent($iId);
		if (!empty($aStr)) {
			if ($aStr["type"] == "post") {
				$this->loadPost($aStr["subject_id"], $iId, $aStr['path']);
				$this->aUBox = array($this->aUBox);
			}
			else {
				$aResult = array();
				foreach ($this->aTree as $aItem) {
					$aAds = explode(',', $aItem['additional_subject_ids']);
					if (!empty($aAds)) {
						foreach ($aAds AS $sStrId) {
							$iStrId = (int) $sStrId;
							if ($iStrId) {
								$iPostId = (int)MSF_OrgStructureFacade::getInstance()->getPostIdByStructureId($iStrId);
								if ($iPostId) {
									$this->loadPost($iPostId);
									$aResult[] = array($this->aUBox);
								}
								
							}

						}
						
					}

					if ($aItem["type"] == "post" && substr($aItem["path"], 0, strlen($aStr["path"])) == $aStr["path"]) {
						$this->loadPost($aItem["subject_id"], $aItem["node_id"], $aItem["path"]);
						$aResult[] = $this->aUBox;
					}

				}
				
				$this->aUBox = $aResult;
			}
			
		}

	}
	
	private function defineDefaultParams($iStrId = 0, $sStrPath = '0')
	{
		//echo ':' . $iStrId . '<br/>';
		//echo "<pre>";
		//debug_print_backtrace();
		//echo "</pre>";
		$bLast = false;
		foreach ($this->aUBox["posts"] as $aPost) {
			if ($aPost["last_selected"] == 1) {
				$bLast = true;
			}

			$aCont = $this->getStructureContent($aPost["node_id"]);
			if (!isset($aPost["boss"]) && !empty($aCont["additional_subject_ids"])) {
				$this->aUBox["extraStructureIds"] = array_merge(MSF_SL::getParam($this->aUBox, 'extraStructureIds', array()), explode(",", $aCont["additional_subject_ids"]));
			}
			
		}
		
		foreach ($this->aUBox["posts"] as $aPost) {
			if ($aPost["last_selected"] == 1 || !$bLast) {
				$this->aUBox["currentPostId"] 	= $aPost["id"];
				$iIndex 						= 0;
				//debug($this->aUBox["structure"]["id_list"],$aPost["id"]);
				foreach ($this->aUBox["structure"]["id_list"] as $iKey => $iItem) {
					if (($iStrId > 0) && ($iStrId == $iItem)) {
						$iIndex 							= $iKey;
						$this->aUBox["currentStructureId"] 	= $iStrId;
						break;
					}
					else if (($iStrId == 0) && ($aPost["node_id"] == $iItem)) {
						$iIndex 							= $iKey;
						$this->aUBox["currentStructureId"] 	= $iItem;
						break;
					}
				
				}
				
				if (isset($this->aUBox["structure"]["paths_list"][$iIndex])) {
					$this->aUBox["currentStructurePath"] = $this->aUBox["structure"]["paths_list"][$iIndex];
				}
				
				break;
			}
			
		}
		
		//echo '- ' . $this->aUBox["currentStructurePath"] . '<br/>';
	}
	
}

?>