<?php


require_once "debug.php";
require_once "exitApp.php";
require_once "msfErrorHandler.php";
require_once "MSF_ThrowableException.php";
require_once "MSF_IOException.php";
require_once "MSF_ParsingException.php";
require_once "MSF_PageNotFoundException.php";
require_once "MSF_Settings.php";
require_once "utils/MSF_XmlWrapper.php";
require_once "utils/MSF_SL.php";
require_once "utils/MSF_Serialization.php";
require_once "utils/MSF_Charset.php";
require_once "utils/MSF_String.php";
require_once "utils/MSF_Date.php";
require_once "utils/MSF_File.php";
require_once "utils/MSF_Dir.php";
require_once "utils/MSF_Json.php";
require_once "utils/MSF_DB.php";
require_once "MSF_Registry.php";
require_once "MSF_Template.php";
require_once "MSF_Router.php";
require_once "MSF_Locale.php";
require_once "MSF_Mcap.php";
require_once "mvc/MSF_ControllerSupport.php";
require_once "mvc/MSF_ViewSupportXml.php";
require_once "mvc/MSF_ViewSupportJson.php";
require_once "mvc/MSF_WSControllerSupport.php";
require_once "mvc/MSF_DataControllerSupport.php";
require_once "MSF_MsfxSupport.php";
require_once "MSF_ServiceLoader.php";
require_once "MSF_IService.php";
require_once "MSF_ActionSupport.php";
require_once "MSF_NotValidClassException.php";
require_once "MSF_IDaemon.php";
require_once "mvc/MSF_ModelDBSupport.php";
require_once "utils/MSF_Debugger.php";
require_once "utils/MSF_AccessDenied.php";
require_once "utils/MSF_HtmlEntities.php";
include_once "utils/MSF_Cache.php";
include_once "utils/MSF_Logger.php";
include_once "MSF_Hook.php";
require_once 'MSF_DbCache.php';

final class MSF_Core
{
	const __VERSION = "1.2.2";

	private static $bIsCoreLoaded	= false;

	private $aMsfx	 				= array();

	private $Router 				= null;

	private $bIsTestMode			= false;

	private $Settings				= null;
	
	private $oCache					= null;
	
	private $isMainUpdater 			= 0; 
	
	public static function initialize()
	{
		if (!self::$bIsCoreLoaded) {
			new MSF_Core();
			self::$bIsCoreLoaded = true;
		}

		return self::$bIsCoreLoaded;
	}

	private function __clone()
	{}

	private function __construct()
	{
		$this->processMain();
	}

	private function processMain()
	{
		try {
			$this->initSession();
			if ($this->updaterTime()) exit;
			//echo time() . "<br/>";
			$this->initConstants();
			$this->obStart();
			$this->initErrorHandler();
			$this->initCache();
			$this->Settings = $this->oCache->get("MSF_Settings");
			$oLocale 		= $this->oCache->get("MSF_Locale");
			if ((MSF_SL::getParam($_GET, "cache", "on")!== "off") && $this->oCache->isEnabled() && ($this->Settings !== null) && ($oLocale !== null)) {
                            MSF_Registry::set("MSF_Settings", $this->Settings);
				$this->oCache->set("MSF_Settings", MSF_Registry::get("MSF_Settings"));
				$this->initSettingsConstants();
                                $this->initDB();
				MSF_Registry::set("MSF_Locale", $oLocale);
			}
			else {
				$this->initSettings();
				$this->initSettingsConstants();
                                $this->initDB();
				$this->initLocale();
				$this->cache();
			}

			$this->initTemplate();			
			$this->initHook();
			$this->initIncludePath();
	        $this->initRouter();
	        if (!$this->Router->isWs() && !$this->Router->isData()) {
	        	$this->initMsfx();
	        }
	     
    	        $this->initController();
	        $this->shutdownDB();
			$this->oCache->serializeAndSave();
			
		}
		catch (MSF_TrowableException $TE) {
			MSF_Logger::logTrace($TE);
			exitApp($this->getTrace($TE));
		}
		catch (Exception $E) {
			MSF_Logger::logTrace($E);
			exitApp($this->getTrace($E));
		}

		$this->obEndFlush();
	}
	
	private function initHook()
	{
		$oHook = new MSF_Hook();
		$oHook->process();
	}
	
	private function updaterTime()
	{
		if (strstr(MSF_SL::getParam($_GET, "msf_path"), "updater") && isset($_SESSION["MSF_Updater"])){
			if (!isset($_SESSION["MSF_Updater"]["busy"])) {
				$_SESSION["MSF_Updater"]["busy"] = time();
			}
			if (time() - $_SESSION["MSF_Updater"]["busy"] > 30) {			
				$_SESSION["MSF_Updater"]["busy"] = time();
				$this->isMainUpdater = 1;
			}
			else {
				echo json_encode(array("id"	=> 777, "count" => $_SESSION["MSF_Updater"]["lmsInboxNewNum"], "alert" => ""));
				return true;
			}
			
		}
		
		return false;
	}
	
	private function initCache() 
	{
		$this->oCache = MSF_Cache::getInstance();
		MSF_Registry::set("MSF_Cache", $this->oCache);
	}
	
	private function cache()
	{
		$oCache		= MSF_Registry::get("MSF_Cache");
		$aParams 	= MSF_Registry::get("MSF_Settings")->getCacheParams();
		$oCache->set("MSF_Cache.isEnabled", $aParams['enabled']);
		if ($aParams['enabled'] == 1) {
			$oCache->connect($aParams["mc_host"], $aParams["mc_port"]);
			$oCache->set("MSF_Settings", MSF_Registry::get("MSF_Settings"));
			$oCache->set("MSF_Locale", MSF_Registry::get("MSF_Locale"));	
		}

	}
	
	private function initSettingsConstants()
	{
            $support_locale = "";
		foreach ($this->Settings->getConstants() AS $sConstantName => $sConstantValue) {
			if ($sConstantName == 'CNF_ABS_PATH')
				continue; 
                        if ($sConstantName == 'CNF_LANGUAGE'){
                            $default_lang = "";
                            if(is_array($sConstantValue)){
                                foreach ($sConstantValue as $lang => $default){
                                    $support_locale .= "$lang:'$lang',";
                                    if($default == "default"){
                                        $default_lang = $lang;
                                    }
                                }
                            } else {
                                $default_lang ='ru';
                                $support_locale .= "ru:'ru'";
                            }
                            $sConstantValue = $default_lang;
                        }
			DEFINE($sConstantName, $sConstantValue);
		}
                $support_locale = trim($support_locale, ',');
		DEFINE('SUPPORT_LOCALE',$support_locale);
		date_default_timezone_set(CNF_TIME_ZONE);
	}


	private function initSession()
	{
		session_name("MSF" . substr(md5($_SERVER["REMOTE_ADDR"]), 0, 5));
		session_start();
	}

	private function initConstants()
	{
		DEFINE("MSF_TIME_START",		MSF_SL::getMicrotime());
		DEFINE("MSF_DATE" , 			gmdate("D, d M Y H:i:s") . " GMT");
		DEFINE("MSF_CLIENT", 			MSF_SL::getParam($_SERVER, "REMOTE_ADDR"));
		DEFINE("MSF_HOST", 				MSF_SL::getParam($_SERVER, "SERVER_NAME"));
		DEFINE("MSF_PORT", 				MSF_SL::getParam($_SERVER, "SERVER_PORT"));
		DEFINE("MSF_VERSION", 			self::__VERSION);
		DEFINE("MSF_USE_GZIP",			(strstr(MSF_SL::getParam($_SERVER, "HTTP_ACCEPT_ENCODING"), "gzip") == false) ? false : true);
		DEFINE("IS_REQUEST", 			(
												(!empty($_POST) && (
																		(!isset($_POST["login"]) && !isset($_POST["password"])
																	)   || (count($_POST) != 2))) 
											||  (MSF_SL::getParam($_GET, "request") == "yes") 
											||  (!empty($_FILES)))
										);
										
	}

	private function initSettings()
	{
		//php settings 
		ini_set("mbstring.internal_encoding", "UTF-8");
		$this->Settings 	= MSF_Settings::getInstance(MSF_File::read(CNF_ABS_PATH . "/settings.xml"));
		MSF_Registry::set("MSF_Settings", $this->Settings);
	}

	private function initLocale()
	{
             if(isset($_SESSION["MSF_UserBox"])){
                 if(isset($_SESSION["MSF_UserBox"]["user"]["language"])){
                     if($_SESSION["MSF_UserBox"]["user"]["language"] != ""){

                         define('CNF_LANGUAGE_2', $_SESSION["MSF_UserBox"]["user"]["language"]);
                         $Locale = new MSF_Locale(CNF_ABS_PATH . "/languages/" . CNF_LANGUAGE_2 . ".ini");
//                            foreach ($this->Settings->getModulesList() AS $sModulName) {
//                                $sModuleDictionaryPath = CNF_ABS_PATH . "/modules/" . $sModulName . "/languages/" . CNF_LANGUAGE_2 . ".ini";
//                                if (MSF_File::Exists($sModuleDictionaryPath)) {
//                                        $Locale->loadDictionary($sModuleDictionaryPath);
//                                }
//                            }
                     } else {
                         $Locale = new MSF_Locale(CNF_ABS_PATH . "/languages/" . CNF_LANGUAGE . ".ini");
//                         foreach ($this->Settings->getModulesList() AS $sModulName) {
//                            $sModuleDictionaryPath = CNF_ABS_PATH . "/modules/" . $sModulName . "/languages/" . CNF_LANGUAGE . ".ini";
//                            if (MSF_File::Exists($sModuleDictionaryPath)) {
//                                    $Locale->loadDictionary($sModuleDictionaryPath);
//                            }
//                        }
                     }
                    
                } else {
                    $UserReferenceDB = new UserReferenceDB(MSF_Registry::get("MSF_DB"));
                    $UserReferenceDB->patchColumnTemplate('ref_system_users', 'language', 'string','Язык',true,"'ru'");
                    $tes = CNF_ABS_PATH . "/languages/" . CNF_LANGUAGE . ".ini";
                    $Locale = new MSF_Locale(CNF_ABS_PATH . "/languages/" . CNF_LANGUAGE . ".ini");
                }
             } else {
		$Locale = new MSF_Locale(CNF_ABS_PATH . "/languages/" . CNF_LANGUAGE . ".ini");
//		foreach ($this->Settings->getModulesList() AS $sModulName) {
//			$sModuleDictionaryPath = CNF_ABS_PATH . "/modules/" . $sModulName . "/languages/" . CNF_LANGUAGE . ".ini";
//			if (MSF_File::Exists($sModuleDictionaryPath)) {
//				$Locale->loadDictionary($sModuleDictionaryPath);
//			}
//
//		}
             }
		MSF_Registry::set("MSF_Locale", $Locale);
	}

	private function initRouter()
	{
		$this->Router = new MSF_Router(MSF_SL::getParam($_GET, "msf_path"));
		MSF_Registry::set("MSF_Router", $this->Router);
	}

	private function initErrorHandler()
	{
		error_reporting(E_ALL);
		set_error_handler("msfErrorHandler");
	}

	private function initTemplate()
	{
		$sPage = "page.xsl";
		$sXslPath = CNF_ABS_PATH . "/templates/" . CNF_SKIN . "/" . $sPage;
		$Template = new MSF_Template();
		$Template->loadXsl($sXslPath);
		MSF_Registry::set("MSF_Template", $Template);
        }
	private function shutdownDB()
	{
		MSF_Registry::get("MSF_DB")->close();
	}

	private function initDB()
	{
		$DB = new MSF_DB(); // throwable MSF_DBException
		// connect to db
		$DB->connect(
				CNF_SQL_HOST,
				CNF_SQL_DATABASE,
				CNF_SQL_USER,
				CNF_SQL_PASSWORD,
				CNF_SQL_SCHEME,
                                CNF_SQL_PORT
				); // throwable MSF_DBException
		
		MSF_Registry::set("MSF_DB", $DB);
	}

	private function initMsfx()
	{
		$iTime = getMicrotime();
		foreach ($this->Settings->getMsfxsList() AS $aMsfx) {
			include_once CNF_ABS_PATH . "/". $aMsfx["classpath"];
			$Class 		= new ReflectionClass($aMsfx["classname"]);
			$Extension = $Class->newInstance();
			// think about creating of msfxException
			if ($Extension instanceof MSF_MsfxSupport) {
				$Extension->setModule($this->Router->getModule());
				$Extension->setController($this->Router->getController());
				$Extension->execute();
			}
			else {
				throw new MSF_NotValidClassException($aMsfx["classname"]);
			}

		}

		DEFINE("MSF_TIME_MSFX", getMicrotime() - $iTime);
	}

	private function initController()
	{
		$this->Router->delegate($this->isMainUpdater);
	}
	
	private function initCLI($argv)
	{
		$sModule	= $argv[1];
		$sDaemon	= $argv[2];
	
		include_once CNF_ABS_PATH . "/modules/" . $sModule . "/daemons/" . $sDaemon . ".php";
		$Class 	= new ReflectionClass($sDaemon);
		$Daemon	= $Class->newInstance();
		$Daemon->run();
	}

	private function getTrace($e)
	{
		$aTrace = $e->getTrace();
		$sMsg 	= "Exception: " . "\n" . $e->getMessage() . "\n\n\n";
		foreach ($aTrace AS $k => $v) {
			$sMsg 	.=get_class($e).": #" . $k . "\n"
					. "Location: " 	. ((array_key_exists("class", $v)) 	? $v["class"] 	: "?") 	. ((array_key_exists("type", $v)) ? $v["type"] : "?") . ((array_key_exists("function", $v)) ? $v["function"] : "?") . "\n"
					. "File: " 		. ((array_key_exists("file", $v)) 	? $v["file"] 	: "?") 	. "\n"
					. "Line: " 		. ((array_key_exists("line", $v)) 	? $v["line"] 	: "?") 	. "\n"
					. "Args: "		. "\n" . "[ " . "\n";
			if (array_key_exists("args", $v) && is_array($v["args"]) && (count($v["args"]) > 0)) {
				foreach ($v["args"] AS $k => $v) {
					$type = gettype($v);
					switch (true) {
						case is_object($v):
							$v = get_class($v);
							break;
						case is_array($v):
							try {
								$v = serialize($v);
							}
							catch (Exception $e) {
								$v = "not defined";
							}

						case is_string($v):
							$v = $this->trn2space($v);
							break;
						case is_null($v):
							$v = "null";
							break;
					}

					$sMsg 	.= "    arg: #" . $k . " : (" . $type . ") " . $v . "\n";
				}

			}
			else {
				$sMsg 	.="    no args" . "\n";
			}

			$sMsg 	.= "]\n\n\n";

		}

		$sMsg 	.="Address: " 	. (isset($_SERVER["REMOTE_ADDR"]) ? $_SERVER["REMOTE_ADDR"] : "A.A.A.A") . "\n"
				. "Date: " 		. MSF_DATE . "\n";

		return $sMsg;

	}

	private function trn2space($str)
	{
		$characters = array("\t", "\r", "\n");
		$length 	= count($characters);
		for ($i = 0; $i < $length; $i++)
			$str = str_replace($characters[$i], " ", $str);
		return $str;
	}

	private function obStart()
	{
		ob_start();
	}


	private function obEndFlush()
	{
		ob_end_flush();
	}

	private function initIncludePath()
	{
		set_include_path(CNF_ABS_PATH . PATH_SEPARATOR . get_include_path());
	}

}
?>
