<?php
/*
 * File: MSF_Router.php
 * Created at: 03.01.2008
 */
defined("__MSF")
	or die("Direct access is not allowed");
include_once "system/security/MSF_LicenseManager.php";
include_once "modules/workflow/library/WF_Utils.php";
/**
 * @package MSF_Core
 * @author Detrix
 * @copyright Copyright (c) 2007, Detrix
 */
class MSF_Router
{
	private $aPath			= array();

	private $sController	= "";

	private $sModule		= "";

	private $sAction		= "";

	private $aParams		= array();
	
	private $isMainUpdater;

	public function __construct($sPath)
	{		
		$this->aPath = explode("/", $sPath);
		$this->checkMode();
		$this->sModule 	= strtolower($this->getParam(0, CNF_DEFAULT_MODULE));
		$this->sAction 	= strtolower($this->getParam(1, "index")) . "Action";		
		$this->initParams();
		
		
	}

	public function getModule()
	{
		return $this->sModule;
	}

	public function getController()
	{
		return $this->sController;
	}

	public function getAction()
	{
		return $this->sAction;
	}

	public function getParams()
	{
		return $this->aParams;
	}
	
	private function isConcurrent()
	{
		// active user is a user who has a timeout less than 1800
		// select all active users
		$iT					= CNF_CONC_TIMEOUT  <= 0 ? 20*60: abs(CNF_CONC_TIMEOUT*20*60);
		$sSql 				= "SELECT user_id as id FROM users_sessions WHERE timeout > " . (time() - $iT) . " OFFSET 0 LIMIT " . MSF_NUMBER_OF_LICENSES;
		$iCurrentUserId 	= MSF_Registry::get("MSF_UserBox")->getUserAttribute("id");
		$aAllowedUsersList 	= MSF_Registry::get("MSF_DB")->select($sSql);
		$bResult			= false;		
		for ($i = 0; $i < count($aAllowedUsersList); $i++) {
			if ($aAllowedUsersList[$i]["id"] == $iCurrentUserId) {
				if(!$this->isMainUpdater) {
					$sSql = "UPDATE users_sessions SET timeout = " . time() . " WHERE user_id = " . $iCurrentUserId;
					MSF_Registry::get("MSF_DB")->executeQuery($sSql);
				}
				
				$bResult = true;
				break;
			}
			
		}
		
		if (!$bResult) {
			if (count($aAllowedUsersList) < MSF_NUMBER_OF_LICENSES) {
				if(!$this->isMainUpdater) {
					$sSql = "UPDATE users_sessions SET timeout = " . time() . " WHERE user_id = " . $iCurrentUserId;
					MSF_Registry::get("MSF_DB")->executeQuery($sSql);
				}
				
				$bResult = true;
			}
			
		}
		
		return $bResult;
	}

	public function delegate($isMainUpdater)
	{		
		$this->isMainUpdater = $isMainUpdater;	
		if ($this->isWs()|| $this->isData()) {
			$this->runController();
		}
		else if ($this->isAllowed()) {
			if ($this->isLicensed()) {
				if (($this->isConcurrent() && CNF_IS_CONCURRENT) || ($this->isAllowedUser()  && !CNF_IS_CONCURRENT)) {					
						$this->runController();					
				}
				else {					
					//MSF_AccessDenied::exitApp("=(", "LICENCE_TOO_MANY_USERS");
					$this->sAction		= "licenseTooManyUsersAction";
					$this->sController	= "MSF_FrontController";
					$this->sModule		= "main";
					$this->runController();
				}
				
			}
			else {				
				MSF_AccessDenied::exitApp("=(", "LICENCE_KEY_CRASHED");								
			}
			
		}
		else {				
			//MSF_AccessDenied::exitApp("=(");
			$this->sAction		= "notAccessAction";
			$this->sController	= "MSF_FrontController";
			$this->sModule		= "main";			
			$this->runController();
			
		}
		
	}
	
	public function isWs()
	{
		return ($this->sController == 'MSF_WSController') || ($this->sController == 'MSF_WebServiceController');
	}
	
	public function isData()
	{
		return $this->sController == 'MSF_DataController';
	}
	
	private function runController()
	{		
				DEFINE('MSF_MODULE',  $this->sModule);
				DEFINE('MSF_IS_ADMIN', $this->isAdminMode());
		$sModulePath 	= CNF_ABS_PATH . "/modules/" . $this->sModule;
		$sSource 		= $sModulePath . "/controllers/" . $this->sController . ".php";
		if(file_exists($sSource)) {
			try {
				include_once $sSource;
			}
			catch(Exception $oException) {
				MSF_Logger::logTrace($oException);
			}
			
		}
		else {
			$this->sModule 	   = "main";
			$this->sController = "MSF_FrontController";
			$this->sAction 	   = "notFoundAction";
			$sModulePath 	= CNF_ABS_PATH . "/modules/" . $this->sModule;
			$sSource 		= $sModulePath . "/controllers/" . $this->sController . ".php";
			include_once $sSource;
		}			

		$Class	 	= new ReflectionClass($this->sController);		
		$Controller = $Class->newInstance();
		$Controller->setModule($this->sModule);
		$Controller->setController($this->sController);
		$Controller->setAction($this->sAction);
		$Controller->setParams($this->aParams);
		$Controller->before();
		try {
			$Method = new ReflectionMethod($this->sController, $this->sAction);
			$Method->invoke($Controller);
		}
		catch(Exception $oException) {			
			if (CNF_MODE == "debug") {
				throw $oException;
			}
			else {
				MSF_Logger::logTrace($oException);
				$Controller->notEditedRequestAction();
			}
						
		}		
		
		$Controller->after();
	}
	
	private function isAllowedUser()
	{
		$sSql 				= "SELECT id FROM ref_system_users WHERE is_active LIKE '%|^|row|^|1' ORDER BY is_admin DESC, id OFFSET 0 LIMIT " . MSF_NUMBER_OF_LICENSES;
		$iCurrentUserId 	= MSF_Registry::get("MSF_UserBox")->getUserAttribute("id");
		$aAllowedUsersList 	= MSF_Registry::get("MSF_DB")->select($sSql);
		$bResult			= false;		
		for ($i = 0; $i < count($aAllowedUsersList); $i++) {
			if ($aAllowedUsersList[$i]["id"] == $iCurrentUserId) {
				$bResult = true;
				break;
			}
			
		}
		
		return $bResult;
	}
	
	private function isLicensed()
	{
		if (MSF_File::exists(CNF_ABS_PATH . "/license.key")) {
			$oDb 		= MSF_Registry::get("MSF_DB");
			$oLm		= new MSF_LicenseManager($oDb, CNF_ABS_PATH . "/license.key");
			return $oLm->execute();
		}
		else {
			MSF_AccessDenied::exitApp("=(", "LICENCE_KEY_NOT_FOUND");
		}
	}

	
	private function isAllowed()
	{
		return $this->isAdminMode() ? MSF_Registry::get("MSF_UserBox")->getUserAttribute("is_admin") : true;
	}
	
	public function isAdminMode()
	{
		return ($this->sController == "MSF_BackController") ? true : false;
	}

	private function checkMode()
	{
		array_shift($this->aPath);
		if ((isset($this->aPath[0])) && (strtolower($this->aPath[0]) == "admin")) {
			$this->sController = "MSF_BackController";
			array_shift($this->aPath);
		}
		else if ((isset($this->aPath[0])) && (strtolower($this->aPath[0]) == "ws")) {
			$this->sController = "MSF_WSController";
			array_shift($this->aPath);
		}
		else if ((isset($this->aPath[0])) && (strtolower($this->aPath[0]) == "webservice")) {
			$this->sController = "MSF_WebServiceController";
			array_shift($this->aPath);
		}
		else if ((isset($this->aPath[0])) && (strtolower($this->aPath[0]) == "data")) {
			$this->sController = "MSF_DataController";
			array_shift($this->aPath);
		}
		else {
			$this->sController = "MSF_FrontController";
		}

	}

	private function initParams()
	{
		$this->aParams = array();
		for ($i = 2; $i < count($this->aPath); $i++) {
			if (strlen(trim($this->aPath[$i])) > 0) {
				$this->aParams[] = strtolower($this->aPath[$i]);
			}

		}		
	}

	private function getParam($iId, $sDefault)
	{
		$sResult = "";
		if (isset($this->aPath[$iId]) && (strlen($this->aPath[$iId]) > 0)) {
			$sResult = $this->aPath[$iId];
		}
		else {
			$sResult = $sDefault;
		}

		return $sResult;
	}

}
?>