<?php
/*
 * File: 	MSF_Settings.php
 * Date: 	23.01.2008
 * Time:	10:41:39
 * Detrix
 *
 */

/**
 * @package 	MSF_Core
 * @copyright 	Copyright (c) 2007, Detrix
 */
class MSF_Settings
{
	private static $Instance 	= null;

	/**
	 * @var MSF_XmlWrapper 
	 */
	private $XmlWrapper 		= null;

	public function getCacheParams()
	{
		return $this->XmlWrapper->get("caching")->getValue();
	}
	
	/**
	 * @return MSF_Settings
	 */
	public static function getInstance($sXml)
	{
		if (self::$Instance == null) {
			self::$Instance = new self($sXml);
		}

		return self::$Instance;
	}
	
	public static function clearInstance()
	{
		self::$Instance = null;	
	}

	private function __clone()
	{}

	/**
	 * @param string $sXml settings
	 * @return array
	 */
	public function __construct($sXml)
	{
		$this->XmlWrapper = new MSF_XmlWrapper($sXml);
	}

	/**
	 * Returns list of constants
	 * @return void
	 */
	public function getConstants()
	{
		$aConstantsHash = array();
		$aConstantsList = array_merge(
				$this->getCommonConstantsList(),
				$this->getModulesConstantsList(),
				$this->getMsfxsConstantsList()
				);

		for ($i = 0; $i < count($aConstantsList); $i++) {
			$sName = strtoupper($aConstantsList[$i]["name"]);
			$aConstantsHash[$sName] = $aConstantsList[$i]["value"];
		}

		return $aConstantsHash;
	}

	private function getCommonConstantsList()
	{
		$aConstantsList = array();
		foreach ((array) $this->XmlWrapper->get("settings")->getValue() AS $sConstantName => $sConstantValue) {
			$aConstantsList[] = array(
					"name" 	=> "CNF_" . $sConstantName,
					"value" => $sConstantValue
					);
		}
		
		return $aConstantsList;
	}

	private function getModulesConstantsList()
	{
		$aConstantsList = array();
		foreach ($this->XmlWrapper->get("modules")->getValue() AS $sModuleName => $aBody) {
			if (!array_key_exists("settings", $aBody)) {
				continue;
			}

			foreach ($aBody["settings"] AS $sConstantName => $sConstantValue) {
				$aConstantsList[] = array(
						"name" 	=> "CNM_" . $sModuleName . "_" . $sConstantName,
						"value" => $sConstantValue
						);
			}

		}

		return $aConstantsList;
	}

	private function getMsfxsConstantsList()
	{
		$aConstantsList = array();
		foreach ($this->XmlWrapper->get("extensions")->getValue() AS $sMsfxName => $aBody) {
			if (!array_key_exists("settings", $aBody)) {
				continue;
			}

			foreach ($aBody["settings"] AS $sConstantName => $sConstantValue) {
				$aConstantsList[] = array(
						"name" 	=> "CNX_" . $sMsfxName . "_" . $sConstantName,
						"value" => $sConstantValue
						);
			}

		}

		return $aConstantsList;
	}

	/**
	 * Returns list of installed msfx
	 * @return array
	 */
	public function getMsfxsList()
	{
		$aModulesList 	= array();
		foreach ($this->XmlWrapper->get("extensions")->getValue() AS $sMsfxName => $aBody) {
			$aModulesList[] = array(
					"classname" => $aBody["classname"],
					"classpath" => "/msfx/" . $sMsfxName . "/" . $aBody["classname"] . ".php"
					);
		}

		return $aModulesList;
	}

	/**
	 * Returns list of installed services
	 * @return array
	 */
	public function getServicesList()
	{
		$aServicesList 	= array();
		foreach ($this->XmlWrapper->get("modules")->getValue() AS $sModuleName => $aBody) {
			if (!array_key_exists("services", $aBody)) {
				continue;
			}

			foreach ($aBody["services"] AS $sServiceClassname) {
				$aServicesList[] = array(
						"classpath" 	=> "/modules/" . $sModuleName . "/services/" . $sServiceClassname . ".php",
						"classname" 	=> $sServiceClassname,
						"modulename" 	=> $sModuleName,
						"visible" 		=> $aBody["settings"]["visible"]
						);
			}

		}

		return $aServicesList;
	}

	/**
	 * Returns list of installed modules
	 * @return array
	 */
	public function getModulesList()
	{
		$aModulesList = array();
		foreach ($this->XmlWrapper->get("modules")->getValue() AS $sModuleName => $aBody) {
			$aModulesList[] = strtolower($sModuleName);
		}

		return $aModulesList;
	}

	public function getXmlWrapper()
	{
		return $this->XmlWrapper;
	}
	
	public function getModule($sModule)
	{
		return $this->XmlWrapper->get("modules")->getValue($sModule);
	}



}
?>
