<?php
if (defined("__MSF") <> true) {
    exit("Direct access is not allowed");
}

/**
 * frozen at 2007.01.11 17:58
 * @package MSF_Core
 * @author Detrix
 * @copyright Copyright (c) 2007, Detrix
 */
class MSF_Template
{
	const XML_VERSION				= "1.0";

	const XML_CHARSET				= "UTF-8";

	private $oDomDocument       	= null;

	private $oDomNodeRoot       	= null;
	
	private $oDomNodeEtc 			= null;
	
	private $oDomNodeMsfx 			= null;

	private $oDomNodeModule		    = null;

	private $oXslStylesheet    		= null;

	private $aXslList				= array();

	private $aXslPathList			= array();
	
	private $aVariables				= array();
	
	private $aLinks 				= array();

    public function __construct()
    {
		$this->oDomDocument		= new DOMDocument(self::XML_VERSION, self::XML_CHARSET);
		$this->oDomNodeRoot		= $this->oDomDocument->createElement("page", 	"");
		$this->oDomNodeEtc		= $this->oDomDocument->createElement("etc", 	"");
		$this->oDomNodeMsfx		= $this->oDomDocument->createElement("msfx", 	"");
		$this->oDomNodeModule	= $this->oDomDocument->createElement("module", 	"");
		$this->oDomNodeRoot->appendChild($this->oDomNodeEtc);
		$this->oDomNodeRoot->appendChild($this->oDomNodeMsfx);
		$this->oDomNodeRoot->appendChild($this->oDomNodeModule);
		$this->oDomDocument->appendChild($this->oDomNodeRoot);
		$this->setModuleAttrs("", "", 0);
		$this->oDomNodeModule->setAttribute("systemVersion", MSF_VERSION);
		$this->oDomNodeModule->setAttribute("currentYear", date("Y"));
                $this->oDomNodeModule->setAttribute("support_locale", SUPPORT_LOCALE);
                if(defined('CNF_LANGUAGE_2')){
                    $this->oDomNodeModule->setAttribute("current_locale", CNF_LANGUAGE_2);
                } else {
                    $this->oDomNodeModule->setAttribute("current_locale", CNF_LANGUAGE);
                }
                
                $jsDictionary = MSF_Registry::get("MSF_Locale")->getDictionary();
                $jsLocale = "";
                foreach($jsDictionary as $key=>$val){
                    $key_match = preg_match("/JS_LOCALE/", $key, $matches);
                    if($key_match == 1){
                        $jsLocale .= "$key:'$val',\n";
                    }
                }
                $this->oDomNodeModule->setAttribute("jsLocale",$jsLocale);
    }
    
    public function appendXmlString($Element, $sXml)
    {
   			$Fragment = $this->oDomDocument->createDocumentFragment();
			$Fragment->appendXml(MSF_HtmlEntities::replace($sXml));
			$Element->appendChild($Fragment);

    }
    
    public function setLinks(array $aLinks)
    {
    	$this->aLinks = $aLinks;
    }
    
    public function getLinks()
    {
    	return $this->aLinks;
    }

    public function loadXsl($sXslPath)
    {
    	$this->sXslPath = $sXslPath;
    	$sXsl = MSF_File::read($sXslPath);
    	$this->oXslStylesheet = new DOMDocument(self::XML_VERSION, self::XML_CHARSET);
    	//$this->oXslStylesheet->validate();
    	$this->oXslStylesheet->loadXml($sXsl);
        foreach ($this->aLinks AS $Link) {
    		$this->appendLink($Link['path'], $Link['type']);
    	}
    	
    }

    public function createElement($sTagName, $sText = "")
    {
    	$sText = str_replace('&amp;', '&', $sText);
    	$sText = str_replace('&', '&amp;', $sText);
		return $this->oDomDocument->createElement($sTagName, $sText);
    }

    public function getXml()
    {
    	return $this->oDomDocument->saveXML();
    }
    
    public function getXsl()
    {
    	$this->load();
    	return $this->oXslStylesheet->saveXML();
    }

    public function addLink($sFilePath, $sType)
    {
    	$this->aLinks[] = array(
    		"path" 	=> $sFilePath,
    		"type"	=> $sType
    	);
		
    	$this->appendLink($sFilePath, $sType);

    }
    
    private function appendLink($sFilePath, $sType)
    {
		$sAbsFilePath 	= CNF_ABS_PATH . $sFilePath;
		$sFilePath 		= $sFilePath . "?v=" . MSF_VERSION;
		list($sFileName, $sFileExt)	= explode(".", basename($sFilePath));
		if (!MSF_File::exists($sAbsFilePath)) {
			throw new MSF_FileNotFoundException($sAbsFilePath);
		}
		else if ($sType == "text/css") {
            $ImportTpl = $this->oXslStylesheet->createElement("link");
            $ImportTpl->setAttribute("rel", "stylesheet");
			$ImportTpl->setAttribute("href", $sFilePath);
						
		}
		else if ($sType == "text/javascript") {
			$ImportTpl = $this->oXslStylesheet->createElement("script", "");
			$ImportTpl->setAttribute("src", $sFilePath);
			//$ImportTpl->setAttribute("onreadystatechange", "ieProgressBarLauncher(this)");
			$ImportTpl->setAttribute("onload", "doProgressBar()");
		}
		else {
			throw new MSF_IOException($sType);
		}

	
		$ImportTpl->setAttribute("type", $sType);
		
		$oBdos = $this->oXslStylesheet->getElementsByTagName("bdo");
		if ($oBdos->length) {
			$Root 	= $oBdos->item(0);
			$Root->insertBefore($ImportTpl, $Root->lastChild);			
		}

		//$Root->insertBefore($this->increaseScriptCount(), $Root->lastChild);
		$this->increaseScriptCount();
    }
    
    private function increaseScriptCount()
    {
    	static $i = 1;
    	$this->oDomNodeModule->setAttribute("totalScripts", $i);
    	$i++;
    //	return $this->oXslStylesheet->createElement("script", "doProgressBar()");
    }

    public function addVariable($sName, $sValue)
    {
    	$this->aVariables[] = array(
    			"name"	=> $sName,
    			"value" => $sValue
    			);
    }

	public function createMsfx($sName)
	{
		$oElmMsfx = $this->createElement($sName);
		$this->oDomNodeMsfx->appendChild($oElmMsfx);
		return $oElmMsfx;
	}
	
	public function createEtc($sName)
	{
		$oElmEtc = $this->createElement($sName);
		$this->oDomNodeEtc->appendChild($oElmEtc);
		return $oElmEtc;
	}

    public function getModule()
    {
    	return $this->oDomNodeModule;
    }
    
    public function setModuleAttrs($sTitle, $sName, $iId)
    {
    	$this->oDomNodeModule->setAttribute("title", 	$sTitle);
    	$this->oDomNodeModule->setAttribute("name", 	$sName);
    	$this->oDomNodeModule->setAttribute("id", 		$iId);
    }

    public function setPageTitle($sTitle)
    {        
//        debug($sTitle);
//        exit;
        $this->oDomNodeModule->setAttribute("title", MSF_String::decodeSpecialChars($sTitle));
    }
    
    public function process()
    {
        $Processor = new XSLTProcessor();
    	try {
    		$this->load();
    		$Processor->importStylesheet($this->oXslStylesheet);

    	}
    	catch (MSF_ThrowableException $oException) {
    		throw new MSF_ThrowableException(
    				"*********************************** Message *********************************\n\n" .
    				$oException->getMessage() . "\n\n" .
    				"*********************************** Main-template ***************************\n\n" .
    				$this->sXslPath. "\n\n".
    				"*********************************** Imports *********************************\n\n" .
    				implode("\n", $this->aXslPathList) . "\n\n" .
    				"****************************************************************************"
    				);
    	}
    	
        try {
            /*test*/
            //$xmltest = $this->oDomDocument->saveXML();
			$sResult = $Processor->transformToXml($this->oDomDocument);
    	}
    	catch (MSF_ThrowableException $oException) {
    		throw new MSF_ThrowableException(
    				"*********************************** Message *********************************\n\n" .
    				$oException->getMessage() . "\n\n"  .
    				"*********************************** Message *********************************"
    				);
    	}
        
        return $sResult;
    }

	public function addXsl($sName, $sValue, $sXml)
	{
		$aData = array (
				"name" 	=> $sName,
				"value" => $sValue,
				"xml"	=> $sXml
				);
		if (!in_array($aData, $this->aXslList)) {
			$bResult = true;
			$this->aXslList[] = $aData;
		}
		else {
			$bResult = false;
		}

		return $bResult;
	}

	public function addXslPath($sXslPath)
	{
		if (!in_array($sXslPath, $this->aXslPathList)) {
			$bResult = true;
			$this->aXslPathList[] = CNF_ABS_PATH . $sXslPath;
		}
		else {
			$bResult = false;
		}

		return $bResult;


	}

	public function addLinksOf($sIniPath)
	{
		$sContent 		= MSF_File::read($sIniPath);
		foreach (explode("\n", str_replace("\r", "", $sContent)) AS $sLink) {
			if($sLink == "") {
				continue;
			}

			$aTemp = pathinfo($sLink);
			switch ($aTemp["extension"]) {
				case "js":
					$this->addLink("/" . $sLink, "text/javascript");
					break;
				case "css":
					$this->addLink("/" . $sLink, "text/css");
					break;
			}

		}

	}
	
    private function load()
    {
        for ($i = 0; $i < count($this->aVariables); $i++) {
			$this->loadVariable($this->aVariables[$i]["name"], $this->aVariables[$i]["value"]);
    	}
    	
    	for ($i = 0; $i < count($this->aXslPathList); $i++) {
			$this->import($this->aXslPathList[$i]);
    	}

    	for ($i = 0; $i < count($this->aXslList); $i++) {
			$this->create($this->aXslList[$i]);
    	}
    	

    }
    
    private function loadVariable($sName, $sValue)
    {
    	if (preg_match('/^\w*$/i', $sName) == 0) {
    		throw new MSF_ThrowableException("Incorrect name for xsl-variable:\n" . $sName);
    	}
    	
      	$ElmVariable = $this->oXslStylesheet->createElementNS("http://www.w3.org/1999/XSL/Transform", "variable", $sValue);
    	$ElmVariable->setAttribute("name", $sName);
        $this->oXslStylesheet->documentElement->insertBefore(
        		$ElmVariable,
        		$this->oXslStylesheet->documentElement->firstChild
        		);
    }

    private function create($aData)
    {
     	$oXsl = $this->oXslStylesheet->createElementNS("http://www.w3.org/1999/XSL/Transform", "template");
    	$oXsl->setAttribute($aData["name"], $aData["value"]);
    	$oXml = $this->oXslStylesheet->createDocumentFragment();
    	
    	$oXml->appendXML($aData["xml"]);
		$oXsl->appendChild($oXml);

		$this->oXslStylesheet->documentElement->appendChild($oXsl);
    }

    public function getDomDocument()
    {
    	return $this->oDomDocument;
    }
    
    private function import($sXslPath)
    {
       	if (!MSF_File::exists($sXslPath)) {
    		throw new MSF_FileNotFoundException($sXslPath);
    	}

    	//try {
	        $ImportTpl  = $this->oXslStylesheet->createElementNS("http://www.w3.org/1999/XSL/Transform", "import");
	        $ImportTpl->setAttribute("href", $sXslPath);
	        $this->oXslStylesheet->documentElement->insertBefore($ImportTpl, $this->oXslStylesheet->documentElement->firstChild);
    /*	}
    /	catch (MSF_ThrowableException $oException) {
    		echo "a";
    		throw new MSF_ThrowableException(
    				$oException->getMessage() . "\n\n" .
    				"Template " . $sXslPath . " not imported " . $this->sXslPath
    				);
    	}*/

    }

}
?>