<?php
/*
 * File: MSF_BaseController.php
 * Created at: 08.01.2008
 */
defined("__MSF")
	or die("Direct access is not allowed");

/**
 * frozen at 2007.01.11 16:46
 * @package MSF_Core
 * @author Detrix
 * @copyright Copyright (c) 2007, Detrix
 */
abstract class MSF_ControllerSupport extends MSF_Mcap
{
	protected $DB 		= null;

	/**
	 * Enter description here...
	 *
	 * @var MSF_Template
	 */
	protected $Template = null;

	/**
	 * @var MSF_Locale
	 */
	protected $Locale 	= null;

	private $View		= null;

	public function __construct()
	{
		$this->DB 			= MSF_Registry::get("MSF_DB");
		$this->Locale 		= MSF_Registry::get("MSF_Locale");
		$this->Template 	= MSF_Registry::get("MSF_Template");
		$this->Messenger	= MSF_Registry::get("MSF_Messenger");
		
		//$_POST = $this->stripPostSlashes($_POST);
	}
	
	private function stripPostSlashes($aPost)
	{
		$aResult = array();
		
		foreach ($aPost as $sKey => $mValue) {
			if (is_array($mValue)) {
				$aResult[$sKey] = $this->stripPostSlashes($mValue);
			}
			else {
				$aResult[$sKey] = $this->stripAll($mValue);
			}
		}
		
		return $aResult;
	}
	
	private function stripAll($sValue)
	{
		//$sValue = str_replace("\\\"", "\"", $sValue);
		//$sValue = str_replace("\'", "'", $sValue);
		$sValue = stripslashes($sValue);
		$sValue = str_replace("\\\\", "", $sValue);
		
		return $sValue;
	}

	public function setView($View)
	{
		$this->View = $View;
	}

	public function before()
	{	
		$iTime		= getMicrotime();
	
		if(isset($_SESSION["menu"])){
			$oDOMDocumentMenu 	= new DOMDocument(MSF_Template::XML_VERSION,MSF_Template::XML_CHARSET);
			$oDOMDocumentMenu->loadXML($_SESSION["menu"]);
			$oNodeMenu = $oDOMDocumentMenu->getElementsByTagName("menu")->item(0);
			 
			$oDOMDocument 		= MSF_Registry::get("MSF_Template")->getDomDocument();

			$oNode = $oDOMDocument->importNode($oNodeMenu, true);
			$oMsfx = $oDOMDocument->getElementsByTagName("msfx")->item(0);
			$oMsfx->appendChild($oNode);
			
			MSF_Registry::get("MSF_Menu")->setRoot($oNode);
		}	
		else {
			$Settings 	= MSF_Registry::get("MSF_Settings");
			$iTime		= getMicrotime();
			foreach ($Settings->getServicesList() AS $aService) {
				include_once CNF_ABS_PATH . $aService["classpath"];
				$Class 		= new ReflectionClass($aService["classname"]);
				$Extension 	= $Class->newInstance();
				if ($Extension instanceof MSF_IService) {
					try {
						$Extension->execute();
					}
					catch (Exception $oE) {
						throw new MSF_ThrowableException($aService["classname"] . ": " . $oE->getMessage());
					}
					
				}
				else {
					throw new MSF_NotValidClassException("Its not a service " . $aService["classname"]);
				}
				
			}
	
			$oDomDocument 		= MSF_Registry::get("MSF_Template")->getDomDocument();
			foreach ($oDomDocument->getElementsByTagName('menu') as $sMenu);
			$_SESSION["menu"]	= $oDomDocument->saveXML($sMenu);
		}
		DEFINE("MSF_TIME_SERVICES", getMicrotime() - $iTime);
		return true;
	}

	public function after()
	{
		if ($this->View !== null) {
			$this->View->render();
		}

	}

	public function redirect($sUrl, $sMessage = "")
    {
    	$this->Messenger->redirect($sUrl, $sMessage);
    }

    public function locale($sWord)
    {
    	return $this->Locale->getWord($sWord);
    }

    public function addAction($sTitle, $sUrl,  $sImage = CNX_ACTIONBAR_IMG, $sOnClick = "", $bNotSelect = false, $sSide = "left", $sHelp = "")
    {
    	MSF_Registry::get("MSF_ActionBar")->addAction($sTitle, $sUrl,  $sImage, $sOnClick, $bNotSelect, $sSide, $sHelp);
    }

    public function addMessage($sMessage)
    {
		$this->Messenger->addMessage($sMessage);
    }

    public function addError($sMessage)
    {
		$this->Messenger->addError($sMessage);
    }

    public function getTemplate()
    {
    	return $this->Template;
    }
    
    public function notFoundAction()
    {
    	$oView = new MSF_ViewSupportXml();    	    	  	
    	$this->setView($oView);
    	MSF_Registry::get("MSF_Messenger")->addError($this->Locale->getWord("ROOT_404_PAGE_PAGE"));
    }
    
	public function notEditedRequestAction()
    {
    	$oView = new MSF_ViewSupportXml();    	
    	$this->setView($oView);		
		MSF_Registry::get("MSF_Messenger")->addError($this->Locale->getWord("ROOT_REQUEST_NOT_EDIT"));
    }
    
    public function notAccessAction()
    {  
    	$oView = new MSF_ViewSupportXml();
    	$this->setView($oView);  	    	
		MSF_Registry::get("MSF_Messenger")->addError($this->Locale->getWord("ROOT_403_PAGE_MESSAGE"));		
    }    
    
	public function licenseTooManyUsersAction()
    {
    	$oView = new MSF_ViewSupportXml();
		$this->setView($oView);
		MSF_Registry::get("MSF_Messenger")->addError($this->Locale->getWord("LICENCE_TOO_MANY_USERS"));		
    }
        
}
?>