<?php
/*
 * File: MSF_ViewSupport.php
 * Created at: 11.01.2008
 */
defined("__MSF")
	or die("Direct access is not allowed");

/**
 * frozen at 2007.01.11 18:12
 * @package MSF_Core
 * @author Detrix
 * @copyright Copyright (c) 2007, Detrix
 */
class MSF_ViewSupportXml
{
	/**
	 * @var MSF_Template
	 */
	protected $Template 	= null;

	protected $Locale		= null;

	protected $Messenger	= null;

	protected $ActionBar	= null;

	protected $Root			= null;

	final public function __construct()
	{
		$this->Template 	= MSF_Registry::get("MSF_Template");
		$this->Locale		= MSF_Registry::get("MSF_Locale");
		$this->Messenger	= MSF_Registry::get("MSF_Messenger");
		$this->ActionBar	= MSF_Registry::get("MSF_ActionBar");
		$this->Root			= $this->Template->getModule();
		$this->initialize();
	}

	public function initialize()
	{}

	final public function addLinksOf($sModule)
	{
		$sSettingsPath 	= CNF_ABS_PATH . "/modules/" . $sModule . "/view/links.ini";
		$this->Template->addLinksOf($sSettingsPath);

	}

	final public function render($sTemplate = null)
	{
		$this->Template->getModule()->setAttribute(
				"hideMenu", 
				(int)isset($_GET['hideMenu'])
				);
		$this->Template->getModule()->setAttribute(
				"hideBarAndCrumbs", 
				(int)isset($_GET['hideBarAndCrumbs'])
				);
		$this->Template->getModule()->setAttribute(
				"hideTitle", 
				(int)isset($_GET['hideTitle'])
				);
		$this->Template->getModule()->setAttribute(
				"hideLog", 
				(int)isset($_GET['hideLog'])
				);
		$this->Template->getModule()->setAttribute(
				"hideToolbar", 
				(int)isset($_GET['hideToolbar'])
				);
		
		$aWords = array();
		if ($this->Locale->getWord("MSF_LICENSE_OWNER") == "MSF_LICENSE_OWNER") {
			$this->Locale->addWord("MSF_LICENSE_OWNER", "");
		}
		
		foreach ($this->Locale->getDictionary() AS $sName => $sValue) {
			$sName = strtoupper($sName);
			$aWords[$sName] = $sValue;
			$this->Template->addVariable($sName, $sValue);
		}
		
		$this->Template->addVariable("CNF_MODE", CNF_MODE);
		$this->Template->addVariable("CURRENT_LOGIN", str_replace("&", "&amp;", $_SESSION["MSF_UserBox"]["user"]["login"]));
		switch (MSF_SL::getParam($_GET, "msf_view")) {
			case "xml":
				ob_end_clean();
				ob_start();
				HEADER("Content-type: text/xml; charset=utf-8");
				echo $this->Template->getXml();
				break;
			case "xsl":
				ob_end_clean();
				ob_start();
				HEADER("Content-type: text/xml; charset=utf-8");
				echo $this->Template->getXsl();
				break;
			default:
				HEADER("Content-type: text/html; charset=utf-8");				
				$this->Messenger->appendMessages();
				$this->Messenger->appendErrors();
				$iBefore 	= getMicrotime();
				$sResult 	= $this->Template->process();
				$iTotal 	= getMicrotime() - MSF_TIME_START;
				echo $sResult;
				echo  "\n<!--\n"
					. "Total: ".round($iTotal, 3)."s\n"
					. "MSFX: ".round(MSF_TIME_MSFX, 3)."s\n"
					. "Services: ".round(MSF_TIME_SERVICES, 3)."s\n"
					. "Module: ".round(($iTotal - MSF_TIME_SERVICES - MSF_TIME_MSFX), 3)."s\n"
					. "XSLT: ".round((getMicrotime() - $iBefore), 3)."s\n"
					. "-->";
				break;
		}

	}

	final public function hash2dom($sName, $aArr, $sType = "attrs")
    {
        $Elm = $this->Template->createElement($sName);
        switch ($sType) {
             case "attrs":
                    foreach ($aArr AS $sAttrName => $sAttrValue) {
                    	$Elm->setAttribute($sAttrName, $sAttrValue);
                    }

                  break;

             case "tags":
                    foreach ($aArr AS $sAttrName => $v) {
                    	$Elm->appendChild($this->Template->createElement($sAttrName, $sAttrValue));
                    }

                    break;
        }

        return $Elm;
    }

    final public function addTemplate($sFileName)
    {
		$this->Template->addXslPath($sFileName);
    }

    final public function addError($sError)
    {
		$this->Messenger->addError($sError);
    }

    final public function addLocalizedError($sWord)
    {
		$this->addError($this->Locale->getWord($sWord));
    }

    final public function addMessage($sMessage)
    {
		$this->Messenger->addMessage($sMessage);
    }

    final public function addLocalizedMessage($sWord)
    {
		$this->addMessage($this->Locale->getWord($sWord));
    }

    final public function getWord($sWord)
    {
		return $this->Locale->getWord($sWord);
    }

    final public function addAction($sTitle, $sUrl,  $sImage = CNX_ACTIONBAR_IMG, $sOnClick = "", $bNotSelect = false, $sSide = "left")
    {
		$this->ActionBar->addAction($sTitle, $sUrl,  $sImage, $sOnClick, $bNotSelect, $sSide);
    }

	final public function redirect($sUrl, $sMessage)
    {
    	$this->Messenger->redirect($sUrl, $sMessage);
    }

}
?>