<?php
require_once "MSF_SheduleService.php";
class MSF_Shedule
{
	private $iMaxProcess				= 5;
	
	private $aChildren					= array();
	
	private $iSleepTime					= 3;
	
	private $iSettingsCheckFrequency	= 60;
	
	private $iLastSetingsCheck			= 0;
	
	private $aServices					= array();
	
	private $sSettingsFile				= "settings.xml";
	
	private $sDirRoot					= "";
	
	private $sExecuteFile				= "";
	
	private $iMaxExecutionTime			= 300;
	
	public function __construct($aConf)
	{
		$this->sDirRoot					= $aConf["dir_root"];
		$this->iMaxProcess				= $aConf["max_process"];
		$this->iSettingsCheckFrequency	= $aConf["settings_check_frequency"];
		$this->iMaxExecutionTime		= $aConf["max_executuion_time"];
		
		
		$this->sSettingsFile			= $aConf["settings_file"];	
		$this->sExecuteFile				= $aConf["execute_file"];
		
		fputs(STDOUT, "MSF_Shedule: run!\n");
		$this->parseSettings();	
	}
	
	private function parseSettings()
	{		
		$Settings	= new MSF_XmlWrapper(MSF_File::read($this->sSettingsFile));
		$Settings	= $Settings->getAsArray();
		$aModules	= $Settings["modules"]["value"];
		
		
		foreach ($aModules as $sModule=>$aConf) {	
			if(isset($aConf["daemons"])) {						
				foreach($aConf["daemons"] as $sName=>$aParams) {
					$Service	= $this->getDaemon($sModule . "::" . $sName);
					if($Service) {
						$Service->setFrequency($aParams["frequency"]);
					}
					else {
						$Service	= new MSF_SheduleService(
							$sModule, 
							$sName, 
							$aParams["frequency"] * 60,
							$this->sDirRoot,
							$this->sExecuteFile
						);					
						array_push($this->aServices, $Service);
					}					
								
				}	
				
			}
					
		}			
		
		$this->iLastSetingsCheck	= time();		
	}
	
	private function getDaemon($sServiceKey) {
		foreach($this->aServices as $Service) {
			if($Service->getServiceKey() == $sServiceKey){
				return $Service;
			}
		}
		
		return null;
	}
	
	private function checkSettings()
	{		
		if((time() - $this->iLastSetingsCheck) > $this->iSettingsCheckFrequency) {
			$this->parseSettings();
		}
	}
	
	private function next()
	{		
		usort($this->aServices, array($this, "sort"));	
		
		return $this->aServices[0];
	}
	
	private function sort($a, $b)
	{
		if ($a->getNextAttempt() == $b->getNextAttempt()) {
	        return 0;
	    }
	    
	    return ($a->getNextAttempt() < $b->getNextAttempt()) ? -1 : 1;		
	}
	
	public function sigChild($signal)
	{		
		$mStatus	= false;
		pcntl_signal(SIGCHLD, array($this, "sigChild"));
		while(($iPid = pcntl_wait($mStatus, WNOHANG)) > 0) {
			unset($this->aChildren[$iPid]);
		}
	}
	
	public function run()
	{			
		declare(ticks = 1);
		pcntl_signal(SIGCHLD, array($this, "sigChild"));
		while(true) {
			$this->checkSettings();
			$iNow	= time();
			if(count($this->aChildren) < $this->iMaxProcess) {
				$Service	= $this->next();
				if($iNow < $Service->getNextAttempt()) {
					sleep($this->iSleepTime);
					continue;
				}
								
				$Service->setNextAttempt();
				$iPid	= pcntl_fork();
				if($iPid) {
					$this->aChildren[$iPid]	= $Service;	
				}
				else {
					pcntl_alarm($this->iMaxExecutionTime);					
					$Service->run();					
					exit();
				}
				
			}
			else {
				sleep($this->iSleepTime);
			}						
			
		}		
		
	}	
	
}
?>