<?php
/*
 * File: MSF_TestSuite.php
 * Created at: 14.01.2008
 */
defined("__MSF")
	or die("Direct access is not allowed");

/**
 * @package MSF_
 * @author Detrix
 * @copyright Copyright (c) 2007, Detrix
 */
class MSF_TestSuite
{
	public function __construct()
	{}

	public function test()
	{
		$aTests 	= array();
		$sPath 		= CNF_ABS_PATH . "/tests.xml";
		if (MSF_File::exists($sPath)) {
			$XW = new MSF_XmlWrapper(MSF_File::read($sPath));
			foreach ($XW AS $sTestId => $XWData) {
				$aTests[] = $XWData->getValue();
			}

		}

		return $this->runTests($aTests);
	}

	private function runTests($aTests)
	{
		$aResults = array();
		for ($i = 0; $i < count($aTests); $i++) {
			include_once CNF_ABS_PATH . $aTests[$i]["testpath"];
			$Class 	= new ReflectionClass($aTests[$i]["testname"]);
			$Test	= $Class->newInstance();
			if ($Test instanceof MSF_TestUnit) {
				try {
					$Test->setUp();
					try {
						$Test->test();
					}
					catch (Exception $E) {
						$Test->fail(get_class($E) . ":\n" . $E->getMessage());
					}
					
					$Test->setDown();
				}
				catch (Exception $E) {
					$Test->fail(get_class($E) . ":\n" . $E->getMessage());
				}
				
				$aResults[] = array
				(
					"classname" => $aTests[$i]["testname"],
					"status" 	=> $Test->getStatus(),
					"messages"	=> $Test->getMessages()
				);

			}

		}

		return $aResults;
	}

	public function getResultsAsString($aResults)
	{
		$sMsg = "";

		foreach ($aResults AS $aResult) {
			$sMsg .= "------------------------------------------------" . "\r\n";
			$sMsg .= "\r\n";
			$sMsg .= " test: " . $aResult["classname"] . "\r\n";
			$sMsg .= " status: " . $aResult["status"] . "\r\n";
			$sMsg .= " time: " . gmdate("Y.m.d H:i:s") . "\r\n";
			$sMsg .= "\r\n";
			$sMsg .= "------------------------------------------------" . "\r\n";
			if (is_array($aResult["messages"]) && !empty($aResult["messages"])) {
				foreach ($aResult["messages"] AS $iKey => $aValue) {
					$sMsg .= "Method: " . $aValue["type"] 	. "\r\n";
					$sMsg .= "Status: " . $aValue["status"] . "\r\n";
					$sMsg .= $aValue["message"] . "\r\n\r\n";
				}

			}

		}

		return $sMsg;
	}

}
?>