<?php
if(!defined("__MSF")) {
	die("Direct access is not allowed");
}
include_once "MSF_DBRecord.php";
include_once "MSF_DBException.php";

/**
 * @package MSF_DB
 */
class MSF_DB
{
	/**
	 * @var resource connection link
	 */
	private $rLink					= false;

	/**
	 * @var resource result set
	 */
	private $rResult				= false;

	private $sScheme				= "";
	
	private $iCounter				= 0;
	
	private $iDuration				= 0;

	/**
	 * Creating the connection with data base throws exception in the case of failure
	 * @access public
	 * @return void
	 * @throws MSF_DBException
	 */
	public function connect($sHost, $sDatabase, $sUsername, $sPassword, $sScheme, $sPort,$sRDBMS = "undefined")
	// throws MSF_DBException
	{
		$sConnectString		= "host=" . $sHost . " port=" . $sPort . " dbname=" . $sDatabase . " user=" . $sUsername . " password=" . $sPassword;
		$this->sScheme 		= $sScheme;
		$this->rLink = pg_connect($sConnectString);
		if (!$this->rLink) {
			throw new MSF_DBException("Connection failed");
		}

		$this->executeQuery("SET client_encoding TO UNICODE");
		$this->executeQuery("SET search_path TO " . $sScheme);

	}

	/**
	 * Execute's SQL statement
	 * @param string $sSql SQL statement
	 * @access public
	 * @return void
	 */
	public function executeQuery($SQL, $aParams = null)
	{
		try {
			$iStartTime 		 = getMicrotime();
			if ($aParams == null) {
				$this->rResult   = pg_query($this->rLink, $SQL);
			}
			else {
				$this->rResult   = pg_query_params($this->rLink, $SQL, $aParams);
			}
			
			$this->iDuration  	+= getMicrotime() - $iStartTime;
			$this->iCounter++;
			return $this;
		}
		catch (Exception $E) {
			throw new MSF_DBException("query:\n" . $SQL . "\npostgres said:\n" . pg_last_error($this->rLink));
		}

	}
	
	public function getDuration()
	{
		return round($this->iDuration, 3);
	}

	/**
	 * Fetches the next row from a result set
	 * @access public
	 * @return mixed
	 */
	public function fetchArray()
	{
		$mResult = pg_fetch_assoc($this->rResult);
		if (!$mResult) {
			$mResult = array();
		}
		
		return $mResult;
	}

	/**
	 * Returns the number of rows affected by the last SQL statement
	 * @access public
	 * @return integer
	 */
	public function affectedRows()
	{
		return pg_affected_rows($this->rResult);
	}

	/**
	 * Returns the ID of the last inserted row or sequence value
	 * @access public
	 * @return integer
	 */
	public function insertId()
	{
		$sSelect	= "SELECT LASTVAL() as last_id";
		return (int) $this->select($sSelect, true, "last_id");
		/*
		return pg_last_oid($this->rResult);
		*/
		/*
		$sSelect	= "SELECT currval('" . $sTablename . "_" . $sId . "_seq') AS last_id";
		return (int) $this->select($sSelect, true, "last_id");
		*/
	}

	/**
	 * Releasing of the result which is occupied by last query
	 * @access public
	 * @return void
	 */
	public function freeResult()
	{
		$this->iCounter++;
		pg_free_result($this->rResult);
	}

	/**
	 * Closing the connection with RDBMS
	 * Current method is a stub
	 * @access public
	 * @return boolean;
	 */
	public function close()
	{
		return pg_close($this->rLink);
	}

	/**
	 * Returns the number of queries
	 * @access public
	 * @return integer
	 */
	public function getCount()
	{
		return $this->iCounter;
	}

	/**
	 * Data selection
	 * @access public
	 * @return array
	 * @throws MSF_DBException
	 */
	public function select($SQL, $bOneRow = false, $sRowName = null)
	// throws MSF_DBException
	{
		$mResult = array();

		// execute
		$this->executeQuery($SQL);
		if ($bOneRow) {
			$mResult = $this->fetchArray();
			if($mResult === false) {
				$mResult = array();
			}
			
			if (!is_null($sRowName)) {
				if (array_key_exists($sRowName, $mResult)) {
					$mResult = $mResult[$sRowName];
				}
				else {
					throw new MSF_DBException("Index not found " . $sRowName);
				}

			}

		}
		else {
			while ($aRow = $this->fetchArray()) {
				array_push($mResult, $aRow);
			}

		}

		return $mResult;
	}

	/**
	 * Initiates a transaction
	 * @access public
	 * @return void
	 */
	public function begin()
	{
		$this->executeQuery("BEGIN");
	}

	/**
	 * Commits a transaction
	 * @access public
	 * @return void
	 */
	public function commit()
	{
		$this->executeQuery("COMMIT");
	}

	/**
	 * Rolls back a transaction
	 * @access public
	 * @return void
	 */
	public function rollBack()
	{
		$this->executeQuery("ROLLBACK");
	}
	
	public function escapeString($mString)
	{
		if (is_array($mString)) {
			return $mString;
		}
		else {
			return pg_escape_string($this->rLink, $mString);
		}
	}

}


?>
