<?php
// tst
//include_once '../../ts/header.php';

// prd
if (!defined("__MSF")) {
	exit("Direct access is not allowed");
}

/**
 * @package 	
 * @uses 		MSF_Registry
 * @uses 		MSF_DB
 * @copyright 	Copyright (c) 2007, Detrix
 */
class MSF_DBRecord
{
	/**
	 * @var MSF_DB
	 */
	private $oDb;

	/**
	 * @var string
	 */
	private $sTable;

	/**
	 * @var string
	 */
	private $sPKey;

	/**
	 * @var string
	 */
	private $sSql 	= '';

	/**
	 * @var array
	 */
	private $aBind 	= array();

	/**
	 * @param string $sTable table name
	 * @param string $sPKey primary keys name
	 */
	protected function __construct($sTable, $sPKey)
	{
		$this->oDb		= MSF_Registry::get("MSF_DB");
		$this->sTable 	= $sTable;
		$this->sPKey	= $sPKey;
	}

	/**
	 * @return MSF_DBRecord
	 */
	protected function removeOne($iPKey)
	{
		$sSql = "DELETE FROM " . $this->sTable . " WHERE " . $this->sPKey . " = " . $iPKey;
		$this->oDb->executeQuery($sSql);
		return $this;
	}

	/**
	 * @return MSF_DBRecord
	 */
	protected function remove($sSql, array $aBind = array())
	{
		$this->reset();
		$this->query('DELETE FROM ' . $this->sTable . ' WHERE ' . $sSql);
		$this->bindAll($aBind);
		$this->oDb->executeQuery($this->prepareSql());
		$this->reset();
		return $this;
	}
	
	protected function findById($sWhat, $iId) {
		$this->reset();
		return $this->query('SELECT')
				 	->query($sWhat)
				 	->query('FROM')
				 	->query($this->sTable)
				 	->query('WHERE')
				 	->query($this->sPKey . ' = :id')
				 	->bind('id', $iId)
				 	->fetch();
	}
	
	protected function find($sWhat, $sWhere, array $aBind)
	{
		$this->reset();
		return $this->query('SELECT')
				 	->query($sWhat)
					->query('FROM')
				 	->query($this->sTable)
				 	->query('WHERE')
				 	->query($sWhere)
				 	->bindAll($aBind)
				 	->execute()
				 	->fetchAll();
	}
	
	protected function findOne($sWhat, $sWhere, array $aBind)
	{
		$this->reset();
		return $this->query('SELECT')
				 	->query($sWhat)
					->query('FROM')
				 	->query($this->sTable)
				 	->query('WHERE')
				 	->query($sWhere)
				 	->bindAll($aBind)
				 	->execute()
				 	->fetch();
	}
	
	protected function findCell($sCell, $sWhat, $sWhere, array $aBind)
	{
		$this->reset();
		return $this->query('SELECT')
				 	->query($sWhat)
					->query('FROM')
				 	->query($this->sTable)
				 	->query('WHERE')
				 	->query($sWhere)
				 	->bindAll($aBind)
				 	->execute()
				 	->fetchCell($sCell);
	}

	/**
	 * @return MSF_DBRecord
	 */
	protected function query($sSql) {
		$this->sSql .= $sSql . "\n";
		return $this;
	}

	/**
	 * @return array
	 */
	protected function fetchAll() {
		$aResult = array(); 
		while (($aRow = $this->fetch()) !== array()) {
			array_push($aResult, $aRow);
		}
		
		return $aResult;
	}
	
	protected function fetch() {
		return (array) $this->oDb->fetchArray();
	}

	/**
	 * @return string
	 */
	protected function fetchCell($sCell)
	{
		$aRow = $this->fetch();
		if (isset($aRow[$sCell])) {
			return $aRow[$sCell];
		}
		else {
			return null;
		}
		
	}

	/**
	 * @return MSF_DBRecord
	 */
	protected function bind($sKey, $sValue)
	{
		$this->aBind[':' . $sKey] = "'" . $this->oDb->escapeString($sValue) . "'";
		return $this;
	}

	/**
	 * @return MSF_DBRecord
	 */
	protected function bindAll(array $aBind)
	{
		foreach ($aBind AS $sKey => $sValue) {
			$this->bind($sKey, $sValue);	
		}

		return $this;
	}

	/**
	 * @return string
	 */
	protected function getSql() {
		return $this->sSql;
	}
	
	/**
	 * Return insert id
	 * @return int
	 */
	protected function insert(array $aFields)
	{
		$this->reset();
		$aKeys 		= array();
		$aValues 	= array();
		foreach ($aFields AS $sKey => $sValue) {
			$aKeys[] 	= $sKey;
			$aValues[] 	= "'" . $this->oDb->escapeString($sValue) . "'";
		}
		
		$this->query('INSERT INTO ' . $this->sTable . '('.implode(',', $aKeys) . ') VALUES ('.implode(',', $aValues) . ')');
		$this->execute();
		return $this->insertId();
	}
	
	/**
	 * Returns affected rows
	 * @return int
	 */
	protected function update(array $aFields, $sWhere, array $aBind = array())
	{
		$this->reset();
		$this->query('UPDATE')
			 ->query($this->sTable)
			 ->query('SET');
		$aSet 	= array();
		foreach ($aFields AS $sKey => $sValue) {
			$aSet[] = $sKey . ' = ' . "'" . $this->oDb->escapeString($sValue) . "'";
		}

		$this->query(implode(', ', $aSet));
		$this->query('WHERE');
		$this->query($sWhere);
		$this->bindAll($aBind);
		$this->execute();
		return $this->affectedRows();
	}
	
	/**
	 * Returns affected rows
	 * @return int
	 */
	protected function updateOne(array $aFields, $iPKeyValue)
	{
		return $this->update(
				$aFields,
				$this->sPKey . ' = :id', 
				array('id' => $iPKeyValue)
				);
	}
	
	/**
	 * @return MSF_DBRecord
	 */
	protected function execute() {
		$this->oDb->executeQuery($this->prepareSql());
		$this->reset();
		return $this;
	}
	
	/**
	 * @return int
	 */
	protected function affectedRows() {
		return $this->oDb->affectedRows();
	}
	
	/**
	 * @return int
	 */
	protected function insertId() {
		return $this->oDb->insertId();
	}
	
	/**
	 * @return bool
	 */
	protected function exists($iId) {
		$sTotal = $this->query('SELECT COUNT(*) AS total FROM ' . $this->sTable . ' WHERE ' . $this->sPKey . ' = :id')
 	 		 		   ->bind('id', $iId)
 	 		 		   ->execute()
		 	 		   ->fetchCell('total');
		return ($sTotal > 0);
	}

	/**
	 * @return MSF_DBRecord
	 */
	protected function reset() {
		$this->sSql 	= '';
		$this->aBind 	= array();
		return $this;
	}
	
	/**
	 * @return string
	 */
	private function prepareSql() {
		$sSql = $this->sSql;
		foreach ($this->aBind AS $sKey => $sValue) {
			$sSql = str_replace($sKey, $sValue, $sSql);
		}
		
		return $sSql;
	}

	protected static function test() {
		// In this case, used table wfa_control
		$oRecord 	= new MSF_DBRecord('wfa_control', 'control_id');
		tsEcho('Testing query(), bindAll(), fetchCell(), bind(used by bindAll)...');
		$sSql		= 'SELECT 1 AS total';
		$sTotal 	= $oRecord->query($sSql)
							  ->execute()
							  ->fetchCell('total');
	  	if ($sTotal == '1') {
	 		tsEchoSuccess();
	  	}
	  	else {
	  		tsEchoFailure();
	  	}
	  	
	  	//
	  	$aController = array('controller' => 'Detrix');
	  	tsEcho('Testing insert(), exists()...');
	  	$iInsertId = $oRecord->insert($aController);
	 	if ($oRecord->exists($iInsertId)) {
	  		tsEchoSuccess();
	  	}
	  	else {
	  		tsEchoFailure();
	  	}
	  	
	  	//
	  	tsEcho('Testing removeOne(), exists()...');
	  	$oRecord->removeOne($iInsertId);
		if ($oRecord->exists($iInsertId)) {
			tsEchoFailure();
	  	}
	  	else {
	  		tsEchoSuccess();
	  	}
	  	
	  	//
	  	tsEcho('Testing insert(), exists() ...');
	  	$iInsertId = $oRecord->insert($aController);
	  	if ($oRecord->exists($iInsertId)) {
	  		tsEchoSuccess();
	  	}
	  	else {
	  		tsEchoFailure();
	  	}
	  	
	  	//
		tsEcho('Testing remove(), exists()...');
	  	$oRecord->remove('controller = :controller', $aController);
		if ($oRecord->exists($iInsertId)) {
		 	tsEchoFailure();
	  	}
	  	else {
	  		tsEchoSuccess();
	  	}
	  	
	}

}

//MSF_DBRecord::test();
?>