<?php
/*
 * File: MSF_Date.php
 * Created at: 04.01.2008
 */
defined("__MSF")
	or die("Direct access is not allowed");

/**
 * @package MSF_Core
 * @author Detrix
 * @copyright Copyright (c) 2007, Detrix
 */
class MSF_Date
{
	public static $TIMESTAMP_PATTERN			= "Y-m-d H:i:s";

	public static $DATE_VIEW_PATTERN			= "Y-m-d";

	public static $TIMESTAMP_VIEW_PATTERN		= "Y-m-d H:i";

	public static $TIME_VIEW_PATTERN			= "H:i";


	private $iTime						= 0;

	private $iYear						= 0;

	private $iMonth						= 0;

	private $iDay						= 0;

	private $iHour						= 0;

	private $iMinute					= 0;

	private $iSecond 					= 0;



	public function __construct($iTime = 0)
	{
		if ($iTime == 0) {
			$this->iTime = time();
		}
		else {
			$this->iTime = $iTime;
		}
		
		$this->setAllValues();
	}

	/**
	 * @param String $sDatetime Format Y-m-d H:i:s
	 * @return void
	 * @throws MSF_ParsingException
	 */
	public function parse($sTimestamp)
	{
		//echo $sTimestamp;
		$mTime = strToTime($sTimestamp, 0);		
		if ($mTime == -1) {
			throw new MSF_ParsingException($sTimestamp);
		}
		//echo date("c", $mTime) . "<br/>";

		$this->iTime	= $mTime;
		$this->setAllValues();	
	}
	
	public static function toDbFormat($sTimestamp)
	{
		$sReturn	= "";
		$aDateTime	= split(" ", $sTimestamp);		
		$aDate		= split("\.", $aDateTime[0]);
		
		if(count($aDate) > 1) {
			$sReturn	= $aDate[2] . "-" . $aDate[1] . "-" . $aDate[0];
		}
		else {
			$sReturn	= $aDateTime[0];
		}
		
		if(isset($aDateTime[1])) {
			$sReturn .= " " . $aDateTime[1];
		}
		
		return $sReturn;
	}

	public static function toViewFormat($sTimestamp)
	{
		$sReturn	= "";
		$aDateTime	= split(" ", $sTimestamp);		
		$aDate		= split("-", $aDateTime[0]);
		
		if(count($aDate) > 1) {
			$sReturn	= $aDate[2] . "." . $aDate[1] . "." . $aDate[0];
		}
		else {
			$sReturn	= $aDateTime[0];
		}
		
		if(isset($aDateTime[1])) {
			$aTime	= split(":", $aDateTime[1]);
			//$sReturn .= " " . $aDateTime[1];
			$sReturn	.= " " . $aTime[0] . ":" . $aTime[1];
		}
		
		return $sReturn;
	}
	
	/**
	 * @param String $sPattern Optional, Format pattern
	 * @return string
	 */
	public function format($sPattern = "")
	{
		$sTimestamp = "";
		
		if (is_numeric($this->iTime)) {
			if (strlen($sPattern) == 0) {
				$sTimestamp = date(self::$TIMESTAMP_PATTERN, $this->iTime);
			}
			else {
				$sTimestamp = date($sPattern, $this->iTime);
			}
		}

		return $sTimestamp;
	}

	public function formatDate()
	{
		return $this->format(self::$DATE_VIEW_PATTERN);
	}

	public function formatTimestamp()
	{
		return $this->format(self::$TIMESTAMP_VIEW_PATTERN);
	}

	public function formatTime()
	{
		return $this->format(self::$TIME_VIEW_PATTERN);
	}

	public function getYear() {
		return $this->iYear;
	}

	public function getMonth() {
		return $this->iMonth;
	}

	public function getDay() {
		return $this->iDay;
	}

	public function getHour() {
		return $this->iHour;
	}

	public function getMinute() {
		return $this->iMinute;
	}

	public function getSeconds() {
		return $this->iSecond;
	}

	public function parseAndFormat($sTimeValue, $sPattern = "") 
	{
		$this->parse($sTimeValue);
		return $this->format($sPattern);
	}
	
	private function setAllValues()
	{
		$this->iYear	= $this->format("Y");
		$this->iMonth	= $this->format("n");
		$this->iDay		= $this->format("j");
		$this->iHour	= $this->format("H");
		$this->iMinute	= (int)$this->format("i");
		$this->iSecond	= (int)$this->format("s");
	}	
	
	public function getTime()
	{
		return $this->iTime;
	}
}
?>